@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 2004 - The Freeciv Project
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -43,8 +43,12 @@
 #include "advdata.h"
 
 /* ai */
+#include "handicaps.h"
+
+/* ai/default */
 #include "aidata.h"
 #include "aiguard.h"
+#include "ailog.h"
 #include "aiplayer.h"
 #include "aitools.h"
 #include "aiunit.h"
@@ -78,7 +82,9 @@
 /* Logging in find_ferry functions */
 #define LOGLEVEL_FINDFERRY LOG_DEBUG
 /* Extra consistency checks */
-#undef LOGLEVEL_FERRY_STATS
+#ifdef DEBUG
+#define LOGLEVEL_FERRY_STATS LOG_NORMAL
+#endif
 
 
 /* ========= managing statistics and boat/passanger assignments ======== */
@@ -103,7 +109,7 @@ void aiferry_init_stats(struct ai_type *ait, struct player *pplayer)
   unit_list_iterate(pplayer->units, punit) {
     struct unit_ai *unit_data = def_ai_unit_data(punit, ait);
 
-    if (dai_is_ferry(punit)) {
+    if (dai_is_ferry(punit, ait)) {
       ai->stats.boats++;
       if (unit_data->passenger == FERRY_AVAILABLE) {
         ai->stats.available_boats++;
@@ -130,7 +136,7 @@ static void aiferry_print_stats(struct ai_type *ait, struct player *pplayer)
   log_base(LOGLEVEL_FERRY_STATS, "Registered: %d free out of total %d",
            ai->stats.available_boats, ai->stats.boats);
   unit_list_iterate(pplayer->units, punit) {
-    if (dai_is_ferry(punit)) {
+    if (dai_is_ferry(punit, ait)) {
       log_base(LOGLEVEL_FERRY_STATS, "#%d. %s[%d], psngr=%d", n,
                unit_rule_name(punit), punit->id,
                def_ai_unit_data(punit, ait)->passenger);
@@ -143,30 +149,19 @@ static void aiferry_print_stats(struct ai_type *ait, struct player *pplayer)
 /**************************************************************************
   Should unit type be considered a ferry?
 **************************************************************************/
-static bool dai_is_ferry_type(struct unit_type *pferry)
+bool dai_is_ferry_type(struct unit_type *pferry, struct ai_type *ait)
 {
-  if (pferry->transport_capacity > 0
-      && dai_uclass_move_type(utype_class(pferry)) != UMT_LAND) {
-    unit_class_iterate(pclass) {
-      enum unit_move_type mt = dai_uclass_move_type(pclass);
+  struct unit_type_ai *utai = utype_ai_data(pferry, ait);
 
-      if (mt == UMT_LAND
-          && can_unit_type_transport(pferry, pclass)) {
-        /* Can transport some land moving unit. */
-        return TRUE;
-      }
-    } unit_class_iterate_end;
-  }
-
-  return FALSE;
+  return utai->ferry;
 }
 
 /**************************************************************************
   Should unit be considered a ferry?
 **************************************************************************/
-bool dai_is_ferry(struct unit *pferry)
+bool dai_is_ferry(struct unit *pferry, struct ai_type *ait)
 {
-  return dai_is_ferry_type(unit_type(pferry));
+  return dai_is_ferry_type(unit_type_get(pferry), ait);
 }
 
 /**************************************************************************
@@ -174,7 +169,7 @@ bool dai_is_ferry(struct unit *pferry)
 **************************************************************************/
 void dai_ferry_init_ferry(struct ai_type *ait, struct unit *ferry)
 {
-  if (dai_is_ferry(ferry)) {
+  if (dai_is_ferry(ferry, ait)) {
     struct unit_ai *unit_data = def_ai_unit_data(ferry, ait);
     struct ai_plr *ai = dai_plr_data_get(ait, unit_owner(ferry), NULL);
 
@@ -190,8 +185,8 @@ void dai_ferry_init_ferry(struct ai_type *ait, struct unit *ferry)
 void dai_ferry_transformed(struct ai_type *ait, struct unit *ferry,
                            struct unit_type *old)
 {
-  bool old_f = dai_is_ferry_type(old);
-  bool new_f = dai_is_ferry(ferry);
+  bool old_f = dai_is_ferry_type(old, ait);
+  bool new_f = dai_is_ferry(ferry, ait);
 
   if (old_f && !new_f) {
     struct ai_plr *ai = dai_plr_data_get(ait, unit_owner(ferry), NULL);
@@ -223,12 +218,12 @@ void dai_ferry_lost(struct ai_type *ait, struct unit *punit)
 {
   /* Ignore virtual units. */
   if (punit->id != 0 && is_ai_data_phase_open(ait, unit_owner(punit))) {
-    bool close;
+    bool close_here;
     struct unit_ai *unit_data = def_ai_unit_data(punit, ait);
     struct player *pplayer = unit_owner(punit);
-    struct ai_plr *ai = dai_plr_data_get(ait, pplayer, &close);
+    struct ai_plr *ai = dai_plr_data_get(ait, pplayer, &close_here);
 
-    if (dai_is_ferry(punit)) {
+    if (dai_is_ferry(punit, ait)) {
       ai->stats.boats--;
       if (unit_data->passenger == FERRY_AVAILABLE) {
         ai->stats.available_boats--;
@@ -240,7 +235,7 @@ void dai_ferry_lost(struct ai_type *ait, struct unit *punit)
       }
     }
 
-    if (close) {
+    if (close_here) {
       dai_data_phase_finished(ait, pplayer);
     }
   }
@@ -361,16 +356,15 @@ int aiferry_avail_boats(struct ai_type *ait, struct player *pplayer)
   int boats = 0;
 
   unit_list_iterate(pplayer->units, punit) {
-    if (dai_is_ferry(punit)
+    if (dai_is_ferry(punit, ait)
         && def_ai_unit_data(punit, ait)->passenger == FERRY_AVAILABLE) {
       boats++;
     }
   } unit_list_iterate_end;
 
   if (boats != ai->stats.available_boats) {
-    log_base(LOGLEVEL_FERRY_STATS,
-             "Player[%d] in turn %d: boats miscounted.",
-             player_number(pplayer), game.info.turn);
+    log_error("Player[%d] in turn %d: boats miscounted.",
+              player_number(pplayer), game.info.turn);
     aiferry_print_stats(ait, pplayer);
   }
 #endif /* LOGLEVEL_FERRY_STATS */
@@ -394,21 +388,23 @@ int aiferry_avail_boats(struct ai_type *ait, struct player *pplayer)
   excluded using a TB call-back.
 **************************************************************************/
 static int combined_land_sea_move(const struct tile *src_tile,
-                                  enum direction8 dir,
+                                  enum pf_move_scope src_scope,
                                   const struct tile *tgt_tile,
+                                  enum pf_move_scope dst_scope,
                                   const struct pf_parameter *param)
 {
   int move_cost;
 
-  if (is_ocean_tile(tgt_tile)) {
+  if (!((PF_MS_NATIVE | PF_MS_CITY) & dst_scope)) {
     /* Any-to-Sea */
     move_cost = 0;
-  } else if (is_ocean_tile(src_tile)) {
+  } else if (!((PF_MS_NATIVE | PF_MS_CITY) & src_scope)) {
     /* Sea-to-Land */
     move_cost = PF_IMPOSSIBLE_MC;
   } else {
     /* Land-to-Land */
-    move_cost = map_move_cost(param->owner, param->uclass, src_tile, tgt_tile);
+    move_cost = map_move_cost(param->owner, param->utype,
+                              src_tile, tgt_tile);
   }
 
   return move_cost;
@@ -448,7 +444,7 @@ bool is_boat_free(struct ai_type *ait, struct unit *boat,
    * - Only units that can travel at sea are eligible.
    * - Units that require fuel are lose hitpoints are not eligible.
    */
-  struct unit_class *ferry_class = unit_class(boat);
+  struct unit_class *ferry_class = unit_class_get(boat);
   struct unit_ai *boat_data = def_ai_unit_data(boat, ait);
 
   return (can_unit_transport(boat, punit)
@@ -459,7 +455,7 @@ bool is_boat_free(struct ai_type *ait, struct unit *boat,
           && (get_transporter_capacity(boat) 
               - get_transporter_occupancy(boat) >= cap)
           && ferry_class->adv.sea_move != MOVE_NONE
-          && !unit_type(boat)->fuel
+          && !unit_type_get(boat)->fuel
           && !is_losing_hp(boat));
 }
 
@@ -497,6 +493,7 @@ int aiferry_find_boat(struct ai_type *ait, struct unit *punit,
   int best_id = 0;
   struct pf_parameter param;
   struct pf_map *search_map;
+  struct player *pplayer = unit_owner(punit);
 
   /* currently assigned ferry */
   int ferryboat = def_ai_unit_data(punit, ait)->ferryboat;
@@ -514,7 +511,7 @@ int aiferry_find_boat(struct ai_type *ait, struct unit *punit,
                     || FERRY_WANTED == ferryboat, 0);
   UNIT_LOG(LOGLEVEL_FINDFERRY, punit, "asked aiferry_find_boat for a boat");
 
-  if (aiferry_avail_boats(ait, unit_owner(punit)) <= 0 
+  if (aiferry_avail_boats(ait, pplayer) <= 0 
       && ferryboat <= 0) {
     /* No boats to be found (the second check is to ensure that we are not 
      * the ones keeping the last boat busy) */
@@ -522,9 +519,11 @@ int aiferry_find_boat(struct ai_type *ait, struct unit *punit,
   }
 
   pft_fill_unit_parameter(&param, punit);
+  param.omniscience = !has_handicap(pplayer, H_MAP);
   param.get_TB = no_fights_or_unknown;
   param.get_EC = sea_move;
   param.get_MC = combined_land_sea_move;
+  param.ignore_none_scopes = FALSE;
 
   search_map = pf_map_new(&param);
 
@@ -548,7 +547,7 @@ int aiferry_find_boat(struct ai_type *ait, struct unit *punit,
           /* Turns for the boat to get to the rendezvous pnt */
           int f_turns = ((pos.total_EC / PF_TURN_FACTOR * 16 
                           - aunit->moves_left) 
-                         / unit_type(aunit)->move_rate);
+                         / unit_type_get(aunit)->move_rate);
           int turns = MAX(u_turns, f_turns);
 
           if (turns < best_turns) {
@@ -668,7 +667,7 @@ bool dai_amphibious_goto_constrained(struct ai_type *ait,
       if (!pf_path_advance(path, unit_tile(passenger))) {
         /* Somehow we got thrown away from our route.
          * This can happen if our movement caused alliance breakup. */
-        return unit_alive(pass_id);
+        return unit_is_alive(pass_id);
       }
       next_tile = path->positions[1].tile;
       if (!is_ocean_tile(next_tile)) {
@@ -684,7 +683,7 @@ bool dai_amphibious_goto_constrained(struct ai_type *ait,
          * ferry to die. That has happened at least when passenger
          * destroyed city cutting the civ1-style channel (cities in
          * a chain) ferry was in. */
-	if (unit_alive(ferry_id) && 0 < ferry->moves_left
+	if (unit_is_alive(ferry_id) && 0 < ferry->moves_left
             && (!alive || unit_tile(ferry) != unit_tile(passenger))) {
 	  /* The passenger is no longer on the ferry,
 	   * and the ferry can still act.
@@ -696,7 +695,7 @@ bool dai_amphibious_goto_constrained(struct ai_type *ait,
 
           /* It is theoretically possible passenger died here due to
            * autoattack against another passing unit at its location. */
-          alive = unit_alive(pass_id);
+          alive = unit_is_alive(pass_id);
         }
       }
       /* else at sea */
@@ -821,7 +820,7 @@ bool aiferry_gobyboat(struct ai_type *ait, struct player *pplayer,
       return FALSE;
     }
 
-    handle_unit_load(pplayer, punit->id, ferryboat->id);
+    handle_unit_load(pplayer, punit->id, ferryboat->id, ferryboat->tile->index);
     fc_assert(unit_transported(punit));
   }
 
@@ -842,7 +841,7 @@ bool aiferry_gobyboat(struct ai_type *ait, struct player *pplayer,
       /* Grab bodyguard */
       if (bodyguard
           && !same_pos(unit_tile(punit), unit_tile(bodyguard))) {
-        if (!goto_is_sane(ait, bodyguard, unit_tile(punit), TRUE)
+        if (!goto_is_sane(bodyguard, unit_tile(punit))
             || !dai_unit_goto(ait, bodyguard, unit_tile(punit))) {
           /* Bodyguard can't get there or died en route */
           aiguard_request_guard(ait, punit);
@@ -862,7 +861,8 @@ bool aiferry_gobyboat(struct ai_type *ait, struct player *pplayer,
       }
       if (bodyguard) {
         fc_assert(same_pos(unit_tile(punit), unit_tile(bodyguard)));
-        handle_unit_load(pplayer, bodyguard->id, ferryboat->id);
+        handle_unit_load(pplayer, bodyguard->id, ferryboat->id,
+                         ferryboat->tile->index);
       }
       if (!aiferry_goto_amphibious(ait, ferryboat, punit, dest_tile)) {
         /* died */
@@ -904,6 +904,7 @@ static bool aiferry_findcargo(struct ai_type *ait, struct unit *pferry)
   struct pf_map *pfm;
   struct pf_parameter parameter;
   int passengers = dai_plr_data_get(ait, unit_owner(pferry), NULL)->stats.passengers;
+  struct player *pplayer;
 
   if (passengers <= 0) {
     /* No passangers anywhere */
@@ -912,7 +913,9 @@ static bool aiferry_findcargo(struct ai_type *ait, struct unit *pferry)
 
   UNIT_LOG(LOGLEVEL_FERRY, pferry, "Ferryboat is looking for cargo.");
 
+  pplayer = unit_owner(pferry);
   pft_fill_unit_overlap_param(&parameter, pferry);
+  parameter.omniscience = !has_handicap(pplayer, H_MAP);
   /* If we have omniscience, we use it, since paths to some places
    * might be "blocked" by unknown.  We don't want to fight though */
   parameter.get_TB = no_fights;
@@ -997,21 +1000,21 @@ static bool aiferry_find_interested_city(struct ai_type *ait,
       int turns = city_production_turns_to_build(pcity, TRUE);
 
       UNIT_LOG(LOGLEVEL_FERRY, pferry, "%s (%d, %d) looks promising...", 
-               city_name(pcity), TILE_XY(pcity->tile));
+               city_name_get(pcity), TILE_XY(pcity->tile));
 
       if (pos.turn > turns
           && VUT_UTYPE == pcity->production.kind
           && utype_has_role(pcity->production.value.utype, L_FERRYBOAT)) {
         UNIT_LOG(LOGLEVEL_FERRY, pferry, "%s is NOT suitable: "
                  "will finish building its own ferry too soon",
-                 city_name(pcity));
+                 city_name_get(pcity));
         continue;
       }
 
       if (turns >= turns_horizon) {
         UNIT_LOG(LOGLEVEL_FERRY, pferry, "%s is NOT suitable: "
                  "has just started building",
-                 city_name(pcity));
+                 city_name_get(pcity));
         continue;
       }
 
@@ -1021,7 +1024,7 @@ static bool aiferry_find_interested_city(struct ai_type *ait,
 
           UNIT_LOG(LOGLEVEL_FERRY, pferry, "%s is NOT suitable: "
                    "has another ferry",
-                   city_name(pcity));
+                   city_name_get(pcity));
 	  really_needed = FALSE;
 	  break;
 	}
@@ -1030,7 +1033,7 @@ static bool aiferry_find_interested_city(struct ai_type *ait,
       if (really_needed) {
         UNIT_LOG(LOGLEVEL_FERRY, pferry, "will go to %s unless we "
                  "find something better",
-                 city_name(pcity));
+                 city_name_get(pcity));
 	pferry->goto_tile = pos.tile;
         turns_horizon = turns;
         needed = TRUE;
@@ -1059,14 +1062,15 @@ void dai_manage_ferryboat(struct ai_type *ait, struct player *pplayer,
   int sanity = punit->id;
   struct unit_ai *unit_data;
   int bossid;
+  struct unit_type *ptype;
 
   CHECK_UNIT(punit);
 
   /* Try to recover hitpoints if we are in a city, before we do anything */
-  if (punit->hp < unit_type(punit)->hp 
+  if (punit->hp < unit_type_get(punit)->hp 
       && (pcity = tile_city(unit_tile(punit)))) {
     UNIT_LOG(LOGLEVEL_FERRY, punit, "waiting in %s to recover hitpoints", 
-             city_name(pcity));
+             city_name_get(pcity));
     def_ai_unit_data(punit, ait)->done = TRUE;
     return;
   }
@@ -1135,7 +1139,7 @@ void dai_manage_ferryboat(struct ai_type *ait, struct player *pplayer,
       fc_assert_ret(NULL != boss);
 
       if (unit_has_type_flag(boss, UTYF_SETTLERS)
-          || unit_has_type_flag(boss, UTYF_CITIES)) {
+          || unit_is_cityfounder(boss)) {
         /* Temporary hack: settlers all go in the end, forcing them 
          * earlier might mean uninitialised cache, so just wait for them */
         return;
@@ -1181,7 +1185,9 @@ void dai_manage_ferryboat(struct ai_type *ait, struct player *pplayer,
 
   /* Not carrying anyone, even the ferryman */
 
-  if (IS_ATTACKER(punit) && punit->moves_left > 0) {
+  ptype = unit_type_get(punit);
+
+  if (IS_ATTACKER(ptype) && punit->moves_left > 0) {
      /* AI used to build frigates to attack and then use them as ferries 
       * -- Syela */
     dai_unit_new_task(ait, punit, AIUNIT_NONE, NULL);
@@ -1244,13 +1250,14 @@ void dai_manage_ferryboat(struct ai_type *ait, struct player *pplayer,
   };
 
   if (punit->moves_left > 0) {
-    struct city *pcity = find_nearest_safe_city(punit);
-    if (pcity) {
-      punit->goto_tile = pcity->tile;
+    struct city *safe_city = find_nearest_safe_city(punit);
+
+    if (safe_city != NULL) {
+      punit->goto_tile = safe_city->tile;
       UNIT_LOG(LOGLEVEL_FERRY, punit, "No work, going home");
       unit_data->done = TRUE;
       dai_unit_new_task(ait, punit, AIUNIT_NONE, NULL);
-      (void) dai_unit_goto(ait, punit, pcity->tile);
+      (void) dai_unit_goto(ait, punit, safe_city->tile);
     }
   }
  
