@@ -1,4 +1,4 @@
-/**********************************************************************
+/***********************************************************************
  Freeciv - Copyright (C) 2005 The Freeciv Team
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #include "city.h"
 #include "game.h"
 #include "player.h"
+#include "research.h"
 #include "unit.h"
 #include "unitlist.h"
 
@@ -41,7 +42,12 @@
 #include "utilities.h"
 
 /* ai */
+#include "handicaps.h"
+
+/* ai/default */
 #include "aicity.h"
+#include "aidata.h"
+#include "ailog.h"
 #include "aiplayer.h"
 #include "aiunit.h"
 #include "aitools.h"
@@ -60,7 +66,7 @@ static struct tile *find_best_tile_to_paradrop_to(struct ai_type *ait,
   int best = 0;
   int val;
   struct tile* best_tile = NULL;
-  int range = unit_type(punit)->paratroopers_range;
+  int range = unit_type_get(punit)->paratroopers_range;
   struct city* acity;
   struct player* pplayer = unit_owner(punit);
 
@@ -85,8 +91,8 @@ static struct tile *find_best_tile_to_paradrop_to(struct ai_type *ait,
     acity = tile_city(best_tile);
     UNIT_LOG(LOGLEVEL_PARATROOPER, punit, 
              "Choose to jump in order to protect allied city %s (%d %d). "
-	     "Benefit: %d",
-	     city_name(acity), TILE_XY(best_tile), best);
+             "Benefit: %d",
+             city_name_get(acity), TILE_XY(best_tile), best);
     return best_tile;
   }
 
@@ -96,7 +102,7 @@ static struct tile *find_best_tile_to_paradrop_to(struct ai_type *ait,
     if (acity && pplayers_at_war(unit_owner(punit), city_owner(acity)) &&
         (unit_list_size(ptile->units) == 0)) {
       if (!map_is_known_and_seen(ptile, pplayer, V_MAIN)
-          && ai_handicap(pplayer, H_FOG)) {
+          && has_handicap(pplayer, H_FOG)) {
         continue;
       }
       /* Prefer big cities on other continents */
@@ -113,7 +119,7 @@ static struct tile *find_best_tile_to_paradrop_to(struct ai_type *ait,
     acity = tile_city(best_tile);
     UNIT_LOG(LOGLEVEL_PARATROOPER, punit, 
              "Choose to jump into enemy city %s (%d %d). Benefit: %d",
-	     city_name(acity), TILE_XY(best_tile), best);
+             city_name_get(acity), TILE_XY(best_tile), best);
     return best_tile;
   }
 
@@ -138,14 +144,14 @@ static struct tile *find_best_tile_to_paradrop_to(struct ai_type *ait,
       if (unit_list_size(target->units) == 0
           || !can_unit_attack_tile(punit, target)
           || is_ocean_tile(target)
-          || (ai_handicap(pplayer, H_FOG)
+          || (has_handicap(pplayer, H_FOG)
               && !map_is_known_and_seen(target, pplayer, V_MAIN))) {
         continue;
       }
       val = 0;
       if (is_stack_vulnerable(target)) {
         unit_list_iterate(target->units, victim) {
-          if ((!ai_handicap(pplayer, H_FOG)
+          if ((!has_handicap(pplayer, H_FOG)
                || can_player_see_unit(pplayer, victim))
               && (unit_attack_unit_at_tile_result(punit, victim, target) == ATT_OK)) {
             val += victim->hp * 100;
@@ -193,7 +199,7 @@ void dai_manage_paratrooper(struct ai_type *ait, struct player *pplayer,
   }
 
   /* check to recover hit points */
-  if ((punit->hp < unit_type(punit)->hp / 2) && pcity) {
+  if ((punit->hp < unit_type_get(punit)->hp / 2) && pcity) {
     UNIT_LOG(LOGLEVEL_PARATROOPER, punit, "Recovering hit points.");
     return;
   }
@@ -238,10 +244,10 @@ void dai_manage_paratrooper(struct ai_type *ait, struct player *pplayer,
     acity = find_nearest_safe_city(punit);
 
     if (acity) {
-      UNIT_LOG(LOGLEVEL_PARATROOPER, punit, "Going to %s", city_name(acity));
+      UNIT_LOG(LOGLEVEL_PARATROOPER, punit, "Going to %s", city_name_get(acity));
       if (!dai_unit_goto(ait, punit, acity->tile)) {
-	/* die or unsuccessfull move */
-	return;
+        /* die or unsuccessfull move */
+        return;
       }
     } else {
       UNIT_LOG(LOGLEVEL_PARATROOPER, punit,
@@ -258,7 +264,7 @@ void dai_manage_paratrooper(struct ai_type *ait, struct player *pplayer,
 static int calculate_want_for_paratrooper(struct unit *punit,
 				          struct tile *ptile_city)
 {
-  struct unit_type* u_type = unit_type(punit);
+  struct unit_type* u_type = unit_type_get(punit);
   int range = u_type->paratroopers_range;
   int profit = 0;
   struct player* pplayer = unit_owner(punit);
@@ -325,14 +331,17 @@ static int calculate_want_for_paratrooper(struct unit *punit,
 /*******************************************************************
   Chooses to build a paratroopers if necessary
 *******************************************************************/
-void dai_choose_paratrooper(struct player *pplayer, struct city *pcity,
+void dai_choose_paratrooper(struct ai_type *ait,
+                            struct player *pplayer, struct city *pcity,
                             struct adv_choice *choice)
 {
+  const struct research *presearch;
   int profit;
   Tech_type_id tech_req;
   Tech_type_id requirements[A_LAST];
   int num_requirements = 0;
   int i;
+  struct ai_plr *plr_data = def_ai_player_data(pplayer, ait);
 
   /* military_advisor_choose_build does something idiotic,
    * this function should not be called if there is danger... */
@@ -350,6 +359,12 @@ void dai_choose_paratrooper(struct player *pplayer, struct city *pcity,
       continue;
     }
 
+    /* Temporary hack because pathfinding can't handle Fighters. */
+    if (!uclass_has_flag(utype_class(u_type), UCF_MISSILE)
+        && 1 == utype_fuel(u_type)) {
+      continue;
+    }
+
     /* assign tech for paratroopers */
     tech_req = advance_index(u_type->require_advance);
     if (tech_req != A_NONE && tech_req != A_UNSET) {
@@ -382,26 +397,27 @@ void dai_choose_paratrooper(struct player *pplayer, struct city *pcity,
       choice->type = CT_ATTACKER;
       choice->need_boat = FALSE;
       log_base(LOGLEVEL_PARATROOPER, "%s wants to build %s (want=%d)",
-               city_name(pcity), utype_rule_name(u_type), profit);
+               city_name_get(pcity), utype_rule_name(u_type), profit);
     }
   } unit_type_iterate_end;
 
   /* we raise want if the required tech is not known */
+  presearch = research_get(pplayer);
   for (i = 0; i < num_requirements; i++) {
     tech_req = requirements[i];
-    pplayer->ai_common.tech_want[tech_req] += 2;
+    plr_data->tech_want[tech_req] += 2;
     log_base(LOGLEVEL_PARATROOPER, "Raising tech want in city %s for %s "
-             "stimulating %s with %d (%d) and req",
-             city_name(pcity),
+             "stimulating %s with %d (" ADV_WANT_PRINTF ") and req",
+             city_name_get(pcity),
              player_name(pplayer),
-             advance_name_by_player(pplayer, tech_req),
+             advance_rule_name(advance_by_number(tech_req)),
              2,
-             pplayer->ai_common.tech_want[tech_req]);
+             plr_data->tech_want[tech_req]);
 
     /* now, we raise want for prerequisites */
     advance_index_iterate(A_FIRST, k) {
-      if (is_tech_a_req_for_goal(pplayer, k, tech_req)) {
-        pplayer->ai_common.tech_want[k] += 1;
+      if (research_goal_tech_req(presearch, tech_req, k)) {
+        plr_data->tech_want[k] += 1;
       }
     } advance_index_iterate_end;
   }
