@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 2004 - The Freeciv Team
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -51,9 +51,13 @@
 #include "infracache.h"
 
 /* ai */
+#include "handicaps.h"
+
+/* ai/default */
 #include "aidata.h"
 #include "aicity.h"
 #include "aiferry.h"
+#include "ailog.h"
 #include "aiplayer.h"
 #include "aitools.h"
 #include "aiunit.h"
@@ -132,11 +136,9 @@ static void tile_data_cache_destroy(struct tile_data_cache *ptdc);
 
 /* struct tdcache_hash. */
 #define SPECHASH_TAG tile_data_cache
-#define SPECHASH_KEY_TYPE int
-#define SPECHASH_DATA_TYPE struct tile_data_cache *
-#define SPECHASH_KEY_TO_PTR FC_INT_TO_PTR
-#define SPECHASH_PTR_TO_KEY FC_PTR_TO_INT
-#define SPECHASH_DATA_FREE tile_data_cache_destroy
+#define SPECHASH_INT_KEY_TYPE
+#define SPECHASH_IDATA_TYPE struct tile_data_cache *
+#define SPECHASH_IDATA_FREE tile_data_cache_destroy
 #include "spechash.h"
 
 struct ai_settler {
@@ -192,11 +194,11 @@ static void cityresult_destroy(struct cityresult *result);
 
 static struct cityresult *cityresult_fill(struct ai_type *ait,
                                           struct player *pplayer,
-                                          struct tile *ptile);
+                                          struct tile *center);
 static bool food_starvation(const struct cityresult *result);
 static bool shield_starvation(const struct cityresult *result);
-static int defense_bonus(struct player *pplayer,
-                         const struct cityresult *result);
+static int result_defense_bonus(struct player *pplayer,
+                                const struct cityresult *result);
 static int naval_bonus(const struct cityresult *result);
 static void print_cityresult(struct player *pplayer,
                              const struct cityresult *cr);
@@ -270,25 +272,25 @@ static void cityresult_destroy(struct cityresult *result)
 **************************************************************************/
 static struct cityresult *cityresult_fill(struct ai_type *ait,
                                           struct player *pplayer,
-                                          struct tile *ptile)
+                                          struct tile *center)
 {
-  struct city *pcity = tile_city(ptile);
+  struct city *pcity = tile_city(center);
   struct government *curr_govt = government_of_player(pplayer);
   struct player *saved_owner = NULL;
   struct tile *saved_claimer = NULL;
   bool virtual_city = FALSE;
-  bool handicap = ai_handicap(pplayer, H_MAP);
+  bool handicap = has_handicap(pplayer, H_MAP);
   struct adv_data *adv = adv_data_get(pplayer, NULL);
   struct ai_plr *ai = dai_plr_data_get(ait, pplayer, NULL);
   struct cityresult *result;
 
   fc_assert_ret_val(ai != NULL, NULL);
-  fc_assert_ret_val(ptile != NULL, NULL)
+  fc_assert_ret_val(center != NULL, NULL);
 
   pplayer->government = adv->goal.govt.gov;
 
   /* Create a city result and set default values. */
-  result = cityresult_new(ptile);
+  result = cityresult_new(center);
 
   if (!pcity) {
     pcity = create_city_virtual(pplayer, result->tile, "Virtuaville");
@@ -540,7 +542,7 @@ static void tdc_plr_set(struct ai_type *ait, struct player *plr, int tindex,
   fc_assert_ret(ai != NULL);
   fc_assert_ret(ai->settler != NULL);
   fc_assert_ret(ai->settler->tdc_hash != NULL);
-  fc_assert_ret(ptdc)
+  fc_assert_ret(ptdc != NULL);
 
 #ifdef DEBUG
     ai->settler->cache.save++;
@@ -579,38 +581,28 @@ static bool shield_starvation(const struct cityresult *result)
   Calculate defense bonus, which is a % of total results equal to a
   given % of the defense bonus %.
 **************************************************************************/
-static int defense_bonus(struct player *pplayer,
-                         const struct cityresult *result)
+static int result_defense_bonus(struct player *pplayer,
+                                const struct cityresult *result)
 {
   /* Defense modification (as tie breaker mostly) */
-  int defense_bonus = 
+  int defense_bonus =
     10 + tile_terrain(result->tile)->defense_bonus / 10;
-  int road_bonus = 0;
-  int base_bonus = 0;
+  int extra_bonus = 0;
   struct tile *vtile = tile_virtual_new(result->tile);
   struct city *vcity = create_city_virtual(pplayer, vtile, "");
 
   tile_set_worked(vtile, vcity); /* Link tile_city(vtile) to vcity. */
-  /* Give city free roads and bases. */
-  upgrade_city_roads(vcity, NULL);
-  upgrade_city_bases(vcity, NULL);
-  road_type_iterate(proad) {
-    if (tile_has_road(vtile, proad)) {
+  upgrade_city_extras(vcity, NULL); /* Give city free extras. */
+  extra_type_iterate(pextra) {
+    if (tile_has_extra(vtile, pextra)) {
       /* TODO: Do not use full bonus of those road types
        *       that are not native to all important units. */
-      road_bonus += proad->defense_bonus;
+      extra_bonus += pextra->defense_bonus;
     }
-  } road_type_iterate_end;
-  base_type_iterate(pbase) {
-    if (tile_has_base(vtile, pbase)) {
-      /* TODO: Do not use full bonus of those base types
-       *       that are not native to all important units. */
-      base_bonus += pbase->defense_bonus;
-    }
-  } base_type_iterate_end;
+  } extra_type_iterate_end;
   tile_virtual_destroy(vtile);
 
-  defense_bonus += (defense_bonus * (road_bonus + base_bonus)) / 100;
+  defense_bonus += (defense_bonus * extra_bonus) / 100;
 
   return 100 / (result->total + 1) * (100 / defense_bonus * DEFENSE_EMPHASIS);
 }
@@ -688,7 +680,7 @@ static void print_cityresult(struct player *pplayer,
            cr->best_other.cindex, cr->best_other.tdc->sum);
   log_test("- corr %d - waste %d + remaining %d"
            " + defense bonus %d + naval bonus %d", cr->corruption,
-           cr->waste, cr->remaining, defense_bonus(pplayer, cr),
+           cr->waste, cr->remaining, result_defense_bonus(pplayer, cr),
            naval_bonus(cr));
   log_test("= %d (%d)", cr->total, cr->result);
 
@@ -717,7 +709,7 @@ struct cityresult *city_desirability(struct ai_type *ait, struct player *pplayer
   fc_assert_ret_val(ai, NULL);
 
   if (!city_can_be_built_here(ptile, punit)
-      || (ai_handicap(pplayer, H_MAP)
+      || (has_handicap(pplayer, H_MAP)
           && !map_is_known(ptile, pplayer))) {
     return NULL;
   }
@@ -761,7 +753,7 @@ struct cityresult *city_desirability(struct ai_type *ait, struct player *pplayer
     return NULL;
   }
 
-  cr->total += defense_bonus(pplayer, cr);
+  cr->total += result_defense_bonus(pplayer, cr);
   cr->total += naval_bonus(cr);
 
   /* Add remaining points, which is our potential */
@@ -797,7 +789,7 @@ static struct cityresult *settler_map_iterate(struct ai_type *ait,
   pf_map_move_costs_iterate(pfm, ptile, move_cost, FALSE) {
     int turns;
 
-    if (boat_cost == 0 && unit_class(punit)->adv.sea_move == MOVE_NONE
+    if (boat_cost == 0 && unit_class_get(punit)->adv.sea_move == MOVE_NONE
         && tile_continent(ptile) != tile_continent(unit_tile(punit))) {
       /* We have an accidential land bridge. Ignore it. It will in all
        * likelihood go away next turn, or even in a few nanoseconds. */
@@ -892,7 +884,6 @@ static struct cityresult *find_best_city_placement(struct ai_type *ait,
   struct pf_parameter parameter;
   struct player *pplayer = unit_owner(punit);
   struct unit *ferry = NULL;
-  struct unit_class *ferry_class = NULL;
   struct cityresult *cr1 = NULL, *cr2 = NULL;
 
   fc_assert_ret_val(pplayer->ai_controlled, NULL);
@@ -902,6 +893,7 @@ static struct cityresult *find_best_city_placement(struct ai_type *ait,
   /* Phase 1: Consider building cities on our continent */
 
   pft_fill_unit_parameter(&parameter, punit);
+  parameter.omniscience = !has_handicap(pplayer, H_MAP);
   cr1 = settler_map_iterate(ait, &parameter, punit, 0);
 
   if (cr1 && cr1->result > RESULT_IS_ENOUGH) {
@@ -930,10 +922,12 @@ static struct cityresult *find_best_city_placement(struct ai_type *ait,
         boattype = get_role_unit(L_FERRYBOAT, 0);
 
         if (NULL != boattype
-         && A_NEVER != boattype->require_advance) {
-          pplayer->ai_common.tech_want[advance_index(boattype->require_advance)]
+            && A_NEVER != boattype->require_advance) {
+          struct ai_plr *plr_data = def_ai_player_data(pplayer, ait);
+
+          plr_data->tech_want[advance_index(boattype->require_advance)]
             += FERRY_TECH_WANT;
-          TECH_LOG(LOG_DEBUG, pplayer, boattype->require_advance,
+          TECH_LOG(ait, LOG_DEBUG, pplayer, boattype->require_advance,
                    "+ %d for %s to ferry settler",
                    FERRY_TECH_WANT,
                    utype_rule_name(boattype));
@@ -945,10 +939,9 @@ static struct cityresult *find_best_city_placement(struct ai_type *ait,
       unit_tile_set(ferry, unit_tile(punit));
     }
 
-    ferry_class = unit_class(ferry);
-
-    fc_assert(ferry_class->adv.sea_move != MOVE_NONE);
+    fc_assert(dai_is_ferry(ferry, ait));
     pft_fill_unit_overlap_param(&parameter, ferry);
+    parameter.omniscience = !has_handicap(pplayer, H_MAP);
     parameter.get_TB = no_fights_or_unknown;
 
     /* FIXME: Maybe penalty for using an existing boat is too high?
@@ -1020,9 +1013,10 @@ void dai_auto_settler_run(struct ai_type *ait, struct player *pplayer,
 {
   int best_impr = 0;            /* best terrain improvement we can do */
   enum unit_activity best_act;
-  struct act_tgt best_target;
+  struct extra_type *best_target;
   struct tile *best_tile = NULL;
   struct pf_path *path = NULL;
+  struct city *pcity = NULL;
 
   /* time it will take worker to complete its given task */
   int completion_time = 0;
@@ -1076,21 +1070,26 @@ BUILD_CITY:
   /*** Try find some work ***/
 
   if (unit_has_type_flag(punit, UTYF_SETTLERS)) {
+    struct worker_task *best_task;
+
     TIMING_LOG(AIT_WORKERS, TIMER_START);
 
     /* Have nearby cities requests? */
-    best_impr = settler_evaluate_city_requests(punit, &best_act, &best_target,
-                                               &best_tile, &path, state);
+    pcity = settler_evaluate_city_requests(punit, &best_task, &path, state);
 
-    if (best_impr > 0) {
+    if (pcity != NULL) {
       if (path != NULL) {
         completion_time = pf_path_last_position(path)->turn;
+        best_impr = 1;
+        best_act = best_task->act;
+        best_target = best_task->tgt;
+        best_tile = best_task->ptile;
       } else {
-        best_impr = 0;
+        pcity = NULL;
       }
     }
 
-    if (best_impr <= 0) {
+    if (pcity == NULL) {
       best_impr = settler_evaluate_improvements(punit, &best_act, &best_target,
                                                 &best_tile, &path, state);
       if (path) {
@@ -1101,7 +1100,7 @@ BUILD_CITY:
     TIMING_LOG(AIT_WORKERS, TIMER_STOP);
   }
 
-  if (unit_has_type_flag(punit, UTYF_CITIES)) {
+  if (unit_is_cityfounder(punit)) {
     struct cityresult *result;
 
     /* may use a boat: */
@@ -1112,7 +1111,7 @@ BUILD_CITY:
       UNIT_LOG(LOG_DEBUG, punit, "city want %d", result->result);
       if (tile_city(result->tile)) {
         UNIT_LOG(LOG_DEBUG, punit, "immigrates to %s (%d, %d)", 
-                 city_name(tile_city(result->tile)),
+                 city_name_get(tile_city(result->tile)),
                  TILE_XY(result->tile));
       } else {
         UNIT_LOG(LOG_DEBUG, punit, "makes city at (%d, %d)", 
@@ -1156,9 +1155,13 @@ BUILD_CITY:
     adv_unit_new_task(punit, AUT_AUTO_SETTLER, best_tile);
   }
 
-  auto_settler_setup_work(pplayer, punit, state, 0, path,
-                          best_tile, best_act, &best_target,
-                          completion_time);
+  if (auto_settler_setup_work(pplayer, punit, state, 0, path,
+                              best_tile, best_act, &best_target,
+                              completion_time)) {
+    if (pcity != NULL) {
+      clear_worker_tasks(pcity);
+    }
+  }
 
 CLEANUP:
 
@@ -1183,8 +1186,8 @@ void dai_auto_settler_cont(struct ai_type *ait, struct player *pplayer,
 **************************************************************************/
 void dai_auto_settler_reset(struct ai_type *ait, struct player *pplayer)
 {
-  bool close;
-  struct ai_plr *ai = dai_plr_data_get(ait, pplayer, &close);
+  bool caller_closes;
+  struct ai_plr *ai = dai_plr_data_get(ait, pplayer, &caller_closes);
 
   fc_assert_ret(ai != NULL);
   fc_assert_ret(ai->settler != NULL);
@@ -1204,7 +1207,7 @@ void dai_auto_settler_reset(struct ai_type *ait, struct player *pplayer)
 
   tile_data_cache_hash_clear(ai->settler->tdc_hash);
 
-  if (close) {
+  if (caller_closes) {
     dai_data_phase_finished(ait, pplayer);
   }
 }
@@ -1270,7 +1273,7 @@ static bool dai_do_build_city(struct ai_type *ait, struct player *pplayer,
   initialize_infrastructure_cache(pplayer);
 
   /* Init ai.choice. Handling ferryboats might use it. */
-  init_choice(&def_ai_city_data(pcity, ait)->choice);
+  adv_init_choice(&def_ai_city_data(pcity, ait)->choice);
 
   return TRUE;
 }
@@ -1284,7 +1287,13 @@ void contemplate_new_city(struct ai_type *ait, struct city *pcity)
   struct unit *virtualunit;
   struct tile *pcenter = city_tile(pcity);
   struct player *pplayer = city_owner(pcity);
-  struct unit_type *unit_type = best_role_unit(pcity, UTYF_CITIES); 
+  struct unit_type *unit_type;
+
+  if (game.scenario.prevent_new_cities) {
+    return;
+  }
+
+  unit_type = best_role_unit(pcity, UTYF_CITIES); 
 
   if (unit_type == NULL) {
     log_debug("No UTYF_CITIES role unit available");
