@@ -19,9 +19,6 @@
 #include "ai.h"
 
 /* default ai */
-#include "advdiplomacy.h"
-#include "advdomestic.h"
-#include "advmilitary.h"
 #include "aicity.h"
 #include "aidata.h"
 #include "aiferry.h"
@@ -30,6 +27,9 @@
 #include "aiplayer.h"
 #include "aisettler.h"
 #include "aitools.h"
+#include "daidiplomacy.h"
+#include "daidomestic.h"
+#include "daimilitary.h"
 
 /* threaded ai */
 #include "taimsg.h"
@@ -113,7 +113,7 @@ static void twai_control_gained(struct player *pplayer)
 {
   TAI_AIT;
   TAI_TFUNC(tai_control_gained, pplayer);
-  TAI_DFUNC(dai_assess_danger_player, pplayer);
+  TAI_DFUNC(dai_gained_control, pplayer);
 }
 
 /**************************************************************************
@@ -128,10 +128,21 @@ static void twai_control_lost(struct player *pplayer)
 /**************************************************************************
   Call default ai with threaded ai type as parameter.
 **************************************************************************/
-static void twai_split_by_civil_war(struct player *pplayer)
+static void twai_split_by_civil_war(struct player *original,
+                                    struct player *created)
 {
   TAI_AIT;
-  TAI_DFUNC(dai_assess_danger_player, pplayer);
+  TAI_DFUNC(dai_assess_danger_player, original);
+}
+
+/**************************************************************************
+  Call default ai with threaded ai type as parameter.
+**************************************************************************/
+static void twai_created_by_civil_war(struct player *original,
+                                      struct player *created)
+{
+  TAI_AIT;
+  TAI_DFUNC(dai_player_copy, original, created);
 }
 
 /**************************************************************************
@@ -227,6 +238,16 @@ static void twai_build_adv_adjust(struct player *pplayer, struct city *wonder_ci
   TAI_DFUNC(dai_build_adv_adjust, pplayer, wonder_city);
 }
 
+/**************************************************************************
+  Call default ai with threaded ai type as parameter.
+**************************************************************************/
+static void twai_gov_value(struct player *pplayer, struct government *gov,
+                           adv_want *val, bool *override)
+{
+  TAI_AIT;
+  TAI_DFUNC(dai_gov_value, pplayer, gov, val, override);
+}
+
 /**************************************************************************
   Call default ai with threaded ai type as parameter.
 **************************************************************************/
@@ -368,6 +389,16 @@ static void twai_auto_settler_cont(struct player *pplayer, struct unit *punit,
   TAI_DFUNC(dai_auto_settler_cont, pplayer, punit, state);
 }
 
+/**************************************************************************
+  Call default ai with threaded ai type as parameter.
+**************************************************************************/
+static void twai_switch_to_explore(struct unit *punit, struct tile *target,
+                                  enum override_bool *allow)
+{
+  TAI_AIT;
+  TAI_DFUNC(dai_switch_to_explore, punit, target, allow);
+}
+
 /**************************************************************************
   Call default ai with threaded ai type as parameter.
 **************************************************************************/
@@ -458,7 +489,7 @@ static void twai_unit_log(char *buffer, int buflength, const struct unit *punit)
   Call default ai with threaded ai type as parameter.
 **************************************************************************/
 static void twai_consider_plr_dangerous(struct player *plr1, struct player *plr2,
-                                        enum danger_consideration *result)
+                                        enum override_bool *result)
 {
   TAI_AIT;
   TAI_DFUNC(dai_consider_plr_dangerous, plr1, plr2, result);
@@ -468,7 +499,7 @@ static void twai_consider_plr_dangerous(struct player *plr1, struct player *plr2
   Call default ai with threaded ai type as parameter.
 **************************************************************************/
 static void twai_consider_tile_dangerous(struct tile *ptile, struct unit *punit,
-                                         enum danger_consideration *result)
+                                         enum override_bool *result)
 {
   TAI_AIT;
   TAI_DFUNC(dai_consider_tile_dangerous, ptile, punit, result);
@@ -522,6 +553,7 @@ bool fc_ai_threaded_setup(struct ai_type *ai)
   ai->funcs.gained_control = twai_control_gained;
   ai->funcs.lost_control = twai_control_lost;
   ai->funcs.split_by_civil_war = twai_split_by_civil_war;
+  ai->funcs.created_by_civil_war = twai_created_by_civil_war;
 
   ai->funcs.phase_begin = twai_phase_begin;
   ai->funcs.phase_finished = twai_phase_finished;
@@ -535,6 +567,8 @@ bool fc_ai_threaded_setup(struct ai_type *ai)
   ai->funcs.build_adv_init = twai_build_adv_init;
   ai->funcs.build_adv_adjust_want = twai_build_adv_adjust;
 
+  ai->funcs.gov_value = twai_gov_value;
+
   ai->funcs.units_ruleset_init = twai_units_ruleset_init;
   ai->funcs.units_ruleset_close = twai_units_ruleset_close;
 
@@ -556,7 +590,12 @@ bool fc_ai_threaded_setup(struct ai_type *ai)
   ai->funcs.settler_run = twai_auto_settler_run;
   ai->funcs.settler_cont = twai_auto_settler_cont;
 
+  ai->funcs.want_to_explore = twai_switch_to_explore;
+
   ai->funcs.first_activities = twai_first_activities;
+  /* Do complete run after savegame loaded - we don't know what has been
+     done before. */
+  ai->funcs.restart_phase = twai_first_activities;
   ai->funcs.diplomacy_actions = twai_diplomacy_actions;
   ai->funcs.last_activities = twai_last_activities;
 
