@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -48,14 +48,13 @@
  Note the returned string may not be exactly the right length; that
  is handled later.
 *************************************************************************/
-
 static const char *cr_entry_cityname(const struct city *pcity,
-				     const void *data)
+                                     const void *data)
 {
   /* We used to truncate the name to 14 bytes.  This should not be needed
    * in any modern GUI library and may give an invalid string if a
    * multibyte character is clipped. */
-  return city_name(pcity);
+  return city_name_get(pcity);
 }
 
 /************************************************************************
@@ -72,9 +71,10 @@ static const char *cr_entry_nation(const struct city *pcity,
   allocated and its contents change when this function is called again.
 *************************************************************************/
 static const char *cr_entry_size(const struct city *pcity,
-				 const void *data)
+                                 const void *data)
 {
   static char buf[8];
+
   fc_snprintf(buf, sizeof(buf), "%2d", city_size_get(pcity));
   return buf;
 }
@@ -85,7 +85,7 @@ static const char *cr_entry_size(const struct city *pcity,
   this function is called again.
 *************************************************************************/
 static const char *cr_entry_hstate_concise(const struct city *pcity,
-					   const void *data)
+                                           const void *data)
 {
   static char buf[4];
   fc_snprintf(buf, sizeof(buf), "%s",
@@ -100,9 +100,10 @@ static const char *cr_entry_hstate_concise(const struct city *pcity,
   this function is called again.
 *************************************************************************/
 static const char *cr_entry_hstate_verbose(const struct city *pcity,
-					   const void *data)
+                                           const void *data)
 {
   static char buf[32];
+
   fc_snprintf(buf, sizeof(buf), "%s",
               (city_celebrating(pcity) ? Q_("?city_state:Celebrating")
                : (city_unhappy(pcity) ? Q_("?city_state:Disorder")
@@ -116,9 +117,10 @@ static const char *cr_entry_hstate_verbose(const struct city *pcity,
   this function is called again.
 *************************************************************************/
 static const char *cr_entry_workers(const struct city *pcity,
-				    const void *data)
+                                    const void *data)
 {
   static char buf[32];
+
   fc_snprintf(buf, sizeof(buf), "%d/%d/%d/%d",
               pcity->feel[CITIZEN_HAPPY][FEELING_FINAL],
               pcity->feel[CITIZEN_CONTENT][FEELING_FINAL],
@@ -141,6 +143,46 @@ static const char *cr_entry_happy(const struct city *pcity,
   return buf;
 }
 
+/************************************************************************
+  Returns city total culture written to string
+*************************************************************************/
+static const char *cr_entry_culture(const struct city *pcity,
+                                    const void *data)
+{
+  static char buf[8];
+  fc_snprintf(buf, sizeof(buf), "%3d", pcity->client.culture);
+  return buf;
+}
+
+/************************************************************************
+  Returns city history culture value written to string
+*************************************************************************/
+static const char *cr_entry_history(const struct city *pcity,
+                                    const void *data)
+{
+  static char buf[8];
+  fc_snprintf(buf, sizeof(buf), "%3d", pcity->history);
+  return buf;
+}
+
+/************************************************************************
+  Returns city performance culture value written to string
+*************************************************************************/
+static const char *cr_entry_performance(const struct city *pcity,
+                                        const void *data)
+{
+  static char buf[8];
+
+  /*
+   * Infer the actual performance component of culture from server-supplied
+   * values, rather than using the client's guess at EFT_PERFORMANCE.
+   * XXX: if culture ever gets more complicated than history+performance,
+   * this will need revising, possibly to use a server-supplied value.
+   */
+  fc_snprintf(buf, sizeof(buf), "%3d", pcity->client.culture - pcity->history);
+  return buf;
+}
+
 /************************************************************************
   Returns number of content citizens written to string.
   Returned string is statically allocated and its contents change when
@@ -223,7 +265,7 @@ static const char *cr_entry_attack(const struct city *pcity,
 
   unit_list_iterate(pcity->tile->units, punit) {
     /* What about allied units?  Should we just count them? */
-    attack_best[3] = unit_type(punit)->attack_strength;
+    attack_best[3] = unit_type_get(punit)->attack_strength;
 
     /* Now that the element is appended to the end of the list, we simply
        do an insertion sort. */
@@ -260,12 +302,13 @@ static const char *cr_entry_defense(const struct city *pcity,
 
   unit_list_iterate(pcity->tile->units, punit) {
     /* What about allied units?  Should we just count them? */
-    defense_best[3] = unit_type(punit)->defense_strength;
+    defense_best[3] = unit_type_get(punit)->defense_strength;
 
     /* Now that the element is appended to the end of the list, we simply
        do an insertion sort. */
     for (i = 2; i >= 0 && defense_best[i] < defense_best[i + 1]; i--) {
       int tmp = defense_best[i];
+
       defense_best[i] = defense_best[i + 1];
       defense_best[i + 1] = tmp;
     }
@@ -519,7 +562,7 @@ static const char *cr_entry_building(const struct city *pcity,
   static char buf[192];
   const char *from_worklist =
     worklist_is_empty(&pcity->worklist) ? "" :
-    concise_city_production ? "+" : _("(worklist)");
+    gui_options.concise_city_production ? "+" : _("(worklist)");
 
   if (city_production_has_flag(pcity, IF_GOLD)) {
     fc_snprintf(buf, sizeof(buf), "%s (%d)%s",
@@ -615,6 +658,17 @@ static const char *cr_entry_plague_risk(const struct city *pcity,
   return buf;
 }
 
+/************************************************************************
+  Returns number of continent
+*************************************************************************/
+static const char *cr_entry_continent(const struct city *pcity, 
+                                   const void *data)
+{
+  static char buf[8];
+  fc_snprintf(buf, sizeof(buf), "%3d", pcity->tile->continent);
+  return buf;
+}
+
 /************************************************************************
   Returns city cma description.
   Returned string is statically allocated and its contents change when
@@ -702,6 +756,14 @@ static const struct city_report_spec base_city_report_specs[] = {
     NULL, FUNC_TAG(luxury) },
   { FALSE, 3, 1, NULL, N_("?Science:S"), N_("Economy: Science"),
     NULL, FUNC_TAG(science) },
+  { FALSE, 3, 1, NULL, N_("?Culture:Clt"), N_("Culture (History+Performance)"),
+    NULL, FUNC_TAG(culture) },
+  { FALSE, 3, 1, NULL, N_("?History:Hst"), N_("Culture: History"),
+    NULL, FUNC_TAG(history) },
+  { FALSE, 3, 1, NULL, N_("?Performance:Prf"), N_("Culture: Performance"),
+    NULL, FUNC_TAG(performance) },
+  { FALSE, 3, 1, NULL, N_("?Continent:C"), N_("Continent number"),
+    NULL, FUNC_TAG(continent) },
   { FALSE,  1, 1, N_("?number_trade_routes:n"), N_("?number_trade_routes:R"),
                   N_("Number (and total value) of trade routes"),
     NULL, FUNC_TAG(trade_routes) },
@@ -765,19 +827,20 @@ void init_city_report_game_data(void)
 
   fc_snprintf(sp_explanations, sizeof(sp_explanations),
               "%s", _("Specialists: "));
-  specialist_type_iterate(i) {
-    struct specialist *s = specialist_by_number(i);
+  specialist_type_iterate(sp) {
+    struct specialist *s = specialist_by_number(sp);
+
     p->show = FALSE;
     p->width = 2;
     p->space = 1;
     p->title1 = Q_("?specialist:S");
     p->title2 = specialist_abbreviation_translation(s);
-    fc_snprintf(sp_explanation[i], sizeof(sp_explanation[i]),
+    fc_snprintf(sp_explanation[sp], sizeof(sp_explanation[sp]),
                 _("Specialists: %s"), specialist_plural_translation(s));
     cat_snprintf(sp_explanations, sizeof(sp_explanations),
-                 "%s%s", (i == 0) ? "" : ", ",
+                 "%s%s", (sp == 0) ? "" : ", ",
                  specialist_plural_translation(s));
-    p->explanation = sp_explanation[i];
+    p->explanation = sp_explanation[sp];
     p->data = s;
     p->func = cr_entry_specialist;
     p->tagname = specialist_rule_name(s);
@@ -787,6 +850,7 @@ void init_city_report_game_data(void)
   /* Summary column for all specialists. */
   {
     static char sp_summary[128];
+
     p->show = FALSE;
     p->width = MAX(7, specialist_count()*2-1);
     p->space = 1;
