@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,14 +15,7 @@
 #include <fc_config.h>
 #endif
 
-/* Must be before <windows.h> */
-#ifdef HAVE_WINSOCK
-#ifdef HAVE_WINSOCK2
-#include <winsock2.h>
-#else  /* HAVE_WINSOCK2 */
-#include <winsock.h>
-#endif /* HAVE_WINSOCK2 */
-#endif /* HAVE_WINSOCK */
+#include "fc_prehdrs.h"
 
 #ifdef WIN32_NATIVE
 #include <windows.h>	/* LoadLibrary() */
@@ -39,7 +32,9 @@
 #include "bitvector.h"
 #include "capstr.h"
 #include "dataio.h"
+#include "deprecations.h"
 #include "fcbacktrace.h"
+#include "fc_cmdline.h"
 #include "fciconv.h"
 #include "fcintl.h"
 #include "log.h"
@@ -61,6 +56,7 @@
 #include "netintf.h"
 #include "packets.h"
 #include "player.h"
+#include "research.h"
 #include "version.h"
 
 /* client/include */
@@ -91,9 +87,9 @@
 #include "control.h" 
 #include "editor.h"
 #include "global_worklist.h"
-#include "ggzclient.h"
 #include "helpdata.h"           /* boot_help_texts() */
 #include "mapview_common.h"
+#include "music.h"
 #include "options.h"
 #include "overview_common.h"
 #include "packhand.h"
@@ -101,6 +97,7 @@
 #include "themes_common.h"
 #include "update_queue.h"
 #include "voteinfo.h"
+#include "zoom.h"
 
 /* client/agents */
 #include "agents.h"
@@ -135,9 +132,10 @@ static void fc_interface_init_client(void);
 char *logfile = NULL;
 char *scriptfile = NULL;
 char *savefile = NULL;
-static char tileset_name[512] = "\0";
+char forced_tileset_name[512] = "\0";
 char sound_plugin_name[512] = "\0";
 char sound_set_name[512] = "\0";
+char music_set_name[512] = "\0";
 char server_host[512] = "\0";
 char user_name[512] = "\0";
 char password[MAX_LEN_PASSWORD] = "\0";
@@ -145,7 +143,6 @@ char metaserver[512] = "\0";
 int  server_port = -1;
 bool auto_connect = FALSE; /* TRUE = skip "Connect to Freeciv Server" dialog */
 bool auto_spawn = FALSE; /* TRUE = skip main menu, start local server */
-bool in_ggz = FALSE;
 enum announce_type announce;
 
 struct civclient client;
@@ -160,6 +157,10 @@ bool waiting_for_end_turn = FALSE;
  */
 static bool server_busy = FALSE;
 
+#ifdef DEBUG
+bool hackless = FALSE;
+#endif
+
 static bool client_quitting = FALSE;
 
 /**************************************************************************
@@ -218,12 +219,22 @@ static void charsets_init(void)
   dio_set_get_conv_callback(get_conv);
 }
 
+/**************************************************************************
+ This is called at program exit in any emergency. This is registered
+ as at_quick_exit() callback, so no destructor kind of actions here
+**************************************************************************/
+static void emergency_exit(void)
+{
+  client_kill_server(TRUE);
+}
+
 /**************************************************************************
  This is called at program exit.
 **************************************************************************/
 static void at_exit(void)
 {
-  client_kill_server(TRUE);
+  emergency_exit();
+  packets_deinit();
   fc_shutdown_network();
   update_queue_free();
   fc_destroy_ow_mutex();
@@ -249,6 +260,7 @@ static void client_game_init(void)
               mapimg_client_tile_owner, mapimg_client_tile_city,
               mapimg_client_tile_unit, mapimg_client_plrcolor_count,
               mapimg_client_plrcolor_get);
+  animations_init();
 }
 
 /**************************************************************************
@@ -258,6 +270,7 @@ static void client_game_free(void)
 {
   editgui_popdown_all();
 
+  animations_free();
   mapimg_free();
   packhand_free();
   server_options_free();
@@ -267,6 +280,8 @@ static void client_game_free(void)
   free_help_texts();
   attribute_free();
   agents_free();
+  game.client.ruleset_init = FALSE;
+  game.client.ruleset_ready = FALSE;
   game_free();
   /* update_queue_init() is correct at this point. The queue is reset to
      a clean state which is also needed if the client is not connected to
@@ -309,30 +324,31 @@ int client_main(int argc, char *argv[])
   enum log_level loglevel = LOG_NORMAL;
   int ui_options = 0;
   bool ui_separator = FALSE;
-  char *option=NULL;
-  bool user_tileset = FALSE;
+  char *option = NULL;
   int fatal_assertions = -1;
   int aii;
 
   /* Load win32 post-crash debugger */
 #ifdef WIN32_NATIVE
-# ifndef NDEBUG
+# ifndef FREECIV_NDEBUG
   if (LoadLibrary("exchndl.dll") == NULL) {
-#  ifdef DEBUG
+#  ifdef FREECIV_DEBUG
     fprintf(stderr, "exchndl.dll could not be loaded, no crash debugger\n");
-#  endif /* DEBUG */
+#  endif /* FREECIV_DEBUG */
   }
-# endif /* NDEBUG */
+# endif /* FREECIV_NDEBUG */
 #endif /* WIN32_NATIVE */
 
   i_am_client(); /* Tell to libfreeciv that we are client */
 
   fc_interface_init_client();
 
+  game.client.ruleset_init = FALSE;
+
   /* Ensure that all AIs are initialized to unused state
    * Not using ai_type_iterate as it would stop at
    * current ai type count, ai_type_get_count(), i.e., 0 */
-  for (aii = 0; aii < FC_AI_LAST; aii++) {
+  for (aii = 0; aii < FREECIV_AI_MOD_LAST; aii++) {
     struct ai_type *ai = get_ai_type(aii);
 
     init_ai(ai);
@@ -340,7 +356,7 @@ int client_main(int argc, char *argv[])
 
   init_nls();
 #ifdef ENABLE_NLS
-  (void) bindtextdomain("freeciv-nations", LOCALEDIR);
+  (void) bindtextdomain("freeciv-nations", get_locale_dir());
 #endif
 
   registry_module_init();
@@ -368,32 +384,36 @@ int client_main(int argc, char *argv[])
                     "(IPv4/IPv6/none)"));
       cmdhelp_add(help, "a", "autoconnect",
                   _("Skip connect dialog"));
-#ifdef DEBUG
+#ifdef FREECIV_DEBUG
       cmdhelp_add(help, "d",
                   /* TRANS: "debug" is exactly what user must type, do not translate. */
                   _("debug NUM"),
                   _("Set debug log level (%d to %d, or "
                     "%d:file1,min,max:...)"), LOG_FATAL, LOG_DEBUG,
                   LOG_DEBUG);
-#else
+#else  /* FREECIV_DEBUG */
       cmdhelp_add(help, "d",
                   /* TRANS: "debug" is exactly what user must type, do not translate. */
                   _("debug NUM"),
                   _("Set debug log level (%d to %d)"),
                   LOG_FATAL, LOG_VERBOSE);
-#endif /* DEBUG */
-#ifndef NDEBUG
+#endif /* FREECIV_DEBUG */
+#ifndef FREECIV_NDEBUG
       cmdhelp_add(help, "F",
                   /* TRANS: "Fatal" is exactly what user must type, do not translate. */
                   _("Fatal [SIGNAL]"),
                   _("Raise a signal on failed assertion"));
-#endif /* NDEBUG */
+#endif /* FREECIV_NDEBUG */
       cmdhelp_add(help, "f",
                   /* TRANS: "file" is exactly what user must type, do not translate. */
                   _("file FILE"),
                   _("Load saved game FILE"));
       cmdhelp_add(help, "h", "help",
                   _("Print a summary of the options"));
+#ifdef DEBUG
+      cmdhelp_add(help, "H", "Hackless",
+                  _("Do not request hack access to local, but not spawned, server"));
+#endif /* DEBUG */
       cmdhelp_add(help, "l",
                   /* TRANS: "log" is exactly what user must type, do not translate. */
                   _("log FILE"),
@@ -427,12 +447,18 @@ int client_main(int argc, char *argv[])
                   /* TRANS: "Sound" is exactly what user must type, do not translate. */
                   _("Sound FILE"),
                   _("Read sound tags from FILE"));
+      cmdhelp_add(help, "m",
+                  /* TRANS: "music" is exactly what user must type, do not translate. */
+                  _("music FILE"),
+                  _("Read music tags from FILE"));
       cmdhelp_add(help, "t",
                   /* TRANS: "tiles" is exactly what user must type, do not translate. */
                   _("tiles FILE"),
                   _("Use data file FILE.tilespec for tiles"));
       cmdhelp_add(help, "v", "version",
                   _("Print the version number"));
+      cmdhelp_add(help, "w", "warnings",
+                  _("Warn about deprecated modpack constructs"));
 
       /* The function below prints a header and footer for the options.
        * Furthermore, the options are sorted. */
@@ -440,12 +466,16 @@ int client_main(int argc, char *argv[])
       cmdhelp_destroy(help);
 
       exit(EXIT_SUCCESS);
-    } else if (is_option("--version",argv[i])) {
+    } else if (is_option("--version", argv[i])) {
       fc_fprintf(stderr, "%s %s\n", freeciv_name_version(), client_string);
       exit(EXIT_SUCCESS);
-    } else if ((option = get_option_malloc("--log", argv, &i, argc))) {
-      logfile = option; /* never free()d */
-#ifndef NDEBUG
+#ifdef FREECIV_DEBUG
+    } else if (is_option("--Hackless", argv[i])) {
+      hackless = TRUE;
+#endif /* FREECIV_DEBUG */
+    } else if ((option = get_option_malloc("--log", argv, &i, argc, TRUE))) {
+      logfile = option;
+#ifndef FREECIV_NDEBUG
     } else if (is_option("--Fatal", argv[i])) {
       if (i + 1 >= argc || '-' == argv[i + 1][0]) {
         fatal_assertions = SIGABRT;
@@ -457,25 +487,28 @@ int client_main(int argc, char *argv[])
         fc_fprintf(stderr, _("Try using --help.\n"));
         exit(EXIT_FAILURE);
       }
-#endif /* NDEBUG */
-    } else  if ((option = get_option_malloc("--read", argv, &i, argc))) {
-      scriptfile = option; /* never free()d */
-    } else if ((option = get_option_malloc("--file", argv, &i, argc))) {
-      savefile = option; /* never free()d */
+#endif /* FREECIV_NDEBUG */
+    } else  if ((option = get_option_malloc("--read", argv, &i, argc, TRUE))) {
+      scriptfile = option;
+    } else if ((option = get_option_malloc("--file", argv, &i, argc, TRUE))) {
+      savefile = option;
       auto_spawn = TRUE;
-    } else if ((option = get_option_malloc("--name", argv, &i, argc))) {
+    } else if ((option = get_option_malloc("--name", argv, &i, argc, FALSE))) {
       sz_strlcpy(user_name, option);
       free(option);
-    } else if ((option = get_option_malloc("--Meta", argv, &i, argc))) {
+    } else if ((option = get_option_malloc("--Meta", argv, &i, argc, FALSE))) {
       sz_strlcpy(metaserver, option);
       free(option);
-    } else if ((option = get_option_malloc("--Sound", argv, &i, argc))) {
+    } else if ((option = get_option_malloc("--Sound", argv, &i, argc, FALSE))) {
       sz_strlcpy(sound_set_name, option);
       free(option);
-    } else if ((option = get_option_malloc("--Plugin", argv, &i, argc))) {
+    } else if ((option = get_option_malloc("--music", argv, &i, argc, FALSE))) {
+      sz_strlcpy(music_set_name, option);
+      free(option);
+    } else if ((option = get_option_malloc("--Plugin", argv, &i, argc, FALSE))) {
       sz_strlcpy(sound_plugin_name, option);
       free(option);
-    } else if ((option = get_option_malloc("--port",argv,&i,argc))) {
+    } else if ((option = get_option_malloc("--port", argv, &i, argc, FALSE))) {
       if (!str_to_int(option, &server_port)) {
         fc_fprintf(stderr,
                    _("Invalid port \"%s\" specified with --port option.\n"),
@@ -484,12 +517,12 @@ int client_main(int argc, char *argv[])
         exit(EXIT_FAILURE);
       }
       free(option);
-    } else if ((option = get_option_malloc("--server", argv, &i, argc))) {
+    } else if ((option = get_option_malloc("--server", argv, &i, argc, FALSE))) {
       sz_strlcpy(server_host, option);
       free(option);
     } else if (is_option("--autoconnect", argv[i])) {
       auto_connect = TRUE;
-    } else if ((option = get_option_malloc("--debug", argv, &i, argc))) {
+    } else if ((option = get_option_malloc("--debug", argv, &i, argc, FALSE))) {
       if (!log_parse_level_str(option, &loglevel)) {
         fc_fprintf(stderr,
                    _("Invalid debug level \"%s\" specified with --debug "
@@ -498,16 +531,15 @@ int client_main(int argc, char *argv[])
         exit(EXIT_FAILURE);
       }
       free(option);
-    } else if ((option = get_option_malloc("--tiles", argv, &i, argc))) {
-      sz_strlcpy(tileset_name, option);
+    } else if ((option = get_option_malloc("--tiles", argv, &i, argc, FALSE))) {
+      sz_strlcpy(forced_tileset_name, option);
       free(option);
-      user_tileset = TRUE;
-    } else if ((option = get_option_malloc("--Announce", argv, &i, argc))) {
+    } else if ((option = get_option_malloc("--Announce", argv, &i, argc, FALSE))) {
       if (!strcasecmp(option, "ipv4")) {
         announce = ANNOUNCE_IPV4;
       } else if (!strcasecmp(option, "none")) {
         announce = ANNOUNCE_NONE;
-#ifdef IPV6_SUPPORT
+#ifdef FREECIV_IPV6_SUPPORT
       } else if(!strcasecmp(option, "ipv6")) {
         announce = ANNOUNCE_IPV6;
 #endif /* IPv6 support */
@@ -516,6 +548,8 @@ int client_main(int argc, char *argv[])
         exit(EXIT_FAILURE);
       }
       free(option);
+    } else if (is_option("--warnings", argv[i])) {
+      deprecation_warnings_enable();
     } else if (is_option("--", argv[i])) {
       ui_separator = TRUE;
     } else {
@@ -544,15 +578,16 @@ int client_main(int argc, char *argv[])
 
   /* after log_init: */
 
-  (void)user_username(default_user_name, MAX_LEN_NAME);
-  if (!is_valid_username(default_user_name)) {
-    char buf[sizeof(default_user_name)];
+  (void)user_username(gui_options.default_user_name, MAX_LEN_NAME);
+  if (!is_valid_username(gui_options.default_user_name)) {
+    char buf[sizeof(gui_options.default_user_name)];
 
-    fc_snprintf(buf, sizeof(buf), "_%s", default_user_name);
+    fc_snprintf(buf, sizeof(buf), "_%s", gui_options.default_user_name);
     if (is_valid_username(buf)) {
-      sz_strlcpy(default_user_name, buf);
+      sz_strlcpy(gui_options.default_user_name, buf);
     } else {
-      fc_snprintf(default_user_name, sizeof(default_user_name),
+      fc_snprintf(gui_options.default_user_name,
+                  sizeof(gui_options.default_user_name),
                   "player%d", fc_rand(10000));
     }
   }
@@ -571,6 +606,7 @@ int client_main(int argc, char *argv[])
 
   /* register exit handler */ 
   atexit(at_exit);
+  fc_at_quick_exit(emergency_exit);
 
   init_our_capability();
   init_player_dlg_common();
@@ -581,48 +617,67 @@ int client_main(int argc, char *argv[])
 
   script_client_init();
 
-  if (tileset_name[0] == '\0') {
-    sz_strlcpy(tileset_name, default_tileset_name);
+  if (sound_set_name[0] == '\0') {
+    sz_strlcpy(sound_set_name, gui_options.default_sound_set_name);
+  }
+  if (music_set_name[0] == '\0') {
+    sz_strlcpy(music_set_name, gui_options.default_music_set_name);
+  }
+  if (sound_plugin_name[0] == '\0') {
+    sz_strlcpy(sound_plugin_name, gui_options.default_sound_plugin_name); 
+  }
+  if (server_host[0] == '\0') {
+    sz_strlcpy(server_host, gui_options.default_server_host);
+  } else if (gui_options.use_prev_server) {
+    sz_strlcpy(gui_options.default_server_host, server_host);
+  }
+  if (user_name[0] == '\0') {
+    sz_strlcpy(user_name, gui_options.default_user_name); 
   }
-  if (sound_set_name[0] == '\0') 
-    sz_strlcpy(sound_set_name, default_sound_set_name); 
-  if (sound_plugin_name[0] == '\0')
-    sz_strlcpy(sound_plugin_name, default_sound_plugin_name); 
-  if (server_host[0] == '\0')
-    sz_strlcpy(server_host, default_server_host); 
-  if (user_name[0] == '\0')
-    sz_strlcpy(user_name, default_user_name); 
   if (metaserver[0] == '\0') {
     /* FIXME: Find a cleaner way to achieve this. */
     /* www.cazfi.net/freeciv/metaserver/ was default metaserver
      * over one release when meta.freeciv.org was unavailable. */
     const char *oldaddr = "http://www.cazfi.net/freeciv/metaserver/";
-    if (0 == strcmp(default_metaserver, oldaddr)) {
+
+    if (0 == strcmp(gui_options.default_metaserver, oldaddr)) {
       log_normal(_("Updating old metaserver address \"%s\"."), oldaddr);
-      sz_strlcpy(default_metaserver, DEFAULT_METASERVER_OPTION);
+      sz_strlcpy(gui_options.default_metaserver, DEFAULT_METASERVER_OPTION);
       log_normal(_("Default metaserver has been set to value \"%s\"."),
                  DEFAULT_METASERVER_OPTION);
     }
-    if (0 == strcmp(default_metaserver, DEFAULT_METASERVER_OPTION)) {
-      sz_strlcpy(metaserver, META_URL);
+    if (0 == strcmp(gui_options.default_metaserver, DEFAULT_METASERVER_OPTION)) {
+      sz_strlcpy(metaserver, FREECIV_META_URL);
     } else {
-      sz_strlcpy(metaserver, default_metaserver);
+      sz_strlcpy(metaserver, gui_options.default_metaserver);
     }
   }
-  if (server_port == -1) server_port = default_server_port;
+  if (server_port == -1) {
+    server_port = gui_options.default_server_port;
+  } else if (gui_options.use_prev_server) {
+    gui_options.default_server_port = server_port;
+  }
 
   /* This seed is not saved anywhere; randoms in the client should
      have cosmetic effects only (eg city name suggestions).  --dwp */
   fc_srand(time(NULL));
   helpdata_init();
-  boot_help_texts(NULL);
+  boot_help_texts();
 
-  tilespec_try_read(tileset_name, user_tileset);
+  fill_topo_ts_default();
 
-  audio_real_init(sound_set_name, sound_plugin_name);
-  audio_play_music("music_start", NULL);
+  if (forced_tileset_name[0] != '\0') {
+    if (!tilespec_try_read(forced_tileset_name, TRUE, -1, TRUE)) {
+      log_error(_("Can't load requested tileset %s!"), forced_tileset_name);
+      client_exit();
+      return EXIT_FAILURE;
+    }
+  } else {
+    tilespec_try_read(gui_options.default_tileset_name, FALSE, -1, TRUE);
+  }
 
-  ggz_initialize();
+  audio_real_init(sound_set_name, music_set_name, sound_plugin_name);
+  start_menu_music("music_menu", NULL);
 
   editor_init();
 
@@ -659,7 +714,7 @@ void client_exit(void)
     client_remove_all_cli_conn();
   }
 
-  if (save_options_on_exit) {
+  if (gui_options.save_options_on_exit) {
     options_save(log_option_save_msg);
   }
 
@@ -686,6 +741,7 @@ void client_exit(void)
 
   backtrace_deinit();
   log_close();
+  cmdline_option_values_free();
 
   exit(EXIT_SUCCESS);
 }
@@ -703,7 +759,8 @@ void client_packet_input(void *packet, int type)
       && PACKET_SERVER_JOIN_REPLY != type
       && PACKET_AUTHENTICATION_REQ != type
       && PACKET_SERVER_SHUTDOWN != type
-      && PACKET_CONNECT_MSG != type) {
+      && PACKET_CONNECT_MSG != type
+      && PACKET_EARLY_CHAT_MSG != type) {
     log_error("Received packet %s (%d) before establishing connection!",
               packet_name(type), type);
     disconnect_from_server();
@@ -797,9 +854,11 @@ void set_client_state(enum client_states newstate)
   }
 
   if (oldstate == C_S_RUNNING && newstate != C_S_PREPARING) {
+    stop_style_music();
+
     if (!is_client_quitting()) {
       /* Back to menu */
-      audio_play_music("music_start", NULL);
+      start_menu_music("music_menu", NULL);
     }
   }
 
@@ -828,9 +887,7 @@ void set_client_state(enum client_states newstate)
       }
     }
 
-    if (!with_ggz) {
-      set_client_page(in_ggz ? PAGE_GGZ : PAGE_MAIN);
-    }
+    set_client_page(PAGE_MAIN);
     break;
 
   case C_S_PREPARING:
@@ -859,18 +916,17 @@ void set_client_state(enum client_states newstate)
   case C_S_RUNNING:
     if (oldstate == C_S_PREPARING) {
       popdown_races_dialog();
-      audio_stop();     /* stop intro sound loop. */
+      stop_menu_music();     /* stop intro sound loop. */
     }
 
     init_city_report_game_data();
     options_dialogs_set();
     create_event(NULL, E_GAME_START, ftc_client, _("Game started."));
-    precalc_tech_data();
     if (pplayer) {
-      player_research_update(pplayer);
+      research_update(research_get(pplayer));
     }
     role_unit_precalcs();
-    boot_help_texts(pplayer);   /* reboot with player */
+    boot_help_texts();   /* reboot with player */
     global_worklists_build();
     can_slide = FALSE;
     unit_focus_update();
@@ -889,9 +945,10 @@ void set_client_state(enum client_states newstate)
     unit_focus_update();
     update_unit_info_label(get_units_in_focus());
 
-    if (auto_center_each_turn) {
+    if (gui_options.auto_center_each_turn) {
       center_on_something();
     }
+    start_style_music();
     break;
 
   case C_S_OVER:
@@ -912,12 +969,11 @@ void set_client_state(enum client_states newstate)
       /* From C_S_PREPARING. */
       init_city_report_game_data();
       options_dialogs_set();
-      precalc_tech_data();
       if (pplayer) {
-        player_research_update(pplayer);
+        research_update(research_get(pplayer));
       }
       role_unit_precalcs();
-      boot_help_texts(pplayer);            /* reboot */
+      boot_help_texts();            /* reboot */
       global_worklists_build();
       unit_focus_set(NULL);
       set_client_page(PAGE_GAME);
@@ -956,6 +1012,9 @@ enum client_states client_state(void)
 **************************************************************************/
 void client_remove_cli_conn(struct connection *pconn)
 {
+  fc_assert_msg(pconn != NULL,
+                "Trying to remove a non existing connection");
+
   if (NULL != pconn->playing) {
     conn_list_remove(pconn->playing->connections, pconn);
   }
@@ -971,6 +1030,9 @@ void client_remove_cli_conn(struct connection *pconn)
 **************************************************************************/
 void client_remove_all_cli_conn(void)
 {
+  fc_assert_msg(game.all_connections != NULL,
+                "Connection list missing");
+
   while (conn_list_size(game.all_connections) > 0) {
     struct connection *pconn = conn_list_get(game.all_connections, 0);
     client_remove_cli_conn(pconn);
@@ -1008,9 +1070,15 @@ bool client_is_observer(void)
 static double seconds_to_turndone = 0.0;
 static struct timer *turndone_timer;
 
+/* The timer tells how long since server informed us about starting
+ * turn-change activities. */
+static struct timer *between_turns = NULL;
+static bool waiting_turn_change = FALSE;
+
 /* This value shows what value the timeout label is currently showing for
  * the seconds-to-turndone. */
 static int seconds_shown_to_turndone;
+static int seconds_shown_to_new_turn;
 
 /**************************************************************************
   Reset the number of seconds to turndone from an "authentic" source.
@@ -1020,7 +1088,7 @@ static int seconds_shown_to_turndone;
 **************************************************************************/
 void set_seconds_to_turndone(double seconds)
 {
-  if (client_current_turn_timeout() > 0) {
+  if (current_turn_timeout() > 0) {
     seconds_to_turndone = seconds;
     turndone_timer = timer_renew(turndone_timer, TIMER_USER, TIMER_ACTIVE);
     timer_start(turndone_timer);
@@ -1032,12 +1100,41 @@ void set_seconds_to_turndone(double seconds)
 }
 
 /**************************************************************************
-  Return the number of seconds until turn-done.  Don't call this unless
-  client_current_turn_timeout() != 0.
+  Are we in turn-change wait state?
+**************************************************************************/
+bool is_waiting_turn_change(void)
+{
+  return waiting_turn_change;
+}
+
+/**************************************************************************
+  Start waiting of the server turn change activities.
+**************************************************************************/
+void start_turn_change_wait(void)
+{
+  seconds_shown_to_new_turn = ceil(game.tinfo.last_turn_change_time) + 0.1;
+  between_turns = timer_renew(between_turns, TIMER_USER, TIMER_ACTIVE);
+  timer_start(between_turns);
+
+  waiting_turn_change = TRUE;
+}
+
+/**************************************************************************
+  Server is responsive again
+**************************************************************************/
+void stop_turn_change_wait(void)
+{
+  waiting_turn_change = FALSE;
+  update_timeout_label();
+}
+
+/**************************************************************************
+  Return the number of seconds until turn-done. Don't call this unless
+  current_turn_timeout() != 0.
 **************************************************************************/
 int get_seconds_to_turndone(void)
 {
-  if (client_current_turn_timeout() > 0) {
+  if (current_turn_timeout() > 0) {
     return seconds_shown_to_turndone;
   } else {
     /* This shouldn't happen. */
@@ -1045,6 +1142,15 @@ int get_seconds_to_turndone(void)
   }
 }
 
+/**************************************************************************
+  Return the number of seconds until turn-done.  Don't call this unless
+  game.info.timeout != 0.
+**************************************************************************/
+int get_seconds_to_new_turn(void)
+{
+  return seconds_shown_to_new_turn;
+}
+
 /**************************************************************************
   This function should be called at least once per second.  It does various
   updates (idle animations and timeout updates).  It returns the number of
@@ -1065,6 +1171,8 @@ double real_timer_callback(void)
     return time_until_next_call;
   }
 
+  time_until_next_call = zoom_update(time_until_next_call);
+
   {
     double blink_time = blink_turn_done_button();
 
@@ -1077,9 +1185,9 @@ double real_timer_callback(void)
     time_until_next_call = MIN(time_until_next_call, blink_time);
   }
 
-  /* It is possible to have client_current_turn_timeout() > 0 but !turndone_timer,
+  /* It is possible to have current_turn_timeout() > 0 but !turndone_timer,
    * in the first moments after the timeout is set. */
-  if (client_current_turn_timeout() > 0 && turndone_timer) {
+  if (current_turn_timeout() > 0 && turndone_timer) {
     double seconds = seconds_to_turndone - timer_read_seconds(turndone_timer);
     int iseconds = ceil(seconds) + 0.1; /* Turn should end right on 0. */
 
@@ -1091,6 +1199,25 @@ double real_timer_callback(void)
     time_until_next_call = MIN(time_until_next_call,
 			       seconds - floor(seconds) + 0.001);
   }
+  if (waiting_turn_change) {
+    double seconds = game.tinfo.last_turn_change_time - timer_read_seconds(between_turns);
+    int iseconds = ceil(seconds) + 0.1; /* Turn should end right on 0. */
+
+    if (iseconds < game.tinfo.last_turn_change_time) {
+      seconds_shown_to_new_turn = iseconds;
+      update_timeout_label();
+    }
+  }
+
+  {
+    static long counter = 0;
+
+    counter++;
+
+    if (gui_options.heartbeat_enabled && (counter % (20 * 10) == 0)) {
+      send_packet_client_heartbeat(&client.conn);
+    }
+  }
 
   /* Make sure we wait at least 50 ms, otherwise we may not give any other
    * code time to run. */
@@ -1227,7 +1354,8 @@ static void fc_interface_init_client(void)
 {
   struct functions *funcs = fc_interface_funcs();
 
-  funcs->destroy_base = NULL;
+  funcs->create_extra = NULL;
+  funcs->destroy_extra = NULL;
   funcs->player_tile_vision_get = client_map_is_known_and_seen;
   funcs->gui_color_free = color_free;
 
@@ -1332,26 +1460,6 @@ static struct rgbcolor *mapimg_client_plrcolor_get(int i)
   return NULL;
 }
 
-/****************************************************************************
-  Return timeout value for the current turn.
-****************************************************************************/
-int client_current_turn_timeout(void)
-{
-  if (game.info.turn == 0) {
-    struct option *opt = optset_option_by_name(server_optset, "first_timeout");
-
-    if (opt != NULL) {
-      int val = option_int_get(opt);
-
-      if (val != -1) {
-        return val;
-      }
-    }
-  }
-
-  return game.info.timeout;
-}
-
 /****************************************************************************
   Is the client marked as one going down?
 ****************************************************************************/
