@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,9 +11,9 @@
    GNU General Public License for more details.
 ***********************************************************************/
 
-/********************************************************************** 
-This module contains various general - mostly highlevel - functions
-used throughout the client.
+/*********************************************************************** 
+  This module contains various general - mostly highlevel - functions
+  used throughout the client.
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
@@ -111,7 +111,7 @@ void client_remove_unit(struct unit *punit)
     }
 
     log_debug("map city %s, %s, (%d %d)",
-              city_name(pcity), nation_rule_name(nation_of_city(pcity)),
+              city_name_get(pcity), nation_rule_name(nation_of_city(pcity)),
               TILE_XY(city_tile(pcity)));
   }
 
@@ -120,7 +120,7 @@ void client_remove_unit(struct unit *punit)
     if (NULL != pcity) {
       refresh_city_dialog(pcity);
       log_debug("home city %s, %s, (%d %d)",
-                city_name(pcity), nation_rule_name(nation_of_city(pcity)),
+                city_name_get(pcity), nation_rule_name(nation_of_city(pcity)),
                 TILE_XY(city_tile(pcity)));
     }
   }
@@ -137,11 +137,11 @@ void client_remove_city(struct city *pcity)
   struct tile *ptile = city_tile(pcity);
   struct city old_city = *pcity;
 
-  log_debug("client_remove_city() %d, %s", pcity->id, city_name(pcity));
+  log_debug("client_remove_city() %d, %s", pcity->id, city_name_get(pcity));
 
   /* Explicitly remove all improvements, to properly remove any global effects
      and to handle the preservation of "destroyed" effects. */
-  effect_update=FALSE;
+  effect_update = FALSE;
 
   city_built_iterate(pcity, pimprove) {
     effect_update = TRUE;
@@ -159,10 +159,10 @@ void client_remove_city(struct city *pcity)
 }
 
 /**************************************************************************
-Change all cities building X to building Y, if possible.  X and Y
-could be improvements or units. X and Y are compound ids.
+  Change all cities building X to building Y, if possible.  X and Y
+  could be improvements or units. X and Y are compound ids.
 **************************************************************************/
-void client_change_all(struct universal from, struct universal to)
+void client_change_all(struct universal *from, struct universal *to)
 {
   if (!can_client_issue_orders()) {
     return;
@@ -170,16 +170,16 @@ void client_change_all(struct universal from, struct universal to)
 
   create_event(NULL, E_CITY_PRODUCTION_CHANGED, ftc_client,
                _("Changing production of every %s into %s."),
-               VUT_UTYPE == from.kind
-               ? utype_name_translation(from.value.utype)
-               : improvement_name_translation(from.value.building),
-               VUT_UTYPE == to.kind
-               ? utype_name_translation(to.value.utype)
-               : improvement_name_translation(to.value.building));
+               VUT_UTYPE == from->kind
+               ? utype_name_translation(from->value.utype)
+               : improvement_name_translation(from->value.building),
+               VUT_UTYPE == to->kind
+               ? utype_name_translation(to->value.utype)
+               : improvement_name_translation(to->value.building));
 
   connection_do_buffer(&client.conn);
   city_list_iterate (client.conn.playing->cities, pcity) {
-    if (are_universals_equal(&pcity->production, &from)
+    if (are_universals_equal(&pcity->production, from)
         && can_city_build_now(pcity, to)) {
       city_change_production(pcity, to);
     }
@@ -236,7 +236,7 @@ const char *get_vision_status(const struct player *me,
 }
 
 /**************************************************************************
-Copy a string that describes the given clause into the return buffer.
+  Copy a string that describes the given clause into the return buffer.
 **************************************************************************/
 void client_diplomacy_clause_string(char *buf, int bufsiz,
 				    struct Clause *pclause)
@@ -247,14 +247,14 @@ void client_diplomacy_clause_string(char *buf, int bufsiz,
   case CLAUSE_ADVANCE:
     fc_snprintf(buf, bufsiz, _("The %s give %s"),
                 nation_plural_for_player(pclause->from),
-                advance_name_for_player(client.conn.playing, pclause->value));
+                advance_name_translation(advance_by_number(pclause->value)));
     break;
   case CLAUSE_CITY:
     pcity = game_city_by_number(pclause->value);
     if (pcity) {
       fc_snprintf(buf, bufsiz, _("The %s give %s"),
                   nation_plural_for_player(pclause->from),
-                  city_name(pcity));
+                  city_name_get(pcity));
     } else {
       fc_snprintf(buf, bufsiz,_("The %s give an unknown city"),
                   nation_plural_for_player(pclause->from));
@@ -348,20 +348,18 @@ void nuclear_winter_scaled(int *chance, int *rate, int max)
 struct sprite *client_research_sprite(void)
 {
   if (NULL != client.conn.playing && can_client_change_view()) {
-    const struct player_research *presearch =
-        player_research_get(client_player());
-    int index = 0;
+    const struct research *presearch = research_get(client_player());
+    int idx = 0;
 
-    if (is_future_tech(presearch->researching)
-        || NULL != valid_advance_by_number(presearch->researching)) {
-      index = (NUM_TILES_PROGRESS * presearch->bulbs_researched
-               / (presearch->client.researching_cost + 1));
+    if (A_UNSET != presearch->researching) {
+      idx = (NUM_TILES_PROGRESS * presearch->bulbs_researched
+             / (presearch->client.researching_cost + 1));
     }
 
     /* This clipping can be necessary since we can end up with excess
      * research */
-    index = CLIP(0, index, NUM_TILES_PROGRESS - 1);
-    return get_indicator_sprite(tileset, INDICATOR_BULB, index);
+    idx = CLIP(0, idx, NUM_TILES_PROGRESS - 1);
+    return get_indicator_sprite(tileset, INDICATOR_BULB, idx);
   } else {
     return get_indicator_sprite(tileset, INDICATOR_BULB, 0);
   }
@@ -372,15 +370,16 @@ struct sprite *client_research_sprite(void)
 **************************************************************************/
 struct sprite *client_warming_sprite(void)
 {
-  int index;
+  int idx;
+
   if (can_client_change_view()) {
     /* Highest sprite kicks in at about 25% risk */
-    global_warming_scaled(&index, NULL, (NUM_TILES_PROGRESS-1)*4);
-    index = CLIP(0, index, NUM_TILES_PROGRESS-1);
+    global_warming_scaled(&idx, NULL, (NUM_TILES_PROGRESS-1)*4);
+    idx = CLIP(0, idx, NUM_TILES_PROGRESS-1);
   } else {
-    index = 0;
+    idx = 0;
   }
-  return get_indicator_sprite(tileset, INDICATOR_WARMING, index);
+  return get_indicator_sprite(tileset, INDICATOR_WARMING, idx);
 }
 
 /**************************************************************************
@@ -388,15 +387,16 @@ struct sprite *client_warming_sprite(void)
 **************************************************************************/
 struct sprite *client_cooling_sprite(void)
 {
-  int index;
+  int idx;
+
   if (can_client_change_view()) {
     /* Highest sprite kicks in at about 25% risk */
-    nuclear_winter_scaled(&index, NULL, (NUM_TILES_PROGRESS-1)*4);
-    index = CLIP(0, index, NUM_TILES_PROGRESS-1);
+    nuclear_winter_scaled(&idx, NULL, (NUM_TILES_PROGRESS-1)*4);
+    idx = CLIP(0, idx, NUM_TILES_PROGRESS-1);
   } else {
-    index = 0;
+    idx = 0;
   }
-  return get_indicator_sprite(tileset, INDICATOR_COOLING, index);
+  return get_indicator_sprite(tileset, INDICATOR_COOLING, idx);
 }
 
 /**************************************************************************
@@ -449,14 +449,14 @@ void center_on_something(void)
     fc_assert_ret(punit != NULL);
     center_tile_mapcanvas(unit_tile(punit));
   } else {
-    struct tile *ctile = native_pos_to_tile(map.xsize / 2, map.ysize / 2);
+    struct tile *ctile = native_pos_to_tile(game.map.xsize / 2, game.map.ysize / 2);
 
     /* Just any known tile will do; search near the middle first. */
     /* Iterate outward from the center tile.  We have to give a radius that
      * is guaranteed to be larger than the map will be.  Although this is
      * a misuse of map.xsize and map.ysize (which are native dimensions),
      * it should give a sufficiently large radius. */
-    iterate_outward(ctile, map.xsize + map.ysize, ptile) {
+    iterate_outward(ctile, game.map.xsize + game.map.ysize, ptile) {
       if (client_tile_get_known(ptile) != TILE_UNKNOWN) {
 	ctile = ptile;
 	break;
@@ -514,16 +514,16 @@ cid cid_encode_from_city(const struct city *pcity)
 /**************************************************************************
   Decode the CID into a city_production structure.
 **************************************************************************/
-struct universal cid_decode(cid cid)
+struct universal cid_decode(cid id)
 {
   struct universal target;
 
-  if (cid >= B_LAST) {
+  if (id >= B_LAST) {
     target.kind = VUT_UTYPE;
-    target.value.utype = utype_by_number(cid - B_LAST);
+    target.value.utype = utype_by_number(id - B_LAST);
   } else {
     target.kind = VUT_IMPROVEMENT;
-    target.value.building = improvement_by_number(cid);
+    target.value.building = improvement_by_number(id);
   }
 
   return target;
@@ -534,14 +534,15 @@ struct universal cid_decode(cid cid)
   production type (returns FALSE if the production is a building).
 ****************************************************************************/
 bool city_unit_supported(const struct city *pcity,
-			 struct universal target)
+                         const struct universal *target)
 {
-  if (VUT_UTYPE == target.kind) {
-    struct unit_type *tvtype = target.value.utype;
+  if (VUT_UTYPE == target->kind) {
+    struct unit_type *tvtype = target->value.utype;
 
     unit_list_iterate(pcity->units_supported, punit) {
-      if (unit_type(punit) == tvtype)
-	return TRUE;
+      if (unit_type_get(punit) == tvtype) {
+        return TRUE;
+      }
     } unit_list_iterate_end;
   }
   return FALSE;
@@ -552,14 +553,15 @@ bool city_unit_supported(const struct city *pcity,
   production type (returns FALSE if the production is a building).
 ****************************************************************************/
 bool city_unit_present(const struct city *pcity,
-		       struct universal target)
+                       const struct universal *target)
 {
-  if (VUT_UTYPE == target.kind) {
-    struct unit_type *tvtype = target.value.utype;
+  if (VUT_UTYPE == target->kind) {
+    struct unit_type *tvtype = target->value.utype;
 
     unit_list_iterate(pcity->tile->units, punit) {
-      if (unit_type(punit) == tvtype)
-	return TRUE;
+      if (unit_type_get(punit) == tvtype) {
+        return TRUE;
+      }
     }
     unit_list_iterate_end;
   }
@@ -570,10 +572,10 @@ bool city_unit_present(const struct city *pcity,
   A TestCityFunc to tell whether the item is a building and is present.
 ****************************************************************************/
 bool city_building_present(const struct city *pcity,
-			   struct universal target)
+                           const struct universal *target)
 {
-  return VUT_IMPROVEMENT == target.kind
-      && city_has_building(pcity, target.value.building);
+  return VUT_IMPROVEMENT == target->kind
+    && city_has_building(pcity, target->value.building);
 }
 
 /**************************************************************************
@@ -603,7 +605,7 @@ static int target_get_section(struct universal target)
 /**************************************************************************
  Helper for name_and_sort_items.
 **************************************************************************/
-static int my_cmp(const void *p1, const void *p2)
+static int fc_cmp(const void *p1, const void *p2)
 {
   const struct item *i1 = p1, *i2 = p2;
   int s1 = target_get_section(i1->item);
@@ -664,7 +666,7 @@ void name_and_sort_items(struct universal *targets, int num_targets,
     }
   }
 
-  qsort(items, num_targets, sizeof(struct item), my_cmp);
+  qsort(items, num_targets, sizeof(struct item), fc_cmp);
 }
 
 /**************************************************************************
@@ -673,21 +675,21 @@ void name_and_sort_items(struct universal *targets, int num_targets,
   FIXME: this should probably take a pplayer argument.
 **************************************************************************/
 int collect_production_targets(struct universal *targets,
-			       struct city **selected_cities,
-			       int num_selected_cities, bool append_units,
-			       bool append_wonders, bool change_prod,
-			       TestCityFunc test_func)
+                               struct city **selected_cities,
+                               int num_selected_cities, bool append_units,
+                               bool append_wonders, bool change_prod,
+                               TestCityFunc test_func)
 {
   cid first = append_units ? B_LAST : 0;
   cid last = (append_units
-	      ? utype_count() + B_LAST
-	      : improvement_count());
-  cid cid;
+              ? utype_count() + B_LAST
+              : improvement_count());
+  cid id;
   int items_used = 0;
 
-  for (cid = first; cid < last; cid++) {
+  for (id = first; id < last; id++) {
     bool append = FALSE;
-    struct universal target = cid_decode(cid);
+    struct universal target = cid_decode(id);
 
     if (!append_units && (append_wonders != is_wonder(target.value.building))) {
       continue;
@@ -696,18 +698,18 @@ int collect_production_targets(struct universal *targets,
     if (!change_prod) {
       if (client_has_player()) {
         city_list_iterate(client_player()->cities, pcity) {
-          append |= test_func(pcity, cid_decode(cid));
+          append |= test_func(pcity, &target);
         } city_list_iterate_end;
       } else {
         cities_iterate(pcity) {
-          append |= test_func(pcity, cid_decode(cid));
+          append |= test_func(pcity, &target);
         } cities_iterate_end;
       }
     } else {
       int i;
 
       for (i = 0; i < num_selected_cities; i++) {
-	append |= test_func(selected_cities[i], cid_decode(cid));
+        append |= test_func(selected_cities[i], &target);
       }
     }
 
@@ -730,7 +732,7 @@ int collect_currently_building_targets(struct universal *targets)
 {
   bool mapping[MAX_NUM_PRODUCTION_TARGETS];
   int cids_used = 0;
-  cid cid;
+  cid id;
 
   if (NULL == client.conn.playing) {
     return 0;
@@ -742,12 +744,13 @@ int collect_currently_building_targets(struct universal *targets)
   }
   city_list_iterate_end;
 
-  for (cid = 0; cid < ARRAY_SIZE(mapping); cid++) {
-    if (mapping[cid]) {
-      targets[cids_used] = cid_decode(cid);
+  for (id = 0; id < ARRAY_SIZE(mapping); id++) {
+    if (mapping[id]) {
+      targets[cids_used] = cid_decode(id);
       cids_used++;
     }
   }
+
   return cids_used;
 }
 
@@ -937,7 +940,7 @@ int num_present_units_in_city(struct city *pcity)
   Handles a chat or event message.
 **************************************************************************/
 void handle_event(const char *featured_text, struct tile *ptile,
-		  enum event_type event, int conn_id)
+                  enum event_type event, int turn, int phase, int conn_id)
 {
   char plain_text[MAX_LEN_MSG];
   struct text_tag_list *tags;
@@ -970,7 +973,7 @@ void handle_event(const char *featured_text, struct tile *ptile,
    * about us. */
   if (-1 != conn_id
       && client.conn.id != conn_id
-      && ft_color_requested(highlight_our_names)) {
+      && ft_color_requested(gui_options.highlight_our_names)) {
     const char *username = client.conn.username;
     size_t userlen = strlen(username);
     const char *playername = ((client_player() && !client_is_observer())
@@ -991,7 +994,7 @@ void handle_event(const char *featured_text, struct tile *ptile,
           && 0 == fc_strncasecmp(p, username, userlen)) {
         struct text_tag *ptag = text_tag_new(TTT_COLOR, p - plain_text,
                                              p - plain_text + userlen,
-                                             highlight_our_names);
+                                             gui_options.highlight_our_names);
 
         fc_assert(ptag != NULL);
 
@@ -1003,7 +1006,7 @@ void handle_event(const char *featured_text, struct tile *ptile,
                  && 0 == fc_strncasecmp(p, playername, playerlen)) {
         struct text_tag *ptag = text_tag_new(TTT_COLOR, p - plain_text,
                                              p - plain_text + playerlen,
-                                             highlight_our_names);
+                                             gui_options.highlight_our_names);
 
         fc_assert(ptag != NULL);
 
@@ -1049,7 +1052,9 @@ void handle_event(const char *featured_text, struct tile *ptile,
     output_window_event(plain_text, tags, conn_id);
   }
 
-  play_sound_for_event(event);
+  if (turn == game.info.turn) {
+    play_sound_for_event(event);
+  }
 
   /* Free tags */
   text_tag_list_destroy(tags);
@@ -1074,9 +1079,9 @@ void create_event(struct tile *ptile, enum event_type event,
 
     featured_text_apply_tag(message, colored_text, sizeof(colored_text),
                             TTT_COLOR, 0, FT_OFFSET_UNSET, color);
-    handle_event(colored_text, ptile, event, -1);
+    handle_event(colored_text, ptile, event, game.info.turn, game.info.phase, -1);
   } else {
-    handle_event(message, ptile, event, -1);
+    handle_event(message, ptile, event, game.info.turn, game.info.phase, -1);
   }
 }
 
@@ -1193,7 +1198,7 @@ void common_taxrates_callback(int i)
 ****************************************************************************/
 bool can_units_do_connect(struct unit_list *punits,
 			  enum unit_activity activity,
-                          struct act_tgt *tgt)
+                          struct extra_type *tgt)
 {
   unit_list_iterate(punits, punit) {
     if (can_unit_do_connect(punit, activity, tgt)) {
@@ -1204,6 +1209,140 @@ bool can_units_do_connect(struct unit_list *punits,
   return FALSE;
 }
 
+/**************************************************************************
+  Returns TRUE if a unit of the given type can act given an actor player,
+  a target player and their diplomatic state.
+
+  Note: only DRO_FOREIGN and the values of diplstate_type are tested.
+**************************************************************************/
+static bool can_unit_act_diplstate(struct unit_type *act_unit_type,
+                                   const int act_id,
+                                   const struct player *act_player,
+                                   const struct player *tgt_player)
+{
+  if (act_player == tgt_player) {
+    /* The actor player is the target player. */
+    return can_utype_do_act_if_tgt_diplrel(act_unit_type, act_id,
+                                           DRO_FOREIGN, FALSE);
+  } else {
+    /* The actor player and the target player are different. */
+    struct player_diplstate *diplstate;
+
+    /* Get the diplstate between actor player and target player. */
+    diplstate = player_diplstate_get(act_player, tgt_player);
+    fc_assert(diplstate);
+
+    return can_utype_do_act_if_tgt_diplrel(act_unit_type, act_id,
+                                           diplstate->type, TRUE);
+  }
+}
+
+/**************************************************************************
+  Returns TRUE if the unit can do a generalized action against its own
+  tile. May contain false positives.
+**************************************************************************/
+bool can_unit_act_against_own_tile(struct unit *act_unit)
+{
+  struct player *act_player;
+  struct player *tgt_player;
+  struct city *tgt_city;
+  struct tile *tgt_tile;
+
+  if (!utype_may_act_at_all(unit_type_get(act_unit))) {
+    /* Not an actor unit. */
+    return FALSE;
+  }
+
+  act_player = unit_owner(act_unit);
+  fc_assert(act_player);
+
+  tgt_tile = unit_tile(act_unit);
+  fc_assert(tgt_tile);
+
+  if ((tgt_city = tile_city(tgt_tile))) {
+    /* Target city detected. */
+
+    tgt_player = city_owner(tgt_city);
+    fc_assert(tgt_player);
+
+    action_iterate(act) {
+      if (action_id_get_actor_kind(act) != AAK_UNIT
+          || action_id_get_target_kind(act) != ATK_CITY) {
+        /* Not relevant. */
+        continue;
+      }
+
+      /* Can't return yet unless TRUE. Another action vs the city may be
+       * possible. It may also be possible to act against a unit target on
+       * the tile. */
+      if (can_unit_act_diplstate(unit_type_get(act_unit), act,
+                                 act_player, tgt_player)) {
+        /* City target confirmed possible. */
+        return TRUE;
+      }
+    } action_iterate_end;
+  }
+
+  unit_list_iterate(tgt_tile->units, tgt_unit) {
+    /* Checking this target unit. */
+
+    tgt_player = unit_owner(tgt_unit);
+    fc_assert(tgt_player);
+
+    action_iterate(act) {
+      if (action_id_get_actor_kind(act) != AAK_UNIT
+          || action_id_get_target_kind(act) != ATK_UNIT) {
+        /* Not relevant. */
+        continue;
+      }
+
+      /* Can't return yet unless TRUE. Another action vs the unit may be
+       * possible. It may also be possible to act against another unit
+       * target at the tile. */
+      if (can_unit_act_diplstate(unit_type_get(act_unit), act,
+                                 act_player, tgt_player)) {
+        /* Unit target confirmed possible. */
+        return TRUE;
+      }
+    } action_iterate_end;
+  } unit_list_iterate_end;
+
+  /* No action against any kind of target possible. */
+  return FALSE;
+}
+
+/**************************************************************************
+  Returns TRUE if any of the units in the provided list can do a
+  generalized action against a target at its own tile.
+**************************************************************************/
+bool can_units_act_against_own_tile(struct unit_list *punits)
+{
+  unit_list_iterate(punits, punit) {
+    /* Can't return unless TRUE. Another unit may be able to act against a
+     * target at its won tile. */
+    if (can_unit_act_against_own_tile(punit)) {
+      return TRUE;
+    }
+  } unit_list_iterate_end;
+
+  /* No unit in the list were able to act against a target located at its
+   * own tile. */
+  return FALSE;
+}
+
+/**************************************************************************
+  Initialize the action probability cache. Shouldn't be kept around
+  permanently. Its data is quickly outdated.
+**************************************************************************/
+void client_unit_init_act_prob_cache(struct unit *punit)
+{
+  /* A double init would cause a leak. */
+  fc_assert_ret(punit->client.act_prob_cache == NULL);
+
+  punit->client.act_prob_cache = (struct act_prob*)fc_malloc(
+        ACTION_COUNT * sizeof(*punit->client.act_prob_cache));
+}
+
 /****************************************************************************
   Determines which color type should be used for unit background.
   This is only guesswork based on unit properties. One should not
@@ -1345,7 +1484,7 @@ void client_player_maps_reset(void)
 bool mapimg_client_define(void)
 {
   char str[MAX_LEN_MAPDEF];
-  char map[MAPIMG_LAYER_COUNT + 1];
+  char mi_map[MAPIMG_LAYER_COUNT + 1];
   enum mapimg_layer layer;
   int map_pos = 0;
 
@@ -1355,37 +1494,37 @@ bool mapimg_client_define(void)
   }
 
   /* Map image definition: zoom, turns */
-  fc_snprintf(str, sizeof(str), "zoom=%d:turns=0:format=%s", mapimg_zoom,
-              mapimg_format);
+  fc_snprintf(str, sizeof(str), "zoom=%d:turns=0:format=%s",
+              gui_options.mapimg_zoom, gui_options.mapimg_format);
 
   /* Map image definition: show */
   if (client_is_global_observer()) {
     cat_snprintf(str, sizeof(str), ":show=all");
     /* use all available knowledge */
-    mapimg_layer[MAPIMG_LAYER_KNOWLEDGE] = FALSE;
+    gui_options.mapimg_layer[MAPIMG_LAYER_KNOWLEDGE] = FALSE;
   } else {
     cat_snprintf(str, sizeof(str), ":show=plrid:plrid=%d",
                  player_index(client.conn.playing));
     /* use only player knowledge */
-    mapimg_layer[MAPIMG_LAYER_KNOWLEDGE] = TRUE;
+    gui_options.mapimg_layer[MAPIMG_LAYER_KNOWLEDGE] = TRUE;
   }
 
   /* Map image definition: map */
   for (layer = mapimg_layer_begin(); layer != mapimg_layer_end();
        layer = mapimg_layer_next(layer)) {
-    if (mapimg_layer[layer]) {
-      cat_snprintf(map, sizeof(map), "%s",
+    if (gui_options.mapimg_layer[layer]) {
+      cat_snprintf(mi_map, sizeof(mi_map), "%s",
                    mapimg_layer_name(layer));
-      map[map_pos++] = mapimg_layer_name(layer)[0];
+      mi_map[map_pos++] = mapimg_layer_name(layer)[0];
     }
   }
-  map[map_pos] = '\0';
+  mi_map[map_pos] = '\0';
 
   if (map_pos == 0) {
     /* no value set - use dummy setting */
-    sz_strlcpy(map, "-");
+    sz_strlcpy(mi_map, "-");
   }
-  cat_snprintf(str, sizeof(str), ":map=%s", map);
+  cat_snprintf(str, sizeof(str), ":map=%s", mi_map);
 
   log_debug("client map image definition: %s", str);
 
@@ -1407,7 +1546,7 @@ bool mapimg_client_createmap(const char *filename)
   char mapimgfile[512];
 
   if (NULL == filename || '\0' == filename[0]) {
-    sz_strlcpy(mapimgfile, mapimg_filename);
+    sz_strlcpy(mapimgfile, gui_options.mapimg_filename);
   } else {
     sz_strlcpy(mapimgfile, filename);
   }
