@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -34,8 +34,8 @@ void client_remove_player(int plrno);
 void client_remove_city(struct city *pcity);
 void client_remove_unit(struct unit *punit);
 
-void client_change_all(struct universal from,
-		       struct universal to);
+void client_change_all(struct universal *from,
+                       struct universal *to);
 
 const char *get_embassy_status(const struct player *me,
 				const struct player *them);
@@ -71,42 +71,42 @@ struct universal cid_decode(cid cid);
 #define cid_production cid_decode
 
 bool city_unit_supported(const struct city *pcity,
-			 struct universal target);
+                         const struct universal *target);
 bool city_unit_present(const struct city *pcity,
-		       struct universal target);
+                       const struct universal *target);
 bool city_building_present(const struct city *pcity,
-			   struct universal target);
+                           const struct universal *target);
 
 struct item {
   struct universal item;
   char descr[MAX_LEN_NAME + 40];
 };
 
-typedef bool (*TestCityFunc)(const struct city *, struct universal);
+typedef bool (*TestCityFunc)(const struct city *, const struct universal *);
 
 #define MAX_NUM_PRODUCTION_TARGETS (U_LAST + B_LAST)
 void name_and_sort_items(struct universal *targets, int num_items,
-			 struct item *items,
-			 bool show_cost, struct city *pcity);
+                         struct item *items,
+                         bool show_cost, struct city *pcity);
 int collect_production_targets(struct universal *targets,
-			       struct city **selected_cities,
-			       int num_selected_cities, bool append_units,
-			       bool append_wonders, bool change_prod,
-			       TestCityFunc test_func);
+                               struct city **selected_cities,
+                               int num_selected_cities, bool append_units,
+                               bool append_wonders, bool change_prod,
+                               TestCityFunc test_func);
 int collect_currently_building_targets(struct universal *targets);
 int collect_buildable_targets(struct universal *targets);
 int collect_eventually_buildable_targets(struct universal *targets,
-					 struct city *pcity,
-					 bool advanced_tech);
+                                         struct city *pcity,
+                                         bool advanced_tech);
 int collect_already_built_targets(struct universal *targets,
-				  struct city *pcity);
+                                  struct city *pcity);
 
 /* the number of units in city */
 int num_present_units_in_city(struct city* pcity);
 int num_supported_units_in_city(struct city* pcity);	
 
 void handle_event(const char *featured_text, struct tile *ptile,
-                  enum event_type event, int conn_id);
+                  enum event_type event, int turn, int phase, int conn_id);
 void create_event(struct tile *ptile, enum event_type event,
                   const struct ft_color color, const char *format, ...)
                   fc__attribute((__format__ (__printf__, 4, 5)));
@@ -117,8 +117,13 @@ void cityrep_buy(struct city *pcity);
 void common_taxrates_callback(int i);
 
 bool can_units_do_connect(struct unit_list *punits,
-			  enum unit_activity activity,
-                          struct act_tgt *tgt);
+                          enum unit_activity activity,
+                          struct extra_type *tgt);
+
+bool can_unit_act_against_own_tile(struct unit *punit);
+bool can_units_act_against_own_tile(struct unit_list *punits);
+
+void client_unit_init_act_prob_cache(struct unit *punit);
 
 enum unit_bg_color_type { UNIT_BG_HP_LOSS,
                           UNIT_BG_LAND,
