@@ -1,4 +1,4 @@
-/**********************************************************************
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,12 +15,14 @@
 #include <fc_config.h>
 #endif
 
+#include "fc_prehdrs.h"
+
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#ifdef HAVE_SYS_TYPES_H
+#ifdef FREECIV_HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
 #ifdef HAVE_SYS_SOCKET_H
@@ -53,13 +55,6 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#ifdef HAVE_WINSOCK
-#ifdef HAVE_WINSOCK2
-#include <winsock2.h>
-#else  /* HAVE_WINSOCK2 */
-#include <winsock.h>
-#endif /* HAVE_WINSOCK2 */
-#endif /* HAVE_WINSOCK */
 
 /* utility */
 #include "capstr.h"
@@ -85,8 +80,6 @@
 #include "connectdlg_common.h"
 #include "connectdlg_g.h"
 #include "dialogs_g.h"		/* popdown_races_dialog() */
-#include "ggzclient.h"
-#include "ggz_g.h"
 #include "gui_main_g.h"		/* add_net_input(), remove_net_input() */
 #include "mapview_common.h"	/* unqueue_mapview_update */
 #include "menu_g.h"
@@ -105,6 +98,7 @@
 /* In autoconnect mode, try to connect 100 times */
 #define MAX_AUTOCONNECT_ATTEMPTS	100
 
+extern char forced_tileset_name[512];
 static struct fc_sockaddr_list *list = NULL;
 static int name_count;
 
@@ -114,12 +108,6 @@ static int name_count;
 **************************************************************************/
 static void close_socket_nomessage(struct connection *pc)
 {
-  if (with_ggz || in_ggz) {
-    remove_ggz_input();
-  }
-  if (in_ggz) {
-    gui_ggz_embed_leave_table();
-  }
   connection_common_close(pc);
   remove_net_input();
   popdown_races_dialog(); 
@@ -148,9 +136,6 @@ static void client_conn_close_callback(struct connection *pconn)
   log_error("Lost connection to server: %s.", reason);
   output_window_printf(ftc_client, _("Lost connection to server (%s)!"),
                        reason);
-  if (with_ggz) {
-    client_exit();
-  }
 }
 
 /**************************************************************************
@@ -240,11 +225,11 @@ static int try_to_connect(const char *username, char *errbuf, int errbufsize)
   client.conn.sock = sock;
   if (client.conn.sock == -1) {
     (void) fc_strlcpy(errbuf, fc_strerror(err), errbufsize);
-#ifdef HAVE_WINSOCK
+#ifdef FREECIV_HAVE_WINSOCK
     return -1;
 #else
     return err;
-#endif /* HAVE_WINSOCK */
+#endif /* FREECIV_HAVE_WINSOCK */
   }
 
   make_connection(client.conn.sock, username);
@@ -271,18 +256,24 @@ int connect_to_server(const char *username, const char *hostname, int port,
     return -1;
   }
 
+  forced_tileset_name[0] = '\0';
+  if (gui_options.use_prev_server) {
+    sz_strlcpy(gui_options.default_server_host, hostname);
+    gui_options.default_server_port = port;
+  }
+
   return 0;
 }
 
 /**************************************************************************
   Called after a connection is completed (e.g., in try_to_connect).
 **************************************************************************/
-void make_connection(int socket, const char *username)
+void make_connection(int sock, const char *username)
 {
   struct packet_server_join_req req;
 
   connection_common_init(&client.conn);
-  client.conn.sock = socket;
+  client.conn.sock = sock;
   client.conn.client.last_request_id_used = 0;
   client.conn.client.last_processed_request_id_seen = 0;
   client.conn.client.request_id_of_currently_handled_packet = 0;
@@ -313,6 +304,9 @@ void disconnect_from_server(void)
   const bool force = !client.conn.used;
 
   attribute_flush();
+
+  stop_turn_change_wait();
+
   /* If it's internal server - kill him 
    * We assume that we are always connected to the internal server  */
   if (!force) {
@@ -323,13 +317,11 @@ void disconnect_from_server(void)
     client_kill_server(TRUE);
   }
   output_window_append(ftc_client, _("Disconnected from server."));
-  if (with_ggz) {
-    client_exit();
-  }
-  if (save_options_on_exit) {
+
+  if (gui_options.save_options_on_exit) {
     options_save(NULL);
   }
-}  
+}
 
 /****************************************************************************
   A wrapper around read_socket_data() which also handles the case the
