@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -76,7 +76,7 @@ static GtkWidget  *races_notebook;
 static GtkWidget  *races_properties;
 static GtkWidget  *races_leader;
 static GtkWidget  *races_sex[2];
-static GtkWidget  *races_city_style_list;
+static GtkWidget  *races_style_list;
 static GtkTextBuffer *races_text;
 
 static void create_races_dialog(struct player *pplayer);
@@ -84,14 +84,14 @@ static void races_response(GtkWidget *w, gint response, gpointer data);
 static void races_nation_callback(GtkTreeSelection *select, gpointer data);
 static void races_leader_callback(void);
 static void races_sex_callback(GtkWidget *w, gpointer data);
-static void races_city_style_callback(GtkTreeSelection *select, gpointer data);
+static void races_style_callback(GtkTreeSelection *select, gpointer data);
 static gboolean races_selection_func(GtkTreeSelection *select,
 				     GtkTreeModel *model, GtkTreePath *path,
 				     gboolean selected, gpointer data);
 
 static int selected_nation;
 static int selected_sex;
-static int selected_city_style;
+static int selected_style;
 
 static int is_showing_pillage_dialog = FALSE;
 static int unit_to_use_to_pillage;
@@ -159,7 +159,7 @@ static void notify_goto_response(GtkWidget *w, gint response)
   case 2:
     pcity = tile_city(ptile);
 
-    if (center_when_popup_city) {
+    if (gui_options.center_when_popup_city) {
       center_tile_mapcanvas(ptile);
     }
 
@@ -320,21 +320,10 @@ static void pillage_callback(GtkWidget *w, gpointer data)
 
   punit = game_unit_by_number(unit_to_use_to_pillage);
   if (punit) {
-    struct act_tgt target;
-
-    if (what >= S_LAST + game.control.num_base_types) {
-      target.type = ATT_ROAD;
-      target.obj.road = what - S_LAST - game.control.num_base_types;
-    } else if (what >= S_LAST) {
-      target.type = ATT_BASE;
-      target.obj.base = what - S_LAST;
-    } else {
-      target.type = ATT_SPECIAL;
-      target.obj.spe = what;
-    }
+    struct extra_type *target = extra_by_number(what);
 
     request_new_unit_activity_targeted(punit, ACTIVITY_PILLAGE,
-                                       &target);
+                                       target);
   }
 }
 
@@ -349,15 +338,12 @@ static void pillage_destroy_callback(GtkWidget *w, gpointer data)
 /****************************************************************
   Opens pillage dialog listing possible pillage targets.
 *****************************************************************/
-void popup_pillage_dialog(struct unit *punit,
-			  bv_special spe,
-                          bv_bases bases,
-                          bv_roads roads)
+void popup_pillage_dialog(struct unit *punit, bv_extras extras)
 {
   GtkWidget *shl;
 
   if (!is_showing_pillage_dialog) {
-    struct act_tgt tgt;
+    struct extra_type *tgt;
 
     is_showing_pillage_dialog = TRUE;
     unit_to_use_to_pillage = punit->id;
@@ -366,39 +352,19 @@ void popup_pillage_dialog(struct unit *punit,
 			       _("What To Pillage"),
 			       _("Select what to pillage:"));
 
-    while (get_preferred_pillage(&tgt, spe, bases, roads)) {
-      int what = S_LAST;
-      bv_special what_spe;
-      bv_bases what_base;
-      bv_roads what_road;
+    while ((tgt = get_preferred_pillage(extras))) {
+      int what;
 
-      BV_CLR_ALL(what_spe);
-      BV_CLR_ALL(what_base);
-      BV_CLR_ALL(what_road);
+      what = extra_index(tgt);
+      BV_CLR(extras, what);
 
-      switch (tgt.type) {
-        case ATT_SPECIAL:
-          BV_SET(what_spe, tgt.obj.spe);
-          what = tgt.obj.spe;
-          clear_special(&spe, tgt.obj.spe);
-          break;
-        case ATT_BASE:
-          BV_SET(what_base, tgt.obj.base);
-          what = tgt.obj.base + S_LAST;
-          BV_CLR(bases, tgt.obj.base);
-          break;
-        case ATT_ROAD:
-          BV_SET(what_road, tgt.obj.road);
-          what = tgt.obj.road + S_LAST + game.control.num_base_types;
-          BV_CLR(roads, tgt.obj.road);
-          break;
-      }
-
-      choice_dialog_add(shl, get_infrastructure_text(what_spe, what_base, what_road),
-                        G_CALLBACK(pillage_callback), GINT_TO_POINTER(what));
+      choice_dialog_add(shl, extra_name_translation(tgt),
+                        G_CALLBACK(pillage_callback),
+                        GINT_TO_POINTER(what),
+                        FALSE, NULL);
     }
 
-    choice_dialog_add(shl, GTK_STOCK_CANCEL, 0, 0);
+    choice_dialog_add(shl, GTK_STOCK_CANCEL, 0, 0, FALSE, NULL);
 
     choice_dialog_end(shl);
 
@@ -540,7 +506,7 @@ static void populate_leader_list(void)
 *****************************************************************************/
 static void select_nation(int nation,
                           const char *leadername, bool is_male,
-                          int city_style)
+                          int style_id)
 {
   selected_nation = nation;
 
@@ -565,24 +531,23 @@ static void select_nation(int nation,
 
     /* Select the appropriate city style entry. */
     {
-      int i, j;
+      int i;
+      int j = 0;
       GtkTreePath *path;
 
-      for (i = 0, j = 0; i < game.control.styles_count; i++) {
-        if (city_style_has_requirements(&city_styles[i])) {
-          continue;
-        }
+      styles_iterate(pstyle) {
+        i = basic_city_style_for_style(pstyle);
 
-        if (i < city_style) {
+        if (i >= 0 && i < style_id) {
           j++;
         } else {
           break;
         }
-      }
+      } styles_iterate_end;
 
       path = gtk_tree_path_new();
       gtk_tree_path_append_index(path, j);
-      gtk_tree_view_set_cursor(GTK_TREE_VIEW(races_city_style_list), path,
+      gtk_tree_view_set_cursor(GTK_TREE_VIEW(races_style_list), path,
                                NULL, FALSE);
       gtk_tree_path_free(path);
     }
@@ -609,7 +574,7 @@ static void select_nation(int nation,
     /* City style */
     {
       GtkTreeSelection* select
-        = gtk_tree_view_get_selection(GTK_TREE_VIEW(races_city_style_list));
+        = gtk_tree_view_get_selection(GTK_TREE_VIEW(races_style_list));
       gtk_tree_selection_unselect_all(select);
     }
     /* Nation description */
@@ -726,6 +691,10 @@ static void create_nation_selection_lists(void)
   
   for (i = 0; i < nation_group_count(); i++) {
     struct nation_group* group = (nation_group_by_number(i));
+    if (is_nation_group_hidden(group)) {
+      races_nation_list[i] = NULL;
+      continue;
+    }
     nation_list = create_list_of_nations_in_group(group, i);
     if (nation_list) {
       group_name_label = gtk_label_new(nation_group_name_translation(group));
@@ -1022,19 +991,28 @@ static void create_races_dialog(struct player *pplayer)
     gtk_notebook_set_tab_pos(GTK_NOTEBOOK(races_notebook), GTK_POS_LEFT);  
 
     /* Suppress notebook tabs if there will be only one ("All") */
-    if (nation_group_count() == 0) {
-      gtk_notebook_set_show_tabs(GTK_NOTEBOOK(races_notebook), FALSE);
-    } else {
-      label = g_object_new(GTK_TYPE_LABEL,
-          "use-underline", TRUE,
-          "label", _("Nation _Groups:"),
-          "xalign", 0.0,
-          "yalign", 0.5,
-          NULL);
-      gtk_label_set_mnemonic_widget(GTK_LABEL(label), races_notebook);
-      gtk_box_pack_start(GTK_BOX(nation_selection_list), label,
-                         FALSE, FALSE, 0);  
-      gtk_notebook_set_show_tabs(GTK_NOTEBOOK(races_notebook), TRUE);
+    {
+      bool show_groups = FALSE;
+      nation_groups_iterate(pgroup) {
+        if (!is_nation_group_hidden(pgroup)) {
+          show_groups = TRUE;
+          break;
+        }
+      } nation_groups_iterate_end;
+      if (!show_groups) {
+        gtk_notebook_set_show_tabs(GTK_NOTEBOOK(races_notebook), FALSE);
+      } else {
+        label = g_object_new(GTK_TYPE_LABEL,
+            "use-underline", TRUE,
+            "label", _("Nation _Groups:"),
+            "xalign", 0.0,
+            "yalign", 0.5,
+            NULL);
+        gtk_label_set_mnemonic_widget(GTK_LABEL(label), races_notebook);
+        gtk_box_pack_start(GTK_BOX(nation_selection_list), label,
+                           FALSE, FALSE, 0);  
+        gtk_notebook_set_show_tabs(GTK_NOTEBOOK(races_notebook), TRUE);
+      }
     }
 
     gtk_box_pack_start(GTK_BOX(nation_selection_list), races_notebook,
@@ -1097,11 +1075,11 @@ static void create_races_dialog(struct player *pplayer)
       GDK_TYPE_PIXBUF, G_TYPE_STRING);
 
   list = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
-  races_city_style_list = list;
+  races_style_list = list;
   g_object_unref(store);
   gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(list), FALSE);
   g_signal_connect(gtk_tree_view_get_selection(GTK_TREE_VIEW(list)), "changed",
-      G_CALLBACK(races_city_style_callback), NULL);
+      G_CALLBACK(races_style_callback), NULL);
 
   sw = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
@@ -1132,26 +1110,26 @@ static void create_races_dialog(struct player *pplayer)
 
   gtk_table_set_row_spacing(GTK_TABLE(table), 1, 12);
 
-  /* Populate city style store. */
-  for (i = 0; i < game.control.styles_count; i++) {
+  /* Populate style store. */
+  styles_iterate(pstyle) {
     GdkPixbuf *img;
     struct sprite *s;
     GtkTreeIter it;
 
-    if (city_style_has_requirements(&city_styles[i])) {
-      continue;
+    i = basic_city_style_for_style(pstyle);
+
+    if (i >= 0) {
+      gtk_list_store_append(store, &it);
+
+      s = crop_blankspace(get_sample_city_sprite(tileset, i));
+      img = sprite_get_pixbuf(s);
+      g_object_ref(img);
+      free_sprite(s);
+      gtk_list_store_set(store, &it, 0, i, 1, img, 2,
+                         city_style_name_translation(i), -1);
+      g_object_unref(img);
     }
-
-    gtk_list_store_append(store, &it);
-
-    s = crop_blankspace(get_sample_city_sprite(tileset, i));
-    img = sprite_get_pixbuf(s);
-    g_object_ref(img);
-    free_sprite(s);
-    gtk_list_store_set(store, &it, 0, i, 1, img, 2,
-                       city_style_name_translation(i), -1);
-    g_object_unref(img);
-  }
+  } styles_iterate_end;
 
   /* Legend pane. */
   label = gtk_label_new_with_mnemonic(_("_Description"));
@@ -1200,7 +1178,7 @@ static void create_races_dialog(struct player *pplayer)
     select_nation(nation_number(races_player->nation),
                   player_name(races_player),
                   races_player->is_male,
-                  races_player->city_style);
+                  style_number(races_player->style));
     /* Make sure selected nation is visible
      * (last page, "All", will certainly contain it) */
     fc_assert(gtk_notebook_get_n_pages(GTK_NOTEBOOK(races_notebook)) > 0);
@@ -1307,7 +1285,7 @@ static void races_nation_callback(GtkTreeSelection *select, gpointer data)
       if (newnation != selected_nation) {
         /* Choose a random leader */
         select_nation(newnation, NULL, FALSE,
-                      city_style_of_nation(nation_by_number(newnation)));
+                      style_number(style_of_nation(nation_by_number(newnation))));
       }
       return;
     }
@@ -1363,15 +1341,15 @@ static gboolean races_selection_func(GtkTreeSelection *select,
 /**************************************************************************
   City style has been chosen
 **************************************************************************/
-static void races_city_style_callback(GtkTreeSelection *select, gpointer data)
+static void races_style_callback(GtkTreeSelection *select, gpointer data)
 {
   GtkTreeModel *model;
   GtkTreeIter it;
 
   if (gtk_tree_selection_get_selected(select, &model, &it)) {
-    gtk_tree_model_get(model, &it, 0, &selected_city_style, -1);
+    gtk_tree_model_get(model, &it, 0, &selected_style, -1);
   } else {
-    selected_city_style = -1;
+    selected_style = -1;
   }
 }
 
@@ -1393,8 +1371,8 @@ static void races_response(GtkWidget *w, gint response, gpointer data)
       return;
     }
 
-    if (selected_city_style == -1) {
-      output_window_append(ftc_client, _("You must select your city style."));
+    if (selected_style == -1) {
+      output_window_append(ftc_client, _("You must select your style."));
       return;
     }
 
@@ -1408,8 +1386,9 @@ static void races_response(GtkWidget *w, gint response, gpointer data)
     }
 
     dsend_packet_nation_select_req(&client.conn,
-				   player_number(races_player), selected_nation,
-				   selected_sex, s, selected_city_style);
+                                   player_number(races_player), selected_nation,
+                                   selected_sex, s,
+                                   selected_style);
   } else if (response == GTK_RESPONSE_NO) {
     dsend_packet_nation_select_req(&client.conn,
 				   player_number(races_player),
@@ -1523,10 +1502,13 @@ void popdown_all_game_dialogs(void)
 *****************************************************************/
 void show_tech_gained_dialog(Tech_type_id tech)
 {
-  if (gui_gtk2_popup_tech_help == GUI_POPUP_TECH_HELP_ENABLED
-      || (gui_gtk2_popup_tech_help == GUI_POPUP_TECH_HELP_RULESET
-          && game.control.popup_tech_help)) {
-    popup_help_dialog_typed(advance_name_for_player(client.conn.playing, tech), HELP_TECH);
+  const struct advance *padvance = valid_advance_by_number(tech);
+
+  if (NULL != padvance
+      && (gui_options.gui_gtk2_popup_tech_help == GUI_POPUP_TECH_HELP_ENABLED
+          || (gui_options.gui_gtk2_popup_tech_help == GUI_POPUP_TECH_HELP_RULESET
+              && game.control.popup_tech_help))) {
+    popup_help_dialog_typed(advance_name_translation(padvance), HELP_TECH);
   }
 }
 
@@ -1550,3 +1532,31 @@ void show_tileset_error(const char *msg)
     gtk_widget_destroy(dialog);
   }
 }
+
+/****************************************************************
+  Give a warning when user is about to edit scenario with manually
+  set properties.
+*****************************************************************/
+bool handmade_scenario_warning(void)
+{
+  /* Just tell the client common code to handle this. */
+  return FALSE;
+}
+
+/****************************************************************
+  Unit wants to get into some transport on given tile.
+*****************************************************************/
+bool request_transport(struct unit *pcargo, struct tile *ptile)
+{
+  return FALSE; /* Unit was not handled here. */
+}
+
+/***************************************************************************
+  Popup detailed information about battle or save information for
+  some kind of statistics
+***************************************************************************/
+void popup_combat_info(int attacker_unit_id, int defender_unit_id,
+                       int attacker_hp, int defender_hp,
+                       bool make_winner_veteran)
+{
+}
