@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -310,6 +310,7 @@ void update_intel_dialog(struct player *p)
   struct intel_dialog *pdialog = get_intel_dialog(p);
 
   if (pdialog) {
+    const struct research *mresearch, *presearch;
     GtkTreeIter diplstates[DS_LAST];
     int i;
 
@@ -328,7 +329,7 @@ void update_intel_dialog(struct player *p)
 
       gtk_tree_store_append(pdialog->diplstates, &it, NULL);
       g_value_init(&v, G_TYPE_STRING);
-      g_value_set_static_string(&v, diplstate_text(i));
+      g_value_set_static_string(&v, diplstate_type_translated_name(i));
       gtk_tree_store_set_value(pdialog->diplstates, &it, 0, &v);
       g_value_unset(&v);
       diplstates[i] = it;
@@ -354,16 +355,20 @@ void update_intel_dialog(struct player *p)
     /* techs tab. */
     gtk_list_store_clear(pdialog->techs);
 
-    advance_index_iterate(A_FIRST, i) {
-      if(player_invention_state(p, i)==TECH_KNOWN) {
-	GtkTreeIter it;
+    mresearch = research_get(client_player());
+    presearch = research_get(p);
+    advance_index_iterate(A_FIRST, advi) {
+      if (research_invention_state(presearch, advi) == TECH_KNOWN) {
+        GtkTreeIter it;
 
-	gtk_list_store_append(pdialog->techs, &it);
+        gtk_list_store_append(pdialog->techs, &it);
 
-	gtk_list_store_set(pdialog->techs, &it,
-			   0, (TECH_KNOWN != player_invention_state(client.conn.playing, i)),
-			   1, advance_name_for_player(p, i),
-			   -1);
+        gtk_list_store_set(pdialog->techs, &it,
+                           0, research_invention_state(mresearch, advi)
+                           != TECH_KNOWN,
+                           1, research_advance_name_translation(presearch,
+                                                                advi),
+                           -1);
       }
     } advance_index_iterate_end;
 
@@ -385,7 +390,7 @@ void update_intel_dialog(struct player *p)
         case LABEL_CAPITAL:
           pcity = player_capital(p);
           /* TRANS: "unknown" location */
-          buf = g_strdup((!pcity) ? _("(unknown)") : city_name(pcity));
+          buf = g_strdup((!pcity) ? _("(unknown)") : city_name_get(pcity));
           break;
         case LABEL_GOLD:
           buf = g_strdup_printf("%d", p->economic.gold);
@@ -401,7 +406,7 @@ void update_intel_dialog(struct player *p)
           break;
         case LABEL_RESEARCHING:
           {
-            struct player_research *research = player_research_get(p);
+            struct research *research = research_get(p);
 
             switch (research->researching) {
             case A_UNKNOWN:
@@ -414,8 +419,9 @@ void update_intel_dialog(struct player *p)
               break;
             default:
               buf = g_strdup_printf("%s(%d/%d)",
-				    advance_name_researching(p),
-				    research->bulbs_researched,
+                                    research_advance_name_translation
+                                        (research, research->researching),
+                                    research->bulbs_researched,
                                     research->client.researching_cost);
               break;
             }
