@@ -101,38 +101,37 @@ void popdown_players_dialog(void)
   }
 }
 
-/**************************************************************************
-  Create pixbuf for player
-**************************************************************************/
-static GdkPixbuf *create_player_icon(const struct player *plr)
+/***************************************************************************
+  Create a small colored square representing the player color, for use
+  in player lists. 
+  May return NULL if the player has no color yet.
+***************************************************************************/
+GdkPixbuf *create_player_icon(const struct player *plr)
 {
   int width, height;
   GdkPixbuf *tmp;
-  cairo_surface_t *surface;
-  struct color *color;
-  cairo_t *cr;
+  GdkPixmap *pixmap;
 
-  gtk_icon_size_lookup_for_settings(
-      gtk_settings_get_for_screen(gtk_widget_get_screen(top_notebook)), 
-      GTK_ICON_SIZE_MENU, &width, &height);
-  surface = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, width, height);
-  
-  cr = cairo_create(surface);
+  if (!player_has_color(tileset, plr)) {
+    return NULL;
+  }
 
-  color = get_color(tileset, COLOR_PLAYER_COLOR_BACKGROUND);
-  gdk_cairo_set_source_rgba(cr, &color->color);
-  cairo_rectangle(cr, 0, 0, width, height);
-  cairo_fill(cr);
+  gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height);
 
-  color = get_player_color(tileset, plr);
-  gdk_cairo_set_source_rgba(cr, &color->color);
-  cairo_rectangle(cr, 1, 1, width - 2, height - 2);
-  cairo_fill(cr);
+  pixmap = gdk_pixmap_new(root_window, width, height, -1);
 
-  cairo_destroy(cr);
-  tmp = surface_get_pixbuf(surface, width, height);
-  cairo_surface_destroy(surface);
+  gdk_gc_set_foreground(civ_gc,
+                        &get_color(tileset,
+				   COLOR_PLAYER_COLOR_BACKGROUND)->color);
+  gdk_draw_rectangle(pixmap, civ_gc, TRUE, 0, 0, width, height);
 
+  gdk_gc_set_foreground(civ_gc, &get_player_color(tileset, plr)->color);
+  gdk_draw_rectangle(pixmap, civ_gc, TRUE, 1, 1, width - 2, height - 2);
+
+  tmp = gdk_pixbuf_get_from_drawable(NULL, pixmap, NULL, 
+      0, 0, 0, 0, -1, -1);
+
+  g_object_unref(pixmap);
   return tmp;
 }
 
@@ -309,7 +308,7 @@ static void toggle_view(GtkCheckMenuItem* item, gpointer data)
 *************************************************************************/
 static void toggle_dead_players(GtkCheckMenuItem* item, gpointer data)
 {
-  player_dlg_show_dead_players = 
+  gui_options.player_dlg_show_dead_players = 
     gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(item));
   real_players_dialog_update();
 }
@@ -395,10 +394,10 @@ static GtkWidget* create_show_menu(void)
   
   item = gtk_check_menu_item_new_with_label(Q_("?show:Dead Players"));
   gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item),
-                                 player_dlg_show_dead_players);
+                                 gui_options.player_dlg_show_dead_players);
   gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
   g_signal_connect(item, "toggled", G_CALLBACK(toggle_dead_players), NULL);
-  
+
   return menu;
 }
 
@@ -426,8 +425,6 @@ void create_players_dialog(void)
 
   players_list = gtk_tree_view_new_with_model(GTK_TREE_MODEL
                                               (players_dialog_store));
-  gtk_widget_set_hexpand(players_list, TRUE);
-  gtk_widget_set_vexpand(players_list, TRUE);
   g_object_unref(players_dialog_store);
   gtk_widget_set_name(players_list, "small_font");
 
@@ -505,20 +502,21 @@ void create_players_dialog(void)
 		                 GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
   gtk_container_add(GTK_CONTAINER(sw), players_list);
 
-  gtk_container_add(GTK_CONTAINER(players_dialog_shell->vbox), sw);
+  gtk_box_pack_start(GTK_BOX(players_dialog_shell->vbox), sw,
+		     TRUE, TRUE, 0);
 
-  vbox = gtk_grid_new();
-  gtk_orientable_set_orientation(GTK_ORIENTABLE(vbox),
-                                 GTK_ORIENTATION_VERTICAL);
+  vbox = gtk_vbox_new(FALSE, 0);
   
-  sep = gtk_separator_new(GTK_ORIENTATION_HORIZONTAL);
-  gtk_container_add(GTK_CONTAINER(vbox), sep);
+  sep = gtk_hseparator_new();
+  gtk_box_pack_start(GTK_BOX(vbox), sep, FALSE, FALSE, 0);
 
   menubar = gtk_aux_menu_bar_new();
-  gtk_container_add(GTK_CONTAINER(vbox), menubar);
+  gtk_box_pack_start(GTK_BOX(vbox), menubar, TRUE, TRUE, 0);
 
 
   gui_dialog_add_widget(players_dialog_shell, vbox);
+  gtk_box_set_child_packing(GTK_BOX(players_dialog_shell->action_area), 
+                            vbox, FALSE, FALSE, 0, GTK_PACK_START);
 
   item = gtk_menu_item_new_with_mnemonic(_("Di_plomacy"));
   menu = create_diplomacy_menu();
@@ -549,9 +547,9 @@ void create_players_dialog(void)
   sep = gtk_separator_menu_item_new();
   gtk_menu_shell_append(GTK_MENU_SHELL(menu), sep);
 
-  for (level = 0; level < AI_LEVEL_LAST; level++) {
+  for (level = 0; level < AI_LEVEL_COUNT; level++) {
     if (is_settable_ai_level(level)) {
-      const char *level_name = ai_level_name(level);
+      const char *level_name = ai_level_translated_name(level);
 
       item = gtk_menu_item_new_with_label(level_name);
       g_signal_connect(item, "activate",
@@ -608,14 +606,11 @@ GdkPixbuf *get_flag(const struct nation_type *nation)
   /* croping */
   im = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, w, h);
   if (im != NULL) {
-    GdkPixbuf *pixbuf = sprite_get_pixbuf(flag);
-
-    gdk_pixbuf_copy_area(pixbuf, x0, y0, w, h,
+    gdk_pixbuf_copy_area(sprite_get_pixbuf(flag), x0, y0, w, h,
                          im, 0, 0);
-    g_object_unref(G_OBJECT(pixbuf));
   }
 
-  /* and finally store the scaled flag pixbuf in the static flags array */
+  /* and finaly store the scaled flag pixbuf in the static flags array */
   return im;
 }
 
@@ -695,7 +690,7 @@ static void fill_row(GtkListStore *store, GtkTreeIter *it,
 **************************************************************************/
 static bool player_should_be_shown(const struct player *pplayer)
 {
-  return NULL != pplayer && (player_dlg_show_dead_players
+  return NULL != pplayer && (gui_options.player_dlg_show_dead_players
                              || pplayer->is_alive)
          && (!is_barbarian(pplayer));
 }
@@ -800,7 +795,7 @@ void players_war_callback(GtkMenuItem *item, gpointer data)
     newstate = cancel_pact_result(oldstate);
 
     /* TRANS: %s is a diplomatic state: "Cancel Cease-fire" */
-    astr_set(&title, _("Cancel %s"), diplstate_text(oldstate));
+    astr_set(&title, _("Cancel %s"), diplstate_type_translated_name(oldstate));
 
     if (newstate == DS_WAR) {
       astr_set(&question, _("Really declare war on the %s?"),
@@ -809,8 +804,8 @@ void players_war_callback(GtkMenuItem *item, gpointer data)
       /* TRANS: "Cancel Belgian Alliance? ... will be Armistice." */
       astr_set(&question, _("Cancel %s %s? New diplomatic state will be %s."),
                nation_adjective_for_player(aplayer),
-               diplstate_text(oldstate),
-               diplstate_text(newstate));
+               diplstate_type_translated_name(oldstate),
+               diplstate_type_translated_name(newstate));
     }
 
     /* can be any pact clause */
