@@ -33,8 +33,9 @@
 #include "client_main.h"
 #include "cma_fec.h"
 #include "messagewin_g.h"
+#include "options.h"
 
-/* client/gtk-3.0 */
+/* client/gui-gtk-3.0 */
 #include "cityrep.h"
 #include "dialogs.h"
 #include "gui_stuff.h"
@@ -207,14 +208,14 @@ static void cell_data_func(GtkTreeViewColumn *col, GtkCellRenderer *cell,
 }
 
 /**************************************************************************
- instantiates a new struct for each city_dialog window that is open.
+  Instantiates a new struct for each city_dialog window that is open.
 **************************************************************************/
 struct cma_dialog *create_cma_dialog(struct city *pcity, bool tiny)
 {
   struct cma_dialog *pdialog;
   struct cm_parameter param;
   GtkWidget *frame, *page, *hbox, *label, *table;
-  GtkWidget *vbox, *sw, *hscale, *button, *align, *image;
+  GtkWidget *vbox, *sw, *hscale, *button, *image;
   GtkListStore *store;
   GtkCellRenderer *rend;
   GtkWidget *view;
@@ -354,7 +355,9 @@ struct cma_dialog *create_cma_dialog(struct city *pcity, bool tiny)
 
     pdialog->minimal_surplus[i] = hscale =
         gtk_scale_new(GTK_ORIENTATION_HORIZONTAL, NULL);
-    gtk_range_set_range(GTK_RANGE(hscale), -20, 20);
+    gtk_range_set_range(GTK_RANGE(hscale),
+                        gui_options.gui_gtk3_governor_range_min,
+                        gui_options.gui_gtk3_governor_range_max);
     gtk_range_set_increments(GTK_RANGE(hscale), 1, 1);
     pango_layout_get_pixel_size(gtk_scale_get_layout(GTK_SCALE(hscale)),
                                 &layout_width, NULL);
@@ -426,23 +429,16 @@ struct cma_dialog *create_cma_dialog(struct city *pcity, bool tiny)
   g_signal_connect(button, "clicked",
                    G_CALLBACK(help_callback), NULL);
   gtk_container_add(GTK_CONTAINER(hbox), button);
+  gtk_button_box_set_child_non_homogeneous(GTK_BUTTON_BOX(hbox),
+                                           button, TRUE);
 
   pdialog->active_command = gtk_toggle_button_new();
+  gtk_button_set_use_underline(GTK_BUTTON(pdialog->active_command), TRUE);
+  gtk_button_set_image_position(GTK_BUTTON(pdialog->active_command),
+                                GTK_POS_TOP);
+  gtk_widget_set_name(pdialog->active_command, "comment_label");
   gtk_container_add(GTK_CONTAINER(hbox), pdialog->active_command);
 
-  align = gtk_alignment_new(0.5, 0.5, 0.0, 0.0);
-  gtk_container_add(GTK_CONTAINER(pdialog->active_command), align);
-
-  vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 2);
-  gtk_container_add(GTK_CONTAINER(align), vbox);
-
-  pdialog->active_image = gtk_image_new();
-  gtk_box_pack_start(GTK_BOX(vbox), pdialog->active_image, FALSE, FALSE, 0);
-
-  pdialog->active_label = gtk_label_new(NULL);
-  gtk_widget_set_name(pdialog->active_label, "comment_label");
-  gtk_box_pack_end(GTK_BOX(vbox), pdialog->active_label, FALSE, FALSE, 0);
-
   gtk_widget_show_all(pdialog->shell);
 
   dialog_list_prepend(dialog_list, pdialog);
@@ -493,16 +489,19 @@ void refresh_cma_dialog(struct city *pcity, enum cma_refresh refresh)
       G_CALLBACK(cma_active_callback), pdialog);
 
   if (controlled) {
-    gtk_image_set_from_stock(GTK_IMAGE(pdialog->active_image),
-	GTK_STOCK_YES, GTK_ICON_SIZE_DND);
-    gtk_label_set_text_with_mnemonic(GTK_LABEL(pdialog->active_label),
-	_("Governor Enabl_ed"));
+    GtkWidget *image = gtk_image_new_from_stock(
+        GTK_STOCK_YES, GTK_ICON_SIZE_DND);
+    gtk_button_set_image(GTK_BUTTON(pdialog->active_command), image);
+    gtk_button_set_label(GTK_BUTTON(pdialog->active_command),
+                         _("Governor Enabl_ed"));
   } else {
-    gtk_image_set_from_stock(GTK_IMAGE(pdialog->active_image),
-	GTK_STOCK_NO, GTK_ICON_SIZE_DND);
-    gtk_label_set_text_with_mnemonic(GTK_LABEL(pdialog->active_label),
-	_("Governor Disabl_ed"));
+    GtkWidget *image = gtk_image_new_from_stock(
+        GTK_STOCK_NO, GTK_ICON_SIZE_DND);
+    gtk_button_set_image(GTK_BUTTON(pdialog->active_command), image);
+    gtk_button_set_label(GTK_BUTTON(pdialog->active_command),
+                         _("Governor Disabl_ed"));
   }
+  gtk_button_set_always_show_image(GTK_BUTTON(pdialog->active_command), TRUE);
 
   if (pdialog->result_label != NULL) {
     gtk_widget_set_sensitive(pdialog->result_label, controlled);
