@@ -34,7 +34,7 @@
 /* client */
 #include "options.h"
 
-/* client/gui-gtk-2.0 */
+/* client/gui-gtk-3.0 */
 #include "chatline.h"
 #include "citydlg.h"
 #include "gui_main.h"
@@ -51,6 +51,7 @@ struct meswin_dialog {
 
 /* Those values must match meswin_dialog_store_new(). */
 enum meswin_columns {
+  MESWIN_COL_ICON,
   MESWIN_COL_MESSAGE,
 
   /* Not visible. */
@@ -74,6 +75,7 @@ static struct meswin_dialog meswin = { NULL, };
 static GtkListStore *meswin_dialog_store_new(void)
 {
   return gtk_list_store_new(MESWIN_COL_NUM,
+                            GDK_TYPE_PIXBUF,    /* MESWIN_COL_ICON */
                             G_TYPE_STRING,      /* MESWIN_COL_MESSAGE */
                             G_TYPE_INT,         /* MESWIN_COL_WEIGHT */
                             G_TYPE_INT,         /* MESWIN_COL_STYLE */
@@ -141,21 +143,38 @@ static void meswin_dialog_refresh(struct meswin_dialog *pdialog)
 
   gtk_list_store_clear(store);
   for (i = 0; i < num; i++) {
+    GdkPixbuf *pb;
+    struct sprite *icon;
+    int x0, y0, x1, y1, w, h;
+    GdkPixbuf *pixbuf;
+
     pmsg = meswin_get_message(i);
 
-    if (gui_gtk2_new_messages_go_to_top) {
+    if (gui_options.gui_gtk3_new_messages_go_to_top) {
       gtk_list_store_prepend(store, &iter);
     } else {
       gtk_list_store_append(store, &iter);
     }
 
+    icon = get_event_sprite(tileset, pmsg->event);
+    sprite_get_bounding_box(icon, &x0, &y0, &x1, &y1);
+    w = (x1 - x0) + 1;
+    h = (y1 - y0) + 1;
+    pb = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, w, h);
+    pixbuf = sprite_get_pixbuf(icon);
+    gdk_pixbuf_copy_area(pixbuf, x0, y0, w, h,
+                         pb, 0, 0);
+    g_object_unref(G_OBJECT(pixbuf));
+
     meswin_dialog_visited_get_attr(pmsg->visited, &weight, &style);
     gtk_list_store_set(store, &iter,
+                       MESWIN_COL_ICON, pb,
                        MESWIN_COL_MESSAGE, pmsg->descr,
                        MESWIN_COL_WEIGHT, weight,
                        MESWIN_COL_STYLE, style,
                        MESWIN_COL_ID, i,
                        -1);
+    g_object_unref(pb);
     if (i == selected) {
       /* Restore the selection. */
       gtk_tree_selection_select_iter(selection, &iter);
@@ -300,7 +319,7 @@ static void meswin_dialog_response_callback(struct gui_dialog *pgui_dialog,
 static void meswin_dialog_init(struct meswin_dialog *pdialog)
 {
   GtkWidget *view, *sw, *cmd, *notebook;
-  GtkBox *vbox;
+  GtkContainer *vbox;
   GtkListStore *store;
   GtkTreeSelection *selection;
   GtkCellRenderer *renderer;
@@ -308,7 +327,7 @@ static void meswin_dialog_init(struct meswin_dialog *pdialog)
 
   fc_assert_ret(NULL != pdialog);
 
-  if (gui_gtk2_message_chat_location == GUI_GTK_MSGCHAT_SPLIT) {
+  if (gui_options.gui_gtk3_message_chat_location == GUI_GTK_MSGCHAT_SPLIT) {
     notebook = right_notebook;
   } else {
     notebook = bottom_notebook;
@@ -316,17 +335,19 @@ static void meswin_dialog_init(struct meswin_dialog *pdialog)
 
   gui_dialog_new(&pdialog->shell, GTK_NOTEBOOK(notebook), pdialog, TRUE);
   gui_dialog_set_title(pdialog->shell, _("Messages"));
-  vbox = GTK_BOX(pdialog->shell->vbox);
+  vbox = GTK_CONTAINER(pdialog->shell->vbox);
 
   sw = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
                                       GTK_SHADOW_ETCHED_IN);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
                                  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
-  gtk_box_pack_start(vbox, sw, TRUE, TRUE, 0);
+  gtk_container_add(vbox, sw);
 
   store = meswin_dialog_store_new();
   view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
+  gtk_widget_set_hexpand(view, TRUE);
+  gtk_widget_set_vexpand(view, TRUE);
   g_object_unref(store);
   gtk_tree_view_columns_autosize(GTK_TREE_VIEW(view));
   gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view), FALSE);
@@ -336,6 +357,12 @@ static void meswin_dialog_init(struct meswin_dialog *pdialog)
                    G_CALLBACK(meswin_dialog_button_press_callback), NULL);
   pdialog->tree_view = GTK_TREE_VIEW(view);
 
+  renderer = gtk_cell_renderer_pixbuf_new();
+  col = gtk_tree_view_column_new_with_attributes(NULL, renderer,
+                                                 "pixbuf", MESWIN_COL_ICON, NULL);
+  gtk_tree_view_append_column(GTK_TREE_VIEW(view), col);
+  gtk_tree_view_column_set_visible(col, !gui_options.gui_gtk3_small_display_layout);
+
   renderer = gtk_cell_renderer_text_new();
   col = gtk_tree_view_column_new_with_attributes(NULL, renderer,
                                                  "text", MESWIN_COL_MESSAGE,
@@ -349,18 +376,19 @@ static void meswin_dialog_init(struct meswin_dialog *pdialog)
   g_signal_connect(selection, "changed",
                    G_CALLBACK(meswin_dialog_selection_callback), pdialog);
 
-  if (gui_gtk2_show_message_window_buttons) {
-    cmd = gui_dialog_add_stockbutton(pdialog->shell, GTK_STOCK_JUMP_TO,
-                                     _("Goto _Location"), MESWIN_RES_GOTO);
-    gtk_widget_set_sensitive(cmd, FALSE);
+  gui_dialog_add_button(pdialog->shell, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
 
+  if (gui_options.gui_gtk3_show_message_window_buttons) {
     cmd = gui_dialog_add_stockbutton(pdialog->shell, GTK_STOCK_ZOOM_IN,
                                      _("I_nspect City"),
                                      MESWIN_RES_POPUP_CITY);
     gtk_widget_set_sensitive(cmd, FALSE);
+
+    cmd = gui_dialog_add_stockbutton(pdialog->shell, GTK_STOCK_JUMP_TO,
+                                     _("Goto _Location"), MESWIN_RES_GOTO);
+    gtk_widget_set_sensitive(cmd, FALSE);
   }
 
-  gui_dialog_add_button(pdialog->shell, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
   gui_dialog_response_set_callback(pdialog->shell,
                                    meswin_dialog_response_callback);
   gui_dialog_set_default_size(pdialog->shell, 520, 300);
