@@ -58,6 +58,7 @@
 #include "gui_stuff.h"
 #include "helpdlg.h"
 #include "plrdlg.h"
+#include "sprite.h"
 
 #include "repodlgs.h"
 
@@ -69,7 +70,7 @@ struct science_report {
   struct gui_dialog *shell;
   GtkComboBox *reachable_techs;
   GtkComboBox *reachable_goals;
-  GtkWidget *button_reachable;
+  GtkWidget *button_show_all;
   GtkLabel *main_label;         /* Gets science_dialog_text(). */
   GtkProgressBar *progress_bar;
   GtkLabel *goal_label;
@@ -91,15 +92,15 @@ static gboolean science_diagram_update(GtkWidget *widget,
                                        cairo_t *cr,
                                        gpointer data);
 static GtkWidget *science_diagram_new(void);
-static void science_diagram_data(GtkWidget *widget, bool reachable);
+static void science_diagram_data(GtkWidget *widget, bool show_all);
 static void science_diagram_center(GtkWidget *diagram, Tech_type_id tech);
 static void science_report_redraw(struct science_report *preport);
 static gint cmp_func(gconstpointer a_p, gconstpointer b_p);
 static void science_report_update(struct science_report *preport);
 static void science_report_current_callback(GtkComboBox *combo,
                                             gpointer data);
-static void science_report_unreachable_callback(GtkComboBox *combo,
-                                                gpointer data);
+static void science_report_show_all_callback(GtkComboBox *combo,
+                                             gpointer data);
 static void science_report_goal_callback(GtkComboBox *combo, gpointer data);
 static void science_report_init(struct science_report *preport);
 static void science_report_free(struct science_report *preport);
@@ -136,11 +137,13 @@ static inline void science_report_store_set(GtkListStore *store,
                                             GtkTreeIter *iter,
                                             Tech_type_id tech)
 {
+  const struct research *presearch = research_get(client_player());
+
   gtk_list_store_set(store, iter,
                      SRD_COL_NAME,
-                     advance_name_for_player(client_player(), tech),
+                     research_advance_name_translation(presearch, tech),
                      SRD_COL_STEPS,
-                     num_unknown_techs_for_goal(client_player(), tech),
+                     research_goal_unknown_techs(presearch, tech),
                      SRD_COL_ID, tech,
                      -1);
 }
@@ -194,6 +197,7 @@ static void science_report_combo_set_active(GtkComboBox *combo,
 static gboolean science_diagram_button_release_callback(GtkWidget *widget,
     GdkEventButton *event, gpointer data)
 {
+  const struct research *presearch = research_get(client_player());
   struct reqtree *reqtree = g_object_get_data(G_OBJECT(widget), "reqtree");
   Tech_type_id tech = get_tech_on_reqtree(reqtree, event->x, event->y);
 
@@ -203,12 +207,14 @@ static gboolean science_diagram_button_release_callback(GtkWidget *widget,
 
   if (event->button == 3) {
     /* RMB: get help */
-    popup_help_dialog_typed(advance_name_for_player(client_player(), tech),
+    popup_help_dialog_typed(research_advance_name_translation(presearch,
+                                                              tech),
                             HELP_TECH);
   } else {
     if (event->button == 1 && can_client_issue_orders()) {
       /* LMB: set research or research goal */
-      switch (player_invention_state(client_player(), tech)) {
+      switch (research_invention_state(research_get(client_player()),
+                                       tech)) {
        case TECH_PREREQS_KNOWN:
          dsend_packet_player_research(&client.conn, tech);
          break;
@@ -281,17 +287,17 @@ static GtkWidget *science_diagram_new(void)
 /****************************************************************************
   Recreate the req tree.
 ****************************************************************************/
-static void science_diagram_data(GtkWidget *widget, bool reachable)
+static void science_diagram_data(GtkWidget *widget, bool show_all)
 {
   struct reqtree *reqtree;
   int width, height;
 
   if (can_conn_edit(&client.conn)) {
     /* Show all techs in editor mode, not only currently reachable ones */
-    reqtree = create_reqtree(NULL, FALSE);
+    reqtree = create_reqtree(NULL, TRUE);
   } else {
     /* Show only at some point reachable techs */
-    reqtree = create_reqtree(client_player(), reachable);
+    reqtree = create_reqtree(client_player(), show_all);
   }
 
   get_reqtree_dimensions(reqtree, &width, &height);
@@ -345,11 +351,11 @@ static void science_report_redraw(struct science_report *preport)
   fc_assert_ret(NULL != preport);
 
   science_diagram_data(GTK_WIDGET(preport->drawing_area),
-                       !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-                         preport->button_reachable)));
+                       gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
+                         preport->button_show_all)));
 
   if (client_has_player()) {
-    researching = player_research_get(client_player())->researching;
+    researching = research_get(client_player())->researching;
   } else {
     researching = A_UNSET;
   }
@@ -365,9 +371,10 @@ static gint cmp_func(gconstpointer a_p, gconstpointer b_p)
 {
   const gchar *a_str, *b_str;
   gint a = GPOINTER_TO_INT(a_p), b = GPOINTER_TO_INT(b_p);
+  const struct research *presearch = research_get(client_player());
 
-  a_str = advance_name_for_player(client_player(), a);
-  b_str = advance_name_for_player(client_player(), b);
+  a_str = research_advance_name_translation(presearch, a);
+  b_str = research_advance_name_translation(presearch, b);
 
   return fc_strcoll(a_str, b_str);
 }
@@ -380,7 +387,7 @@ static void science_report_update(struct science_report *preport)
   GtkListStore *store;
   GtkTreeIter iter;
   GList *sorting_list, *item;
-  struct player_research *presearch = player_research_get(client_player());
+  struct research *presearch = research_get(client_player());
   const char *text;
   double pct;
   Tech_type_id tech;
@@ -449,7 +456,7 @@ static void science_report_update(struct science_report *preport)
 
   /* Collect all techs which are reachable in next 10 steps. */
   advance_index_iterate(A_FIRST, i) {
-    if (player_invention_reachable(client_player(), i, TRUE)
+    if (research_invention_reachable(presearch, i)
         && TECH_KNOWN != presearch->inventions[i].state
         && (i == presearch->tech_goal
             || 10 >= presearch->inventions[i].num_required_techs)) {
@@ -494,15 +501,15 @@ static void science_report_current_callback(GtkComboBox *combo,
     dsend_packet_player_research(&client.conn, tech);
   }
   /* Revert, or we will be not synchron with the server. */
-  science_report_combo_set_active(combo, player_research_get
+  science_report_combo_set_active(combo, research_get
                                   (client_player())->researching);
 }
 
 /****************************************************************************
   Show or hide unreachable techs.
 ****************************************************************************/
-static void science_report_unreachable_callback(GtkComboBox *combo,
-                                                gpointer data)
+static void science_report_show_all_callback(GtkComboBox *combo,
+                                             gpointer data)
 {
   struct science_report *preport = (struct science_report *) data;
 
@@ -527,7 +534,7 @@ static void science_report_goal_callback(GtkComboBox *combo, gpointer data)
     dsend_packet_player_tech_goal(&client.conn, tech);
   }
   /* Revert, or we will be not synchron with the server. */
-  science_report_combo_set_active(combo, player_research_get
+  science_report_combo_set_active(combo, research_get
                                   (client_player())->tech_goal);
 }
 
@@ -536,7 +543,7 @@ static void science_report_goal_callback(GtkComboBox *combo, gpointer data)
 ****************************************************************************/
 static void science_report_init(struct science_report *preport)
 {
-  GtkWidget *frame, *table, *help_button, *reachable_button, *sw, *w;
+  GtkWidget *frame, *table, *help_button, *show_all_button, *sw, *w;
   GtkSizeGroup *group;
   GtkContainer *vbox;
   GtkListStore *store;
@@ -622,15 +629,15 @@ static void science_report_init(struct science_report *preport)
   gtk_widget_set_size_request(w, -1, 25);
   preport->goal_label = GTK_LABEL(w);
 
-  /* Toggle unreachable button. */
-  /* TRANS: As in 'Show all (even currently not reachable) techs'. */
-  reachable_button = gtk_toggle_button_new_with_label(_("Show all"));
-  gtk_grid_attach(GTK_GRID(table), reachable_button, 5, 0, 1, 1);
-  g_signal_connect(reachable_button, "toggled",
-                   G_CALLBACK(science_report_unreachable_callback), preport);
-  gtk_widget_set_sensitive(reachable_button, can_client_issue_orders()
+  /* Toggle "Show All" button. */
+  /* TRANS: As in 'Show all (even not reachable) techs'. */
+  show_all_button = gtk_toggle_button_new_with_label(_("Show all"));
+  gtk_grid_attach(GTK_GRID(table), show_all_button, 5, 0, 1, 1);
+  g_signal_connect(show_all_button, "toggled",
+                   G_CALLBACK(science_report_show_all_callback), preport);
+  gtk_widget_set_sensitive(show_all_button, can_client_issue_orders()
                                              && !client_is_global_observer());
-  preport->button_reachable = reachable_button;
+  preport->button_show_all = show_all_button;
 
   /* Science diagram. */
   sw = gtk_scrolled_window_new(NULL, NULL);
@@ -671,8 +678,7 @@ static void science_report_free(struct science_report *preport)
 ****************************************************************************/
 void science_report_dialog_popup(bool raise)
 {
-  struct player_research *presearch =
-      (client_has_player() ? player_research_get(client_player()) : NULL);
+  struct research *presearch = research_get(client_player());
 
   if (NULL == science_report.shell) {
     science_report_init(&science_report);
@@ -841,7 +847,7 @@ static void economy_report_update(struct economy_report *preport)
     struct improvement_entry *pentry = building_entries + i;
     struct impr_type *pimprove = pentry->type;
     struct sprite *sprite = get_building_sprite(tileset, pimprove);
-    cid cid = cid_encode_building(pimprove);
+    cid id = cid_encode_building(pimprove);
 
     pix = sprite_get_pixbuf(sprite);
     gtk_list_store_append(store, &iter);
@@ -853,10 +859,10 @@ static void economy_report_update(struct economy_report *preport)
                        ERD_COL_COST, pentry->cost,
                        ERD_COL_TOTAL_COST, pentry->total_cost,
                        ERD_COL_IS_IMPROVEMENT, TRUE,
-                       ERD_COL_CID, cid,
+                       ERD_COL_CID, id,
                        -1);
     g_object_unref(G_OBJECT(pix));
-    if (selected == cid) {
+    if (selected == id) {
       /* Restore the selection. */
       gtk_tree_selection_select_iter(selection, &iter);
     }
@@ -869,7 +875,7 @@ static void economy_report_update(struct economy_report *preport)
     struct unit_type *putype = pentry->type;
     struct sprite *sprite = get_unittype_sprite(tileset, putype,
                                                 direction8_invalid(), TRUE);
-    cid cid = cid_encode_unit(putype);
+    cid id = cid_encode_unit(putype);
 
     pix = sprite_get_pixbuf(sprite);
     gtk_list_store_append(store, &iter);
@@ -881,10 +887,10 @@ static void economy_report_update(struct economy_report *preport)
                        ERD_COL_COST, pentry->cost,
                        ERD_COL_TOTAL_COST, pentry->total_cost,
                        ERD_COL_IS_IMPROVEMENT, FALSE,
-                       ERD_COL_CID, cid,
+                       ERD_COL_CID, id,
                        -1);
     g_object_unref(G_OBJECT(pix));
-    if (selected == cid) {
+    if (selected == id) {
       /* Restore the selection. */
       gtk_tree_selection_select_iter(selection, &iter);
     }
@@ -909,7 +915,7 @@ static void economy_report_command_callback(struct gui_dialog *pdialog,
   GtkTreeIter iter;
   GtkWidget *shell;
   struct universal selected;
-  cid cid;
+  cid id;
   char buf[256] = "";
 
   switch (response) {
@@ -927,8 +933,8 @@ static void economy_report_command_callback(struct gui_dialog *pdialog,
     return;
   }
 
-  gtk_tree_model_get(model, &iter, ERD_COL_CID, &cid, -1);
-  selected = cid_decode(cid);
+  gtk_tree_model_get(model, &iter, ERD_COL_CID, &id, -1);
+  selected = cid_decode(id);
 
   switch (selected.kind) {
   case VUT_IMPROVEMENT:
@@ -1024,10 +1030,10 @@ static void economy_report_selection_callback(GtkTreeSelection *selection,
   if (can_client_issue_orders()
       && gtk_tree_selection_get_selected(selection, &model, &iter)) {
     struct universal selected;
-    cid cid;
+    cid id;
 
-    gtk_tree_model_get(model, &iter, ERD_COL_CID, &cid, -1);
-    selected = cid_decode(cid);
+    gtk_tree_model_get(model, &iter, ERD_COL_CID, &id, -1);
+    selected = cid_decode(id);
     switch (selected.kind) {
     case VUT_IMPROVEMENT:
       {
@@ -1269,16 +1275,16 @@ static const struct {
     N_("Upgradable"),             TRUE,   URD_COL_UPG_VISIBLE },
   { /* URD_COL_N_UPGRADABLE */ G_TYPE_INT,     "" /* merge with previous col */,
     NULL,                         TRUE,   URD_COL_NUPG_VISIBLE },
-  /* TRANS: "In progress" abbrevation. */
+  /* TRANS: "In progress" abbreviation. */
   { /* URD_COL_IN_PROGRESS */  G_TYPE_INT,     N_("In-Prog"),
     N_("In progress"),            TRUE,   -1 },
   { /* URD_COL_ACTIVE */       G_TYPE_INT,     N_("Active"),
     NULL,                         TRUE,   -1 },
   { /* URD_COL_SHIELD */       G_TYPE_INT,     N_("Shield"),
     N_("Total shield upkeep"),    TRUE,   -1 },
-  { /* URD_COL_SHIELD */       G_TYPE_INT,     N_("Food"),
+  { /* URD_COL_FOOD */         G_TYPE_INT,     N_("Food"),
     N_("Total food upkeep"),      TRUE,   -1 },
-  { /* URD_COL_SHIELD */       G_TYPE_INT,     N_("Gold"),
+  { /* URD_COL_GOLD */         G_TYPE_INT,     N_("Gold"),
     N_("Total gold upkeep"),      TRUE,   -1 },
   { /* URD_COL_TEXT_WEIGHT */  G_TYPE_INT,     NULL /* ... */ },
   { /* URD_COL_UPG_VISIBLE */  G_TYPE_BOOLEAN, NULL /* ... */ },
@@ -1335,7 +1341,7 @@ static void units_report_update(struct units_report *preport)
     }
 
     unit_list_iterate(pplayer->units, punit) {
-      info = unit_array + utype_index(unit_type(punit));
+      info = unit_array + utype_index(unit_type_get(punit));
 
       if (0 != punit->homecity) {
         output_type_iterate(o) {
@@ -1488,7 +1494,7 @@ static struct unit *find_nearest_unit(const struct unit_type *utype,
     }
 
     unit_list_iterate(pplayer->units, punit) {
-      if (utype == unit_type(punit)
+      if (utype == unit_type_get(punit)
           && FOCUS_AVAIL == punit->client.focus_status
           && 0 < punit->moves_left
           && !punit->done_moving
