@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -41,7 +41,7 @@
 #include "control.h"
 #include "mapview_common.h"
 
-/* client/gui-gtk-3.0 */
+/* client/gui-gtk-3.22 */
 #include "colors.h"
 #include "gui_main.h"
 #include "gui_stuff.h"
@@ -144,9 +144,11 @@ static void inputline_return(GtkEntry *w, gpointer data)
   theinput = gtk_entry_get_text(w);
   
   if (*theinput) {
-    if (client_state() == C_S_RUNNING && gui_gtk3_allied_chat_only
+    if (client_state() == C_S_RUNNING
+        && GUI_GTK_OPTION(allied_chat_only)
         && is_plain_public_message(theinput)) {
       char buf[MAX_LEN_MSG];
+
       fc_snprintf(buf, sizeof(buf), ". %s", theinput);
       send_chat(buf);
     } else {
@@ -154,11 +156,11 @@ static void inputline_return(GtkEntry *w, gpointer data)
     }
 
     if (genlist_size(history_list) >= MAX_CHATLINE_HISTORY) {
-      void *data;
+      void *history_data;
 
-      data = genlist_get(history_list, -1);
-      genlist_remove(history_list, data);
-      free(data);
+      history_data = genlist_get(history_list, -1);
+      genlist_remove(history_list, history_data);
+      free(history_data);
     }
 
     genlist_prepend(history_list, fc_strdup(theinput));
@@ -397,7 +399,7 @@ static gboolean inputline_handler(GtkWidget *w, GdkEventKey *ev)
       return TRUE;
 
     case GDK_KEY_Tab:
-      if (gui_gtk3_chatline_autocompletion) {
+      if (GUI_GTK_OPTION(chatline_autocompletion)) {
         return chatline_autocomplete(GTK_EDITABLE(w));
       }
 
@@ -445,8 +447,8 @@ void inputline_make_tag(GtkEntry *entry, enum text_tag_type type)
 
     if (0 == featured_text_apply_tag(selection, buf, sizeof(buf),
                                      TTT_COLOR, 0, FT_OFFSET_UNSET,
-                                     ft_color(fg_color_text,
-                                              bg_color_text))) {
+                                     ft_color_construct(fg_color_text,
+                                                        bg_color_text))) {
       goto CLEAN_UP;
     }
   } else if (0 == featured_text_apply_tag(selection, buf, sizeof(buf),
@@ -893,7 +895,7 @@ void real_output_window_append(const char *astring,
   gtk_text_buffer_insert(buf, &iter, "\n", -1);
   mark = gtk_text_buffer_create_mark(buf, NULL, &iter, TRUE);
 
-  if (gui_gtk3_show_chat_message_time) {
+  if (GUI_GTK_OPTION(show_chat_message_time)) {
     char timebuf[64];
     time_t now;
     struct tm *now_tm;
@@ -1131,9 +1133,9 @@ static void select_color_callback(GtkToolButton *button, gpointer data)
               (const char *) g_object_get_data(G_OBJECT(button),
                                                "color_info"));
   dialog = gtk_dialog_new_with_buttons(buf, NULL, GTK_DIALOG_MODAL,
-                                       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                                       GTK_STOCK_CLEAR, GTK_RESPONSE_REJECT,
-                                       GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
+                                       _("Cancel"), GTK_RESPONSE_CANCEL,
+                                       _("Clear"), GTK_RESPONSE_REJECT,
+                                       _("OK"), GTK_RESPONSE_OK, NULL);
   setup_dialog(dialog, toplevel);
   g_object_set_data(G_OBJECT(dialog), "button", button);
   g_signal_connect(dialog, "response", G_CALLBACK(color_selected), data);
@@ -1172,7 +1174,11 @@ static gboolean move_toolkit(GtkWidget *toolkit_view,
     if (ptoolkit->toolbar_displayed) {
       gtk_widget_hide(ptoolkit->toolbar);
     }
-    gtk_widget_reparent(ptoolkit->main_widget, toolkit_view);
+    g_object_ref(ptoolkit->main_widget); /* Make sure reference count stays above 0
+                                          * during the transition to new parent. */
+    gtk_container_remove(GTK_CONTAINER(parent), ptoolkit->main_widget);
+    gtk_container_add(GTK_CONTAINER(toolkit_view), ptoolkit->main_widget);
+    g_object_unref(ptoolkit->main_widget);
     if (ptoolkit->toolbar_displayed) {
       gtk_widget_show(ptoolkit->toolbar);
     }
@@ -1335,7 +1341,9 @@ void chatline_init(void)
   toolkit.toolbar = toolbar;
 
   /* Bold button. */
-  item = gtk_tool_button_new_from_stock(GTK_STOCK_BOLD);
+  item = gtk_tool_button_new(gtk_image_new_from_icon_name("format-text-bold", 0),
+                             _("Bold"));
+
   gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
   g_object_set_data(G_OBJECT(item), "text_tag_type",
                     GINT_TO_POINTER(TTT_BOLD));
@@ -1343,7 +1351,8 @@ void chatline_init(void)
   gtk_widget_set_tooltip_text(GTK_WIDGET(item), _("Bold (Ctrl-B)"));
 
   /* Italic button. */
-  item = gtk_tool_button_new_from_stock(GTK_STOCK_ITALIC);
+  item = gtk_tool_button_new(gtk_image_new_from_icon_name("format-text-italic", 0),
+                             _("Italic"));
   gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
   g_object_set_data(G_OBJECT(item), "text_tag_type",
                     GINT_TO_POINTER(TTT_ITALIC));
@@ -1351,7 +1360,8 @@ void chatline_init(void)
   gtk_widget_set_tooltip_text(GTK_WIDGET(item), _("Italic (Ctrl-I)"));
 
   /* Strike button. */
-  item = gtk_tool_button_new_from_stock(GTK_STOCK_STRIKETHROUGH);
+  item = gtk_tool_button_new(gtk_image_new_from_icon_name("format-text-strikethrough", 0),
+                             _("Strikethrough"));
   gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
   g_object_set_data(G_OBJECT(item), "text_tag_type",
                     GINT_TO_POINTER(TTT_STRIKE));
@@ -1359,7 +1369,8 @@ void chatline_init(void)
   gtk_widget_set_tooltip_text(GTK_WIDGET(item), _("Strikethrough (Ctrl-S)"));
 
   /* Underline button. */
-  item = gtk_tool_button_new_from_stock(GTK_STOCK_UNDERLINE);
+  item = gtk_tool_button_new(gtk_image_new_from_icon_name("format-text-underline", 0),
+                             _("Underline"));
   gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
   g_object_set_data(G_OBJECT(item), "text_tag_type",
                     GINT_TO_POINTER(TTT_UNDERLINE));
@@ -1367,7 +1378,7 @@ void chatline_init(void)
   gtk_widget_set_tooltip_text(GTK_WIDGET(item), _("Underline (Ctrl-U)"));
 
   /* Color button. */
-  item = gtk_tool_button_new_from_stock(GTK_STOCK_SELECT_COLOR);
+  item = gtk_tool_button_new(NULL, _("Color"));
   gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
   g_object_set_data(G_OBJECT(item), "text_tag_type",
                     GINT_TO_POINTER(TTT_COLOR));
@@ -1414,7 +1425,7 @@ void chatline_init(void)
                      gtk_separator_tool_item_new(), -1);
 
   /* Return button. */
-  item = gtk_tool_button_new_from_stock(GTK_STOCK_OK);
+  item = gtk_tool_button_new(NULL, _("OK"));
   gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
   g_signal_connect_swapped(item, "clicked",
                            G_CALLBACK(inputline_return), entry);
@@ -1432,8 +1443,7 @@ void chatline_init(void)
   g_object_set(button, "margin", 2, NULL);
   gtk_container_add(GTK_CONTAINER(hbox), button);
   gtk_button_set_image(GTK_BUTTON(button),
-                       gtk_image_new_from_stock(GTK_STOCK_EDIT,
-                                                GTK_ICON_SIZE_MENU));
+                       gtk_image_new_from_icon_name("gtk-edit", 0));
   g_signal_connect(button, "toggled", G_CALLBACK(button_toggled), &toolkit);
   gtk_widget_set_tooltip_text(GTK_WIDGET(button), _("Chat tools"));
   toolkit.toggle_button = button;
@@ -1467,7 +1477,7 @@ static gboolean version_message_main_thread(gpointer user_data)
 /**************************************************************************
   Got version message from metaserver thread.
 **************************************************************************/
-void version_message(char *vertext)
+void version_message(const char *vertext)
 {
   int len = strlen(vertext) + 1;
   char *persistent = fc_malloc(len);
