@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996-2005 - Freeciv Development Team
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,8 +18,10 @@
 
 #include <gtk/gtk.h>
 
+/* utility */
 #include "support.h"
 
+/* gui-gtk-3.22 */
 #include "gui_main.h"
 #include "gui_stuff.h"
 
@@ -30,6 +32,14 @@
   placed vertically
 ****************************************************************/
 
+/***********************************************************************
+  Get the number of buttons in the choice dialog.
+***********************************************************************/
+int choice_dialog_get_number_of_buttons(GtkWidget *cd)
+{
+  return GPOINTER_TO_INT(g_object_get_data(G_OBJECT(cd), "nbuttons"));
+}
+
 /****************************************************************
   Get nth button widget from dialog
 *****************************************************************/
@@ -65,6 +75,28 @@ void choice_dialog_button_set_label(GtkWidget *cd, int number,
   gtk_button_set_label(GTK_BUTTON(button), label);
 }
 
+/****************************************************************
+  Set tool tip for choice dialog button.
+*****************************************************************/
+void choice_dialog_button_set_tooltip(GtkWidget *cd, int number,
+                                      const char* tool_tip)
+{
+  GtkWidget* button = choice_dialog_get_nth_button(cd, number);
+  gtk_widget_set_tooltip_text(button, tool_tip);
+}
+
+/****************************************************************
+  Move the specified button to the end.
+*****************************************************************/
+void choice_dialog_button_move_to_the_end(GtkWidget *cd,
+                                          const int number)
+{
+  GtkWidget *button = choice_dialog_get_nth_button(cd, number);
+  GtkWidget *bbox = g_object_get_data(G_OBJECT(cd), "bbox");
+
+  gtk_box_reorder_child(GTK_BOX(bbox), button, -1);
+}
+
 /****************************************************************
   Create choice dialog
 *****************************************************************/
@@ -124,19 +156,20 @@ static void choice_dialog_clicked(GtkWidget *w, gpointer data)
   Add button to choice dialog.
 *****************************************************************/
 void choice_dialog_add(GtkWidget *dshell, const gchar *label,
-			GCallback handler, gpointer data)
+                       GCallback handler, gpointer data,
+                       bool meta, const gchar *tool_tip)
 {
   GtkWidget *button, *bbox;
   char name[512];
   int nbuttons;
 
   bbox = g_object_get_data(G_OBJECT(dshell), "bbox");
-  nbuttons = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(dshell), "nbuttons"));
+  nbuttons = choice_dialog_get_number_of_buttons(dshell);
   g_object_set_data(G_OBJECT(dshell), "nbuttons", GINT_TO_POINTER(nbuttons+1));
 
   fc_snprintf(name, sizeof(name), "button%d", nbuttons);
 
-  button = gtk_button_new_from_stock(label);
+  button = gtk_button_new_with_label(label);
   gtk_container_add(GTK_CONTAINER(bbox), button);
   g_object_set_data(G_OBJECT(dshell), name, button);
 
@@ -144,8 +177,15 @@ void choice_dialog_add(GtkWidget *dshell, const gchar *label,
     g_signal_connect(button, "clicked", handler, data);
   }
 
-  g_signal_connect_after(button, "clicked",
-			 G_CALLBACK(choice_dialog_clicked), dshell);
+  if (!meta) {
+    /* This button makes the choice. */
+    g_signal_connect_after(button, "clicked",
+                           G_CALLBACK(choice_dialog_clicked), dshell);
+  }
+
+  if (tool_tip != NULL) {
+    gtk_widget_set_tooltip_text(button, tool_tip);
+  }
 }
 
 /****************************************************************
@@ -190,7 +230,7 @@ GtkWidget *popup_choice_dialog(GtkWindow *parent, const gchar *dialogname,
     handler = va_arg(args, GCallback);
     data = va_arg(args, gpointer);
 
-    choice_dialog_add(dshell, name, handler, data);
+    choice_dialog_add(dshell, name, handler, data, FALSE, NULL);
   }
 
   va_end(args);
