@@ -1,4 +1,4 @@
-/**********************************************************************
+/***********************************************************************
  Freeciv - Copyright (C) 2001 - R. Falke
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -37,8 +37,6 @@ struct cma_dialog {
   GtkWidget *result_label;
   GtkWidget *add_preset_command;
   GtkWidget *del_preset_command;
-  GtkWidget *active_label;
-  GtkWidget *active_image;
   GtkWidget *active_command;
   GtkWidget *minimal_surplus[O_LAST];
   GtkWidget *happy_button;
