@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -66,6 +66,6 @@ void close_connection_dialog()
   gtk client does nothing here. This gets called when one is rejected
   from game.
 **************************************************************************/
-void gui_server_connect(void)
+void server_connect(void)
 {
 }
