@@ -1,4 +1,4 @@
-/**********************************************************************
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@
 /* client */
 #include "options.h"
 
-/* client/gui-gtk-3.0 */
+/* client/gui-gtk-3.22 */
 #include "colors.h"
 #include "gui_main.h"
 
@@ -43,6 +43,8 @@ static GList *dialog_list;
 
 static GtkSizeGroup *gui_action;
 
+static GtkCssProvider *dlg_tab_provider = NULL;
+
 
 /**************************************************************************
   Draw widget now
@@ -53,32 +55,36 @@ void gtk_expose_now(GtkWidget *w)
 }
 
 /**************************************************************************
-  Set widget position relative to reference widget
+  Set window position relative to reference window
 **************************************************************************/
-void gtk_set_relative_position(GtkWidget *ref, GtkWidget *w, int px, int py)
+void set_relative_window_position(GtkWindow *ref, GtkWindow *w, int px, int py)
 {
   gint x, y, width, height;
 
-  gtk_window_get_position(GTK_WINDOW(ref), &x, &y);
-  gtk_window_get_size(GTK_WINDOW(ref), &width, &height);
+  gtk_window_get_position(ref, &x, &y);
+  gtk_window_get_size(ref, &width, &height);
 
-  x += px*width/100;
-  y += py*height/100;
+  x += px * width / 100;
+  y += py * height / 100;
 
-  gtk_window_move(GTK_WINDOW(w), x, y);
+  gtk_window_move(w, x, y);
 }
 
 /**************************************************************************
-  Create new stock button
+  Create new icon button with label
 **************************************************************************/
-GtkWidget *gtk_stockbutton_new(const gchar *stock, const gchar *label_text)
+GtkWidget *icon_label_button_new(const gchar *icon_name,
+                                 const gchar *label_text)
 {
   GtkWidget *button;
   GtkWidget *image;
-  
+
   button = gtk_button_new_with_mnemonic(label_text);
-  image = gtk_image_new_from_stock(stock, GTK_ICON_SIZE_BUTTON);
-  gtk_button_set_image(GTK_BUTTON(button), image);
+
+  if (icon_name != NULL) {
+    image = gtk_image_new_from_icon_name(icon_name, GTK_ICON_SIZE_BUTTON);
+    gtk_button_set_image(GTK_BUTTON(button), image);
+  }
 
   return button;
 }
@@ -278,7 +284,7 @@ static void close_callback(GtkDialog *dialog, gpointer data)
 ***********************************************************************/
 void setup_dialog(GtkWidget *shell, GtkWidget *parent)
 {
-  if (gui_gtk3_dialogs_on_top || fullscreen_mode) {
+  if (GUI_GTK_OPTION(dialogs_on_top) || GUI_GTK_OPTION(fullscreen)) {
     gtk_window_set_transient_for(GTK_WINDOW(shell),
                                  GTK_WINDOW(parent));
     gtk_window_set_type_hint(GTK_WINDOW(shell),
@@ -294,7 +300,6 @@ void setup_dialog(GtkWidget *shell, GtkWidget *parent)
   }
 }
 
-
 /**************************************************************************
   Emit a dialog response.
 **************************************************************************/
@@ -426,7 +431,8 @@ static void gui_dialog_switch_page_handler(GtkNotebook *notebook,
   n = gtk_notebook_page_num(GTK_NOTEBOOK(dlg->v.tab.notebook), dlg->vbox);
 
   if (n == num) {
-    gtk_widget_override_color(dlg->v.tab.label, GTK_STATE_FLAG_NORMAL, NULL);
+    gtk_style_context_remove_class(gtk_widget_get_style_context(dlg->v.tab.label),
+                                   "alert");
   }
 }
 
@@ -517,7 +523,7 @@ void gui_dialog_new(struct gui_dialog **pdlg, GtkNotebook *notebook,
   dlg->default_width = 200;
   dlg->default_height = 300;
 
-  if (gui_gtk3_enable_tabs) {
+  if (GUI_GTK_OPTION(enable_tabs)) {
     dlg->type = GUI_DIALOG_TAB;
   } else {
     dlg->type = GUI_DIALOG_WINDOW;
@@ -532,9 +538,9 @@ void gui_dialog_new(struct gui_dialog **pdlg, GtkNotebook *notebook,
   action_area = gtk_grid_new();
   gtk_grid_set_row_spacing(GTK_GRID(action_area), 4);
   gtk_grid_set_column_spacing(GTK_GRID(action_area), 4);
-  if (gui_gtk3_enable_tabs &&
-      (check_top && notebook != GTK_NOTEBOOK(top_notebook))
-      && !gui_gtk3_small_display_layout) {
+  if (GUI_GTK_OPTION(enable_tabs)
+      && (check_top && notebook != GTK_NOTEBOOK(top_notebook))
+      && !GUI_GTK_OPTION(small_display_layout)) {
     /* We expect this to be short (as opposed to tall); maximise usable
      * height by putting buttons down the right hand side */
     gtk_orientable_set_orientation(GTK_ORIENTABLE(action_area),
@@ -576,17 +582,15 @@ void gui_dialog_new(struct gui_dialog **pdlg, GtkNotebook *notebook,
       gint w, h;
       gchar *buf;
 
-      gtk_icon_size_lookup_for_settings(
-        gtk_settings_get_for_screen(gtk_widget_get_screen(vbox)),
-        GTK_ICON_SIZE_MENU, &w, &h);
+      gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &w, &h);
 
       hbox = gtk_grid_new();
 
       label = gtk_label_new(NULL);
       gtk_widget_set_halign(label, GTK_ALIGN_START);
       gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
-      gtk_widget_set_margin_left(label, 4);
-      gtk_widget_set_margin_right(label, 4);
+      gtk_widget_set_margin_start(label, 4);
+      gtk_widget_set_margin_end(label, 4);
       gtk_widget_set_margin_top(label, 0);
       gtk_widget_set_margin_bottom(label, 0);
       gtk_container_add(GTK_CONTAINER(hbox), label);
@@ -600,9 +604,9 @@ void gui_dialog_new(struct gui_dialog **pdlg, GtkNotebook *notebook,
       gtk_widget_set_tooltip_text(button, buf);
       g_free(buf);
 
-      image = gtk_image_new_from_stock(GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
-      gtk_widget_set_margin_left(image, 0);
-      gtk_widget_set_margin_right(image, 0);
+      image = gtk_image_new_from_icon_name("window-close", GTK_ICON_SIZE_MENU);
+      gtk_widget_set_margin_start(image, 0);
+      gtk_widget_set_margin_end(image, 0);
       gtk_widget_set_margin_top(image, 0);
       gtk_widget_set_margin_bottom(image, 0);
       gtk_button_set_image(GTK_BUTTON(button), image);
@@ -621,6 +625,9 @@ void gui_dialog_new(struct gui_dialog **pdlg, GtkNotebook *notebook,
 	    G_CALLBACK(gui_dialog_switch_page_handler), dlg);
       dlg->v.tab.child = vbox;
 
+      gtk_style_context_add_provider(gtk_widget_get_style_context(label),
+                                     GTK_STYLE_PROVIDER(dlg_tab_provider),
+                                     GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
       dlg->v.tab.label = label;
       dlg->v.tab.notebook = GTK_WIDGET(notebook);
       
@@ -665,7 +672,7 @@ static void action_widget_activated(GtkWidget *button, GtkWidget *vbox)
   Places a button into a dialog, taking care of setting up signals, etc.
 **************************************************************************/
 static void gui_dialog_pack_button(struct gui_dialog *dlg, GtkWidget *button,
-				   int response)
+                                   int response)
 {
   gint signal_id;
 
@@ -687,31 +694,16 @@ static void gui_dialog_pack_button(struct gui_dialog *dlg, GtkWidget *button,
   gtk_size_group_add_widget(dlg->gui_button, button);
 }
 
-/**************************************************************************
-  Adds a button to a dialog, allowing the choice of a special stock item.
-**************************************************************************/
-GtkWidget *gui_dialog_add_stockbutton(struct gui_dialog *dlg,
-				      const char *stock,
-				      const char *text, int response)
-{
-  GtkWidget *button;
-
-  button = gtk_stockbutton_new(stock, text);
-  gtk_widget_set_can_default(button, TRUE);
-  gui_dialog_pack_button(dlg, button, response);
-
-  return button;
-}
-
 /**************************************************************************
   Adds a button to a dialog.
 **************************************************************************/
 GtkWidget *gui_dialog_add_button(struct gui_dialog *dlg,
-				 const char *text, int response)
+                                 const char *icon_name,
+                                 const char *text, int response)
 {
   GtkWidget *button;
 
-  button = gtk_button_new_from_stock(text);
+  button = icon_label_button_new(icon_name, text);
   gtk_widget_set_can_default(button, TRUE);
   gui_dialog_pack_button(dlg, button, response);
 
@@ -852,9 +844,9 @@ void gui_dialog_present(struct gui_dialog *dlg)
 
       if (current != n) {
 	GtkWidget *label = dlg->v.tab.label;
-	GdkRGBA color = {.red = 1.0, .green = 0, .blue = 0, .alpha = 1.0};
 
-	gtk_widget_override_color(label, GTK_STATE_FLAG_NORMAL, &color);
+        gtk_style_context_add_class(gtk_widget_get_style_context(label),
+                                    "alert");
       }
     }
     break;
@@ -904,9 +896,9 @@ void gui_dialog_alert(struct gui_dialog *dlg)
 
       if (current != n) {
         GtkWidget *label = dlg->v.tab.label;
-        GdkRGBA color = {.red = 0, .green = 0, .blue =1.0, .alpha = 1.0};
 
-        gtk_widget_override_color(label, GTK_STATE_FLAG_NORMAL, &color);
+        gtk_style_context_add_class(gtk_widget_get_style_context(label),
+                                    "alert");
       }
     }
     break;
@@ -1140,5 +1132,20 @@ GtkTreeViewColumn *add_treeview_column(GtkWidget *view, const char *title,
   col = gtk_tree_view_column_new_with_attributes(title, rend, attr,
                                                  model_index, NULL);
   gtk_tree_view_append_column(GTK_TREE_VIEW(view), col);
+
   return col;
 }
+
+/**************************************************************************
+  Prepare dialog tab style provider.
+**************************************************************************/
+void dlg_tab_provider_prepare(void)
+{
+  dlg_tab_provider = gtk_css_provider_new();
+
+  gtk_css_provider_load_from_data(dlg_tab_provider,
+                                  ".alert {\n"
+                                  "color: rgba(255, 0, 0, 255);\n"
+                                  "}\n",
+                                  -1, NULL);
+}
