@@ -32,7 +32,7 @@
 #include "text.h"
 #include "tilespec.h"
 
-/* client/gui-gtk-3.0 */
+/* client/gui-gtk-3.22 */
 #include "graphics.h"
 #include "gui_main.h"
 #include "gui_stuff.h"
@@ -42,8 +42,8 @@
 /* semi-arbitrary number that controls the width of the happiness widget */
 #define HAPPINESS_PIX_WIDTH 30
 
-#define	PIXCOMM_WIDTH	(HAPPINESS_PIX_WIDTH * tileset_small_sprite_width(tileset))
-#define	PIXCOMM_HEIGHT	(tileset_small_sprite_height(tileset))
+#define	FEELING_WIDTH	(HAPPINESS_PIX_WIDTH * tileset_small_sprite_width(tileset))
+#define	FEELING_HEIGHT	(tileset_small_sprite_height(tileset))
 
 #define NUM_HAPPINESS_MODIFIERS 6
 
@@ -53,7 +53,8 @@ struct happiness_dialog {
   struct city *pcity;
   GtkWidget *shell;
   GtkWidget *cityname_label;
-  GtkWidget *hpixmaps[NUM_HAPPINESS_MODIFIERS];
+  cairo_surface_t *feeling_surfaces[NUM_HAPPINESS_MODIFIERS];
+  GtkWidget *feeling_images[NUM_HAPPINESS_MODIFIERS];
   GtkWidget *happiness_ebox[NUM_HAPPINESS_MODIFIERS];
   GtkWidget *happiness_label[NUM_HAPPINESS_MODIFIERS];
   GtkWidget *close;
@@ -155,16 +156,16 @@ static gboolean show_happiness_popup(GtkWidget *w,
 
     label = gtk_label_new(buf);
     gtk_widget_set_name(label, "city_happiness_label");
-    gtk_widget_set_margin_left(label, 4);
-    gtk_widget_set_margin_right(label, 4);
+    gtk_widget_set_margin_start(label, 4);
+    gtk_widget_set_margin_end(label, 4);
     gtk_widget_set_margin_top(label, 4);
     gtk_widget_set_margin_bottom(label, 4);
     gtk_container_add(GTK_CONTAINER(frame), label);
     gtk_widget_show_all(p);
 
-    gdk_device_grab(ev->device, gtk_widget_get_window(p),
-                    GDK_OWNERSHIP_NONE, TRUE, GDK_BUTTON_RELEASE_MASK, NULL,
-                    ev->time);
+    gdk_seat_grab(gdk_device_get_seat(ev->device), gtk_widget_get_window(p),
+                  GDK_SEAT_CAPABILITY_ALL_POINTING,
+                  TRUE, NULL, (GdkEvent *)ev, NULL, NULL);
     gtk_grab_add(p);
 
     g_signal_connect_after(p, "button_release_event",
@@ -177,13 +178,13 @@ static gboolean show_happiness_popup(GtkWidget *w,
 /**************************************************************************
   Clear the happiness popup.
 **************************************************************************/
-static gboolean show_happiness_button_release(GtkWidget *w,
-                                              GdkEventButton *ev,
+static gboolean show_happiness_button_release(GtkWidget *w, GdkEventButton *ev,
                                               gpointer data)
 {
   gtk_grab_remove(w);
-  gdk_device_ungrab(ev->device, ev->time);
+  gdk_seat_ungrab(gdk_device_get_seat(ev->device));
   gtk_widget_destroy(w);
+
   return FALSE;
 }
 
@@ -228,6 +229,8 @@ static struct happiness_dialog *create_happiness_dialog(struct city *pcity,
   gtk_container_add(GTK_CONTAINER(pdialog->cityname_label), table);
 
   for (i = 0; i < NUM_HAPPINESS_MODIFIERS; i++) {
+    GdkPixbuf *pb;
+
     /* set spacing between lines of citizens*/
 
     /* happiness labels */
@@ -241,17 +244,21 @@ static struct happiness_dialog *create_happiness_dialog(struct city *pcity,
 
     /* list of citizens */
     ebox = gtk_event_box_new();
-    gtk_widget_set_margin_left(ebox, 5);
+    gtk_widget_set_margin_start(ebox, 5);
     gtk_event_box_set_visible_window(GTK_EVENT_BOX(ebox), FALSE);
     g_object_set_data(G_OBJECT(ebox), "pdialog", pdialog);
     g_signal_connect(ebox, "button_press_event",
                      G_CALLBACK(show_happiness_popup), GUINT_TO_POINTER(i));
     pdialog->happiness_ebox[i] = ebox;
 
-    pdialog->hpixmaps[i] = gtk_pixcomm_new(PIXCOMM_WIDTH, PIXCOMM_HEIGHT);
-    gtk_container_add(GTK_CONTAINER(ebox), pdialog->hpixmaps[i]);
-    gtk_widget_set_halign(pdialog->hpixmaps[i], GTK_ALIGN_START);
-    gtk_widget_set_valign(pdialog->hpixmaps[i], GTK_ALIGN_START);
+    pdialog->feeling_surfaces[i] = cairo_image_surface_create(CAIRO_FORMAT_ARGB32,
+                                                              FEELING_WIDTH, FEELING_HEIGHT);
+    pb = surface_get_pixbuf(pdialog->feeling_surfaces[i], FEELING_WIDTH, FEELING_HEIGHT);
+    pdialog->feeling_images[i] = gtk_image_new_from_pixbuf(pb);
+    g_object_unref(pb);
+    gtk_container_add(GTK_CONTAINER(ebox), pdialog->feeling_images[i]);
+    gtk_widget_set_halign(pdialog->feeling_images[i], GTK_ALIGN_START);
+    gtk_widget_set_valign(pdialog->feeling_images[i], GTK_ALIGN_START);
 
     gtk_grid_attach(GTK_GRID(table), ebox, 1, i, 1, 1);
   }
@@ -275,22 +282,28 @@ static struct happiness_dialog *create_happiness_dialog(struct city *pcity,
 }
 
 /**************************************************************************
-  Refresh citizens pixcomm
+  Refresh citizens surface
 **************************************************************************/
-static void refresh_pixcomm(GtkPixcomm *dst, struct city *pcity,
-			    enum citizen_feeling index)
+static void refresh_feeling_surface(cairo_surface_t *dst, struct city *pcity,
+                                    enum citizen_feeling index)
 {
-  enum citizen_category citizens[MAX_CITY_SIZE];
+  enum citizen_category categories[MAX_CITY_SIZE];
   int i;
-  int num_citizens = get_city_citizen_types(pcity, index, citizens);
-  int offset = MIN(tileset_small_sprite_width(tileset), PIXCOMM_WIDTH / num_citizens);
+  int num_citizens = get_city_citizen_types(pcity, index, categories);
+  int offset = MIN(tileset_small_sprite_width(tileset), FEELING_WIDTH / num_citizens);
+  cairo_t *cr;
 
-  gtk_pixcomm_clear(dst);
+  cr = cairo_create(dst);
 
   for (i = 0; i < num_citizens; i++) {
-    gtk_pixcomm_copyto(dst, get_citizen_sprite(tileset, citizens[i], i, pcity),
-		       i * offset, 0);
+    cairo_set_source_surface(cr,
+                             get_citizen_sprite(tileset, categories[i], i, pcity)->surface,
+                             i * offset, 0);
+    cairo_rectangle(cr, i * offset, 0, offset, FEELING_HEIGHT);
+    cairo_fill(cr);
   }
+
+  cairo_destroy(cr);
 }
 
 /**************************************************************************
@@ -302,7 +315,13 @@ void refresh_happiness_dialog(struct city *pcity)
   struct happiness_dialog *pdialog = get_happiness_dialog(pcity);
 
   for (i = 0; i < FEELING_LAST; i++) {
-    refresh_pixcomm(GTK_PIXCOMM(pdialog->hpixmaps[i]), pdialog->pcity, i);
+    GdkPixbuf *pb;
+
+    refresh_feeling_surface(pdialog->feeling_surfaces[i], pdialog->pcity, i);
+
+    pb = surface_get_pixbuf(pdialog->feeling_surfaces[i], FEELING_WIDTH, FEELING_HEIGHT);
+    gtk_image_set_from_pixbuf(GTK_IMAGE(pdialog->feeling_images[i]), pb);
+    g_object_unref(pb);
   }
 }
 
@@ -312,6 +331,8 @@ void refresh_happiness_dialog(struct city *pcity)
 void close_happiness_dialog(struct city *pcity)
 {
   struct happiness_dialog *pdialog = get_happiness_dialog(pcity);
+  int i;
+
   if (pdialog == NULL) {
     /* City which is being investigated doesn't contain happiness tab */
     return;
@@ -322,6 +343,11 @@ void close_happiness_dialog(struct city *pcity)
   dialog_list_remove(dialog_list, pdialog);
 
   gtk_widget_destroy(pdialog->shell);
+
+  for (i = 0; i < NUM_HAPPINESS_MODIFIERS; i++) {
+    cairo_surface_destroy(pdialog->feeling_surfaces[i]);
+  }
+
   free(pdialog);
 }
 
