@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
 #include "log.h"
 #include "mem.h"
 
-/* client/gui-gtk-3.0 */
+/* client/gui-gtk-3.22 */
 #include "gui_main.h"
 #include "gui_stuff.h"
 
@@ -65,13 +65,13 @@ GtkWidget *input_dialog_create(GtkWindow *parent, const char *dialogname,
 
   cb->response_callback = response_callback;
   cb->response_cli_data = response_cli_data;
-  
+
   shell = gtk_dialog_new_with_buttons(dialogname,
-        parent,
-        GTK_DIALOG_DESTROY_WITH_PARENT,
-        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-        GTK_STOCK_OK, GTK_RESPONSE_OK,
-        NULL);
+                                      parent,
+                                      GTK_DIALOG_DESTROY_WITH_PARENT,
+                                      _("Cancel"), GTK_RESPONSE_CANCEL,
+                                      _("OK"), GTK_RESPONSE_OK,
+                                      NULL);
   gtk_dialog_set_default_response(GTK_DIALOG(shell), GTK_RESPONSE_OK);
   setup_dialog(shell, GTK_WIDGET(parent));
   g_signal_connect(shell, "response", G_CALLBACK(input_dialog_response), cb);
