@@ -29,7 +29,7 @@
 /* client */
 #include "options.h"
 
-/* client/gui-gtk-3.0 */
+/* client/gui-gtk-3.22 */
 #include "chatline.h"
 #include "gui_main.h"
 #include "gui_stuff.h"
@@ -186,7 +186,7 @@ static void luaconsole_dialog_create(struct luaconsole_data *pdialog)
 
   fc_assert_ret(NULL != pdialog);
 
-  if (gui_gtk3_message_chat_location == GUI_GTK_MSGCHAT_SPLIT) {
+  if (GUI_GTK_OPTION(message_chat_location) == GUI_GTK_MSGCHAT_SPLIT) {
     notebook = right_notebook;
   } else {
     notebook = bottom_notebook;
@@ -239,10 +239,11 @@ static void luaconsole_dialog_create(struct luaconsole_data *pdialog)
                    &pdialog->entry);
 
   /* Load lua script command button. */
-  gui_dialog_add_button(pdialog->shell, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
+  gui_dialog_add_button(pdialog->shell, "window-close", _("Close"),
+                        GTK_RESPONSE_CLOSE);
 
-  gui_dialog_add_stockbutton(pdialog->shell, GTK_STOCK_OPEN,
-                             _("Load Lua Script"), LUACONSOLE_RES_OPEN);
+  gui_dialog_add_button(pdialog->shell, "document-open",
+                        _("Load Lua Script"), LUACONSOLE_RES_OPEN);
   gui_dialog_response_set_callback(pdialog->shell,
                                    luaconsole_response_callback);
 
@@ -268,11 +269,11 @@ static void luaconsole_input_return(GtkEntry *w, gpointer data)
     script_client_do_string(theinput);
 
     if (genlist_size(pdialog->history_list) >= MAX_LUACONSOLE_HISTORY) {
-      void *data;
+      void *history_data;
 
-      data = genlist_get(pdialog->history_list, -1);
-      genlist_remove(pdialog->history_list, data);
-      free(data);
+      history_data = genlist_get(pdialog->history_list, -1);
+      genlist_remove(pdialog->history_list, history_data);
+      free(history_data);
     }
 
     genlist_prepend(pdialog->history_list, fc_strdup(theinput));
@@ -312,10 +313,10 @@ static void luaconsole_load_file_popup(void)
 
   /* Create the selector */
   filesel = gtk_file_chooser_dialog_new("Load Lua file", GTK_WINDOW(toplevel),
-                GTK_FILE_CHOOSER_ACTION_OPEN,
-                GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                GTK_STOCK_OPEN, GTK_RESPONSE_OK,
-                NULL);
+                                        GTK_FILE_CHOOSER_ACTION_OPEN,
+                                        _("Cancel"), GTK_RESPONSE_CANCEL,
+                                        _("Open"), GTK_RESPONSE_OK,
+                                        NULL);
   setup_dialog(filesel, toplevel);
   gtk_window_set_position(GTK_WINDOW(filesel), GTK_WIN_POS_MOUSE);
 
@@ -471,7 +472,7 @@ void real_luaconsole_append(const char *astring,
   gtk_text_buffer_insert(buf, &iter, "\n", -1);
   mark = gtk_text_buffer_create_mark(buf, NULL, &iter, TRUE);
 
-  if (gui_gtk3_show_chat_message_time) {
+  if (GUI_GTK_OPTION(show_chat_message_time)) {
     char timebuf[64];
     time_t now;
     struct tm *now_tm;
