@@ -48,8 +48,9 @@
 #include "overview_common.h"
 #include "tilespec.h"
 #include "text.h"
+#include "zoom.h"
 
-/* client/gui-gtk-3.0 */
+/* client/gui-gtk-3.22 */
 #include "citydlg.h" /* For reset_city_dialogs() */
 #include "editgui.h"
 #include "graphics.h"
@@ -80,15 +81,29 @@ void update_turn_done_button(bool do_restore)
   }
 
   if ((do_restore && flip) || !do_restore) {
-    GdkRGBA fore;
-    GdkRGBA back;
-    GtkStyleContext *context = gtk_widget_get_style_context(turn_done_button);
+    static GtkCssProvider *tdb_provider = NULL;
+    GtkStyleContext *scontext = gtk_widget_get_style_context(turn_done_button);
 
-    gtk_style_context_get_color(context, GTK_STATE_FLAG_NORMAL, &fore);
-    gtk_style_context_get_background_color(context, GTK_STATE_FLAG_NORMAL, &back);
+    if (tdb_provider == NULL) {
+      tdb_provider = gtk_css_provider_new();
 
-    gtk_widget_override_color(turn_done_button, GTK_STATE_FLAG_NORMAL, &back);
-    gtk_widget_override_background_color(turn_done_button, GTK_STATE_FLAG_NORMAL, &fore);
+      gtk_css_provider_load_from_data(tdb_provider,
+                                      ".lighted {\n"
+                                      "color: rgba(235, 127, 235, 255);\n"
+                                      "background-color: rgba(127, 127, 127, 255);\n"
+                                      "}\n",
+                                      -1, NULL);
+
+      gtk_style_context_add_provider(scontext,
+                                     GTK_STYLE_PROVIDER(tdb_provider),
+                                     GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+    }
+
+    if (flip) {
+      gtk_style_context_add_class(scontext, "lighted");
+    } else {
+      gtk_style_context_remove_class(scontext, "lighted");
+    }
 
     flip = !flip;
   }
@@ -101,10 +116,16 @@ void update_timeout_label(void)
 {
   gtk_label_set_text(GTK_LABEL(timeout_label), get_timeout_label_text());
 
-  if (client_current_turn_timeout() > 0) {
-    gtk_widget_set_tooltip_text(timeout_label, _("Time to forced turn change"));
+  if (current_turn_timeout() > 0) {
+    gtk_widget_set_tooltip_text(timeout_label,
+                                _("Time to forced turn change,\n"
+                                  "or estimated time to finish turn change "
+                                  "processing."));
   } else {
-    gtk_widget_set_tooltip_text(timeout_label, _("Turn timeout disabled"));
+    gtk_widget_set_tooltip_text(timeout_label,
+                                _("Turn timeout disabled.\n"
+                                  "Between turns this shows estimated time "
+                                  "to finish turn change processing."));
   }
 }
 
@@ -145,42 +166,51 @@ void update_info_label(void)
   }
 
   gtk_label_set_text(GTK_LABEL(main_label_info),
-                     get_info_label_text(!gui_gtk3_small_display_layout));
+                     get_info_label_text(!GUI_GTK_OPTION(small_display_layout)));
 
   set_indicator_icons(client_research_sprite(),
-		      client_warming_sprite(),
-		      client_cooling_sprite(),
-		      client_government_sprite());
+                      client_warming_sprite(),
+                      client_cooling_sprite(),
+                      client_government_sprite());
 
   if (NULL != client.conn.playing) {
     int d = 0;
 
     for (; d < client.conn.playing->economic.luxury /10; d++) {
-      struct sprite *sprite = get_tax_sprite(tileset, O_LUXURY);
+      struct sprite *spr = get_tax_sprite(tileset, O_LUXURY);
+      GdkPixbuf *pb;
 
-      gtk_pixcomm_set_from_sprite(GTK_PIXCOMM(econ_label[d]), sprite);
+      pb = sprite_get_pixbuf(spr);
+      gtk_image_set_from_pixbuf(GTK_IMAGE(econ_label[d]), pb);
+      g_object_unref(pb);
     }
- 
+
     for (; d < (client.conn.playing->economic.science
 		+ client.conn.playing->economic.luxury) / 10; d++) {
-      struct sprite *sprite = get_tax_sprite(tileset, O_SCIENCE);
+      struct sprite *spr = get_tax_sprite(tileset, O_SCIENCE);
+      GdkPixbuf *pb;
 
-      gtk_pixcomm_set_from_sprite(GTK_PIXCOMM(econ_label[d]), sprite);
+      pb = sprite_get_pixbuf(spr);
+      gtk_image_set_from_pixbuf(GTK_IMAGE(econ_label[d]), pb);
+      g_object_unref(pb);
     }
- 
-    for (; d < 10; d++) {
-      struct sprite *sprite = get_tax_sprite(tileset, O_GOLD);
 
-      gtk_pixcomm_set_from_sprite(GTK_PIXCOMM(econ_label[d]), sprite);
+    for (; d < 10; d++) {
+      struct sprite *spr = get_tax_sprite(tileset, O_GOLD);
+      GdkPixbuf *pb;
+
+      pb = sprite_get_pixbuf(spr);
+      gtk_image_set_from_pixbuf(GTK_IMAGE(econ_label[d]), pb);
+      g_object_unref(pb);
     }
   }
- 
+
   update_timeout_label();
 
   /* update tooltips. */
   gtk_widget_set_tooltip_text(econ_ebox,
-		       _("Shows your current luxury/science/tax rates; "
-			 "click to toggle them."));
+                              _("Shows your current luxury/science/tax rates; "
+                                "click to toggle them."));
 
   gtk_widget_set_tooltip_text(bulb_ebox, get_bulb_tooltip());
   gtk_widget_set_tooltip_text(sun_ebox, get_global_warming_tooltip());
@@ -249,7 +279,6 @@ void update_unit_info_label(struct unit_list *punits)
   update_unit_pix_label(punits);
 }
 
-
 /**************************************************************************
   Get sprite for treaty acceptance or rejection.
 **************************************************************************/
@@ -264,12 +293,22 @@ GdkPixbuf *get_thumb_pixbuf(int onoff)
   indicator.
 ****************************************************************************/
 void set_indicator_icons(struct sprite *bulb, struct sprite *sol,
-			 struct sprite *flake, struct sprite *gov)
+                         struct sprite *flake, struct sprite *gov)
 {
-  gtk_pixcomm_set_from_sprite(GTK_PIXCOMM(bulb_label), bulb);
-  gtk_pixcomm_set_from_sprite(GTK_PIXCOMM(sun_label), sol);
-  gtk_pixcomm_set_from_sprite(GTK_PIXCOMM(flake_label), flake);
-  gtk_pixcomm_set_from_sprite(GTK_PIXCOMM(government_label), gov);
+  GdkPixbuf *pb;
+
+  pb = sprite_get_pixbuf(bulb);
+  gtk_image_set_from_pixbuf(GTK_IMAGE(bulb_label), pb);
+  g_object_unref(pb);
+  pb = sprite_get_pixbuf(sol);
+  gtk_image_set_from_pixbuf(GTK_IMAGE(sun_label), pb);
+  g_object_unref(pb);
+  pb = sprite_get_pixbuf(flake);
+  gtk_image_set_from_pixbuf(GTK_IMAGE(flake_label), pb);
+  g_object_unref(pb);
+  pb = sprite_get_pixbuf(gov);
+  gtk_image_set_from_pixbuf(GTK_IMAGE(government_label), pb);
+  g_object_unref(pb);
 }
 
 /****************************************************************************
@@ -290,7 +329,8 @@ void get_overview_area_dimensions(int *width, int *height)
 void overview_size_changed(void)
 {
   gtk_widget_set_size_request(overview_canvas,
-			      overview.width, overview.height);
+                              gui_options.overview.width,
+                              gui_options.overview.height);
   update_map_canvas_scrollbars_size();
 }
 
@@ -319,11 +359,11 @@ struct canvas *get_overview_window(void)
 gboolean overview_canvas_draw(GtkWidget *w, cairo_t *cr, gpointer data)
 {
   gpointer source = (can_client_change_view()) ?
-                     (gpointer)overview.window : (gpointer)radar_gfx_sprite;
+                     (gpointer)gui_options.overview.window : (gpointer)radar_gfx_sprite;
 
   if (source) {
     cairo_surface_t *surface = (can_client_change_view()) ?
-                                overview.window->surface :
+                                gui_options.overview.window->surface :
                                 radar_gfx_sprite->surface;
 
     cairo_set_source_surface(cr, surface, 0, 0);
@@ -369,6 +409,7 @@ gboolean map_canvas_configure(GtkWidget *w, GdkEventConfigure *ev,
                               gpointer data)
 {
   map_canvas_resized(ev->width, ev->height);
+
   return TRUE;
 }
 
@@ -382,6 +423,7 @@ gboolean map_canvas_draw(GtkWidget *w, cairo_t *cr, gpointer data)
      * any pending updates, to make sure only the most up-to-date data
      * is written (otherwise drawing bugs happen when old data is copied
      * to screen).  Then we draw all changed areas to the screen. */
+    update_animation();
     unqueue_mapview_updates(FALSE);
     cairo_set_source_surface(cr, mapview.store->surface, 0, 0);
     cairo_paint(cr);
@@ -456,54 +498,76 @@ void update_city_descriptions(void)
 }
 
 /**************************************************************************
-  Fill pixcomm with unit gfx
+  Fill image with unit gfx
 **************************************************************************/
-void put_unit_gpixmap(struct unit *punit, GtkPixcomm *p)
+void put_unit_image(struct unit *punit, GtkImage *p, int height)
 {
-  struct canvas canvas_store = FC_STATIC_CANVAS_INIT;
+  struct canvas store = FC_STATIC_CANVAS_INIT;
+  int width;
 
-  canvas_store.surface = gtk_pixcomm_get_surface(p);
+  if (height <= 0) {
+    height = tileset_full_tile_height(tileset);
+  }
+  width = tileset_full_tile_width(tileset);
 
-  gtk_pixcomm_clear(p);
+  store.surface = cairo_image_surface_create(CAIRO_FORMAT_ARGB32,
+                                             width, height);
 
-  put_unit(punit, &canvas_store, 0, 0);
+  put_unit(punit, &store, 1.0, 0, 0);
+
+  gtk_image_set_from_surface(p, store.surface);
+  cairo_surface_destroy(store.surface);
 }
 
-
 /**************************************************************************
   FIXME:
   For now only two food, two gold one shield and two masks can be drawn per
   unit, the proper way to do this is probably something like what Civ II does.
   (One food/shield/mask drawn N times, possibly one top of itself. -- SKi 
 **************************************************************************/
-void put_unit_gpixmap_city_overlays(struct unit *punit, GtkPixcomm *p,
-                                    int *upkeep_cost, int happy_cost)
+void put_unit_image_city_overlays(struct unit *punit, GtkImage *p,
+                                  int height,
+                                  int *upkeep_cost, int happy_cost)
 {
   struct canvas store = FC_STATIC_CANVAS_INIT;
- 
-  store.surface = gtk_pixcomm_get_surface(p);
+  GdkPixbuf *pb;
+  int width = tileset_full_tile_width(tileset);
+
+  store.surface = cairo_image_surface_create(CAIRO_FORMAT_ARGB32,
+                                             width, height);
+
+  put_unit(punit, &store, 1.0, 0, 0);
 
   put_unit_city_overlays(punit, &store, 0, tileset_unit_layout_offset_y(tileset),
                          upkeep_cost, happy_cost);
+
+  pb = surface_get_pixbuf(store.surface, width, height);
+  gtk_image_set_from_pixbuf(p, pb);
+  g_object_unref(pb);
+  cairo_surface_destroy(store.surface);
 }
 
 /**************************************************************************
   Put overlay tile to pixmap
 **************************************************************************/
-void pixmap_put_overlay_tile(GdkWindow *pixmap,
-			     int canvas_x, int canvas_y,
-			     struct sprite *ssprite)
+void pixmap_put_overlay_tile(GdkWindow *pixmap, float zoom,
+                             int canvas_x, int canvas_y,
+                             struct sprite *ssprite)
 {
   cairo_t *cr;
+  GdkDrawingContext *ctx;
 
   if (!ssprite) {
     return;
   }
 
-  cr = gdk_cairo_create(pixmap);
+  ctx = gdk_window_begin_draw_frame(pixmap,
+                                    gdk_window_get_clip_region(pixmap));
+  cr = gdk_drawing_context_get_cairo_context(ctx);
+  cairo_scale(cr, zoom, zoom);
   cairo_set_source_surface(cr, ssprite->surface, canvas_x, canvas_y);
   cairo_paint(cr);
-  cairo_destroy(cr);
+  gdk_window_end_draw_frame(pixmap, ctx);
 }
 
 /**************************************************************************
@@ -582,11 +646,11 @@ void pixmap_put_overlay_tile_draw(struct canvas *pcanvas,
 **************************************************************************/
 void put_cross_overlay_tile(struct tile *ptile)
 {
-  int canvas_x, canvas_y;
+  float canvas_x, canvas_y;
 
   if (tile_to_canvas_pos(&canvas_x, &canvas_y, ptile)) {
-    pixmap_put_overlay_tile(gtk_widget_get_window(map_canvas),
-			    canvas_x, canvas_y,
+    pixmap_put_overlay_tile(gtk_widget_get_window(map_canvas), map_zoom,
+			    canvas_x / map_zoom, canvas_y / map_zoom,
 			    get_attention_crosshair_sprite(tileset));
   }
 }
@@ -635,7 +699,8 @@ void update_map_canvas_scrollbars(void)
 **************************************************************************/
 void update_map_canvas_scrollbars_size(void)
 {
-  int xmin, ymin, xmax, ymax, xsize, ysize, xstep, ystep;
+  float xmin, ymin, xmax, ymax;
+  int xsize, ysize, xstep, ystep;
 
   get_mapview_scroll_window(&xmin, &ymin, &xmax, &ymax, &xsize, &ysize);
   get_mapview_scroll_step(&xstep, &ystep);
@@ -690,6 +755,8 @@ void draw_selection_rectangle(int canvas_x, int canvas_y, int w, int h)
   double dashes[2] = {4.0, 4.0};
   struct color *pcolor;
   cairo_t *cr;
+  GdkDrawingContext *ctx;
+  GdkWindow *wndw;
 
   if (w == 0 || h == 0) {
     return;
@@ -700,22 +767,17 @@ void draw_selection_rectangle(int canvas_x, int canvas_y, int w, int h)
     return;
   }
 
-  cr = gdk_cairo_create(gtk_widget_get_window(map_canvas));
+  wndw = gtk_widget_get_window(map_canvas);
+  ctx = gdk_window_begin_draw_frame(wndw,
+                                    gdk_window_get_clip_region(wndw));
+  cr = gdk_drawing_context_get_cairo_context(ctx);
   gdk_cairo_set_source_rgba(cr, &pcolor->color);
   cairo_set_line_width(cr, 2.0);
   cairo_set_dash(cr, dashes, 2, 0);
-#ifdef WIN32_NATIVE
-  if (cairo_version() < CAIRO_VERSION_ENCODE(1, 12, 0)) {
-    /* Cairo has crashing CAIRO_OPERATOR_DIFFERENCE on win32 surface */
-    cairo_set_operator(cr, CAIRO_OPERATOR_XOR);
-  } else
-#endif /* WIN32_NATIVE */
-  {
-    cairo_set_operator(cr, CAIRO_OPERATOR_DIFFERENCE);
-  }
+  cairo_set_operator(cr, CAIRO_OPERATOR_DIFFERENCE);
   cairo_rectangle(cr, canvas_x, canvas_y, w, h);
   cairo_stroke(cr);
-  cairo_destroy(cr);
+  gdk_window_end_draw_frame(wndw, ctx);
 }
 
 /**************************************************************************
