@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996-2005 - Freeciv Development Team
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,11 +20,14 @@
 #include "mem.h"
 #include "shared.h"
 
-/* client/gtk-3.0 */
+/* client/gui-gtk-3.22 */
 #include "colors.h"
+#include "mapview.h"
 
 #include "sprite.h"
 
+#define MAX_FILE_EXTENSIONS 50
+
 /****************************************************************************
   Create a new sprite by cropping and taking only the given portion of
   the image.
@@ -41,12 +44,16 @@
   origin of the source image.  The pixel at (mask_offset_x,mask_offset_y)
   in the mask image will be used to clip pixel (0,0) in the source image
   which is pixel (-x,-y) in the new image.
+
+  scale gives scale of new tileset
+  smooth means if scaling might be bilinear, if set to false use nearest
+  neighbor
 ****************************************************************************/
 struct sprite *crop_sprite(struct sprite *source,
 			   int x, int y,
 			   int width, int height,
-			   struct sprite *mask,
-			   int mask_offset_x, int mask_offset_y)
+			   struct sprite *mask, int mask_offset_x, int mask_offset_y,
+               float scale, bool smooth)
 {
   struct sprite *new = fc_malloc(sizeof(*new));
   cairo_t *cr;
@@ -107,15 +114,52 @@ void get_sprite_dimensions(struct sprite *sprite, int *width, int *height)
 ****************************************************************************/
 const char **gfx_fileextensions(void)
 {
-  static const char *ext[] =
+  /* Includes space for hardcoded 'png' and termination NULL */
+  static const char *ext[MAX_FILE_EXTENSIONS + 2] =
   {
-    "png",
     NULL
   };
 
+  if (ext[0] == NULL) {
+    int count = 0;
+    GSList *formats = gdk_pixbuf_get_formats();
+    GSList *next = formats;
+
+    while ((next = g_slist_next(next)) != NULL && count < MAX_FILE_EXTENSIONS) {
+      GdkPixbufFormat *format = g_slist_nth_data(next, 0);
+      gchar **mimes = gdk_pixbuf_format_get_mime_types(format);
+      int i;
+
+      /* Consider .png to be supported even when there's no mime-type called "png" */
+      ext[count++] = fc_strdup("png");
+
+      for (i = 0; mimes[i] != NULL && count < MAX_FILE_EXTENSIONS; i++) {
+        char *end = strstr(mimes[i], "/");
+
+        if (end != NULL) {
+          ext[count++] = fc_strdup(end + 1);
+        }
+      }
+
+      g_strfreev(mimes);
+    }
+
+    g_slist_free(formats);
+
+    ext[count] = NULL;
+  }
+
   return ext;
 }
 
+/****************************************************************************
+  Called when the cairo surface with freeciv allocated data is destroyed.
+****************************************************************************/
+static void surf_destroy_callback(void *data)
+{
+  free(data);
+}
+
 /****************************************************************************
   Load the given graphics file into a sprite.  This function loads an
   entire image file, which may later be broken up into individual sprites
@@ -123,59 +167,101 @@ const char **gfx_fileextensions(void)
 ****************************************************************************/
 struct sprite *load_gfxfile(const char *filename)
 {
-  struct sprite *new = fc_malloc(sizeof(*new));
+  struct sprite *spr;
+  GError *err = NULL;;
+  GdkPixbuf *pb = gdk_pixbuf_new_from_file(filename, &err);
+  int width;
+  int height;
+  unsigned char *pbdata;
+  int rs;
+  unsigned char *cairo_data;
+  unsigned char *data;
+  int i, j;
+  int cairo_stride;
+  bool has_alpha;
+  int channels;
 
-  new->surface = cairo_image_surface_create_from_png(filename);
+  if (pb == NULL) {
+    log_error(_("Can't load %s: %s"), filename, err->message);
+    return NULL;
+  }
 
-  if (cairo_image_surface_get_format(new->surface) == CAIRO_FORMAT_RGB24) {
-    /* Make format right */
-    cairo_surface_t *tmp;
-    int width = cairo_image_surface_get_width(new->surface);
-    int height = cairo_image_surface_get_height(new->surface);
-    unsigned char *old_data;
-    unsigned char *new_data;
-    int i, j;
+  spr = fc_malloc(sizeof(*spr));
+  width = gdk_pixbuf_get_width(pb);
+  height = gdk_pixbuf_get_height(pb);
+  pbdata = gdk_pixbuf_get_pixels(pb);
+  rs = gdk_pixbuf_get_rowstride(pb);
+  has_alpha = gdk_pixbuf_get_has_alpha(pb);
+  channels = gdk_pixbuf_get_n_channels(pb);
 
-    /* Surface with correct format */
-    tmp = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, width, height);
+  cairo_stride = cairo_format_stride_for_width(CAIRO_FORMAT_ARGB32, width);
+  if (cairo_stride <= 0) {
+    log_error("Cairo does not give stride for width %d", width);
+    free(spr);
+    return NULL;
+  }
 
-    old_data = cairo_image_surface_get_data(new->surface);
-    new_data = cairo_image_surface_get_data(tmp);
+  cairo_data = fc_malloc(height * cairo_stride * 4);
+  data = cairo_data;
 
-    for (i = 0; i < width; i++) {
-      for (j = 0; j < height; j++) {
-#ifndef WORDS_BIGENDIAN
-        /* Add alpha channel */
-        new_data[(j * width + i) * 4 + 3] = 0xff;
-        /* Copy RGB */
-        new_data[(j * width + i) * 4 + 1] = old_data[(j * width + i) * 4 + 1];
-        new_data[(j * width + i) * 4 + 2] = old_data[(j * width + i) * 4 + 2];
-        new_data[(j * width + i) * 4 + 0] = old_data[(j * width + i) * 4 + 0];
+  for (i = 0; i < height; i++) {
+    for (j = 0; j < width; j++) {
+      if (has_alpha) {
+        unsigned char tmp;
+
+#define MULTI_UNc(a,b) ((a * b - (b / 2)) / 0xFF)
+
+#ifdef WORDS_BIGENDIAN
+        tmp = pbdata[j * channels + 3];
+        data[j * 4 + 3] = MULTI_UNc(pbdata[j * channels + 2], tmp);
+        data[j * 4 + 2] = MULTI_UNc(pbdata[j * channels + 1], tmp);
+        data[j * 4 + 1] = MULTI_UNc(pbdata[j * channels + 0], tmp);
+        data[j * 4 + 0] = tmp;
 #else  /* WORDS_BIGENDIAN */
-        /* Add alpha channel */
-        new_data[(j * width + i) * 4] = 0xff;
-        /* Copy RGB */
-        new_data[(j * width + i) * 4 + 1] = old_data[(j * width + i) * 4 + 1];
-        new_data[(j * width + i) * 4 + 2] = old_data[(j * width + i) * 4 + 2];
-        new_data[(j * width + i) * 4 + 3] = old_data[(j * width + i) * 4 + 3];
-#endif  /* WORDS_BIGENDIAN */
+        tmp = MULTI_UNc(pbdata[j * channels + 2], pbdata[j * channels + 3]);
+        data[j * 4 + 1] = MULTI_UNc(pbdata[j * channels + 1], pbdata[j * channels + 3]);
+        data[j * 4 + 2] = MULTI_UNc(pbdata[j * channels + 0], pbdata[j * channels + 3]);
+        data[j * 4 + 0] = tmp;
+        data[j * 4 + 3] = pbdata[j * channels + 3];
+#endif /* WORDS_BIGENDIAN */
+
+#undef MULTI_UNc
+
+      } else {
+        data[j * 4 + 3] = 255;
+        data[j * 4 + 0] = pbdata[j * channels + 2];
+        data[j * 4 + 1] = pbdata[j * channels + 1];
+        data[j * 4 + 2] = pbdata[j * channels + 0];
       }
     }
 
-    cairo_surface_mark_dirty(tmp);
-    cairo_surface_destroy(new->surface);
-
-    new->surface = tmp;
+    data += cairo_stride;
+    pbdata += rs;
   }
 
-  fc_assert(cairo_image_surface_get_format(new->surface) == CAIRO_FORMAT_ARGB32);
+  g_object_unref(pb);
 
-  if (cairo_surface_status(new->surface) != CAIRO_STATUS_SUCCESS) {
+  spr->surface = cairo_image_surface_create_for_data(cairo_data, CAIRO_FORMAT_ARGB32,
+                                                     width, height, cairo_stride);
+  if (spr->surface == NULL || cairo_surface_status(spr->surface) != CAIRO_STATUS_SUCCESS) {
+    log_error("Cairo image surface creation error");
+    free(spr);
+    free(cairo_data);
+
+    return NULL;
+  }
+
+  cairo_surface_set_user_data(spr->surface, NULL, cairo_data, surf_destroy_callback);
+
+  fc_assert(cairo_image_surface_get_format(spr->surface) == CAIRO_FORMAT_ARGB32);
+
+  if (cairo_surface_status(spr->surface) != CAIRO_STATUS_SUCCESS) {
     log_fatal("Failed reading graphics file: \"%s\"", filename);
+
     exit(EXIT_FAILURE);
   }
 
-  return new;
+  return spr;
 }
 
 /****************************************************************************
@@ -292,7 +378,8 @@ struct sprite *crop_blankspace(struct sprite *s)
 
   sprite_get_bounding_box(s, &x1, &y1, &x2, &y2);
 
-  return crop_sprite(s, x1, y1, x2 - x1 + 1, y2 - y1 + 1, NULL, -1, -1);
+  return crop_sprite(s, x1, y1, x2 - x1 + 1, y2 - y1 + 1, NULL, -1, -1,
+                     1.0, FALSE);
 }
 
 /********************************************************************
@@ -383,3 +470,41 @@ GdkPixbuf *surface_get_pixbuf(cairo_surface_t *surf, int width, int height)
 
   return pb;
 }
+
+/************************************************************************//**
+  Create a pixbuf containing a representative image for the given extra
+  type, to be used as an icon in the GUI.
+
+  May return NULL on error.
+
+  NB: You must call g_object_unref on the non-NULL return value when you
+  no longer need it.
+****************************************************************************/
+GdkPixbuf *create_extra_pixbuf(const struct extra_type *pextra)
+{
+  struct drawn_sprite sprs[80];
+  int count, w, h, canvas_x, canvas_y;
+  GdkPixbuf *pixbuf;
+  struct canvas canvas = FC_STATIC_CANVAS_INIT;
+  cairo_t *cr;
+
+  w = tileset_tile_width(tileset);
+  h = tileset_tile_height(tileset);
+
+  canvas.surface = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, w, h);
+  canvas_x = 0;
+  canvas_y = 0;
+
+  cr = cairo_create(canvas.surface);
+  cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
+  cairo_paint(cr);
+  cairo_destroy(cr);
+
+  count = fill_basic_extra_sprite_array(tileset, sprs, pextra);
+  put_drawn_sprites(&canvas, 1.0, canvas_x, canvas_y, count, sprs, FALSE);
+
+  pixbuf = surface_get_pixbuf(canvas.surface, w, h);
+  cairo_surface_destroy(canvas.surface);
+
+  return pixbuf;
+}
