@@ -17,14 +17,17 @@
 
 // Qt
 #include <QFontMetrics>
+#include <QPainter>
 
 // qt-client
 #include "canvas.h"
-#include "sprite.h"
 #include "colors.h"
+#include "fc_client.h"
+#include "fonts.h"
+#include "qtg_cxxside.h"
+#include "sprite.h"
 
-QFont *get_font(enum client_font font);
-
+static QFont *get_font(enum client_font font);
 /****************************************************************************
   Create a canvas of the given size.
 ****************************************************************************/
@@ -45,6 +48,22 @@ void qtg_canvas_free(struct canvas *store)
   delete store;
 }
 
+/****************************************************************************
+  Set canvas zoom for future drawing operations.
+****************************************************************************/
+void qtg_canvas_set_zoom(struct canvas *store, float zoom)
+{
+  /* Qt-client has no zoom support */
+}
+
+/****************************************************************************
+  This gui has zoom support.
+****************************************************************************/
+bool qtg_has_zoom_support()
+{
+  return FALSE;
+}
+
 /****************************************************************************
   Copies an area from the source canvas to the destination canvas.
 ****************************************************************************/
@@ -86,6 +105,25 @@ void pixmap_copy(QPixmap *dest, QPixmap *src, int src_x, int src_y,
   p.end();
 }
 
+/****************************************************************************
+  Copies an area from the source image to the destination image.
+****************************************************************************/
+void image_copy(QImage *dest, QImage *src, int src_x, int src_y,
+                 int dest_x, int dest_y, int width, int height)
+{
+  QRectF source_rect(src_x, src_y, width, height);
+  QRectF dest_rect(dest_x, dest_y, width, height);
+  QPainter p;
+
+  if (!width || !height) {
+    return;
+  }
+
+  p.begin(dest);
+  p.drawImage(dest_rect, *src, source_rect);
+  p.end();
+}
+
 /****************************************************************************
   Draw some or all of a sprite onto the canvas.
 ****************************************************************************/
@@ -175,20 +213,6 @@ void qtg_canvas_fill_sprite_area(struct canvas *pcanvas,
   qtg_canvas_put_rectangle(pcanvas, pcolor, canvas_x, canvas_y, width, height);
 }
 
-/****************************************************************************
-  Fill the area covered by the sprite with fog.
-****************************************************************************/
-void qtg_canvas_fog_sprite_area(struct canvas *pcanvas, struct sprite *psprite,
-                                int canvas_x, int canvas_y)
-{
-  QPainter p;
-
-  p.begin(&pcanvas->map_pixmap);
-  p.setCompositionMode(QPainter::CompositionMode_DestinationIn);
-  p.drawPixmap(canvas_x, canvas_y, *psprite->pm);
-  p.end();
-}
-
 /****************************************************************************
   Draw a 1-pixel-width colored line onto the canvas.
 ****************************************************************************/
@@ -271,39 +295,12 @@ void qtg_canvas_put_curved_line(struct canvas *pcanvas, struct color *pcolor,
   p.end();
 }
 
-/****************************************************************************
-  Returns given font
-****************************************************************************/
-QFont *get_font(client_font font)
-{
-  QFont *qf;
-  switch (font) {
-  case FONT_CITY_NAME:
-    qf = gui()->fc_fonts.get_font("gui_qt_font_city_names");
-    break;
-  case FONT_CITY_PROD:
-    qf = gui()->fc_fonts.get_font("gui_qt_font_city_productions");
-    break;
-  case FONT_REQTREE_TEXT:
-    qf = gui()->fc_fonts.get_font("gui_qt_font_reqtree_text");
-    break;
-  case FONT_COUNT:
-    qf = NULL;
-    break;
-  default:
-    qf = NULL;
-    break;
-  }
-  return qf;
-}
-
-
 /****************************************************************************
   Return the size of the given text in the given font.  This size should
   include the ascent and descent of the text.  Either of width or height
   may be NULL in which case those values simply shouldn't be filled out.
 ****************************************************************************/
-void qtg_get_text_size (int *width, int *height,
+void qtg_get_text_size(int *width, int *height,
                         enum client_font font, const char *text)
 {
   QFont *afont;
@@ -347,3 +344,84 @@ void qtg_canvas_put_text(struct canvas *pcanvas, int canvas_x, int canvas_y,
   p.end();
   delete fm;
 }
+
+/****************************************************************************
+  Returns given font
+****************************************************************************/
+QFont *get_font(client_font font)
+{
+  QFont *qf;
+  int ssize;
+  
+  switch (font) {
+  case FONT_CITY_NAME:
+  qf = fc_font::instance()->get_font(fonts::city_names);
+    if (gui()->map_scale != 1.0f) {
+     ssize = ceil(gui()->map_scale * fc_font::instance()->city_fontsize);
+      if (qf->pointSize() != ssize) {
+        qf->setPointSize(ssize);
+      }
+    }
+    break;
+  case FONT_CITY_PROD:
+    qf = fc_font::instance()->get_font(fonts::city_productions);
+    if (gui()->map_scale != 1.0f) {
+      ssize = ceil(gui()->map_scale * fc_font::instance()->prod_fontsize);
+      if (qf->pointSize() != ssize) {
+        qf->setPointSize(ssize);
+      }
+    }
+    break;
+  case FONT_REQTREE_TEXT:
+    qf = fc_font::instance()->get_font(fonts::reqtree_text);
+    break;
+  case FONT_COUNT:
+    qf = NULL;
+    break;
+  default:
+    qf = NULL;
+    break;
+  }
+  return qf;
+}
+
+/****************************************************************************
+  Return rectangle containing pure image (crops transparency)
+****************************************************************************/
+QRect zealous_crop_rect(QImage &p)
+{
+  int r, t, b, l;
+  int oh, ow;
+
+  ow = p.width();
+  l = ow;
+  r = 0;
+  oh = p.height();
+  t = oh;
+  b = 0;
+  for (int y = 0; y < oh; y++) {
+    QRgb row[ow];
+    bool row_filled = false;
+    int x;
+
+    /* Copy to a location with guaranteed QRgb suitable alignment.
+     * That fixes clang compiler warning. */
+    memcpy(row, p.scanLine(y), ow * sizeof(QRgb));
+
+    for (x = 0; x < ow; ++x) {
+      if (qAlpha(row[x])) {
+        row_filled = true;
+        r = qMax(r, x);
+        if (l > x) {
+          l = x;
+          x = r;
+        }
+      }
+    }
+    if (row_filled) {
+      t = qMin(t, y);
+      b = y;
+    }
+  }
+  return QRect(l, t, qMax(0, r - l + 1), qMax(0, b - t + 1));
+}
