@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,14 +18,18 @@
 // Qt
 #include <QApplication>
 #include <QHeaderView>
-#include <QMessageBox>
+#include <QVBoxLayout>
+
+// client
+#include "client_main.h"
 
 // gui-qt
-#include "qtg_cxxside.h"
 #include "cityrep.h"
+#include "fc_client.h"
+#include "hudwidget.h"
 
 static bool can_city_sell_universal(const struct city *pcity,
-                                    struct universal target);
+                                    const struct universal *target);
 
 /***************************************************************************
   Overriden compare for sorting items
@@ -168,6 +172,7 @@ city_model::city_model(QObject *parent): QAbstractListModel(parent)
 city_model::~city_model()
 {
   qDeleteAll(city_list);
+  city_list.clear();
 }
 
 /***************************************************************************
@@ -358,21 +363,22 @@ city_widget::city_widget(city_report *ctr): QTreeView()
   setItemsExpandable(false);
   setAutoScroll(true);
   setProperty("uniformRowHeights", "true");
+  setAlternatingRowColors(true);
   header()->setContextMenuPolicy(Qt::CustomContextMenu);
   header()->setMinimumSectionSize(10);
   setContextMenuPolicy(Qt::CustomContextMenu);
   hide_columns();
-  connect(header(), SIGNAL(customContextMenuRequested(const QPoint &)),
-          this, SLOT(display_header_menu(const QPoint &)));
+  connect(header(), &QWidget::customContextMenuRequested,
+          this, &city_widget::display_header_menu);
   connect(selectionModel(),
           SIGNAL(selectionChanged(const QItemSelection &,
                                   const QItemSelection &)),
           SLOT(cities_selected(const QItemSelection &,
                                const QItemSelection &)));
-  connect(this, SIGNAL(doubleClicked(QModelIndex)), this, 
-          SLOT(city_doubleclick(QModelIndex)));
-  connect(this, SIGNAL(customContextMenuRequested(const QPoint&)), 
-          this, SLOT(display_list_menu(const QPoint&)));
+  connect(this, &QAbstractItemView::doubleClicked, this, 
+          &city_widget::city_doubleclick);
+  connect(this, &QWidget::customContextMenuRequested, 
+          this, &city_widget::display_list_menu);
 }
 
 /***************************************************************************
@@ -396,7 +402,7 @@ void city_widget::city_view()
   pcity = selected_cities[0];
 
   Q_ASSERT(pcity != NULL);
-  if (center_when_popup_city) {
+  if (gui_options.center_when_popup_city) {
     center_tile_mapcanvas(pcity->tile);
   }
   qtg_real_city_dialog_popup(pcity);
@@ -456,13 +462,13 @@ void city_widget::display_list_menu(const QPoint &)
   QMenu *some_menu;
   QMenu *tmp2_menu;
   QMenu *tmp_menu;
-  QMessageBox ask(this);
+  hud_message_box ask(gui()->central_wdg);
   QVariant qvar, qvar2;
   bool sell_ask;
   bool need_clear = true;
   bool select_only = false;
   char buf[200];
-  cid cid;
+  cid id;
   const char *imprname;
   enum menu_labels m_state;
   int sell_count;
@@ -510,7 +516,7 @@ void city_widget::display_list_menu(const QPoint &)
 
     tmp_menu = some_menu->addMenu(_("Worklist"));
     tmp_menu->addAction(&wl_clear);
-    connect(&wl_clear, SIGNAL(triggered()), this, SLOT(clear_worlist()));
+    connect(&wl_clear, &QAction::triggered, this, &city_widget::clear_worlist);
     tmp2_menu = tmp_menu->addMenu(_("Add"));
     gen_worklist_labels(cma_labels);
     if (cma_labels.count() == 0) {
@@ -536,11 +542,11 @@ void city_widget::display_list_menu(const QPoint &)
   gen_select_labels(some_menu);
   if (select_only == false) {
     list_menu.addAction(&cty_view);
-    connect(&cty_view, SIGNAL(triggered()), this, SLOT(city_view()));
+    connect(&cty_view, &QAction::triggered, this, &city_widget::city_view);
     list_menu.addAction(&cty_buy);
-    connect(&cty_buy, SIGNAL(triggered()), this, SLOT(buy()));
+    connect(&cty_buy, &QAction::triggered, this, &city_widget::buy);
     list_menu.addAction(&cty_center);
-    connect(&cty_center, SIGNAL(triggered()), this, SLOT(center()));
+    connect(&cty_center, &QAction::triggered, this, &city_widget::center);
   }
   act = 0;
   sell_count = 0;
@@ -551,19 +557,19 @@ void city_widget::display_list_menu(const QPoint &)
     qvar2 = act->property("FC");
     m_state = static_cast<menu_labels>(qvar2.toInt());
     qvar = act->data();
-    cid = qvar.toInt();
-    target = cid_decode(cid);
+    id = qvar.toInt();
+    target = cid_decode(id);
 
-    city_list_iterate(client_player()->cities, pcity) {
-      if (NULL != pcity) {
+    city_list_iterate(client_player()->cities, iter_city) {
+      if (NULL != iter_city) {
         switch (m_state) {
         case SELECT_IMPR:
           if (need_clear) {
             clearSelection();
           }
           need_clear = false;
-          if (city_building_present(pcity, target)) {
-            select_city(pcity);
+          if (city_building_present(iter_city, &target)) {
+            select_city(iter_city);
           }
           break;
         case SELECT_WONDERS:
@@ -571,8 +577,8 @@ void city_widget::display_list_menu(const QPoint &)
             clearSelection();
           }
           need_clear = false;
-          if (city_building_present(pcity, target)) {
-            select_city(pcity);
+          if (city_building_present(iter_city, &target)) {
+            select_city(iter_city);
           }
           break;
         case SELECT_SUPP_UNITS:
@@ -580,8 +586,8 @@ void city_widget::display_list_menu(const QPoint &)
             clearSelection();
           }
           need_clear = false;
-          if (city_unit_supported(pcity, target)) {
-            select_city(pcity);
+          if (city_unit_supported(iter_city, &target)) {
+            select_city(iter_city);
           }
           break;
         case SELECT_PRES_UNITS:
@@ -589,8 +595,8 @@ void city_widget::display_list_menu(const QPoint &)
             clearSelection();
           }
           need_clear = false;
-          if (city_unit_present(pcity, target)) {
-            select_city(pcity);
+          if (city_unit_present(iter_city, &target)) {
+            select_city(iter_city);
           }
           break;
         case SELECT_AVAIL_UNITS:
@@ -598,8 +604,8 @@ void city_widget::display_list_menu(const QPoint &)
             clearSelection();
           }
           need_clear = false;
-          if (can_city_build_now(pcity, target)) {
-            select_city(pcity);
+          if (can_city_build_now(iter_city, &target)) {
+            select_city(iter_city);
           }
           break;
         case SELECT_AVAIL_IMPR:
@@ -607,8 +613,8 @@ void city_widget::display_list_menu(const QPoint &)
             clearSelection();
           }
           need_clear = false;
-          if (can_city_build_now(pcity, target)) {
-            select_city(pcity);
+          if (can_city_build_now(iter_city, &target)) {
+            select_city(iter_city);
           }
           break;
         case SELECT_AVAIL_WONDERS:
@@ -616,8 +622,8 @@ void city_widget::display_list_menu(const QPoint &)
             clearSelection();
           }
           need_clear = false;
-          if (can_city_build_now(pcity, target)) {
-            select_city(pcity);
+          if (can_city_build_now(iter_city, &target)) {
+            select_city(iter_city);
           }
           break;
         default:
@@ -627,20 +633,20 @@ void city_widget::display_list_menu(const QPoint &)
     } city_list_iterate_end;
 
     foreach (pcity, selected_cities) {
-      if (NULL != pcity) {
+      if (nullptr != pcity) {
         switch (m_state) {
         case CHANGE_PROD_NOW:
-          city_change_production(pcity, target);
+          city_change_production(pcity, &target);
           break;
         case CHANGE_PROD_NEXT:
-          city_queue_insert(pcity, 1, target);
+          city_queue_insert(pcity, 1, &target);
           break;
         case CHANGE_PROD_BEF_LAST:
           city_queue_insert(pcity, worklist_length(&pcity->worklist), 
-                            target);
+                            &target);
           break;
         case CHANGE_PROD_LAST:
-          city_queue_insert(pcity, -1, target);
+          city_queue_insert(pcity, -1, &target);
           break;
         case SELL:
           building = target.value.building;
@@ -650,11 +656,9 @@ void city_widget::display_list_menu(const QPoint &)
                         _("Are you sure you want to sell those %s?"),
                         imprname);
             sell_ask = false;
-            ask.setText(buf);
             ask.setStandardButtons(QMessageBox::Cancel | QMessageBox::Ok);
             ask.setDefaultButton(QMessageBox::Cancel);
-            ask.setIcon(QMessageBox::Question);
-            ask.setWindowTitle(" ");
+            ask.set_text_title(buf, _("Sell?"));
             sell_ret = ask.exec();
           }
           if (sell_ret == QMessageBox::Ok) {
@@ -667,11 +671,11 @@ void city_widget::display_list_menu(const QPoint &)
           break;
         case CMA:
           if (NULL != pcity) {
-            if (CMA_NONE == cid) {
+            if (CMA_NONE == id) {
               cma_release_city(pcity);
             } else {
               cma_put_city_under_agent(pcity,
-                                       cmafec_preset_get_parameter(cid));
+                                       cmafec_preset_get_parameter(id));
             }
           }
 
@@ -679,14 +683,14 @@ void city_widget::display_list_menu(const QPoint &)
         case WORKLIST_ADD:
           if (worklist_defined) {
             city_queue_insert_worklist(pcity, -1,
-              global_worklist_get(global_worklist_by_id(cid)));
+                           global_worklist_get(global_worklist_by_id(id)));
           }
           break;
 
         case WORKLIST_CHANGE:
           if (worklist_defined) {
             city_set_queue(pcity,
-                           global_worklist_get(global_worklist_by_id(cid)));
+                           global_worklist_get(global_worklist_by_id(id)));
           }
           break;
         default:
@@ -930,29 +934,29 @@ void city_widget::gen_select_labels(QMenu *menu)
   QMap<QString, cid> custom_labels;
 
   act = menu->addAction(_("All Cities"));
-  connect(act, SIGNAL(triggered()), this, SLOT(select_all()));
+  connect(act, &QAction::triggered, this, &city_widget::select_all);
   act = menu->addAction(_("No Cities"));
-  connect(act, SIGNAL(triggered()), this, SLOT(select_none()));
+  connect(act, &QAction::triggered, this, &city_widget::select_none);
   act = menu->addAction(_("Invert Selection"));
-  connect(act, SIGNAL(triggered()), this, SLOT(invert_selection()));
+  connect(act, &QAction::triggered, this, &city_widget::invert_selection);
   menu->addSeparator();
   act = menu->addAction(_("Coastal Cities"));
-  connect(act, SIGNAL(triggered()), this, SLOT(select_coastal()));
+  connect(act, &QAction::triggered, this, &city_widget::select_coastal);
   act = menu->addAction(_("Same Island"));
-  connect(act, SIGNAL(triggered()), this, SLOT(select_same_island()));
+  connect(act, &QAction::triggered, this, &city_widget::select_same_island);
   if (selected_cities.isEmpty()){
     act->setDisabled(true);
   }
   menu->addSeparator();
   act = menu->addAction(_("Building Units"));
   act->setData("unit");
-  connect(act, SIGNAL(triggered()), this, SLOT(select_building_something()));
+  connect(act, &QAction::triggered, this, &city_widget::select_building_something);
   act = menu->addAction(_("Building Improvements"));
   act->setData("impr");
-  connect(act, SIGNAL(triggered()), this, SLOT(select_building_something()));
+  connect(act, &QAction::triggered, this, &city_widget::select_building_something);
   act = menu->addAction(_("Building Wonders"));
   act->setData("wonder");
-  connect(act, SIGNAL(triggered()), this, SLOT(select_building_something()));
+  connect(act, &QAction::triggered, this, &city_widget::select_building_something);
   menu->addSeparator();
   tmp_menu = menu->addMenu(_("Improvements in City"));
   gen_production_labels(SELECT_IMPR, custom_labels, false, false,
@@ -1008,15 +1012,15 @@ void city_widget::gen_production_labels(city_widget::menu_labels what,
                                         TestCityFunc test_func,
                                         bool global)
 {
-
   struct universal targets[MAX_NUM_PRODUCTION_TARGETS];
   struct item items[MAX_NUM_PRODUCTION_TARGETS];
   int i, item, targets_used;
   QString str;
   char *row[4];
   char buf[4][64];
-  struct city **data;
+  struct city **city_data;
   int num_sel = 0;
+
   if (global) {
     num_sel = list_model->rowCount();
   } else {
@@ -1035,9 +1039,9 @@ void city_widget::gen_production_labels(city_widget::menu_labels what,
       array[i] = selected_cities.at(i);
     }
   }
-  data = &array[0];
+  city_data = &array[0];
   targets_used
-      = collect_production_targets(targets, data, num_sel, append_units,
+      = collect_production_targets(targets, city_data, num_sel, append_units,
                                    append_wonders, true, test_func);
   name_and_sort_items(targets, targets_used, items, true, NULL);
   for (i = 0; i < 4; i++) {
@@ -1047,8 +1051,9 @@ void city_widget::gen_production_labels(city_widget::menu_labels what,
   for (item = 0; item < targets_used; item++) {
     struct universal target = items[item].item;
     char txt[256];
+
     str.clear();
-    get_city_dialog_production_row(row, sizeof(buf[0]), target, NULL);
+    get_city_dialog_production_row(row, sizeof(buf[0]), &target, NULL);
     fc_snprintf(txt, ARRAY_SIZE(txt), "%s ", row[0]);
     str = str + QString(txt);
     list.insert(str, cid_encode(target));
@@ -1069,6 +1074,8 @@ void city_widget::update_city(city *pcity)
 ***************************************************************************/
 void city_widget::update_model()
 {
+  QFont f = QApplication::font();
+  QFontMetrics fm(f);
   QStringList sl;
   QString s, str;
   int width;
@@ -1083,8 +1090,6 @@ void city_widget::update_model()
       sl = str.split('\n');
       width = 0;
       foreach (s, sl) {
-        QFont f = QApplication::font();
-        QFontMetrics fm(f);
         width = qMax(width, fm.width(s));
       }
       header()->resizeSection(j, width + 10);
@@ -1211,7 +1216,7 @@ city_report::~city_report()
 void city_report::init()
 {
   gui()->gimme_place(this, "CTS");
-  index = gui()->add_game_tab(this, _("Cities"));
+  index = gui()->add_game_tab(this);
   gui()->game_tab_widget->setCurrentIndex(index);
 }
 /***************************************************************************
@@ -1234,10 +1239,10 @@ void city_report::update_city(struct city *pcity)
   Same as can_city_sell_building(), but with universal argument.
 *****************************************************************/
 static bool can_city_sell_universal(const struct city *pcity,
-                                    struct universal target)
+                                    const struct universal *target)
 {
-  return (target.kind == VUT_IMPROVEMENT
-          && can_city_sell_building(pcity, target.value.building));
+  return (target->kind == VUT_IMPROVEMENT
+          && can_city_sell_building(pcity, target->value.building));
 }
 
 
@@ -1258,8 +1263,13 @@ void city_report_dialog_popup(bool raise)
     i = gui()->gimme_index_of("CTS");
     fc_assert(i != -1);
     w = gui()->game_tab_widget->widget(i);
+    if (w->isVisible() == true) {
+      gui()->game_tab_widget->setCurrentIndex(0);
+      return;
+    }
     cr = reinterpret_cast<city_report*>(w);
     gui()->game_tab_widget->setCurrentWidget(cr);
+    cr->update_report();
   }
 }
 
@@ -1274,10 +1284,11 @@ void real_city_report_dialog_update(void)
 
   if (gui()->is_repo_dlg_open("CTS")) {
     i = gui()->gimme_index_of("CTS");
-    fc_assert(i != -1);
-    w = gui()->game_tab_widget->widget(i);
-    cr = reinterpret_cast<city_report *>(w);
-    cr->update_report();
+    if (gui()->game_tab_widget->currentIndex() == i) {
+      w = gui()->game_tab_widget->widget(i);
+      cr = reinterpret_cast<city_report *>(w);
+      cr->update_report();
+    }
   }
 }
 
@@ -1292,10 +1303,11 @@ void real_city_report_update_city(struct city *pcity)
 
   if (gui()->is_repo_dlg_open("CTS")) {
     i = gui()->gimme_index_of("CTS");
-    fc_assert(i != -1);
-    w = gui()->game_tab_widget->widget(i);
-    cr = reinterpret_cast<city_report *>(w);
-    cr->update_city(pcity);
+    if (gui()->game_tab_widget->currentIndex() == i) {
+      w = gui()->game_tab_widget->widget(i);
+      cr = reinterpret_cast<city_report *>(w);
+      cr->update_city(pcity);
+    }
   }
 }
 
