@@ -15,18 +15,38 @@
 #include <fc_config.h>
 #endif
 
-#ifdef HAVE_CONFIG_H
-#include <fc_config.h>
-#endif
+// Qt
+#include <QApplication>
+#include <QCloseEvent>
+#include <QGridLayout>
+#include <QHeaderView>
+#include <QLabel>
+#include <QMenu>
+#include <QSpinBox>
+#include <QPainter>
+#include <QPushButton>
+
+// common
+#include "government.h"
+#include "player.h"
+
+// client
+#include "client_main.h"
 
 // gui-qt
-#include "qtg_cxxside.h"
 #include "colors.h"
 #include "diplodlg.h"
+#include "fc_client.h"
+#include "sidebar.h"
+
+extern "C" {
+  int client_player_number(void);
+  struct player *player_by_number(const int player_id);
+}
 
 typedef advance *p_advance;
 typedef city *p_city;
-
+extern QApplication *qapp;
 /****************************************************************************
   Constructor for diplomacy widget
 ****************************************************************************/
@@ -35,7 +55,7 @@ diplo_wdg::diplo_wdg(int counterpart, int initiated_from): QWidget()
   color *colr;
   QString text;
   QString text2;
-  QString text_color;
+  QString text_tooltip;
   QLabel *plr1_label;
   QLabel *plr2_label;
   QLabel *label;
@@ -50,10 +70,11 @@ diplo_wdg::diplo_wdg(int counterpart, int initiated_from): QWidget()
   QPalette palette;
   struct sprite *sprite, *sprite2;
   char plr_buf[4 * MAX_LEN_NAME];
+  const struct player_diplstate *state;
   QHeaderView *header;
   struct color *textcolors[2] = {
     get_color(tileset, COLOR_MAPVIEW_CITYTEXT),
-    get_color(tileset, COLOR_MAPVIEW_UNKNOWN)
+    get_color(tileset, COLOR_MAPVIEW_CITYTEXT_DARK)
   };
   if (counterpart == initiated_from) {
     initiated_from = client_player_number();
@@ -66,16 +87,26 @@ diplo_wdg::diplo_wdg(int counterpart, int initiated_from): QWidget()
 
   init_treaty(&treaty, player_by_number(counterpart),
               player_by_number(initiated_from));
+  state = player_diplstate_get(player_by_number(player1),
+                               player_by_number(player2));
+  text_tooltip = QString(diplstate_type_translated_name(state->type));
+  if (state->turns_left > 0) {
+    text_tooltip = text_tooltip + " (";
+    text_tooltip = text_tooltip + QString(PL_("%1 turn left",
+                                              "%1 turns left",
+                                             state->turns_left))
+                                          .arg(state->turns_left);
+    text_tooltip = text_tooltip + ")";
+  }
   label3 = new QLabel;
   text = "<b><h3><center>"
          + QString(nation_plural_for_player(player_by_number(initiated_from)))
          + "</center></h3></b>";
   colr = get_player_color(tileset, player_by_number(initiated_from));
   text = "<style>h3{background-color: "
-         + colr->qcolor.name() + ";}</style>" + text;
-  palette.setColor(QPalette::WindowText, color_best_contrast(colr,
-                             textcolors, ARRAY_SIZE(textcolors))->qcolor);
-  label3->setPalette(palette);
+         + colr->qcolor.name() + ";" + "color: " + color_best_contrast(colr,
+             textcolors, ARRAY_SIZE(textcolors))->qcolor.name()
+         + "}</style>" + text;
   label3->setText(text);
   label3->setMinimumWidth(300);
   label4 = new QLabel;
@@ -84,13 +115,11 @@ diplo_wdg::diplo_wdg(int counterpart, int initiated_from): QWidget()
          + "</center></h3></b></body>";
   colr = get_player_color(tileset, player_by_number(counterpart));
   text = "<style>h3{background-color: "
-         + colr->qcolor.name() + ";}</style>" + text;
-  palette.setColor(QPalette::WindowText, color_best_contrast(colr,
-                             textcolors, ARRAY_SIZE(textcolors))->qcolor);
-  label4->setPalette(palette);
-
-  label4->setText(text);
+         + colr->qcolor.name() + ";" + "color: " + color_best_contrast(colr,
+             textcolors, ARRAY_SIZE(textcolors))->qcolor.name()
+         + "}</style>" + text;
   label4->setMinimumWidth(300);
+  label4->setText(text);
   layout->addWidget(label3, 0, 5);
   layout->addWidget(label4, 5, 5);
   plr1_label = new QLabel;
@@ -109,7 +138,6 @@ diplo_wdg::diplo_wdg(int counterpart, int initiated_from): QWidget()
   text = "<b><center>" + text + "</center></b>";
   label->setText(text);
   plr1_accept = new QLabel;
-  sprite = get_treaty_thumb_sprite(tileset, false);
   layout->addWidget(plr1_label, 1, 0);
   layout->addWidget(label, 1, 5);
   layout->addWidget(plr1_accept, 1, 10);
@@ -140,19 +168,22 @@ diplo_wdg::diplo_wdg(int counterpart, int initiated_from): QWidget()
   gold_edit2 = new QSpinBox;
   gold_edit1->setMinimum(0);
   gold_edit2->setMinimum(0);
+  gold_edit1->setFocusPolicy(Qt::ClickFocus);
+  gold_edit2->setFocusPolicy(Qt::ClickFocus);
   if (game.info.trading_gold) {
     gold_edit2->setMaximum(player_by_number(player1)->economic.gold);
     gold_edit1->setMaximum(player_by_number(player2)->economic.gold);
   }
   connect(gold_edit1, SIGNAL(valueChanged(int)), SLOT(gold_changed1(int)));
   connect(gold_edit2, SIGNAL(valueChanged(int)), SLOT(gold_changed2(int)));
-  pix = get_arrow_sprite(tileset, ARROW_PLUS)->pm;
   add_clause1 = new QPushButton(style()->standardIcon(QStyle::SP_ArrowRight),
                                 _("Add Clause..."));
   add_clause2 = new QPushButton(style()->standardIcon(QStyle::SP_ArrowRight),
                                 _("Add Clause..."));
-  connect(add_clause1, SIGNAL(clicked()), SLOT(show_menu_p2()));
-  connect(add_clause2, SIGNAL(clicked()), SLOT(show_menu_p1()));
+  add_clause1->setFocusPolicy(Qt::ClickFocus);
+  add_clause2->setFocusPolicy(Qt::ClickFocus);
+  connect(add_clause1, &QAbstractButton::clicked, this, &diplo_wdg::show_menu_p2);
+  connect(add_clause2, &QAbstractButton::clicked, this, &diplo_wdg::show_menu_p1);
   layout->addWidget(goldlab1, 7, 4);
   layout->addWidget(goldlab2, 3, 4);
   layout->addWidget(gold_edit1, 7, 5);
@@ -168,10 +199,11 @@ diplo_wdg::diplo_wdg(int counterpart, int initiated_from): QWidget()
   text_edit->verticalHeader()->setVisible(false);
   text_edit->horizontalHeader()->setVisible(false);
   text_edit->setSelectionMode(QAbstractItemView::SingleSelection);
+  text_edit->setFocusPolicy(Qt::NoFocus);
   header = text_edit->horizontalHeader();
   header->setStretchLastSection(true);
-  connect(text_edit, SIGNAL(itemDoubleClicked(QTableWidgetItem *)),
-          SLOT(dbl_click(QTableWidgetItem *)));
+  connect(text_edit, &QTableWidget::itemDoubleClicked,
+          this, &diplo_wdg::dbl_click);
   text_edit->clearContents();
   text_edit->setRowCount(0);
   layout->addWidget(text_edit, 9, 0, 8, 11);
@@ -181,12 +213,26 @@ diplo_wdg::diplo_wdg(int counterpart, int initiated_from): QWidget()
   cancel_treaty =
     new QPushButton(style()->standardIcon(QStyle::SP_DialogNoButton),
                     _("Cancel meeting"));
-  connect(accept_treaty, SIGNAL(clicked()), SLOT(response_accept()));
-  connect(cancel_treaty, SIGNAL(clicked()), SLOT(response_cancel()));
+  connect(accept_treaty, &QAbstractButton::clicked, this, &diplo_wdg::response_accept);
+  connect(cancel_treaty, &QAbstractButton::clicked, this, &diplo_wdg::response_cancel);
   layout->addWidget(accept_treaty, 17, 5);
   layout->addWidget(cancel_treaty, 17, 6);
+
+  if (client_player_number() != counterpart) {
+    label4->setToolTip(text_tooltip);
+    plr2_label->setToolTip(text_tooltip);
+  }
+  if (client_player_number() != initiated_from) {
+    label3->setToolTip(text_tooltip);
+    plr1_label->setToolTip(text_tooltip);
+  }
+
+  accept_treaty->setAutoDefault(true);
+  accept_treaty->setDefault(true);
+  cancel_treaty->setAutoDefault(true);
   setLayout(layout);
   update_wdg();
+
 }
 
 /****************************************************************************
@@ -281,28 +327,33 @@ void diplo_wdg::show_menu(int player)
   /* Maps */
   map_menu = menu.addMenu(_("Maps"));
   world_map = new QAction(_("World-map"), this);
-  connect(world_map, SIGNAL(triggered()), this, SLOT(world_map_clause()));
+  connect(world_map, &QAction::triggered, this, &diplo_wdg::world_map_clause);
   map_menu->addAction(world_map);
   sea_map = new QAction(_("Sea-map"), this);
-  connect(sea_map, SIGNAL(triggered()), this, SLOT(sea_map_clause()));
+  connect(sea_map, &QAction::triggered, this, &diplo_wdg::sea_map_clause);
   map_menu->addAction(sea_map);
 
   /* Trading: advances */
   if (game.info.trading_tech) {
+    const struct research *gresearch = research_get(pgiver);
+    const struct research *oresearch = research_get(pother);
     adv_menu = menu.addMenu(_("Advances"));
     advance_iterate(A_FIRST, padvance) {
       Tech_type_id i = advance_number(padvance);
-      if (player_invention_state(pgiver, i) == TECH_KNOWN
-          && player_invention_reachable(pother, i, FALSE)
-          && (player_invention_state(pother, i) == TECH_UNKNOWN
-              || player_invention_state(pother, i) == TECH_PREREQS_KNOWN)) {
+
+      if (research_invention_state(gresearch, i) == TECH_KNOWN
+          && research_invention_gettable(oresearch, i,
+                                         game.info.tech_trade_allow_holes)
+          && (research_invention_state(oresearch, i) == TECH_UNKNOWN
+              || research_invention_state(oresearch, i)
+                 == TECH_PREREQS_KNOWN)) {
         adv_list.insert(advance_name_translation(padvance), padvance);
       }
     } advance_iterate_end;
 
     /* All advances */
     all_advancs = new QAction(_("All advances"), this);
-    connect(all_advancs, SIGNAL(triggered()), this, SLOT(all_advances()));
+    connect(all_advancs, &QAction::triggered, this, &diplo_wdg::all_advances);
     adv_menu->addAction(all_advancs);
     adv_menu->addSeparator();
 
@@ -346,14 +397,14 @@ void diplo_wdg::show_menu(int player)
     }
   }
   some_action = new QAction(_("Give shared vision"), this);
-  connect(some_action, SIGNAL(triggered()), this,
-          SLOT(give_shared_vision()));
+  connect(some_action, &QAction::triggered, this,
+          &diplo_wdg::give_shared_vision);
   menu.addAction(some_action);
   if (gives_shared_vision(pgiver, pother)) {
     some_action->setDisabled(true);
   }
   some_action = new QAction(_("Give embassy"), this);
-  connect(some_action, SIGNAL(triggered()), this, SLOT(give_embassy()));
+  connect(some_action, &QAction::triggered, this, &diplo_wdg::give_embassy);
   menu.addAction(some_action);
   if (player_has_real_embassy(pother, pgiver)) {
     some_action->setDisabled(true);
@@ -364,19 +415,19 @@ void diplo_wdg::show_menu(int player)
     pacts_menu = menu.addMenu(_("Pacts"));
     ds = player_diplstate_get(pgiver, pother)->type;
     some_action = new QAction(Q_("?diplomatic_state:Cease-fire"), this);
-    connect(some_action, SIGNAL(triggered()), this, SLOT(pact_ceasfire()));
+    connect(some_action, &QAction::triggered, this, &diplo_wdg::pact_ceasfire);
     pacts_menu->addAction(some_action);
     if (ds == DS_CEASEFIRE || ds == DS_TEAM) {
       some_action->setDisabled(true);
     }
     some_action = new QAction(Q_("?diplomatic_state:Peace"), this);
-    connect(some_action, SIGNAL(triggered()), this, SLOT(pact_peace()));
+    connect(some_action, &QAction::triggered, this, &diplo_wdg::pact_peace);
     pacts_menu->addAction(some_action);
     if (ds == DS_PEACE || ds == DS_TEAM) {
       some_action->setDisabled(true);
     }
     some_action = new QAction(Q_("?diplomatic_state:Alliance"), this);
-    connect(some_action, SIGNAL(triggered()), this, SLOT(pact_allianze()));
+    connect(some_action, &QAction::triggered, this, &diplo_wdg::pact_allianze);
     pacts_menu->addAction(some_action);
     if (ds == DS_ALLIANCE || ds == DS_TEAM) {
       some_action->setDisabled(true);
@@ -525,6 +576,7 @@ void diplo_wdg::give_advance(int tech)
 void diplo_wdg::all_advances()
 {
   int giver, dest, other;
+  const struct research *dresearch, *gresearch;
 
   giver = curr_player;
   if (curr_player == player1) {
@@ -546,17 +598,22 @@ void diplo_wdg::all_advances()
   fc_assert_ret(NULL != pgiver);
   fc_assert_ret(NULL != pdest);
 
-  advance_iterate(A_FIRST, padvance) {
-    Tech_type_id i = advance_number(padvance);
+   dresearch = research_get(pdest);
+   gresearch = research_get(pgiver);
 
-    if (player_invention_state(pgiver, i) == TECH_KNOWN
-        && player_invention_reachable(pdest, i, FALSE)
-        && (player_invention_state(pdest, i) == TECH_UNKNOWN
-            || player_invention_state(pdest, i) == TECH_PREREQS_KNOWN)) {
-      dsend_packet_diplomacy_create_clause_req(&client.conn, other, giver,
-                                               CLAUSE_ADVANCE, i);
-    }
-  } advance_iterate_end;
+   advance_iterate(A_FIRST, padvance) {
+     Tech_type_id i = advance_number(padvance);
+
+     if (research_invention_state(gresearch, i) == TECH_KNOWN
+         && research_invention_gettable(dresearch, i,
+                                        game.info.tech_trade_allow_holes)
+         && (research_invention_state(dresearch, i) == TECH_UNKNOWN
+             || research_invention_state(dresearch, i)
+                == TECH_PREREQS_KNOWN)) {
+       dsend_packet_diplomacy_create_clause_req(&client.conn, other, giver,
+                                                CLAUSE_ADVANCE, i);
+     }
+   } advance_iterate_end;
 }
 
 /****************************************************************************
@@ -644,11 +701,24 @@ void diplo_wdg::update_wdg()
   }
 }
 
+/****************************************************************************
+  Restores original nations pixmap
+****************************************************************************/
+void diplo_wdg::restore_pixmap()
+{
+  gui()->sw_diplo->set_pixmap(fc_icons::instance()->get_pixmap("nations"));
+  gui()->sw_diplo->resize_pixmap(gui()->sw_diplo->width(),
+                                 gui()->sw_diplo->height());
+  gui()->sw_diplo->set_custom_labels(QString());
+  gui()->sw_diplo->update_final_pixmap();
+}
+
 /****************************************************************************
   Button 'Accept treaty' has been clicked
 ****************************************************************************/
 void diplo_wdg::response_accept()
 {
+  restore_pixmap();
   dsend_packet_diplomacy_accept_treaty_req(&client.conn,
                                            player_number(treaty.plr0));
 }
@@ -658,6 +728,7 @@ void diplo_wdg::response_accept()
 ****************************************************************************/
 void diplo_wdg::response_cancel()
 {
+  restore_pixmap();
   dsend_packet_diplomacy_cancel_meeting_req(&client.conn,
                                             player_number(treaty.plr0));
 }
@@ -668,6 +739,7 @@ void diplo_wdg::response_cancel()
 diplo_dlg::diplo_dlg(int counterpart, int initiated_from): QTabWidget()
 {
   add_widget(counterpart, initiated_from);
+  setFocusPolicy(Qt::ClickFocus);
 }
 
 /****************************************************************************
@@ -724,13 +796,8 @@ bool diplo_dlg::init(bool raise)
   }
   setAttribute(Qt::WA_DeleteOnClose);
   gui()->gimme_place(this, "DDI");
-  index = gui()->add_game_tab(this, _("Diplomacy"));
-
-  if (raise == false) {
-    gui()->game_tab_widget->change_color(index, Qt::red);
-  } else {
-    gui()->game_tab_widget->setCurrentIndex(index);
-  }
+  index = gui()->add_game_tab(this);
+  gui()->game_tab_widget->setCurrentIndex(index);
 
   return true;
 }
@@ -750,6 +817,7 @@ diplo_dlg::~diplo_dlg()
     dw->deleteLater();
   }
   gui()->remove_repo_dlg("DDI");
+  gui()->game_tab_widget->setCurrentIndex(0);
 }
 
 /****************************************************************************
@@ -825,14 +893,52 @@ void handle_diplomacy_init_meeting(int counterpart, int initiated_from)
 
   int i;
   diplo_dlg *dd;
+  QPainter p;
+  QPixmap *pix, *def_pix, *pix2, *pix3, *def_pix_del;
   QWidget *w;
   QWidget *fw;
 
-  if (client_is_observer()){
+  if (client_is_observer()) {
     return;
   }
+
+  if (gui()->current_page() != PAGE_GAME) {
+    gui()->switch_page(PAGE_GAME);
+  }
+
+  pix2 = new QPixmap();
+  def_pix_del = new QPixmap();
+  pix = get_nation_flag_sprite(tileset,
+                                  nation_of_player(player_by_number
+                                                   (counterpart)))->pm;
+  *pix2 = pix->scaledToWidth(gui()->sw_diplo->width() - 2,
+                             Qt::SmoothTransformation);
+  if (pix2->height() > gui()->sw_diplo->height()) {
+    *pix2 = pix->scaledToHeight(gui()->sw_diplo->height(),
+                             Qt::SmoothTransformation);
+  }
+  pix3 = new QPixmap(gui()->sw_diplo->width(), gui()->sw_diplo->height());
+  pix3->fill(Qt::transparent);
+  def_pix = fc_icons::instance()->get_pixmap("nations");
+  *def_pix_del = def_pix->scaled(gui()->sw_diplo->width(),
+                                 gui()->sw_diplo->height(),
+                                 Qt::IgnoreAspectRatio,
+                                 Qt::SmoothTransformation);
+  p.begin(pix3);
+  p.drawPixmap(1, 1, *pix2);
+  p.drawPixmap(0, 0, *def_pix_del);
+  p.end();
+  gui()->sw_diplo->set_pixmap(pix3);
+  gui()->sw_diplo->resize_pixmap(gui()->sw_diplo->width(),
+                                 gui()->sw_diplo->height());
+  gui()->sw_diplo->set_custom_labels(QString(nation_plural_for_player(
+                                            player_by_number(counterpart))));
+  gui()->sw_diplo->update_final_pixmap();
+  delete pix2;
+  delete def_pix_del;
+
   if (!gui()->is_repo_dlg_open("DDI")) {
-    diplo_dlg *dd = new diplo_dlg(counterpart, initiated_from);
+    dd = new diplo_dlg(counterpart, initiated_from);
 
     if (!dd->init(false)) {
       delete dd;
@@ -848,7 +954,7 @@ void handle_diplomacy_init_meeting(int counterpart, int initiated_from)
   fw = dd->find_widget(counterpart);
   if (fw == NULL) {
     dd->add_widget(counterpart, initiated_from);
-    gui()->game_tab_widget->change_color(i, Qt::red);
+    gui()->game_tab_widget->setCurrentIndex(i);
   }
   dd->make_active(counterpart);
 
@@ -938,6 +1044,7 @@ void close_all_diplomacy_dialogs(void)
   diplo_dlg *dd;
   QWidget *w;
 
+  qapp->alert(gui()->central_wdg);
   if (!gui()->is_repo_dlg_open("DDI")) {
     return;
   }
@@ -946,4 +1053,5 @@ void close_all_diplomacy_dialogs(void)
   w = gui()->game_tab_widget->widget(i);
   dd = qobject_cast<diplo_dlg *>(w);
   dd->close();
+  delete dd;
 }
