@@ -17,37 +17,55 @@
 
 // Qt
 #include <QApplication>
-#include <QCompleter>
-#include <QMainWindow>
+#include <QComboBox>
+#include <QFormLayout>
 #include <QLineEdit>
+#include <QMainWindow>
+#include <QPainter>
+#include <QResizeEvent>
 #include <QScrollBar>
+#include <QSettings>
+#include <QSignalMapper>
+#include <QSocketNotifier>
+#include <QSpinBox>
+#include <QStackedLayout>
+#include <QStandardPaths>
 #include <QStatusBar>
+#include <QStyleFactory>
 #include <QTabBar>
+#include <QTextBlock>
+#include <QTextCodec>
 #include <QTextEdit>
 
-// client
-#include "connectdlg_common.h"
-
 // common
 #include "game.h"
 
+// client
+#include "connectdlg_common.h"
+
 // gui-qt
 #include "fc_client.h"
+#include "fonts.h"
 #include "gui_main.h"
 #include "optiondlg.h"
+#include "sidebar.h"
 #include "sprite.h"
 
-
-extern QApplication *qapp;
 fc_icons* fc_icons::m_instance = 0;
+fc_font* fc_font::m_instance = 0;
+extern "C" {
+  bool get_turn_done_button_state();
+  void real_science_report_dialog_update(void);
+}
+extern void write_shortcuts();
+
+QString current_theme;
+
 /****************************************************************************
   Constructor
 ****************************************************************************/
 fc_client::fc_client() : QMainWindow()
 {
-#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
-  QTextCodec::setCodecForCStrings(QTextCodec::codecForName("UTF-8"));
-#endif
   QTextCodec::setCodecForLocale(QTextCodec::codecForName("UTF-8"));
   /**
    * Somehow freeciv-client-common asks to switch to page when all widgets
@@ -57,7 +75,6 @@ fc_client::fc_client() : QMainWindow()
    * After adding new QObjects null them here.
    */
   main_wdg = NULL;
-  chat_completer = NULL;
   connect_lan = NULL;
   connect_metaserver = NULL;
   central_layout = NULL;
@@ -77,25 +94,21 @@ fc_client::fc_client() : QMainWindow()
   info_widget = NULL;
   saves_load = NULL;
   scenarios_load = NULL;
-  start_players = NULL;
   meta_scan_timer = NULL;
   lan_scan_timer = NULL;
   status_bar = NULL;
   status_bar_label = NULL;
   menu_bar = NULL;
   mapview_wdg = NULL;
+  sidebar_wdg = nullptr;
   msgwdg = NULL;
   infotab = NULL;
-  game_info_label = NULL;
-  end_turn_rect = NULL;
   central_wdg = NULL;
   game_tab_widget = NULL;
   start_players_tree = NULL;
   unit_sel = NULL;
   info_tile_wdg = NULL;
   opened_dialog = NULL;
-  current_unit_id = -1;
-  current_unit_target_id = -1;
   current_file = "";
   status_bar_queue.clear();
   quitting = false;
@@ -103,7 +116,12 @@ fc_client::fc_client() : QMainWindow()
   x_vote = NULL;
   gtd = NULL;
   update_info_timer = nullptr;
-  for (int i = 0; i <= PAGE_GGZ; i++) {
+  game_layout = nullptr;
+  unitinfo_wdg = nullptr;
+  battlelog_wdg = nullptr;
+  interface_locked = false;
+  map_scale = 1.0f;
+  for (int i = 0; i <= PAGE_GAME; i++) {
     pages_layout[i] = NULL;
     pages[i] = NULL;
   }
@@ -115,17 +133,19 @@ fc_client::fc_client() : QMainWindow()
 ****************************************************************************/
 void fc_client::init()
 {
+  fc_font::instance()->init_fonts();
+  read_settings();
+  QApplication::setFont(*fc_font::instance()->get_font(fonts::default_font));
   QString path;
   central_wdg = new QWidget;
-  central_layout = new QGridLayout;
-  chat_completer = new QCompleter;
-  central_layout->setContentsMargins(2, 2, 2, 2);
+  central_layout = new QStackedLayout;
 
   // General part not related to any single page
-  history_pos = -1;
-  fc_fonts.init_fonts();
   menu_bar = new mr_menu();
-  menu_bar->setup_menus();
+  corner_wid = new fc_corner(this);
+  if (gui_options.gui_qt_show_titlebar == false) {
+    menu_bar->setCornerWidget(corner_wid);
+  }
   setMenuBar(menu_bar);
   status_bar = statusBar();
   status_bar_label = new QLabel;
@@ -157,30 +177,47 @@ void fc_client::init()
   pages[PAGE_NETWORK]->setVisible(false);
 
   // PAGE_GAME
+  gui_options.gui_qt_allied_chat_only = true;
   path = QStandardPaths::writableLocation(QStandardPaths::ConfigLocation);
   if (path.isEmpty() == false) {
     QSettings::setPath(QSettings::NativeFormat, QSettings::UserScope, path);
   }
-  read_settings();
   pages[PAGE_GAME] = new QWidget(central_wdg);
   init_mapcanvas_and_overview();
   create_game_page();
-  pages[PAGE_GAME]->setVisible(false);
+  
+  pages[PAGE_GAME + 1] = new QWidget(central_wdg);
+  create_loading_page();
+
+  pages_layout[PAGE_GAME]->setContentsMargins(0, 0, 0, 0);
+
+  pages[PAGE_MAIN]->setLayout(pages_layout[PAGE_MAIN]);
+  pages[PAGE_NETWORK]->setLayout(pages_layout[PAGE_NETWORK]);
+  pages[PAGE_LOAD]->setLayout(pages_layout[PAGE_LOAD]);
+  pages[PAGE_SCENARIO]->setLayout(pages_layout[PAGE_SCENARIO]);
+  pages[PAGE_START]->setLayout(pages_layout[PAGE_START]);
+  pages[PAGE_GAME]->setLayout(pages_layout[PAGE_GAME]);
+  pages[PAGE_GAME + 1]->setLayout(pages_layout[PAGE_GAME + 1]);
+
+  central_layout->addWidget(pages[PAGE_MAIN]);
+  central_layout->addWidget(pages[PAGE_NETWORK]);
+  central_layout->addWidget(pages[PAGE_LOAD]);
+  central_layout->addWidget(pages[PAGE_SCENARIO]);
+  central_layout->addWidget(pages[PAGE_START]);
+  central_layout->addWidget(pages[PAGE_GAME]);
+  central_layout->addWidget(pages[PAGE_GAME + 1]);
 
-  // PAGE_GGZ
-  pages[PAGE_GGZ] = NULL;
-  central_layout->addLayout(pages_layout[PAGE_MAIN], 1, 1);
-  central_layout->addLayout(pages_layout[PAGE_NETWORK], 1, 1);
-  central_layout->addLayout(pages_layout[PAGE_LOAD], 1, 1);
-  central_layout->addLayout(pages_layout[PAGE_SCENARIO], 1, 1);
-  central_layout->addLayout(pages_layout[PAGE_START], 1, 1);
-  central_layout->addLayout(pages_layout[PAGE_GAME], 1, 1);
   central_wdg->setLayout(central_layout);
   setCentralWidget(central_wdg);
 
   connect(switch_page_mapper, SIGNAL(mapped( int)),
                 this, SLOT(switch_page(int)));
+  resize(pages[PAGE_MAIN]->minimumSizeHint());
   setVisible(true);
+
+  game_tab_widget->init();
+  chat_listener::listen();
+
 }
 
 /****************************************************************************
@@ -189,13 +226,16 @@ void fc_client::init()
 fc_client::~fc_client()
 {
   status_bar_queue.clear();
-  fc_fonts.release_fonts();
 }
 
+
 /****************************************************************************
-  Main part of gui-qt
+  Main part of gui-qt.
+  This is not called simply 'fc_client::main()', since SDL includes
+  ould sometimes cause 'main' to be considered an macro that expands to
+  'SDL_main'
 ****************************************************************************/
-void fc_client::main(QApplication *qapp)
+void fc_client::fc_main(QApplication *qapp)
 {
   qRegisterMetaType<QTextCursor>("QTextCursor");
   qRegisterMetaType<QTextBlock>("QTextBlock");
@@ -207,7 +247,7 @@ void fc_client::main(QApplication *qapp)
   set_client_state(C_S_DISCONNECTED);
 
   startTimer(TIMER_INTERVAL);
-  connect(qapp, SIGNAL(aboutToQuit()), this, SLOT(closing()));
+  connect(qapp, &QCoreApplication::aboutToQuit, this, &fc_client::closing);
   qapp->exec();
 
   free_mapcanvas_and_overview();
@@ -230,17 +270,26 @@ void fc_client::closing()
   quitting = true;
 }
 
+/****************************************************************************
+  Slot to send fake chat messages. Do not use in new code.
+****************************************************************************/
+void fc_client::send_fake_chat_message(const QString &message)
+{
+  send_chat_message(message);
+}
 
 /****************************************************************************
   Appends text to chat window
 ****************************************************************************/
-void fc_client::append_output_window(const QString &str)
+void fc_client::chat_message_received(const QString &message,
+                                      const struct text_tag_list *tags)
 {
   QTextCursor cursor;
+  QColor col = output_window->palette().color(QPalette::Text);
+  QString str = apply_tags(message, tags, col);
 
   if (output_window != NULL) {
     output_window->append(str);
-    chat_line->setCompleter(chat_completer);
     output_window->verticalScrollBar()->setSliderPosition(
                               output_window->verticalScrollBar()->maximum());
   }
@@ -268,6 +317,7 @@ void fc_client::switch_page(int new_pg)
 {
   char buf[256];
   enum client_pages new_page;
+  int i_page;
 
   new_page = static_cast<client_pages>(new_pg);
 
@@ -285,16 +335,11 @@ void fc_client::switch_page(int new_pg)
   if (status_bar != nullptr) {
     status_bar->setVisible(true);
   }
-
-  for (int i = 0; i < PAGE_GGZ + 1; i++) {
-    if (i == new_page) {
-      show_children(pages_layout[i], true);
-    } else {
-      show_children(pages_layout[i], false);
-    }
-  }
+  QApplication::alert(gui()->central_wdg);
+  central_layout->setCurrentWidget(pages[new_pg]);
   page = new_page;
-  switch (page) {
+  i_page = new_page;
+  switch (i_page) {
   case PAGE_MAIN:
     break;
   case PAGE_START:
@@ -305,15 +350,17 @@ void fc_client::switch_page(int new_pg)
     update_load_page();
     break;
   case PAGE_GAME:
+    if (gui_options.gui_qt_show_titlebar == false) {
+      setWindowFlags(Qt::Window | Qt::CustomizeWindowHint);
+    }
+    showMaximized();
+    /* For MS Windows, it might ingore first */
+    showMaximized();
     gui()->infotab->chtwdg->update_widgets();
     status_bar->setVisible(false);
-    gui()->infotab->chtwdg->update_font();
-    showMaximized();
-      /* For MS Windows, it might ingore first */
-    showMaximized();
-    if (fullscreen_mode){
+    if (gui_options.gui_qt_fullscreen){
       gui()->showFullScreen();
-      gui()->mapview_wdg->showFullScreen();
+      gui()->game_tab_widget->showFullScreen();
     }
     menuBar()->setVisible(true);
     mapview_wdg->setFocus();
@@ -322,6 +369,9 @@ void fc_client::switch_page(int new_pg)
     update_info_label();
     minimapview_wdg->reset();
     overview_size_changed();
+    update_sidebar_tooltips();
+    real_science_report_dialog_update();
+    show_new_turn_info();
     break;
   case PAGE_SCENARIO:
     update_scenarios_page();
@@ -336,7 +386,8 @@ void fc_client::switch_page(int new_pg)
     connect_password_edit->setDisabled(true);
     connect_confirm_password_edit->setDisabled(true);
     break;
-  case PAGE_GGZ:
+  case (PAGE_GAME + 1):
+    break;
   default:
     if (client.conn.used) {
       disconnect_from_server();
@@ -361,8 +412,8 @@ void fc_client::add_server_source(int sock)
 {
   server_notifier = new QSocketNotifier(sock, QSocketNotifier::Read);
 
-  connect(server_notifier, SIGNAL(activated(int)), this, 
-          SLOT(server_input(int)));
+  connect(server_notifier, &QSocketNotifier::activated, this, 
+          &fc_client::server_input);
 }
 
 /****************************************************************************
@@ -380,41 +431,6 @@ bool fc_client::event(QEvent *event)
   }
 }
 
-
-/****************************************************************************
-  Event filter
-****************************************************************************/
-bool fc_client::eventFilter(QObject *obj, QEvent *event)
-{
-  if (obj == chat_line) {
-    if (event->type() == QEvent::KeyPress) {
-      QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
-      if (keyEvent->key() == Qt::Key_Up) {
-        history_pos++;
-        history_pos = qMin(chat_history.count(), history_pos);
-        if (history_pos < chat_history.count()) {
-          chat_line->setText(chat_history.at(history_pos));
-        }
-        if (history_pos == chat_history.count()) {
-          chat_line->setText("");
-        }
-        return true;
-      } else if (keyEvent->key() == Qt::Key_Down) {
-        history_pos--;
-        history_pos = qMax(-1, history_pos);
-        if (history_pos < chat_history.count() && history_pos != -1) {
-          chat_line->setText(chat_history.at(history_pos));
-        }
-        if (history_pos == -1) {
-          chat_line->setText("");
-        }
-        return true;
-      }
-    }
-  }
-  return QObject::eventFilter(obj, event);
-}
-
 /****************************************************************************
   Closes main window
 ****************************************************************************/
@@ -428,7 +444,7 @@ void fc_client::closeEvent(QCloseEvent *event) {
 ****************************************************************************/
 void fc_client::remove_server_source()
 {
-  delete server_notifier;
+  server_notifier->deleteLater();
 }
 
 /****************************************************************************
@@ -436,24 +452,9 @@ void fc_client::remove_server_source()
 ****************************************************************************/
 void fc_client::server_input(int sock)
 {
-#if defined(Q_WS_WIN)
   server_notifier->setEnabled(false);
-#endif
   input_from_server(sock);
-#if defined(Q_WS_WIN)
   server_notifier->setEnabled(true);
-#endif
-}
-
-/****************************************************************************
-  Enter pressed on chatline
-****************************************************************************/
-void fc_client::chat()
-{
-  send_chat(chat_line->text().toUtf8().data());
-  chat_history.prepend(chat_line->text());
-  chat_line->clear();
-  history_pos = -1;
 }
 
 /****************************************************************************
@@ -474,7 +475,11 @@ void fc_client::timerEvent(QTimerEvent *event)
 ****************************************************************************/
 void fc_client::quit()
 {
-  qapp->quit();
+  QApplication *qapp = current_app();
+
+  if (qapp != nullptr) {
+    qapp->quit();
+  }
 }
 
 /****************************************************************************
@@ -521,29 +526,6 @@ void fc_client::slot_pregame_start()
   }
 }
 
-/****************************************************************************
-  Shows all widgets for given layout
-****************************************************************************/
-void fc_client::show_children(const QLayout* layout, bool show)
-{
-  QLayoutItem *item = NULL;
-  QWidget *widget = NULL;
-
-  if (layout) {
-    for (int i = 0; i < layout->count(); i++) {
-      item = layout->itemAt(i);
-      widget = item ? item->widget() : 0;
-
-      if (widget) {
-        widget->setVisible(show);
-      }
-
-      if (item->layout()) {
-        show_children(item->layout(), show);
-      }
-    }
-  }
-}
 
 /****************************************************************************
   Finds not used index on game_view_tab and returns it
@@ -588,6 +570,10 @@ int fc_client::gimme_index_of(QString str)
   int i;
   QWidget *w;
 
+  if (str == "MAP") {
+    return 0;
+  }
+
   w = opened_repo_dlgs.value(str);
   i = game_tab_widget->indexOf(w);
   return i;
@@ -598,21 +584,135 @@ int fc_client::gimme_index_of(QString str)
 ****************************************************************************/
 void fc_client::read_settings()
 {
-  QSettings s(QSettings::IniFormat, QSettings::UserScope, 
+  QSettings s(QSettings::IniFormat, QSettings::UserScope,
               "freeciv-qt-client");
-  if (s.contains("InfoTab-xsize")) {
-    qt_settings.infotab_width = s.value("InfoTab-xsize").toInt();
-  } else {
-    qt_settings.infotab_width = 95;
+  if (s.contains("Fonts-set") == false) {
+    configure_fonts();
   }
-  if (s.contains("InfoTab-ysize")) {
-    qt_settings.infotab_height = s.value("InfoTab-ysize").toInt();
+  if (s.contains("Chat-fx-size")) {
+    qt_settings.chat_fwidth = s.value("Chat-fx-size").toFloat();
   } else {
-    qt_settings.infotab_height = 29;
+    qt_settings.chat_fwidth = 0.2;
+  }
+  if (s.contains("Chat-fy-size")) {
+    qt_settings.chat_fheight = s.value("Chat-fy-size").toFloat();
+  } else {
+    qt_settings.chat_fheight = 0.4;
+  }
+  if (s.contains("Chat-fx-pos")) {
+    qt_settings.chat_fx_pos = s.value("Chat-fx-pos").toFloat();
+  } else {
+    qt_settings.chat_fx_pos = 0.0;
+  }
+  if (s.contains("Chat-fy-pos")) {
+    qt_settings.chat_fy_pos = s.value("Chat-fy-pos").toFloat();
+  } else {
+    qt_settings.chat_fy_pos = 0.6;
+  }
+  if (s.contains("unit_fx")) {
+    qt_settings.unit_info_pos_fx = s.value("unit_fx").toFloat();
+  } else {
+    qt_settings.unit_info_pos_fx = 0.33;
+  }
+  if (s.contains("unit_fy")) {
+    qt_settings.unit_info_pos_fy = s.value("unit_fy").toFloat();
+  } else {
+    qt_settings.unit_info_pos_fy = 0.88;
+  }
+  if (s.contains("minimap_x")) {
+    qt_settings.minimap_x = s.value("minimap_x").toFloat();
+  } else {
+    qt_settings.minimap_x = 0.84;
+  }
+  if (s.contains("minimap_y")) {
+    qt_settings.minimap_y = s.value("minimap_y").toFloat();
+  } else {
+    qt_settings.minimap_y = 0.79;
+  }
+  if (s.contains("minimap_width")) {
+    qt_settings.minimap_width = s.value("minimap_width").toFloat();
+  } else {
+    qt_settings.minimap_width = 0.15;
+  }
+  if (s.contains("minimap_height")) {
+    qt_settings.minimap_height = s.value("minimap_height").toFloat();
+  } else {
+    qt_settings.minimap_height = 0.2;
+  }
+  if (s.contains("battlelog_scale")) {
+    qt_settings.battlelog_scale = s.value("battlelog_scale").toFloat();
+  } else {
+    qt_settings.battlelog_scale = 1;
+  }
+
+  if (s.contains("City-dialog")) {
+    qt_settings.city_geometry = s.value("City-dialog").toByteArray();
+  }
+  if (s.contains("splitter1")) {
+    qt_settings.city_splitter1 = s.value("splitter1").toByteArray();
+  }
+  if (s.contains("splitter2")) {
+    qt_settings.city_splitter2 = s.value("splitter2").toByteArray();
+  }
+  if (s.contains("splitter3")) {
+    qt_settings.city_splitter3 = s.value("splitter3").toByteArray();
+  }
+  if (s.contains("help-dialog")) {
+    qt_settings.help_geometry = s.value("help-dialog").toByteArray();
+  }
+  if (s.contains("help_splitter1")) {
+    qt_settings.help_splitter1 = s.value("help_splitter1").toByteArray();
+  }
+  if (s.contains("new_turn_text")) {
+    qt_settings.show_new_turn_text = s.value("new_turn_text").toBool();
+  } else {
+    qt_settings.show_new_turn_text = true;
+  }
+  if (s.contains("show_battle_log")) {
+    qt_settings.show_battle_log = s.value("show_battle_log").toBool();
+  } else {
+    qt_settings.show_battle_log = false;
+  }
+  if (s.contains("battlelog_x")) {
+    qt_settings.battlelog_x = s.value("battlelog_x").toFloat();
+  } else {
+    qt_settings.battlelog_x = 0.0;
+  }
+  if (s.contains("minimap_y")) {
+    qt_settings.battlelog_y = s.value("battlelog_y").toFloat();
+  } else {
+    qt_settings.battlelog_y = 0.0;
   }
   qt_settings.player_repo_sort_col = -1;
   qt_settings.city_repo_sort_col = -1;
 
+  if (qt_settings.chat_fx_pos < 0 || qt_settings.chat_fx_pos >= 1) {
+    qt_settings.chat_fx_pos = 0.0;
+  }
+  if (qt_settings.chat_fy_pos >= 1 || qt_settings.chat_fy_pos < 0) {
+    qt_settings.chat_fy_pos = 0.6;
+  }
+  if (qt_settings.chat_fwidth < 0.05 || qt_settings.chat_fwidth > 0.9) {
+    qt_settings.chat_fwidth = 0.2;
+  }
+  if (qt_settings.chat_fheight < 0.05 || qt_settings.chat_fheight > 0.9) {
+    qt_settings.chat_fheight = 0.33;
+  }
+  if (qt_settings.battlelog_x < 0.0) {
+    qt_settings.battlelog_x = 0.33;
+  }
+  if (qt_settings.battlelog_y < 0.0) {
+    qt_settings.battlelog_y = 0.0;
+  }
+  if (qt_settings.battlelog_scale > 5.0) {
+    qt_settings.battlelog_y = 5.0;
+  }
+  if (qt_settings.minimap_x < 0 || qt_settings.minimap_x > 1) {
+    qt_settings.chat_fx_pos = 0.84;
+  }
+  if (qt_settings.minimap_y < 0 || qt_settings.minimap_y > 1) {
+    qt_settings.chat_fx_pos = 0.79;
+  }
 }
 
 /****************************************************************************
@@ -622,38 +722,29 @@ void fc_client::write_settings()
 {
   QSettings s(QSettings::IniFormat, QSettings::UserScope,
               "freeciv-qt-client");
-  s.setValue("InfoTab-xsize", qt_settings.infotab_width);
-  s.setValue("InfoTab-ysize", qt_settings.infotab_height);
-}
-
-/****************************************************************************
-  Updates autocompleter for chat widgets
-****************************************************************************/
-void fc_client::update_completer()
-{
-  QStringList wordlist;
-  QString str;
-
-  conn_list_iterate(game.est_connections, pconn) {
-    if (pconn->playing) {
-      wordlist << pconn->playing->name;
-      wordlist << pconn->playing->username;
-    } else {
-      wordlist << pconn->username;
-    }
-  } conn_list_iterate_end;
-  players_iterate (pplayer){
-    str = pplayer->name;
-    if (!wordlist.contains(str)){
-      wordlist << str;
-    }
-  } players_iterate_end
-  if (chat_completer != NULL) {
-    delete chat_completer;
-  }
-  chat_completer = new QCompleter(wordlist);
-  chat_completer->setCaseSensitivity(Qt::CaseInsensitive);
-  chat_completer->setCompletionMode(QCompleter::InlineCompletion);
+  s.setValue("Fonts-set", true);
+  s.setValue("Chat-fx-size", qt_settings.chat_fwidth);
+  s.setValue("Chat-fy-size", qt_settings.chat_fheight);
+  s.setValue("Chat-fx-pos", qt_settings.chat_fx_pos);
+  s.setValue("Chat-fy-pos", qt_settings.chat_fy_pos);
+  s.setValue("City-dialog", qt_settings.city_geometry);
+  s.setValue("splitter1", qt_settings.city_splitter1);
+  s.setValue("splitter2", qt_settings.city_splitter2);
+  s.setValue("splitter3", qt_settings.city_splitter3);
+  s.setValue("help-dialog", qt_settings.help_geometry);
+  s.setValue("help_splitter1", qt_settings.help_splitter1);
+  s.setValue("unit_fx", qt_settings.unit_info_pos_fx);
+  s.setValue("unit_fy", qt_settings.unit_info_pos_fy);
+  s.setValue("minimap_x", qt_settings.minimap_x);
+  s.setValue("minimap_y", qt_settings.minimap_y);
+  s.setValue("minimap_width", qt_settings.minimap_width);
+  s.setValue("minimap_height", qt_settings.minimap_height);
+  s.setValue("battlelog_scale", qt_settings.battlelog_scale);
+  s.setValue("battlelog_x", qt_settings.battlelog_x);
+  s.setValue("battlelog_y", qt_settings.battlelog_y);
+  s.setValue("new_turn_text", qt_settings.show_new_turn_text);
+  s.setValue("show_battle_log", qt_settings.show_battle_log);
+  write_shortcuts();
 }
 
 /****************************************************************************
@@ -664,10 +755,10 @@ void fc_client::toggle_unit_sel_widget(struct tile *ptile)
   if (unit_sel != NULL) {
     unit_sel->close();
     delete unit_sel;
-    unit_sel = new unit_select(ptile, gui()->mapview_wdg);
+    unit_sel = new units_select(ptile, gui()->mapview_wdg);
     unit_sel->show();
   } else {
-    unit_sel = new unit_select(ptile, gui()->mapview_wdg);
+    unit_sel = new units_select(ptile, gui()->mapview_wdg);
     unit_sel->show();
   }
 }
@@ -736,75 +827,83 @@ void fc_client::create_cursors(void)
   }
 }
 
-/****************************************************************************
-  Returns desired font
-****************************************************************************/
-QFont *fc_font::get_font(QString name)
-{
-  /**
-   * example: get_font("gui_qt_font_city_label")
-   */
 
-  if (font_map.contains(name)) {
-    return font_map.value(name);
+/****************************************************************************
+  Contructor for corner widget (used for menubar)
+****************************************************************************/
+fc_corner::fc_corner(QMainWindow *qmw): QWidget()
+{
+  QHBoxLayout *hb;
+  QPushButton *qpb;
+  int h;
+  QFont *f = fc_font::instance()->get_font(fonts::default_font);
+
+  if (f->pointSize() > 0) {
+    h = f->pointSize();
   } else {
-    return NULL;
+    h = f->pixelSize();
+  }
+  mw = qmw;
+  hb = new QHBoxLayout();
+  qpb = new QPushButton(fc_icons::instance()->get_icon("cmin"), "");
+  qpb->setFixedSize(h, h);
+  connect(qpb, &QAbstractButton::clicked, this, &fc_corner::minimize);
+  hb->addWidget(qpb);
+  qpb = new QPushButton(fc_icons::instance()->get_icon("cmax"), "");
+  qpb->setFixedSize(h, h);
+  connect(qpb, &QAbstractButton::clicked, this, &fc_corner::maximize);
+  hb->addWidget(qpb);
+  qpb = new QPushButton(fc_icons::instance()->get_icon("cclose"), "");
+  qpb->setFixedSize(h, h);
+  connect(qpb, &QAbstractButton::clicked, this, &fc_corner::close_fc);
+  hb->addWidget(qpb);
+  setLayout(hb);
+}
+
+/****************************************************************************
+  Slot for closing freeciv via corner widget
+****************************************************************************/
+void fc_corner::close_fc()
+{
+  mw->close();
+}
+
+/****************************************************************************
+  Slot for maximizing freeciv window via corner widget
+****************************************************************************/
+void fc_corner::maximize()
+{
+  if (mw->isMaximized() == false) {
+    mw->showMaximized();
+  } else {
+    mw->showNormal();
   }
 }
 
 /****************************************************************************
-  Initiazlizes fonts from client options
+  Slot for minimizing freeciv window via corner widget
 ****************************************************************************/
-void fc_font::init_fonts()
+void fc_corner::minimize()
 {
-  QFont *f;
-  QString s;
-
-  /**
-   * default font names are:
-   * gui_qt_font_city_label
-   * gui_qt_font_notify_label and so on.
-   * (full list is in options.c in client dir)
-   */
-
-  options_iterate(client_optset, poption) {
-    if (option_type(poption) == OT_FONT) {
-      f = new QFont;
-      s = option_font_get(poption);
-      f->fromString(s);
-      s = option_name(poption);
-      set_font(s, f);
-    }
-  } options_iterate_end;
-}
-
-/****************************************************************************
-  Deletes all fonts
-****************************************************************************/
-void fc_font::release_fonts()
-{
-  foreach (QFont *f, font_map) {
-    delete f;
-  }
-}
-
-
-/****************************************************************************
-  Adds new font or overwrite old one
-****************************************************************************/
-void fc_font::set_font(QString name, QFont * qf)
-{
-  font_map.insert(name,qf);
+  mw->showMinimized();
 }
 
 /****************************************************************************
   Game tab widget constructor
 ****************************************************************************/
-fc_game_tab_widget::fc_game_tab_widget(): QTabWidget()
+fc_game_tab_widget::fc_game_tab_widget(): QStackedWidget()
 {
-  connect(this, SIGNAL(currentChanged(int)), SLOT(restore_label_color(int)));
 }
 
+/****************************************************************************
+  Init default settings for game_tab_widget
+****************************************************************************/
+void fc_game_tab_widget::init()
+{
+  connect(this, &QStackedWidget::currentChanged, this, &fc_game_tab_widget::current_changed);
+}
+
+
 /****************************************************************************
   Icon provider constructor
 ****************************************************************************/
@@ -838,37 +937,139 @@ void fc_icons::drop()
 ****************************************************************************/
 QIcon fc_icons::get_icon(const QString &id)
 {
-  QIcon icon = QIcon(fileinfoname(get_data_dirs(),
-                            QString("themes/gui-qt/icons/"
+  QIcon icon;
+  QString str;
+
+  str = QString("themes") + DIR_SEPARATOR + "gui-qt" + DIR_SEPARATOR;
+  /* Try custom icon from theme */
+  icon.addFile(fileinfoname(get_data_dirs(),
+                            QString(str.toLocal8Bit().data() + current_theme
+                                    + DIR_SEPARATOR
                                     + id + ".png").toLocal8Bit().data()));
+  str = str + "icons" + DIR_SEPARATOR;
+  /* Try icon from icons dir */
+  if (icon.isNull()) {
+  icon.addFile(fileinfoname(get_data_dirs(),
+                            QString(str.toLocal8Bit().data()
+                                    + id + ".png").toLocal8Bit().data()));
+  }
   return QIcon(icon);
 }
 
+/****************************************************************************
+  Returns pixmap by given name, pixmap needs to be deleted by someone else
+****************************************************************************/
+QPixmap* fc_icons::get_pixmap(const QString &id)
+{
+  QPixmap *pm;
+  bool status;
+  QString str;
+
+  pm = new QPixmap;
+  if (pixmap_cache.find(id, pm)) {
+    return pm;
+  }
+  str = QString("themes") + DIR_SEPARATOR + "gui-qt" + DIR_SEPARATOR;
+  status = pm->load(fileinfoname(get_data_dirs(),
+                                 QString(str + current_theme
+                                 + DIR_SEPARATOR
+                                 + id + ".png").toLocal8Bit().data()));
+
+  if (status == false) {
+    str = str + "icons" + DIR_SEPARATOR;
+    pm->load(fileinfoname(get_data_dirs(), QString(str
+                          + id + ".png").toLocal8Bit().data()));
+  }
+  pixmap_cache.insert(id);
+  return pm;
+}
+
 /****************************************************************************
   Returns path for icon
 ****************************************************************************/
 QString fc_icons::get_path(const QString &id)
 {
+  QString str;
+  
+  str = QString("themes") + DIR_SEPARATOR + "gui-qt"
+        + DIR_SEPARATOR + "icons" + DIR_SEPARATOR;
   return fileinfoname(get_data_dirs(),
-                      QString("themes/gui-qt/icons/"
-                              + id + ".png").toLocal8Bit().data());
-}
-
-
-/****************************************************************************
-  Restores black color of label
-****************************************************************************/
-void fc_game_tab_widget::restore_label_color(int index)
-{
-  change_color(index, QColor(Qt::black));
+                      QString(str + id + ".png").toLocal8Bit().data());
 }
 
 /****************************************************************************
-  Changes color of tab widget's label
+  Resize event for all game tab widgets
 ****************************************************************************/
-void fc_game_tab_widget::change_color(int index, QColor col)
+void fc_game_tab_widget::resizeEvent(QResizeEvent *event)
+{
+  QSize size;
+  size = event->size();
+  if (C_S_RUNNING <= client_state()) {
+    gui()->sidebar_wdg->resize_me(size.height());
+    map_canvas_resized(size.width(), size.height());
+    gui()->infotab->resize(qRound((size.width()
+                             * gui()->qt_settings.chat_fwidth)),
+                             qRound((size.height()
+                             * gui()->qt_settings.chat_fheight)));
+    gui()->infotab->move(qRound((size.width()
+                                 * gui()->qt_settings.chat_fx_pos)),
+                         qRound((size.height()
+                                 * gui()->qt_settings.chat_fy_pos)));
+    gui()->infotab->restore_chat();
+    gui()->minimapview_wdg->move(qRound(gui()->qt_settings.minimap_x
+                                 * mapview.width),
+                                 qRound(gui()->qt_settings.minimap_y
+                                 * mapview.height));
+    gui()->minimapview_wdg->resize(qRound(gui()->qt_settings.minimap_width
+                                   * mapview.width),
+                                   qRound(gui()->qt_settings.minimap_height
+                                   * mapview.height));
+    gui()->battlelog_wdg->set_scale(gui()->qt_settings.battlelog_scale);
+    gui()->battlelog_wdg->move(qRound(gui()->qt_settings.battlelog_x
+                               * mapview.width),
+                               qRound(gui()->qt_settings.battlelog_y
+                               * mapview.height));
+    gui()->x_vote->move(width() / 2 - gui()->x_vote->width() / 2, 0);
+    gui()->update_sidebar_tooltips();
+    side_disable_endturn(get_turn_done_button_state());
+    gui()->mapview_wdg->resize(event->size().width(),size.height());
+    gui()->unitinfo_wdg->update_actions(nullptr);
+  }
+  event->setAccepted(true);
+}
+
+/****************************************************************************
+  Tab has been changed
+****************************************************************************/
+void fc_game_tab_widget::current_changed(int index)
+{
+  QList<fc_sidewidget*> objs;
+  fc_sidewidget *sw;
+
+  if (gui()->is_closing()) {
+    return;
+  }
+  objs = gui()->sidebar_wdg->objects;
+
+  foreach(sw, objs) {
+    sw->update_final_pixmap();
+  }
+  currentWidget()->hide();
+  widget(index)->show();
+
+  /* Set focus to map instead sidebar */
+  if (gui()->mapview_wdg && gui()->current_page() == PAGE_GAME
+     && index == 0) {
+    gui()->mapview_wdg->setFocus();
+  }
+
+}
+
+/****************************************************************************
+  Pregame options contructor
+****************************************************************************/
+pregame_options::pregame_options(QWidget *parent) : QWidget(parent)
 {
-  tabBar()->setTabTextColor(index, col);
 }
 
 /****************************************************************************
@@ -876,23 +1077,29 @@ void fc_game_tab_widget::change_color(int index, QColor col)
 ****************************************************************************/
 void pregame_options::init()
 {
-  QGridLayout *layout;
-  QLabel *l1, *l2, *l3;
+  QFormLayout *layout;
+  QPushButton *qclient_options;
+  QHBoxLayout *hbox = nullptr;
   QPushButton *but;
   int level;
 
-  l1 = new QLabel(_("Number of Players\n(including AI):"));
-  l2 = new QLabel(_("AI Skill Level:"));
-  l3 = new QLabel(_("Ruleset:"));
-  layout = new QGridLayout(this);
+  layout = new QFormLayout(this);
+  nation = new QPushButton(this);
+  qclient_options = new QPushButton(_("Client Options"));
   max_players = new QSpinBox(this);
   ailevel = new QComboBox(this);
   cruleset = new QComboBox(this);
   max_players->setRange(1, MAX_NUM_PLAYERS);
 
-  for (level = AI_LEVEL_AWAY; level < AI_LEVEL_LAST; level++) {
+  // Text and icon set by update_buttons()
+  connect(nation, &QPushButton::clicked,
+          this, &pregame_options::pick_nation);
+  connect(qclient_options, SIGNAL(clicked()), parentWidget(),
+          SLOT(popup_client_options()));
+  for (level = 0; level < AI_LEVEL_COUNT; level++) {
     if (is_settable_ai_level(static_cast<ai_level>(level))) {
-      const char *level_name = ai_level_name(static_cast<ai_level>(level));
+      const char *level_name = ai_level_translated_name(
+                                        static_cast<ai_level>(level));
       ailevel->addItem(level_name, level);
     }
   }
@@ -908,16 +1115,79 @@ void pregame_options::init()
   but = new QPushButton;
   but->setText(_("More Game Options"));
   but->setIcon(fc_icons::instance()->get_icon("preferences-other"));
-  QObject::connect(but, SIGNAL(clicked()), this,
-                   SLOT(popup_server_options()));
-  layout->addWidget(l1, 0, 1);
-  layout->addWidget(l2, 1, 1);
-  layout->addWidget(l3, 2, 1);
-  layout->addWidget(max_players, 0, 2);
-  layout->addWidget(ailevel, 1, 2);
-  layout->addWidget(cruleset, 2, 2);
-  layout->addWidget(but, 3, 1);
+  QObject::connect(but, &QAbstractButton::clicked, this,
+                   &pregame_options::popup_server_options);
+
+  layout->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
+  layout->addRow(_("Nation:"), nation);
+  layout->addRow(_("Rules:"), cruleset);
+
+  hbox = new QHBoxLayout();
+  hbox->addWidget(max_players);
+  hbox->addWidget(ailevel);
+  layout->addRow(_("Players:"), hbox);
+  layout->addWidget(but);
+  layout->addWidget(qclient_options);
   setLayout(layout);
+
+  update_buttons();
+}
+
+/****************************************************************************
+  Update the ruleset list
+****************************************************************************/
+void pregame_options::set_rulesets(int num_rulesets, char **rulesets)
+{
+  int i;
+  int def_idx = -1;
+
+  cruleset->clear();
+  cruleset->blockSignals(true);
+  for (i = 0; i < num_rulesets; i++){
+    cruleset->addItem(rulesets[i], i);
+    if (!strcmp("default", rulesets[i])) {
+      def_idx = i;
+    }
+  }
+
+  /* HACK: server should tell us the current ruleset. */
+  cruleset->setCurrentIndex(def_idx);
+  cruleset->blockSignals(false);
+}
+
+/****************************************************************************
+  Sets the value of the "aifill" option. Doesn't send the new value to the
+  server
+****************************************************************************/
+void pregame_options::set_aifill(int aifill)
+{
+  max_players->blockSignals(true);
+  max_players->setValue(aifill);
+  max_players->blockSignals(false);
+}
+
+/****************************************************************************
+  Updates the buttons whenever the game state has changed
+****************************************************************************/
+void pregame_options::update_buttons()
+{
+  struct sprite *psprite = nullptr;
+  QPixmap *pixmap = nullptr;
+  const struct player *pplayer = client_player();
+
+  // Update the "Select Nation" button
+  if (pplayer != nullptr) {
+    if (pplayer->nation != nullptr) {
+      nation->setText(nation_adjective_for_player(pplayer));
+      psprite = get_nation_shield_sprite(tileset, pplayer->nation);
+      pixmap = psprite->pm;
+      nation->setIconSize(pixmap->size());
+      nation->setIcon(QIcon(*pixmap));
+    } else {
+      nation->setText(_("Random"));
+      nation->setIcon(fc_icons::instance()->get_icon("flush-random"));
+    }
+  }
 }
 
 /****************************************************************************
@@ -939,7 +1209,6 @@ void pregame_options::ailevel_change(int i)
   k = ailevel->currentData().toInt();
   name = ai_level_cmd(static_cast<ai_level>(k));
   send_chat_printf("/%s", name);
-
 }
 
 /****************************************************************************
@@ -952,6 +1221,14 @@ void pregame_options::ruleset_change(int i)
   }
 }
 
+/****************************************************************************
+  Slot for picking a nation
+****************************************************************************/
+void pregame_options::pick_nation()
+{
+  popup_races_dialog(client_player());
+}
+
 /****************************************************************************
   Popups client options
 ****************************************************************************/
