@@ -15,33 +15,28 @@
 #include <fc_config.h>
 #endif
 
-#include <stdlib.h>
-
 // client
 #include "tilespec.h"
 
 // gui-qt
+#include "graphics.h"
 #include "qtg_cxxside.h"
 
-#include "graphics.h"
-
-static struct sprite *intro_gfx_sprite = NULL;
-static struct sprite *radar_gfx_sprite = NULL;
+static struct sprite *intro_gfx_sprite = nullptr;
+static struct sprite *radar_gfx_sprite = nullptr;
 
 /****************************************************************************
-  Return whether the client supports isometric view (isometric tilesets).
+  Return whether the client supports given view type.
 ****************************************************************************/
-bool qtg_isometric_view_supported()
+bool qtg_is_view_supported(enum ts_type type)
 {
-  return true;
-}
+  switch (type) {
+  case TS_ISOMETRIC:
+  case TS_OVERHEAD:
+    return true;
+  }
 
-/****************************************************************************
-  Return whether the client supports "overhead" (non-isometric) view.
-****************************************************************************/
-bool qtg_overhead_view_supported()
-{
-  return true;
+  return false;
 }
 
 /****************************************************************************
