@@ -1,4 +1,4 @@
-/**********************************************************************
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,10 +17,13 @@
 
 #ifdef AUDIO_SDL
 /* Though it would happily compile without this include,
- * it is needed for sound to work as long as SDL-1.2 mixer is
- * being used. It defines "main" macro to rename our main() so that
+ * it is needed for sound to work.
+ * It defines "main" macro to rename our main() so that
  * it can install SDL's own. */
-#ifdef AUDIO_SDL1_2
+#ifdef SDL2_PLAIN_INCLUDE
+#include <SDL.h>
+#include <SDL_mixer.h>
+#elif AUDIO_SDL1_2
 /* SDL */
 #include <SDL/SDL.h>
 #else  /* AUDIO_SDL1_2 */
@@ -38,39 +41,44 @@
 #include <QStyleFactory>
 
 // utility
+#include "fc_cmdline.h"
 #include "fciconv.h"
 #include "log.h"
 
 // client
 #include "client_main.h"
 #include "editgui_g.h"
-#include "ggz_g.h"
 #include "options.h"
-#include "tilespec.h"
 #include "sprite.h"
+#include "tilespec.h"
 
 // gui-qt
 #include "fc_client.h"
-#include "helpdlg.h"
+#include "fonts.h"
 #include "gui_main.h"
+#include "helpdlg.h"
+#include "hudwidget.h"
 #include "qtg_cxxside.h"
 
+extern "C" {
+  void real_science_report_dialog_update();
+}
+extern void restart_notify_dialogs();
+extern void city_font_update();
 
-static QApplication *qapp;
-static fc_client *freeciv_qt;
-const char *client_string = "gui-qt";
-
-const char * const gui_character_encoding = "UTF-8";
 const bool gui_use_transliteration = false;
-
-static QPixmap *unit_pixmap;
-extern char gui_qt_default_theme_name[512];
+const char * const gui_character_encoding = "UTF-8";
+const char *client_string = "gui-qt";
+static fc_client *freeciv_qt;
+static QApplication *qapp = nullptr;
 
 void reset_unit_table(void);
-static void populate_unit_pixmap_table(void);
-static void apply_font(struct option *poption);
 static void apply_city_font(struct option *poption);
+static void apply_font(struct option *poption);
 static void apply_help_font(struct option *poption);
+static void apply_notify_font(struct option *poption);
+static void apply_sidebar(struct option *poption);
+static void apply_titlebar(struct option *poption);
 
 /****************************************************************************
   Return fc_client instance
@@ -80,14 +88,6 @@ class fc_client *gui()
   return freeciv_qt;
 }
 
-/****************************************************************************
-  Return QApplication instance
-****************************************************************************/
-class QApplication *current_app()
-{
-  return qapp;
-}
-
 /****************************************************************************
   Called by the tileset code to set the font size that should be used to
   draw the city names and productions.
@@ -95,6 +95,7 @@ class QApplication *current_app()
 void qtg_set_city_names_font_sizes(int my_city_names_font_size,
                                    int my_city_productions_font_size)
 {
+  /* Ignored in favour of client configuration */
 }
 
 /**************************************************************************
@@ -149,6 +150,20 @@ static bool parse_options(int argc, char **argv)
   return true;
 }
 
+/**************************************************************************
+  Migrate Qt client specific options from freeciv-2.5 options
+**************************************************************************/
+static void migrate_options_from_2_5()
+{
+  if (!gui_options.first_boot) {
+    log_normal(_("Migrating Qt-client options from freeciv-2.5 options."));
+
+    gui_options.gui_qt_fullscreen = gui_options.migrate_fullscreen;
+
+    gui_options.gui_qt_migrated_from_2_5 = TRUE;
+  }
+}
+
 /**************************************************************************
   The main loop for the UI.  This is called from main(), and when it
   exits the client will exit.
@@ -157,25 +172,38 @@ void qtg_ui_main(int argc, char *argv[])
 {
   if (parse_options(argc, argv)) {
     qapp = new QApplication(argc, argv);
-    QPixmap *qpm = new QPixmap;
+    QPixmap *qpm;
     QIcon app_icon;
 
     tileset_init(tileset);
     tileset_load_tiles(tileset);
-    populate_unit_pixmap_table();
     qpm = get_icon_sprite(tileset, ICON_FREECIV)->pm;
     app_icon = ::QIcon(*qpm);
     qapp->setWindowIcon(app_icon);
-    qapp->setStyle(QStyleFactory::create(gui_qt_default_theme_name));
+    if (!gui_options.gui_qt_migrated_from_2_5) {
+      migrate_options_from_2_5();
+    }
+    qtg_gui_load_theme(fileinfoname(get_data_dirs(), QString(
+                      QString("themes") + DIR_SEPARATOR
+                      + "gui-qt" ).toLocal8Bit().data()),
+                       gui_options.gui_qt_default_theme_name);
     freeciv_qt = new fc_client();
-    freeciv_qt->main(qapp);
+    freeciv_qt->fc_main(qapp);
   }
 }
 
+/****************************************************************************
+  Return the running QApplication.
+****************************************************************************/
+QApplication *current_app()
+{
+  return qapp;
+}
+
 /****************************************************************************
   Extra initializers for client options.
 ****************************************************************************/
-void qtg_gui_options_extra_init()
+void qtg_options_extra_init()
 {
     struct option *poption;
 
@@ -185,13 +213,14 @@ void qtg_gui_options_extra_init()
   } else {                                                                  \
     log_error("Didn't find option %s!", #var);                              \
   }
-
   option_var_set_callback(gui_qt_font_city_names,
                           apply_font);
   option_var_set_callback(gui_qt_font_city_productions,
                           apply_font);
   option_var_set_callback(gui_qt_font_reqtree_text,
                           apply_font);
+  option_var_set_callback(gui_qt_font_default,
+                          apply_font);
   option_var_set_callback(gui_qt_font_city_label,
                           apply_city_font);
   option_var_set_callback(gui_qt_font_help_label,
@@ -202,6 +231,12 @@ void qtg_gui_options_extra_init()
                           apply_help_font);
   option_var_set_callback(gui_qt_font_chatline,
                           apply_font);
+  option_var_set_callback(gui_qt_font_notify_label,
+                          apply_notify_font);
+  option_var_set_callback(gui_qt_show_titlebar,
+                          apply_titlebar);
+  option_var_set_callback(gui_qt_sidebar_left,
+                          apply_sidebar);
 #undef option_var_set_callback
 }
 
@@ -212,6 +247,7 @@ void qtg_ui_exit()
 {
   delete freeciv_qt;
   delete qapp;
+  qapp = nullptr;
 }
 
 /**************************************************************************
@@ -219,6 +255,9 @@ void qtg_ui_exit()
 **************************************************************************/
 void qtg_real_conn_list_dialog_update()
 {
+  if (qtg_get_current_client_page() == PAGE_NETWORK) {
+    qtg_real_set_client_page(PAGE_START);
+  }
   gui()->update_start_page();
 }
 
@@ -228,7 +267,8 @@ void qtg_real_conn_list_dialog_update()
 **************************************************************************/
 void qtg_sound_bell()
 {
-  /* PORTME */
+  QApplication::beep();
+  QApplication::alert(gui()->central_wdg);
 }
 
 /**************************************************************************
@@ -253,23 +293,6 @@ void qtg_remove_net_input()
   gui()->remove_server_source();
 }
 
-/**************************************************************************
-  Called to monitor a GGZ socket.
-**************************************************************************/
-void qtg_add_ggz_input(int sock)
-{
-  /* PORTME */
-}
-
-/**************************************************************************
-  Called on disconnection to remove monitoring on the GGZ socket.  Only
-  call this if we're actually in GGZ mode.
-**************************************************************************/
-void qtg_remove_ggz_input()
-{
-  /* PORTME */
-}
-
 /**************************************************************************
   Set one of the unit icons (specified by idx) in the information area
   based on punit.
@@ -324,6 +347,43 @@ void qtg_add_idle_callback(void (callback)(void *), void *data)
   gui()->mr_idler.add_callback(cb);
 }
 
+/****************************************************************************
+  Shows/Hides titlebar
+****************************************************************************/
+void apply_titlebar(struct option *poption)
+{
+  bool val;
+  QWidget *w;
+  Qt::WindowFlags flags = Qt::Window;
+  val = option_bool_get(poption);
+  
+  if (gui()->current_page() < PAGE_GAME) {
+    return;
+  }
+
+  if (val == true) {
+    w = new QWidget();
+    gui()->setWindowFlags(flags);
+    delete gui()->corner_wid;
+    gui()->corner_wid = nullptr;
+    gui()->menu_bar->setCornerWidget(w);
+  } else {
+    flags |= Qt::CustomizeWindowHint;
+    gui()->setWindowFlags(flags);
+    gui()->corner_wid = new fc_corner(gui());
+    gui()->menu_bar->setCornerWidget(gui()->corner_wid);
+  }
+  gui()->show();
+}
+
+/****************************************************************************
+  Change sidebar position
+****************************************************************************/
+void apply_sidebar(struct option *poption)
+{
+  gui()->update_sidebar_position();
+}
+
 /****************************************************************************
   Change the given font.
 ****************************************************************************/
@@ -338,14 +398,20 @@ static void apply_font(struct option *poption)
     s = option_font_get(poption);
     f->fromString(s);
     s = option_name(poption);
-    remove_old = gui()->fc_fonts.get_font(s);
+    remove_old = fc_font::instance()->get_font(s);
     delete remove_old;
-    gui()->fc_fonts.set_font(s, f);
+    fc_font::instance()->set_font(s, f);
     update_city_descriptions();
     gui()->infotab->chtwdg->update_font();
+    QApplication::setFont(*fc_font::instance()->get_font(fonts::default_font));
+    real_science_report_dialog_update();
+    fc_font::instance()->get_mapfont_size();
   }
 }
 
+/****************************************************************************
+  Applies help font
+****************************************************************************/
 static void apply_help_font(struct option *poption)
 {
   QFont *f;
@@ -357,35 +423,37 @@ static void apply_help_font(struct option *poption)
     s = option_font_get(poption);
     f->fromString(s);
     s = option_name(poption);
-    remove_old = gui()->fc_fonts.get_font(s);
+    remove_old = fc_font::instance()->get_font(s);
     delete remove_old;
-    gui()->fc_fonts.set_font(s, f);
+    fc_font::instance()->set_font(s, f);
     update_help_fonts();
   }
 }
 
+/****************************************************************************
+  Applies help font
+****************************************************************************/
+static void apply_notify_font(struct option *poption)
+{
+  if (gui()) {
+    qtg_gui_update_font("notify_label", option_font_get(poption));
+    restart_notify_dialogs();
+  }
+}
+
 
 /****************************************************************************
   Changes city label font
 ****************************************************************************/
 void apply_city_font(option *poption)
 {
-  QFont *f;
-  QFont *remove_old;
-  QString s;
-
   if (gui() && qtg_get_current_client_page() == PAGE_GAME) {
-    f = new QFont;
-    s = option_font_get(poption);
-    f->fromString(s);
-    s = option_name(poption);
-    remove_old = gui()->fc_fonts.get_font(s);
-    delete remove_old;
-    gui()->fc_fonts.set_font(s, f);
-    qtg_popdown_all_city_dialogs();
+    qtg_gui_update_font("city_label", option_font_get(poption));
+    city_font_update();
   }
 }
 
+
 /****************************************************************************
   Stub for editor function
 ****************************************************************************/
@@ -413,7 +481,8 @@ void qtg_editgui_popdown_all()
 /****************************************************************************
   Stub for editor function
 ****************************************************************************/
-void qtg_editgui_notify_object_changed(int objtype, int object_id, bool remove)
+void qtg_editgui_notify_object_changed(int objtype, int object_id,
+                                       bool removal)
 {}
 
 /****************************************************************************
@@ -422,24 +491,22 @@ void qtg_editgui_notify_object_changed(int objtype, int object_id, bool remove)
 void qtg_editgui_notify_object_created(int tag, int id)
 {}
 
-/****************************************************************************
-  Stub for ggz function
-****************************************************************************/
-void qtg_gui_ggz_embed_leave_table()
-{}
-
-/****************************************************************************
-  Stub for ggz function
-****************************************************************************/
-void qtg_gui_ggz_embed_ensure_server()
-{}
-
 /****************************************************************************
   Updates a gui font style.
 ****************************************************************************/
 void qtg_gui_update_font(const char *font_name, const char *font_value)
 {
-  /* PORTME */
+  QFont *f;
+  QFont *remove_old;
+  QString fname;
+
+  fname = "gui_qt_font_" + QString(font_name);
+  f = new QFont;
+  f->fromString(font_value);
+  remove_old = fc_font::instance()->get_font(fname);
+  delete remove_old;
+  fc_font::instance()->set_font(fname, f);
+  fc_font::instance()->get_mapfont_size();
 }
 
 /****************************************************************************
@@ -463,14 +530,12 @@ void reset_unit_table(void)
 **************************************************************************/
 void popup_quit_dialog()
 {
-  QMessageBox ask(gui()->central_wdg);
+  hud_message_box ask(gui()->central_wdg);
   int ret;
 
-  ask.setText(_("Are you sure you want to quit?"));
   ask.setStandardButtons(QMessageBox::Cancel | QMessageBox::Ok);
   ask.setDefaultButton(QMessageBox::Cancel);
-  ask.setIcon(QMessageBox::Warning);
-  ask.setWindowTitle(_("Quit?"));
+  ask.set_text_title(_("Are you sure you want to quit?"),  _("Quit?"));
   ret = ask.exec();
 
   switch (ret) {
@@ -489,14 +554,24 @@ void popup_quit_dialog()
 }
 
 /**************************************************************************
-  Called to build the unit_below pixmap table.  This is the table on the
-  left of the screen that shows all of the inactive units in the current
-  tile.
-
-  It may be called again if the tileset changes.
+  Insert build information to help
 **************************************************************************/
-static void populate_unit_pixmap_table(void)
+void qtg_insert_client_build_info(char *outbuf, size_t outlen)
 {
-  unit_pixmap = new QPixmap(tileset_unit_width(tileset), 
-                            tileset_unit_height(tileset));
+  /* There's separate entry about Qt in help menu.
+   * Should we enable this regardless? As then to place to find such information
+   * would be standard over clients. */
+
+  /*
+  cat_snprintf(outbuf, outlen, _("\nBuilt against Qt %s, using %s"),
+               QT_VERSION_STR, qVersion());
+  */
+}
+
+/**************************************************************************
+  Make dynamic adjustments to first-launch default options.
+**************************************************************************/
+void qtg_adjust_default_options()
+{
+  /* Nothing in case of this gui */
 }
