@@ -33,6 +33,9 @@ class QLabel;
 class QPushButton;
 class QSignalMapper;
 class QScrollArea;
+struct fc_shortcut;
+
+void qt_start_turn();
 
 /** used for indicating menu about current option - for renaming
  * and enabling, disabling */
@@ -50,6 +53,7 @@ enum munit {
   PILLAGE,
   BUILD,
   ROAD,
+  FORTIFY,
   FORTRESS,
   AIRBASE,
   POLLUTION,
@@ -64,50 +68,153 @@ enum munit {
   GOTO_CITY,
   AIRLIFT,
   BUILD_WONDER,
+  AUTOTRADEROUTE,
   ORDER_TRADEROUTE,
   ORDER_DIPLOMAT_DLG,
   NUKE,
   UPGRADE,
   GO_AND_BUILD_CITY,
   NOT_4_OBS,
+  MULTIPLIERS,
+  ENDGAME,
   SAVE
 };
 
-/**************************************************************************
-  Class for filtering chosen units
-**************************************************************************/
-class unit_filter {
-public:
-  unit_filter();
-  void reset_activity();
-  void reset_other();
-  bool any_activity;
-  bool forified;
-  bool idle;
-  bool sentried;
-  bool any;
-  bool full_mp;
-  bool full_hp;
-  bool full_hp_mp;
+enum delay_order{
+  D_GOTO,
+  D_NUKE,
+  D_PARADROP,
+  D_FORT
 };
 
 /**************************************************************************
-  Custom dialog to show information
+  Struct holding rally point for city
 **************************************************************************/
-class fc_message_box : public QDialog
+struct qfc_rally
 {
-  Q_OBJECT
+  struct city *pcity;
+  struct tile *ptile;
+};
 
+/**************************************************************************
+  Class holding city list for rally points
+**************************************************************************/
+class qfc_rally_list
+{
 public:
-  fc_message_box() {};
-  void info(QWidget *parent, const QString &title, const QString &mess);
+  qfc_rally_list() {
+    hover_tile = false;
+    hover_city = false;
+  };
+  void add(qfc_rally* rally);
+  bool clear(struct city *rcity);
+  QList<qfc_rally*> rally_list;
+  void run();
+  bool hover_tile;
+  bool hover_city;
+  struct city *rally_city;
+};
+
+void multiairlift(struct city *acity, Unit_type_id ut);
+
+/**************************************************************************
+  Class representing one unit for delayed goto
+**************************************************************************/
+class qfc_delayed_unit_item
+{
+public:
+  qfc_delayed_unit_item(delay_order dg, int i) {
+    order = dg;
+    id = i;
+    ptile = nullptr;
+  }
+  delay_order order;
+  int id;
+  struct tile *ptile;
+};
+
+/**************************************************************************
+  Class holding unit list for delayed goto
+**************************************************************************/
+class qfc_units_list
+{
+public:
+  qfc_units_list();
+  void add(qfc_delayed_unit_item* fui);
+  void clear();
+  QList<qfc_delayed_unit_item*> unit_list;
+  int nr_units;
+};
+
+/**************************************************************************
+  Helper item for trade calculation
+***************************************************************************/
+class trade_city
+{
+public:
+  trade_city(struct city *pcity);
+
+  bool done;
+  int over_max;
+  int poss_trade_num;
+  int trade_num; // already created + generated
+  QList<struct city *> curr_tr_cities;
+  QList<struct city *> new_tr_cities;
+  QList<struct city *> pos_cities;
+  struct city *city;
+  struct tile *tile;
+
+};
+
+/**************************************************************************
+  Struct of 2 tiles, used for drawing trade routes.
+  Also assigned caravan if it was sent
+***************************************************************************/
+struct qtiles
+{
+  struct tile *t1;
+  struct tile *t2;
+  struct unit *autocaravan;
+
+  bool operator==(const qtiles& a) const
+  {
+    return (t1 == a.t1 && t2 == a.t2 && autocaravan == a.autocaravan);
+  }
+};
+
+/**************************************************************************
+  Class trade generator, used for calulating possible trade routes
+***************************************************************************/
+class trade_generator
+{
+public:
+  trade_generator();
+
+  bool hover_city;
+  QList<qtiles> lines;
+  QList<struct city *> virtual_cities;
+  QList<trade_city*> cities;
+
+  void add_all_cities();
+  void add_city(struct city *pcity);
+  void add_tile(struct tile *ptile);
+  void calculate();
+  void clear_trade_planing();
+  void remove_city(struct city *pcity);
+  void remove_virtual_city(struct tile *ptile);
 
 private:
-  QLabel *label;
-  QScrollArea *scroll;
-  QPushButton *ok_button;
+  bool discard_any(trade_city *tc, int freeroutes);
+  bool discard_one(trade_city *tc);
+  int find_over_max(struct city *pcity);
+  trade_city* find_most_free();
+  void check_if_done(trade_city *tc1, trade_city *tc2);
+  void discard();
+  void discard_trade(trade_city *tc1, trade_city *tc2);
+  void find_certain_routes();
 };
 
+
 /****************************************************************************
   Instantiable government menu.
 ****************************************************************************/
@@ -141,26 +248,45 @@ class mr_menu : public QMenuBar
 {
   Q_OBJECT
   QMenu *menu;
-  QMenu *filter_menu;
-  QActionGroup *filter_act;
-  QActionGroup *filter_any;;
+  QMenu *airlift_menu;
+  QMenu *multiplayer_menu;
+  QActionGroup *airlift_type;
+  QActionGroup *action_vs_city;
+  QActionGroup *action_vs_unit;
+  QMenu *action_unit_menu;
+  QMenu *action_city_menu;
   QHash<munit, QAction*> menu_list;
-  unit_filter u_filter;
+  qfc_units_list units_list;
 public:
   mr_menu();
   void setup_menus();
   void menus_sensitive();
+  void update_airlift_menu();
+  void set_tile_for_order(struct tile *ptile);
+  void execute_shortcut(int sid);
+  QString shortcut_exist(fc_shortcut *fcs);
+  QString shortcut_2_menustring(int sid);
   QAction *minimap_status;
+  QAction *lock_status;
+  QAction *osd_status;
+  QAction *btlog_status;
   QAction *chat_status;
   QAction *messages_status;
+  bool delayed_order;
+  bool quick_airlifting;
+  Unit_type_id airlift_type_id;
 private slots:
   /* game menu */
   void local_options();
+  void shortcut_options();
   void server_options();
   void messages_options();
   void save_options_now();
   void save_game();
   void save_game_as();
+  void save_image();
+  void tileset_custom_load();
+  void load_new_tileset();
   void back_to_menu();
   void quit_game();
 
@@ -199,6 +325,7 @@ private slots:
 
   /*used by combat menu*/
   void slot_unit_fortify();
+  void slot_unit_fortress();
   void slot_unit_airbase();
   void slot_pillage();
   void slot_action();
@@ -207,7 +334,10 @@ private slots:
   /*used by view menu*/
   void slot_center_view();
   void slot_minimap_view();
+  void slot_show_new_turn_text();
+  void slot_battlelog();
   void slot_fullscreen();
+  void slot_lock();
   void slot_city_outlines();
   void slot_city_output();
   void slot_map_grid();
@@ -219,7 +349,10 @@ private slots:
   void slot_city_buycost();
   void slot_city_traderoutes();
   void slot_city_names();
-  
+  void zoom_in();
+  void zoom_reset();
+  void zoom_out();
+
   /*used by select menu */
   void slot_select_one();
   void slot_select_all_tile();
@@ -228,12 +361,28 @@ private slots:
   void slot_select_same_everywhere();
   void slot_done_moving();
   void slot_wait();
-  void slot_filter();
-  void slot_filter_other();
+  void slot_unit_filter();
+
+  /* used by multiplayer menu */
+  void slot_orders_clear();
+  void slot_execute_orders();
+  void slot_delayed_goto();
+  void slot_trade_add_all();
+  void slot_trade_city();
+  void slot_calculate();
+  void slot_clear_trade();
+  void slot_autocaravan();
+  void slot_rally();
+  void slot_quickairlift_set();
+  void slot_quickairlift();
+  void slot_action_vs_unit();
+  void slot_action_vs_city();
 
   /*used by civilization menu */
   void slot_show_map();
+  void calc_trade_routes();
   void slot_popup_tax_rates();
+  void slot_popup_mult_rates();
   void slot_show_eco_report();
   void slot_show_units_report();
   void slot_show_nations();
@@ -241,17 +390,13 @@ private slots:
   void slot_show_research_tab();
   void slot_spaceship();
   void slot_demographics();
+  void slot_achievements();
+  void slot_endgame();
   void slot_top_five();
   void slot_traveler();
-  void slot_show_chat();
-  void slot_show_messages();
 
 private:
-  void unit_select(struct unit_list *punits, 
-                   enum unit_select_type_mode seltype,
-                   enum unit_select_location_mode selloc);
-  void apply_filter(struct unit *punit);
-  void apply_2nd_filter(struct unit *punit);
+  struct tile* find_last_unit_pos(struct unit* punit, int pos);
   QSignalMapper *signal_help_mapper;
 };
 
