@@ -16,25 +16,33 @@
 #endif
 
 // Qt
+#include <QApplication>
+#include <QCheckBox>
 #include <QColorDialog>
 #include <QComboBox>
+#include <QDialogButtonBox>
 #include <QFontDialog>
+#include <QGroupBox>
+#include <QHBoxLayout>
+#include <QLabel>
 #include <QLineEdit>
+#include <QPainter>
+#include <QPushButton>
 #include <QScrollArea>
+#include <QSignalMapper>
 #include <QSpinBox>
 
 // utility
 #include "log.h"
 #include "string_vector.h"
 
-// gui-qt
-#include "qtg_cxxside.h"
-
-#include "optiondlg.h"
-
 // client
 #include "options.h"
-#include <client_main.h>
+#include "client_main.h"
+
+// gui-qt
+#include "fc_client.h"
+#include "optiondlg.h"
 
 enum {
   RESPONSE_CANCEL,
@@ -118,7 +126,7 @@ QString cut_helptext(QString text)
 ****************************************************************************/
 option_dialog::option_dialog(const QString & name,
                              const option_set *options, QWidget *parent)
-  : QDialog (parent)
+  : qfc_dialog(parent)
 {
   QPushButton *but;
 
@@ -219,8 +227,9 @@ void option_dialog::apply_option(int response)
 /****************************************************************************
   Return selected colors (for highlighting chat).
 ****************************************************************************/
-struct ft_color option_dialog::get_color(struct option *poption) {
-
+void option_dialog::get_color(struct option *poption, QByteArray &a1,
+                              QByteArray &a2)
+{
   QPalette pal;
   QColor col1, col2;
   QWidget *w;
@@ -233,8 +242,8 @@ struct ft_color option_dialog::get_color(struct option *poption) {
   but = w->findChild<QPushButton *>("text_background");
   pal = but->palette();
   col2 =  pal.color(QPalette::Button);
-
-  return ft_color(col1.name().toUtf8().data(), col2.name().toUtf8().data());
+  a1 = col1.name().toUtf8();
+  a2 = col2.name().toUtf8();
 }
 
 /****************************************************************************
@@ -242,6 +251,8 @@ struct ft_color option_dialog::get_color(struct option *poption) {
 ****************************************************************************/
 void option_dialog::apply_options()
 {
+  QByteArray ba1, ba2;
+
   options_iterate(curr_options, poption) {
     switch (option_type(poption)) {
     case OT_BOOLEAN:
@@ -251,7 +262,7 @@ void option_dialog::apply_options()
       option_int_set(poption, get_int(poption));
       break;
     case OT_STRING:
-      option_str_set(poption, get_string(poption));
+      option_str_set(poption, get_string(poption).constData());
       break;
     case OT_ENUM:
       option_enum_set_int(poption, get_enum(poption));
@@ -260,10 +271,11 @@ void option_dialog::apply_options()
       option_bitwise_set(poption, get_bitwise(poption));
       break;
     case OT_FONT:
-      option_font_set(poption, get_button_font(poption));
+      option_font_set(poption, get_button_font(poption).constData());
       break;
     case OT_COLOR:
-      option_color_set(poption, get_color(poption));
+      get_color(poption,  ba1,  ba2);
+      option_color_set(poption, ft_color_construct(ba1.data(), ba2.data()));
       break;
     case OT_VIDEO_MODE:
       log_error("Option type %s (%d) not supported yet.",
@@ -330,12 +342,13 @@ void option_dialog::set_font(struct option* poption, QString s)
 
   fp = new QFont();
   fp->fromString(s);
+  qApp->processEvents();
   qp = reinterpret_cast<QPushButton *>(option_get_gui_data(poption));
   ql = s.split(",");
-  qp->setText(ql[0] + " " + ql[1]);
-  qp->setFont(*fp);
-
-  ::gui()->fc_fonts.set_font(QString(option_name(poption)), fp);
+  if (s.isEmpty() == false) {
+    qp->setText(ql[0] + " " + ql[1]);
+    qp->setFont(*fp);
+  }
 }
 
 
@@ -376,17 +389,17 @@ void option_dialog::set_string(struct option *poption, const char *string)
 /****************************************************************************
   Get string for desired option from combobox or lineedit.
 ****************************************************************************/
-char *option_dialog::get_string(struct option *poption)
+QByteArray option_dialog::get_string(struct option *poption)
 {
   QComboBox *cb;
   QLineEdit *le;
 
   if (option_str_values(poption) != NULL) {
     cb = reinterpret_cast<QComboBox *>(option_get_gui_data(poption));
-    return cb->currentText().toUtf8().data();
+    return cb->currentText().toUtf8();
   } else {
     le = reinterpret_cast<QLineEdit *>(option_get_gui_data(poption));
-    return le->displayText().toUtf8().data();
+    return le->displayText().toUtf8();
   }
 }
 
@@ -476,21 +489,21 @@ void option_dialog::set_color(struct option *poption, struct ft_color color)
   QColor col;
   QWidget *w;
   QPushButton *but;
+  QString s1 = "QPushButton { background-color: ";
+  QString s2 = ";}";
 
   w = reinterpret_cast<QPushButton *>(option_get_gui_data(poption));
   but = w->findChild<QPushButton *>("text_color");
   if (NULL != but && NULL != color.foreground 
       && '\0' != color.foreground[0]) {
     col.setNamedColor(color.foreground);
-    pal.setColor(QPalette::Button, col);
-    but->setPalette(pal);
+    but->setStyleSheet(s1 + col.name() + s2);
   }
-  but = w->findChild < QPushButton * >("text_background");
-  if (NULL != but && NULL != color.background 
+  but = w->findChild<QPushButton *>("text_background");
+  if (NULL != but && NULL != color.background
       && '\0' != color.background[0]) {
     col.setNamedColor(color.background);
-    pal2.setColor(QPalette::Button, col);
-    but->setPalette(pal2);
+    but->setStyleSheet(s1 + col.name() + s2);
   }
 }
 
@@ -627,7 +640,9 @@ void option_dialog::add_option(struct option *poption)
 
   if (!categories.contains(category_name)) {
     twidget = new QWidget();
+    twidget->setProperty("doomed", true);
     scroll = new QScrollArea();
+    scroll->setProperty("doomed", true);
     scroll->setWidgetResizable(true);
     twidget->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
     twidget_layout = new QVBoxLayout();
@@ -803,14 +818,14 @@ QFont option_dialog::get_font(struct option *poption)
 /****************************************************************************
   Get font from pushbutton.
 ****************************************************************************/
-char *option_dialog::get_button_font(struct option *poption)
+QByteArray option_dialog::get_button_font(struct option *poption)
 {
   QPushButton *qp;
   QFont f;
 
   qp = reinterpret_cast<QPushButton *>(option_get_gui_data(poption));
   f = qp->font();
-  return f.toString().toUtf8().data();
+  return f.toString().toUtf8();
 }
 
 /****************************************************************************
@@ -876,7 +891,7 @@ void option_gui_update(struct option *poption)
       update_nationset_combo();
     }
     if (strcmp(option_name(poption), "aifill") == 0) {
-      gui()->pr_options->max_players->setValue(option_int_get(poption));
+      gui()->pr_options->set_aifill(option_int_get(poption));
     }
   }
 }
