@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,12 +17,18 @@
 
 // Qt
 #include <QApplication>
+#include <QHBoxLayout>
 #include <QHeaderView>
+#include <QMouseEvent>
+#include <QPainter>
+#include <QPushButton>
+#include <QSortFilterProxyModel>
 #include <QSplitter>
+#include <QVBoxLayout>
 
 // gui-qt
+#include "fc_client.h"
 #include "plrdlg.h"
-#include "qtg_cxxside.h"
 
 /**************************************************************************
   Help function to draw checkbox inside delegate
@@ -138,6 +144,8 @@ bool plr_item::setData(int column, const QVariant &value, int role)
 **************************************************************************/
 QVariant plr_item::data(int column, int role) const
 {
+  QFont f;
+  QFontMetrics *fm;
   QPixmap *pix;
   QString str;
   struct player_dlg_column *pdc;
@@ -152,6 +160,10 @@ QVariant plr_item::data(int column, int role) const
   switch (player_dlg_columns[column].type) {
   case COL_FLAG:
     pix = get_nation_flag_sprite(tileset, nation_of_player(ipplayer))->pm;
+    f = *fc_font::instance()->get_font(fonts::default_font);
+    fm = new QFontMetrics(f);
+    *pix = pix->scaledToHeight(fm->height());
+    delete fm;
     return *pix;
     break;
   case COL_COLOR:
@@ -167,14 +179,13 @@ QVariant plr_item::data(int column, int role) const
     str = pdc->func(ipplayer);
     if (str.toInt() != 0){
       return str.toInt();
-    } else if (str == "?"){
+    } else if (str == "?") {
       return -1;
     }
     return str;
   default:
     return QVariant();
   }
-  return QVariant();
 }
 
 /**************************************************************************
@@ -294,8 +305,8 @@ plr_widget::plr_widget(plr_report *pr): QTreeView()
   setAlternatingRowColors(true);
   header()->setContextMenuPolicy(Qt::CustomContextMenu);
   hide_columns();
-  connect(header(), SIGNAL(customContextMenuRequested(const QPoint &)),
-          this, SLOT(display_header_menu(const QPoint &)));
+  connect(header(), &QWidget::customContextMenuRequested,
+          this, &plr_widget::display_header_menu);
   connect(selectionModel(),
           SIGNAL(selectionChanged(const QItemSelection &,
                                   const QItemSelection &)),
@@ -394,7 +405,7 @@ void plr_widget::nation_selected(const QItemSelection &sl,
   QModelIndexList indexes = sl.indexes();
   struct city *pcity;
   const struct player_diplstate *state;
-  struct player_research *research;
+  struct research *my_research, *research;
   char tbuf[256];
   QString res;
   QString sp = " ";
@@ -427,7 +438,7 @@ void plr_widget::nation_selected(const QItemSelection &sl,
     return;
   }
   pcity = player_capital(pplayer);
-  research = player_research_get(pplayer);
+  research = research_get(pplayer);
 
   switch (research->researching) {
   case A_UNKNOWN:
@@ -437,7 +448,8 @@ void plr_widget::nation_selected(const QItemSelection &sl,
     res = _("(none)");
     break;
   default:
-    res = QString(advance_name_researching(pplayer))
+    res = QString(research_advance_name_translation(research,
+                                                    research->researching))
           + sp + "(" + QString::number(research->bulbs_researched) + "/"
           + QString::number(research->client.researching_cost) + ")";
     break;
@@ -452,7 +464,7 @@ void plr_widget::nation_selected(const QItemSelection &sl,
     + QString("</b></td><td>") + QString(government_name_for_player(pplayer))
     + QString("</td></tr><tr><td><b>") + _("Capital:")
     + QString("</b></td><td>")
-    + QString(((!pcity) ? _("(unknown)") : city_name(pcity)))
+    + QString(((!pcity) ? _("(unknown)") : city_name_get(pcity)))
     + QString("</td></tr><tr><td><b>") + _("Gold:")
     + QString("</b></td><td>") + QString::number(pplayer->economic.gold)
     + QString("</td></tr><tr><td><b>") + _("Tax:")
@@ -478,8 +490,9 @@ void plr_widget::nation_selected(const QItemSelection &sl,
       if (static_cast<int>(state->type) == i) {
         if (added == false) {
           ally_str = ally_str  + QString("<b>")
-                   + QString(diplstate_text(static_cast<diplstate_type>(i)))
-                   + ": "  + QString("</b>") + nl;
+                     + QString(diplstate_type_translated_name(
+                                 static_cast<diplstate_type>(i)))
+                     + ": "  + QString("</b>") + nl;
           added = true;
         }
         if (gives_shared_vision(pplayer, other)) {
@@ -494,6 +507,7 @@ void plr_widget::nation_selected(const QItemSelection &sl,
     }
   }
   me = client_player();
+  my_research = research_get(me);
   if (!client_is_global_observer()) {
     if (player_has_embassy(me, pplayer) && me != pplayer) {
       a = 0;
@@ -504,15 +518,18 @@ void plr_widget::nation_selected(const QItemSelection &sl,
 
       advance_iterate(A_FIRST, padvance) {
         tech_id = advance_number(padvance);
-        if (player_invention_state(me, tech_id) == TECH_KNOWN
-            && (player_invention_state(pplayer, tech_id) != TECH_KNOWN)) {
+        if (research_invention_state(my_research, tech_id) == TECH_KNOWN
+            && (research_invention_state(research, tech_id) 
+                != TECH_KNOWN)) {
           a++;
-          sorted_list_a << advance_name_for_player(pplayer, tech_id);
+          sorted_list_a << research_advance_name_translation(research,
+                                                             tech_id);
         }
-        if (player_invention_state(me, tech_id) != TECH_KNOWN
-            && (player_invention_state(pplayer, tech_id) == TECH_KNOWN)) {
+        if (research_invention_state(my_research, tech_id) != TECH_KNOWN
+            && (research_invention_state(research, tech_id) == TECH_KNOWN)) {
           b++;
-          sorted_list_b << advance_name_for_player(pplayer, tech_id);
+          sorted_list_b << research_advance_name_translation(research,
+                                                             tech_id);
         }
       } advance_iterate_end;
       sorted_list_a.sort(Qt::CaseInsensitive);
@@ -544,8 +561,8 @@ void plr_widget::nation_selected(const QItemSelection &sl,
                arg(nation_plural_for_player(pplayer));
     advance_iterate(A_FIRST, padvance) {
       tech_id = advance_number(padvance);
-      if (player_invention_state(pplayer, tech_id) == TECH_KNOWN) {
-        sorted_list_a << advance_name_for_player(pplayer, tech_id);
+      if (research_invention_state(research, tech_id) == TECH_KNOWN) {
+        sorted_list_a << research_advance_name_translation(research, tech_id);
       }
     } advance_iterate_end;
     sorted_list_a.sort(Qt::CaseInsensitive);
@@ -608,9 +625,12 @@ plr_report::plr_report():QWidget()
   cancel_but->setText(_("Cancel Treaty"));
   withdraw_but = new QPushButton;
   withdraw_but->setText(_("Withdraw Vision"));
+  toggle_ai_but = new QPushButton;
+  toggle_ai_but->setText(_("Toggle AI Mode"));
   meet_but->setDisabled(true);
   cancel_but->setDisabled(true);
   withdraw_but->setDisabled(true);
+  toggle_ai_but->setDisabled(true);
   v_splitter->addWidget(plr_wdg);
   h_splitter->addWidget(plr_label);
   h_splitter->addWidget(ally_label);
@@ -620,11 +640,13 @@ plr_report::plr_report():QWidget()
   hlayout->addWidget(meet_but);
   hlayout->addWidget(cancel_but);
   hlayout->addWidget(withdraw_but);
+  hlayout->addWidget(toggle_ai_but);
   hlayout->addStretch();
   layout->addLayout(hlayout);
-  connect(meet_but, SIGNAL(pressed()), SLOT(req_meeeting()));
-  connect(cancel_but, SIGNAL(pressed()), SLOT(req_caancel_threaty()));
-  connect(withdraw_but, SIGNAL(pressed()), SLOT(req_wiithdrw_vision()));
+  connect(meet_but, &QAbstractButton::pressed, this, &plr_report::req_meeeting);
+  connect(cancel_but, &QAbstractButton::pressed, this, &plr_report::req_caancel_threaty);
+  connect(withdraw_but, &QAbstractButton::pressed, this, &plr_report::req_wiithdrw_vision);
+  connect(toggle_ai_but, &QAbstractButton::pressed, this, &plr_report::toggle_ai_mode);
   setLayout(layout);
   if (gui()->qt_settings.player_repo_sort_col != -1) {
     plr_wdg->sortByColumn(gui()->qt_settings.player_repo_sort_col,
@@ -646,7 +668,7 @@ plr_report::~plr_report()
 void plr_report::init()
 {
   gui()->gimme_place(this, "PLR");
-  index = gui()->add_game_tab(this, Q_("?header:Players"));
+  index = gui()->add_game_tab(this);
   gui()->game_tab_widget->setCurrentIndex(index);
 }
 
@@ -691,6 +713,45 @@ void plr_report::req_wiithdrw_vision()
                                      CLAUSE_VISION);
 }
 
+/**************************************************************************
+  Slot for changing AI mode
+**************************************************************************/
+void plr_report::toggle_ai_mode()
+{
+  QAction *act;
+  QAction *toggle_ai_act;
+  QAction *ai_level_act;
+  QMenu ai_menu(this);
+  int level;
+
+  toggle_ai_act = new QAction(_("Toggle AI Mode"), nullptr);
+  ai_menu.addAction(toggle_ai_act);
+  ai_menu.addSeparator();
+  for (level = 0; level < AI_LEVEL_COUNT; level++) {
+    if (is_settable_ai_level(static_cast<ai_level>(level))) {
+      QString ln = ai_level_translated_name(static_cast<ai_level>(level));
+      ai_level_act = new QAction(ln, nullptr);
+      ai_level_act->setData(QVariant::fromValue(level));
+      ai_menu.addAction(ai_level_act);
+    }
+  }
+  act = 0;
+  act = ai_menu.exec(QCursor::pos());
+  if (act == toggle_ai_act) {
+    send_chat_printf("/aitoggle \"%s\"", player_name(plr_wdg->other_player));
+    return;
+  }
+  if (act && act->isVisible()) {
+    level = act->data().toInt();
+    if (plr_wdg->other_player->ai_controlled == false) {
+      send_chat_printf("/aitoggle \"%s\"", player_name(plr_wdg->other_player));
+    }
+    send_chat_printf("/%s %s", ai_level_cmd(static_cast<ai_level>(level)),
+                     player_name(plr_wdg->other_player));
+  }
+
+}
+
 /**************************************************************************
   Handle mouse click
 **************************************************************************/
@@ -715,7 +776,7 @@ void plr_report::update_report(bool update_selection)
   /* Force updating selected player information */
   if (update_selection == true) {
     qmi = plr_wdg->currentIndex();
-    if (qmi.isValid()){
+    if (qmi.isValid()) {
       plr_wdg->clearSelection();
       plr_wdg->setCurrentIndex(qmi);
     }
@@ -736,6 +797,7 @@ void plr_report::update_report(bool update_selection)
   meet_but->setDisabled(true);
   cancel_but->setDisabled(true);
   withdraw_but->setDisabled(true);
+  toggle_ai_but->setDisabled(true);
   plr_label->setText(plr_wdg->intel_str);
   ally_label->setText(plr_wdg->ally_str);
   tech_label->setText(plr_wdg->tech_str);
@@ -751,6 +813,7 @@ void plr_report::update_report(bool update_selection)
     if (pplayer_can_cancel_treaty(client_player(), other_player) != DIPL_ERROR) {
       cancel_but->setEnabled(true);
     }
+    toggle_ai_but->setEnabled(true);
   }
   if (gives_shared_vision(client_player(), other_player)
       && !players_on_same_team(client_player(), other_player)) {
@@ -768,18 +831,25 @@ void plr_report::update_report(bool update_selection)
 void popup_players_dialog(bool raise)
 {
   int i;
-  plr_report *pr;
   QWidget *w;
+
   if (!gui()->is_repo_dlg_open("PLR")) {
     plr_report *pr = new plr_report;
+
     pr->init();
     pr->update_report();
   } else {
+    plr_report *pr;
+
     i = gui()->gimme_index_of("PLR");
-    fc_assert(i != -1);
     w = gui()->game_tab_widget->widget(i);
+    if (w->isVisible() == true) {
+      gui()->game_tab_widget->setCurrentIndex(0);
+      return;
+    }
     pr = reinterpret_cast<plr_report*>(w);
     gui()->game_tab_widget->setCurrentWidget(pr);
+    pr->update_report();
   }
 }
 
@@ -794,10 +864,11 @@ void real_players_dialog_update(void)
 
   if (gui()->is_repo_dlg_open("PLR")) {
     i = gui()->gimme_index_of("PLR");
-    fc_assert(i != -1);
-    w = gui()->game_tab_widget->widget(i);
-    pr = reinterpret_cast<plr_report *>(w);
-    pr->update_report();
+    if (gui()->game_tab_widget->currentIndex() == i) {
+      w = gui()->game_tab_widget->widget(i);
+      pr = reinterpret_cast<plr_report *>(w);
+      pr->update_report();
+    }
   }
 }
 
