@@ -15,15 +15,18 @@
 #include <fc_config.h>
 #endif
 
-#include <stdlib.h>
+// Qt
+#include <QImageReader>
+#include <QPainter>
 
 // gui-qt
 #include "colors.h"
 #include "fc_client.h"
 #include "qtg_cxxside.h"
-
 #include "sprite.h"
 
+static const char **gfx_array_extensions = nullptr;
+
 /****************************************************************************
   Return a NULL-terminated, permanently allocated array of possible
   graphics types extensions.  Extensions listed first will be checked
@@ -31,16 +34,27 @@
 ****************************************************************************/
 const char **gfx_fileextensions(void)
 {
-  /* PORTME */
+  QList<QByteArray> gfx_ext;
+  QByteArray cp;
+  int j = 0;
 
-  /* hack to allow stub to run */
-  static const char *ext[] = {
-    "png",  /* png should be the default. */
-    /* ...etc... */
-    NULL
-  };
+  if (gfx_array_extensions != nullptr) {
+    return gfx_array_extensions;
+  }
 
-  return ext;
+  gfx_ext = QImageReader::supportedImageFormats();
+
+  gfx_array_extensions = new const char *[gfx_ext.count()];
+  while (gfx_ext.isEmpty() == false) {
+    char *ext;
+    cp = gfx_ext.takeFirst();
+    ext = static_cast<char *>(fc_malloc(sizeof(cp)));
+    strncpy(ext, cp.data(), sizeof(cp));
+    gfx_array_extensions[j] = ext;
+    j++;
+  }
+
+  return gfx_array_extensions;
 }
 
 /****************************************************************************
@@ -81,30 +95,52 @@ struct sprite *qtg_load_gfxfile(const char *filename)
 struct sprite *qtg_crop_sprite(struct sprite *source,
                                int x, int y, int width, int height,
                                struct sprite *mask,
-                               int mask_offset_x, int mask_offset_y)
+                               int mask_offset_x, int mask_offset_y,
+                               float scale, bool smooth)
 {
+  QPainter p;
+  QRectF source_rect;
+  QRectF dest_rect;
+  sprite *cropped;
+  int widthzoom;
+  int heightzoom;
+  int hex = 0;
 
   if (!width || !height) {
     return NULL;
   }
 
-  sprite *cropped = new sprite;
-
-  cropped->pm = new QPixmap(width, height);
+  if (scale != 1.0f && (tileset_hex_height(tileset) > 0
+      || tileset_hex_width(tileset) > 0)) {
+    hex = 1;
+  }
+  widthzoom = ceil(width * scale) + hex;
+  heightzoom = ceil(height * scale) + hex;
+  cropped = new sprite;
+  cropped->pm = new QPixmap(widthzoom, heightzoom);
   cropped->pm->fill(Qt::transparent);
-  QRectF source_rect(x,y,width, height);
-  QRectF dest_rect(0,0,width, height);
+  source_rect = QRectF(x, y, width, height);
+  dest_rect = QRectF(0, 0, widthzoom, heightzoom);
 
-  QPainter p;
   p.begin(cropped->pm);
-  p.drawPixmap(dest_rect, *source->pm,source_rect);
+  if (smooth) {
+    p.setRenderHint(QPainter::SmoothPixmapTransform);
+  }
+  p.setRenderHint(QPainter::Antialiasing);
+  p.drawPixmap(dest_rect, *source->pm, source_rect);
   p.end();
 
+  if (mask) {
+    int mw = mask->pm->width();
+    int mh = mask->pm->height();
 
-  if(mask) {
+    source_rect = QRectF(0, 0, mw, mh);
+    dest_rect = QRectF(mask_offset_x - x, mask_offset_y - y, mw, mh);
     p.begin(cropped->pm);
     p.setCompositionMode(QPainter::CompositionMode_DestinationIn);
-    p.drawPixmap(mask_offset_x-x,mask_offset_y-y,*mask->pm);
+    p.setRenderHint(QPainter::Antialiasing);
+    p.setRenderHint(QPainter::SmoothPixmapTransform);
+    p.drawPixmap(dest_rect, *mask->pm, source_rect);
     p.end();
   }
 
