@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,7 +23,8 @@
 #include <fc_config.h>
 #endif
 
-#include "SDL/SDL.h"
+/* SDL */
+#include <SDL/SDL.h>
 
 /* utility */
 #include "fcintl.h"
@@ -208,7 +209,7 @@ static void popup_load_game_dialog(void)
   area = pWindow->area;
   
   /* close button */
-  pCloseButton = create_themeicon(pTheme->Small_CANCEL_Icon, pWindow->dst,
+  pCloseButton = create_themeicon(current_theme->Small_CANCEL_Icon, pWindow->dst,
                                   WF_WIDGET_HAS_INFO_LABEL
                                   | WF_RESTORE_BACKGROUND);
   pCloseButton->info_label = create_str16_from_char(_("Close Dialog (Esc)"),
@@ -785,7 +786,7 @@ static void popup_conn_list_dialog(void)
 
   /* buttons */
 
-  pBuf = create_themeicon_button_from_chars(pTheme->BACK_Icon, pWindow->dst,
+  pBuf = create_themeicon_button_from_chars(current_theme->BACK_Icon, pWindow->dst,
   				_("Back"), adj_font(12), 0);
   pBuf->size.x = adj_size(10);
   pBuf->size.y = pWindow->size.h - adj_size(10) - pBuf->size.h;
@@ -796,8 +797,8 @@ static void popup_conn_list_dialog(void)
   add_to_gui_list(ID_BUTTON, pBuf);
   pBackButton = pBuf;
 
-  pBuf = create_themeicon_button_from_chars(pTheme->OK_Icon, pWindow->dst,
-  				_("Start"), adj_font(12), 0);
+  pBuf = create_themeicon_button_from_chars(current_theme->OK_Icon, pWindow->dst,
+                                            _("Start"), adj_font(12), 0);
   pBuf->size.x = pWindow->size.w - adj_size(10) - pBuf->size.w;
   pBuf->size.y = pBackButton->size.y;
   pConnDlg->pStartButton = pBuf;
@@ -892,7 +893,7 @@ bool popdown_conn_list_dialog(void)
 /**************************************************************************
   Got version message from metaserver thread.
 **************************************************************************/
-void version_message(char *vertext)
+void version_message(const char *vertext)
 {
   output_window_append(ftc_client, vertext);
 }
