@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,13 +11,13 @@
    GNU General Public License for more details.
 ***********************************************************************/
 
-/**********************************************************************
+/***********************************************************************
                           chatline.h  -  description
                              -------------------
     begin                : Sun Jun 30 2002
     copyright            : (C) 2002 by Rafał Bursig
     email                : Rafał Bursig <bursig@poczta.fm>
- **********************************************************************/
+***********************************************************************/
 
 #ifndef FC__CHATLINE_H
 #define FC__CHATLINE_H
@@ -28,7 +28,7 @@ void popup_input_line(void);
 bool popdown_conn_list_dialog(void);
 void popdown_load_game_dialog(void);
 
-#define set_output_window_text( pString )	\
-	output_window_append( ftc_any, pString )
+#define set_output_window_text(_pstr_) \
+  output_window_append( ftc_any, _pstr_ )
 
 #endif	/* FC__CHATLINE_H */
