@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,7 +15,8 @@
 #include <fc_config.h>
 #endif
 
-#include "SDL/SDL.h"
+/* SDL */
+#include <SDL/SDL.h>
 
 /* utility */
 #include "fcintl.h"
@@ -24,6 +25,7 @@
 /* common */
 #include "diptreaty.h"
 #include "game.h"
+#include "research.h"
 
 /* client */
 #include "client_main.h"
@@ -636,14 +638,18 @@ static struct ADVANCED_DLG * popup_diplomatic_objects(struct player *pPlayer0,
   
   /* Trading: advances */
   if (game.info.trading_tech) {
+    const struct research *presearch0 = research_get(pPlayer0);
+    const struct research *presearch1 = research_get(pPlayer1);
     int flag = A_NONE;
-    
+
     advance_index_iterate(A_FIRST, i) {
-      if (player_invention_state(pPlayer0, i) == TECH_KNOWN &&
-         player_invention_reachable(pPlayer1, i, FALSE) &&
-	(player_invention_state(pPlayer1, i) == TECH_UNKNOWN || 
-	 player_invention_state(pPlayer1, i) == TECH_PREREQS_KNOWN)) {
-	     
+      if (research_invention_state(presearch0, i) == TECH_KNOWN
+          && research_invention_gettable(presearch1, i,
+                                         game.info.tech_trade_allow_holes)
+          && (research_invention_state(presearch1, i) == TECH_UNKNOWN
+              || research_invention_state(presearch1, i)
+                 == TECH_PREREQS_KNOWN)) {
+
 	     pBuf = create_iconlabel_from_chars(NULL, pWindow->dst,
 		_("Advances"), adj_font(12), WF_RESTORE_BACKGROUND);
              pBuf->string16->fgcol = *get_theme_color(COLOR_THEME_DIPLODLG_MEETING_HEADING_TEXT);
@@ -651,9 +657,9 @@ static struct ADVANCED_DLG * popup_diplomatic_objects(struct player *pPlayer0,
 	     height = MAX(height, pBuf->size.h);
              add_to_gui_list(ID_LABEL, pBuf);
 	     count++;
-	     
+
 	     fc_snprintf(cBuf, sizeof(cBuf), "  %s", advance_name_translation(advance_by_number(i)));
-  
+
              pBuf = create_iconlabel_from_chars(NULL, pWindow->dst, cBuf, adj_font(12),
 	         (WF_RESTORE_BACKGROUND|WF_DRAW_TEXT_LABEL_WITH_SPACE));
              pBuf->string16->fgcol = *get_theme_color(COLOR_THEME_DIPLODLG_MEETING_TEXT);
@@ -668,16 +674,18 @@ static struct ADVANCED_DLG * popup_diplomatic_objects(struct player *pPlayer0,
 	     break;
       }
     } advance_index_iterate_end;
-    
-    if(flag > A_NONE) {
+
+    if (flag > A_NONE) {
       advance_index_iterate(flag, i) {
-	if (player_invention_state(pPlayer0, i) == TECH_KNOWN &&
-	   player_invention_reachable(pPlayer1, i, FALSE) &&
-	  (player_invention_state(pPlayer1, i) == TECH_UNKNOWN || 
-	   player_invention_state(pPlayer1, i) == TECH_PREREQS_KNOWN)) {
-	     
+        if (research_invention_state(presearch0, i) == TECH_KNOWN
+            && research_invention_gettable(presearch1, i,
+                                           game.info.tech_trade_allow_holes)
+            && (research_invention_state(presearch1, i) == TECH_UNKNOWN
+                || research_invention_state(presearch1, i)
+                   == TECH_PREREQS_KNOWN)) {
+
 	     fc_snprintf(cBuf, sizeof(cBuf), "  %s", advance_name_translation(advance_by_number(i)));
-  
+
              pBuf = create_iconlabel_from_chars(NULL, pWindow->dst, cBuf, adj_font(12),
 	         (WF_RESTORE_BACKGROUND|WF_DRAW_TEXT_LABEL_WITH_SPACE));
              pBuf->string16->fgcol = *get_theme_color(COLOR_THEME_DIPLODLG_MEETING_TEXT);
@@ -729,24 +737,24 @@ static struct ADVANCED_DLG * popup_diplomatic_objects(struct player *pPlayer0,
       height = MAX(height, pBuf->size.h);
       add_to_gui_list(ID_LABEL, pBuf);
       count++;
-      
+
       qsort(city_list_ptrs, i, sizeof(struct city *), city_name_compare);
-        
+
       for (j = 0; j < i; j++) {
-	fc_snprintf(cBuf, sizeof(cBuf), "  %s", city_name(city_list_ptrs[j]));
-  
+        fc_snprintf(cBuf, sizeof(cBuf), "  %s", city_name_get(city_list_ptrs[j]));
+
         pBuf = create_iconlabel_from_chars(NULL, pWindow->dst, cBuf, adj_font(12),
 	     (WF_RESTORE_BACKGROUND|WF_DRAW_TEXT_LABEL_WITH_SPACE));
         pBuf->string16->fgcol = *get_theme_color(COLOR_THEME_DIPLODLG_MEETING_TEXT);
-	width = MAX(width, pBuf->size.w);
-	height = MAX(height, pBuf->size.h);
+        width = MAX(width, pBuf->size.w);
+        height = MAX(height, pBuf->size.h);
         pBuf->data.cont = pCont;
-	pBuf->action = cities_callback;
-	set_wstate(pBuf, FC_WS_NORMAL);
-	/* MAX_ID is unigned short type range and city id must be in this range */
+        pBuf->action = cities_callback;
+        set_wstate(pBuf, FC_WS_NORMAL);
+        /* MAX_ID is unigned short type range and city id must be in this range */
         fc_assert(city_list_ptrs[j]->id <= MAX_ID);
         add_to_gui_list(MAX_ID - city_list_ptrs[j]->id, pBuf);
-	count++;      
+        count++;
       }
     }
     FC_FREE(city_list_ptrs);    
@@ -881,37 +889,37 @@ static void update_diplomacy_dialog(struct diplomacy_dialog *pdialog)
     pStr = create_str16_from_char(nation_adjective_for_player(pPlayer0), adj_font(12));
     pStr->style |= (TTF_STYLE_BOLD|SF_CENTER);
     pStr->fgcol = *get_theme_color(COLOR_THEME_DIPLODLG_MEETING_TEXT);
-    
+
     pBuf = create_iconlabel(
-    	create_icon_from_theme(pTheme->CANCEL_PACT_Icon, 0),
+    	create_icon_from_theme(current_theme->CANCEL_PACT_Icon, 0),
 		pWindow->dst, pStr,
 		(WF_ICON_ABOVE_TEXT|WF_FREE_PRIVATE_DATA|WF_FREE_THEME|
 						WF_RESTORE_BACKGROUND));
-						
+
     pBuf->private_data.cbox = fc_calloc(1, sizeof(struct CHECKBOX));
     pBuf->private_data.cbox->state = FALSE;
-    pBuf->private_data.cbox->pTRUE_Theme = pTheme->OK_PACT_Icon;
-    pBuf->private_data.cbox->pFALSE_Theme = pTheme->CANCEL_PACT_Icon;
-    
+    pBuf->private_data.cbox->pTRUE_Theme = current_theme->OK_PACT_Icon;
+    pBuf->private_data.cbox->pFALSE_Theme = current_theme->CANCEL_PACT_Icon;
+
     add_to_gui_list(ID_ICON, pBuf);
     
     pStr = create_str16_from_char(nation_adjective_for_player(pPlayer1), adj_font(12));
     pStr->style |= (TTF_STYLE_BOLD|SF_CENTER);
     pStr->fgcol = *get_theme_color(COLOR_THEME_DIPLODLG_MEETING_TEXT);
-    
+
     pBuf = create_iconlabel(
-    	create_icon_from_theme(pTheme->CANCEL_PACT_Icon, 0),
-		pWindow->dst, pStr,
-		(WF_ICON_ABOVE_TEXT|WF_FREE_PRIVATE_DATA|WF_FREE_THEME|
-    						WF_RESTORE_BACKGROUND));
+    	create_icon_from_theme(current_theme->CANCEL_PACT_Icon, 0),
+       	pWindow->dst, pStr,
+        (WF_ICON_ABOVE_TEXT|WF_FREE_PRIVATE_DATA|WF_FREE_THEME|
+         WF_RESTORE_BACKGROUND));
     pBuf->private_data.cbox = fc_calloc(1, sizeof(struct CHECKBOX));
     pBuf->private_data.cbox->state = FALSE;
-    pBuf->private_data.cbox->pTRUE_Theme = pTheme->OK_PACT_Icon;
-    pBuf->private_data.cbox->pFALSE_Theme = pTheme->CANCEL_PACT_Icon;
+    pBuf->private_data.cbox->pTRUE_Theme = current_theme->OK_PACT_Icon;
+    pBuf->private_data.cbox->pFALSE_Theme = current_theme->CANCEL_PACT_Icon;
     add_to_gui_list(ID_ICON, pBuf);
     /* ============================================================= */
-    
-    pBuf = create_themeicon(pTheme->CANCEL_PACT_Icon, pWindow->dst,
+
+    pBuf = create_themeicon(current_theme->CANCEL_PACT_Icon, pWindow->dst,
                             WF_WIDGET_HAS_INFO_LABEL
                             | WF_RESTORE_BACKGROUND);
     pBuf->info_label = create_str16_from_char(_("Cancel meeting"),
@@ -921,8 +929,8 @@ static void update_diplomacy_dialog(struct diplomacy_dialog *pdialog)
     set_wstate(pBuf, FC_WS_NORMAL);
     
     add_to_gui_list(ID_ICON, pBuf);
-    
-    pBuf = create_themeicon(pTheme->OK_PACT_Icon, pWindow->dst,
+
+    pBuf = create_themeicon(current_theme->OK_PACT_Icon, pWindow->dst,
                             WF_FREE_DATA | WF_WIDGET_HAS_INFO_LABEL
                             | WF_RESTORE_BACKGROUND);
     pBuf->info_label = create_str16_from_char(_("Accept treaty"),
@@ -1387,8 +1395,9 @@ static void popup_war_dialog(struct player *pPlayer)
   area.h += pText->h + adj_size(10);
 
 
-  pBuf = create_themeicon_button_from_chars(pTheme->CANCEL_Icon,
-			    pWindow->dst, _("No"), adj_font(12), 0);
+  pBuf = create_themeicon_button_from_chars(current_theme->CANCEL_Icon,
+                                            pWindow->dst, _("No"),
+                                            adj_font(12), 0);
 
   pBuf->action = cancel_sdip_dlg_callback;
   set_wstate(pBuf, FC_WS_NORMAL);
@@ -1397,8 +1406,8 @@ static void popup_war_dialog(struct player *pPlayer)
 
   add_to_gui_list(ID_BUTTON, pBuf);
 
-  pBuf = create_themeicon_button_from_chars(pTheme->OK_Icon, pWindow->dst,
-					      _("Yes"), adj_font(12), 0);
+  pBuf = create_themeicon_button_from_chars(current_theme->OK_Icon, pWindow->dst,
+                                            _("Yes"), adj_font(12), 0);
 
   pBuf->action = cancel_pact_dlg_callback;
   set_wstate(pBuf, FC_WS_NORMAL);
@@ -1537,7 +1546,7 @@ void popup_diplomacy_dialog(struct player *pPlayer)
       }
 
       /* cancel treaty */
-      pBuf = create_themeicon_button_from_chars(pTheme->UNITS2_Icon,
+      pBuf = create_themeicon_button_from_chars(current_theme->UNITS2_Icon,
                                                 pWindow->dst, cBuf, adj_font(12), 0);
 
       pBuf->action = cancel_pact_dlg_callback;
@@ -1557,8 +1566,9 @@ void popup_diplomacy_dialog(struct player *pPlayer)
 
     if (shared) {
       /* shared vision */
-      pBuf = create_themeicon_button_from_chars(pTheme->UNITS2_Icon, pWindow->dst,
-                                                _("Withdraw vision"), adj_font(12), 0);
+      pBuf = create_themeicon_button_from_chars(current_theme->UNITS2_Icon,
+                                                pWindow->dst, _("Withdraw vision"),
+                                                adj_font(12), 0);
 
       pBuf->action = withdraw_vision_dlg_callback;
       set_wstate(pBuf, FC_WS_NORMAL);
@@ -1574,8 +1584,10 @@ void popup_diplomacy_dialog(struct player *pPlayer)
     }
 
     /* meet */
-    pBuf = create_themeicon_button_from_chars(pTheme->PLAYERS_Icon, pWindow->dst,
-					      _("Call Diplomatic Meeting"), adj_font(12), 0);
+    pBuf = create_themeicon_button_from_chars(current_theme->PLAYERS_Icon,
+                                              pWindow->dst,
+                                              _("Call Diplomatic Meeting"),
+                                              adj_font(12), 0);
 
     pBuf->action = call_meeting_dlg_callback;
     set_wstate(pBuf, FC_WS_NORMAL);
@@ -1589,8 +1601,9 @@ void popup_diplomacy_dialog(struct player *pPlayer)
     button_h = MAX(button_h , pBuf->size.h);
     buttons++;
 
-    pBuf = create_themeicon_button_from_chars(pTheme->CANCEL_Icon,
-			    pWindow->dst, _("Send him back"), adj_font(12), 0);
+    pBuf = create_themeicon_button_from_chars(current_theme->CANCEL_Icon,
+                                              pWindow->dst, _("Send him back"),
+                                              adj_font(12), 0);
 
     pBuf->action = cancel_sdip_dlg_callback;
     set_wstate(pBuf, FC_WS_NORMAL);
