@@ -22,7 +22,8 @@
 #ifndef FC__GUI_TILESPEC_H
 #define FC__GUI_TILESPEC_H
 
-#include "SDL/SDL.h"
+/* SDL */
+#include <SDL/SDL.h>
 
 #include "tilespec.h"
 
@@ -185,7 +186,7 @@ struct City_Icon {
   SDL_Surface *specialists[SP_MAX];
 };
 
-extern struct Theme *pTheme;
+extern struct Theme *current_theme;
 extern struct City_Icon *pIcons;
 
 void tilespec_setup_theme(void);
