@@ -22,7 +22,8 @@
 #include <fc_config.h>
 #endif
 
-#include "SDL/SDL.h"
+/* SDL */
+#include <SDL/SDL.h>
 
 /* utility */
 #include "fcintl.h"
