@@ -26,7 +26,8 @@
 #include <stdarg.h>
 #include <stdlib.h>
 
-#include "SDL/SDL.h"
+/* SDL */
+#include <SDL/SDL.h>
 
 /* utility */
 #include "fcintl.h"
@@ -435,24 +436,6 @@ static int none_callback(struct widget *widget)
   return -1;
 }
 
-/****************************************************************************
-  Convert a video mode to string. Returns TRUE on success.
-****************************************************************************/
-static inline bool video_mode_to_string(char *buf, size_t buf_len,
-                                        struct video_mode mode)
-{
-  return (2 < fc_snprintf(buf, buf_len, "%dx%d", mode.width, mode.height));
-}
-
-/****************************************************************************
-  Convert a string to video mode. Returns TRUE on success.
-****************************************************************************/
-static inline bool string_to_video_mode(const char *buf,
-                                        struct video_mode *mode)
-{
-  return (2 == sscanf(buf, "%dx%d", &mode->width, &mode->height));
-}
-
 /****************************************************************************
   Return a string vector containing all video modes.
 ****************************************************************************/
@@ -464,8 +447,9 @@ static struct strvec *video_mode_list(void)
   char buf[64];
 
   for (; NULL != *mode; mode++) {
-    if (video_mode_to_string(buf, sizeof(buf),
-                             video_mode((*mode)->w, (*mode)->h))) {
+    struct video_mode vmode = {.width = (*mode)->w, .height = (*mode)->h };
+
+    if (video_mode_to_string(buf, sizeof(buf), &vmode)) {
       strvec_append(video_modes, buf);
     }
   }
@@ -577,12 +561,11 @@ static struct widget *option_widget_new(struct option *poption,
   case OT_VIDEO_MODE:
     {
       char buf[64];
+      struct video_mode vmode = option_video_mode_get(poption);
 
-      if (!video_mode_to_string(buf, sizeof(buf),
-                                option_video_mode_get(poption))) {
+      if (!video_mode_to_string(buf, sizeof(buf), &vmode)) {
         /* Always fails. */
-        fc_assert(video_mode_to_string(buf, sizeof(buf),
-                                       option_video_mode_get(poption)));
+        fc_assert(video_mode_to_string(buf, sizeof(buf), &vmode));
       }
 
       widget = combo_new_from_chars(NULL, window->dst, adj_font(12),
@@ -663,14 +646,13 @@ static void option_widget_update(struct option *poption)
   case OT_VIDEO_MODE:
     {
       char buf[64];
+      struct video_mode vmode = option_video_mode_get(poption);
 
-      if (video_mode_to_string(buf, sizeof(buf),
-                               option_video_mode_get(poption))) {
+      if (video_mode_to_string(buf, sizeof(buf), &vmode)) {
         copy_chars_to_string16(widget->string16, buf);
       } else {
         /* Always fails. */
-        fc_assert(video_mode_to_string(buf, sizeof(buf),
-                                       option_video_mode_get(poption)));
+        fc_assert(video_mode_to_string(buf, sizeof(buf), &vmode));
       }
     }
     break;
@@ -790,7 +772,7 @@ static struct option_dialog *option_dialog_new(void)
   pdialog->end_widget_list = window;
 
   /* Create close button widget. */
-  close_button = create_themeicon(pTheme->Small_CANCEL_Icon, window->dst,
+  close_button = create_themeicon(current_theme->Small_CANCEL_Icon, window->dst,
                                   WF_WIDGET_HAS_INFO_LABEL
                                   | WF_RESTORE_BACKGROUND);
   close_button->info_label = create_str16_from_char(_("Close Dialog (Esc)"),
@@ -1010,7 +992,7 @@ static void option_dialog_optset_category(struct option_dialog *pdialog,
   window = pdialog->end_widget_list;
 
   /* Create the apply button. */
-  apply_button = create_themeicon(pTheme->Small_OK_Icon, window->dst,
+  apply_button = create_themeicon(current_theme->Small_OK_Icon, window->dst,
                                   WF_WIDGET_HAS_INFO_LABEL
                                   | WF_RESTORE_BACKGROUND);
   apply_button->info_label = create_str16_from_char(_("Apply changes"),
@@ -1340,8 +1322,8 @@ void disable_options_button(void)
 void init_options_button(void)
 {
   char buf[256];
-  
-  pOptions_Button = create_themeicon(pTheme->Options_Icon, Main.gui,
+
+  pOptions_Button = create_themeicon(current_theme->Options_Icon, Main.gui,
                                      WF_WIDGET_HAS_INFO_LABEL
                                      | WF_RESTORE_BACKGROUND);
   pOptions_Button->action = optiondlg_callback;
