@@ -15,7 +15,8 @@
 #include <fc_config.h>
 #endif
 
-#include "SDL/SDL.h"
+/* SDL */
+#include <SDL/SDL.h>
 
 /* utility */
 #include "astring.h"
@@ -346,7 +347,7 @@ void popup_players_dialog(bool raise)
   pPlayers_Dlg->pEndWidgetList = pWindow;
   /* ---------- */
   /* exit button */
-  pBuf = create_themeicon(pTheme->Small_CANCEL_Icon, pWindow->dst,
+  pBuf = create_themeicon(current_theme->Small_CANCEL_Icon, pWindow->dst,
                           WF_WIDGET_HAS_INFO_LABEL | WF_RESTORE_BACKGROUND);
   pBuf->info_label = create_str16_from_char(_("Close Dialog (Esc)"),
                                             adj_font(12));
@@ -499,7 +500,7 @@ void popup_players_dialog(bool raise)
         break;
       }
       
-      copy_chars_to_string16(pStr, diplstate_text(i));
+      copy_chars_to_string16(pStr, diplstate_type_translated_name(i));
       pLogo = create_text_surf_from_str16(pStr);
   
       pBuf = pBuf->prev;
@@ -626,10 +627,10 @@ void popup_players_nations_dialog(void)
   pShort_Players_Dlg->pEndWidgetList = pWindow;
   
   area = pWindow->area;
-  
+
   /* ---------- */
   /* exit button */
-  pBuf = create_themeicon(pTheme->Small_CANCEL_Icon, pWindow->dst,
+  pBuf = create_themeicon(current_theme->Small_CANCEL_Icon, pWindow->dst,
                           WF_WIDGET_HAS_INFO_LABEL | WF_RESTORE_BACKGROUND);
   pBuf->info_label = create_str16_from_char(_("Close Dialog (Esc)"),
                                             adj_font(12));
