@@ -14,7 +14,8 @@
 #ifndef FC__THEMEBACKGROUNDS_H
 #define FC__THEMEBACKGROUNDS_H
 
-#include "SDL/SDL.h"
+/* SDL */
+#include <SDL/SDL.h>
 
 #include "registry.h"
 
