@@ -23,7 +23,8 @@
 #include <fc_config.h>
 #endif
 
-#include "SDL/SDL.h"
+/* SDL */
+#include <SDL/SDL.h>
 
 #include "log.h"
 
@@ -57,18 +58,18 @@ static SDL_Surface *pInfo_Label = NULL;
   Correct backgroud size ( set min size ). Used in create widget
   functions.
 **************************************************************************/
-void correct_size_bcgnd_surf(SDL_Surface * pTheme,
-				    Uint16 * pWidth, Uint16 * pHigh)
+void correct_size_bcgnd_surf(SDL_Surface *ptheme,
+                             Uint16 *width, Uint16 *height)
 {
-  *pWidth = MAX(*pWidth, 2 * (pTheme->w / adj_size(16)));
-  *pHigh = MAX(*pHigh, 2 * (pTheme->h / adj_size(16)));
+  *width = MAX(*width, 2 * (ptheme->w / adj_size(16)));
+  *height = MAX(*height, 2 * (ptheme->h / adj_size(16)));
 }
 
 /**************************************************************************
   Create background image for buttons, iconbuttons and edit fields
   then return  pointer to this image.
 
-  Graphic is taken from pTheme surface and blit to new created image.
+  Graphic is taken from ptheme surface and blit to new created image.
 
   Length and height depend of iText_with, iText_high parameters.
 
@@ -78,46 +79,45 @@ void correct_size_bcgnd_surf(SDL_Surface * pTheme,
     state = 2 - pressed
     state = 3 - disabled
 **************************************************************************/
-SDL_Surface *create_bcgnd_surf(SDL_Surface * pTheme, Uint8 state,
-                               Uint16 Width, Uint16 High)
+SDL_Surface *create_bcgnd_surf(SDL_Surface *ptheme, Uint8 state,
+                               Uint16 width, Uint16 height)
 {
   bool zoom;
   int iTile_width_len_end, iTile_width_len_mid, iTile_count_len_mid;
-  int iTile_width_high_end, iTile_width_high_mid, iTile_count_high_mid;
+  int iTile_width_height_end, iTile_width_height_mid, iTile_count_height_mid;
   int i, j;
-
   SDL_Rect src, des;
   SDL_Surface *pBackground = NULL;
 
-  int iStart_y = (pTheme->h / 4) * state;
+  int iStart_y = (ptheme->h / 4) * state;
 
-  iTile_width_len_end = pTheme->w / 16;
-  iTile_width_len_mid = pTheme->w - (iTile_width_len_end * 2);
+  iTile_width_len_end = ptheme->w / 16;
+  iTile_width_len_mid = ptheme->w - (iTile_width_len_end * 2);
 
   iTile_count_len_mid =
-      (Width - (iTile_width_len_end * 2)) / iTile_width_len_mid;
+      (width - (iTile_width_len_end * 2)) / iTile_width_len_mid;
 
   /* corrections I */
   if (((iTile_count_len_mid *
-	iTile_width_len_mid) + (iTile_width_len_end * 2)) < Width) {
+	iTile_width_len_mid) + (iTile_width_len_end * 2)) < width) {
     iTile_count_len_mid++;
   }
 
-  iTile_width_high_end = pTheme->h / 16;
-  iTile_width_high_mid = (pTheme->h / 4) - (iTile_width_high_end * 2);
-  iTile_count_high_mid =
-      (High - (iTile_width_high_end * 2)) / iTile_width_high_mid;
+  iTile_width_height_end = ptheme->h / 16;
+  iTile_width_height_mid = (ptheme->h / 4) - (iTile_width_height_end * 2);
+  iTile_count_height_mid =
+      (height - (iTile_width_height_end * 2)) / iTile_width_height_mid;
 
   /* corrections II */
-  if (((iTile_count_high_mid *
-	iTile_width_high_mid) + (iTile_width_high_end * 2)) < High) {
-    iTile_count_high_mid++;
+  if (((iTile_count_height_mid *
+	iTile_width_height_mid) + (iTile_width_height_end * 2)) < height) {
+    iTile_count_height_mid++;
   }
 
-  i = MAX(iTile_width_len_end * 2, Width);
-  j = MAX(iTile_width_high_end * 2, High);
-  zoom = ((i != Width) ||  (j != High));
-  
+  i = MAX(iTile_width_len_end * 2, width);
+  j = MAX(iTile_width_height_end * 2, height);
+  zoom = ((i != width) ||  (j != height));
+
   /* now allocate memory */
   pBackground = create_surf_alpha(i, j, SDL_SWSURFACE);
 
@@ -127,26 +127,26 @@ SDL_Surface *create_bcgnd_surf(SDL_Surface * pTheme, Uint8 state,
   src.x = 0;
   src.y = iStart_y;
   src.w = iTile_width_len_end;
-  src.h = iTile_width_high_end;
+  src.h = iTile_width_height_end;
 
   des.x = 0;
   des.y = 0;
-  alphablit(pTheme, &src, pBackground, &des);
+  alphablit(ptheme, &src, pBackground, &des);
 
-  /* copy left middels parts */
-  src.y = iStart_y + iTile_width_high_end;
-  src.h = iTile_width_high_mid;
-  for (i = 0; i < iTile_count_high_mid; i++) {
-    des.y = iTile_width_high_end + i * iTile_width_high_mid;
-    alphablit(pTheme, &src, pBackground, &des);
+  /* copy left middle parts */
+  src.y = iStart_y + iTile_width_height_end;
+  src.h = iTile_width_height_mid;
+  for (i = 0; i < iTile_count_height_mid; i++) {
+    des.y = iTile_width_height_end + i * iTile_width_height_mid;
+    alphablit(ptheme, &src, pBackground, &des);
   }
 
-  /* copy left boton end */
-  src.y = iStart_y + ((pTheme->h / 4) - iTile_width_high_end);
-  src.h = iTile_width_high_end;
-  des.y = pBackground->h - iTile_width_high_end;
+  /* copy left bottom end */
+  src.y = iStart_y + ((ptheme->h / 4) - iTile_width_height_end);
+  src.h = iTile_width_height_end;
+  des.y = pBackground->h - iTile_width_height_end;
   clear_surface(pBackground, &des);
-  alphablit(pTheme, &src, pBackground, &des);
+  alphablit(ptheme, &src, pBackground, &des);
 
   /* copy middle parts without right end part */
 
@@ -160,56 +160,56 @@ SDL_Surface *create_bcgnd_surf(SDL_Surface * pTheme, Uint8 state,
     des.x = iTile_width_len_end + i * iTile_width_len_mid;
     des.y = 0;
     src.y = iStart_y;
-    alphablit(pTheme, &src, pBackground, &des);
+    alphablit(ptheme, &src, pBackground, &des);
 
-    /*  middels */
-    src.y = iStart_y + iTile_width_high_end;
-    src.h = iTile_width_high_mid;
-    for (j = 0; j < iTile_count_high_mid; j++) {
-      des.y = iTile_width_high_end + j * iTile_width_high_mid;
-      alphablit(pTheme, &src, pBackground, &des);
+    /* middle */
+    src.y = iStart_y + iTile_width_height_end;
+    src.h = iTile_width_height_mid;
+    for (j = 0; j < iTile_count_height_mid; j++) {
+      des.y = iTile_width_height_end + j * iTile_width_height_mid;
+      alphablit(ptheme, &src, pBackground, &des);
     }
 
     /* bottom */
-    src.y = iStart_y + ((pTheme->h / 4) - iTile_width_high_end);
-    src.h = iTile_width_high_end;
-    des.y = pBackground->h - iTile_width_high_end;
-    clear_surface(pBackground, &des);    
-    alphablit(pTheme, &src, pBackground, &des);
+    src.y = iStart_y + ((ptheme->h / 4) - iTile_width_height_end);
+    src.h = iTile_width_height_end;
+    des.y = pBackground->h - iTile_width_height_end;
+    clear_surface(pBackground, &des);
+    alphablit(ptheme, &src, pBackground, &des);
   }
 
   /* copy right end */
-  src.x = pTheme->w - iTile_width_len_end;
+  src.x = ptheme->w - iTile_width_len_end;
   src.y = iStart_y;
   src.w = iTile_width_len_end;
 
   des.x = pBackground->w - iTile_width_len_end;
   des.y = 0;
 
-  alphablit(pTheme, &src, pBackground, &des);
+  alphablit(ptheme, &src, pBackground, &des);
 
-  /*  middels */
-  src.y = iStart_y + iTile_width_high_end;
-  src.h = iTile_width_high_mid;
-  for (i = 0; i < iTile_count_high_mid; i++) {
-    des.y = iTile_width_high_end + i * iTile_width_high_mid;
-    alphablit(pTheme, &src, pBackground, &des);
+  /* middle */
+  src.y = iStart_y + iTile_width_height_end;
+  src.h = iTile_width_height_mid;
+  for (i = 0; i < iTile_count_height_mid; i++) {
+    des.y = iTile_width_height_end + i * iTile_width_height_mid;
+    alphablit(ptheme, &src, pBackground, &des);
   }
 
-  /*boton */
-  src.y = iStart_y + ((pTheme->h / 4) - iTile_width_high_end);
-  src.h = iTile_width_high_end;
-  des.y = pBackground->h - iTile_width_high_end;
-  clear_surface(pBackground, &des);  
-  alphablit(pTheme, &src, pBackground, &des);
-  
-  if (zoom)
-  {
-    SDL_Surface *pZoom = ResizeSurface(pBackground, Width, High, 1);
+  /* bottom */
+  src.y = iStart_y + ((ptheme->h / 4) - iTile_width_height_end);
+  src.h = iTile_width_height_end;
+  des.y = pBackground->h - iTile_width_height_end;
+  clear_surface(pBackground, &des);
+  alphablit(ptheme, &src, pBackground, &des);
+
+  if (zoom) {
+    SDL_Surface *pZoom = ResizeSurface(pBackground, width, height, 1);
+
     FREESURFACE(pBackground);
     pBackground = pZoom;
   }
-  
+
   return pBackground;
 }
 
@@ -1101,23 +1101,22 @@ int setup_vertical_widgets_position(int step,
 /**************************************************************************
   Draw Themed Frame.
 **************************************************************************/
-void draw_frame(SDL_Surface * pDest, Sint16 start_x, Sint16 start_y,
-		Uint16 w, Uint16 h)
+void draw_frame(SDL_Surface *pDest, Sint16 start_x, Sint16 start_y,
+                Uint16 w, Uint16 h)
 {
   SDL_Surface *pTmpLeft = 
-    ResizeSurface(pTheme->FR_Left, pTheme->FR_Left->w, h, 1);
+    ResizeSurface(current_theme->FR_Left, current_theme->FR_Left->w, h, 1);
   SDL_Surface *pTmpRight = 
-    ResizeSurface(pTheme->FR_Right, pTheme->FR_Right->w, h, 1);
+    ResizeSurface(current_theme->FR_Right, current_theme->FR_Right->w, h, 1);
   SDL_Surface *pTmpTop =
-    ResizeSurface(pTheme->FR_Top, w, pTheme->FR_Top->h, 1);
+    ResizeSurface(current_theme->FR_Top, w, current_theme->FR_Top->h, 1);
   SDL_Surface *pTmpBottom =
-    ResizeSurface(pTheme->FR_Bottom, w, pTheme->FR_Bottom->h, 1);
-  
+    ResizeSurface(current_theme->FR_Bottom, w, current_theme->FR_Bottom->h, 1);
   SDL_Rect tmp,dst = {start_x, start_y, 0, 0};
 
   tmp = dst;
   alphablit(pTmpLeft, NULL, pDest, &tmp);
-  
+
   dst.x += w - pTmpRight->w;
   tmp = dst;
   alphablit(pTmpRight, NULL, pDest, &tmp);
