@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -112,33 +112,14 @@ void popup_revolution_dialog(void)
 }
 
 /**************************************************************************
-  Popup a dialog giving a player choices when their caravan arrives at
-  a city (other than its home city).  Example:
-    - Establish trade route.
-    - Help build wonder.
-    - Keep moving.
+  Popup a dialog that allows the player to select what action a unit
+  should take.
 **************************************************************************/
-void popup_caravan_dialog(struct unit *punit,
-			  struct city *phomecity, struct city *pdestcity)
-{
-  /* PORTME */
-}
-
-/**************************************************************************
-  Is there currently a caravan dialog open?  This is important if there
-  can be only one such dialog at a time; otherwise return FALSE.
-**************************************************************************/
-bool caravan_dialog_is_open(int *unit_id, int *city_id)
-{
-  /* PORTME */
-  return FALSE;
-}
-
-/**************************************************************************
-  Popup a dialog giving a diplomatic unit some options when moving into
-  the target tile.
-**************************************************************************/
-void popup_diplomat_dialog(struct unit *punit, struct tile *ptile)
+void popup_action_selection(struct unit *actor_unit,
+                                   struct city *target_city,
+                                   struct unit *target_unit,
+                                   struct tile *target_tile,
+                                   const struct act_prob *act_probs)
 {
   /* PORTME */
 }
@@ -174,8 +155,7 @@ void popup_sabotage_dialog(struct unit *actor, struct city *pcity)
   Popup a dialog asking the unit which improvement they would like to
   pillage.
 **************************************************************************/
-void popup_pillage_dialog(struct unit *punit, bv_special may_pillage,
-                          bv_bases bases, bv_roads roads)
+void popup_pillage_dialog(struct unit *punit, bv_extras may_pillage)
 {
   /* PORTME */
 }
@@ -206,6 +186,15 @@ void popup_soundset_suggestion_dialog(void)
   /* PORTME */
 }
 
+/****************************************************************
+  Ruleset (modpack) has suggested loading certain musicset. Confirm from
+  user and load.
+*****************************************************************/
+void popup_musicset_suggestion_dialog(void)
+{
+  /* PORTME */
+}
+
 /**************************************************************************
   Tileset (modpack) has suggested loading certain theme. Confirm from
   user and load.
@@ -225,27 +214,68 @@ void popdown_all_game_dialogs(void)
   /* PORTME */
 }
 
-/****************************************************************
-  Returns id of a diplomat currently handled in diplomat dialog
-*****************************************************************/
-int diplomat_handled_in_diplomat_dialog(void)
+/**************************************************************************
+  Returns the id of the actor unit currently handled in action selection
+  dialog when the action selection dialog is open.
+  Returns IDENTITY_NUMBER_ZERO if no action selection dialog is open.
+**************************************************************************/
+int action_selection_actor_unit(void)
 {
   /* PORTME */    
-  return -1;  
+  return IDENTITY_NUMBER_ZERO;
+}
+
+/**************************************************************************
+  Returns id of the target city of the actions currently handled in action
+  selection dialog when the action selection dialog is open and it has a
+  city target. Returns IDENTITY_NUMBER_ZERO if no action selection dialog
+  is open or no city target is present in the action selection dialog.
+**************************************************************************/
+int action_selection_target_city(void)
+{
+  /* PORTME */
+  return IDENTITY_NUMBER_ZERO;
+}
+
+/**************************************************************************
+  Returns id of the target unit of the actions currently handled in action
+  selection dialog when the action selection dialog is open and it has a
+  unit target. Returns IDENTITY_NUMBER_ZERO if no action selection dialog
+  is open or no unit target is present in the action selection dialog.
+**************************************************************************/
+int action_selection_target_unit(void)
+{
+  /* PORTME */
+  return IDENTITY_NUMBER_ZERO;
+}
+
+/**************************************************************************
+  Updates the action selection dialog with new information.
+**************************************************************************/
+void action_selection_refresh(struct unit *actor_unit,
+                              struct city *target_city,
+                              struct unit *target_unit,
+                              struct tile *target_tile,
+                              const struct act_prob *act_probs)
+{
+  /* TODO: port me. */
 }
 
 /****************************************************************
-  Closes the diplomat dialog
+  Closes the action selection dialog
 ****************************************************************/
-void close_diplomat_dialog(void)
+void action_selection_close(void)
 {
   /* PORTME */
 }
 
-/****************************************************************
-  Updates caravan dialog
-****************************************************************/
-void caravan_dialog_update(void)
+/**************************************************************************
+  Let the non shared client code know that the action selection process
+  no longer is in progress for the specified unit.
+
+  This allows the client to clean up any client specific assumptions.
+**************************************************************************/
+void action_selection_no_longer_in_progress_gui_specific(int actor_id)
 {
   /* PORTME */
 }
@@ -265,3 +295,31 @@ void show_tileset_error(const char *msg)
 {
   /* PORTME */
 }
+
+/****************************************************************
+  Give a warning when user is about to edit scenario with manually
+  set properties.
+*****************************************************************/
+bool gui_handmade_scenario_warning(void)
+{
+  /* Just tell the client common code to handle this. */
+  return FALSE;
+}
+
+/****************************************************************
+  Unit wants to get into some transport on given tile.
+*****************************************************************/
+bool gui_request_transport(struct unit *pcargo, struct tile *ptile)
+{
+  return FALSE; /* Unit was not handled here. */
+}
+
+/***************************************************************************
+  Popup detailed information about battle or save information for
+  some kind of statistics
+***************************************************************************/
+void gui_popup_combat_info(int attacker_unit_id, int defender_unit_id,
+                           int attacker_hp, int defender_hp,
+                           bool make_winner_veteran)
+{
+}
