@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 #include <stdio.h>
 
 /* utility */
+#include "fc_cmdline.h"
 #include "fciconv.h"
 #include "log.h"
 
@@ -28,7 +29,6 @@
 #include "gui_cbsetter.h"
 #include "client_main.h"
 #include "editgui_g.h"
-#include "ggz_g.h"
 #include "options.h"
 
 #include "gui_main.h"
@@ -46,7 +46,7 @@ void gui_set_city_names_font_sizes(int my_city_names_font_size,
                                    int my_city_productions_font_size)
 {
   log_error("Unimplemented set_city_names_font_sizes.");
-  /* PORTME */
+  /* PORTME. Optional to pay any attention to this. */
 }
 
 /**************************************************************************
@@ -120,7 +120,7 @@ void gui_ui_main(int argc, char *argv[])
 /****************************************************************************
   Extra initializers for client options.
 ****************************************************************************/
-void gui_gui_options_extra_init(void)
+void gui_options_extra_init(void)
 {
   /* Nothing to do. */
 }
@@ -180,23 +180,6 @@ void gui_remove_net_input(void)
   /* PORTME */
 }
 
-/**************************************************************************
-  Called to monitor a GGZ socket.
-**************************************************************************/
-void gui_add_ggz_input(int sock)
-{
-  /* PORTME */
-}
-
-/**************************************************************************
-  Called on disconnection to remove monitoring on the GGZ socket.  Only
-  call this if we're actually in GGZ mode.
-**************************************************************************/
-void gui_remove_ggz_input(void)
-{
-  /* PORTME */
-}
-
 /**************************************************************************
   Set one of the unit icons (specified by idx) in the information area
   based on punit.
@@ -276,7 +259,7 @@ void gui_editgui_popdown_all(void)
 /****************************************************************************
   Stub for editor function
 ****************************************************************************/
-void gui_editgui_notify_object_changed(int objtype, int object_id, bool remove)
+void gui_editgui_notify_object_changed(int objtype, int object_id, bool removal)
 {}
 
 /****************************************************************************
@@ -285,19 +268,6 @@ void gui_editgui_notify_object_changed(int objtype, int object_id, bool remove)
 void gui_editgui_notify_object_created(int tag, int id)
 {}
 
-/****************************************************************************
-  Stub for ggz function
-****************************************************************************/
-void gui_gui_ggz_embed_leave_table(void)
-{}
-
-/****************************************************************************
-  Stub for ggz function
-****************************************************************************/
-void gui_gui_ggz_embed_ensure_server(void)
-{}
-
-
 /**************************************************************************
   Updates a gui font style.
 **************************************************************************/
@@ -305,3 +275,19 @@ void gui_gui_update_font(const char *font_name, const char *font_value)
 {
   /* PORTME */
 }
+
+/**************************************************************************
+  Insert build information to help
+**************************************************************************/
+void gui_insert_client_build_info(char *outbuf, size_t outlen)
+{
+  /* PORTME */
+}
+
+/**************************************************************************
+  Make dynamic adjustments to first-launch default options.
+**************************************************************************/
+void gui_adjust_default_options(void)
+{
+  /* Nothing in case of this gui */
+}
