@@ -15,11 +15,14 @@
 #include <fc_config.h>
 #endif
 
-/* common & utility */
+/* utility */
 #include "fcintl.h"
-#include "game.h"
 #include "support.h"
 
+/* common */
+#include "calendar.h"
+#include "game.h"
+
 /* gui main header */
 #include "gui_stub.h"
 
@@ -47,7 +50,7 @@ void update_info_label(void)
                 "Gold %d\n"
                 "Tax: %d Lux: %d Sci: %d"),
               population_to_text(civ_population(client_player())),
-              textyear(game.info.year),
+              calendar_text(),
               client.conn.playing->economic.gold,
               client.conn.playing->economic.tax,
               client.conn.playing->economic.luxury,
