@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -538,25 +538,26 @@ struct city_dialog *create_city_dialog(struct city *pcity)
   Widget relative;
   enum citizen_category c = CITIZEN_SPECIALIST + DEFAULT_SPECIALIST;
 
-  if (tileset_tile_height(tileset)<45) dummy_improvement_list[5]=0;
+  if (tileset_tile_height(tileset) < 45) {
+    dummy_improvement_list[5] = 0;
+  }
 
-  if (concise_city_production) {
+  if (gui_options.concise_city_production) {
     dummy_improvement_list[0] = "XXXXXXXXXXXXXXXXXXXXXXXXXXXX";
   }
 
-  pdialog=fc_malloc(sizeof(struct city_dialog));
-  pdialog->pcity=pcity;
-  pdialog->support_unit_base=0;
-  pdialog->present_unit_base=0;
+  pdialog = fc_malloc(sizeof(struct city_dialog));
+  pdialog->pcity = pcity;
+  pdialog->support_unit_base = 0;
+  pdialog->present_unit_base = 0;
   pdialog->worklist_shell = NULL;
 
   pdialog->shell=
-    XtVaCreatePopupShell(city_name(pcity),
-/*			 make_modal ? transientShellWidgetClass :*/
-			 topLevelShellWidgetClass,
-			 toplevel, 
-			 XtNallowShellResize, True, 
-			 NULL);
+    XtVaCreatePopupShell(city_name_get(pcity),
+                         topLevelShellWidgetClass,
+                         toplevel, 
+                         XtNallowShellResize, True, 
+                         NULL);
 
   pdialog->main_form=
     XtVaCreateManagedWidget("citymainform", 
@@ -650,15 +651,15 @@ struct city_dialog *create_city_dialog(struct city *pcity)
 
   pdialog->building_label=
     XtVaCreateManagedWidget("citybuildinglabel",
-			    labelWidgetClass,
-			    pdialog->sub_form,
-			    XtNfromHoriz, 
-			    (XtArgVal)pdialog->map_canvas,
-			    XtNlabel,
-			    concise_city_production
-				? "XXXXXXXXXXXXXXXXXXXXXXXXXXXX"
-				: "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX",
-			    NULL);
+                            labelWidgetClass,
+                            pdialog->sub_form,
+                            XtNfromHoriz,
+                            (XtArgVal)pdialog->map_canvas,
+                            XtNlabel,
+                            gui_options.concise_city_production
+                                ? "XXXXXXXXXXXXXXXXXXXXXXXXXXXX"
+                                : "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX",
+                            NULL);
 
   get_contents_of_progress(NULL, lblbuf, sizeof(lblbuf));
   pdialog->progress_label=
@@ -1400,7 +1401,8 @@ void present_units_callback(Widget w, XtPointer client_data,
       XtSetSensitive(XtNameToWidget(wd, "*button5"), FALSE);
     }
     if (NULL == client.conn.playing
-	|| (NULL == can_upgrade_unittype(client.conn.playing, unit_type(punit)))) {
+        || (NULL == can_upgrade_unittype(client.conn.playing,
+                                         unit_type_get(punit)))) {
       XtSetSensitive(XtNameToWidget(wd, "*button6"), FALSE);
     }
   }
@@ -1432,13 +1434,13 @@ void rename_callback(Widget w, XtPointer client_data, XtPointer call_data)
   struct city_dialog *pdialog;
 
   pdialog=(struct city_dialog *)client_data;
-  
-  input_dialog_create(pdialog->shell, 
-		      "shellrenamecity", 
-		      _("What should we rename the city to?"),
-		      city_name(pdialog->pcity),
-		      rename_city_callback, (XtPointer)pdialog,
-		      rename_city_callback, (XtPointer)0);
+
+  input_dialog_create(pdialog->shell,
+                      "shellrenamecity", 
+                      _("What should we rename the city to?"),
+                      city_name_get(pdialog->pcity),
+                      rename_city_callback, (XtPointer)pdialog,
+                      rename_city_callback, (XtPointer)0);
 }
 
 /****************************************************************
@@ -1461,13 +1463,13 @@ void trade_callback(Widget w, XtPointer client_data, XtPointer call_data)
   int nleft = sizeof(buf);
   struct city_dialog *pdialog;
 
-  pdialog=(struct city_dialog *)client_data;
+  pdialog = (struct city_dialog *)client_data;
 
   fc_snprintf(buf, sizeof(buf),
-	      _("These trade routes have been established with %s:\n"),
-	      city_name(pdialog->pcity));
+              _("These trade routes have been established with %s:\n"),
+              city_name_get(pdialog->pcity));
   bptr = end_of_strn(bptr, &nleft);
-  
+
   for (i = 0; i < MAX_TRADE_ROUTES; i++)
     if(pdialog->pcity->trade[i]) {
       struct city *pcity;
@@ -1475,7 +1477,7 @@ void trade_callback(Widget w, XtPointer client_data, XtPointer call_data)
       total+=pdialog->pcity->trade_value[i];
       if ((pcity = game_city_by_number(pdialog->pcity->trade[i]))) {
 	fc_snprintf(bptr, nleft, _("%32s: %2d Trade/Year\n"),
-		    city_name(pcity), pdialog->pcity->trade_value[i]);
+                    city_name_get(pcity), pdialog->pcity->trade_value[i]);
 	bptr = end_of_strn(bptr, &nleft);
       } else {	
 	fc_snprintf(bptr, nleft, _("%32s: %2d Trade/Year\n"), _("Unknown"),
@@ -1591,14 +1593,14 @@ static void citizen_callback(Widget w, XtPointer client_data,
 *****************************************************************/
 void city_dialog_update_citizens(struct city_dialog *pdialog)
 {
-  enum citizen_category citizens[MAX_CITY_SIZE];
+  enum citizen_category categories[MAX_CITY_SIZE];
   int i;
   struct city *pcity=pdialog->pcity;
-  int num_citizens = get_city_citizen_types(pcity, FEELING_FINAL, citizens);
+  int num_citizens = get_city_citizen_types(pcity, FEELING_FINAL, categories);
 
   for (i = 0; i < num_citizens && i < pdialog->num_citizens_shown; i++) {
     xaw_set_bitmap(pdialog->citizen_labels[i],
-		   get_citizen_pixmap(citizens[i], i, pcity));
+		   get_citizen_pixmap(categories[i], i, pcity));
 
     /* HACK: set sensitivity/callbacks on the widget */
     XtRemoveAllCallbacks(pdialog->citizen_labels[i], XtNcallback);
@@ -1747,7 +1749,7 @@ void city_dialog_update_supported_units(struct city_dialog *pdialog,
       continue;
 
     XawPixcommClear(pixcomm); /* STG */
-    put_unit(punit, &store, 0, 0);
+    put_unit(punit, &store, 1.0, 0, 0);
     put_unit_pixmap_city_overlays(punit,
                                   XawPixcommPixmap(pixcomm),
                                   punit->upkeep, happy_cost);
@@ -1809,7 +1811,7 @@ void city_dialog_update_present_units(struct city_dialog *pdialog, int unitid)
       continue;
 
     XawPixcommClear(pixcomm); /* STG */
-    put_unit(punit, &store, 0, 0);
+    put_unit(punit, &store, 1.0, 0, 0);
 
     xaw_expose_now(pixcomm);
 
@@ -1834,17 +1836,18 @@ void city_dialog_update_title(struct city_dialog *pdialog)
 {
   char buf[512];
   String now;
-  
+
   fc_snprintf(buf, sizeof(buf), _("%s - %s citizens  Governor: %s"),
-	      city_name(pdialog->pcity),
-	      population_to_text(city_population(pdialog->pcity)),
-                   cmafec_get_short_descr_of_city(pdialog->pcity));
+              city_name_get(pdialog->pcity),
+              population_to_text(city_population(pdialog->pcity)),
+              cmafec_get_short_descr_of_city(pdialog->pcity));
 
   XtVaGetValues(pdialog->cityname_label, XtNlabel, &now, NULL);
-  if(strcmp(now, buf) != 0) {
+  if (strcmp(now, buf) != 0) {
     XtVaSetValues(pdialog->cityname_label, XtNlabel, (XtArgVal)buf, NULL);
     xaw_horiz_center(pdialog->cityname_label);
-    XtVaSetValues(pdialog->shell, XtNtitle, (XtArgVal)city_name(pdialog->pcity), NULL);
+    XtVaSetValues(pdialog->shell, XtNtitle,
+                  (XtArgVal)city_name_get(pdialog->pcity), NULL);
   }
 }
 
@@ -2057,27 +2060,27 @@ void disband_callback(Widget w, XtPointer client_data, XtPointer call_data)
 
 
 /****************************************************************
-...
+  Production change callback
 *****************************************************************/
 static void change_to_callback(Widget w, XtPointer client_data,
-			       XtPointer call_data)
+                               XtPointer call_data)
 {
   struct city_dialog *pdialog;
   XawListReturnStruct *ret;
 
-  pdialog=(struct city_dialog *)client_data;
+  pdialog = (struct city_dialog *)client_data;
 
-  ret=XawListShowCurrent(pdialog->change_list);
+  ret = XawListShowCurrent(pdialog->change_list);
 
   if (ret->list_index != XAW_LIST_NONE) {
     struct universal target =
       universal_by_number((ret->list_index >= pdialog->change_list_num_improvements)
-			     ? VUT_UTYPE : VUT_IMPROVEMENT,
-			     pdialog->change_list_ids[ret->list_index]);
+                          ? VUT_UTYPE : VUT_IMPROVEMENT,
+                          pdialog->change_list_ids[ret->list_index]);
 
-    city_change_production(pdialog->pcity, target);
+    city_change_production(pdialog->pcity, &target);
   }
-  
+
   XtDestroyWidget(XtParent(XtParent(w)));
   XtSetSensitive(pdialog->shell, TRUE);
 }
@@ -2227,19 +2230,18 @@ void change_callback(Widget w, XtPointer client_data, XtPointer call_data)
 
   n = 0;
   improvement_iterate(pimprove) {
-    if(can_city_build_improvement_now(pdialog->pcity, pimprove)) {
+    if (can_city_build_improvement_now(pdialog->pcity, pimprove)) {
       production.kind = VUT_IMPROVEMENT;
       production.value.building = pimprove;
       get_city_dialog_production_full(pdialog->change_list_names[n],
                                       sizeof(pdialog->change_list_names[n]),
-                                      production, pdialog->pcity);
-      pdialog->change_list_names_ptrs[n]=pdialog->change_list_names[n];
+                                      &production, pdialog->pcity);
+      pdialog->change_list_names_ptrs[n] = pdialog->change_list_names[n];
       pdialog->change_list_ids[n++] = improvement_number(pimprove);
     }
   } improvement_iterate_end;
-  
-  pdialog->change_list_num_improvements=n;
 
+  pdialog->change_list_num_improvements = n;
 
   unit_type_iterate(punittype) {
     if (can_city_build_unit_now(pdialog->pcity, punittype)) {
@@ -2247,16 +2249,16 @@ void change_callback(Widget w, XtPointer client_data, XtPointer call_data)
       production.value.utype = punittype;
       get_city_dialog_production_full(pdialog->change_list_names[n],
                                       sizeof(pdialog->change_list_names[n]),
-                                      production, pdialog->pcity);
-      pdialog->change_list_names_ptrs[n]=pdialog->change_list_names[n];
+                                      &production, pdialog->pcity);
+      pdialog->change_list_names_ptrs[n] = pdialog->change_list_names[n];
       pdialog->change_list_ids[n++] = utype_number(punittype);
     }
   } unit_type_iterate_end;
-  
-  pdialog->change_list_names_ptrs[n]=0;
+
+  pdialog->change_list_names_ptrs[n] = 0;
 
   XawListChange(pdialog->change_list, pdialog->change_list_names_ptrs, 
-		0, 0, False);
+                0, 0, False);
   /* force refresh of viewport so the scrollbar is added.
    * Buggy sun athena requires this */
   XtVaSetValues(cview, XtNforceBars, True, NULL);
@@ -2460,16 +2462,20 @@ void cityopt_callback(Widget w, XtPointer client_data,
   struct city *pcity = pdialog->pcity;
   int i;
 
-  if(cityopt_shell) {
+  if (cityopt_shell) {
     XtDestroyWidget(cityopt_shell);
   }
-  cityopt_shell=create_cityopt_dialog(city_name(pcity));
+
+  cityopt_shell = create_cityopt_dialog(city_name_get(pcity));
+
   /* Doing this here makes the "No"'s centered consistently */
-  for(i=0; i<NUM_CITYOPT_TOGGLES; i++) {
+  for (i = 0; i < NUM_CITYOPT_TOGGLES; i++) {
     bool state = is_city_option_set(pcity, i);
+
     XtVaSetValues(cityopt_toggles[i], XtNstate, state,
-		  XtNlabel, state?_("Yes"):_("No"), NULL);
+                  XtNlabel, state?_("Yes"):_("No"), NULL);
   }
+
   if (is_city_option_set(pcity, CITYO_NEW_EINSTEIN)) {
     newcitizen_index = 1;
   } else if (is_city_option_set(pcity, CITYO_NEW_TAXMAN)) {
@@ -2477,10 +2483,11 @@ void cityopt_callback(Widget w, XtPointer client_data,
   } else {
     newcitizen_index = 0;
   }
+
   XtVaSetValues(cityopt_triggle, XtNstate, 1,
-		XtNlabel, _(newcitizen_labels[newcitizen_index]),
-		NULL);
-  
+                XtNlabel, _(newcitizen_labels[newcitizen_index]),
+                NULL);
+
   cityopt_city_id = pcity->id;
 
   xaw_set_relative_position(pdialog->shell, cityopt_shell, 15, 15);
