@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,20 +30,23 @@
 #include <X11/Xaw/Toggle.h>
 #include <X11/Xaw/Viewport.h>
 
-/* common & utility */
-#include "city.h"
+/* utility */
 #include "fcintl.h"
 #include "log.h"
 #include "mem.h"
-#include "packets.h"
 #include "shared.h"
 #include "support.h"
+
+/* common */
+#include "city.h"
+#include "packets.h"
 #include "unit.h"
 
 /* client */
 #include "client_main.h"
 #include "climisc.h"
 
+/* gui-xaw */
 #include "chatline.h"
 #include "citydlg.h"
 #include "cityrepdata.h"
@@ -378,19 +381,19 @@ void city_list_callback(Widget w, XtPointer client_data,
 ...
 *****************************************************************/
 void city_change_callback(Widget w, XtPointer client_data, 
-			 XtPointer call_data)
+                          XtPointer call_data)
 {
-  XawListReturnStruct *ret=XawListShowCurrent(city_list);
+  XawListReturnStruct *ret = XawListShowCurrent(city_list);
   struct city *pcity;
   struct universal production;
 
-  if(ret->list_index!=XAW_LIST_NONE && 
-     (pcity=cities_in_list[ret->list_index])) {
+  if (ret->list_index != XAW_LIST_NONE
+      && (pcity=cities_in_list[ret->list_index])) {
     cid my_cid = (cid) XTPOINTER_TO_INT(client_data);
-      
+
     production = cid_decode(my_cid);
 
-    city_change_production(pcity, production);
+    city_change_production(pcity, &production);
   }
 }
 
@@ -478,11 +481,12 @@ void city_popup_callback(Widget w, XtPointer client_data,
 {
   XawListReturnStruct *ret=XawListShowCurrent(city_list);
 
-  if(ret->list_index!=XAW_LIST_NONE) {
+  if (ret->list_index!=XAW_LIST_NONE) {
     struct city *pcity;
-    if((pcity=cities_in_list[ret->list_index])) {
-      if (center_when_popup_city) {
-	center_tile_mapcanvas(pcity->tile);
+
+    if ((pcity = cities_in_list[ret->list_index])) {
+      if (gui_options.center_when_popup_city) {
+        center_tile_mapcanvas(pcity->tile);
       }
       popup_city_dialog(pcity);
     }
@@ -602,11 +606,15 @@ void real_city_report_update_city(struct city *pcity)
       char new_city_line[MAX_LEN_CITY_TEXT];
 
       XtVaGetValues(city_list, XtNnumberStrings, &n, XtNlist, &list, NULL);
-      if (0 != strcmp(city_name(pcity), list[i])) {
-	break;
+      if (0 != strcmp(city_name_get(pcity), list[i])) {
+        break;
       }
       get_city_text(pcity, new_city_line, sizeof(new_city_line));
-      if(strcmp(new_city_line, list[i])==0) return; /* no change */
+
+      if (strcmp(new_city_line, list[i]) == 0) {
+        return; /* no change */
+      }
+
       fc_strlcpy(list[i], new_city_line, MAX_LEN_CITY_TEXT);
 
       /* It seems really inefficient to regenerate the whole list just to
@@ -812,12 +820,11 @@ static void chgall_update_widgets_state (struct chgall_data *state);
 /**************************************************************************
 ...
 **************************************************************************/
-
-static void popup_chgall_dialog (Widget parent)
+static void popup_chgall_dialog(Widget parent)
 {
   struct chgall_data *state;
   Widget shell;
-  Widget main_form;
+  Widget chgall_main;
   Widget fr_viewport;
   Widget to_viewport;
   Position x, y;
@@ -858,7 +865,7 @@ static void popup_chgall_dialog (Widget parent)
   XtAddCallback (state->w.shell, XtNdestroyCallback,
 		 chgall_shell_destroy, state);
 
-  main_form =
+  chgall_main =
     XtVaCreateManagedWidget
     (
      "chgallform",
@@ -872,7 +879,7 @@ static void popup_chgall_dialog (Widget parent)
     (
      "chgallfrviewport",
      viewportWidgetClass,
-     main_form,
+     chgall_main,
      NULL
     );
 
@@ -890,7 +897,7 @@ static void popup_chgall_dialog (Widget parent)
     (
      "chgalltoviewport",
      viewportWidgetClass,
-     main_form,
+     chgall_main,
      NULL
     );
 
@@ -908,7 +915,7 @@ static void popup_chgall_dialog (Widget parent)
     (
      "chgallchangecommand",
      commandWidgetClass,
-     main_form,
+     chgall_main,
      NULL
     ));
 
@@ -917,7 +924,7 @@ static void popup_chgall_dialog (Widget parent)
     (
      "chgallrefreshcommand",
      commandWidgetClass,
-     main_form,
+     chgall_main,
      NULL
     ));
 
@@ -926,7 +933,7 @@ static void popup_chgall_dialog (Widget parent)
     (
      "chgallcancelcommand",
      commandWidgetClass,
-     main_form,
+     chgall_main,
      NULL
     ));
 
@@ -1032,23 +1039,25 @@ static void chgall_to_list_callback (Widget w, XtPointer client_data,
 /**************************************************************************
 ...
 **************************************************************************/
-
 static void chgall_change_command_callback (Widget w, XtPointer client_data,
-					    XtPointer call_data)
+                                            XtPointer call_data)
 {
   struct chgall_data *state = (struct chgall_data *)client_data;
+  struct universal fr;
+  struct universal to;
 
   chgall_update_widgets_state (state);
 
-  if (!(state->may_change))
-    {
-      return;
-    }
+  if (!(state->may_change)) {
+    return;
+  }
 
-  client_change_all(cid_decode(state->fr_cids[state->fr_index]),
-		    cid_decode(state->to_cids[state->to_index]));
+  fr = cid_decode(state->fr_cids[state->fr_index]);
+  to = cid_decode(state->to_cids[state->to_index]);
 
-  XtDestroyWidget (state->w.shell);
+  client_change_all(&fr, &to);
+
+  XtDestroyWidget(state->w.shell);
 }
 
 /**************************************************************************
