@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -46,7 +46,6 @@
 #include "map.h"
 #include "packets.h"
 #include "player.h"
-#include "traderoutes.h"
 #include "unitlist.h"
 
 /* client */
@@ -145,22 +144,12 @@ void unit_select_all_callback(Widget w, XtPointer client_data,
 static int city_style_idx[64];     /* translation table basic style->city_style  */
 static int city_style_ridx[64];    /* translation table the other way            */
                                    /* they in fact limit the num of styles to 64 */
-static int b_s_num; /* num basic city styles, i.e. those that you can start with */
-
 
 /******************************************************************/
 
 int is_showing_pillage_dialog = FALSE;
 int unit_to_use_to_pillage;
 
-int caravan_city_id;
-int caravan_unit_id;
-
-struct city *pcity_caravan_dest;
-struct unit *punit_caravan;
-
-static Widget caravan_dialog;
-
 /****************************************************************
 ...
 *****************************************************************/
@@ -342,7 +331,7 @@ void popup_notify_goto_dialog(const char *headline, const char *lines,
                               const struct text_tag_list *tags,
                               struct tile *ptile)
 {
-  Widget notify_dialog_shell, notify_form, notify_command, notify_goto_command;
+  Widget notify_gdialog_shell, notify_form, notify_command, notify_goto_command;
   Widget notify_headline, notify_label;
   Dimension width, width2, width_1, width_2;
   
@@ -350,13 +339,13 @@ void popup_notify_goto_dialog(const char *headline, const char *lines,
     popup_notify_dialog("Message:", headline, lines);
     return;
   }
-  notify_dialog_shell = XtCreatePopupShell("Message:",
-					   transientShellWidgetClass,
-					   toplevel, NULL, 0);
+  notify_gdialog_shell = XtCreatePopupShell("Message:",
+                                            transientShellWidgetClass,
+                                            toplevel, NULL, 0);
 
-  notify_form = XtVaCreateManagedWidget("notifyform", 
-					 formWidgetClass, 
-					 notify_dialog_shell, NULL);
+  notify_form = XtVaCreateManagedWidget("notifyform",
+                                        formWidgetClass, 
+                                        notify_gdialog_shell, NULL);
 
   notify_headline=XtVaCreateManagedWidget("notifyheadline", 
 			  labelWidgetClass, notify_form, 
@@ -392,8 +381,8 @@ void popup_notify_goto_dialog(const char *headline, const char *lines,
   XtAddCallback(notify_command, XtNcallback, notify_no_goto_command_callback, NULL);
   XtAddCallback(notify_goto_command, XtNcallback, notify_goto_command_callback, NULL);
   notify_goto_add_widget_tile(notify_goto_command, ptile);
-  xaw_set_relative_position(toplevel, notify_dialog_shell, 25, 5);
-  XtPopup(notify_dialog_shell, XtGrabNone);
+  xaw_set_relative_position(toplevel, notify_gdialog_shell, 25, 5);
+  XtPopup(notify_gdialog_shell, XtGrabNone);
   /*  XtSetSensitive(toplevel, FALSE); */
 }
 
@@ -408,92 +397,6 @@ void popup_connect_msg(const char *headline, const char *message)
   output_window_append(ftc_client, message);
 }
 
-/****************************************************************
-...
-*****************************************************************/
-static void caravan_establish_trade_callback(Widget w, XtPointer client_data,
-					     XtPointer call_data)
-{
-  dsend_packet_unit_establish_trade(&client.conn, caravan_unit_id);
-  destroy_message_dialog(w);
-  caravan_dialog = 0;
-  process_caravan_arrival(NULL);
-}
-
-
-/****************************************************************
-...
-*****************************************************************/
-static void caravan_help_build_wonder_callback(Widget w,
-					       XtPointer client_data,
-					       XtPointer call_data)
-{
-  dsend_packet_unit_help_build_wonder(&client.conn, caravan_unit_id);
-
-  destroy_message_dialog(w);
-  caravan_dialog = 0;
-  process_caravan_arrival(NULL);
-}
-
-
-/****************************************************************
-...
-*****************************************************************/
-static void caravan_keep_moving_callback(Widget w, XtPointer client_data, 
-					 XtPointer call_data)
-{
-  destroy_message_dialog(w);
-  caravan_dialog = 0;
-  process_caravan_arrival(NULL);
-}
-
-
-/****************************************************************
-...
-*****************************************************************/
-void popup_caravan_dialog(struct unit *punit,
-			  struct city *phomecity, struct city *pdestcity)
-{
-  char buf[128];
-  
-  fc_snprintf(buf, sizeof(buf),
-              _("Your %s from %s reaches the city of %s.\nWhat now?"),
-              unit_name_translation(punit),
-              city_name(phomecity), city_name(pdestcity));
-  
-  caravan_city_id=pdestcity->id; /* callbacks need these */
-  caravan_unit_id=punit->id;
-  
-  caravan_dialog=popup_message_dialog(toplevel, "caravandialog", 
-			   buf,
-			   caravan_establish_trade_callback, 0, 0,
-			   caravan_help_build_wonder_callback, 0, 0,
-			   caravan_keep_moving_callback, 0, 0,
-			   NULL);
-  
-  if (!can_cities_trade(phomecity, pdestcity))
-    XtSetSensitive(XtNameToWidget(caravan_dialog, "*button0"), FALSE);
-  
-  if(!unit_can_help_build_wonder(punit, pdestcity))
-    XtSetSensitive(XtNameToWidget(caravan_dialog, "*button1"), FALSE);
-}
-
-/****************************************************************
-...
-*****************************************************************/
-bool caravan_dialog_is_open(int* unit_id, int* city_id)
-{
-  return BOOL_VAL(caravan_dialog);
-}
-
-/**************************************************************************
-  Updates caravan dialog
-**************************************************************************/
-void caravan_dialog_update(void)
-{
-/* PORT ME */
-}
-
 /****************************************************************
 ...
 *****************************************************************/
@@ -548,22 +451,13 @@ static void pillage_callback(Widget w, XtPointer client_data,
   if (client_data) {
     struct unit *punit = game_unit_by_number(unit_to_use_to_pillage);
     if (punit) {
-      struct act_tgt target;
+      struct extra_type *target;
       int what = XTPOINTER_TO_INT(client_data);
 
-      if (what >= S_LAST + game.control.num_base_types) {
-        target.type = ATT_ROAD;
-        target.obj.road = what - S_LAST - game.control.num_base_types;
-      } else if (what >= S_LAST) {
-        target.type = ATT_BASE;
-        target.obj.base = what - S_LAST;
-      } else {
-        target.type = ATT_SPECIAL;
-        target.obj.spe = what;
-      }
+      target = extra_by_number(what);
 
       request_new_unit_activity_targeted(punit, ACTIVITY_PILLAGE,
-                                         &target);
+                                         target);
     }
   }
 
@@ -575,13 +469,10 @@ static void pillage_callback(Widget w, XtPointer client_data,
   Popup a dialog asking the unit which improvement they would like to
   pillage.
 **************************************************************************/
-void popup_pillage_dialog(struct unit *punit,
-			  bv_special spe,
-                          bv_bases bases,
-                          bv_roads roads)
+void popup_pillage_dialog(struct unit *punit, bv_extras extras)
 {
   Widget shell, form, dlabel, button, prev;
-  struct act_tgt tgt;
+  struct extra_type *tgt;
 
   if (is_showing_pillage_dialog) {
     return;
@@ -597,41 +488,18 @@ void popup_pillage_dialog(struct unit *punit,
   dlabel = I_L(XtVaCreateManagedWidget("dlabel", labelWidgetClass, form, NULL));
 
   prev = dlabel;
-  while (get_preferred_pillage(&tgt, spe, bases, roads)) {
-    bv_special what_spe;
-    bv_bases what_base;
-    bv_roads what_road;
-    int what = S_LAST;
+  while ((tgt = get_preferred_pillage(extras))) {
+    int what;
 
-    BV_CLR_ALL(what_spe);
-    BV_CLR_ALL(what_base);
-    BV_CLR_ALL(what_road);
+    what = extra_index(tgt);
 
-    switch (tgt.type) {
-      case ATT_SPECIAL:
-        BV_SET(what_spe, tgt.obj.spe);
-        what = tgt.obj.spe;
-        clear_special(&spe, tgt.obj.spe);
-        break;
-      case ATT_BASE:
-        BV_SET(what_base, tgt.obj.base);
-        what = tgt.obj.base + S_LAST;
-        BV_CLR(bases, tgt.obj.base);
-        break;
-      case ATT_ROAD:
-        BV_SET(what_road, tgt.obj.road);
-        what = tgt.obj.road + S_LAST + game.control.num_base_types;
-        BV_CLR(roads, tgt.obj.road);
-        break;
-    }
+    BV_CLR(extras, what);
 
     button =
       XtVaCreateManagedWidget ("button", commandWidgetClass, form,
                                XtNfromVert, prev,
                                XtNlabel,
-                               (XtArgVal)(get_infrastructure_text(what_spe,
-                                                                  what_base,
-                                                                  what_road)),
+                               (XtArgVal)(extra_name_translation(tgt)),
                                NULL);
     XtAddCallback(button, XtNcallback, pillage_callback,
                   INT_TO_XTPOINTER(what));
@@ -826,29 +694,34 @@ void unit_select_dialog_popup(struct tile *ptile)
 
   fill_tile_unit_list(ptile, unit_list);
 
-  for(i=0; i<n; i++) {
+  for (i = 0; i < n; i++) {
     struct unit *punit = unit_list[i];
-    struct unit_type *punittemp=unit_type(punit);
+    struct unit_type *punittemp = unit_type_get(punit);
     struct city *pcity;
     struct canvas store;
-    
-    if(!(i%r))  {
-      nargs=0;
-      if(i)  { XtSetArg(args[nargs], XtNfromHoriz, column); nargs++;}
+
+    if (!(i % r))  {
+      nargs = 0;
+      if ( i ) {
+        XtSetArg(args[nargs], XtNfromHoriz, column);
+        nargs++;
+      }
       column = XtCreateManagedWidget("column", formWidgetClass,
-				     unit_select_form,
-				     args, nargs);
-      if(!i) firstcolumn=column;
+                                     unit_select_form,
+                                     args, nargs);
+      if (!i) {
+        firstcolumn = column;
+      }
     }
 
     unit_select_ids[i]=punit->id;
 
     pcity = player_city_by_number(client_player(), punit->homecity);
-    
-    fc_snprintf(buffer, sizeof(buffer), "%s(%s)\n%s", 
-	    utype_name_translation(punittemp), 
-	    pcity ? city_name(pcity) : "",
-	    unit_activity_text(punit));
+
+    fc_snprintf(buffer, sizeof(buffer), "%s(%s)\n%s",
+                utype_name_translation(punittemp),
+                pcity ? city_name_get(pcity) : "",
+                unit_activity_text(punit));
 
     unit_select_pixmaps[i]=XCreatePixmap(display, XtWindow(map_canvas), 
 					 tileset_full_tile_width(tileset), tileset_full_tile_height(tileset),
@@ -857,7 +730,7 @@ void unit_select_dialog_popup(struct tile *ptile)
     XFillRectangle(display, unit_select_pixmaps[i], fill_bg_gc,
 		   0, 0, tileset_full_tile_width(tileset), tileset_full_tile_height(tileset));
     store.pixmap = unit_select_pixmaps[i];
-    put_unit(punit, &store, 0, 0);
+    put_unit(punit, &store, 1.0, 0, 0);
 
     nargs=0;
     XtSetArg(args[nargs], XtNbitmap, (XtArgVal)unit_select_pixmaps[i]);nargs++;
@@ -1182,37 +1055,41 @@ void create_races_dialog(struct player *pplayer)
 
   /* find out styles that can be used at the game beginning */
   /* Limit of 64 city_styles should be deleted. -ev */
-  for (i = 0, b_s_num = 0; i < game.control.styles_count && i < 64; i++) {
-    if (!city_style_has_requirements(&city_styles[i])) {
-      city_style_idx[b_s_num] = i;
-      city_style_ridx[i] = b_s_num;
-      b_s_num++;
+  styles_iterate(pstyle) {
+    i = basic_city_style_for_style(pstyle);
+
+    if (i >= 0) {
+      int sn = style_number(pstyle);
+
+      city_style_idx[sn] = i;
+      city_style_ridx[i] = sn;
     }
-  }
+  } styles_iterate_end;
 
   races_style_label =
     I_L(XtVaCreateManagedWidget("racesstylelabel", 
-				labelWidgetClass, 
-				races_form,
-				XtNfromVert, races_sex_form,
-/*				XtNfromHoriz, races_toggles_viewport,*/
-				NULL));  
+                                labelWidgetClass, 
+                                races_form,
+                                XtNfromVert, races_sex_form,
+                                /* XtNfromHoriz, races_toggles_viewport,*/
+                                NULL));
 
   races_style_form =
     XtVaCreateManagedWidget("racesstyleform", 
-			    formWidgetClass, 
-			    races_form, 
-			    XtNfromVert, races_style_label,
-/*			    XtNfromHoriz, races_toggles_viewport,*/
-			    NULL);   
+                            formWidgetClass, 
+                            races_form, 
+                            XtNfromVert, races_style_label,
+                            /* XtNfromHoriz, races_toggles_viewport,*/
+                            NULL);
 
   free(races_style_toggles);
-  races_style_toggles = fc_calloc(b_s_num,sizeof(Widget));
+  races_style_toggles = fc_calloc(game.control.num_styles, sizeof(Widget));
 
-  for( i = 0; i < ((b_s_num-1)/per_row)+1; i++) {
+  for (i = 0; i < ((game.control.num_styles - 1) / per_row) + 1; i++) {
     index = i * per_row;
     fc_snprintf(namebuf, sizeof(namebuf), "racesstyle%d", index);
-    if( i == 0 ) {
+
+    if (i == 0) {
       races_style_toggles[index] =
 	XtVaCreateManagedWidget(namebuf,
 				toggleWidgetClass,
@@ -1234,9 +1111,11 @@ void create_races_dialog(struct player *pplayer)
 
     for( j = 1; j < per_row; j++) {
       index = i * per_row + j;
-      if( index >= b_s_num ) break;
+      if (index >= game.control.num_styles) {
+        break;
+      }
       fc_snprintf(namebuf, sizeof(namebuf), "racesstyle%d", index);
-      if( i == 0 ) {
+      if (i == 0) {
 	races_style_toggles[index] =
 	  XtVaCreateManagedWidget(namebuf,
 				  toggleWidgetClass,
@@ -1341,7 +1220,7 @@ void create_races_dialog(struct player *pplayer)
     }
   }
 
-  for(i=0; i<b_s_num; i++) {
+  for (i = 0; i < game.control.num_styles; i++) {
     XtVaSetValues(races_style_toggles[i], XtNlabel,
 		  (XtArgVal)city_style_name_translation(city_style_idx[i]), NULL);
   }
@@ -1456,7 +1335,7 @@ void races_toggles_callback(Widget w, XtPointer client_data,
 
   x_simulate_button_click
   (
-   races_style_toggles[city_style_ridx[city_style_of_nation(race)]]
+   races_style_toggles[city_style_ridx[style_number(style_of_nation(race))]]
   );
 }
 
@@ -1541,16 +1420,18 @@ int races_style_buttons_get_current(void)
   int i;
   XtPointer dp, yadp;
 
-  if(b_s_num==1)
+  if (game.control.num_styles == 1) {
     return 0;
+  }
 
   if(!(dp=XawToggleGetCurrent(races_style_toggles[0])))
     return -1;
 
-  for(i=0; i<b_s_num; i++) {
+  for (i = 0; i < game.control.num_styles; i++) {
     XtVaGetValues(races_style_toggles[i], XtNradioData, &yadp, NULL);
-    if(dp==yadp)
+    if (dp == yadp) {
       return i;
+    }
   }
 
   return -1;
@@ -1604,8 +1485,8 @@ void races_ok_command_callback(Widget w, XtPointer client_data,
     return;
   }
 
-  if((selected_style=races_style_buttons_get_current())==-1) {
-    output_window_append(ftc_client, _("You must select your city style."));
+  if ((selected_style = races_style_buttons_get_current()) == -1) {
+    output_window_append(ftc_client, _("You must select your style."));
     return;
   }
 
@@ -1621,7 +1502,8 @@ void races_ok_command_callback(Widget w, XtPointer client_data,
 				 player_number(races_player),
 				 nation_index(races_toggles_to_nations[selected_index]),
 				 selected_sex ? FALSE : TRUE,
-				 dp, city_style_idx[selected_style]);
+				 dp,
+                                 city_style_idx[selected_style]);
   popdown_races_dialog();
 }
 
@@ -1690,6 +1572,14 @@ void popup_soundset_suggestion_dialog(void)
 {
 }
 
+/****************************************************************
+  Ruleset (modpack) has suggested loading certain musicset. Confirm from
+  user and load.
+*****************************************************************/
+void popup_musicset_suggestion_dialog(void)
+{
+}
+
 /**************************************************************************
   Tileset (modpack) has suggested loading certain theme. Confirm from
   user and load.
@@ -1730,3 +1620,31 @@ void show_tileset_error(const char *msg)
 {
   /* PORTME */
 }
+
+/****************************************************************
+  Give a warning when user is about to edit scenario with manually
+  set properties.
+*****************************************************************/
+bool handmade_scenario_warning(void)
+{
+  /* Just tell the client common code to handle this. */
+  return FALSE;
+}
+
+/****************************************************************
+  Unit wants to get into some transport on given tile.
+*****************************************************************/
+bool request_transport(struct unit *pcargo, struct tile *ptile)
+{
+  return FALSE; /* Unit was not handled here. */
+}
+
+/***************************************************************************
+  Popup detailed information about battle or save information for
+  some kind of statistics
+***************************************************************************/
+void popup_combat_info(int attacker_unit_id, int defender_unit_id,
+                       int attacker_hp, int defender_hp,
+                       bool make_winner_veteran)
+{
+}
