@@ -17,8 +17,7 @@
 
 #include "dialogs_g.h"
 
-struct tile;
-
+void popup_revolution_dialog(struct government *pgovernment);
 Widget popup_message_dialog(Widget parent, const char *shellname,
 			    const char *text, ...);
 void destroy_message_dialog(Widget button);
