@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -150,7 +150,7 @@ void popup_find_dialog(void)
 /**************************************************************************
 ...
 **************************************************************************/
-void update_find_dialog(Widget find_list)
+void update_find_dialog(Widget search_list)
 {
   int j = 0;
 
@@ -163,17 +163,16 @@ void update_find_dialog(Widget find_list)
   
   players_iterate(pplayer) {
     city_list_iterate(pplayer->cities, pcity) {
-      *(city_name_ptrs+j++)=fc_strdup(city_name(pcity));
+      *(city_name_ptrs+j++)=fc_strdup(city_name_get(pcity));
     } city_list_iterate_end;
   } players_iterate_end;
   
-  if(ncities_total) {
+  if (ncities_total) {
     qsort(city_name_ptrs, ncities_total, sizeof(char *), compare_strings_ptrs);
-    XawListChange(find_list, city_name_ptrs, ncities_total, 0, True);
+    XawListChange(search_list, city_name_ptrs, ncities_total, 0, True);
   }
 }
 
-
 /**************************************************************************
 ...
 **************************************************************************/
@@ -197,8 +196,8 @@ void find_center_command_callback(Widget w, XtPointer client_data,
 {
   struct city *pcity;
   XawListReturnStruct *ret;
-  
-  ret=XawListShowCurrent(find_list);
+
+  ret = XawListShowCurrent(find_list);
 
   if (ret->list_index != XAW_LIST_NONE) {
     if ((pcity = game_city_by_name(ret->string))) {
