@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,10 +18,13 @@
 
 #ifdef AUDIO_SDL
 /* Though it would happily compile without this include,
- * it is needed for sound to work as long as SDL-1.2 mixer is
- * being used. It defines "main" macro to rename our main() so that
+ * it is needed for sound to work.
+ * It defines "main" macro to rename our main() so that
  * it can install SDL's own. */
-#ifdef AUDIO_SDL1_2
+#ifdef SDL2_PLAIN_INCLUDE
+#include <SDL.h>
+#include <SDL_mixer.h>
+#elif AUDIO_SDL1_2
 /* SDL */
 #include <SDL/SDL.h>
 #else  /* AUDIO_SDL1_2 */
@@ -51,6 +54,7 @@
 #include "pixcomm.h"
 
 /* utility */
+#include "fc_cmdline.h"
 #include "fciconv.h"
 #include "fcintl.h"
 #include "log.h"
@@ -70,13 +74,12 @@
 #include "clinet.h"
 #include "control.h"
 #include "editgui_g.h"
-#include "ggz_g.h"
 #include "options.h"
 #include "text.h"
 #include "tilespec.h"
 
 /* gui-xaw */
-#include "actions.h"
+#include "xaw_actions.h"
 #include "colors.h"
 #include "dialogs.h"
 #include "graphics.h"
@@ -253,7 +256,7 @@ static int myerr(Display *p, XErrorEvent *e)
   Print extra usage information, including one line help on each option,
   to stderr.
 **************************************************************************/
-static void print_usage(const char *argv0)
+static void print_usage(void)
 {
   /* add client-specific usage information here */
   fc_fprintf(stderr,
@@ -266,7 +269,7 @@ static void print_usage(const char *argv0)
 }
 
 /**************************************************************************
-  Parse and enact any client-specific options.
+  Handle commandline options specific to this gui
 **************************************************************************/
 static void parse_options(int argc, char **argv)
 {
@@ -274,7 +277,7 @@ static void parse_options(int argc, char **argv)
 
   while (i < argc) {
     if (is_option("--help", argv[i])) {
-      print_usage(argv[0]);
+      print_usage();
       exit(EXIT_SUCCESS);
     } else {
       fc_fprintf(stderr, _("Unrecognized option: \"%s\"\n"), argv[i]);
@@ -308,7 +311,7 @@ void ui_init(void)
 /****************************************************************************
   Extra initializers for client options.
 ****************************************************************************/
-void gui_options_extra_init(void)
+void options_extra_init(void)
 {
   /* Nothing to do. */
 }
@@ -875,23 +878,6 @@ void remove_net_input(void)
   XUndefineCursor(display, XtWindow(map_canvas));
 }
 
-/**************************************************************************
-  Called to monitor a GGZ socket.
-**************************************************************************/
-void add_ggz_input(int sock)
-{
-  /* PORTME */
-}
-
-/**************************************************************************
-  Called on disconnection to remove monitoring on the GGZ socket.  Only
-  call this if we're actually in GGZ mode.
-**************************************************************************/
-void remove_ggz_input(void)
-{
-  /* PORTME */
-}
-
 /**************************************************************************
 ...
 **************************************************************************/
@@ -935,7 +921,7 @@ void set_unit_icon(int idx, struct unit *punit)
   if (punit) {
     struct canvas store = {XawPixcommPixmap(w)};
 
-    put_unit(punit, &store, 0, 0);
+    put_unit(punit, &store, 1.0, 0, 0);
     xaw_expose_now(w);
   }
 }
@@ -1139,7 +1125,7 @@ void editgui_popdown_all(void)
 /****************************************************************************
   Stub for editor function
 ****************************************************************************/
-void editgui_notify_object_changed(int objtype, int object_id, bool remove)
+void editgui_notify_object_changed(int objtype, int object_id, bool removal)
 {}
 
 /****************************************************************************
@@ -1148,19 +1134,6 @@ void editgui_notify_object_changed(int objtype, int object_id, bool remove)
 void editgui_notify_object_created(int tag, int id)
 {}
 
-/****************************************************************************
-  Stub for ggz function
-****************************************************************************/
-void gui_ggz_embed_leave_table(void)
-{}
-
-/****************************************************************************
-  Stub for ggz function
-****************************************************************************/
-void gui_ggz_embed_ensure_server(void)
-{}
-
-
 /**************************************************************************
   Updates a gui font style.
 **************************************************************************/
@@ -1168,3 +1141,19 @@ void gui_update_font(const char *font_name, const char *font_value)
 {
   /* PORTME */
 }
+
+/**************************************************************************
+  Insert build information to help
+**************************************************************************/
+void insert_client_build_info(char *outbuf, size_t outlen)
+{
+  /* PORTME */
+}
+
+/**************************************************************************
+  Make dynamic adjustments to first-launch default options.
+**************************************************************************/
+void adjust_default_options(void)
+{
+  /* Nothing in case of this gui */
+}
