@@ -253,10 +253,11 @@ void real_players_dialog_update(void)
 	pds = player_diplstate_get(client.conn.playing, pplayer);
 	if (pds->type == DS_CEASEFIRE) {
 	  fc_snprintf(dsbuf, sizeof(dsbuf), "%s (%d)",
-		      diplstate_text(pds->type), pds->turns_left);
+		      diplstate_type_translated_name(pds->type),
+		      pds->turns_left);
 	} else {
 	  fc_snprintf(dsbuf, sizeof(dsbuf), "%s",
-		      diplstate_text(pds->type));
+		      diplstate_type_translated_name(pds->type));
 	}
       }
 
