@@ -198,7 +198,7 @@ void create_science_dialog(bool make_modal)
   int num_list, j = 0, flag = 0;
 
   if (NULL != client.conn.playing) {
-    struct player_research *research = player_research_get(client.conn.playing);
+    struct research *research = research_get(client_player());
 
     if (research->researching == A_UNSET) {
       fc_snprintf(current_text, sizeof(current_text),
@@ -209,8 +209,9 @@ void create_science_dialog(bool make_modal)
     } else {
       fc_snprintf(current_text, sizeof(current_text),
 		  _("Researching %s: %d/%d"),
-		  advance_name_researching(client.conn.playing),
-		  research->bulbs_researched,
+                  research_advance_name_translation(research,
+                                                    research->researching),
+                  research->bulbs_researched,
                   research->client.researching_cost);
     }
 
@@ -223,12 +224,12 @@ void create_science_dialog(bool make_modal)
       fc_snprintf(goal_text, sizeof(goal_text),
 		  _("Goal: %s (%d steps)"),
 		  advance_name_translation(advance_by_number(research->tech_goal)),
-		  num_unknown_techs_for_goal(client.conn.playing,
-					     research->tech_goal));
+                  research_goal_unknown_techs(research,
+                                              research->tech_goal));
     }
 
     advance_index_iterate(A_FIRST, i) {
-      if (TECH_KNOWN == player_invention_state(client.conn.playing, i)) {
+      if (TECH_KNOWN == research_invention_state(research, i)) {
 	tech_list_names_ptrs[j] = advance_name_translation(advance_by_number(i));
 	j++;
       }
@@ -318,7 +319,7 @@ void create_science_dialog(bool make_modal)
 
     advance_index_iterate(A_FIRST, i) {
       if (TECH_PREREQS_KNOWN ==
-            player_invention_state(client.conn.playing, i)) {
+            research_invention_state(research, i)) {
 	Widget entry =
 	  XtVaCreateManagedWidget(advance_name_translation(advance_by_number(i)),
 				  smeBSBObjectClass,
@@ -336,9 +337,9 @@ void create_science_dialog(bool make_modal)
 
     flag = 0;
     advance_index_iterate(A_FIRST, i) {
-      if (player_invention_reachable(client.conn.playing, i, TRUE)
-	  && TECH_KNOWN != player_invention_state(client.conn.playing, i)
-	  && (11 > num_unknown_techs_for_goal(client.conn.playing, i)
+      if (research_invention_reachable(research, i)
+          && TECH_KNOWN != research_invention_state(research, i)
+          && (11 > research_goal_unknown_techs(research, i)
 	      || i == research->tech_goal)) {
 	Widget entry =
 	  XtVaCreateManagedWidget(advance_name_translation(advance_by_number(i)),
@@ -469,7 +470,7 @@ void real_science_report_dialog_update(void)
     static const char *tech_list_names_ptrs[A_LAST + 1];
     int j, flag;
     const char *report_title;
-    struct player_research *research = player_research_get(client.conn.playing);
+    struct research *research = research_get(client_player());
     
     /* TRANS: Research report title */
     report_title = get_report_title_plus(_("Research"), science_dialog_text());
@@ -485,8 +486,9 @@ void real_science_report_dialog_update(void)
     } else {
       fc_snprintf(text, sizeof(text),
 		  _("Researching %s: %d/%d"),
-		  advance_name_researching(client.conn.playing),
-		  research->bulbs_researched,
+                  research_advance_name_translation(research,
+                                                    research->researching),
+                  research->bulbs_researched,
                   research->client.researching_cost);
     }
 
@@ -501,15 +503,15 @@ void real_science_report_dialog_update(void)
       fc_snprintf(text, sizeof(text),
 		  _("Goal: %s (%d steps)"),
 		  advance_name_translation(advance_by_number(research->tech_goal)),
-		  num_unknown_techs_for_goal(client.conn.playing,
-					     research->tech_goal));
+                  research_goal_unknown_techs(research,
+                                              research->tech_goal));
     }
 
     xaw_set_label(science_goal_label, text);
 
     j=0;
     advance_index_iterate(A_FIRST, i) {
-      if (TECH_KNOWN == player_invention_state(client.conn.playing, i)) {
+      if (TECH_KNOWN == research_invention_state(research, i)) {
 	tech_list_names_ptrs[j]=advance_name_translation(advance_by_number(i));
 	j++;
       }
@@ -529,7 +531,7 @@ void real_science_report_dialog_update(void)
     flag=0;
     advance_index_iterate(A_FIRST, i) {
       if (TECH_PREREQS_KNOWN ==
-            player_invention_state(client.conn.playing, i)) {
+            research_invention_state(research, i)) {
 	Widget entry=
 	  XtVaCreateManagedWidget(advance_name_translation(advance_by_number(i)),
 				  smeBSBObjectClass,
@@ -552,9 +554,9 @@ void real_science_report_dialog_update(void)
     
     flag=0;
     advance_index_iterate(A_FIRST, i) {
-      if (player_invention_reachable(client.conn.playing, i, TRUE)
-	  && TECH_KNOWN != player_invention_state(client.conn.playing, i)
-	  && (11 > num_unknown_techs_for_goal(client.conn.playing, i)
+      if (research_invention_reachable(research, i)
+          && TECH_KNOWN != research_invention_state(research, i)
+          && (11 > research_goal_unknown_techs(research, i)
 	      || i == research->tech_goal)) {
 	Widget entry=
 	  XtVaCreateManagedWidget(advance_name_translation(advance_by_number(i)),
@@ -1088,7 +1090,7 @@ void real_units_report_dialog_update(void)
     memset(unitarray, '\0', sizeof(unitarray));
 
     unit_list_iterate(client.conn.playing->units, punit) {
-      Unit_type_id uti = utype_index(unit_type(punit));
+      Unit_type_id uti = utype_index(unit_type_get(punit));
 
       (unitarray[uti].active_count)++;
       if (punit->homecity) {
