@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -801,35 +801,39 @@ void worklist_avail_callback(Widget w, XtPointer client_data,
 
 *****************************************************************/
 void insert_into_worklist(struct worklist_dialog *pdialog,
-			  int before, cid cid)
+                          int before, cid id)
 {
   int i, first_free;
-  struct universal target = cid_decode(cid);
+  struct universal target = cid_decode(id);
 
   /* If this worklist is a city worklist, double check that the city
      really can (eventually) build the target.  We've made sure that
      the list of available targets is okay for this city, but a global
      worklist may try to insert an odd-ball unit or target. */
   if (pdialog->pcity
-      && !can_city_build_later(pdialog->pcity, target)) {
+      && !can_city_build_later(pdialog->pcity, &target)) {
     /* Nope, this city can't build this target, ever.  Don't put it into
        the worklist. */
     return;
   }
 
   /* Find the first free element in the worklist */
-  for (first_free = 0; first_free < MAX_LEN_WORKLIST; first_free++)
-    if (pdialog->worklist_ids[first_free] == WORKLIST_END)
+  for (first_free = 0; first_free < MAX_LEN_WORKLIST; first_free++) {
+    if (pdialog->worklist_ids[first_free] == WORKLIST_END) {
       break;
+    }
+  }
 
-  if (first_free >= MAX_LEN_WORKLIST-1)
+  if (first_free >= MAX_LEN_WORKLIST-1) {
     /* No room left in the worklist! (remember, we need to keep space
        open for the WORKLIST_END sentinel.) */
     return;
+  }
 
-  if (first_free < before && before != MAX_LEN_WORKLIST)
+  if (first_free < before && before != MAX_LEN_WORKLIST) {
     /* True weirdness. */
     return;
+  }
 
   if (before < MAX_LEN_WORKLIST) {
     /* Slide all the later elements in the worklist down. */
@@ -847,7 +851,7 @@ void insert_into_worklist(struct worklist_dialog *pdialog,
   pdialog->worklist_ids[first_free] = WORKLIST_END;
   pdialog->worklist_names_ptrs[first_free] = NULL;
 
-  pdialog->worklist_ids[before] = cid;
+  pdialog->worklist_ids[before] = id;
   
   worklist_id_to_name(pdialog->worklist_names[before],
 		      target, pdialog->pcity);
@@ -1068,24 +1072,24 @@ void worklist_ok_callback(Widget w, XtPointer client_data, XtPointer call_data)
   struct worklist wl;
   struct universal production;
   int i;
-  
-  pdialog=(struct worklist_dialog *)client_data;
-  
-  /* Fill in this worklist with the parameters set in the worklist 
+
+  pdialog = (struct worklist_dialog *)client_data;
+
+  /* Fill in this worklist with the parameters set in the worklist
      dialog. */
   worklist_init(&wl);
-  
+
   for (i = 0; i < MAX_LEN_WORKLIST; i++) {
     if (pdialog->worklist_ids[i] == WORKLIST_END) {
       continue;
     } else if (pdialog->worklist_ids[i] >= B_LAST) {
       production.kind = VUT_UTYPE;
       production.value.utype = utype_by_number(pdialog->worklist_ids[i] - B_LAST);
-      worklist_append(&wl, production);
+      worklist_append(&wl, &production);
     } else if (pdialog->worklist_ids[i] >= 0) {
       production.kind = VUT_IMPROVEMENT;
       production.value.building = improvement_by_number(pdialog->worklist_ids[i]);
-      worklist_append(&wl, production);
+      worklist_append(&wl, &production);
     } else {
       continue;
     }
@@ -1136,9 +1140,10 @@ void worklist_worklist_help_callback(Widget w, XtPointer client_data,
   pdialog=(struct worklist_dialog *)client_data;
 
   ret = XawListShowCurrent(pdialog->worklist);
-  if(ret->list_index!=XAW_LIST_NONE) {
-    cid cid = pdialog->worklist_ids[ret->list_index];
-    production = cid_decode(cid);
+  if (ret->list_index != XAW_LIST_NONE) {
+    cid id = pdialog->worklist_ids[ret->list_index];
+
+    production = cid_decode(id);
   } else {
     production.kind = VUT_NONE;
     production.value.building = NULL;
