@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,12 +18,12 @@
 /* client */
 #include "client_main.h"
 #include "editgui_g.h"
-#include "ggz_g.h"
 #include "options.h"
 
 #include "chatline_g.h"
 #include "citydlg_g.h"
 #include "connectdlg_g.h"
+#include "dialogs_g.h"
 #include "editgui_g.h"
 #include "graphics_g.h"
 #include "gui_main_g.h"
@@ -70,7 +70,7 @@ void ui_exit(void)
 /**************************************************************************
   Call version_message callback
 **************************************************************************/
-void version_message(char *vertext)
+void version_message(const char *vertext)
 {
   funcs.version_message(vertext);
 }
@@ -94,19 +94,27 @@ enum gui_type get_gui_type(void)
 }
 
 /**************************************************************************
-  Call isometric_view_supported callback
+  Call insert_client_build_info callback
 **************************************************************************/
-bool isometric_view_supported(void)
+void insert_client_build_info(char *outbuf, size_t outlen)
 {
-  return funcs.isometric_view_supported();
+  funcs.insert_client_build_info(outbuf, outlen);
 }
 
 /**************************************************************************
-  Call overhead_view_supported callback
+  Call adjust_default_options callback
 **************************************************************************/
-bool overhead_view_supported(void)
+void adjust_default_options(void)
 {
-  return funcs.overhead_view_supported();
+  funcs.adjust_default_options();
+}
+
+/**************************************************************************
+  Call is_view_supported callback
+**************************************************************************/
+bool is_view_supported(enum ts_type type)
+{
+  return funcs.is_view_supported(type);
 }
 
 /**************************************************************************
@@ -146,11 +154,11 @@ void get_sprite_dimensions(struct sprite *sprite, int *width, int *height)
 **************************************************************************/
 struct sprite *crop_sprite(struct sprite *source,
 			   int x, int y, int width, int height,
-			   struct sprite *mask,
-			   int mask_offset_x, int mask_offset_y)
+			   struct sprite *mask, int mask_offset_x, int mask_offset_y,
+               float scale, bool smooth)
 {
   return funcs.crop_sprite(source, x, y, width, height, mask,
-                           mask_offset_x, mask_offset_y);
+                           mask_offset_x, mask_offset_y, scale, smooth);
 }
 
 /**************************************************************************
@@ -193,6 +201,22 @@ void canvas_free(struct canvas *store)
   funcs.canvas_free(store);
 }
 
+/**************************************************************************
+  Call canvas_set_zoom callback
+**************************************************************************/
+void canvas_set_zoom(struct canvas *store, float zoom)
+{
+  funcs.canvas_set_zoom(store, zoom);
+}
+
+/**************************************************************************
+  Call has_zoom_support callback
+**************************************************************************/
+bool has_zoom_support(void)
+{
+  return funcs.has_zoom_support();
+}
+
 /**************************************************************************
   Call canvas_copy callback
 **************************************************************************/
@@ -258,15 +282,6 @@ void canvas_fill_sprite_area(struct canvas *pcanvas,
   funcs.canvas_fill_sprite_area(pcanvas, psprite, pcolor, canvas_x, canvas_y);
 }
 
-/**************************************************************************
-  Call canvas_fog_sprite_area callback
-**************************************************************************/
-void canvas_fog_sprite_area(struct canvas *pcanvas, struct sprite *psprite,
-			    int canvas_x, int canvas_y)
-{
-  funcs.canvas_fog_sprite_area(pcanvas, psprite, canvas_x, canvas_y);
-}
-
 /**************************************************************************
   Call canvas_put_line callback
 **************************************************************************/
@@ -308,27 +323,27 @@ void canvas_put_text(struct canvas *pcanvas, int canvas_x, int canvas_y,
 }
 
 /**************************************************************************
-  Call gui_set_rulesets callback
+  Call set_rulesets callback
 **************************************************************************/
-void gui_set_rulesets(int num_rulesets, char **rulesets)
+void set_rulesets(int num_rulesets, char **rulesets)
 {
-  funcs.gui_set_rulesets(num_rulesets, rulesets);
+  funcs.set_rulesets(num_rulesets, rulesets);
 }
 
 /**************************************************************************
-  Call gui_options_extra_init callback
+  Call options_extra_init callback
 **************************************************************************/
-void gui_options_extra_init(void)
+void options_extra_init(void)
 {
-  funcs.gui_options_extra_init();
+  funcs.options_extra_init();
 }
 
 /**************************************************************************
-  Call gui_server_connect callback
+  Call server_connect callback
 **************************************************************************/
-void gui_server_connect(void)
+void server_connect(void)
 {
-  funcs.gui_server_connect();
+  funcs.server_connect();
 }
 
 /**************************************************************************
@@ -456,9 +471,9 @@ void editgui_notify_object_created(int tag, int id)
 /****************************************************************************
   Call editgui_notify_object_changed callback
 ****************************************************************************/
-void editgui_notify_object_changed(int objtype, int object_id, bool remove)
+void editgui_notify_object_changed(int objtype, int object_id, bool removal)
 {
-  funcs.editgui_notify_object_changed(objtype, object_id, remove);
+  funcs.editgui_notify_object_changed(objtype, object_id, removal);
 }
 
 /****************************************************************************
@@ -485,36 +500,15 @@ void editgui_popdown_all(void)
   funcs.editgui_popdown_all();
 }
 
-/**************************************************************************
-  Call gui_ggz_embed_ensure_server callback
-**************************************************************************/
-void gui_ggz_embed_ensure_server(void)
-{
-  funcs.gui_ggz_embed_ensure_server();
-}
-
-/**************************************************************************
-  Call add_ggz_input callback
-**************************************************************************/
-void add_ggz_input(int sock)
-{
-  funcs.add_ggz_input(sock);
-}
-
-/**************************************************************************
-  Call remove_ggz_input callback
-**************************************************************************/
-void remove_ggz_input(void)
-{
-  funcs.remove_ggz_input();
-}
-
 /****************************************************************************
-  Call gui_ggz_embed_leave_table callback
+  Call popup_combat_info callback
 ****************************************************************************/
-void gui_ggz_embed_leave_table(void)
+void popup_combat_info(int attacker_unit_id, int defender_unit_id,
+                       int attacker_hp, int defender_hp,
+                       bool make_winner_veteran)
 {
-  funcs.gui_ggz_embed_leave_table();
+  funcs.popup_combat_info(attacker_unit_id, defender_unit_id,
+                          attacker_hp, defender_hp, make_winner_veteran);
 }
 
 /****************************************************************************
@@ -557,6 +551,14 @@ void popdown_all_city_dialogs()
   funcs.popdown_all_city_dialogs();
 }
 
+/**************************************************************************
+  Call handmade_scenario_warning callback
+**************************************************************************/
+bool handmade_scenario_warning()
+{
+  return funcs.handmade_scenario_warning();
+}
+
 /**************************************************************************
   Call refresh_unit_city_dialogs callback
 **************************************************************************/
@@ -573,6 +575,14 @@ bool city_dialog_is_open(struct city *pcity)
   return funcs.city_dialog_is_open(pcity);
 }
 
+/**************************************************************************
+  Call request_transport callback
+**************************************************************************/
+bool request_transport(struct unit *pcargo, struct tile *ptile)
+{
+  return funcs.request_transport(pcargo, ptile);
+}
+
 /**************************************************************************
   Call gui_load_theme callback
 **************************************************************************/
