@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,20 +27,24 @@ extern "C" {
 #include "canvas_g.h"
 #include "pages_g.h"
 
+/* client */
+#include "tilespec.h"
+
 struct gui_funcs {
   void (*ui_init)(void);
   void (*ui_main)(int argc, char *argv[]);
   void (*ui_exit)(void);
 
   enum gui_type (*get_gui_type)(void);
+  void (*insert_client_build_info)(char *outbuf, size_t outlen);
+  void (*adjust_default_options)(void);
 
-  void (*version_message)(char *vertext);
+  void (*version_message)(const char *vertext);
   void (*real_output_window_append)(const char *astring,
                                     const struct text_tag_list *tags,
                                     int conn_id);
 
-  bool (*isometric_view_supported)(void);
-  bool (*overhead_view_supported)(void);
+  bool (*is_view_supported)(enum ts_type type);
   void (*free_intro_radar_sprites)(void);
   struct sprite * (*load_gfxfile)(const char *filename);
   struct sprite * (*create_sprite)(int width, int height, struct color *pcolor);
@@ -48,7 +52,8 @@ struct gui_funcs {
   struct sprite * (*crop_sprite)(struct sprite *source,
                                  int x, int y, int width, int height,
                                  struct sprite *mask,
-                                 int mask_offset_x, int mask_offset_y);
+                                 int mask_offset_x, int mask_offset_y,
+                                 float scale, bool smooth);
   void (*free_sprite)(struct sprite *s);
 
   struct color *(*color_alloc)(int r, int g, int b);
@@ -56,6 +61,8 @@ struct gui_funcs {
 
   struct canvas *(*canvas_create)(int width, int height);
   void (*canvas_free)(struct canvas *store);
+  void (*canvas_set_zoom)(struct canvas *store, float zoom);
+  bool (*has_zoom_support)(void);
   void (*canvas_copy)(struct canvas *dest, struct canvas *src,
                       int src_x, int src_y, int dest_x, int dest_y, int width,
                       int height);
@@ -77,9 +84,6 @@ struct gui_funcs {
   void (*canvas_fill_sprite_area)(struct canvas *pcanvas,
                                   struct sprite *psprite, struct color *pcolor,
                                   int canvas_x, int canvas_y);
-  void (*canvas_fog_sprite_area)(struct canvas *pcanvas,
-                                 struct sprite *psprite,
-                                 int canvas_x, int canvas_y);
   void (*canvas_put_line)(struct canvas *pcanvas, struct color *pcolor,
                           enum line_type ltype, int start_x, int start_y,
                           int dx, int dy);
@@ -92,9 +96,9 @@ struct gui_funcs {
                           enum client_font font, struct color *pcolor,
                           const char *text);
 
-  void (*gui_set_rulesets)(int num_rulesets, char **rulesets);
-  void (*gui_options_extra_init)(void);
-  void (*gui_server_connect)(void);
+  void (*set_rulesets)(int num_rulesets, char **rulesets);
+  void (*options_extra_init)(void);
+  void (*server_connect)(void);
   void (*add_net_input)(int sock);
   void (*remove_net_input)(void);
   void (*real_conn_list_dialog_update)(void);
@@ -114,24 +118,25 @@ struct gui_funcs {
 
   void (*editgui_refresh)(void);
   void (*editgui_notify_object_created)(int tag, int id);
-  void (*editgui_notify_object_changed)(int objtype, int object_id, bool remove);
+  void (*editgui_notify_object_changed)(int objtype, int object_id, bool removal);
   void (*editgui_popup_properties)(const struct tile_list *tiles, int objtype);
   void (*editgui_tileset_changed)(void);
   void (*editgui_popdown_all)(void);
 
-  void (*gui_ggz_embed_ensure_server)(void);
-  void (*gui_ggz_embed_leave_table)(void);
-  void (*add_ggz_input)(int sock);
-  void (*remove_ggz_input)(void);
-
+  void (*popup_combat_info)(int attacker_unit_id, int defender_unit_id,
+                            int attacker_hp, int defender_hp,
+                            bool make_winner_veteran);
   void (*update_timeout_label)(void);
   void (*real_city_dialog_popup)(struct city *pcity);
   void (*real_city_dialog_refresh)(struct city *pcity);
   void (*popdown_city_dialog)(struct city *pcity);
   void (*popdown_all_city_dialogs)(void);
+  bool (*handmade_scenario_warning)(void);
   void (*refresh_unit_city_dialogs)(struct unit *punit);
   bool (*city_dialog_is_open)(struct city *pcity);
 
+  bool (*request_transport)(struct unit *pcargo, struct tile *ptile);
+
   void (*gui_load_theme)(const char *directory, const char *theme_name);
   void (*gui_clear_theme)(void);
   char **(*get_gui_specific_themes_directories)(int *count);
