@@ -17,4 +17,9 @@
 
 GUI_FUNC_PROTO(void, popup_rates_dialog, void)
 
+GUI_FUNC_PROTO(void, real_multipliers_dialog_update, void)
+
+/* Actually defined in update_queue.c */
+void multipliers_dialog_update(void);
+
 #endif  /* FC__RATESDLG_G_H */
