@@ -1,4 +1,4 @@
-/**********************************************************************
+/***********************************************************************
  Freeciv - Copyright (C) 2002 - The Freeciv Project
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,7 +27,9 @@
 #include "support.h"
 
 /* common */
+#include "calendar.h"
 #include "citizens.h"
+#include "clientutils.h"
 #include "combat.h"
 #include "fc_types.h" /* LINE_BREAK */
 #include "game.h"
@@ -153,6 +155,7 @@ const char *popup_info_text(struct tile *ptile)
   char username[MAX_LEN_NAME + 32];
   char nation[2 * MAX_LEN_NAME + 32];
   int tile_x, tile_y, nat_x, nat_y;
+  bool first;
 
   astr_clear(&str);
   index_to_map_pos(&tile_x, &tile_y, tile_index(ptile));
@@ -169,9 +172,17 @@ const char *popup_info_text(struct tile *ptile)
   astr_add_line(&str, _("Terrain: %s"),  tile_get_info_text(ptile, TRUE, 0));
   astr_add_line(&str, _("Food/Prod/Trade: %s"),
 		get_tile_output_text(ptile));
-  if (tile_has_special(ptile, S_HUT)) {
-    astr_add_line(&str, _("Minor Tribe Village"));
-  }
+  first = TRUE;
+  extra_type_iterate(pextra) {
+    if (pextra->category == ECAT_BONUS && tile_has_visible_extra(ptile, pextra)) {
+      if (!first) {
+        astr_add(&str, ",%s", extra_name_translation(pextra));
+      } else {
+        astr_add_line(&str, "%s", extra_name_translation(pextra));
+        first = FALSE;
+      }
+    }
+  } extra_type_iterate_end;
   if (BORDERS_DISABLED != game.info.borders && !pcity) {
     struct player *owner = tile_owner(ptile);
 
@@ -233,7 +244,7 @@ const char *popup_info_text(struct tile *ptile)
     if (NULL == client.conn.playing || owner == client.conn.playing) {
       /* TRANS: "City: <city name> | <username> (<nation + team>)" */
       astr_add_line(&str, _("City: %s | %s (%s)"),
-                    city_name(pcity), username, nation);
+                    city_name_get(pcity), username, nation);
     } else {
       struct player_diplstate *ds
         = player_diplstate_get(client_player(), owner);
@@ -245,7 +256,7 @@ const char *popup_info_text(struct tile *ptile)
         astr_add_line(&str, PL_("City: %s | %s (%s, %d turn cease-fire)",
                                 "City: %s | %s (%s, %d turn cease-fire)",
                                 turns),
-                      city_name(pcity), username, nation, turns);
+                      city_name_get(pcity), username, nation, turns);
       } else if (ds->type == DS_ARMISTICE) {
         int turns = ds->turns_left;
 
@@ -254,12 +265,12 @@ const char *popup_info_text(struct tile *ptile)
         astr_add_line(&str, PL_("City: %s | %s (%s, %d turn armistice)",
                                 "City: %s | %s (%s, %d turn armistice)",
                                 turns),
-                      city_name(pcity), username, nation, turns);
+                      city_name_get(pcity), username, nation, turns);
       } else {
         /* TRANS: "City: <city name> | <username>
          * (<nation + team>, <diplomatic state>)" */
         astr_add_line(&str, _("City: %s | %s (%s, %s)"),
-                      city_name(pcity), username, nation,
+                      city_name_get(pcity), username, nation,
                       diplo_city_adjectives[ds->type]);
       }
     }
@@ -299,20 +310,19 @@ const char *popup_info_text(struct tile *ptile)
     unit_list_iterate(get_units_in_focus(), pfocus_unit) {
       struct city *hcity = game_city_by_number(pfocus_unit->homecity);
 
-      if (unit_has_type_flag(pfocus_unit, UTYF_TRADE_ROUTE)
-	  && can_cities_trade(hcity, pcity)
-	  && can_establish_trade_route(hcity, pcity)) {
+      if (utype_can_do_action(unit_type_get(pfocus_unit), ACTION_TRADE_ROUTE)
+          && can_cities_trade(hcity, pcity)
+          && can_establish_trade_route(hcity, pcity)) {
 	/* TRANS: "Trade from Warsaw: 5" */
 	astr_add_line(&str, _("Trade from %s: %d"),
-		      city_name(hcity),
-		      trade_between_cities(hcity, pcity));
+                      city_name_get(hcity),
+                      trade_between_cities(hcity, pcity));
       }
     } unit_list_iterate_end;
   }
   {
-    const char *infratext = get_infrastructure_text(ptile->special,
-                                                    ptile->bases,
-                                                    ptile->roads);
+    const char *infratext = get_infrastructure_text(ptile->extras);
+
     if (*infratext != '\0') {
       astr_add_line(&str, _("Infrastructure: %s"), infratext);
     }
@@ -323,13 +333,13 @@ const char *popup_info_text(struct tile *ptile)
   }
   if (punit && !pcity) {
     struct player *owner = unit_owner(punit);
-    struct unit_type *ptype = unit_type(punit);
+    struct unit_type *ptype = unit_type_get(punit);
 
     get_full_username(username, sizeof(username), owner);
     get_full_nation(nation, sizeof(nation), owner);
 
     if (!client_player() || owner == client_player()) {
-      struct city *pcity = player_city_by_number(owner, punit->homecity);
+      struct city *hcity = player_city_by_number(owner, punit->homecity);
 
       /* TRANS: "Unit: <unit type> | <username> (<nation + team>)" */
       astr_add_line(&str, _("Unit: %s | %s (%s)"),
@@ -337,10 +347,10 @@ const char *popup_info_text(struct tile *ptile)
 
       if (game.info.citizen_nationality
           && unit_nationality(punit) != unit_owner(punit)) {
-        if (pcity) {
+        if (hcity != NULL) {
           /* TRANS: on own line immediately following \n, "from <city> |
            * <nationality> people" */
-          astr_add_line(&str, _("from %s | %s people"), city_name(pcity),
+          astr_add_line(&str, _("from %s | %s people"), city_name_get(hcity),
                         nation_adjective_for_player(unit_nationality(punit)));
         } else {
           /* TRANS: Nationality of the people comprising a unit, if
@@ -348,9 +358,9 @@ const char *popup_info_text(struct tile *ptile)
           astr_add_line(&str, _("%s people"),
                         nation_adjective_for_player(unit_nationality(punit)));
         }
-      } else if (pcity) {
+      } else if (hcity != NULL) {
         /* TRANS: on own line immediately following \n, ... <city> */
-        astr_add_line(&str, _("from %s"), city_name(pcity));
+        astr_add_line(&str, _("from %s"), city_name_get(hcity));
       }
     } else if (NULL != owner) {
       struct player_diplstate *ds = player_diplstate_get(client_player(),
@@ -424,11 +434,12 @@ const char *popup_info_text(struct tile *ptile)
     if (unit_owner(punit) == client_player()
         || client_is_global_observer()) {
       /* Show bribe cost for own units. */
-      astr_add_line(&str, _("Bribe cost: %d"), unit_bribe_cost(punit));
+      astr_add_line(&str, _("Probable bribe cost: %d"),
+                    unit_bribe_cost(punit, NULL));
     } else {
       /* We can only give an (lower) boundary for units of other players. */
       astr_add_line(&str, _("Estimated bribe cost: > %d"),
-                    unit_bribe_cost(punit));
+                    unit_bribe_cost(punit, client_player()));
     }
 
     if ((NULL == client.conn.playing || owner == client.conn.playing)
@@ -442,126 +453,7 @@ const char *popup_info_text(struct tile *ptile)
   return astr_str(&str);
 }
 
-/****************************************************************************
-  Creates the activity progress text for the given tile.
-
-  This should only be used inside popup_info_text and should eventually be
-  made static.
-****************************************************************************/
-const char *concat_tile_activity_text(struct tile *ptile)
-{
-  int activity_total[ACTIVITY_LAST];
-  int activity_units[ACTIVITY_LAST];
-  int base_total[MAX_BASE_TYPES];
-  int base_units[MAX_BASE_TYPES];
-  int road_total[MAX_ROAD_TYPES];
-  int road_units[MAX_ROAD_TYPES];
-  int num_activities = 0;
-  int pillaging = 0;
-  int remains, turns;
-  static struct astring str = ASTRING_INIT;
-
-  astr_clear(&str);
-
-  memset(activity_total, 0, sizeof(activity_total));
-  memset(activity_units, 0, sizeof(activity_units));
-  memset(base_total, 0, sizeof(base_total));
-  memset(base_units, 0, sizeof(base_units));
-  memset(road_total, 0, sizeof(road_total));
-  memset(road_units, 0, sizeof(road_units));
-
-  unit_list_iterate(ptile->units, punit) {
-    if (punit->activity == ACTIVITY_PILLAGE) {
-      pillaging = 1;
-    } else if (punit->activity == ACTIVITY_BASE) {
-      base_total[punit->activity_target.obj.base] += punit->activity_count;
-      base_total[punit->activity_target.obj.base] += get_activity_rate_this_turn(punit);
-      base_units[punit->activity_target.obj.base] += get_activity_rate(punit);
-    } else if (punit->activity == ACTIVITY_GEN_ROAD) {
-      road_total[punit->activity_target.obj.road] += punit->activity_count;
-      road_total[punit->activity_target.obj.road] += get_activity_rate_this_turn(punit);
-      road_units[punit->activity_target.obj.road] += get_activity_rate(punit);
-    } else {
-      activity_total[punit->activity] += punit->activity_count;
-      activity_total[punit->activity] += get_activity_rate_this_turn(punit);
-      activity_units[punit->activity] += get_activity_rate(punit);
-    }
-  } unit_list_iterate_end;
-
-  if (pillaging) {
-    bv_special pillage_spe = get_unit_tile_pillage_set(ptile);
-    bv_bases pillage_bases = get_unit_tile_pillage_base_set(ptile);
-    bv_roads pillage_roads = get_unit_tile_pillage_road_set(ptile);
-    if (BV_ISSET_ANY(pillage_spe)
-        || BV_ISSET_ANY(pillage_bases)
-        || BV_ISSET_ANY(pillage_roads)) {
-      astr_add(&str, "%s(%s)", _("Pillage"),
-               get_infrastructure_text(pillage_spe, pillage_bases, pillage_roads));
-    } else {
-      /* Untargeted pillaging is happening. */
-      astr_add(&str, "%s", _("Pillage"));
-    }
-    num_activities++;
-  }
-
-  activity_type_iterate(i) {
-    if (i == ACTIVITY_BASE) {
-      base_type_iterate(bp) {
-        Base_type_id b = base_index(bp);
-	if (base_units[b] > 0) {
-	  remains = tile_activity_base_time(ptile, b) - base_total[b];
-	  if (remains > 0) {
-	    turns = 1 + (remains + base_units[b] - 1) / base_units[b];
-	  } else {
-	    /* base will be finished this turn */
-	    turns = 1;
-	  }
-	  if (num_activities > 0) {
-	    astr_add(&str, "/");
-	  }
-	  astr_add(&str, "%s(%d)", base_name_translation(bp), turns);
-	  num_activities++;
-	}
-      } base_type_iterate_end;
-    } else if (i == ACTIVITY_GEN_ROAD) {
-      road_type_iterate(rp) {
-        Road_type_id r = road_index(rp);
-	if (road_units[r] > 0) {
-	  remains = tile_activity_road_time(ptile, r) - road_total[r];
-	  if (remains > 0) {
-	    turns = 1 + (remains + road_units[r] - 1) / road_units[r];
-	  } else {
-	    /* road will be finished this turn */
-	    turns = 1;
-	  }
-	  if (num_activities > 0) {
-	    astr_add(&str, "/");
-	  }
-	  astr_add(&str, "%s(%d)", road_name_translation(rp), turns);
-	  num_activities++;
-	}
-      } road_type_iterate_end;
-    } else if (is_build_or_clean_activity(i) && activity_units[i] > 0) {
-      if (num_activities > 0) {
-	astr_add(&str, "/");
-      }
-      remains = tile_activity_time(i, ptile) - activity_total[i];
-      if (remains > 0) {
-	turns = 1 + (remains + activity_units[i] - 1) / activity_units[i];
-      } else {
-	/* activity will be finished this turn */
-	turns = 1;
-      }
-      astr_add(&str, "%s(%d)", get_activity_text(i), turns);
-      num_activities++;
-    }
-  } activity_type_iterate_end;
-
-  return astr_str(&str);
-}
-
 #define FAR_CITY_SQUARE_DIST (2*(6*6))
-
 /****************************************************************************
   Returns the text describing the city and its distance.
 ****************************************************************************/
@@ -577,18 +469,18 @@ const char *get_nearest_city_text(struct city *pcity, int sq_dist)
   }
 
   astr_add(&str, (sq_dist >= FAR_CITY_SQUARE_DIST)
-                 /* TRANS: on own line immediately following \n, ... <city> */
-                 ? _("far from %s")
-                 : (sq_dist > 0)
-                   /* TRANS: on own line immediately following \n, ... <city> */
-                   ? _("near %s")
-                   : (sq_dist == 0)
-                     /* TRANS: on own line immediately following \n, ... <city> */
-                     ? _("in %s")
-                     : "%s",
-                 pcity
-                 ? city_name(pcity)
-                 : "");
+           /* TRANS: on own line immediately following \n, ... <city> */
+           ? _("far from %s")
+           : (sq_dist > 0)
+           /* TRANS: on own line immediately following \n, ... <city> */
+           ? _("near %s")
+           : (sq_dist == 0)
+           /* TRANS: on own line immediately following \n, ... <city> */
+           ? _("in %s")
+           : "%s",
+           pcity
+           ? city_name_get(pcity)
+           : "");
 
   return astr_str(&str);
 }
@@ -608,7 +500,7 @@ const char *unit_description(struct unit *punit)
   struct city *pcity =
       player_city_by_number(owner, punit->homecity);
   struct city *pcity_near = get_nearest_city(punit, &pcity_near_dist);
-  struct unit_type *ptype = unit_type(punit);
+  struct unit_type *ptype = unit_type_get(punit);
   static struct astring str = ASTRING_INIT;
   const struct player *pplayer = client_player();
 
@@ -633,7 +525,7 @@ const char *unit_description(struct unit *punit)
 
   if (pcity) {
     /* TRANS: on own line immediately following \n, ... <city> */
-    astr_add_line(&str, _("from %s"), city_name(pcity));
+    astr_add_line(&str, _("from %s"), city_name_get(pcity));
   } else {
     astr_add(&str, "\n");
   }
@@ -765,34 +657,31 @@ const char *get_airlift_text(const struct unit_list *punits,
 ****************************************************************************/
 static int get_bulbs_per_turn(int *pours, bool *pteam, int *ptheirs)
 {
-  const struct player_research *presearch;
+  const struct research *presearch;
   int ours = 0, theirs = 0;
   bool team = FALSE;
 
   if (!client_has_player()) {
     return 0;
   }
-  presearch = player_research_get(client_player());
+  presearch = research_get(client_player());
 
   /* Sum up science */
-  players_iterate(pplayer) {
+  research_players_iterate(presearch, pplayer) {
     if (pplayer == client_player()) {
       city_list_iterate(pplayer->cities, pcity) {
         ours += pcity->surplus[O_SCIENCE];
       } city_list_iterate_end;
-
-      if (game.info.tech_upkeep_style != TECH_UPKEEP_NONE) {
-        ours -= pplayer->client.tech_upkeep;
-      }
-    } else if (presearch == player_research_get(pplayer)) {
+    } else {
       team = TRUE;
-      theirs += pplayer->client.bulbs_prod;
-
-      if (game.info.tech_upkeep_style != TECH_UPKEEP_NONE) {
-        theirs -= pplayer->client.tech_upkeep;
-      }
+      theirs -= pplayer->client.tech_upkeep;
     }
-  } players_iterate_end;
+  } research_players_iterate_end;
+
+  if (team) {
+    theirs += presearch->client.total_bulbs_prod - ours;
+  }
+  ours -= client_player()->client.tech_upkeep;
 
   if (pours) {
     *pours = ours;
@@ -806,6 +695,52 @@ static int get_bulbs_per_turn(int *pours, bool *pteam, int *ptheirs)
   return ours + theirs;
 }
 
+/************************************************************************//**
+  Return turns until research complete. -1 for never.
+****************************************************************************/
+static int turns_to_research_done(const struct research *presearch,
+                                  int per_turn)
+{
+  if (per_turn > 0) {
+    return ceil((double)(presearch->client.researching_cost
+                         - presearch->bulbs_researched) / per_turn);
+  } else {
+    return -1;
+  }
+}
+
+/************************************************************************//**
+  Return turns per advance (based on currently researched advance).
+  -1 for no progress.
+****************************************************************************/
+static int turns_per_advance(const struct research *presearch, int per_turn)
+{
+  if (per_turn > 0) {
+    return MAX(1, ceil((double)presearch->client.researching_cost) / per_turn);
+  } else {
+    return -1;
+  }
+}
+
+/************************************************************************//**
+  Return turns until an advance is lost due to tech upkeep.
+  -1 if we're not on the way to losing an advance.
+****************************************************************************/
+static int turns_to_tech_loss(const struct research *presearch, int per_turn)
+{
+  if (per_turn >= 0 || game.info.techloss_forgiveness == -1) {
+    /* With techloss_forgiveness == -1, we'll never lose a tech, just
+     * get further into debt. */
+    return -1;
+  } else {
+    int bulbs_to_loss = presearch->bulbs_researched
+                        + (presearch->client.researching_cost
+                           * game.info.techloss_forgiveness / 100);
+
+    return ceil((double)bulbs_to_loss / -per_turn);
+  }
+}
+
 /****************************************************************************
   Returns the text to display in the science dialog.
 ****************************************************************************/
@@ -815,13 +750,13 @@ const char *science_dialog_text(void)
   int ours, theirs, perturn, upkeep;
   static struct astring str = ASTRING_INIT;
   struct astring ourbuf = ASTRING_INIT, theirbuf = ASTRING_INIT;
-  struct player_research *research;
+  struct research *research;
 
   astr_clear(&str);
 
   perturn = get_bulbs_per_turn(&ours, &team, &theirs);
 
-  research = player_research_get(client_player());
+  research = research_get(client_player());
   upkeep = client_player()->client.tech_upkeep;
 
   if (NULL == client.conn.playing || (ours == 0 && theirs == 0
@@ -829,29 +764,28 @@ const char *science_dialog_text(void)
     return _("Progress: no research");
   }
 
-  if (!is_future_tech(research->researching)
-      && NULL == valid_advance_by_number(research->researching)) {
+  if (A_UNSET == research->researching) {
     astr_add(&str, _("Progress: no research"));
   } else {
-    int done = research->bulbs_researched;
-    int total = research->client.researching_cost;
-
-    if (perturn > 0) {
-      int turns = MAX(1, ceil((double)total) / perturn);
+    int turns;
 
+    if ((turns = turns_per_advance(research, perturn)) >= 0) {
       astr_add(&str, PL_("Progress: %d turn/advance",
                          "Progress: %d turns/advance",
                          turns), turns);
-    } else if (perturn < 0 ) {
-      /* negative number of bulbs per turn due to tech upkeep */
-      int turns = ceil((double) done / -perturn);
-
+    } else if ((turns = turns_to_tech_loss(research, perturn)) >= 0) {
+      /* FIXME: turns to next loss is not a good predictor of turns to
+       * following loss, due to techloss_restore etc. But it'll do. */
       astr_add(&str, PL_("Progress: %d turn/advance loss",
                          "Progress: %d turns/advance loss",
                          turns), turns);
     } else {
-      /* no research */
-      astr_add(&str, _("Progress: none"));
+      /* no forward progress -- no research, or tech loss disallowed */
+      if (perturn < 0) {
+        astr_add(&str, _("Progress: decreasing!"));
+      } else {
+        astr_add(&str, _("Progress: none"));
+      }
     }
   }
   astr_set(&ourbuf, PL_("%d bulb/turn", "%d bulbs/turn", ours), ours);
@@ -889,7 +823,7 @@ const char *science_dialog_text(void)
 ****************************************************************************/
 const char *get_science_target_text(double *percent)
 {
-  struct player_research *research = player_research_get(client_player());
+  struct research *research = research_get(client_player());
   static struct astring str = ASTRING_INIT;
 
   if (!research) {
@@ -897,8 +831,7 @@ const char *get_science_target_text(double *percent)
   }
 
   astr_clear(&str);
-  if (!is_future_tech(research->researching)
-      && NULL == valid_advance_by_number(research->researching)) {
+  if (research->researching == A_UNSET) {
     astr_add(&str, _("%d/- (never)"), research->bulbs_researched);
     if (percent) {
       *percent = 0.0;
@@ -907,20 +840,17 @@ const char *get_science_target_text(double *percent)
     int total = research->client.researching_cost;
     int done = research->bulbs_researched;
     int perturn = get_bulbs_per_turn(NULL, NULL, NULL);
+    int turns;
 
-    if (perturn > 0) {
-      int turns = ceil( (double)(total - done) / perturn );
-
+    if ((turns = turns_to_research_done(research, perturn)) >= 0) {
       astr_add(&str, PL_("%d/%d (%d turn)", "%d/%d (%d turns)", turns),
                done, total, turns);
-    } else if (perturn < 0 ) {
-      /* negative number of bulbs per turn due to tech upkeep */
-      int turns = ceil( (double)done / -perturn );
-
-      astr_add(&str, PL_("%d/%d (%d turn)", "%d/%d (%d turns)", turns),
-               done, perturn, turns);
+    } else if ((turns = turns_to_tech_loss(research, perturn)) >= 0) {
+      astr_add(&str, PL_("%d/%d (%d turn to loss)",
+                         "%d/%d (%d turns to loss)", turns),
+               done, total, turns);
     } else {
-      /* no research */
+      /* no forward progress -- no research, or tech loss disallowed */
       astr_add(&str, _("%d/%d (never)"), done, total);
     }
     if (percent) {
@@ -937,11 +867,11 @@ const char *get_science_target_text(double *percent)
 ****************************************************************************/
 const char *get_science_goal_text(Tech_type_id goal)
 {
-  int steps = num_unknown_techs_for_goal(client.conn.playing, goal);
-  int bulbs_needed = total_bulbs_required_for_goal(client.conn.playing, goal);
+  const struct research *research = research_get(client_player());
+  int steps = research_goal_unknown_techs(research, goal);
+  int bulbs_needed = research_goal_bulbs_required(research, goal);
   int turns;
   int perturn = get_bulbs_per_turn(NULL, NULL, NULL);
-  struct player_research* research = player_research_get(client_player());
   static struct astring str = ASTRING_INIT;
   struct astring buf1 = ASTRING_INIT,
                  buf2 = ASTRING_INIT,
@@ -953,8 +883,7 @@ const char *get_science_goal_text(Tech_type_id goal)
 
   astr_clear(&str);
 
-  if (is_tech_a_req_for_goal(client.conn.playing,
-			     research->researching, goal)
+  if (research_goal_tech_req(research, goal, research->researching)
       || research->researching == goal) {
     bulbs_needed -= research->bulbs_researched;
   }
@@ -996,7 +925,7 @@ const char *get_info_label_text(bool moreinfo)
 		  population_to_text(civ_population(client.conn.playing)));
   }
   astr_add_line(&str, _("Year: %s (T%d)"),
-		textyear(game.info.year), game.info.turn);
+		calendar_text(), game.info.turn);
 
   if (NULL != client.conn.playing) {
     astr_add_line(&str, _("Gold: %d (%+d)"),
@@ -1045,10 +974,11 @@ const char *get_info_label_text_popup(void)
     astr_add_line(&str, _("%s People"),
 		  population_to_text(civ_population(client.conn.playing)));
   }
-  astr_add_line(&str, _("Year: %s"), textyear(game.info.year));
+  astr_add_line(&str, _("Year: %s"), calendar_text());
   astr_add_line(&str, _("Turn: %d"), game.info.turn);
 
   if (NULL != client.conn.playing) {
+    const struct research *presearch = research_get(client_player());
     int perturn = get_bulbs_per_turn(NULL, NULL, NULL);
     int upkeep = client_player()->client.tech_upkeep;
 
@@ -1062,7 +992,8 @@ const char *get_info_label_text_popup(void)
 		  client.conn.playing->economic.luxury,
 		  client.conn.playing->economic.science);
     astr_add_line(&str, _("Researching %s: %s"),
-		  advance_name_researching(client.conn.playing),
+                  research_advance_name_translation(presearch,
+                                                    presearch->researching),
 		  get_science_target_text(NULL));
     /* perturn is defined as: (bulbs produced) - upkeep */
     if (game.info.tech_upkeep_style != TECH_UPKEEP_NONE) {
@@ -1183,10 +1114,8 @@ const char *get_unit_info_label_text2(struct unit_list *punits, int linebreaks)
     astr_add_line(&str, "%s", tile_get_info_text(unit_tile(punit), TRUE,
                                                  linebreaks));
     {
-      const char *infratext
-        = get_infrastructure_text(unit_tile(punit)->special,
-                                  unit_tile(punit)->bases,
-                                  unit_tile(punit)->roads);
+      const char *infratext = get_infrastructure_text(unit_tile(punit)->extras);
+
       if (*infratext != '\0') {
         astr_add_line(&str, "%s", infratext);
       } else {
@@ -1194,7 +1123,7 @@ const char *get_unit_info_label_text2(struct unit_list *punits, int linebreaks)
       }
     }
     if (pcity) {
-      astr_add_line(&str, "%s", city_name(pcity));
+      astr_add_line(&str, "%s", city_name_get(pcity));
     } else {
       astr_add_line(&str, " ");
     }
@@ -1225,7 +1154,7 @@ const char *get_unit_info_label_text2(struct unit_list *punits, int linebreaks)
       } else {
 	mil++;
       }
-      types_count[utype_index(unit_type(punit))]++;
+      types_count[utype_index(unit_type_get(punit))]++;
     } unit_list_iterate_end;
 
     top[0] = top[1] = top[2] = NULL;
@@ -1323,9 +1252,9 @@ bool get_units_upgrade_info(char *buf, size_t bufsz,
     unit_list_iterate(punits, punit) {
       if (unit_owner(punit) == client_player()
           && UU_OK == unit_upgrade_test(punit, FALSE)) {
-	struct unit_type *from_unittype = unit_type(punit);
+	struct unit_type *from_unittype = unit_type_get(punit);
 	struct unit_type *to_unittype = can_upgrade_unittype(client.conn.playing,
-							     unit_type(punit));
+							     from_unittype);
 	int cost = unit_upgrade_price(unit_owner(punit),
 					   from_unittype, to_unittype);
 
@@ -1422,28 +1351,26 @@ const char *get_bulb_tooltip(void)
 			"researching the current technology."));
 
   if (NULL != client.conn.playing) {
-    struct player_research *research = player_research_get(client_player());
+    struct research *research = research_get(client_player());
 
-    if (!is_future_tech(research->researching)
-        && NULL == valid_advance_by_number(research->researching)) {
-      astr_add_line(&str, _("no research target."));
+    if (research->researching == A_UNSET) {
+      astr_add_line(&str, _("No research target."));
     } else {
-      int turns = 0;
+      int turns;
       int perturn = get_bulbs_per_turn(NULL, NULL, NULL);
-      int done = research->bulbs_researched;
-      int total = research->client.researching_cost;
       struct astring buf1 = ASTRING_INIT, buf2 = ASTRING_INIT;
 
-      if (perturn > 0) {
-        turns = MAX(1, ceil((double) (total - done) / perturn));
-      } else if (perturn < 0 ) {
-        turns = ceil((double) done / -perturn);
-      }
-
-      if (turns == 0) {
-        astr_set(&buf1, _("No progress"));
-      } else {
+      if ((turns = turns_to_research_done(research, perturn)) >= 0) {
         astr_set(&buf1, PL_("%d turn", "%d turns", turns), turns);
+      } else if ((turns = turns_to_tech_loss(research, perturn)) >= 0) {
+        astr_set(&buf1, PL_("%d turn to loss", "%d turns to loss", turns),
+                 turns);
+      } else {
+        if (perturn < 0) {
+          astr_set(&buf1, _("Decreasing"));
+        } else {
+          astr_set(&buf1, _("No progress"));
+        }
       }
 
       /* TRANS: <perturn> bulbs/turn */
@@ -1451,7 +1378,8 @@ const char *get_bulb_tooltip(void)
 
       /* TRANS: <tech>: <amount>/<total bulbs> */
       astr_add_line(&str, _("%s: %d/%d (%s, %s)."),
-                    advance_name_researching(client.conn.playing),
+                    research_advance_name_translation(research,
+                                                      research->researching),
                     research->bulbs_researched,
                     research->client.researching_cost,
                     astr_str(&buf1), astr_str(&buf2));
@@ -1595,10 +1523,20 @@ const char *get_timeout_label_text(void)
 
   astr_clear(&str);
 
-  if (client_current_turn_timeout() <= 0) {
-    astr_add(&str, "%s", Q_("?timeout:off"));
+  if (is_waiting_turn_change() && game.tinfo.last_turn_change_time >= 1.5) {
+    double wt = get_seconds_to_new_turn();
+
+    if (wt < 0.01) {
+      astr_add(&str, "%s", Q_("?timeout:wait"));
+    } else {
+      astr_add(&str, "%s: %s", Q_("?timeout:eta"), format_duration(wt)); 
+    }
   } else {
-    astr_add(&str, "%s", format_duration(get_seconds_to_turndone()));
+    if (current_turn_timeout() <= 0) {
+      astr_add(&str, "%s", Q_("?timeout:off"));
+    } else {
+      astr_add(&str, "%s", format_duration(get_seconds_to_turndone()));
+    }
   }
 
   return astr_str(&str);
@@ -1710,11 +1648,11 @@ const char *get_report_title(const char *report_name)
      * syntax. */
     astr_add_line(&str, _("%s - %s"),
                   ruler_title_for_player(pplayer, buf, sizeof(buf)),
-                  textyear(game.info.year));
+                  calendar_text());
   } else {
     /* TRANS: "Observer - 1985 AD" */
     astr_add_line(&str, _("Observer - %s"),
-		  textyear(game.info.year));
+		  calendar_text());
   }
   return astr_str(&str);
 }
