@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,7 +11,7 @@
    GNU General Public License for more details.
 ***********************************************************************/
 
-/********************************************************************** 
+/*********************************************************************** 
   Reading and using the tilespec files, which describe
   the files and contents of tilesets.
 ***********************************************************************/
@@ -44,6 +44,46 @@ struct resource;
   TYPED_VECTOR_ITERATE(struct sprite *, sprite_vec, psprite)
 #define sprite_vector_iterate_end VECTOR_ITERATE_END
 
+#define SPECENUM_NAME ts_type
+#define SPECENUM_VALUE0 TS_OVERHEAD
+#define SPECENUM_VALUE0NAME N_("Overhead")
+#define SPECENUM_VALUE1 TS_ISOMETRIC
+#define SPECENUM_VALUE1NAME N_("Isometric")
+#include "specenum_gen.h"
+
+#define SPECENUM_NAME fog_style
+/* Fog is automatically appended by the code. */
+#define SPECENUM_VALUE0 FOG_AUTO
+#define SPECENUM_VALUE0NAME "Auto"
+/* A single fog sprite is provided by the tileset (tx.fog). */
+#define SPECENUM_VALUE1 FOG_SPRITE
+#define SPECENUM_VALUE1NAME "Sprite"
+/* No fog, or fog derived from darkness style. */
+#define SPECENUM_VALUE2 FOG_DARKNESS
+#define SPECENUM_VALUE2NAME "Darkness"
+#include "specenum_gen.h"
+
+#define SPECENUM_NAME darkness_style
+/* No darkness sprites are drawn. */
+#define SPECENUM_VALUE0 DARKNESS_NONE
+#define SPECENUM_VALUE0NAME "None"
+/* 1 sprite that is split into 4 parts and treated as a darkness4.  Only
+ * works in iso-view. */
+#define SPECENUM_VALUE1 DARKNESS_ISORECT
+#define SPECENUM_VALUE1NAME "IsoRect"
+/* 4 sprites, one per direction.  More than one sprite per tile may be
+ * drawn. */
+#define SPECENUM_VALUE2 DARKNESS_CARD_SINGLE
+#define SPECENUM_VALUE2NAME "CardinalSingle"
+/* 15=2^4-1 sprites.  A single sprite is drawn, chosen based on whether
+ * there's darkness in _each_ of the cardinal directions. */
+#define SPECENUM_VALUE3 DARKNESS_CARD_FULL
+#define SPECENUM_VALUE3NAME "CardinalFull"
+/* Corner darkness & fog.  3^4 = 81 sprites. */
+#define SPECENUM_VALUE4 DARKNESS_CORNER
+#define SPECENUM_VALUE4NAME "Corner"
+#include "specenum_gen.h"
+
 /* An edge is the border between two tiles.  This structure represents one
  * edge.  The tiles are given in the same order as the enumeration name. */
 enum edge_type {
@@ -97,6 +137,7 @@ enum mapview_layer {
   LAYER_CITYBAR,
   LAYER_FOCUS_UNIT,
   LAYER_GOTO,
+  LAYER_WORKERTASK,
   LAYER_EDITOR,
   LAYER_COUNT
 };
@@ -115,6 +156,8 @@ enum mapview_layer {
 
 #define NUM_TILES_PROGRESS 8
 
+#define MAX_NUM_CITIZEN_SPRITES 6
+
 enum arrow_type {
   ARROW_RIGHT,
   ARROW_PLUS,
@@ -127,9 +170,12 @@ struct tileset;
 extern struct tileset *tileset;
 
 struct strvec;
-const struct strvec *get_tileset_list(void);
+const struct strvec *get_tileset_list(const struct option *poption);
 
-struct tileset *tileset_read_toplevel(const char *tileset_name, bool verbose);
+void tileset_error(enum log_level level, const char *format, ...);
+
+struct tileset *tileset_read_toplevel(const char *tileset_name, bool verbose,
+                                      int topology_id, float scale);
 void tileset_init(struct tileset *t);
 void tileset_free(struct tileset *tileset);
 void tileset_load_tiles(struct tileset *t);
@@ -139,9 +185,12 @@ bool tileset_is_fully_loaded(void);
 
 void finish_loading_sprites(struct tileset *t);
 
-void tilespec_try_read(const char *tileset_name, bool verbose);
-void tilespec_reread(const char *tileset_name, bool game_fully_initialized);
+bool tilespec_try_read(const char *tileset_name, bool verbose, int topo_id,
+                       bool global_default);
+bool tilespec_reread(const char *tileset_name, bool game_fully_initialized,
+                     float scale);
 void tilespec_reread_callback(struct option *poption);
+void tilespec_reread_frozen_refresh(const char *tname);
 
 void tileset_setup_specialist_type(struct tileset *t, Specialist_type_id id);
 void tileset_setup_unit_type(struct tileset *t, struct unit_type *punittype);
@@ -153,10 +202,8 @@ void tileset_setup_tile_type(struct tileset *t,
 			     const struct terrain *pterrain);
 void tileset_setup_resource(struct tileset *t,
 			    const struct resource *presource);
-void tileset_setup_road(struct tileset *t,
-                        struct road_type *proad);
-void tileset_setup_base(struct tileset *t,
-                        const struct base_type *pbase);
+void tileset_setup_extra(struct tileset *t,
+                         struct extra_type *pextra);
 void tileset_setup_government(struct tileset *t,
 			      struct government *gov);
 void tileset_setup_nation_flag(struct tileset *t, 
@@ -181,12 +228,6 @@ int fill_basic_terrain_layer_sprite_array(struct tileset *t,
                                           struct drawn_sprite *sprs,
                                           int layer,
                                           struct terrain *pterrain);
-int fill_basic_road_sprite_array(const struct tileset *t,
-                                 struct drawn_sprite *sprs,
-                                 const struct road_type *proad);
-int fill_basic_base_sprite_array(const struct tileset *t,
-                                 struct drawn_sprite *sprs,
-                                 const struct base_type *pbase);
 
 double get_focus_unit_toggle_timeout(const struct tileset *t);
 void reset_focus_unit_state(struct tileset *t);
@@ -269,6 +310,8 @@ struct editor_sprites {
     *military_base;
 };
 
+#define NUM_WALL_TYPES 7
+
 struct sprite *get_spaceship_sprite(const struct tileset *t,
 				    enum spaceship_part part);
 struct sprite *get_citizen_sprite(const struct tileset *t,
@@ -279,17 +322,19 @@ struct sprite *get_city_flag_sprite(const struct tileset *t,
 				    const struct city *pcity);
 struct sprite *get_nation_flag_sprite(const struct tileset *t,
 				      const struct nation_type *nation);
+struct sprite *get_nation_shield_sprite(const struct tileset *t,
+                                        const struct nation_type *nation);
 struct sprite *get_tech_sprite(const struct tileset *t, Tech_type_id tech);
 struct sprite *get_building_sprite(const struct tileset *t,
-				   struct impr_type *pimprove);
+                                   struct impr_type *pimprove);
 struct sprite *get_government_sprite(const struct tileset *t,
-				     const struct government *gov);
+                                     const struct government *gov);
 struct sprite *get_unittype_sprite(const struct tileset *t,
-				   const struct unit_type *punittype,
+                                   const struct unit_type *punittype,
                                    enum direction8 facing,
                                    bool icon);
 struct sprite *get_sample_city_sprite(const struct tileset *t,
-				      int city_style);
+                                      int style_idx);
 struct sprite *get_arrow_sprite(const struct tileset *t,
 				enum arrow_type arrow);
 struct sprite *get_tax_sprite(const struct tileset *t, Output_type_id otype);
@@ -317,21 +362,23 @@ struct sprite *get_unit_upkeep_sprite(const struct tileset *t,
 struct sprite *get_basic_fog_sprite(const struct tileset *t);
 struct sprite *get_resource_sprite(const struct tileset *t,
                                    const struct resource *presouce);
-struct sprite *get_basic_special_sprite(const struct tileset *t,
-                                        enum tile_special_type special);
-struct sprite *get_basic_mine_sprite(const struct tileset *t);
+int fill_basic_extra_sprite_array(const struct tileset *t,
+                                  struct drawn_sprite *sprs,
+                                  const struct extra_type *pextra);
+struct sprite *get_event_sprite(const struct tileset *t, enum event_type event);
 
 struct sprite *tiles_lookup_sprite_tag_alt(struct tileset *t,
                                            enum log_level level,
                                            const char *tag, const char *alt,
                                            const char *what,
-                                           const char *name);
+                                           const char *name,
+                                           bool scale);
 
 struct color_system;
 struct color_system *get_color_system(const struct tileset *t);
 
 /* Tileset accessor functions. */
-const char *tileset_get_name(const struct tileset *t);
+const char *tileset_basename(const struct tileset *t);
 bool tileset_is_isometric(const struct tileset *t);
 int tileset_hex_width(const struct tileset *t);
 int tileset_hex_height(const struct tileset *t);
@@ -342,17 +389,33 @@ int tileset_full_tile_height(const struct tileset *t);
 int tileset_unit_width(const struct tileset *t);
 int tileset_unit_height(const struct tileset *t);
 int tileset_unit_with_upkeep_height(const struct tileset *t);
+int tileset_unit_with_small_upkeep_height(const struct tileset *t);
 int tileset_unit_layout_offset_y(const struct tileset *t);
+int tileset_unit_layout_small_offset_y(const struct tileset *t);
 int tileset_small_sprite_width(const struct tileset *t);
 int tileset_small_sprite_height(const struct tileset *t);
 int tileset_citybar_offset_y(const struct tileset *t);
 int tileset_tilelabel_offset_y(const struct tileset *t);
+float tileset_scale(const struct tileset *t);
 const char *tileset_main_intro_filename(const struct tileset *t);
 const char *tileset_mini_intro_filename(const struct tileset *t);
 int tileset_num_city_colors(const struct tileset *t);
-void tileset_use_prefered_theme(const struct tileset *t);
+void tileset_use_preferred_theme(const struct tileset *t);
 bool tileset_use_hard_coded_fog(const struct tileset *t);
 
+/* These are used as array index -> can't be changed freely to values
+   bigger than size of those arrays. */
+#define TS_TOPO_OVERHEAD 0
+#define TS_TOPO_ISO      1
+#define TS_TOPO_HEX      (1 << 1)
+#define TS_TOPO_ISOHEX   (TS_TOPO_ISO + TS_TOPO_HEX)
+
+const char *tileset_name_get(struct tileset *t);
+const char *tileset_version(struct tileset *t);
+const char *tileset_summary(struct tileset *t);
+const char *tileset_description(struct tileset *t);
+int tileset_topo_index(struct tileset *t);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
