@@ -1,4 +1,4 @@
-/**********************************************************************
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,43 +30,47 @@
 
 #include "caravan.h"
 
-/***************************************************************************
+/****************************************************************************
   Create a valid parameter with default values.
- ***************************************************************************/
+****************************************************************************/
 void caravan_parameter_init_default(struct caravan_parameter *parameter)
 {
   parameter->horizon = FC_INFINITY;
-  parameter->discount = .95;
+  parameter->discount = 0.95;
   parameter->consider_windfall = TRUE;
   parameter->consider_trade = TRUE;
   parameter->consider_wonders = TRUE; /* see also init_from_unit */
   parameter->account_for_broken_routes = TRUE;
-  parameter->allow_foreign_trade = FALSE;
+  parameter->allow_foreign_trade = FTL_NATIONAL_ONLY;
   parameter->ignore_transit_time = FALSE;
   parameter->convert_trade = FALSE;
   parameter->callback = NULL;
 }
 
-/***************************************************************************
+/****************************************************************************
   Create a valid parameter with default values based on the caravan.
- ***************************************************************************/
+****************************************************************************/
 void caravan_parameter_init_from_unit(struct caravan_parameter *parameter,
                                       const struct unit *caravan)
 {
   caravan_parameter_init_default(parameter);
-  if (!unit_has_type_flag(caravan, UTYF_TRADE_ROUTE)) {
-    parameter->consider_windfall = FALSE;
+  if (!unit_can_do_action(caravan, ACTION_TRADE_ROUTE)) {
     parameter->consider_trade = FALSE;
   }
-  if (!unit_has_type_flag(caravan, UTYF_HELP_WONDER)) {
+  if (!unit_can_do_action(caravan, ACTION_MARKETPLACE)
+      && !unit_can_do_action(caravan, ACTION_TRADE_ROUTE)) {
+    parameter->consider_windfall = FALSE;
+  }
+  if (!unit_can_do_action(caravan, ACTION_HELP_WONDER)) {
     parameter->consider_wonders = FALSE;
   }
 }
 
-/***************************************************************************
+/****************************************************************************
   Check for legality.
- ***************************************************************************/
-bool caravan_parameter_is_legal(const struct caravan_parameter *parameter) {
+****************************************************************************/
+bool caravan_parameter_is_legal(const struct caravan_parameter *parameter)
+{
   /* a discount > 1.0 means money later is worth more than money now,
      which is ridiculous. */
   if (parameter->discount > 1.0) {
@@ -82,16 +86,34 @@ bool caravan_parameter_is_legal(const struct caravan_parameter *parameter) {
   if (parameter->horizon == FC_INFINITY && parameter->discount == 1.0) {
     return FALSE;
   }
+
   return TRUE;
 }
 
-/***************************************************************************
+/****************************************************************************
   For debugging, print out the parameter.
-***************************************************************************/
+****************************************************************************/
 void caravan_parameter_log_real(const struct caravan_parameter *parameter,
                                 enum log_level level, const char *file,
                                 const char *function, int line)
 {
+  const char *foreign = "<illegal>";
+
+  switch (parameter->allow_foreign_trade) {
+  case FTL_NATIONAL_ONLY:
+    foreign = "no";
+    break;
+  case FTL_ALLIED:
+    foreign = "allied";
+    break;
+  case FTL_PEACEFUL:
+    foreign = "peaceful";
+    break;
+  case FTL_NONWAR:
+    foreign = "anything but enemies";
+    break;
+  }
+
   do_log(file, function, line, FALSE, level,
          "parameter {\n"
          "  horizon   = %d\n"
@@ -108,15 +130,15 @@ void caravan_parameter_log_real(const struct caravan_parameter *parameter,
          parameter->consider_trade ? "trade" : "-",
          parameter->consider_wonders ? "wonders" : "-",
          parameter->account_for_broken_routes ? "yes" : "no",
-         parameter->allow_foreign_trade ? "yes" : "no",
+         foreign,
          parameter->ignore_transit_time ? "yes" : "no",
          parameter->convert_trade ? "yes" : "no");
 }
 
 
-/***************************************************************************
+/****************************************************************************
   Initialize the result to be worth zero and go from nowhere to nowhere.
- ***************************************************************************/
+****************************************************************************/
 void caravan_result_init_zero(struct caravan_result *result)
 {
   result->src = result->dest = NULL;
@@ -126,10 +148,10 @@ void caravan_result_init_zero(struct caravan_result *result)
   result->required_boat = FALSE;
 }
 
-/***************************************************************************
+/****************************************************************************
   Initialize the result to go from src to dest with the given amount
   of time.  This is useful for calling get_discounted_reward and the such.
- ***************************************************************************/
+****************************************************************************/
 static void caravan_result_init(struct caravan_result *result,
                                 const struct city *src,
                                 const struct city *dest,
@@ -138,7 +160,7 @@ static void caravan_result_init(struct caravan_result *result,
   result->src = src;
   result->dest = dest;
   result->arrival_time = arrival_time;
-  
+
   result->value = 0;
   result->help_wonder = FALSE;
   if ((src != NULL) && (dest != NULL)) {
@@ -152,15 +174,15 @@ static void caravan_result_init(struct caravan_result *result,
   }
 }
 
-/***************************************************************************
+/****************************************************************************
   Compare the two results for sorting.
- ***************************************************************************/
+****************************************************************************/
 int caravan_result_compare(const struct caravan_result *a,
                            const struct caravan_result *b)
 {
-  if(a->value > b->value) {
+  if (a->value > b->value) {
     return 1;
-  } else if(a->value < b->value) {
+  } else if (a->value < b->value) {
     return -1;
   } else {
     /* faster time is better, so reverse-sort on time. */
@@ -168,14 +190,14 @@ int caravan_result_compare(const struct caravan_result *a,
   }
 }
 
-/***************************************************************************
+/****************************************************************************
   We use the path finding in several places.
   This provides a single implementation of that.  It is critical that
   this function be re-entrant since we call it recursively.
 
   The callback should return TRUE if it wants to stop searching,
   FALSE otherwise.
- ***************************************************************************/
+****************************************************************************/
 typedef bool (*search_callback) (void *data, const struct city *pcity,
                                  int arrival_turn, int arrival_moves_left);
 
@@ -183,6 +205,7 @@ static void caravan_search_from(const struct unit *caravan,
                                 const struct caravan_parameter *param,
                                 struct tile *start_tile,
                                 int turns_before, int moves_left_before,
+                                bool omniscient,
                                 search_callback callback,
                                 void *callback_data) {
   struct pf_map *pfm;
@@ -195,6 +218,7 @@ static void caravan_search_from(const struct unit *caravan,
   pft_fill_unit_parameter(&pfparam, (struct unit *) caravan);
   pfparam.start_tile = start_tile;
   pfparam.moves_left_initially = moves_left_before;
+  pfparam.omniscience = omniscient;
   pfm = pf_map_new(&pfparam);
 
   /* For every tile in distance order:
@@ -212,7 +236,7 @@ static void caravan_search_from(const struct unit *caravan,
 
     pcity = tile_city(pos.tile);
     if (pcity && callback(callback_data, pcity, turns_before + pos.turn,
-			  pos.moves_left)) {
+                          pos.moves_left)) {
       break;
     }
   } pf_map_positions_iterate_end;
@@ -220,48 +244,42 @@ static void caravan_search_from(const struct unit *caravan,
   pf_map_destroy(pfm);
 }
 
-
-
-
-/***************************************************************************
+/****************************************************************************
   When the caravan arrives, compute the benefit from the immediate windfall,
   taking into account the parameter's objective.
- ***************************************************************************/
-static double windfall_benefit(const struct city *src,
+****************************************************************************/
+static double windfall_benefit(const struct unit *caravan,
+                               const struct city *src,
                                const struct city *dest,
-                               const struct caravan_parameter *param) {
+                               const struct caravan_parameter *param)
+{
   if (!param->consider_windfall || !can_cities_trade(src, dest)) {
     return 0;
-  }
-  else {
-    int bonus = get_caravan_enter_city_trade_bonus(src, dest);
-    bool can_establish = can_establish_trade_route(src, dest);
-
-    /* we get the full bonus only if this is a new trade route.
-     * Really, g_c_e_c_t_b should compute this.  I copy from unithand.c */
-    if (!can_establish) {
-      bonus += 2;
-      bonus /= 3;
-    }
+  } else {
+    bool can_establish = (unit_can_do_action(caravan, ACTION_TRADE_ROUTE)
+                          && can_establish_trade_route(src, dest));
+    int bonus = get_caravan_enter_city_trade_bonus(src, dest,
+                                                   can_establish);
 
     /* bonus goes to both sci and gold. */
     bonus *= 2;
+
     return bonus;
   }
 }
 
-
-/***************************************************************************
+/****************************************************************************
   Compute the change in the per-turn trade.
- ***************************************************************************/
+****************************************************************************/
 
-/***************************************************************************
+/****************************************************************************
   How much does the city benefit from the new trade route?
   How much does the former partner lose?
- ***************************************************************************/
+****************************************************************************/
 static int one_city_trade_benefit(const struct city *pcity,
-				  const struct player *pplayer,
-                                  bool countloser, int newtrade) {
+                                  const struct player *pplayer,
+                                  bool countloser, int newtrade)
+{
   int losttrade = 0;
 
   /* if the city is owned by someone else, we don't benefit from the
@@ -305,15 +323,16 @@ static int one_city_trade_benefit(const struct city *pcity,
   return newtrade - losttrade;
 }
 
-/***************************************************************************
+/****************************************************************************
   Compute one_trade_benefit for both cities and do some other logic.
   This yields the total benefit in terms of trade per turn of establishing
   a route from src to dest.
- ***************************************************************************/
+****************************************************************************/
 static double trade_benefit(const struct player *caravan_owner,
                             const struct city *src,
                             const struct city *dest,
-                            const struct caravan_parameter *param) {
+                            const struct caravan_parameter *param)
+{
   /* do we care about trade at all? */
   if (!param->consider_trade) {
     return 0;
@@ -344,13 +363,13 @@ static double trade_benefit(const struct player *caravan_owner,
   }
 }
 
-/***************************************************************************
+/****************************************************************************
   Check the benefit of helping build the wonder in dest.
   This is based on how much the caravan would help if it arrived
   after turns_delay turns during which the city managed the same
   production it currently gets (i.e. no other caravans, no
   population growth or terrain improvement, ...)
- ***************************************************************************/
+****************************************************************************/
 static double wonder_benefit(const struct unit *caravan, int arrival_time,
                              const struct city *dest,
                              const struct caravan_parameter *param) {
@@ -376,40 +395,64 @@ static double wonder_benefit(const struct unit *caravan, int arrival_time,
   return costwithout - costwith;
 }
 
-/***************************************************************************
+/****************************************************************************
   Discount a value by the given discount rate.
   The payment occurs as a lump sum in 'term' turns.
- ***************************************************************************/
+****************************************************************************/
 static double presentvalue(double payment, int term, double rate)
 {
   return payment * pow(rate, term);
 }
 
-/***************************************************************************
+/****************************************************************************
   Compute the net present value of an perpetuity given the discount rate.
   A perpetuity is an annual payment for an infinite number of turns.
- ***************************************************************************/
+****************************************************************************/
 static double perpetuity(double payment, double rate)
 {
-  return payment / (1.0-rate);
+  return payment / (1.0 - rate);
 }
 
-/***************************************************************************
+/****************************************************************************
   Compute the net present value of an annuity given the discount rate.
   An annuity is an annual payment for a fixed term (number of turns).
- ***************************************************************************/
+****************************************************************************/
 static double annuity(double payment, int term, double rate)
 {
   return perpetuity(payment, rate) * (1.0 - 1.0 / pow(rate, term));
 }
 
-/***************************************************************************
+/****************************************************************************
+  Are the two players allowed to trade by the parameter settings?
+****************************************************************************/
+static bool does_foreign_trade_param_allow(const struct caravan_parameter *param,
+                                           struct player *src, struct player *dest)
+{
+  switch (param->allow_foreign_trade) {
+    case FTL_NATIONAL_ONLY:
+      return (src == dest);
+      break;
+    case FTL_ALLIED:
+      return pplayers_allied(src, dest);
+      break;
+    case FTL_PEACEFUL:
+      return pplayers_in_peace(src, dest);
+      break;
+    case FTL_NONWAR:
+      return !pplayers_at_war(src, dest);
+  }
+
+  fc_assert(FALSE);
+  return FALSE;
+}
+
+/****************************************************************************
   Compute the discounted reward from the trade route that is indicated
   by the src, dest, and arrival_time fields of the result:  Fills in
   the value and help_wonder fields.
   Assumes the owner of src is the owner of the caravan.
- ***************************************************************************/
-static void get_discounted_reward(const struct unit *caravan,
+****************************************************************************/
+static bool get_discounted_reward(const struct unit *caravan,
                                   const struct caravan_parameter *parameter,
                                   struct caravan_result *result)
 {
@@ -422,61 +465,88 @@ static void get_discounted_reward(const struct unit *caravan,
   double discount = parameter->discount;
   struct player *pplayer_src = city_owner(src);
   struct player *pplayer_dest = city_owner(dest);
+  bool consider_wonder;
+  bool consider_trade;
+  bool consider_windfall;
 
   /* if no foreign trade is allowed, just quit. */
-  if (!parameter->allow_foreign_trade && pplayer_src != pplayer_dest) {
+  if (!does_foreign_trade_param_allow(parameter, pplayer_src, pplayer_dest)) {
     caravan_result_init_zero(result);
-    return;
-  } else {
-    /* foreign trade allowed, we only do business with allies */
-    if (pplayers_allied(pplayer_src, pplayer_dest)) {
-      /* do some business */
-    } else {
-      caravan_result_init_zero(result);
-      return;
-    }
+    return FALSE;
+  }
+
+  consider_wonder = parameter->consider_wonders
+    && is_action_enabled_unit_on_city_full(ACTION_HELP_WONDER,
+                                           caravan, dest, src, TRUE);
+  consider_trade = parameter->consider_trade
+    && is_action_enabled_unit_on_city_full(ACTION_TRADE_ROUTE,
+                                           caravan, dest, src, TRUE);
+  consider_windfall = parameter->consider_windfall
+    && is_action_enabled_unit_on_city_full(ACTION_MARKETPLACE,
+                                           caravan, dest, src, TRUE);
+
+  if (!consider_wonder && !consider_trade && !consider_windfall) {
+    /* No caravan action is possible against this target. */
+    caravan_result_init_zero(result);
+
+    return FALSE;
   }
 
   trade = trade_benefit(pplayer_src, src, dest, parameter);
-  windfall = windfall_benefit(src, dest, parameter);
-  if (pplayer_src == pplayer_dest) {
+  windfall = windfall_benefit(caravan, src, dest, parameter);
+  if (consider_wonder) {
     wonder = wonder_benefit(caravan, arrival_time, dest, parameter);
     /* we want to aid for wonder building */
     wonder *= 2;
+
+    wonder = presentvalue(wonder, arrival_time, discount);
   } else {
-    wonder = 0;
+    wonder = -1.0;
   }
 
-  if (parameter->horizon == FC_INFINITY) {
-    trade = perpetuity(trade, discount);
+  if (consider_trade) {
+    if (parameter->horizon == FC_INFINITY) {
+      trade = perpetuity(trade, discount);
+    } else {
+      trade = annuity(trade, parameter->horizon - arrival_time, discount);
+    }
+    trade = presentvalue(trade, arrival_time, discount);
   } else {
-    trade = annuity(trade, parameter->horizon - arrival_time, discount);
+    trade = 0.0;
   }
-  trade = presentvalue(trade, arrival_time, discount);
-  windfall = presentvalue(windfall, arrival_time, discount);
-  wonder = presentvalue(wonder, arrival_time, discount);
 
-  if (trade + windfall >= wonder) {
+  if (consider_windfall) {
+    windfall = presentvalue(windfall, arrival_time, discount);
+  } else {
+    windfall = 0.0;
+  }
+
+  if (consider_trade && trade + windfall >= wonder) {
     result->value = trade + windfall;
     result->help_wonder = FALSE;
-  } else {
+  } else if (consider_wonder) {
     result->value = wonder;
     result->help_wonder = TRUE;
+  } else {
+    caravan_result_init_zero(result);
+    return FALSE;
   }
 
   if (parameter->callback) {
     parameter->callback(result, parameter->callback_data);
   }
+
+  return TRUE;
 }
 
-/***************************************************************************
+/****************************************************************************
   Functions to compute the benefit of moving the caravan to dest.
- ***************************************************************************/
+****************************************************************************/
 
-/***************************************************************************
+/****************************************************************************
   Ignoring the transit time, return the value of moving the caravan to
   dest.
- ***************************************************************************/
+****************************************************************************/
 static void caravan_evaluate_notransit(const struct unit *caravan,
                                        const struct city *dest,
                                        const struct caravan_parameter *param,
@@ -488,11 +558,10 @@ static void caravan_evaluate_notransit(const struct unit *caravan,
   get_discounted_reward(caravan, param, result);
 }
 
-
-/***************************************************************************
+/****************************************************************************
   Structure and callback for the caravan_search invocation in
   caravan_evaluate_withtransit.
- ***************************************************************************/
+****************************************************************************/
 struct cewt_data {
   const struct unit *caravan;
   struct caravan_result *result;
@@ -500,27 +569,30 @@ struct cewt_data {
 };
 
 static bool cewt_callback(void *vdata, const struct city *dest,
-                          int arrival_time, int moves_left) {
+                          int arrival_time, int moves_left)
+{
   struct cewt_data *data = vdata;
 
-  if(dest == data->result->dest) {
+  fc_assert_ret_val(data->result, FALSE);
+
+  if (dest == data->result->dest) {
     data->result->arrival_time = arrival_time;
     get_discounted_reward(data->caravan, data->param, data->result);
     return TRUE;
-  }
-  else {
+  } else {
     return FALSE;
   }
 }
 
-/***************************************************************************
+/****************************************************************************
   Using the caravan_search function to take transit time into account,
   evaluate the benefit of sending the caravan to dest.
- ***************************************************************************/
+****************************************************************************/
 static void caravan_evaluate_withtransit(const struct unit *caravan,
                                          const struct city *dest,
                                          const struct caravan_parameter *param,
-                                         struct caravan_result *result)
+                                         struct caravan_result *result,
+                                         bool omniscient)
 {
   struct cewt_data data;
 
@@ -529,53 +601,57 @@ static void caravan_evaluate_withtransit(const struct unit *caravan,
   caravan_result_init(result, game_city_by_number(caravan->homecity),
                       dest, 0);
   caravan_search_from(caravan, param, unit_tile(caravan), 0,
-                      caravan->moves_left, cewt_callback, &data);
+                      caravan->moves_left, omniscient, cewt_callback, &data);
 }
 
-
-/***************************************************************************
+/****************************************************************************
   Evaluate the value of sending the caravan to dest.
- ***************************************************************************/
+****************************************************************************/
 void caravan_evaluate(const struct unit *caravan,
                       const struct city *dest,
                       const struct caravan_parameter *param,
-                      struct caravan_result *result)
+                      struct caravan_result *result, bool omniscient)
 {
   if (param->ignore_transit_time) {
     caravan_evaluate_notransit(caravan, dest, param, result);
   } else {
-    caravan_evaluate_withtransit(caravan, dest, param, result);
+    caravan_evaluate_withtransit(caravan, dest, param, result, omniscient);
   }
 }
 
 
-/***************************************************************************
+/****************************************************************************
   Find the best destination for the caravan, ignoring transit time.
- ***************************************************************************/
+****************************************************************************/
 static void caravan_find_best_destination_notransit(const struct unit *caravan,
                                          const struct caravan_parameter *param,
-                                                   struct caravan_result *best)
+                                                    struct caravan_result *best)
 {
   struct caravan_result current;
   struct city *pcity = game_city_by_number(caravan->homecity);
-  
+  struct player *src_owner = city_owner(pcity);
+
   caravan_result_init(best, pcity, NULL, 0);
   current = *best;
-  
-  cities_iterate(dest) {
-    caravan_result_init(&current, pcity, dest, 0);
-    get_discounted_reward(caravan, param, &current);
-    
-    if (caravan_result_compare(&current, best) > 0) {
-      *best = current;
+
+  players_iterate(dest_owner) {
+    if (does_foreign_trade_param_allow(param, src_owner, dest_owner)) {
+      city_list_iterate(dest_owner->cities, dest) {
+        caravan_result_init(&current, pcity, dest, 0);
+        get_discounted_reward(caravan, param, &current);
+
+        if (caravan_result_compare(&current, best) > 0) {
+          *best = current;
+        }
+      } city_list_iterate_end;
     }
-  } cities_iterate_end;
+  } players_iterate_end;
 }
 
-/***************************************************************************
+/****************************************************************************
   Callback and struct for caravan_search invocation in
   caravan_find_best_destination_withtransit.
- ***************************************************************************/
+****************************************************************************/
 struct cfbdw_data {
   const struct caravan_parameter *param;
   const struct unit *caravan;
@@ -583,7 +659,8 @@ struct cfbdw_data {
 };
 
 static bool cfbdw_callback(void *vdata, const struct city *dest,
-                           int arrival_time, int moves_left) {
+                           int arrival_time, int moves_left)
+{
   struct cfbdw_data *data = vdata;
   struct caravan_result current;
 
@@ -597,15 +674,16 @@ static bool cfbdw_callback(void *vdata, const struct city *dest,
   return FALSE; /* don't stop. */
 }
 
-/***************************************************************************
+/****************************************************************************
   Using caravan_search, find the best destination.
- ***************************************************************************/
+****************************************************************************/
 static void caravan_find_best_destination_withtransit(
     const struct unit *caravan,
     const struct caravan_parameter *param,
     const struct city *src,
     int turns_before,
     int moves_left,
+    bool omniscient,
     struct caravan_result *result) 
 {
   struct tile *start_tile;
@@ -623,18 +701,17 @@ static void caravan_find_best_destination_withtransit(
   }
 
   caravan_search_from(caravan, param, start_tile, turns_before,
-                      caravan->moves_left, cfbdw_callback, &data);
+                      caravan->moves_left, omniscient, cfbdw_callback, &data);
 }
 
-
-/***************************************************************************
+/****************************************************************************
   Find the best destination city for the caravan.
   Store it in *destout (if destout is non-null); return the value of the
   trade route.
- ***************************************************************************/
+****************************************************************************/
 void caravan_find_best_destination(const struct unit *caravan,
                                    const struct caravan_parameter *parameter,
-                                   struct caravan_result *result)
+                                   struct caravan_result *result, bool omniscient)
 {
   if (parameter->ignore_transit_time) {
     caravan_find_best_destination_notransit(caravan, parameter, result);
@@ -643,54 +720,57 @@ void caravan_find_best_destination(const struct unit *caravan,
 
     fc_assert(src != NULL);
 
-    caravan_find_best_destination_withtransit(caravan, parameter, src, 0, 
-                                              caravan->moves_left, result);
+    caravan_find_best_destination_withtransit(caravan, parameter, src, 0,
+                                              caravan->moves_left, omniscient, result);
   }
 }
 
-
-
-/***************************************************************************
+/****************************************************************************
   Find the best pair-wise trade route, ignoring transit time.
- ***************************************************************************/
+****************************************************************************/
 static void caravan_optimize_notransit(const struct unit *caravan,
                                        const struct caravan_parameter *param,
                                        struct caravan_result *best)
 {
-  struct player *player = unit_owner(caravan);
+  struct player *pplayer = unit_owner(caravan);
 
   /* Iterate over all cities we own (since the caravan could change its
    * home city); iterate over all cities we know about (places the caravan
    * can go to); pick out the best trade route. */
-  city_list_iterate(player->cities, src) {
-    cities_iterate(dest) {
-      struct caravan_result current;
+  city_list_iterate(pplayer->cities, src) {
+    players_iterate(dest_owner) {
+      if (does_foreign_trade_param_allow(param, pplayer, dest_owner)) {
+        city_list_iterate(dest_owner->cities, dest) {
+          struct caravan_result current;
 
-      caravan_result_init(&current, src, dest, 0);
-      get_discounted_reward(caravan, param, &current);
-      if (caravan_result_compare(&current, best) > 0) {
-        *best = current;
+          caravan_result_init(&current, src, dest, 0);
+          get_discounted_reward(caravan, param, &current);
+          if (caravan_result_compare(&current, best) > 0) {
+            *best = current;
+          }
+        } city_list_iterate_end;
       }
-    } cities_iterate_end;
+    } players_iterate_end;
   } city_list_iterate_end;
 }
 
-
-/***************************************************************************
+/****************************************************************************
   Struct and callback for the caravan_search invocation in
   caravan_optimize_withtransit.
 
   For every city we can reach, use caravan_find_best_destination as a
   subroutine.
- ***************************************************************************/
+****************************************************************************/
 struct cowt_data {
   const struct caravan_parameter *param;
   const struct unit *caravan;
   struct caravan_result *best;
+  bool omniscient;
 };
 
 static bool cowt_callback(void *vdata, const struct city *pcity,
-                          int arrival_time, int moves_left) {
+                          int arrival_time, int moves_left)
+{
   struct cowt_data *data = vdata;
   const struct unit *caravan = data->caravan;
   struct caravan_result current;
@@ -707,7 +787,8 @@ static bool cowt_callback(void *vdata, const struct city *pcity,
   /* next, try changing home city (if we're allowed to) */
   if (city_owner(pcity) == unit_owner(caravan)) {
     caravan_find_best_destination_withtransit(
-        caravan, data->param, pcity, arrival_time, moves_left, &current);
+                caravan, data->param, pcity, arrival_time, moves_left, data->omniscient,
+                &current);
     if (caravan_result_compare(&current, data->best) > 0) {
       *data->best = current;
     }
@@ -716,37 +797,38 @@ static bool cowt_callback(void *vdata, const struct city *pcity,
   return FALSE; /* don't stop searching */
 }
 
-/***************************************************************************
+/****************************************************************************
   Find the best src/dest pair (including possibly changing home city), taking
   account of the trip time.
- ***************************************************************************/
+****************************************************************************/
 static void caravan_optimize_withtransit(const struct unit *caravan,
-                                        const struct caravan_parameter *param,
-                                        struct caravan_result *result)
+                                         const struct caravan_parameter *param,
+                                         struct caravan_result *result,
+                                         bool omniscient)
 {
   struct cowt_data data;
 
   data.param = param;
   data.caravan = caravan;
   data.best = result;
+  data.omniscient = omniscient;
   caravan_result_init_zero(data.best);
   caravan_search_from(caravan, param, unit_tile(caravan), 0,
-                      caravan->moves_left, cowt_callback, &data);
+                      caravan->moves_left, omniscient, cowt_callback, &data);
 }
 
-
-/***************************************************************************
+/****************************************************************************
   For every city the caravan can change home in, find the best destination.
   Return the best src/dest pair by reference (if non-null), and return
   the value of that trade route.
- ***************************************************************************/
+****************************************************************************/
 void caravan_optimize_allpairs(const struct unit *caravan,
                                const struct caravan_parameter *param,
-                               struct caravan_result *result)
+                               struct caravan_result *result, bool omniscient)
 {
   if (param->ignore_transit_time) {
     caravan_optimize_notransit(caravan, param, result);
   } else {
-    caravan_optimize_withtransit(caravan, param, result);
+    caravan_optimize_withtransit(caravan, param, result, omniscient);
   }
 }
