@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,7 +22,6 @@ extern "C" {
 #include "log.h"
 
 /* common */
-#include "ai.h" /* FC_AI_LAST */
 #include "fc_types.h"
 #include "name_translation.h"
 #include "improvement.h"
@@ -40,19 +39,25 @@ enum production_class_type {
 
 /* Various city options.  These are stored by the server and can be
  * toggled by the user.  Each one defaults to off.  Adding new ones
- * will break network compatibility.  Reordering them will break savegame
- * compatibility.  If you want to remove one you should replace it with
- * a CITYO_UNUSED entry; new options can just be added at the end.
+ * will break network compatibility. If you want to reorder or remove
+ * an option remeber to load the city option order from the savegame.
+ * It is stored in savefile.city_options_vector
  *
  * Used in the network protocol.
  */
-enum city_options {
-  CITYO_DISBAND,      /* If building a settler at size 1 disbands the city */
-  CITYO_NEW_EINSTEIN, /* If new citizens are science specialists */
-  CITYO_NEW_TAXMAN,   /* If new citizens are gold specialists */
-  CITYO_LAST
-};
-BV_DEFINE(bv_city_options, CITYO_LAST); /* Used in the network protocol. */
+#define SPECENUM_NAME city_options
+/* If building a settler at size 1 disbands the city */
+#define SPECENUM_VALUE0 CITYO_DISBAND
+#define SPECENUM_VALUE0NAME "Disband"
+/* If new citizens are science specialists */
+#define SPECENUM_VALUE1 CITYO_NEW_EINSTEIN
+#define SPECENUM_VALUE1NAME "New_Einstein"
+/* If new citizens are gold specialists */
+#define SPECENUM_VALUE2 CITYO_NEW_TAXMAN
+#define SPECENUM_VALUE2NAME "New_Taxman"
+#define SPECENUM_COUNT CITYO_LAST
+#define SPECENUM_BITVECTOR bv_city_options
+#include "specenum_gen.h"
 
 /* Changing the max radius requires updating network capabilities and results
  * in incompatible savefiles. */
@@ -302,6 +307,7 @@ struct city {
   struct player *owner; /* Cannot be NULL. */
   struct player *original; /* Cannot be NULL. */
   int id;
+  int style;
 
   /* the people */
   citizens size;
@@ -377,6 +383,10 @@ struct city {
 
   struct unit_list *units_supported;
 
+  int history;                 /* Cumulative culture */
+
+  struct worker_task_list *task_reqs;
+
   union {
     struct {
       /* Only used in the server (./ai/ and ./server/). */
@@ -405,20 +415,19 @@ struct city {
       bool debug;                   /* not saved */
 
       struct adv_city *adv;
-      void *ais[FC_AI_LAST];
+      void *ais[FREECIV_AI_MOD_LAST];
 
       struct vision *vision;
-
-      struct worker_task task_req;
     } server;
 
     struct {
       /* Only used at the client (the server is omniscient; ./client/). */
       bool occupied;
-      bool walls;
+      int  walls;
       bool happy;
       bool unhappy;
       int  city_image;
+      int  culture;
 
       /* The color is an index into the city_colors array in mapview_common */
       bool colored;
@@ -434,6 +443,8 @@ struct city {
 
       /* Updates needed for the city. */
       enum city_updates need_updates;
+
+      unsigned char first_citizen_index;
     } client;
   };
 };
@@ -442,13 +453,10 @@ struct citystyle {
   struct name_translation name;
   char graphic[MAX_LEN_NAME];
   char graphic_alt[MAX_LEN_NAME];
-  char oceanic_graphic[MAX_LEN_NAME];
-  char oceanic_graphic_alt[MAX_LEN_NAME];
   char citizens_graphic[MAX_LEN_NAME];
   char citizens_graphic_alt[MAX_LEN_NAME];
   struct requirement_vector reqs;
-  int replaced_by;              /* index to replacing style          */
-};                              /* not incl. wall and occupied tiles */
+};
 
 extern struct citystyle *city_styles;
 extern const Output_type_id num_output_types;
@@ -510,7 +518,7 @@ void set_city_production(struct city *pcity);
 
 /* properties */
 
-const char *city_name(const struct city *pcity);
+const char *city_name_get(const struct city *pcity);
 struct player *city_owner(const struct city *pcity);
 struct tile *city_tile(const struct city *pcity);
 
@@ -553,11 +561,11 @@ bool can_city_build_unit_now(const struct city *pcity,
 			     const struct unit_type *punittype);
 
 bool can_city_build_direct(const struct city *pcity,
-			   struct universal target);
+                           const struct universal *target);
 bool can_city_build_later(const struct city *pcity,
-			  struct universal target);
+                          const struct universal *target);
 bool can_city_build_now(const struct city *pcity,
-			struct universal target);
+                        const struct universal *target);
 
 bool city_can_use_specialist(const struct city *pcity,
 			     Specialist_type_id type);
@@ -565,7 +573,6 @@ bool city_has_building(const struct city *pcity,
 		       const struct impr_type *pimprove);
 bool is_capital(const struct city *pcity);
 bool is_gov_center(const struct city *pcity);
-bool city_got_citywalls(const struct city *pcity);
 bool city_got_defense_effect(const struct city *pcity,
                              const struct unit_type *attacker);
 
@@ -577,12 +584,12 @@ int city_production_buy_gold_cost(const struct city *pcity);
 bool city_production_has_flag(const struct city *pcity,
 			      enum impr_flag_id flag);
 int city_production_turns_to_build(const struct city *pcity,
-				   bool include_shield_stock);
+                                   bool include_shield_stock);
 
 int city_change_production_penalty(const struct city *pcity,
-				   struct universal target);
+                                   const struct universal *target);
 int city_turns_to_build(const struct city *pcity,
-			struct universal target,
+                        const struct universal *target,
                         bool include_shield_stock);
 int city_turns_to_grow(const struct city *pcity);
 bool city_can_grow_to(const struct city *pcity, int pop_size);
@@ -645,10 +652,6 @@ const char *city_style_name_translation(const int style);
 int city_style_by_rule_name(const char *s);
 int city_style_by_translated_name(const char *s);
 
-bool city_style_has_requirements(const struct citystyle *style);
-int city_style_of_player(const struct player *plr);
-int style_of_city(const struct city *pcity);
-
 struct city *is_enemy_city_tile(const struct tile *ptile,
 				const struct player *pplayer);
 struct city *is_allied_city_tile(const struct tile *ptile,
@@ -741,8 +744,6 @@ bool is_free_worked(const struct city *pcity, const struct tile *ptile);
   (CITY_MAP_CENTER_TILE_INDEX == city_tile_index)
 #define FREE_WORKED_TILES (1)
 
-enum citytile_type citytile_by_rule_name(const char *name);
-
 void *city_ai_data(const struct city *pcity, const struct ai_type *ai);
 void city_set_ai_data(struct city *pcity, const struct ai_type *ai,
                       void *data);
