@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,11 +15,13 @@
 #include <fc_config.h>
 #endif
 
+#include "fc_prehdrs.h"
+
 #include <errno.h>
 #include <string.h>
 #include <time.h>
 
-#ifdef HAVE_SYS_TYPES_H
+#ifdef FREECIV_HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
 #ifdef HAVE_SYS_SELECT_H
@@ -29,13 +31,6 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#ifdef HAVE_WINSOCK
-#ifdef HAVE_WINSOCK2
-#include <winsock2.h>
-#else  /* HAVE_WINSOCK2 */
-#include <winsock.h>
-#endif /* HAVE_WINSOCK2 */
-#endif /* HAVE_WINSOCK */
 
 /* utility */
 #include "fcintl.h"
@@ -102,7 +97,6 @@ void connection_close(struct connection *pconn, const char *reason)
   (*conn_close_callback) (pconn);
 }
 
-
 /**************************************************************************
   Make sure that there is at least extra_space bytes free space in buffer,
   allocating more memory if needed.
@@ -244,6 +238,7 @@ void flush_connection_send_buffer_all(struct connection *pc)
 /**************************************************************************
   flush'em
 **************************************************************************/
+#ifndef FREECIV_JSON_CONNECTION
 static void flush_connection_send_buffer_packets(struct connection *pc)
 {
   if(pc && pc->used && pc->send_buffer->ndata >= MAX_LEN_PACKET) {
@@ -254,6 +249,7 @@ static void flush_connection_send_buffer_packets(struct connection *pc)
     }
   }
 }
+#endif /* FREECIV_JSON_CONNECTION */
 
 /****************************************************************************
   Add data to send to the connection.
@@ -294,6 +290,8 @@ bool connection_send_data(struct connection *pconn,
   }
 
   pconn->statistics.bytes_send += len;
+
+#ifndef FREECIV_JSON_CONNECTION
   if (0 < pconn->send_buffer->do_buffer_sends) {
     flush_connection_send_buffer_packets(pconn);
     if (!add_connection_data(pconn, data, len)) {
@@ -302,7 +300,9 @@ bool connection_send_data(struct connection *pconn,
       return FALSE;
     }
     flush_connection_send_buffer_packets(pconn);
-  } else {
+  } else
+#endif /* FREECIV_JSON_CONNECTION */
+  {
     flush_connection_send_buffer_all(pconn);
     if (!add_connection_data(pconn, data, len)) {
       log_verbose("cut connection %s due to huge send buffer (2)",
@@ -412,6 +412,9 @@ struct connection *conn_by_user_prefix(const char *user_name,
 struct connection *conn_by_number(int id)
 {
   conn_list_iterate(game.all_connections, pconn) {
+    fc_assert_msg(pconn != NULL,
+                  "Trying to look at the id of a non existing connection");
+
     if (pconn->id == id) {
       return pconn;
     }
@@ -540,12 +543,11 @@ static void init_packet_hashs(struct connection *pc)
 
   pc->phs.sent = fc_malloc(sizeof(*pc->phs.sent) * PACKET_LAST);
   pc->phs.received = fc_malloc(sizeof(*pc->phs.received) * PACKET_LAST);
-  pc->phs.variant = fc_malloc(sizeof(*pc->phs.variant) * PACKET_LAST);
+  pc->phs.handlers = packet_handlers_initial();
 
   for (i = 0; i < PACKET_LAST; i++) {
     pc->phs.sent[i] = NULL;
     pc->phs.received[i] = NULL;
-    pc->phs.variant[i] = -1;
   }
 }
 
@@ -575,11 +577,6 @@ static void free_packet_hashes(struct connection *pc)
     free(pc->phs.received);
     pc->phs.received = NULL;
   }
-
-  if (pc->phs.variant) {
-    free(pc->phs.variant);
-    pc->phs.variant = NULL;
-  }
 }
 
 /**************************************************************************
@@ -636,6 +633,16 @@ void connection_common_close(struct connection *pconn)
   }
 }
 
+/****************************************************************************
+  Set the network capability string for 'pconn'.
+****************************************************************************/
+void conn_set_capability(struct connection *pconn, const char *capability)
+{
+  fc_assert(strlen(capability) < sizeof(pconn->capability));
+  sz_strlcpy(pconn->capability, capability);
+  pconn->phs.handlers = packet_handlers_get(capability);
+}
+
 /**************************************************************************
   Remove all is-game-info cached packets from the connection. This resets
   the delta-state partially.
