@@ -145,11 +145,20 @@ extern "C" {
 /* A unit with population cost was built; the city shrinks. */
 #define SPECENUM_VALUE109 E_UNIT_BUILT_POP_COST
 #define SPECENUM_VALUE110 E_DISASTER
+#define SPECENUM_VALUE111 E_ACHIEVEMENT
+#define SPECENUM_VALUE112 E_TECH_LOST
+/* Used to notify tech events for foreigner players. */
+#define SPECENUM_VALUE113 E_TECH_EMBASSY
+#define SPECENUM_VALUE114 E_MY_SPY_STEAL_GOLD
+#define SPECENUM_VALUE115 E_ENEMY_SPY_STEAL_GOLD
+#define SPECENUM_VALUE116 E_UNIT_ILLEGAL_ACTION
+#define SPECENUM_VALUE117 E_DEPRECATION_WARNING
+
 /*
  * Note: If you add a new event, make sure you make a similar change
- * to the events array in "common/events.c" using GEN_EV, and to
+ * to the events array in "common/events.c" using GEN_EV, to
  * "data/stdsounds.soundspec", which serves as the documentation to
- * soundset authors.
+ * soundset authors, and to "data/misc/events.spec".
  */
 #define SPECENUM_COUNT E_COUNT
 /* The sound system also generates "e_game_quit", although there's no
@@ -161,7 +170,7 @@ extern enum event_type sorted_events[]; /* [E_COUNT], sorted by the
                                            translated message text */
 
 const char *get_event_message_text(enum event_type event);
-const char *get_event_sound_tag(enum event_type event);
+const char *get_event_tag(enum event_type event);
 
 bool is_city_event(enum event_type event);
 
