@@ -17,17 +17,22 @@
 extern "C" {
 #endif /* __cplusplus */
 
+/* utility */
 #include "support.h"
 
+/* common */
+#include "vision.h"
+
 struct player;
 struct tile;
-enum vision_layer;
 struct color;
-struct base_type;
+struct extra_type;
 
 /* The existence of each function should be checked in interface_init()! */
 struct functions {
-  void (*destroy_base)(struct tile *ptile, struct base_type *pbase);
+  void (*create_extra)(struct tile *ptile, struct extra_type *pextra,
+                       struct player *pplayer);
+  void (*destroy_extra)(struct tile *ptile, struct extra_type *pextra);
   /* Returns iff the player 'pplayer' has the vision in the layer
      'vision' at tile given by 'ptile'. */
   bool (*player_tile_vision_get)(const struct tile *ptile,
