@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 #include "game.h"
 #include "map.h"
 #include "tech.h"
+#include "victory.h"
 
 #include "improvement.h"
 
@@ -53,6 +54,8 @@ void improvements_init(void)
 
     p->item_number = i;
     requirement_vector_init(&p->reqs);
+    requirement_vector_init(&p->obsolete_by);
+    p->disabled = FALSE;
   }
 }
 
@@ -67,6 +70,7 @@ static void improvement_free(struct impr_type *p)
   }
 
   requirement_vector_free(&p->reqs);
+  requirement_vector_free(&p->obsolete_by);
 }
 
 /***************************************************************
@@ -79,6 +83,48 @@ void improvements_free()
   } improvement_iterate_end;
 }
 
+/**************************************************************************
+  Cache features of the improvement
+**************************************************************************/
+void improvement_feature_cache_init(void)
+{
+  improvement_iterate(pimprove) {
+    pimprove->allows_units = FALSE;
+    unit_type_iterate(putype) {
+      if (putype->need_improvement == pimprove) {
+        pimprove->allows_units = TRUE;
+        break;
+      }
+    } unit_type_iterate_end;
+
+    pimprove->allows_extras = FALSE;
+    extra_type_iterate(pextra) {
+      if (requirement_needs_improvement(pimprove, &pextra->reqs)) {
+        pimprove->allows_extras = TRUE;
+        break;
+      }
+    } extra_type_iterate_end;
+
+    pimprove->prevents_disaster = FALSE;
+    disaster_type_iterate(pdis) {
+      if (!requirement_fulfilled_by_improvement(pimprove, &pdis->reqs)) {
+        pimprove->prevents_disaster = TRUE;
+        break;
+      }
+    } disaster_type_iterate_end;
+
+    pimprove->protects_vs_actions = FALSE;
+    action_enablers_iterate(act) {
+      if (!requirement_fulfilled_by_improvement(pimprove,
+                                                &act->target_reqs)) {
+        pimprove->protects_vs_actions = TRUE;
+        break;
+      }
+    } action_enablers_iterate_end;
+
+  } improvement_iterate_end;
+}
+
 /**************************************************************************
   Return the first item of improvements.
 **************************************************************************/
@@ -157,7 +203,7 @@ struct impr_type *valid_improvement(struct impr_type *pimprove)
     return NULL;
   }
 
-  if (!game.info.spacerace
+  if (!victory_enabled(VC_SPACERACE)
       && (building_has_effect(pimprove, EFT_SS_STRUCTURAL)
 	  || building_has_effect(pimprove, EFT_SS_COMPONENT)
 	  || building_has_effect(pimprove, EFT_SS_MODULE))) {
@@ -185,7 +231,7 @@ struct impr_type *valid_improvement_by_number(const Impr_type_id id)
 **************************************************************************/
 const char *improvement_name_translation(const struct impr_type *pimprove)
 {
-  return name_translation(&pimprove->name);
+  return name_translation_get(&pimprove->name);
 }
 
 /****************************************************************************
@@ -194,7 +240,7 @@ const char *improvement_name_translation(const struct impr_type *pimprove)
 ****************************************************************************/
 const char *improvement_rule_name(const struct impr_type *pimprove)
 {
-  return rule_name(&pimprove->name);
+  return rule_name_get(&pimprove->name);
 }
 
 /****************************************************************************
@@ -307,50 +353,188 @@ bool is_improvement_visible(const struct impr_type *pimprove)
 **************************************************************************/
 bool can_improvement_go_obsolete(const struct impr_type *pimprove)
 {
-  return pimprove->obsolete_by != NULL;
+  return requirement_vector_size(&pimprove->obsolete_by) > 0;
 }
 
 /**************************************************************************
   Returns TRUE if the improvement or wonder is obsolete
 **************************************************************************/
 bool improvement_obsolete(const struct player *pplayer,
-			  const struct impr_type *pimprove)
+			  const struct impr_type *pimprove,
+                          const struct city *pcity)
 {
-  if (!valid_advance(pimprove->obsolete_by)) {
-    return FALSE;
+  struct tile *ptile = NULL;
+
+  if (pcity != NULL) {
+    ptile = city_tile(pcity);
   }
 
-  if (is_great_wonder(pimprove)) {
-    /* a great wonder is obsolete, as soon as *any* player researched the
-       obsolete tech */
-   return game.info.global_advances[advance_index(pimprove->obsolete_by)];
-  }
+  requirement_vector_iterate(&pimprove->obsolete_by, preq) {
+    if (is_req_active(pplayer, NULL, pcity, pimprove, ptile, NULL, NULL,
+                      NULL, NULL, preq, RPT_CERTAIN)) {
+      return TRUE;
+    }
+  } requirement_vector_iterate_end;
 
-  return (player_invention_state(pplayer, advance_number(pimprove->obsolete_by)) == TECH_KNOWN);
+  return FALSE;
 }
 
 /**************************************************************************
-  Returns TRUE iff improvement provides units buildable by player
+  Returns TRUE iff improvement provides units buildable in city
 **************************************************************************/
-bool impr_provides_buildable_units(const struct player *pplayer,
-                                   const struct impr_type *pimprove)
+static bool impr_provides_buildable_units(const struct city *pcity,
+                                          struct impr_type *pimprove)
 {
   /* Fast check */
-  if (! pimprove->allows_units) {
+  if (!pimprove->allows_units) {
     return FALSE;
   }
 
   unit_type_iterate(ut) {
-    if (ut->need_improvement == pimprove) {
-      if (can_player_build_unit_now(pplayer, ut)) {
-        return TRUE;
-      }
+    if (ut->need_improvement == pimprove
+        && can_city_build_unit_now(pcity, ut)) {
+      return TRUE;
     }
   } unit_type_iterate_end;
 
   return FALSE;
 }
 
+/**************************************************************************
+  Returns TRUE iff improvement provides extras buildable in city
+**************************************************************************/
+static bool impr_provides_buildable_extras(const struct city *pcity,
+                                           struct impr_type *pimprove)
+{
+
+  /* Fast check */
+  if (!pimprove->allows_extras) {
+    return FALSE;
+  }
+
+  extra_type_iterate(pextra) {
+    if (requirement_needs_improvement(pimprove, &pextra->reqs)) {
+      city_tile_iterate(city_map_radius_sq_get(pcity),
+                        city_tile(pcity), ptile) {
+        if (player_can_build_extra(pextra, city_owner(pcity), ptile)) {
+          return TRUE;
+        }
+      } city_tile_iterate_end;
+    }
+  } extra_type_iterate_end;
+
+  return FALSE;
+}
+
+/**************************************************************************
+  Returns TRUE iff improvement prevents a disaster in city
+**************************************************************************/
+static bool impr_prevents_disaster(const struct city *pcity,
+                                   struct impr_type *pimprove)
+{
+  /* Fast check */
+  if (!pimprove->prevents_disaster) {
+    return FALSE;
+  }
+
+  disaster_type_iterate(pdis) {
+    if (!requirement_fulfilled_by_improvement(pimprove, &pdis->reqs)
+        && !can_disaster_happen(pdis, pcity)) {
+      return TRUE;
+    }
+  } disaster_type_iterate_end;
+
+  return FALSE;
+}
+
+/**************************************************************************
+  Returns TRUE iff improvement protects against an action on the city
+  FIXME: This is prone to false positives: for example, if one requires
+         a special tech or unit to perform an action, and no other player
+         has or can gain that tech or unit, protection is still claimed.
+**************************************************************************/
+static bool impr_protects_vs_actions(const struct city *pcity,
+                                     struct impr_type *pimprove)
+{
+  /* Fast check */
+  if (!pimprove->protects_vs_actions) {
+    return FALSE;
+  }
+
+  action_enablers_iterate(act) {
+    if (!requirement_fulfilled_by_improvement(pimprove, &act->target_reqs)
+        && !is_action_possible_on_city(act->action, NULL, pcity)) {
+      return TRUE;
+    }
+  } action_enablers_iterate_end;
+
+  return FALSE;
+}
+
+
+/**************************************************************************
+  Check if an improvement has side effects for a city.  Side effects
+  are any benefits that accrue that are not tracked by the effects
+  system.
+
+  Note that this function will always return FALSE if the improvement does
+  not currently provide a benefit to the city (for example, if the improvement
+  has not yet been built, or another city benefits from the improvement in
+  this city (i.e. Wonders)).
+**************************************************************************/
+static bool improvement_has_side_effects(const struct city *pcity,
+                                         struct impr_type *pimprove)
+{
+    /* FIXME: There should probably also be a test as to whether
+     *        the improvement *enables* an action (somewhere else),
+     *        but this is hard to determine at city scope. */
+
+    return (impr_provides_buildable_units(pcity, pimprove)
+            || impr_provides_buildable_extras(pcity, pimprove)
+            || impr_prevents_disaster(pcity, pimprove)
+            || impr_protects_vs_actions(pcity, pimprove));
+}
+
+/**************************************************************************
+  Returns TRUE iff improvement provides some effect (good or bad).
+**************************************************************************/
+static bool improvement_has_effects(const struct city *pcity,
+                                    struct impr_type *pimprove)
+{
+  struct universal source = { .kind = VUT_IMPROVEMENT,
+                              .value = { .building = pimprove } };
+  struct effect_list *plist = get_req_source_effects(&source);
+
+  if (!plist || improvement_obsolete(city_owner(pcity), pimprove, pcity)) {
+    return FALSE;
+  }
+
+  effect_list_iterate(plist, peffect) {
+    if (0 != get_potential_improvement_bonus(pimprove, pcity,
+                                             peffect->type, RPT_CERTAIN)) {
+      return TRUE;
+    }
+  } effect_list_iterate_end;
+
+  return FALSE;
+}
+
+/**************************************************************************
+  Returns TRUE if an improvement in a city is productive, in some way.
+
+  Note that unproductive improvements may become productive later, if
+  appropriate conditions are met (e.g. a special building that isn't
+  producing units under the current government might under another).
+**************************************************************************/
+bool is_improvement_productive(const struct city *pcity,
+                               struct impr_type *pimprove)
+{
+    return (!improvement_obsolete(city_owner(pcity), pimprove, pcity)
+            && (improvement_has_flag(pimprove, IF_GOLD)
+                || improvement_has_side_effects(pcity, pimprove)
+                || improvement_has_effects(pcity, pimprove)));
+}
+
 /**************************************************************************
   Returns TRUE if an improvement in a city is redundant, that is, the
   city wouldn't lose anything by losing the improvement, or gain anything
@@ -371,17 +555,15 @@ bool is_improvement_redundant(const struct city *pcity,
     return FALSE;
   }
 
-  /* If an improvement allows building of units, don't claim it's redundant.
-   * (FIXME: if enabling units were done via the effects system, then we
-   * could decide which of two improvements enabling a unit was "better".) */
-  if (impr_provides_buildable_units(city_owner(pcity), pimprove)) {
+  /* If an improvement has side effects, don't claim it's redundant. */
+  if (improvement_has_side_effects(pcity, pimprove)) {
     return FALSE;
   }
 
   /* Otherwise, it's redundant if its effects are available by other means,
    * or if it's an obsolete wonder (great or small). */
   return is_building_replaced(pcity, pimprove, RPT_CERTAIN)
-      || improvement_obsolete(city_owner(pcity), pimprove);
+    || improvement_obsolete(city_owner(pcity), pimprove, pcity);
 }
 
 /**************************************************************************
@@ -399,8 +581,8 @@ bool can_player_build_improvement_direct(const struct player *p,
 
   requirement_vector_iterate(&pimprove->reqs, preq) {
     if (preq->range >= REQ_RANGE_PLAYER
-        && !is_req_active(p, NULL, NULL, NULL, NULL, NULL, NULL, preq,
-                          RPT_CERTAIN)) {
+        && !is_req_active(p, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+                          NULL, preq, RPT_CERTAIN)) {
       return FALSE;
     }
   } requirement_vector_iterate_end;
@@ -451,7 +633,7 @@ bool can_player_build_improvement_now(const struct player *p,
   if (!can_player_build_improvement_direct(p, pimprove)) {
     return FALSE;
   }
-  if (improvement_obsolete(p, pimprove)) {
+  if (improvement_obsolete(p, pimprove, NULL)) {
     return FALSE;
   }
   return TRUE;
@@ -468,7 +650,7 @@ bool can_player_build_improvement_later(const struct player *p,
   if (!valid_improvement(pimprove)) {
     return FALSE;
   }
-  if (improvement_obsolete(p, pimprove)) {
+  if (improvement_obsolete(p, pimprove, NULL)) {
     return FALSE;
   }
   if (is_great_wonder(pimprove) && !great_wonder_is_available(pimprove)) {
@@ -481,8 +663,8 @@ bool can_player_build_improvement_later(const struct player *p,
   requirement_vector_iterate(&pimprove->reqs, preq) {
     if (preq->range >= REQ_RANGE_PLAYER
 	&& is_req_unchanging(preq)
-        && !is_req_active(p, NULL, NULL, NULL, NULL, NULL, NULL, preq,
-                          RPT_POSSIBLE)) {
+	&& !is_req_active(p, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+                          NULL, preq, RPT_POSSIBLE)) {
       return FALSE;
     }
   } requirement_vector_iterate_end;
@@ -531,16 +713,16 @@ bool is_special_improvement(const struct impr_type *pimprove)
 void wonder_built(const struct city *pcity, const struct impr_type *pimprove)
 {
   struct player *pplayer;
-  int index = improvement_number(pimprove);
+  int windex = improvement_number(pimprove);
 
   fc_assert_ret(NULL != pcity);
   fc_assert_ret(is_wonder(pimprove));
 
   pplayer = city_owner(pcity);
-  pplayer->wonders[index] = pcity->id;
+  pplayer->wonders[windex] = pcity->id;
 
   if (is_great_wonder(pimprove)) {
-    game.info.great_wonder_owners[index] = player_number(pplayer);
+    game.info.great_wonder_owners[windex] = player_number(pplayer);
   }
 }
 
@@ -552,19 +734,19 @@ void wonder_destroyed(const struct city *pcity,
                       const struct impr_type *pimprove)
 {
   struct player *pplayer;
-  int index = improvement_number(pimprove);
+  int windex = improvement_number(pimprove);
 
   fc_assert_ret(NULL != pcity);
   fc_assert_ret(is_wonder(pimprove));
 
   pplayer = city_owner(pcity);
-  fc_assert_ret(pplayer->wonders[index] == pcity->id);
-  pplayer->wonders[index] = WONDER_LOST;
+  fc_assert_ret(pplayer->wonders[windex] == pcity->id);
+  pplayer->wonders[windex] = WONDER_LOST;
 
   if (is_great_wonder(pimprove)) {
-    fc_assert_ret(game.info.great_wonder_owners[index]
+    fc_assert_ret(game.info.great_wonder_owners[windex]
                    == player_number(pplayer));
-    game.info.great_wonder_owners[index] = WONDER_DESTROYED;
+    game.info.great_wonder_owners[windex] = WONDER_DESTROYED;
   }
 }
 
@@ -612,7 +794,7 @@ struct city *city_from_wonder(const struct player *pplayer,
     return NULL;
   }
 
-#ifdef DEBUG
+#ifdef FREECIV_DEBUG
   if (is_server()) {
     /* On client side, this info is not always known. */
     struct city *pcity = player_city_by_number(pplayer, city_id);
@@ -628,13 +810,13 @@ struct city *city_from_wonder(const struct player *pplayer,
                 "%s (nb %d), the city %s (nb %d) doesn't have this wonder.",
                 player_name(pplayer), player_number(pplayer),
                 improvement_rule_name(pimprove),
-                improvement_number(pimprove), city_name(pcity), pcity->id);
+                improvement_number(pimprove), city_name_get(pcity), pcity->id);
       return NULL;
     }
 
     return pcity;
   }
-#endif /* DEBUG */
+#endif /* FREECIV_DEBUG */
 
   return player_city_by_number(pplayer, city_id);
 }
@@ -683,7 +865,7 @@ struct city *city_from_great_wonder(const struct impr_type *pimprove)
   fc_assert_ret_val(is_great_wonder(pimprove), NULL);
 
   if (WONDER_OWNED(player_id)) {
-#ifdef DEBUG
+#ifdef FREECIV_DEBUG
     const struct player *pplayer = player_by_number(player_id);
     struct city *pcity = city_from_wonder(pplayer, pimprove);
 
@@ -698,7 +880,7 @@ struct city *city_from_great_wonder(const struct impr_type *pimprove)
     return pcity;
 #else
     return city_from_wonder(player_by_number(player_id), pimprove);
-#endif /* DEBUG */
+#endif /* FREECIV_DEBUG */
   } else {
     return NULL;
   }
@@ -791,3 +973,19 @@ enum test_result test_player_sell_building_now(struct player *pplayer,
 
   return TR_SUCCESS;
 }
+
+
+/****************************************************************************
+  Try to find a sensible replacement building, based on other buildings
+  that may have caused this one to become obsolete.
+****************************************************************************/
+struct impr_type *improvement_replacement(const struct impr_type *pimprove)
+{
+  requirement_vector_iterate(&pimprove->obsolete_by, pobs) {
+    if (pobs->source.kind == VUT_IMPROVEMENT && pobs->present) {
+      return pobs->source.value.building;
+    }
+  } requirement_vector_iterate_end;
+
+  return NULL;
+}
