@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,6 +15,8 @@
 #include <fc_config.h>
 #endif
 
+#include "fc_prehdrs.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -26,16 +28,10 @@
 #ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
-#ifdef HAVE_WINSOCK
-#ifdef HAVE_WINSOCK2
-#include <winsock2.h>
-#else  /* HAVE_WINSOCK2 */
-#include <winsock.h>
-#endif /* HAVE_WINSOCK2 */
-#endif /* HAVE_WINSOCK */
 
 /* utility */
 #include "capability.h"
+#include "fc_cmdline.h"
 #include "fcintl.h"
 #include "log.h"
 #include "mem.h"
@@ -76,12 +72,24 @@
  */
 #define PACKET_SIZE_STATISTICS 0
 
+extern const char *const packet_functional_capability;
+
+#define SPECHASH_TAG packet_handler
+#define SPECHASH_ASTR_KEY_TYPE
+#define SPECHASH_IDATA_TYPE struct packet_handlers *
+#define SPECHASH_IDATA_FREE (packet_handler_hash_data_free_fn_t) free
+#include "spechash.h"
+
+static struct packet_handler_hash *packet_handlers = NULL;
+
 #ifdef USE_COMPRESSION
 static int stat_size_alone = 0;
 static int stat_size_uncompressed = 0;
 static int stat_size_compressed = 0;
 static int stat_size_no_compression = 0;
 
+#define MAX_DECOMPRESSION 400
+
 /****************************************************************************
   Returns the compression level. Initilialize it if needed.
 ****************************************************************************/
@@ -371,6 +379,7 @@ void *get_packet_from_connection(struct connection *pc,
   int header_size = 0;
 #endif
   void *data;
+  void *(*receive_handler)(struct connection *);
 
   if (!pc->used) {
     return NULL;		/* connection was closed, stop reading */
@@ -426,25 +435,34 @@ void *get_packet_from_connection(struct connection *pc,
 
   if (compressed_packet) {
     uLong compressed_size = whole_packet_len - header_size;
-    /* 
-     * We don't know the decompressed size. We assume a bad case
-     * here: an expansion by an factor of 100. 
-     */
-    unsigned long int decompressed_size = 100 * compressed_size;
-    void *decompressed = fc_malloc(decompressed_size);
-    int error;
+    int decompress_factor = 80;
+    unsigned long int decompressed_size = decompress_factor * compressed_size;
+    int error = Z_DATA_ERROR;
     struct socket_packet_buffer *buffer = pc->buffer;
-    
-    error =
-	uncompress(decompressed, &decompressed_size,
-		   ADD_TO_POINTER(buffer->data, header_size), 
-		   compressed_size);
-    if (error != Z_OK) {
-      log_verbose("Uncompressing of the packet stream failed. "
-                  "The connection will be closed now.");
-      connection_close(pc, _("decoding error"));
-      return NULL;
-    }
+    void *decompressed = fc_malloc(decompressed_size);
+
+    do {
+      error =
+        uncompress(decompressed, &decompressed_size,
+                   ADD_TO_POINTER(buffer->data, header_size),
+                   compressed_size);
+
+      if (error == Z_DATA_ERROR) {
+        decompress_factor += 50;
+        decompressed_size = decompress_factor * compressed_size;
+        decompressed = fc_realloc(decompressed, decompressed_size);
+      }
+
+      if (error != Z_OK) {
+        if (error != Z_DATA_ERROR || decompress_factor > MAX_DECOMPRESSION ) {
+          log_verbose("Uncompressing of the packet stream failed. "
+                      "The connection will be closed now.");
+          connection_close(pc, _("decoding error"));
+          return NULL;
+        }
+      }
+
+    } while (error != Z_OK);
 
     buffer->ndata -= whole_packet_len;
     /* 
@@ -495,6 +513,16 @@ void *get_packet_from_connection(struct connection *pc,
   dio_get_type(&din, pc->packet_header.type, &utype.itype);
   utype.type = utype.itype;
 
+  if (utype.type < 0
+      || utype.type >= PACKET_LAST
+      || (receive_handler = pc->phs.handlers->receive[utype.type]) == NULL) {
+    log_verbose("Received unsupported packet type %d (%s). The connection "
+                "will be closed now.",
+                utype.type, packet_name(utype.type));
+    connection_close(pc, _("unsupported packet type"));
+    return NULL;
+  }
+
   log_packet("got packet type=(%s)%d len=%d from %s",
              packet_name(utype.type), utype.itype, whole_packet_len,
              is_server() ? pc->username : "server");
@@ -549,7 +577,7 @@ void *get_packet_from_connection(struct connection *pc,
     }
   }
 #endif /* PACKET_SIZE_STATISTICS */
-  data = get_packet_from_connection_helper(pc, utype.type);
+  data = receive_handler(pc);
   if (!data) {
     connection_close(pc, _("incompatible packet contents"));
     return NULL;
@@ -771,3 +799,86 @@ void pre_send_packet_player_attribute_chunk(struct connection *pc,
              packet->offset, packet->total_length, packet->chunk_length);
 
 }
+
+/****************************************************************************
+  Destroy the packet handler hash table.
+****************************************************************************/
+static void packet_handlers_free(void)
+{
+  if (packet_handlers != NULL) {
+    packet_handler_hash_destroy(packet_handlers);
+    packet_handlers = NULL;
+  }
+}
+
+/****************************************************************************
+  Returns the packet handlers variant with no special capability.
+****************************************************************************/
+const struct packet_handlers *packet_handlers_initial(void)
+{
+  static struct packet_handlers default_handlers;
+  static bool initialized = FALSE;
+
+  if (!initialized) {
+    memset(&default_handlers, 0, sizeof(default_handlers));
+    packet_handlers_fill_initial(&default_handlers);
+    initialized = TRUE;
+  }
+
+  return &default_handlers;
+}
+
+/****************************************************************************
+  Returns the packet handlers variant for 'capability'.
+****************************************************************************/
+const struct packet_handlers *packet_handlers_get(const char *capability)
+{
+  struct packet_handlers *phandlers;
+  char functional_capability[MAX_LEN_CAPSTR] = "";
+  char *tokens[MAX_LEN_CAPSTR / 2];
+  int tokens_num;
+  int i;
+
+  fc_assert(strlen(capability) < sizeof(functional_capability));
+
+  /* Get functional network capability string. */
+  tokens_num = get_tokens(capability, tokens, ARRAY_SIZE(tokens), " \t\n,");
+  qsort(tokens, tokens_num, sizeof(*tokens), compare_strings_ptrs);
+  for (i = 0; i < tokens_num; i++) {
+    if (!has_capability(tokens[i], packet_functional_capability)) {
+      continue;
+    }
+    if (functional_capability[0] != '\0') {
+      sz_strlcat(functional_capability, " ");
+    }
+    sz_strlcat(functional_capability, tokens[i]);
+  }
+  free_tokens(tokens, tokens_num);
+
+  /* Ensure the hash table is created. */
+  if (packet_handlers == NULL) {
+    packet_handlers = packet_handler_hash_new();
+  }
+
+  /* Lookup handlers for the capabilities or create new handlers. */
+  if (!packet_handler_hash_lookup(packet_handlers, functional_capability,
+                                  &phandlers)) {
+    phandlers = fc_malloc(sizeof(*phandlers));
+    memcpy(phandlers, packet_handlers_initial(), sizeof(*phandlers));
+    packet_handlers_fill_capability(phandlers, functional_capability);
+    packet_handler_hash_insert(packet_handlers, functional_capability,
+                               phandlers);
+  }
+
+  fc_assert(phandlers != NULL);
+  return phandlers;
+}
+
+/****************************************************************************
+  Call when there is no longer a requirement for protocol processing.
+  All connections must have been closed.
+****************************************************************************/
+void packets_deinit(void)
+{
+  packet_handlers_free();
+}
