@@ -16,7 +16,10 @@
 #endif
 
 /* common */
+#include "achievements.h"
+#include "actions.h"
 #include "citizens.h"
+#include "culture.h"
 #include "game.h"
 #include "government.h"
 #include "improvement.h"
@@ -199,16 +202,82 @@ int api_methods_city_inspire_partisans(lua_State *L, City *self, Player *inspire
   if (inspired) {
     /* Cannot use get_city_bonus() as it would use city's current owner
      * instead of inspirer. */
-    return get_target_bonus_effects(NULL, inspirer, self, NULL,
-                                    city_tile(self), NULL, NULL,
+    return get_target_bonus_effects(NULL, inspirer, NULL, self, NULL,
+                                    city_tile(self), NULL, NULL, NULL,
                                     NULL, EFT_INSPIRE_PARTISANS);
   }
 
   return 0;
 }
 
+/**************************************************************************
+  How much culture city has?
+**************************************************************************/
+int api_methods_city_culture_get(lua_State *L, City *pcity)
+{
+  LUASCRIPT_CHECK_STATE(L, 0);
+  LUASCRIPT_CHECK_SELF(L, pcity, 0);
+
+  return city_culture(pcity);
+}
+
 /*****************************************************************************
-  Return rule name for Government
+  Return TRUE iff city happy
+*****************************************************************************/
+bool api_methods_is_city_happy(lua_State *L, City *pcity)
+{
+  LUASCRIPT_CHECK_STATE(L, NULL);
+  LUASCRIPT_CHECK_SELF(L, pcity, FALSE);
+
+  return city_happy(pcity);
+}
+
+/*****************************************************************************
+  Return TRUE iff city is unhappy
+*****************************************************************************/
+bool api_methods_is_city_unhappy(lua_State *L, City *pcity)
+{
+  LUASCRIPT_CHECK_STATE(L, NULL);
+  LUASCRIPT_CHECK_SELF(L, pcity, FALSE);
+
+  return city_unhappy(pcity);
+}
+
+/*****************************************************************************
+  Return TRUE iff city is celebrating
+*****************************************************************************/
+bool api_methods_is_city_celebrating(lua_State *L, City *pcity)
+{
+  LUASCRIPT_CHECK_STATE(L, NULL);
+  LUASCRIPT_CHECK_SELF(L, pcity, FALSE);
+
+  return city_celebrating(pcity);
+}
+
+/*****************************************************************************
+  Return TRUE iff city is government center
+*****************************************************************************/
+bool api_methods_is_gov_center(lua_State *L, City *pcity)
+{
+  LUASCRIPT_CHECK_STATE(L, NULL);
+  LUASCRIPT_CHECK_SELF(L, pcity, FALSE);
+
+  return is_gov_center(pcity);
+}
+
+/*****************************************************************************
+  Return TRUE if city is capital
+*****************************************************************************/
+bool api_methods_is_capital(lua_State *L, City *pcity)
+{
+  LUASCRIPT_CHECK_STATE(L, NULL);
+  LUASCRIPT_CHECK_SELF(L, pcity, FALSE);
+
+  return is_capital(pcity);
+}
+
+/*****************************************************************************
+   Return rule name for Government
 *****************************************************************************/
 const char *api_methods_government_rule_name(lua_State *L,
                                              Government *pgovernment)
@@ -268,7 +337,6 @@ const char *api_methods_nation_type_plural_translation(lua_State *L,
   return nation_plural_translation(pnation);
 }
 
-
 /*****************************************************************************
   Return TRUE iff player has wonder
 *****************************************************************************/
@@ -336,7 +404,19 @@ bool api_methods_player_knows_tech(lua_State *L, Player *pplayer,
   LUASCRIPT_CHECK_SELF(L, pplayer, FALSE);
   LUASCRIPT_CHECK_ARG_NIL(L, ptech, 3, Tech_Type, FALSE);
 
-  return player_invention_state(pplayer, advance_number(ptech)) == TECH_KNOWN;
+  return research_invention_state(research_get(pplayer),
+                                  advance_number(ptech)) == TECH_KNOWN;
+}
+
+/**************************************************************************
+  How much culture player has?
+**************************************************************************/
+int api_methods_player_culture_get(lua_State *L, Player *pplayer)
+{
+  LUASCRIPT_CHECK_STATE(L, 0);
+  LUASCRIPT_CHECK_SELF(L, pplayer, 0);
+
+  return player_culture(pplayer);
 }
 
 /*****************************************************************************
@@ -349,7 +429,29 @@ bool api_methods_player_shares_research(lua_State *L, Player *pplayer,
   LUASCRIPT_CHECK_SELF(L, pplayer, FALSE);
   LUASCRIPT_CHECK_ARG_NIL(L, aplayer, 3, Player, FALSE);
 
-  return player_research_get(pplayer) == player_research_get(aplayer);
+  return research_get(pplayer) == research_get(aplayer);
+}
+
+/*****************************************************************************
+  Return name of the research group player belongs to.
+*****************************************************************************/
+const char *api_methods_research_rule_name(lua_State *L, Player *pplayer)
+{
+  LUASCRIPT_CHECK_STATE(L, FALSE);
+  LUASCRIPT_CHECK_SELF(L, pplayer, FALSE);
+
+  return research_rule_name(research_get(pplayer));
+}
+
+/*****************************************************************************
+  Return name of the research group player belongs to.
+*****************************************************************************/
+const char *api_methods_research_name_translation(lua_State *L, Player *pplayer)
+{
+  LUASCRIPT_CHECK_STATE(L, FALSE);
+  LUASCRIPT_CHECK_SELF(L, pplayer, FALSE);
+
+  return research_name_translation(research_get(pplayer));
 }
 
 /*****************************************************************************
@@ -421,6 +523,17 @@ const char *api_methods_terrain_name_translation(lua_State *L,
   return terrain_name_translation(pterrain);
 }
 
+/*****************************************************************************
+  Return name of the terrain's class
+*****************************************************************************/
+const char *api_methods_terrain_class_name(lua_State *L, Terrain *pterrain)
+{
+  LUASCRIPT_CHECK_STATE(L, NULL);
+  LUASCRIPT_CHECK_SELF(L, pterrain, NULL);
+
+  return terrain_class_name(terrain_type_terrain_class(pterrain));
+}
+
 /*****************************************************************************
   Return rule name for Disaster
 *****************************************************************************/
@@ -444,6 +557,51 @@ const char *api_methods_disaster_name_translation(lua_State *L,
   return disaster_name_translation(pdis);
 }
 
+/*****************************************************************************
+  Return rule name for Achievement
+*****************************************************************************/
+const char *api_methods_achievement_rule_name(lua_State *L, Achievement *pach)
+{
+  LUASCRIPT_CHECK_STATE(L, NULL);
+  LUASCRIPT_CHECK_SELF(L, pach, NULL);
+
+  return achievement_rule_name(pach);
+}
+
+/*****************************************************************************
+  Return translated name for Achievement
+*****************************************************************************/
+const char *api_methods_achievement_name_translation(lua_State *L,
+                                                     Achievement *pach)
+{
+  LUASCRIPT_CHECK_STATE(L, NULL);
+  LUASCRIPT_CHECK_SELF(L, pach, NULL);
+
+  return achievement_name_translation(pach);
+}
+
+/*****************************************************************************
+  Return rule name for Action
+*****************************************************************************/
+const char *api_methods_action_rule_name(lua_State *L, Action *pact)
+{
+  LUASCRIPT_CHECK_STATE(L, NULL);
+  LUASCRIPT_CHECK_SELF(L, pact, NULL);
+
+  return action_id_rule_name(pact->id);
+}
+
+/*****************************************************************************
+  Return translated name for Action
+*****************************************************************************/
+const char *api_methods_action_name_translation(lua_State *L, Action *pact)
+{
+  LUASCRIPT_CHECK_STATE(L, NULL);
+  LUASCRIPT_CHECK_SELF(L, pact, NULL);
+
+  return action_id_name_translation(pact->id);
+}
+
 /*****************************************************************************
   Return the native x coordinate of the tile.
 *****************************************************************************/
@@ -512,6 +670,32 @@ bool api_methods_tile_city_exists_within_max_city_map(lua_State *L,
   return city_exists_within_max_city_map(ptile, may_be_on_center);
 }
 
+/*****************************************************************************
+  Return TRUE if there is a extra with rule name name on ptile.
+  If no name is specified return true if there is a extra on ptile.
+*****************************************************************************/
+bool api_methods_tile_has_extra(lua_State *L, Tile *ptile, const char *name)
+{
+  LUASCRIPT_CHECK_STATE(L, FALSE);
+  LUASCRIPT_CHECK_SELF(L, ptile, FALSE);
+
+  if (!name) {
+    extra_type_iterate(pextra) {
+      if (tile_has_extra(ptile, pextra)) {
+        return TRUE;
+      }
+    } extra_type_iterate_end;
+
+    return FALSE;
+  } else {
+    struct extra_type *pextra;
+
+    pextra = extra_type_by_rule_name(name);
+
+    return (NULL != pextra && tile_has_extra(ptile, pextra));
+  }
+}
+
 /*****************************************************************************
   Return TRUE if there is a base with rule name name on ptile.
   If no name is specified return true if there is any base on ptile.
@@ -522,13 +706,20 @@ bool api_methods_tile_has_base(lua_State *L, Tile *ptile, const char *name)
   LUASCRIPT_CHECK_SELF(L, ptile, FALSE);
 
   if (!name) {
-    return tile_has_any_bases(ptile);
+    extra_type_by_cause_iterate(EC_BASE, pextra) {
+      if (tile_has_extra(ptile, pextra)) {
+        return TRUE;
+      }
+    } extra_type_by_cause_iterate_end;
+
+    return FALSE;
   } else {
-    struct base_type *pbase;
+    struct extra_type *pextra;
 
-    pbase = base_type_by_rule_name(name);
+    pextra = extra_type_by_rule_name(name);
 
-    return (NULL != pbase && tile_has_base(ptile, pbase));
+    return (NULL != pextra && is_extra_caused_by(pextra, EC_BASE)
+            && tile_has_extra(ptile, pextra));
   }
 }
 
@@ -542,13 +733,20 @@ bool api_methods_tile_has_road(lua_State *L, Tile *ptile, const char *name)
   LUASCRIPT_CHECK_SELF(L, ptile, FALSE);
 
   if (!name) {
-    return BV_ISSET_ANY(ptile->roads);
+    extra_type_by_cause_iterate(EC_ROAD, pextra) {
+      if (tile_has_extra(ptile, pextra)) {
+        return TRUE;
+      }
+    } extra_type_by_cause_iterate_end;
+
+    return FALSE;
   } else {
-    struct road_type *proad;
+    struct extra_type *pextra;
+ 
+    pextra = extra_type_by_rule_name(name);
 
-    proad = road_type_by_rule_name(name);
-
-    return (NULL != proad && tile_has_road(ptile, proad));
+    return (NULL != pextra && is_extra_caused_by(pextra, EC_ROAD)
+            && tile_has_extra(ptile, pextra));
   }
 }
 
@@ -580,7 +778,7 @@ Unit_List_Link *api_methods_private_tile_unit_list_head(lua_State *L,
   Will return the next index, or an index < 0 when done
 *****************************************************************************/
 int api_methods_private_tile_next_outward_index(lua_State *L, Tile *pstart,
-                                                int index, int max_dist)
+                                                int tindex, int max_dist)
 {
   int dx, dy;
   int newx, newy;
@@ -589,26 +787,26 @@ int api_methods_private_tile_next_outward_index(lua_State *L, Tile *pstart,
   LUASCRIPT_CHECK_STATE(L, 0);
   LUASCRIPT_CHECK_SELF(L, pstart, 0);
 
-  if (index < 0) {
+  if (tindex < 0) {
     return 0;
   }
 
   index_to_map_pos(&startx, &starty, tile_index(pstart));
 
-  index++;
-  while (index < map.num_iterate_outwards_indices) {
-    if (map.iterate_outwards_indices[index].dist > max_dist) {
+  tindex++;
+  while (tindex < game.map.num_iterate_outwards_indices) {
+    if (game.map.iterate_outwards_indices[tindex].dist > max_dist) {
       return -1;
     }
-    dx = map.iterate_outwards_indices[index].dx;
-    dy = map.iterate_outwards_indices[index].dy;
+    dx = game.map.iterate_outwards_indices[tindex].dx;
+    dy = game.map.iterate_outwards_indices[tindex].dy;
     newx = dx + startx;
     newy = dy + starty;
     if (!normalize_map_pos(&newx, &newy)) {
-      index++;
+      tindex++;
       continue;
     }
-    return index;
+    return tindex;
   }
   return -1;
 }
@@ -617,19 +815,19 @@ int api_methods_private_tile_next_outward_index(lua_State *L, Tile *pstart,
   Return tile for nth iteration index (for internal use)
 *****************************************************************************/
 Tile *api_methods_private_tile_for_outward_index(lua_State *L, Tile *pstart,
-                                                 int index)
+                                                 int tindex)
 {
   int newx, newy;
 
   LUASCRIPT_CHECK_STATE(L, NULL);
   LUASCRIPT_CHECK_SELF(L, pstart, NULL);
-  LUASCRIPT_CHECK_ARG(L, index >= 0
-                         && index < map.num_iterate_outwards_indices, 3,
-                      "index out of bounds", NULL);
+  LUASCRIPT_CHECK_ARG(L,
+                      tindex >= 0 && tindex < game.map.num_iterate_outwards_indices,
+                      3, "index out of bounds", NULL);
 
   index_to_map_pos(&newx, &newy, tile_index(pstart));
-  newx += map.iterate_outwards_indices[index].dx;
-  newy += map.iterate_outwards_indices[index].dy;
+  newx += game.map.iterate_outwards_indices[tindex].dx;
+  newy += game.map.iterate_outwards_indices[tindex].dy;
 
   if (!normalize_map_pos(&newx, &newy)) {
     return NULL;
@@ -682,6 +880,28 @@ Direction api_methods_unit_orientation_get(lua_State *L, Unit *punit)
   return punit->facing;
 }
 
+/*****************************************************************************
+  Return Unit that transports punit, if any.
+*****************************************************************************/
+Unit *api_methods_unit_transporter(lua_State *L, Unit *punit)
+{
+  LUASCRIPT_CHECK_STATE(L, NULL);
+  LUASCRIPT_CHECK_SELF(L, punit, NULL);
+
+  return punit->transporter;
+}
+
+/*****************************************************************************
+  Return list head for cargo list for Unit
+*****************************************************************************/
+Unit_List_Link *api_methods_private_unit_cargo_list_head(lua_State *L,
+                                                         Unit *punit)
+{
+  LUASCRIPT_CHECK_STATE(L, NULL);
+  LUASCRIPT_CHECK_SELF(L, punit, NULL);
+  return unit_list_head(punit->transporting);
+}
+
 /*****************************************************************************
   Return TRUE if punit_type has flag.
 *****************************************************************************/
@@ -767,42 +987,42 @@ const char *api_methods_unit_type_name_translation(lua_State *L,
   Return Unit for list link
 *****************************************************************************/
 Unit *api_methods_unit_list_link_data(lua_State *L,
-                                      Unit_List_Link *link)
+                                      Unit_List_Link *ul_link)
 {
   LUASCRIPT_CHECK_STATE(L, NULL);
 
-  return unit_list_link_data(link);
+  return unit_list_link_data(ul_link);
 }
 
 /*****************************************************************************
   Return next list link or NULL when link is the last link
 *****************************************************************************/
 Unit_List_Link *api_methods_unit_list_next_link(lua_State *L,
-                                                Unit_List_Link *link)
+                                                Unit_List_Link *ul_link)
 {
   LUASCRIPT_CHECK_STATE(L, NULL);
 
-  return unit_list_link_next(link);
+  return unit_list_link_next(ul_link);
 }
 
 /*****************************************************************************
   Return City for list link
 *****************************************************************************/
 City *api_methods_city_list_link_data(lua_State *L,
-                                      City_List_Link *link)
+                                      City_List_Link *cl_link)
 {
   LUASCRIPT_CHECK_STATE(L, NULL);
 
-  return city_list_link_data(link);
+  return city_list_link_data(cl_link);
 }
 
 /*****************************************************************************
   Return next list link or NULL when link is the last link
 *****************************************************************************/
 City_List_Link *api_methods_city_list_next_link(lua_State *L,
-                                                City_List_Link *link)
+                                                City_List_Link *cl_link)
 {
   LUASCRIPT_CHECK_STATE(L, NULL);
 
-  return city_list_link_next(link);
+  return city_list_link_next(cl_link);
 }
