@@ -1,4 +1,4 @@
-/********************************************************************** 
+/**********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -38,23 +38,6 @@ enum special_river_move {
   RMV_FAST_ALWAYS = 3,
 };
 
-/* S_LAST-terminated */
-extern enum tile_special_type infrastructure_specials[];
-
-BV_DEFINE(bv_special, S_LAST); /* Used in the network protocol. */
-
-/* NB: This does not include S_FORTRESS and S_AIRBASE.
- * You must use base_type_iterate and related accessors
- * in base.h for those. */
-#define tile_special_type_iterate(special)                                 \
-{                                                                          \
-  enum tile_special_type special = 0;                                      \
-  for (; special < S_LAST; special++) {                                    \
-    
-#define tile_special_type_iterate_end                                      \
-  }                                                                        \
-}
-
 /* === */
 
 struct resource {
@@ -85,6 +68,7 @@ struct resource {
  * Used in the network protocol. */
 #define MAX_NUM_RESOURCES (MAX_NUM_TERRAINS/2)
 
+/* Used in the network protocol. */
 #define SPECENUM_NAME terrain_class
 #define SPECENUM_VALUE0 TC_LAND
 /* TRANS: terrain class: used adjectivally */
@@ -96,63 +80,72 @@ struct resource {
 
 /* Types of alterations available to terrain.
  * This enum is only used in the effects system; the relevant information
- * is encoded in other members of the terrain structure. */
+ * is encoded in other members of the terrain structure.
+ *
+ * Used in the network protocol. */
 #define SPECENUM_NAME terrain_alteration
 /* Can build irrigation without changing terrain */
 #define SPECENUM_VALUE0 TA_CAN_IRRIGATE
-#define SPECENUM_VALUE0NAME "CanIrrigate"
+/* TRANS: this and following strings may rarely be presented to the player
+ * in ruleset help text, to denote the set of terrains which can be altered
+ * in a particular way */
+#define SPECENUM_VALUE0NAME N_("CanIrrigate")
 /* Can build mine without changing terrain */
 #define SPECENUM_VALUE1 TA_CAN_MINE
-#define SPECENUM_VALUE1NAME "CanMine"
+#define SPECENUM_VALUE1NAME N_("CanMine")
 /* Can build roads and/or railroads */
 #define SPECENUM_VALUE2 TA_CAN_ROAD
-#define SPECENUM_VALUE2NAME "CanRoad"
+#define SPECENUM_VALUE2NAME N_("CanRoad")
 #include "specenum_gen.h"
 
+/* Used in the network protocol. */
 #define SPECENUM_NAME terrain_flag_id
 /* No barbarians summoned on this terrain. */
 #define SPECENUM_VALUE0 TER_NO_BARBS
 /* TRANS: this and following strings are 'terrain flags', which may rarely
  * be presented to the player in ruleset help text */
 #define SPECENUM_VALUE0NAME N_("NoBarbs")
-/* This terrain cannot be polluted. */
-#define SPECENUM_VALUE1 TER_NO_POLLUTION
-#define SPECENUM_VALUE1NAME N_("NoPollution")
 /* No cities on this terrain. */
-#define SPECENUM_VALUE2 TER_NO_CITIES
-#define SPECENUM_VALUE2NAME N_("NoCities")
+#define SPECENUM_VALUE1 TER_NO_CITIES
+#define SPECENUM_VALUE1NAME N_("NoCities")
 /* Players will start on this terrain type. */
-#define SPECENUM_VALUE3 TER_STARTER
-#define SPECENUM_VALUE3NAME N_("Starter")
+#define SPECENUM_VALUE2 TER_STARTER
+#define SPECENUM_VALUE2NAME N_("Starter")
 /* Terrains with this type can have road with "River" flag on them. */
-#define SPECENUM_VALUE4 TER_CAN_HAVE_RIVER
-#define SPECENUM_VALUE4NAME N_("CanHaveRiver")
+#define SPECENUM_VALUE3 TER_CAN_HAVE_RIVER
+#define SPECENUM_VALUE3NAME N_("CanHaveRiver")
 /*this tile is not safe as coast, (all ocean / ice) */
-#define SPECENUM_VALUE5 TER_UNSAFE_COAST
-#define SPECENUM_VALUE5NAME N_("UnsafeCoast")
+#define SPECENUM_VALUE4 TER_UNSAFE_COAST
+#define SPECENUM_VALUE4NAME N_("UnsafeCoast")
 /* Fresh water terrain */
-#define SPECENUM_VALUE6 TER_FRESHWATER
-#define SPECENUM_VALUE6NAME N_("FreshWater")
+#define SPECENUM_VALUE5 TER_FRESHWATER
+#define SPECENUM_VALUE5NAME N_("FreshWater")
 /* Map generator does not place this terrain */
-#define SPECENUM_VALUE7 TER_NOT_GENERATED
-#define SPECENUM_VALUE7NAME N_("NotGenerated")
-#define SPECENUM_VALUE8 TER_USER_1
-#define SPECENUM_VALUE9 TER_USER_2
-#define SPECENUM_VALUE10 TER_USER_3
-#define SPECENUM_VALUE11 TER_USER_4
-#define SPECENUM_VALUE12 TER_USER_5
-#define SPECENUM_VALUE13 TER_USER_6
-#define SPECENUM_VALUE14 TER_USER_7
-#define SPECENUM_VALUE15 TER_USER_LAST
+#define SPECENUM_VALUE6 TER_NOT_GENERATED
+#define SPECENUM_VALUE6NAME N_("NotGenerated")
+/* Units on this terrain are not generating or subject to zoc */
+#define SPECENUM_VALUE7 TER_NO_ZOC
+#define SPECENUM_VALUE7NAME N_("NoZoc")
+/* No unit can fortify on this terrain */
+#define SPECENUM_VALUE8 TER_NO_FORTIFY
+#define SPECENUM_VALUE8NAME N_("NoFortify")
+/* Ice-covered terrain (affects minimap) */
+#define SPECENUM_VALUE9 TER_FROZEN
+#define SPECENUM_VALUE9NAME N_("Frozen")
+#define SPECENUM_VALUE10 TER_USER_1
+#define SPECENUM_VALUE11 TER_USER_2
+#define SPECENUM_VALUE12 TER_USER_3
+#define SPECENUM_VALUE13 TER_USER_4
+#define SPECENUM_VALUE14 TER_USER_5
+#define SPECENUM_VALUE15 TER_USER_6
+#define SPECENUM_VALUE16 TER_USER_7
+#define SPECENUM_VALUE17 TER_USER_LAST
 #define SPECENUM_NAMEOVERRIDE
+#define SPECENUM_BITVECTOR bv_terrain_flags
 #include "specenum_gen.h"
 
-#define TER_MAX 64 /* Changing this breaks network compatability. */
-
 #define MAX_NUM_USER_TER_FLAGS (TER_USER_LAST - TER_USER_1 + 1)
 
-BV_DEFINE(bv_terrain_flags, TER_MAX); /* Used in the network protocol. */
-
 #define SPECENUM_NAME mapgen_terrain_property
 #define SPECENUM_VALUE0 MG_MOUNTAINOUS
 #define SPECENUM_VALUE0NAME "mountainous"
@@ -187,7 +180,9 @@ struct terrain {
   char graphic_str[MAX_LEN_NAME];	/* add tile_ prefix */
   char graphic_alt[MAX_LEN_NAME];
 
-  char identifier; /* Single-character identifier used in savegames. */
+  char identifier; /* Single-character identifier used in games saved. */
+  char identifier_load; /* Single-character identifier that was used in the savegame
+                         * loaded. */
 
 #define TERRAIN_UNKNOWN_IDENTIFIER 'u'
 
@@ -216,6 +211,9 @@ struct terrain {
   int transform_time;
   int clean_pollution_time;
   int clean_fallout_time;
+  int pillage_time;
+
+  struct unit_type *animal;
 
   /* May be NULL if the transformation is impossible. */
   struct terrain *warmer_wetter_result, *warmer_drier_result;
@@ -263,8 +261,6 @@ const char *terrain_name_translation(const struct terrain *pterrain);
 /* Functions to operate on a terrain flag. */
 #define terrain_has_flag(terr, flag) BV_ISSET((terr)->flags, flag)
 
-int terrains_by_flag(enum terrain_flag_id flag, struct terrain **buffer, int bufsize);
-
 bool is_terrain_flag_card_near(const struct tile *ptile,
 			       enum terrain_flag_id flag);
 bool is_terrain_flag_near_tile(const struct tile *ptile,
@@ -297,8 +293,8 @@ int count_terrain_near_tile(const struct tile *ptile,
 			    bool cardinal_only, bool percentage,
 			    const struct terrain *pterrain);
 int count_terrain_property_near_tile(const struct tile *ptile,
-				     bool cardinal_only, bool percentage,
-				     enum mapgen_terrain_property prop);
+                                     bool cardinal_only, bool percentage,
+                                     enum mapgen_terrain_property prop);
 
 bool is_resource_card_near(const struct tile *ptile,
                            const struct resource *pres,
@@ -319,47 +315,16 @@ struct resource *resource_by_rule_name(const char *name);
 const char *resource_rule_name(const struct resource *presource);
 const char *resource_name_translation(const struct resource *presource);
 
-/* General special accessor functions. */
-enum tile_special_type special_by_rule_name(const char *name);
-const char *special_rule_name(enum tile_special_type type);
-const char *special_name_translation(enum tile_special_type type);
-
-void set_special(bv_special *set, enum tile_special_type to_set);
-void clear_special(bv_special *set, enum tile_special_type to_clear);
-void clear_all_specials(bv_special *set);
-bool contains_special(bv_special all,
-		      enum tile_special_type to_test_for);
-bool contains_any_specials(bv_special all);
-
-bool is_native_terrain_to_special(enum tile_special_type special,
-                                  const struct terrain *pterrain);
-bool is_native_tile_to_special(enum tile_special_type special,
-                               const struct tile *ptile);
-
 /* Special helper functions */
-const char *get_infrastructure_text(bv_special pset, bv_bases bases, bv_roads roads);
-enum tile_special_type get_infrastructure_prereq(enum tile_special_type spe);
-bool get_preferred_pillage(struct act_tgt *tgt,
-                           bv_special pset,
-                           bv_bases bases,
-                           bv_roads roads);
+const char *get_infrastructure_text(bv_extras extras);
+struct extra_type *get_preferred_pillage(bv_extras extras);
 
-int terrain_base_time(const struct terrain *pterrain,
-                      Base_type_id base);
-
-int terrain_road_time(const struct terrain *pterrain,
-                      Road_type_id road);
-
-/* Functions to operate on a terrain special. */
-bool is_special_card_near(const struct tile *ptile,
-                          enum tile_special_type spe,
-                          bool check_self);
-bool is_special_near_tile(const struct tile *ptile,
-			  enum tile_special_type spe,
-                          bool check_self);
-int count_special_near_tile(const struct tile *ptile,
-			    bool cardinal_only, bool percentage,
-			    enum tile_special_type spe);
+int terrain_extra_build_time(const struct terrain *pterrain,
+                             enum unit_activity activity,
+                             const struct extra_type *tgt);
+int terrain_extra_removal_time(const struct terrain *pterrain,
+                               enum unit_activity activity,
+                               const struct extra_type *tgt);
 
 /* Functions to operate on a terrain class. */
 const char *terrain_class_name_translation(enum terrain_class tclass);
@@ -372,7 +337,6 @@ int count_terrain_class_near_tile(const struct tile *ptile,
                                   enum terrain_class tclass);
 
 /* Functions to deal with possible terrain alterations. */
-const char *terrain_alteration_name_translation(enum terrain_alteration talter);
 bool terrain_can_support_alteration(const struct terrain *pterrain,
                                     enum terrain_alteration talter);
 
