@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -37,64 +37,87 @@ struct ai_type;
  * Used in the network protocol. */
 #define U_LAST MAX_NUM_ITEMS
 
-#define SPECENUM_NAME unit_class_flag_id
-#define SPECENUM_VALUE0 UCF_TERRAIN_SPEED
-#define SPECENUM_VALUE0NAME "TerrainSpeed"
-#define SPECENUM_VALUE1 UCF_TERRAIN_DEFENSE
-#define SPECENUM_VALUE1NAME "TerrainDefense"
-#define SPECENUM_VALUE2 UCF_DAMAGE_SLOWS
-#define SPECENUM_VALUE2NAME "DamageSlows"
-/* Can occupy enemy cities */
-#define SPECENUM_VALUE3 UCF_CAN_OCCUPY_CITY
-#define SPECENUM_VALUE3NAME "CanOccupyCity"
-#define SPECENUM_VALUE4 UCF_MISSILE
-#define SPECENUM_VALUE4NAME "Missile"
-#define SPECENUM_VALUE5 UCF_BUILD_ANYWHERE
-#define SPECENUM_VALUE5NAME "BuildAnywhere"
-#define SPECENUM_VALUE6 UCF_UNREACHABLE
-#define SPECENUM_VALUE6NAME "Unreachable"
-/* Can collect ransom from barbarian leader */
-#define SPECENUM_VALUE7 UCF_COLLECT_RANSOM
-#define SPECENUM_VALUE7NAME "CollectRansom"
-/* Is subject to ZOC */
-#define SPECENUM_VALUE8 UCF_ZOC
-#define SPECENUM_VALUE8NAME "ZOC"
-/* Can fortify on land squares */
-#define SPECENUM_VALUE9 UCF_CAN_FORTIFY
-#define SPECENUM_VALUE9NAME "CanFortify"
-#define SPECENUM_VALUE10 UCF_CAN_PILLAGE
-#define SPECENUM_VALUE10NAME "CanPillage"
-/* Cities can still work tile when enemy unit on it */
-#define SPECENUM_VALUE11 UCF_DOESNT_OCCUPY_TILE
-#define SPECENUM_VALUE11NAME "DoesntOccupyTile"
-/* Can attack against units on non-native tiles */
-#define SPECENUM_VALUE12 UCF_ATTACK_NON_NATIVE
-#define SPECENUM_VALUE12NAME "AttackNonNative"
-/* Can launch attack from non-native tile (to native tile) */
-#define SPECENUM_VALUE13 UCF_ATT_FROM_NON_NATIVE
-#define SPECENUM_VALUE13NAME "AttFromNonNative"
-/* Kills citizens upon successful attack against a city */
-#define SPECENUM_VALUE14 UCF_KILLCITIZEN
-#define SPECENUM_VALUE14NAME "KillCitizen"
-/* keep this last */
-#define SPECENUM_COUNT UCF_COUNT
-#include "specenum_gen.h"
-
 /* The largest distance a ruleset can allow a unit to paradrop.
+ *
  * Remember to make sure that the field type of PACKET_RULESET_UNIT's
  * paratroopers_range field can transfer the new maximum if you increase
- * it. */
-#define UNIT_MAX_PARADROP_RANGE 255
+ * it.
+ *
+ * The top value is reserved in case a future Freeciv version wants to
+ * implement "no maximum range". It could be used to signal that the unit
+ * can paradrop anywhere. Note that the value below it is high enough to
+ * give the same effect on all maps inside the current size limits.
+ * (No map side can be larger than MAP_MAX_LINEAR_SIZE)
+ */
+#define UNIT_MAX_PARADROP_RANGE (65535 - 1)
+
+/* Used in the network protocol. */
+#define SPECENUM_NAME unit_class_flag_id
+#define SPECENUM_VALUE0 UCF_TERRAIN_SPEED
+#define SPECENUM_VALUE0NAME N_("?uclassflag:TerrainSpeed")
+#define SPECENUM_VALUE1 UCF_TERRAIN_DEFENSE
+#define SPECENUM_VALUE1NAME N_("?uclassflag:TerrainDefense")
+#define SPECENUM_VALUE2 UCF_DAMAGE_SLOWS
+#define SPECENUM_VALUE2NAME N_("?uclassflag:DamageSlows")
+/* Can occupy enemy cities */
+#define SPECENUM_VALUE3 UCF_CAN_OCCUPY_CITY
+#define SPECENUM_VALUE3NAME N_("?uclassflag:CanOccupyCity")
+#define SPECENUM_VALUE4 UCF_MISSILE
+#define SPECENUM_VALUE4NAME N_("?uclassflag:Missile")
+#define SPECENUM_VALUE5 UCF_BUILD_ANYWHERE
+#define SPECENUM_VALUE5NAME N_("?uclassflag:BuildAnywhere")
+#define SPECENUM_VALUE6 UCF_UNREACHABLE
+#define SPECENUM_VALUE6NAME N_("?uclassflag:Unreachable")
+/* Can collect ransom from barbarian leader */
+#define SPECENUM_VALUE7 UCF_COLLECT_RANSOM
+#define SPECENUM_VALUE7NAME N_("?uclassflag:CollectRansom")
+/* Is subject to ZOC */
+#define SPECENUM_VALUE8 UCF_ZOC
+#define SPECENUM_VALUE8NAME N_("?uclassflag:ZOC")
+/* Can fortify on land squares */
+#define SPECENUM_VALUE9 UCF_CAN_FORTIFY
+#define SPECENUM_VALUE9NAME N_("?uclassflag:CanFortify")
+#define SPECENUM_VALUE10 UCF_CAN_PILLAGE
+#define SPECENUM_VALUE10NAME N_("?uclassflag:CanPillage")
+/* Cities can still work tile when enemy unit on it */
+#define SPECENUM_VALUE11 UCF_DOESNT_OCCUPY_TILE
+#define SPECENUM_VALUE11NAME N_("?uclassflag:DoesntOccupyTile")
+/* Can attack against units on non-native tiles */
+#define SPECENUM_VALUE12 UCF_ATTACK_NON_NATIVE
+#define SPECENUM_VALUE12NAME N_("?uclassflag:AttackNonNative")
+/* Can launch attack from non-native tile (to native tile) */
+#define SPECENUM_VALUE13 UCF_ATT_FROM_NON_NATIVE
+#define SPECENUM_VALUE13NAME N_("?uclassflag:AttFromNonNative")
+/* Kills citizens upon successful attack against a city */
+#define SPECENUM_VALUE14 UCF_KILLCITIZEN
+#define SPECENUM_VALUE14NAME N_("?uclassflag:KillCitizen")
+/* Can be airlifted */
+#define SPECENUM_VALUE15 UCF_AIRLIFTABLE
+#define SPECENUM_VALUE15NAME N_("?uclassflag:Airliftable")
+/* keep this last */
+#define SPECENUM_COUNT UCF_COUNT
+#define SPECENUM_BITVECTOR bv_unit_class_flags
+#include "specenum_gen.h"
+
+/* Used in savegame processing and clients. */
+#define SPECENUM_NAME unit_move_type
+#define SPECENUM_VALUE0 UMT_LAND
+#define SPECENUM_VALUE0NAME "Land"
+#define SPECENUM_VALUE1 UMT_SEA
+#define SPECENUM_VALUE1NAME "Sea"
+#define SPECENUM_VALUE2 UMT_BOTH
+#define SPECENUM_VALUE2NAME "Both"
+#include "specenum_gen.h"
 
 /* Used in the network protocol. */
 BV_DEFINE(bv_unit_classes, UCL_LAST);
-BV_DEFINE(bv_unit_class_flags, UCF_COUNT);
 
 enum hut_behavior { HUT_NORMAL, HUT_NOTHING, HUT_FRIGHTEN };
 
 enum move_level { MOVE_NONE, MOVE_PARTIAL, MOVE_FULL };
 
-struct base_type_list;
+struct extra_type_list;
+struct unit_class_list;
 
 struct unit_class {
   Unit_Class_id item_number;
@@ -106,16 +129,18 @@ struct unit_class {
   enum hut_behavior hut_behavior;
   bv_unit_class_flags flags;
 
+  struct strvec *helptext;
+
   struct {
     enum move_level land_move;
     enum move_level sea_move;
   } adv;
 
   struct {
-    struct base_type_list *refuel_bases;
-    struct base_type_list *native_tile_bases;
-    struct road_type_list *native_tile_roads;
-    struct road_type_list *bonus_roads;
+    struct extra_type_list *refuel_bases;
+    struct extra_type_list *native_tile_extras;
+    struct extra_type_list *bonus_roads;
+    struct unit_class_list *subset_movers;
   } cache;
 };
 
@@ -129,12 +154,14 @@ struct unit_class {
  * Used in the network protocol.
  */
 #define SPECENUM_NAME unit_type_flag_id
-#define SPECENUM_VALUE0 UTYF_TRADE_ROUTE
+/* Cannot fortify even if class can */
+#define SPECENUM_VALUE0 UTYF_CANT_FORTIFY
 /* TRANS: this and following strings are 'unit type flags', which may rarely
  * be presented to the player in ruleset help text */
-#define SPECENUM_VALUE0NAME N_("?unitflag:TradeRoute")
-#define SPECENUM_VALUE1 UTYF_HELP_WONDER
-#define SPECENUM_VALUE1NAME N_("?unitflag:HelpWonder")
+#define SPECENUM_VALUE0NAME N_("?unitflag:Cant_Fortify")
+/* Unit has no ZOC */
+#define SPECENUM_VALUE1 UTYF_NOZOC
+#define SPECENUM_VALUE1NAME N_("?unitflag:HasNoZOC")
 #define SPECENUM_VALUE2 UTYF_IGZOC
 /* TRANS: unit type flag (rarely shown): "ignore zones of control" */
 #define SPECENUM_VALUE2NAME N_("?unitflag:IgZOC")
@@ -145,122 +172,131 @@ struct unit_class {
 #define SPECENUM_VALUE4NAME N_("?unitflag:IgTer")
 #define SPECENUM_VALUE5 UTYF_ONEATTACK
 #define SPECENUM_VALUE5NAME N_("?unitflag:OneAttack")
-/* Ignores EFT_DEFEND_BONUS (for example city walls) */
-/* TRANS: unit type flag (rarely shown): "ignore walls */
-#define SPECENUM_VALUE6 UTYF_IGWALL
-#define SPECENUM_VALUE6NAME N_("?unitflag:IgWall")
-#define SPECENUM_VALUE7 UTYF_FIELDUNIT
-#define SPECENUM_VALUE7NAME N_("?unitflag:FieldUnit")
-#define SPECENUM_VALUE8 UTYF_MARINES
-#define SPECENUM_VALUE8NAME N_("?unitflag:Marines")
+#define SPECENUM_VALUE6 UTYF_FIELDUNIT
+#define SPECENUM_VALUE6NAME N_("?unitflag:FieldUnit")
+#define SPECENUM_VALUE7 UTYF_MARINES
+#define SPECENUM_VALUE7NAME N_("?unitflag:Marines")
 /* Invisible except when adjacent (Submarine) */
-#define SPECENUM_VALUE9 UTYF_PARTIAL_INVIS
-#define SPECENUM_VALUE9NAME N_("?unitflag:Partial_Invis")
+#define SPECENUM_VALUE8 UTYF_PARTIAL_INVIS
+#define SPECENUM_VALUE8NAME N_("?unitflag:Partial_Invis")
 /* Does not include ability to found cities */
-#define SPECENUM_VALUE10 UTYF_SETTLERS
-#define SPECENUM_VALUE10NAME N_("?unitflag:Settlers")
-#define SPECENUM_VALUE11 UTYF_DIPLOMAT
-#define SPECENUM_VALUE11NAME N_("?unitflag:Diplomat")
+#define SPECENUM_VALUE9 UTYF_SETTLERS
+#define SPECENUM_VALUE9NAME N_("?unitflag:Settlers")
+#define SPECENUM_VALUE10 UTYF_DIPLOMAT
+#define SPECENUM_VALUE10NAME N_("?unitflag:Diplomat")
 /* Trireme must stay close to the coast */
-#define SPECENUM_VALUE12 UTYF_TRIREME
-#define SPECENUM_VALUE12NAME N_("?unitflag:Trireme")
+#define SPECENUM_VALUE11 UTYF_TRIREME
+#define SPECENUM_VALUE11NAME N_("?unitflag:Trireme")
 /* Nuclear attack effect */
-#define SPECENUM_VALUE13 UTYF_NUCLEAR
-#define SPECENUM_VALUE13NAME N_("?unitflag:Nuclear")
+#define SPECENUM_VALUE12 UTYF_NUCLEAR
+#define SPECENUM_VALUE12NAME N_("?unitflag:Nuclear")
 /* Enhanced spy abilities */
-#define SPECENUM_VALUE14 UTYF_SPY
-#define SPECENUM_VALUE14NAME N_("?unitflag:Spy")
-#define SPECENUM_VALUE15 UTYF_PARATROOPERS
-#define SPECENUM_VALUE15NAME N_("?unitflag:Paratroopers")
+#define SPECENUM_VALUE13 UTYF_SPY
+#define SPECENUM_VALUE13NAME N_("?unitflag:Spy")
+#define SPECENUM_VALUE14 UTYF_PARATROOPERS
+#define SPECENUM_VALUE14NAME N_("?unitflag:Paratroopers")
 /* Can build cities */
-#define SPECENUM_VALUE16 UTYF_CITIES
-#define SPECENUM_VALUE16NAME N_("?unitflag:Cities")
+#define SPECENUM_VALUE15 UTYF_CITIES
+#define SPECENUM_VALUE15NAME N_("?unitflag:Cities")
 /* Cannot attack vs non-native tiles even if class can */
-#define SPECENUM_VALUE17 UTYF_ONLY_NATIVE_ATTACK
-#define SPECENUM_VALUE17NAME N_("?unitflag:Only_Native_Attack")
+#define SPECENUM_VALUE16 UTYF_ONLY_NATIVE_ATTACK
+#define SPECENUM_VALUE16NAME N_("?unitflag:Only_Native_Attack")
 /* unit can add to city population */
-#define SPECENUM_VALUE18 UTYF_ADD_TO_CITY
-#define SPECENUM_VALUE18NAME N_("?unitflag:AddToCity")
+#define SPECENUM_VALUE17 UTYF_ADD_TO_CITY
+#define SPECENUM_VALUE17NAME N_("?unitflag:AddToCity")
 /* Only Fundamentalist government can build these units */
-#define SPECENUM_VALUE19 UTYF_FANATIC
-#define SPECENUM_VALUE19NAME N_("?unitflag:Fanatic")
+#define SPECENUM_VALUE18 UTYF_FANATIC
+#define SPECENUM_VALUE18NAME N_("?unitflag:Fanatic")
 /* Losing this unit means losing the game */
-#define SPECENUM_VALUE20 UTYF_GAMELOSS
-#define SPECENUM_VALUE20NAME N_("?unitflag:GameLoss")
+#define SPECENUM_VALUE19 UTYF_GAMELOSS
+#define SPECENUM_VALUE19NAME N_("?unitflag:GameLoss")
 /* A player can only have one unit of this type */
-#define SPECENUM_VALUE21 UTYF_UNIQUE
-#define SPECENUM_VALUE21NAME N_("?unitflag:Unique")
-/* Cannot be bribed */
-#define SPECENUM_VALUE22 UTYF_UNBRIBABLE
-#define SPECENUM_VALUE22NAME N_("?unitflag:Unbribable")
+#define SPECENUM_VALUE20 UTYF_UNIQUE
+#define SPECENUM_VALUE20NAME N_("?unitflag:Unique")
 /* Cannot be disbanded, won't easily go away */
-#define SPECENUM_VALUE23 UTYF_UNDISBANDABLE
-#define SPECENUM_VALUE23NAME N_("?unitflag:Undisbandable")
+#define SPECENUM_VALUE21 UTYF_UNDISBANDABLE
+#define SPECENUM_VALUE21NAME N_("?unitflag:Undisbandable")
 /* Always wins diplomatic contests */
-#define SPECENUM_VALUE24 UTYF_SUPERSPY
-#define SPECENUM_VALUE24NAME N_("?unitflag:SuperSpy")
+#define SPECENUM_VALUE22 UTYF_SUPERSPY
+#define SPECENUM_VALUE22NAME N_("?unitflag:SuperSpy")
 /* Has no homecity */
-#define SPECENUM_VALUE25 UTYF_NOHOME
-#define SPECENUM_VALUE25NAME N_("?unitflag:NoHome")
+#define SPECENUM_VALUE23 UTYF_NOHOME
+#define SPECENUM_VALUE23NAME N_("?unitflag:NoHome")
 /* Cannot increase veteran level */
-#define SPECENUM_VALUE26 UTYF_NO_VETERAN
-#define SPECENUM_VALUE26NAME N_("?unitflag:NoVeteran")
+#define SPECENUM_VALUE24 UTYF_NO_VETERAN
+#define SPECENUM_VALUE24NAME N_("?unitflag:NoVeteran")
 /* Has the ability to bombard */
-#define SPECENUM_VALUE27 UTYF_BOMBARDER
-#define SPECENUM_VALUE27NAME N_("?unitflag:Bombarder")
+#define SPECENUM_VALUE25 UTYF_BOMBARDER
+#define SPECENUM_VALUE25NAME N_("?unitflag:Bombarder")
 /* Gets double firepower against cities */
-#define SPECENUM_VALUE28 UTYF_CITYBUSTER
-#define SPECENUM_VALUE28NAME N_("?unitflag:CityBuster")
+#define SPECENUM_VALUE26 UTYF_CITYBUSTER
+#define SPECENUM_VALUE26NAME N_("?unitflag:CityBuster")
 /* Unit cannot be built (barb leader etc) */
-#define SPECENUM_VALUE29 UTYF_NOBUILD
-#define SPECENUM_VALUE29NAME N_("?unitflag:NoBuild")
+#define SPECENUM_VALUE27 UTYF_NOBUILD
+#define SPECENUM_VALUE27NAME N_("?unitflag:NoBuild")
 /* Firepower set to 1 when EFT_DEFEND_BONUS applies
  * (for example, land unit attacking city with walls) */
-#define SPECENUM_VALUE30 UTYF_BADWALLATTACKER
-#define SPECENUM_VALUE30NAME N_("?unitflag:BadWallAttacker")
+#define SPECENUM_VALUE28 UTYF_BADWALLATTACKER
+#define SPECENUM_VALUE28NAME N_("?unitflag:BadWallAttacker")
 /* Firepower set to 1 and attackers x2 when in city */
-#define SPECENUM_VALUE31 UTYF_BADCITYDEFENDER
-#define SPECENUM_VALUE31NAME N_("?unitflag:BadCityDefender")
+#define SPECENUM_VALUE29 UTYF_BADCITYDEFENDER
+#define SPECENUM_VALUE29NAME N_("?unitflag:BadCityDefender")
 /* Only barbarians can build this unit */
-#define SPECENUM_VALUE32 UTYF_BARBARIAN_ONLY
-#define SPECENUM_VALUE32NAME N_("?unitflag:BarbarianOnly")
+#define SPECENUM_VALUE30 UTYF_BARBARIAN_ONLY
+#define SPECENUM_VALUE30NAME N_("?unitflag:BarbarianOnly")
 /* upkeep can switch from shield to gold */
-#define SPECENUM_VALUE33 UTYF_SHIELD2GOLD
-#define SPECENUM_VALUE33NAME N_("?unitflag:Shield2Gold")
+#define SPECENUM_VALUE31 UTYF_SHIELD2GOLD
+#define SPECENUM_VALUE31NAME N_("?unitflag:Shield2Gold")
 /* Unit can be captured */
-#define SPECENUM_VALUE34 UTYF_CAPTURABLE
-#define SPECENUM_VALUE34NAME N_("?unitflag:Capturable")
+#define SPECENUM_VALUE32 UTYF_CAPTURABLE
+#define SPECENUM_VALUE32NAME N_("?unitflag:Capturable")
 /* Unit can capture other */
-#define SPECENUM_VALUE35 UTYF_CAPTURER
-#define SPECENUM_VALUE35NAME N_("?unitflag:Capturer")
+#define SPECENUM_VALUE33 UTYF_CAPTURER
+#define SPECENUM_VALUE33NAME N_("?unitflag:Capturer")
+/* Unit can't be built in scenarios where founding new cities is prevented. */
+#define SPECENUM_VALUE34 UTYF_NEWCITY_GAMES_ONLY
+#define SPECENUM_VALUE34NAME N_("?unitflag:NewCityGamesOnly")
 
-#define SPECENUM_VALUE36 UTYF_USER_FLAG_1
-#define SPECENUM_VALUE37 UTYF_USER_FLAG_2
-#define SPECENUM_VALUE38 UTYF_USER_FLAG_3
-#define SPECENUM_VALUE39 UTYF_USER_FLAG_4
-#define SPECENUM_VALUE40 UTYF_USER_FLAG_5
-#define SPECENUM_VALUE41 UTYF_USER_FLAG_6
-#define SPECENUM_VALUE42 UTYF_USER_FLAG_7
-#define SPECENUM_VALUE43 UTYF_USER_FLAG_8
-#define SPECENUM_VALUE44 UTYF_USER_FLAG_9
-#define SPECENUM_VALUE45 UTYF_USER_FLAG_10
-#define SPECENUM_VALUE46 UTYF_USER_FLAG_11
-#define SPECENUM_VALUE47 UTYF_USER_FLAG_12
-#define SPECENUM_VALUE48 UTYF_USER_FLAG_13
-#define SPECENUM_VALUE49 UTYF_USER_FLAG_14
-#define SPECENUM_VALUE50 UTYF_USER_FLAG_15
-#define SPECENUM_VALUE51 UTYF_USER_FLAG_16
+#define SPECENUM_VALUE35 UTYF_USER_FLAG_1
+#define SPECENUM_VALUE36 UTYF_USER_FLAG_2
+#define SPECENUM_VALUE37 UTYF_USER_FLAG_3
+#define SPECENUM_VALUE38 UTYF_USER_FLAG_4
+#define SPECENUM_VALUE39 UTYF_USER_FLAG_5
+#define SPECENUM_VALUE40 UTYF_USER_FLAG_6
+#define SPECENUM_VALUE41 UTYF_USER_FLAG_7
+#define SPECENUM_VALUE42 UTYF_USER_FLAG_8
+#define SPECENUM_VALUE43 UTYF_USER_FLAG_9
+#define SPECENUM_VALUE44 UTYF_USER_FLAG_10
+#define SPECENUM_VALUE45 UTYF_USER_FLAG_11
+#define SPECENUM_VALUE46 UTYF_USER_FLAG_12
+#define SPECENUM_VALUE47 UTYF_USER_FLAG_13
+#define SPECENUM_VALUE48 UTYF_USER_FLAG_14
+#define SPECENUM_VALUE49 UTYF_USER_FLAG_15
+#define SPECENUM_VALUE50 UTYF_USER_FLAG_16
+#define SPECENUM_VALUE51 UTYF_USER_FLAG_17
+#define SPECENUM_VALUE52 UTYF_USER_FLAG_18
+#define SPECENUM_VALUE53 UTYF_USER_FLAG_19
+#define SPECENUM_VALUE54 UTYF_USER_FLAG_20
+#define SPECENUM_VALUE55 UTYF_USER_FLAG_21
+#define SPECENUM_VALUE56 UTYF_USER_FLAG_22
+#define SPECENUM_VALUE57 UTYF_USER_FLAG_23
+#define SPECENUM_VALUE58 UTYF_USER_FLAG_24
+#define SPECENUM_VALUE59 UTYF_USER_FLAG_25
+#define SPECENUM_VALUE60 UTYF_USER_FLAG_26
+#define SPECENUM_VALUE61 UTYF_USER_FLAG_27
+#define SPECENUM_VALUE62 UTYF_USER_FLAG_28
+#define SPECENUM_VALUE63 UTYF_USER_FLAG_29
+#define SPECENUM_VALUE64 UTYF_USER_FLAG_30
+#define SPECENUM_VALUE65 UTYF_USER_FLAG_31
+#define SPECENUM_VALUE66 UTYF_USER_FLAG_32
 /* Note that first role must have value next to last flag */
 
-#define UTYF_LAST_USER_FLAG UTYF_USER_FLAG_16
+#define UTYF_LAST_USER_FLAG UTYF_USER_FLAG_32
 #define MAX_NUM_USER_UNIT_FLAGS (UTYF_LAST_USER_FLAG - UTYF_USER_FLAG_1 + 1)
 #define SPECENUM_NAMEOVERRIDE
+#define SPECENUM_BITVECTOR bv_unit_type_flags
 #include "specenum_gen.h"
 
-#define UTYF_MAX 64 /* Used in the network protocol. */
-
-FC_STATIC_ASSERT(UTYF_LAST_USER_FLAG < UTYF_MAX, too_many_unit_flags);
-
 
 /* Unit "roles": these are similar to unit flags but differ in that
    they don't represent intrinsic properties or abilities of units,
@@ -276,79 +312,91 @@ FC_STATIC_ASSERT(UTYF_LAST_USER_FLAG < UTYF_MAX, too_many_unit_flags);
 #define L_FIRST (UTYF_LAST_USER_FLAG + 1)
 
 #define SPECENUM_NAME unit_role_id
-/* Hack: on S2_5 only, L_FIRST role value in specenum (VALUE52) is
- * not used due to a bug discovered after the network protocol was
- * frozen (gna bug #22272) */
 /* is built first when city established */
-#define SPECENUM_VALUE53 L_FIRSTBUILD
-#define SPECENUM_VALUE53NAME N_("?unitflag:FirstBuild")
+#define SPECENUM_VALUE67 L_FIRSTBUILD
+#define SPECENUM_VALUE67NAME N_("?unitflag:FirstBuild")
 /* initial explorer unit */
-#define SPECENUM_VALUE54 L_EXPLORER
-#define SPECENUM_VALUE54NAME N_("?unitflag:Explorer")
+#define SPECENUM_VALUE68 L_EXPLORER
+#define SPECENUM_VALUE68NAME N_("?unitflag:Explorer")
 /* can be found in hut */
-#define SPECENUM_VALUE55 L_HUT
-#define SPECENUM_VALUE55NAME N_("?unitflag:Hut")
+#define SPECENUM_VALUE69 L_HUT
+#define SPECENUM_VALUE69NAME N_("?unitflag:Hut")
 /* can be found in hut, global tech required */
-#define SPECENUM_VALUE56 L_HUT_TECH
-#define SPECENUM_VALUE56NAME N_("?unitflag:HutTech")
+#define SPECENUM_VALUE70 L_HUT_TECH
+#define SPECENUM_VALUE70NAME N_("?unitflag:HutTech")
 /* is created in Partisan circumstances */
-#define SPECENUM_VALUE57 L_PARTISAN
-#define SPECENUM_VALUE57NAME N_("?unitflag:Partisan")
+#define SPECENUM_VALUE71 L_PARTISAN
+#define SPECENUM_VALUE71NAME N_("?unitflag:Partisan")
 /* ok on defense (AI) */
-#define SPECENUM_VALUE58 L_DEFEND_OK
-#define SPECENUM_VALUE58NAME N_("?unitflag:DefendOk")
+#define SPECENUM_VALUE72 L_DEFEND_OK
+#define SPECENUM_VALUE72NAME N_("?unitflag:DefendOk")
 /* primary purpose is defense (AI) */
-#define SPECENUM_VALUE59 L_DEFEND_GOOD
-#define SPECENUM_VALUE59NAME N_("?unitflag:DefendGood")
-/* quick attacking unit (Horse..Armor) (unused)*/
-#define SPECENUM_VALUE60 L_ATTACK_FAST
-#define SPECENUM_VALUE60NAME N_("?unitflag:AttackFast")
-/* powerful attacking unit (Catapult..) (unused) */
-#define SPECENUM_VALUE61 L_ATTACK_STRONG
-#define SPECENUM_VALUE61NAME N_("?unitflag:AttackStrong")
+#define SPECENUM_VALUE73 L_DEFEND_GOOD
+#define SPECENUM_VALUE73NAME N_("?unitflag:DefendGood")
 /* is useful for ferrying (AI) */
-#define SPECENUM_VALUE62 L_FERRYBOAT
-#define SPECENUM_VALUE62NAME N_("?unitflag:FerryBoat")
+#define SPECENUM_VALUE74 L_FERRYBOAT
+#define SPECENUM_VALUE74NAME N_("?unitflag:FerryBoat")
 /* barbarians unit, land only */
-#define SPECENUM_VALUE63 L_BARBARIAN
-#define SPECENUM_VALUE63NAME N_("?unitflag:Barbarian")
+#define SPECENUM_VALUE75 L_BARBARIAN
+#define SPECENUM_VALUE75NAME N_("?unitflag:Barbarian")
 /* barbarians unit, global tech required */
-#define SPECENUM_VALUE64 L_BARBARIAN_TECH
-#define SPECENUM_VALUE64NAME N_("?unitflag:BarbarianTech")
+#define SPECENUM_VALUE76 L_BARBARIAN_TECH
+#define SPECENUM_VALUE76NAME N_("?unitflag:BarbarianTech")
 /* barbarian boat */
-#define SPECENUM_VALUE65 L_BARBARIAN_BOAT
-#define SPECENUM_VALUE65NAME N_("?unitflag:BarbarianBoat")
+#define SPECENUM_VALUE77 L_BARBARIAN_BOAT
+#define SPECENUM_VALUE77NAME N_("?unitflag:BarbarianBoat")
 /* what barbarians should build */
-#define SPECENUM_VALUE66 L_BARBARIAN_BUILD
-#define SPECENUM_VALUE66NAME N_("?unitflag:BarbarianBuild")
+#define SPECENUM_VALUE78 L_BARBARIAN_BUILD
+#define SPECENUM_VALUE78NAME N_("?unitflag:BarbarianBuild")
 /* barbarians build when global tech */
-#define SPECENUM_VALUE67 L_BARBARIAN_BUILD_TECH
-#define SPECENUM_VALUE67NAME N_("?unitflag:BarbarianBuildTech")
+#define SPECENUM_VALUE79 L_BARBARIAN_BUILD_TECH
+#define SPECENUM_VALUE79NAME N_("?unitflag:BarbarianBuildTech")
 /* barbarian leader */
-#define SPECENUM_VALUE68 L_BARBARIAN_LEADER
-#define SPECENUM_VALUE68NAME N_("?unitflag:BarbarianLeader")
+#define SPECENUM_VALUE80 L_BARBARIAN_LEADER
+#define SPECENUM_VALUE80NAME N_("?unitflag:BarbarianLeader")
 /* sea raider unit */
-#define SPECENUM_VALUE69 L_BARBARIAN_SEA
-#define SPECENUM_VALUE69NAME N_("?unitflag:BarbarianSea")
+#define SPECENUM_VALUE81 L_BARBARIAN_SEA
+#define SPECENUM_VALUE81NAME N_("?unitflag:BarbarianSea")
 /* sea raider unit, global tech required */
-#define SPECENUM_VALUE70 L_BARBARIAN_SEA_TECH
-#define SPECENUM_VALUE70NAME N_("?unitflag:BarbarianSeaTech")
-/* can found cities */
-#define SPECENUM_VALUE71 L_CITIES
-#define SPECENUM_VALUE71NAME N_("?unitflag:Cities")
-/* can improve terrain */
-#define SPECENUM_VALUE72 L_SETTLERS
-#define SPECENUM_VALUE72NAME N_("?unitflag:Settlers")
-/* loss results in loss of game */
-#define SPECENUM_VALUE73 L_GAMELOSS
-#define SPECENUM_VALUE73NAME N_("?unitflag:GameLoss")
-/* can do diplomat actions */
-#define SPECENUM_VALUE74 L_DIPLOMAT
-#define SPECENUM_VALUE74NAME N_("?unitflag:Diplomat")
+#define SPECENUM_VALUE82 L_BARBARIAN_SEA_TECH
+#define SPECENUM_VALUE82NAME N_("?unitflag:BarbarianSeaTech")
+/* Startunit: Cities */
+#define SPECENUM_VALUE83 L_START_CITIES
+#define SPECENUM_VALUE83NAME N_("?unitflag:CitiesStartunit")
+/* Startunit: Worker */
+#define SPECENUM_VALUE84 L_START_WORKER
+#define SPECENUM_VALUE84NAME N_("?unitflag:WorkerStartunit")
+/* Startunit: Explorer */
+#define SPECENUM_VALUE85 L_START_EXPLORER
+#define SPECENUM_VALUE85NAME N_("?unitflag:ExplorerStartunit")
+/* Startunit: King */
+#define SPECENUM_VALUE86 L_START_KING
+#define SPECENUM_VALUE86NAME N_("?unitflag:KingStartunit")
+/* Startunit: Diplomat */
+#define SPECENUM_VALUE87 L_START_DIPLOMAT
+#define SPECENUM_VALUE87NAME N_("?unitflag:DiplomatStartunit")
+/* Startunit: Ferryboat */
+#define SPECENUM_VALUE88 L_START_FERRY
+#define SPECENUM_VALUE88NAME N_("?unitflag:FerryStartunit")
+/* Startunit: DefendOk */
+#define SPECENUM_VALUE89 L_START_DEFEND_OK
+#define SPECENUM_VALUE89NAME N_("?unitflag:DefendOkStartunit")
+/* Startunit: DefendGood */
+#define SPECENUM_VALUE90 L_START_DEFEND_GOOD
+#define SPECENUM_VALUE90NAME N_("?unitflag:DefendGoodStartunit")
+/* Startunit: AttackFast */
+#define SPECENUM_VALUE91 L_START_ATTACK_FAST
+#define SPECENUM_VALUE91NAME N_("?unitflag:AttackFastStartunit")
+/* Startunit: AttackStrong */
+#define SPECENUM_VALUE92 L_START_ATTACK_STRONG
+#define SPECENUM_VALUE92NAME N_("?unitflag:AttackStrongStartunit")
 /* AI hunter type unit */
-#define SPECENUM_VALUE75 L_HUNTER
-#define SPECENUM_VALUE75NAME N_("?unitflag:Hunter")
-#define L_LAST (L_HUNTER+1)
+#define SPECENUM_VALUE93 L_HUNTER
+#define SPECENUM_VALUE93NAME N_("?unitflag:Hunter")
+/* can improve terrain */
+#define SPECENUM_VALUE94 L_SETTLERS
+#define SPECENUM_VALUE94NAME N_("?unitflag:Settlers")
+#define L_LAST (L_SETTLERS + 1)
 
 #include "specenum_gen.h"
 
@@ -357,7 +405,6 @@ FC_STATIC_ASSERT(UTYF_LAST_USER_FLAG < UTYF_MAX, too_many_unit_flags);
 FC_STATIC_ASSERT(L_LAST - L_FIRST <= L_MAX, too_many_unit_roles);
 
 /* Used in the network protocol. */
-BV_DEFINE(bv_unit_type_flags, UTYF_MAX);
 BV_DEFINE(bv_unit_type_roles, L_MAX);
 
 /* Used in the network protocol. */
@@ -374,6 +421,9 @@ struct combat_bonus {
   enum unit_type_flag_id  flag;
   enum combat_bonus_type  type;
   int                     value;
+
+  /* Not listed in the help text. */
+  bool                    quiet;
 };
 
 /* get 'struct combat_bonus_list' and related functions: */
@@ -404,6 +454,7 @@ struct veteran_system {
 struct unit_type {
   Unit_type_id item_number;
   struct name_translation name;
+  bool disabled;                        /* Does not really exist - hole in improvments array */
   char graphic_str[MAX_LEN_NAME];
   char graphic_alt[MAX_LEN_NAME];
   char sound_move[MAX_LEN_NAME];
@@ -415,6 +466,7 @@ struct unit_type {
   int attack_strength;
   int defense_strength;
   int move_rate;
+  int unknown_move_cost; /* See utype_unknown_move_cost(). */
 
   struct advance *require_advance;	/* may be NULL */
   struct impr_type *need_improvement;	/* may be NULL */
@@ -467,6 +519,10 @@ struct unit_type {
 
   struct strvec *helptext;
 
+  struct {
+    bool igwall;
+  } adv;
+
   struct {
     int max_defense_mp; /* Value 0 here does not guarantee that unit never
                          * has CBONUS_DEFENSE_MULTIPLIER, it merely means
@@ -474,7 +530,7 @@ struct unit_type {
     int defense_mp_bonuses[U_LAST];
   } cache;
 
-  void *ais[FC_AI_LAST];
+  void *ais[FREECIV_AI_MOD_LAST];
 };
 
 /* General unit and unit type (matched) routines */
@@ -482,7 +538,7 @@ Unit_type_id utype_count(void);
 Unit_type_id utype_index(const struct unit_type *punittype);
 Unit_type_id utype_number(const struct unit_type *punittype);
 
-struct unit_type *unit_type(const struct unit *punit);
+struct unit_type *unit_type_get(const struct unit *punit);
 struct unit_type *utype_by_number(const Unit_type_id id);
 
 struct unit_type *unit_type_by_rule_name(const char *name);
@@ -524,6 +580,28 @@ bool utype_can_freely_load(const struct unit_type *pcargotype,
 bool utype_can_freely_unload(const struct unit_type *pcargotype,
                              const struct unit_type *ptranstype);
 
+bool utype_may_act_at_all(const struct unit_type *putype);
+bool utype_can_do_action(const struct unit_type *putype,
+                         const int act_id);
+bool utype_acts_hostile(const struct unit_type *putype);
+
+bool can_unit_act_when_ustate_is(const struct unit_type *punit_type,
+                                 const enum ustate_prop prop,
+                                 const bool is_there);
+bool utype_can_do_act_when_ustate(const struct unit_type *punit_type,
+                                  const int act_id,
+                                  const enum ustate_prop prop,
+                                  const bool is_there);
+
+bool can_utype_do_act_if_tgt_diplrel(const struct unit_type *punit_type,
+                                     const int act_id,
+                                     const int prop,
+                                     const bool is_there);
+
+bool utype_may_act_move_frags(struct unit_type *punit_type,
+                              const int act_id,
+                              const int move_fragments);
+
 /* Functions to operate on various flag and roles. */
 typedef bool (*role_unit_callback)(struct unit_type *ptype, void *data);
 
@@ -532,7 +610,7 @@ void role_unit_precalcs_free(void);
 int num_role_units(int role);
 struct unit_type *role_units_iterate(int role, role_unit_callback cb, void *data);
 struct unit_type *role_units_iterate_backwards(int role, role_unit_callback cb, void *data);
-struct unit_type *get_role_unit(int role, int index);
+struct unit_type *get_role_unit(int role, int role_index);
 struct unit_type *best_role_unit(const struct city *pcity, int role);
 struct unit_type *best_role_unit_for_player(const struct player *pplayer,
 					    int role);
@@ -550,7 +628,7 @@ Unit_Class_id uclass_number(const struct unit_class *pclass);
 Unit_Class_id uclass_index(const struct unit_class *pclass);
 #endif /* uclass_index */
 
-struct unit_class *unit_class(const struct unit *punit);
+struct unit_class *unit_class_get(const struct unit *punit);
 struct unit_class *uclass_by_number(const Unit_Class_id id);
 #define utype_class(_t_) (_t_)->uclass
 #ifndef utype_class
@@ -601,7 +679,7 @@ int unit_pop_value(const struct unit *punit);
 int utype_pop_value(const struct unit_type *punittype);
 
 enum unit_move_type utype_move_type(const struct unit_type *punittype);
-enum unit_move_type uclass_move_type(const struct unit_class *pclass);
+void set_unit_move_type(struct unit_class *puclass);
 
 /* player related unit functions */
 int utype_upkeep_cost(const struct unit_type *ut, struct player *pplayer,
@@ -626,6 +704,8 @@ bool can_player_build_unit_now(const struct player *p,
 
 #define utype_fuel(ptype) (ptype)->fuel
 
+bool utype_is_cityfounder(struct unit_type *utype);
+
 /* Initialization and iteration */
 void unit_types_init(void);
 void unit_types_free(void);
@@ -645,10 +725,22 @@ const struct unit_type *unit_type_array_last(void);
   }									\
 }
 
+#define unit_active_type_iterate(_p)                                    \
+  unit_type_iterate(_p) {                                               \
+    if (!_p->disabled) {
+
+#define unit_active_type_iterate_end                                    \
+    }                                                                   \
+  } unit_type_iterate_end;
+
+
 void *utype_ai_data(const struct unit_type *ptype, const struct ai_type *ai);
 void utype_set_ai_data(struct unit_type *ptype, const struct ai_type *ai,
                        void *data);
 
+void unit_type_action_cache_set(struct unit_type *ptype);
+void unit_type_action_cache_init(void);
+
 /* Initialization and iteration */
 void unit_classes_init(void);
 void unit_classes_free(void);
@@ -670,6 +762,22 @@ const struct unit_class *unit_class_array_last(void);
   }									\
 }
 
+#define SPECLIST_TAG unit_class
+#define SPECLIST_TYPE struct unit_class
+#include "speclist.h"
+
+#define unit_class_list_iterate(uclass_list, pclass) \
+  TYPED_LIST_ITERATE(struct unit_class, uclass_list, pclass)
+#define unit_class_list_iterate_end LIST_ITERATE_END
+
+#define SPECLIST_TAG unit_type
+#define SPECLIST_TYPE struct unit_type
+#include "speclist.h"
+
+#define unit_type_list_iterate(utype_list, ptype) \
+  TYPED_LIST_ITERATE(struct unit_type, utype_list, ptype)
+#define unit_type_list_iterate_end LIST_ITERATE_END
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
