@@ -21,6 +21,9 @@
 #include "fc_types.h"
 #include "improvement.h"
 
+/* server/advisors */
+#include "advtools.h"
+
 /* 
  * This file and advdata.c contains global data structures for the AI
  * and some of the functions that fill them with useful values at the 
@@ -74,17 +77,13 @@ struct adv_data {
     /* Counts of specific types of units. */
     struct {
       /* Unit-flag counts. */
-      int triremes, missiles;
+      int triremes, missiles, paratroopers, airliftable;
 
-      /* Move-type counts */
-      int land, sea, amphibious;
+      int byclass[UCL_LAST];
 
       /* Upgradeable units */
       int upgradeable;
-
-      int paratroopers;
     } units;
-    int *workers;     /* cities to workers on continent*/
     int *cities;      /* number of cities we have on continent */
     int average_production;
   } stats;
@@ -111,15 +110,15 @@ struct adv_data {
   int pollution_priority;
 
   /* Government data */
-  int *government_want;
+  adv_want *government_want;
   short govt_reeval;
 
   /* Goals */
   struct {
     struct {
       struct government *gov;        /* The ideal government */
-      int val;        /* Its value (relative to the current gov) */
-      int req;        /* The tech requirement for the ideal gov */
+      adv_want val;                  /* Its value (relative to the current gov) */
+      int req;                       /* The tech requirement for the ideal gov */
     } govt;
     struct government *revolution;   /* The best gov of the now available */
   } goal;
@@ -134,22 +133,6 @@ struct adv_data {
   int max_num_cities;
 };
 
-enum choice_type {
-  CT_NONE = 0,
-  CT_BUILDING = 1,
-  CT_CIVILIAN,
-  CT_ATTACKER,
-  CT_DEFENDER,
-  CT_LAST
-};
-
-struct adv_choice {
-  enum choice_type type;
-  universals_u value; /* what the advisor wants */
-  int want;              /* how much it wants it (0-100) */
-  bool need_boat;        /* unit being built wants a boat */
-};
-
 void adv_data_init(struct player *pplayer);
 void adv_data_default(struct player *pplayer);
 void adv_data_close(struct player *pplayer);
