@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -49,7 +49,7 @@ bool adv_follow_path(struct unit *punit, struct pf_path *path,
 {
   struct tile *old_tile = punit->goto_tile;
   enum unit_activity activity = punit->activity;
-  struct act_tgt tgt = punit->activity_target;
+  struct extra_type *tgt = punit->activity_target;
   bool alive;
 
   if (punit->moves_left <= 0) {
@@ -59,9 +59,12 @@ bool adv_follow_path(struct unit *punit, struct pf_path *path,
   unit_activity_handling(punit, ACTIVITY_GOTO);
   alive = adv_unit_execute_path(punit, path);
   if (alive) {
-    unit_activity_handling(punit, ACTIVITY_IDLE);
-    send_unit_info(NULL, punit); /* FIXME: probably duplicate */
-    unit_activity_handling_targeted(punit, activity, &tgt);
+    if (activity != ACTIVITY_GOTO) {
+      /* Only go via ACTIVITY_IDLE if we are actually changing the activity */
+      unit_activity_handling(punit, ACTIVITY_IDLE);
+      send_unit_info(NULL, punit); /* FIXME: probably duplicate */
+      unit_activity_handling_targeted(punit, activity, &tgt);
+    }
     punit->goto_tile = old_tile; /* May be NULL. */
     send_unit_info(NULL, punit);
   }
@@ -151,7 +154,8 @@ static bool adv_unit_move(struct unit *punit, struct tile *ptile)
 
   /* go */
   unit_activity_handling(punit, ACTIVITY_IDLE);
-  (void) unit_move_handling(punit, ptile, FALSE, TRUE);
+  /* Move */
+  (void) unit_move_handling(punit, ptile, FALSE, TRUE, NULL);
 
   return TRUE;
 }
@@ -160,8 +164,6 @@ static bool adv_unit_move(struct unit *punit, struct tile *ptile)
   Similar to is_my_zoc(), but with some changes:
   - destination (x0,y0) need not be adjacent?
   - don't care about some directions?
-  
-  Note this function only makes sense for ground units.
 
   Fix to bizarre did-not-find bug.  Thanks, Katvrr -- Syela
 **************************************************************************/
@@ -176,7 +178,7 @@ static bool adv_could_be_my_zoc(struct unit *myunit, struct tile *ptile)
   }
 
   adjc_iterate(ptile, atile) {
-    if (!is_ocean_tile(atile)
+    if (!terrain_has_flag(tile_terrain(atile), TER_NO_ZOC)
 	&& is_non_allied_unit_tile(atile, unit_owner(myunit))) {
       return FALSE;
     }
@@ -197,7 +199,9 @@ int adv_could_unit_move_to_tile(struct unit *punit, struct tile *dest_tile)
 {
   enum unit_move_result reason =
       unit_move_to_tile_test(punit, ACTIVITY_IDLE, unit_tile(punit),
-                             dest_tile, unit_has_type_flag(punit, UTYF_IGZOC));
+                             dest_tile, unit_has_type_flag(punit, UTYF_IGZOC),
+                             NULL);
+
   switch (reason) {
   case MR_OK:
     return 1;
@@ -230,7 +234,7 @@ int adv_unittype_att_rating(const struct unit_type *punittype, int veteran,
 ****************************************************************************/
 int adv_unit_att_rating(const struct unit *punit)
 {
-  return adv_unittype_att_rating(unit_type(punit), punit->veteran,
+  return adv_unittype_att_rating(unit_type_get(punit), punit->veteran,
                                  SINGLE_MOVE, punit->hp);
 }
 
@@ -241,7 +245,7 @@ int adv_unit_att_rating(const struct unit *punit)
 int adv_unit_def_rating_basic(const struct unit *punit)
 {
   return base_get_defense_power(punit) * punit->hp *
-    unit_type(punit)->firepower / POWER_DIVIDER;
+    unit_type_get(punit)->firepower / POWER_DIVIDER;
 }
 
 /****************************************************************************
@@ -261,14 +265,14 @@ bool adv_danger_at(struct unit *punit, struct tile *ptile)
   int a = 0, d, db;
   struct player *pplayer = unit_owner(punit);
   struct city *pcity = tile_city(ptile);
-  enum danger_consideration dc = DANG_UNDECIDED;
+  enum override_bool dc = NO_OVERRIDE;
   int extras_bonus = 0;
 
   /* Give AI code possibility to decide itself */
   CALL_PLR_AI_FUNC(consider_tile_dangerous, unit_owner(punit), ptile, punit, &dc);
-  if (dc == DANG_YES) {
+  if (dc == OVERRIDE_TRUE) {
     return TRUE;
-  } else if (dc == DANG_NOT) {
+  } else if (dc == OVERRIDE_FALSE) {
     return FALSE;
   }
 
@@ -280,7 +284,7 @@ bool adv_danger_at(struct unit *punit, struct tile *ptile)
 
   /* Calculate how well we can defend at (x,y) */
   db = 10 + tile_terrain(ptile)->defense_bonus / 10;
-  extras_bonus += tile_extras_defense_bonus(ptile, unit_type(punit));
+  extras_bonus += tile_extras_defense_bonus(ptile, unit_type_get(punit));
   db += (db * extras_bonus) / 100;
   d = adv_unit_def_rating_basic_sq(punit) * db;
 
@@ -353,7 +357,8 @@ static double chance_killed_at(const struct tile *ptile,
 
   /* If we are on defensive terrain, we are more likely to survive */
   db = 10 + tile_terrain(ptile)->defense_bonus / 10;
-  extras_bonus += tile_extras_class_defense_bonus(ptile, param->uclass);
+  extras_bonus += tile_extras_class_defense_bonus(ptile,
+                                                  utype_class(param->utype));
   db += (extras_bonus) / 100;
   p *= 10.0 / db;
 
