@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 #include "timing.h"
 
 /* common */
+#include "ai.h"
 #include "city.h"
 #include "game.h"
 #include "government.h"
@@ -48,11 +49,15 @@
 #include "unittools.h"
 
 /* server/advisors */
+#include "advbuilding.h"
 #include "advdata.h"
 #include "advgoto.h"
 #include "advtools.h"
 #include "infracache.h"
 
+/* ai */
+#include "handicaps.h"
+
 #include "autosettlers.h"
 
 /* This factor is multiplied on when calculating the want.  This is done
@@ -68,6 +73,10 @@ struct settlermap {
   int eta; /* estimated number of turns until enroute arrives */
 };
 
+Activity_type_id as_activities_transform[ACTIVITY_LAST];
+Activity_type_id as_activities_extra[ACTIVITY_LAST];
+Activity_type_id as_activities_rmextra[ACTIVITY_LAST];
+
 static struct timer *as_timer = NULL;
 
 /**************************************************************************
@@ -79,6 +88,32 @@ void adv_settlers_free(void)
   as_timer = NULL;
 }
 
+/**************************************************************************
+  Initialize advisor systems.
+**************************************************************************/
+void advisors_init(void)
+{
+  int i = 0;
+
+  as_activities_transform[i++] = ACTIVITY_IRRIGATE;
+  as_activities_transform[i++] = ACTIVITY_MINE;
+  as_activities_transform[i++] = ACTIVITY_TRANSFORM;
+  as_activities_transform[i++] = ACTIVITY_LAST;
+
+  i = 0;
+  as_activities_extra[i++] = ACTIVITY_IRRIGATE;
+  as_activities_extra[i++] = ACTIVITY_MINE;
+  as_activities_extra[i++] = ACTIVITY_GEN_ROAD;
+  as_activities_extra[i++] = ACTIVITY_BASE;
+  as_activities_extra[i++] = ACTIVITY_LAST;
+
+  i = 0;
+  as_activities_rmextra[i++] = ACTIVITY_POLLUTION;
+  as_activities_rmextra[i++] = ACTIVITY_FALLOUT;
+  /* We could have ACTIVITY_PILLAGE here, but currently we don't */
+  as_activities_rmextra[i++] = ACTIVITY_LAST;
+}
+
 /**************************************************************************
   Calculate the attractiveness of building a road/rail at the given tile.
 
@@ -98,14 +133,16 @@ int adv_settlers_road_bonus(struct tile *ptile, struct road_type *proad)
   struct road_type *pdep_roads[MAX_DEP_ROADS];
   int dep_rnbr[MAX_DEP_ROADS];
   int dep_count = 0;
+  struct extra_type *pextra;
 
   if (proad == NULL) {
     return 0;
   }
 
   rnbr = road_number(proad);
+  pextra = road_extra_get(proad);
 
-  road_deps_iterate(&(proad->reqs), pdep) {
+  road_deps_iterate(&(pextra->reqs), pdep) {
     if (dep_count < MAX_DEP_ROADS) {
       pdep_roads[dep_count] = pdep;
       dep_rnbr[dep_count++] = road_number(pdep);
@@ -121,7 +158,7 @@ int adv_settlers_road_bonus(struct tile *ptile, struct road_type *proad)
       potential_road[i] = FALSE;
       is_slow[i] = FALSE; /* FIXME: should be TRUE? */
     } else {
-      int build_time = terrain_road_time(tile_terrain(tile1), rnbr);
+      int build_time = terrain_extra_build_time(tile_terrain(tile1), ACTIVITY_GEN_ROAD, pextra);
       int j;
 
       real_road[i] = tile_has_road(tile1, proad);
@@ -140,7 +177,11 @@ int adv_settlers_road_bonus(struct tile *ptile, struct road_type *proad)
           if (punit->activity == ACTIVITY_GEN_ROAD) {
             /* If a road, or its dependency is being built here, consider as if it's already
 	     * built. */
-            int build_rnbr = punit->activity_target.obj.road;
+            int build_rnbr;
+
+            fc_assert(punit->activity_target != NULL);
+
+            build_rnbr = road_index(extra_road_get(punit->activity_target));
 
             if (build_rnbr == rnbr) {
               real_road[i] = TRUE;
@@ -159,17 +200,17 @@ int adv_settlers_road_bonus(struct tile *ptile, struct road_type *proad)
 
   if (current_topo_has_flag(TF_HEX)) {
     /* On hex map, road is always a benefit */
-    bonus += 2; /* Later divided by 2 */
+    bonus += 20; /* Later divided by 20 */
 
     /* Road is more valuable when even longer road around does not exist. */
     for (i = 0; i < 12; i++) {
       if (!real_road[i]) {
-        bonus++;
+        bonus += 3;
       }
     }
 
     /* Scale down the bonus. */
-    bonus /= 2;
+    bonus /= 20;
   } else {
     /*
      * Consider the following tile arrangement (numbered in hex):
@@ -256,29 +297,58 @@ int adv_settlers_road_bonus(struct tile *ptile, struct road_type *proad)
 ****************************************************************************/
 static void consider_settler_action(const struct player *pplayer, 
                                     enum unit_activity act,
-                                    struct act_tgt *target,
+                                    struct extra_type *target,
                                     int extra,
                                     int new_tile_value, int old_tile_value,
                                     bool in_use, int delay,
                                     int *best_value,
                                     int *best_old_tile_value,
+                                    int *best_extra,
+                                    bool *improve_worked,
+                                    int *best_delay,
                                     enum unit_activity *best_act,
-                                    struct act_tgt *best_target,
+                                    struct extra_type **best_target,
                                     struct tile **best_tile,
                                     struct tile *ptile)
 {
-  bool consider;
+  bool improves;
   int total_value = 0, base_value = 0;
-  
-  if (extra >= 0) {
-    consider = TRUE;
-  } else {
-    consider = (new_tile_value > old_tile_value);
+  int old_improvement_value;
+
+  if (extra < 0) {
     extra = 0;
   }
 
+  if (new_tile_value > old_tile_value) {
+    improves = TRUE;
+  } else if (new_tile_value == old_tile_value && extra > 0) {
+    improves = TRUE;
+  } else {
+    improves = FALSE;
+  }
+
   /* find the present value of the future benefit of this action */
-  if (consider) {
+  if (improves || extra > 0) {
+    if (!(*improve_worked) && !in_use) {
+      /* Going to improve tile that is not yet in use.
+       * Getting the best possible total for next citizen to work on is more
+       * important than amount tile gets improved. */
+      if (improves && (new_tile_value > *best_value
+                       || (new_tile_value == *best_value && old_tile_value < *best_old_tile_value))) {
+        *best_value = new_tile_value;
+        *best_old_tile_value = old_tile_value;
+        *best_extra = extra;
+        *best_act = act;
+        *best_target = target;
+        *best_tile = ptile;
+        *best_delay = delay;
+      }
+
+      return;
+    }
+
+    /* At least one of the previous best or current tile is in use
+     * Prefer the tile that gets improved more, regarless of the resulting total */
 
     base_value = new_tile_value - old_tile_value;
     total_value = base_value * WORKER_FACTOR;
@@ -289,34 +359,50 @@ static void consider_settler_action(const struct player *pplayer,
 
     /* use factor to prevent rounding errors */
     total_value = amortize(total_value, delay);
-  } else {
-    total_value = 0;
-  }
 
-  if (total_value > *best_value
-      || (total_value == *best_value
-	  && old_tile_value > *best_old_tile_value)) {
-    log_debug("Replacing (%d, %d) = %d with %s (%d, %d) = %d [d=%d b=%d]",
-              TILE_XY(*best_tile), *best_value, get_activity_text(act),
-              TILE_XY(ptile), total_value, delay, base_value);
-    *best_value = total_value;
-    *best_old_tile_value = old_tile_value;
-    *best_act = act;
-    *best_target = *target;
-    *best_tile = ptile;
+    if (*improve_worked) {
+      old_improvement_value = *best_value;
+    } else {
+      /* Convert old best_value to improvement value compatible with in_use
+       * tile value */
+      old_improvement_value = amortize((*best_value - *best_old_tile_value) * WORKER_FACTOR / 2,
+                                       *best_delay);
+    }
+
+    if (total_value > old_improvement_value
+        || (total_value == old_improvement_value
+            && old_tile_value > *best_old_tile_value)) {
+      if (in_use) {
+        *best_value = total_value;
+        *improve_worked = TRUE;
+      } else {
+        *best_value = new_tile_value;
+        *improve_worked = FALSE;
+      }
+      *best_old_tile_value = old_tile_value;
+      *best_extra = extra;
+      *best_act = act;
+      *best_target = target;
+      *best_tile = ptile;
+      *best_delay = delay;
+    }
   }
 }
 
-/**************************************************************************
+/****************************************************************************
   Don't enter in enemy territories.
-**************************************************************************/
-static bool autosettler_enter_territory(const struct player *pplayer,
-                                        const struct tile *ptile)
+****************************************************************************/
+static enum tile_behavior
+autosettler_tile_behavior(const struct tile *ptile,
+                          enum known_type known,
+                          const struct pf_parameter *param)
 {
   const struct player *owner = tile_owner(ptile);
 
-  return (NULL == owner
-          || pplayers_allied(owner, pplayer));
+  if (NULL != owner && !pplayers_allied(owner, param->owner)) {
+    return TB_IGNORE;
+  }
+  return TB_NORMAL;
 }
 
 /****************************************************************************
@@ -338,7 +424,7 @@ static bool autosettler_enter_territory(const struct player *pplayer,
 ****************************************************************************/
 int settler_evaluate_improvements(struct unit *punit,
                                   enum unit_activity *best_act,
-                                  struct act_tgt *best_target,
+                                  struct extra_type **best_target,
                                   struct tile **best_tile,
                                   struct pf_path **path,
                                   struct settlermap *state)
@@ -352,12 +438,16 @@ int settler_evaluate_improvements(struct unit *punit,
                          * newv == best_newv; not initialized to zero,
                          * so that newv = 0 activities are not chosen. */
   int best_newv = 0;
+  bool improve_worked = FALSE;
+  int best_extra = 0;
+  int best_delay = 0;
 
   /* closest worker, if any, headed towards target tile */
   struct unit *enroute = NULL;
 
   pft_fill_unit_parameter(&parameter, punit);
-  parameter.can_invade_tile = autosettler_enter_territory;
+  parameter.omniscience = !has_handicap(pplayer, H_MAP);
+  parameter.get_TB = autosettler_tile_behavior;
   pfm = pf_map_new(&parameter);
 
   city_list_iterate(pplayer->cities, pcity) {
@@ -398,7 +488,7 @@ int settler_evaluate_improvements(struct unit *punit,
       }
 
       if (pf_map_position(pfm, ptile, &pos)) {
-        int eta = FC_INFINITY, inbound_distance = FC_INFINITY, time;
+        int eta = FC_INFINITY, inbound_distance = FC_INFINITY, turns;
 
         if (enroute) {
           eta = state[tile_index(ptile)].eta;
@@ -425,57 +515,117 @@ int settler_evaluate_improvements(struct unit *punit,
           oldv = city_tile_value(pcity, ptile, 0, 0);
 
           /* Now, consider various activities... */
-          activity_type_iterate(act) {
-            struct act_tgt target = { .type = ATT_SPECIAL, .obj.spe = S_LAST };
+          as_transform_activity_iterate(act) {
+            struct extra_type *target = NULL;
+            enum extra_cause cause = activity_to_extra_cause(act);
+            enum extra_rmcause rmcause = activity_to_extra_rmcause(act);
+
+            if (cause != EC_NONE) {
+              target = next_extra_for_tile(ptile, cause, pplayer,
+                                           punit);
+            } else if (rmcause != ERM_NONE) {
+              target = prev_extra_in_tile(ptile, rmcause, pplayer,
+                                          punit);
+            }
 
             if (adv_city_worker_act_get(pcity, cindex, act) >= 0
-                /* These need separate implementations. */
-                && act != ACTIVITY_BASE
-                && act != ACTIVITY_GEN_ROAD
-                && can_unit_do_activity_targeted_at(punit, act, &target,
+                && can_unit_do_activity_targeted_at(punit, act, target,
                                                     ptile)) {
               int extra = 0;
               int base_value = adv_city_worker_act_get(pcity, cindex, act);
 
-              time = pos.turn + get_turns_for_activity_at(punit, act, ptile);
-
-              if (act == ACTIVITY_FALLOUT) {
-                extra = pplayer->ai_common.frost;
-              } else if (act == ACTIVITY_POLLUTION) {
-                extra = pplayer->ai_common.warmth;
+              turns = pos.turn + get_turns_for_activity_at(punit, act, ptile,
+                                                           target);
+              if (pos.moves_left == 0) {
+                /* We need moves left to begin activity immediately. */
+                turns++;
               }
 
-              consider_settler_action(pplayer, act, &target, extra, base_value,
-                                      oldv, in_use, time,
-                                      &best_newv, &best_oldv,
-                                      best_act, best_target, best_tile, ptile);
+              consider_settler_action(pplayer, act, target, extra, base_value,
+                                      oldv, in_use, turns,
+                                      &best_newv, &best_oldv, &best_extra,
+                                      &improve_worked,
+                                      &best_delay, best_act, best_target,
+                                      best_tile, ptile);
 
             } /* endif: can the worker perform this action */
-          } activity_type_iterate_end;
+          } as_transform_activity_iterate_end;
 
-          road_type_iterate(proad) {
-            struct act_tgt target = { .type = ATT_ROAD, .obj.road = road_number(proad) };
-            int base_value = adv_city_worker_road_get(pcity, cindex, proad);
+          extra_type_iterate(pextra) {
+            enum unit_activity act = ACTIVITY_LAST;
+            enum unit_activity eval_act = ACTIVITY_LAST;
+            int base_value;
+            bool removing = tile_has_extra(ptile, pextra);
+
+            if (removing) {
+              as_rmextra_activity_iterate(try_act) {
+                if (is_extra_removed_by_action(pextra, try_act)) {
+                  /* We do not even evaluate actions we can't do.
+                   * Removal is not considered prerequisite for anything */
+                  if (can_unit_do_activity_targeted_at(punit, try_act, pextra,
+                                                       ptile)) {
+                    act = try_act;
+                    eval_act = act;
+                    break;
+                  }
+                }
+              } as_rmextra_activity_iterate_end;
+            } else {
+              as_extra_activity_iterate(try_act) {
+                if (is_extra_caused_by_action(pextra, try_act)) {
+                  eval_act = try_act;
+                  if (can_unit_do_activity_targeted_at(punit, try_act, pextra,
+                                                       ptile)) {
+                    act = try_act;
+                    break;
+                  }
+                }
+              } as_extra_activity_iterate_end;
+            }
+
+            if (eval_act == ACTIVITY_LAST) {
+              /* No activity can provide (or remove) the extra */
+              continue;
+            }
+
+            if (removing) {
+              base_value = adv_city_worker_rmextra_get(pcity, cindex, pextra);
+            } else {
+              base_value = adv_city_worker_extra_get(pcity, cindex, pextra);
+            }
 
             if (base_value >= 0) {
               int extra;
+              struct road_type *proad;
 
-              time = pos.turn + get_turns_for_road_at(punit, proad, ptile);
+              turns = pos.turn + get_turns_for_activity_at(punit, eval_act,
+                                                           ptile, pextra);
+              if (pos.moves_left == 0) {
+                /* We need moves left to begin activity immediately. */
+                turns++;
+              }
 
-              if (proad->move_mode != RMM_NO_BONUS) {
+              proad = extra_road_get(pextra);
+
+              if (proad != NULL && road_provides_move_bonus(proad)) {
                 int mc_multiplier = 1;
                 int mc_divisor = 1;
                 int old_move_cost = tile_terrain(ptile)->movement_cost * SINGLE_MOVE;
 
-                road_type_iterate(pold) {
-                  if (tile_has_road(ptile, pold)) {
+                /* Here 'old' means actually 'without the evaluated': In case of
+                 * removal activity it's the value after the removal. */
+
+                extra_type_by_cause_iterate(EC_ROAD, pold) {
+                  if (tile_has_extra(ptile, pold) && pold != pextra) {
+                    struct road_type *po_road = extra_road_get(pold);
+
                     /* This ignores the fact that new road may be native to units that
                      * old road is not. */
-                    if (pold->move_cost < old_move_cost) {
-                      old_move_cost = pold->move_cost;
+                    if (po_road->move_cost < old_move_cost) {
+                      old_move_cost = po_road->move_cost;
                     }
                   }
-                } road_type_iterate_end;
+                } extra_type_by_cause_iterate_end;
 
                 if (proad->move_cost < old_move_cost) {
                   if (proad->move_cost >= terrain_control.move_fragments) {
@@ -491,89 +641,108 @@ int settler_evaluate_improvements(struct unit *punit,
                 }
 
                 extra = adv_settlers_road_bonus(ptile, proad) * mc_multiplier / mc_divisor;
+
               } else {
                 extra = 0;
               }
 
-              if (can_unit_do_activity_targeted_at(punit, ACTIVITY_GEN_ROAD, &target,
-                                                   ptile)) {
-                consider_settler_action(pplayer, ACTIVITY_GEN_ROAD, &target, extra, base_value,
-                                        oldv, in_use, time,
-                                        &best_newv, &best_oldv,
-                                        best_act, best_target, best_tile, ptile);
+              if (extra_has_flag(pextra, EF_GLOBAL_WARMING)) {
+                extra -= pplayer->ai_common.warmth;
+              }
+              if (extra_has_flag(pextra, EF_NUCLEAR_WINTER)) {
+                extra -= pplayer->ai_common.frost;
+              }
+
+              if (removing) {
+                extra = -extra;
+              }
+
+              if (act != ACTIVITY_LAST) {
+                consider_settler_action(pplayer, act, pextra, extra, base_value,
+                                        oldv, in_use, turns,
+                                        &best_newv, &best_oldv, &best_extra,
+                                        &improve_worked,
+                                        &best_delay, best_act, best_target,
+                                        best_tile, ptile);
               } else {
-                road_deps_iterate(&(proad->reqs), pdep) {
-                  struct act_tgt dep_tgt = { .type = ATT_ROAD, .obj.road = road_number(pdep) };
+                fc_assert(!removing);
+
+                road_deps_iterate(&(pextra->reqs), pdep) {
+                  struct extra_type *dep_tgt;
+
+                  dep_tgt = road_extra_get(pdep);
 
                   if (can_unit_do_activity_targeted_at(punit, ACTIVITY_GEN_ROAD,
-                                                       &dep_tgt, ptile)) {
-                    /* Consider building dependency road for later upgrade to target road.
+                                                       dep_tgt, ptile)) {
+                    /* Consider building dependency road for later upgrade to target extra.
                      * Here we set value to be sum of dependency
-                     * road and target road values, which increases want, and time is sum
-                     * of dependency and target build times, which decreases want. This can
+                     * road and target extra values, which increases want, and turns is sum
+                     * of dependency and target build turns, which decreases want. This can
                      * result in either bigger or lesser want than when checkin dependency
-                     * road for the sake of itself when its turn in road_type_iterate() is. */
-                    int dep_time = time + get_turns_for_road_at(punit, pdep, ptile);
-                    int dep_value = base_value + adv_city_worker_road_get(pcity, cindex, pdep);
+                     * road for the sake of itself when its turn in extra_type_iterate() is. */
+                    int dep_turns = turns + get_turns_for_activity_at(punit,
+                                                                      ACTIVITY_GEN_ROAD,
+                                                                      ptile,
+                                                                      dep_tgt);
+                    int dep_value = base_value + adv_city_worker_extra_get(pcity, cindex, dep_tgt);
 
-                    consider_settler_action(pplayer, ACTIVITY_GEN_ROAD, &dep_tgt, extra,
+                    consider_settler_action(pplayer, ACTIVITY_GEN_ROAD, dep_tgt, extra,
                                             dep_value,
-                                            oldv, in_use, dep_time,
-                                            &best_newv, &best_oldv,
-                                            best_act, best_target, best_tile, ptile);
+                                            oldv, in_use, dep_turns,
+                                            &best_newv, &best_oldv, &best_extra,
+                                            &improve_worked,
+                                            &best_delay, best_act, best_target,
+                                            best_tile, ptile);
                   }
                 } road_deps_iterate_end;
-              }
-            }
-          } road_type_iterate_end;
 
-          base_type_iterate(pbase) {
-            struct act_tgt target = { .type = ATT_BASE, .obj.base = base_number(pbase) };
-            int base_value = adv_city_worker_base_get(pcity, cindex, pbase);
+                base_deps_iterate(&(pextra->reqs), pdep) {
+                  struct extra_type *dep_tgt;
 
-            if (base_value > 0) {
-
-              time = pos.turn + get_turns_for_base_at(punit, pbase, ptile);
-
-              if (can_unit_do_activity_targeted_at(punit, ACTIVITY_BASE, &target,
-                                                   ptile)) {
-                consider_settler_action(pplayer, ACTIVITY_BASE, &target, 0, base_value,
-                                        oldv, in_use, time,
-                                        &best_newv, &best_oldv,
-                                        best_act, best_target, best_tile, ptile);
-              } else {
-                base_deps_iterate(&(pbase->reqs), pdep) {
-                  struct act_tgt dep_tgt = { .type = ATT_BASE, .obj.base = base_number(pdep) };
+                  dep_tgt = base_extra_get(pdep);
 
                   if (can_unit_do_activity_targeted_at(punit, ACTIVITY_BASE,
-                                                       &dep_tgt, ptile)) {
+                                                       dep_tgt, ptile)) {
                     /* Consider building dependency base for later upgrade to
-                     * target base.  See similar road implementation above for
+                     * target extra. See similar road implementation above for
                      * extended commentary. */
-                    int dep_time = time + get_turns_for_base_at(punit, pdep,
-                                                                ptile);
-                    int dep_value = base_value + adv_city_worker_base_get(pcity,
-                                                                          cindex,
-                                                                          pbase);
+                    int dep_turns = turns + get_turns_for_activity_at(punit,
+                                                                      ACTIVITY_BASE,
+                                                                      ptile,
+                                                                      dep_tgt);
+                    int dep_value = base_value + adv_city_worker_extra_get(pcity,
+                                                                           cindex,
+                                                                           dep_tgt);
 
-                    consider_settler_action(pplayer, ACTIVITY_BASE, &dep_tgt,
+                    consider_settler_action(pplayer, ACTIVITY_BASE, dep_tgt,
                                             0, dep_value, oldv, in_use,
-                                            dep_time, &best_newv, &best_oldv,
+                                            dep_turns, &best_newv, &best_oldv,
+                                            &best_extra, &improve_worked,
+                                            &best_delay,
                                             best_act, best_target,
                                             best_tile, ptile);
                   }
                 } base_deps_iterate_end;
               }
             }
-          } base_type_iterate_end;
-        } /* endif: can we finish sooner than current worker, if any? */
+          } extra_type_iterate_end;
+        } /* endif: can we arrive sooner than current worker, if any? */
       } /* endif: are we travelling to a legal destination? */
     } city_tile_iterate_index_end;
   } city_list_iterate_end;
 
-  best_newv /= WORKER_FACTOR;
+  if (!improve_worked) {
+    /* best_newv contains total value of improved tile. Check amount of improvement
+     * instead. */
+    best_newv = amortize((best_newv - best_oldv + best_extra) * WORKER_FACTOR, best_delay);
+  }
 
-  best_newv = MAX(best_newv, 0); /* sanity */
+  if (best_newv > 0) {
+    best_newv /= WORKER_FACTOR;
+    best_newv += 1;
+  } else {
+    best_newv = MAX(best_newv, 0); /* sanity */
+  }
 
   if (best_newv > 0) {
     log_debug("Settler %d@(%d,%d) wants to %s at (%d,%d) with desire %d",
@@ -598,12 +767,10 @@ int settler_evaluate_improvements(struct unit *punit,
 /****************************************************************************
   Return best city request to fulfill.
 ****************************************************************************/
-int settler_evaluate_city_requests(struct unit *punit,
-                                   enum unit_activity *best_act,
-                                   struct act_tgt *best_target,
-                                   struct tile **best_tile,
-                                   struct pf_path **path,
-                                   struct settlermap *state)
+struct city *settler_evaluate_city_requests(struct unit *punit,
+                                            struct worker_task **best_task,
+                                            struct pf_path **path,
+                                            struct settlermap *state)
 {
   const struct player *pplayer = unit_owner(punit);
   struct pf_parameter parameter;
@@ -611,17 +778,17 @@ int settler_evaluate_city_requests(struct unit *punit,
   struct pf_position pos;
   int best_value = -1;
   struct worker_task *best = NULL;
+  struct city *taskcity = NULL;
   int dist = FC_INFINITY;
 
   pft_fill_unit_parameter(&parameter, punit);
-  parameter.can_invade_tile = autosettler_enter_territory;
+  parameter.omniscience = !has_handicap(pplayer, H_MAP);
+  parameter.get_TB = autosettler_tile_behavior;
   pfm = pf_map_new(&parameter);
 
   /* Have nearby cities requests? */
   city_list_iterate(pplayer->cities, pcity) {
-    struct worker_task *ptask = &pcity->server.task_req;
-
-    if (ptask->ptile != NULL) {
+    worker_task_list_iterate(pcity->task_reqs, ptask) {
       bool consider = TRUE;
 
       /* Do not go to tiles that already have workers there. */
@@ -634,7 +801,7 @@ int settler_evaluate_city_requests(struct unit *punit,
       } unit_list_iterate_end;
 
       if (consider
-          && can_unit_do_activity_targeted_at(punit, ptask->act, &ptask->tgt,
+          && can_unit_do_activity_targeted_at(punit, ptask->act, ptask->tgt,
                                               ptask->ptile)) {
         /* closest worker, if any, headed towards target tile */
         struct unit *enroute = NULL;
@@ -666,18 +833,15 @@ int settler_evaluate_city_requests(struct unit *punit,
               dist = pos.turn;
               best = ptask;
               best_value = value;
+              taskcity = pcity;
             }
           }
         }
       }
-    }
+    } worker_task_list_iterate_end;
   } city_list_iterate_end;
 
-  if (best != NULL) {
-    *best_act = best->act;
-    *best_target = best->tgt;
-    *best_tile = best->ptile;
-  }
+  *best_task = best;
 
   if (path != NULL) {
     *path = best ? pf_map_path(pfm, best->ptile) : NULL;
@@ -685,11 +849,7 @@ int settler_evaluate_city_requests(struct unit *punit,
 
   pf_map_destroy(pfm);
 
-  if (best != NULL) {
-    return 1;
-  }
-
-  return 0;
+  return taskcity;
 }
 
 /**************************************************************************
@@ -701,11 +861,12 @@ void auto_settler_findwork(struct player *pplayer,
                            struct settlermap *state,
                            int recursion)
 {
+  struct worker_task *best_task;
   enum unit_activity best_act;
   struct tile *best_tile = NULL;
-  struct act_tgt best_target;
+  struct extra_type *best_target;
   struct pf_path *path = NULL;
-  int value;
+  struct city *taskcity;
 
   /* time it will take worker to complete its given task */
   int completion_time = 0;
@@ -721,24 +882,28 @@ void auto_settler_findwork(struct player *pplayer,
   CHECK_UNIT(punit);
 
   fc_assert_ret(pplayer && punit);
-  fc_assert_ret(unit_has_type_flag(punit, UTYF_CITIES)
+  fc_assert_ret(unit_is_cityfounder(punit)
                 || unit_has_type_flag(punit, UTYF_SETTLERS));
 
   /* Have nearby cities requests? */
 
-  value = settler_evaluate_city_requests(punit, &best_act, &best_target,
-                                         &best_tile, &path, state);
+  taskcity = settler_evaluate_city_requests(punit, &best_task, &path, state);
 
-  if (value > 0) {
+  if (taskcity != NULL) {
     if (path != NULL) {
       completion_time = pf_path_last_position(path)->turn;
     }
 
     adv_unit_new_task(punit, AUT_AUTO_SETTLER, best_tile);
 
-    auto_settler_setup_work(pplayer, punit, state, recursion,
-                            path, best_tile, best_act,
-                            &best_target, completion_time);
+    best_target = best_task->tgt;
+
+    if (auto_settler_setup_work(pplayer, punit, state, recursion,
+                                path, best_task->ptile, best_task->act,
+                                &best_target, completion_time)) {
+      clear_worker_task(taskcity, best_task);
+    }
+
     if (path != NULL) {
       pf_path_destroy(path);
     }
@@ -770,26 +935,27 @@ void auto_settler_findwork(struct player *pplayer,
 }
 
 /**************************************************************************
-  Setup our settler to do the work it has found
+  Setup our settler to do the work it has found. Returns TRUE if
+  started actual work.
 **************************************************************************/
-void auto_settler_setup_work(struct player *pplayer, struct unit *punit,
+bool auto_settler_setup_work(struct player *pplayer, struct unit *punit,
                              struct settlermap *state, int recursion,
                              struct pf_path *path,
                              struct tile *best_tile,
                              enum unit_activity best_act,
-                             struct act_tgt *best_target,
+                             struct extra_type **best_target,
                              int completion_time)
 {
   /* Run the "autosettler" program */
   if (punit->server.adv->task == AUT_AUTO_SETTLER) {
     struct pf_map *pfm = NULL;
     struct pf_parameter parameter;
-
+    bool working = FALSE;
     struct unit *displaced;
 
     if (!best_tile) {
       UNIT_LOG(LOG_DEBUG, punit, "giving up trying to improve terrain");
-      return; /* We cannot do anything */
+      return FALSE; /* We cannot do anything */
     }
 
     /* Mark the square as taken. */
@@ -803,8 +969,8 @@ void auto_settler_setup_work(struct player *pplayer, struct unit *punit,
                     && (real_map_distance(best_tile, unit_tile(punit))
                         < real_map_distance(best_tile,
                                             unit_tile(displaced)))));
-      UNIT_LOG(LOG_DEBUG, punit,
-               "%d (%d,%d) has displaced %d (%d,%d) on %d,%d",
+      UNIT_LOG(displaced->server.debug ? LOG_AI_TEST : LOG_DEBUG, punit,
+               "%d (%d,%d) has displaced %d (%d,%d) for worksite %d,%d",
                punit->id, completion_time,
                real_map_distance(best_tile, unit_tile(punit)),
                displaced->id, state[tile_index(best_tile)].eta,
@@ -814,7 +980,7 @@ void auto_settler_setup_work(struct player *pplayer, struct unit *punit,
 
     state[tile_index(best_tile)].enroute = punit->id;
     state[tile_index(best_tile)].eta = completion_time;
-      
+
     if (displaced) {
       struct tile *goto_tile = punit->goto_tile;
       int saved_id = punit->id;
@@ -826,7 +992,7 @@ void auto_settler_setup_work(struct player *pplayer, struct unit *punit,
         /* Actions of the displaced settler somehow caused this settler
          * to die. (maybe by recursively giving control back to this unit)
          */
-        return;
+        return FALSE;
       }
       if (goto_tile != punit->goto_tile || old_pos != unit_tile(punit)
           || punit->activity != ACTIVITY_IDLE) {
@@ -841,13 +1007,20 @@ void auto_settler_setup_work(struct player *pplayer, struct unit *punit,
                  punit->id,
                  TILE_XY(old_pos), TILE_XY(goto_tile),
                  TILE_XY(unit_tile(punit)), TILE_XY(punit->goto_tile));
-        return;
+        return FALSE;
       }
     }
 
+    UNIT_LOG(LOG_DEBUG, punit,
+             "is heading to do %s(%s) at (%d, %d)",
+             unit_activity_name(best_act),
+             best_target && *best_target ? extra_rule_name(*best_target) : "-",
+             TILE_XY(best_tile));
+
     if (!path) {
       pft_fill_unit_parameter(&parameter, punit);
-      parameter.can_invade_tile = autosettler_enter_territory;
+      parameter.omniscience = !has_handicap(pplayer, H_MAP);
+      parameter.get_TB = autosettler_tile_behavior;
       pfm = pf_map_new(&parameter);
       path = pf_map_path(pfm, best_tile);
     }
@@ -866,18 +1039,30 @@ void auto_settler_setup_work(struct player *pplayer, struct unit *punit,
           unit_activity_handling(punit, best_act);
         }
         send_unit_info(NULL, punit); /* FIXME: probably duplicate */
+
+        UNIT_LOG(LOG_DEBUG, punit,
+                 "reached its worksite and started work");
+        working = TRUE;
+      } else if (alive) {
+        UNIT_LOG(LOG_DEBUG, punit,
+                 "didn't start work yet; got to (%d, %d) with "
+                 "%d move frags left", TILE_XY(unit_tile(punit)),
+                 punit->moves_left);
       }
     } else {
-      log_debug("Autosettler does not find path (%d, %d) -> (%d, %d)",
-                TILE_XY(unit_tile(punit)), TILE_XY(best_tile));
+      UNIT_LOG(LOG_DEBUG, punit,
+               "does not find path (%d, %d) -> (%d, %d)",
+               TILE_XY(unit_tile(punit)), TILE_XY(best_tile));
     }
 
     if (pfm) {
       pf_map_destroy(pfm);
     }
 
-    return;
+    return working;
   }
+
+  return FALSE;
 }
 #undef LOG_SETTLER
 
@@ -893,7 +1078,7 @@ bool adv_settler_safe_tile(const struct player *pplayer, struct unit *punit,
     }
   } unit_list_iterate_end;
 
-  if (is_square_threatened(pplayer, ptile)) {
+  if (is_square_threatened(pplayer, ptile, !has_handicap(pplayer, H_FOG))) {
     return FALSE;
   }
 
@@ -949,7 +1134,7 @@ void auto_settlers_player(struct player *pplayer)
   unit_list_iterate_safe(pplayer->units, punit) {
     if ((punit->ai_controlled || pplayer->ai_controlled)
         && (unit_has_type_flag(punit, UTYF_SETTLERS)
-            || unit_has_type_flag(punit, UTYF_CITIES))
+            || unit_is_cityfounder(punit))
         && !unit_has_orders(punit)
         && punit->moves_left > 0) {
       log_debug("%s settler at (%d, %d) is ai controlled.",
