@@ -26,6 +26,9 @@
 #include "commands.h"
 #include "voting.h"
 
+/* ai */
+#include "difficulty.h"
+
 /* Set the synopsis text to don't be translated. */
 #define SYN_ORIG_(String) "*" String
 /* Test if the synopsis should be translated or not. */
@@ -37,7 +40,7 @@ struct command {
   const char *synopsis;   /* one or few-line summary of usage */
   const char *short_help; /* one line (about 70 chars) description */
   const char *extra_help; /* extra help information; will be line-wrapped */
-  char *(*extra_help_func)(void);
+  char *(*extra_help_func)(const char *cmdname);
                           /* dynamically generated help; if non-NULL,
                            * extra_help is ignored. Must be pre-translated. */
   enum cmd_echo echo;     /* Who will be notified when used. */
@@ -319,16 +322,18 @@ static struct command commands[] = {
    CMD_ECHO_ADMINS, VCF_NONE, 0
   },
   {"create",	ALLOW_CTRL,
-   /* TRANS: translate text between <> only */
+   /* TRANS: translate text between <> and [] only */
    N_("create <player-name> [ai type]"),
    N_("Create an AI player with a given name."),
    /* TRANS: don't translate text between single quotes */
    N_("With the 'create' command a new player with the given name is "
       "created.\n"
+      "If 'player-name' is empty, a random name will be assigned when the "
+      "game begins. Until then the player will be known by a name derived "
+      "from its type.\n"
       "The 'ai type' parameter can be used to select which AI module will be "
-      "used for the created player. This requires that Freeciv has been "
-      "compiled with loadable AI module support and that the respective module "
-      "has been loaded.\n"
+      "used for the created player. This requires that the respective module "
+      "has been loaded or built in to the server.\n"
       "If the game has already started, the new player will have no units or "
       "cities; also, if no free player slots are available, the slot of a "
       "dead player can be reused (removing all record of that player from the "
@@ -339,71 +344,67 @@ static struct command commands[] = {
    /* no translatable parameters */
    SYN_ORIG_("away"),
    N_("Set yourself in away mode. The AI will watch your back."),
-   N_("The AI will govern your nation but do minimal changes."), NULL,
+   NULL, ai_level_help,
    CMD_ECHO_NONE, VCF_NONE, 50
   },
+  {"handicapped",	ALLOW_CTRL,
+   /* TRANS: translate text between <> only */
+   N_("handicapped\n"
+      "handicapped <player-name>"),
+   /* TRANS: translate 'Handicapped' as AI skill level */
+   N_("Set one or all AI players to 'Handicapped'."), NULL, ai_level_help,
+   CMD_ECHO_ALL, VCF_NONE, 50
+  },
   {"novice",	ALLOW_CTRL,
    /* TRANS: translate text between <> only */
    N_("novice\n"
       "novice <player-name>"),
-   N_("Set one or all AI players to 'novice'."),
-   N_("With no arguments, sets all AI players to skill level 'novice', and "
-      "sets the default level for any new AI players to 'novice'. With an "
-      "argument, sets the skill level for that player only."), NULL,
+   /* TRANS: translate 'Novice' as AI skill level */
+   N_("Set one or all AI players to 'Novice'."), NULL, ai_level_help,
    CMD_ECHO_ALL, VCF_NONE, 50
   },
   {"easy",	ALLOW_CTRL,
    /* TRANS: translate text between <> only */
    N_("easy\n"
       "easy <player-name>"),
-   N_("Set one or all AI players to 'easy'."),
-   N_("With no arguments, sets all AI players to skill level 'easy', and "
-      "sets the default level for any new AI players to 'easy'. With an "
-      "argument, sets the skill level for that player only."), NULL,
+   /* TRANS: translate 'Easy' as AI skill level */
+   N_("Set one or all AI players to 'Easy'."), NULL, ai_level_help,
    CMD_ECHO_ALL, VCF_NONE, 50
   },
   {"normal",	ALLOW_CTRL,
    /* TRANS: translate text between <> only */
    N_("normal\n"
       "normal <player-name>"),
-   N_("Set one or all AI players to 'normal'."),
-   N_("With no arguments, sets all AI players to skill level 'normal', and "
-      "sets the default level for any new AI players to 'normal'. With an "
-      "argument, sets the skill level for that player only."), NULL,
+   /* TRANS: translate 'Normal' as AI skill level */
+   N_("Set one or all AI players to 'Normal'."), NULL, ai_level_help,
    CMD_ECHO_ALL, VCF_NONE, 50
   },
   {"hard",	ALLOW_CTRL,
    /* TRANS: translate text between <> only */
    N_("hard\n"
       "hard <player-name>"),
-   N_("Set one or all AI players to 'hard'."),
-   N_("With no arguments, sets all AI players to skill level 'hard', and "
-      "sets the default level for any new AI players to 'hard'. With an "
-      "argument, sets the skill level for that player only."), NULL,
+   /* TRANS: translate 'Hard' as AI skill level */
+   N_("Set one or all AI players to 'Hard'."), NULL, ai_level_help,
    CMD_ECHO_ALL, VCF_NONE, 50
   },
   {"cheating",  ALLOW_CTRL,
    /* TRANS: translate text between <> only */
    N_("cheating\n"
       "cheating <player-name>"),
-   N_("Set one or all AI players to 'cheating'."),
-   N_("With no arguments, sets all AI players to skill level 'cheating', and "
-      "sets the default level for any new AI players to 'cheating'. With an "
-      "argument, sets the skill level for that player only."), NULL,
+   /* TRANS: translate 'Cheating' as AI skill level */
+   N_("Set one or all AI players to 'Cheating'."), NULL, ai_level_help,
    CMD_ECHO_ALL, VCF_NONE, 50
   },
+#ifdef DEBUG
   {"experimental",	ALLOW_CTRL,
    /* TRANS: translate text between <> only */
    N_("experimental\n"
       "experimental <player-name>"),
-   N_("Set one or all AI players to 'experimental'."),
-   N_("With no arguments, sets all AI players to skill 'experimental', and "
-      "sets the default level for any new AI players to this. With an "
-      "argument, sets the skill level for that player only. THIS IS ONLY "
-      "FOR TESTING OF NEW AI FEATURES! For ordinary servers, this option "
-      "has no effect."), NULL,
+   /* TRANS: translate 'Experimental' as AI skill level */
+   N_("Set one or all AI players to 'Experimental'."), NULL, ai_level_help,
    CMD_ECHO_ALL, VCF_NONE, 50
   },
+#endif /* DEBUG */
   {"cmdlevel",	ALLOW_ADMIN,
    /* TRANS: translate text between <> only */
    N_("cmdlevel\n"
@@ -548,7 +549,6 @@ static struct command commands[] = {
       "and use the 'start' command once players have reconnected."), NULL,
    CMD_ECHO_ADMINS, VCF_NONE, 0
   },
-#ifdef DEBUG
   {"scensave",	ALLOW_ADMIN,
    /* TRANS: translate text between <> only */
    N_("scensave\n"
@@ -562,7 +562,6 @@ static struct command commands[] = {
       "and use the 'start' command once players have reconnected."), NULL,
    CMD_ECHO_ADMINS, VCF_NONE, 0
   },
-#endif /* DEBUG */
   {"load",      ALLOW_CTRL,
    /* TRANS: translate text between <> only */
    N_("load\n"
@@ -596,6 +595,15 @@ static struct command commands[] = {
       "  default  - using default values\n"), NULL,
    CMD_ECHO_ALL, VCF_NONE, 50
   },
+  {"default",	ALLOW_CTRL,
+   /* TRANS: translate text between <> only */
+   N_("default <option name>"),
+   N_("Set option to its default value"),
+   N_("Reset the option to its default value. If the default ever changes "
+      "in a future version, the option's value will follow that change."),
+      NULL,
+   CMD_ECHO_ALL, VCF_NONE, 50
+  },
   {"lua", ALLOW_ADMIN,
    /* TRANS: translate text between <> only */
    N_("lua cmd <script line>\n"
@@ -725,7 +733,7 @@ char *command_extra_help(const struct command *pcommand)
 {
   if (pcommand->extra_help_func) {
     fc_assert(pcommand->extra_help == NULL);
-    return pcommand->extra_help_func();
+    return pcommand->extra_help_func(pcommand->name);
   } else if (pcommand->extra_help) {
     return fc_strdup(_(pcommand->extra_help));
   } else {
