@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 2005 - The Freeciv Project
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,7 +19,6 @@
 
 /* utility */
 #include "bitvector.h"
-#include "capability.h"
 #include "fcintl.h"
 #include "log.h"
 #include "shared.h"
@@ -120,7 +119,7 @@ void edithand_send_initial_packets(struct conn_list *dest)
   /* Send map start positions. */
   map_startpos_iterate(psp) {
     startpos.id = tile_index(startpos_tile(psp));
-    startpos.remove = FALSE;
+    startpos.removal = FALSE;
     startpos.tag = 0;
 
     startpos_pack(psp, &startpos_full);
@@ -217,7 +216,7 @@ void handle_edit_mode(struct connection *pc, bool is_edit_mode)
 ****************************************************************************/
 static bool edit_tile_terrain_handling(struct tile *ptile,
                                        struct terrain *pterrain,
-                                       bool send_tile_info)
+                                       bool send_info)
 {
   struct terrain *old_terrain = tile_terrain(ptile);
 
@@ -234,7 +233,7 @@ static bool edit_tile_terrain_handling(struct tile *ptile,
     need_continents_reassigned = TRUE;
   }
 
-  if (send_tile_info) {
+  if (send_info) {
     update_tile_knowledge(ptile);
   }
 
@@ -247,7 +246,7 @@ static bool edit_tile_terrain_handling(struct tile *ptile,
 ****************************************************************************/
 static bool edit_tile_resource_handling(struct tile *ptile,
                                         struct resource *presource,
-                                        bool send_tile_info)
+                                        bool send_info)
 {
   if (presource == tile_resource(ptile)) {
     return FALSE;
@@ -260,7 +259,7 @@ static bool edit_tile_resource_handling(struct tile *ptile,
 
   tile_set_resource(ptile, presource);
 
-  if (send_tile_info) {
+  if (send_info) {
     update_tile_knowledge(ptile);
   }
 
@@ -268,146 +267,35 @@ static bool edit_tile_resource_handling(struct tile *ptile,
 }
 
 /****************************************************************************
-  Base function to edit the special property of a tile. Returns TRUE if
-  the special state has changed.
+  Base function to edit the extras property of a tile. Returns TRUE if
+  the extra state has changed.
 ****************************************************************************/
-static bool edit_tile_special_handling(struct tile *ptile,
-                                       enum tile_special_type special,
-                                       bool remove_mode,
-                                       bool send_tile_info)
+static bool edit_tile_extra_handling(struct tile *ptile,
+                                     struct extra_type *pextra,
+                                     bool remove_mode, bool send_info)
 {
-  fc_assert_ret_val(special != S_OLD_FORTRESS && special != S_OLD_AIRBASE, FALSE);
-  fc_assert_ret_val(special != S_OLD_ROAD && special != S_OLD_RAILROAD, FALSE);
-
   if (remove_mode) {
-    if (!tile_has_special(ptile, special)) {
+    if (!tile_has_extra(ptile, pextra)) {
       return FALSE;
     }
 
-    tile_remove_special(ptile, special);
+    if (!tile_extra_rm_apply(ptile, pextra)) {
+      return FALSE;
+    }
 
     terrain_changed(ptile);
 
   } else {
-    if (tile_has_special(ptile, special)
-        || !is_native_tile_to_special(special, ptile)) {
+    if (tile_has_extra(ptile, pextra)) {
       return FALSE;
     }
 
-    tile_add_special(ptile, special);
-  }
-
-  if (send_tile_info) {
-    update_tile_knowledge(ptile);
-  }
-
-  return TRUE;
-}
-
-/****************************************************************************
-  Recursively add all road dependencies to add given road.
-****************************************************************************/
-static bool add_recursive_roads(struct tile *ptile, struct road_type *proad,
-                                int rec)
-{
-  if (rec > MAX_ROAD_TYPES) {
-    /* Infinite recursion */
-    return FALSE;
-  }
-
-  /* First place dependency roads */
-  road_deps_iterate(&(proad->reqs), pdep) {
-    if (!tile_has_road(ptile, pdep)) {
-      add_recursive_roads(ptile, pdep, rec + 1);
-    }
-  } road_deps_iterate_end;
-
-  /* Is tile native for road after that? */
-  if (!is_native_tile_to_road(proad, ptile)) {
-    return FALSE;
-  }
-
-  tile_add_road(ptile, proad);
-
-  return TRUE;
-}
-
-/****************************************************************************
-  Base function to edit the road property of a tile. Returns TRUE if
-  the road state has changed.
-****************************************************************************/
-static bool edit_tile_road_handling(struct tile *ptile,
-                                    struct road_type *proad,
-                                    bool remove_mode, bool send_tile_info)
-{
-  if (remove_mode) {
-    if (!tile_has_road(ptile, proad)) {
-      return FALSE;
-    }
-
-    tile_remove_road(ptile, proad);
-  } else {
-    if (!add_recursive_roads(ptile, proad, 0)) {
+    if (!tile_extra_apply(ptile, pextra)) {
       return FALSE;
     }
   }
 
-  if (send_tile_info) {
-    update_tile_knowledge(ptile);
-  }
-
-  return TRUE;
-}
-
-/****************************************************************************
-  Recursively add all base dependencies to add given base.
-****************************************************************************/
-static bool add_recursive_bases(struct tile *ptile, struct base_type *pbase,
-                                int rec)
-{
-  if (rec > MAX_BASE_TYPES) {
-    /* Infinite recursion */
-    return FALSE;
-  }
-
-  /* First place dependency roads */
-  base_deps_iterate(&(pbase->reqs), pdep) {
-    if (!tile_has_base(ptile, pdep)) {
-      add_recursive_bases(ptile, pdep, rec + 1);
-    }
-  } base_deps_iterate_end;
-
-  /* Is tile native for base after that? */
-  if (!is_native_tile_to_base(pbase, ptile)) {
-    return FALSE;
-  }
-
-  create_base(ptile, pbase, NULL);
-
-  return TRUE;
-}
-
-/****************************************************************************
-  Base function to edit the base property of a tile. Returns TRUE if
-  the base state has changed.
-****************************************************************************/
-static bool edit_tile_base_handling(struct tile *ptile,
-                                    struct base_type *pbase,
-                                    bool remove_mode, bool send_tile_info)
-{
-  if (remove_mode) {
-    if (!tile_has_base(ptile, pbase)) {
-      return FALSE;
-    }
-
-    tile_remove_base(ptile, pbase);
-  } else {
-    if (!add_recursive_bases(ptile, pbase, 0)) {
-      return FALSE;
-    }
-  }
-
-  if (send_tile_info) {
+  if (send_info) {
     update_tile_knowledge(ptile);
   }
 
@@ -479,13 +367,11 @@ void handle_edit_tile_resource(struct connection *pc, int tile,
 }
 
 /****************************************************************************
-  Handle a request to change one or more tiles' specials. The 'remove'
-  argument controls whether to remove or add the given special of type
-  'special' from the tile.
+  Handle a request to change one or more tiles' extras. The 'remove'
+  argument controls whether to remove or add the given extra from the tile.
 ****************************************************************************/
-void handle_edit_tile_special(struct connection *pc, int tile,
-                              enum tile_special_type special,
-                              bool remove, int size)
+void handle_edit_tile_extra(struct connection *pc, int tile,
+                            int id, bool removal, int size)
 {
   struct tile *ptile_center;
 
@@ -497,88 +383,18 @@ void handle_edit_tile_special(struct connection *pc, int tile,
     return;
   }
 
-  if (!(0 <= special && special < S_LAST)) {
+  if (id < 0 || id >= game.control.num_extra_types) {
     notify_conn(pc->self, ptile_center, E_BAD_COMMAND, ftc_editor,
                 /* TRANS: ..." the tile <tile-coordinates> because"... */
-                _("Cannot modify specials for the tile %s because "
-                  "%d is not a valid terrain special id."),
-                tile_link(ptile_center), special);
-    return;
-  }
-
-  conn_list_do_buffer(game.est_connections);
-  square_iterate(ptile_center, size - 1, ptile) {
-    edit_tile_special_handling(ptile, special, remove, TRUE);
-  } square_iterate_end;
-  conn_list_do_unbuffer(game.est_connections);
-}
-
-/****************************************************************************
-  Handle a request to change the road at one or more than one tile.
-****************************************************************************/
-void handle_edit_tile_road(struct connection *pc, int tile,
-                           Road_type_id id, bool remove, int size)
-{
-  struct tile *ptile_center;
-  struct road_type *proad;
-
-  ptile_center = index_to_tile(tile);
-  if (!ptile_center) {
-    notify_conn(pc->self, NULL, E_BAD_COMMAND, ftc_editor,
-                _("Cannot edit the tile because %d is not a valid "
-                  "tile index on this map!"), tile);
-    return;
-  }
-
-  proad = road_by_number(id);
-
-  if (!proad) {
-    notify_conn(pc->self, ptile_center, E_BAD_COMMAND, ftc_editor,
-                /* TRANS: ..." the tile <tile-coordinates> because"... */
-                _("Cannot modify road for the tile %s because "
-                  "%d is not a valid road type id."),
+                _("Cannot modify extras for the tile %s because "
+                  "%d is not a valid extra id."),
                 tile_link(ptile_center), id);
     return;
   }
 
   conn_list_do_buffer(game.est_connections);
   square_iterate(ptile_center, size - 1, ptile) {
-    edit_tile_road_handling(ptile, proad, remove, TRUE);
-  } square_iterate_end;
-  conn_list_do_unbuffer(game.est_connections);
-}
-
-/****************************************************************************
-  Handle a request to change the military base at one or more than one tile.
-****************************************************************************/
-void handle_edit_tile_base(struct connection *pc, int tile,
-                           Base_type_id id, bool remove, int size)
-{
-  struct tile *ptile_center;
-  struct base_type *pbase;
-
-  ptile_center = index_to_tile(tile);
-  if (!ptile_center) {
-    notify_conn(pc->self, NULL, E_BAD_COMMAND, ftc_editor,
-                _("Cannot edit the tile because %d is not a valid "
-                  "tile index on this map!"), tile);
-    return;
-  }
-
-  pbase = base_by_number(id);
-
-  if (!pbase) {
-    notify_conn(pc->self, ptile_center, E_BAD_COMMAND, ftc_editor,
-                /* TRANS: ..." the tile <tile-coordinates> because"... */
-                _("Cannot modify base for the tile %s because "
-                  "%d is not a valid base type id."),
-                tile_link(ptile_center), id);
-    return;
-  }
-
-  conn_list_do_buffer(game.est_connections);
-  square_iterate(ptile_center, size - 1, ptile) {
-    edit_tile_base_handling(ptile, pbase, remove, TRUE);
+    edit_tile_extra_handling(ptile, extra_by_number(id), removal, TRUE);
   } square_iterate_end;
   conn_list_do_unbuffer(game.est_connections);
 }
@@ -600,34 +416,15 @@ void handle_edit_tile(struct connection *pc,
     return;
   }
 
-  /* Handle changes in specials. */
-  if (!BV_ARE_EQUAL(packet->specials, ptile->special)) {
-    tile_special_type_iterate(spe) {
-      if (edit_tile_special_handling(ptile, spe,
-                                     !BV_ISSET(packet->specials, spe), FALSE)) {
+  /* Handle changes in extras. */
+  if (!BV_ARE_EQUAL(packet->extras, ptile->extras)) {
+    extra_type_iterate(pextra) {
+      if (edit_tile_extra_handling(ptile, pextra,
+                                   !BV_ISSET(packet->extras, extra_number(pextra)),
+                                   FALSE)) {
         changed = TRUE;
       }
-    } tile_special_type_iterate_end;
-  }
-
-  /* Handle changes in roads. */
-  if (!(BV_ARE_EQUAL(packet->roads, ptile->roads))) {
-    road_type_iterate(proad) {
-      edit_tile_road_handling(ptile, proad,
-                              !BV_ISSET(packet->roads, road_number(proad)),
-                              FALSE);
-    } road_type_iterate_end;
-    changed = TRUE;
-  }
-
-  /* Handle changes in bases. */
-  if (!(BV_ARE_EQUAL(packet->bases, ptile->bases))) {
-    base_type_iterate(pbase) {
-      edit_tile_base_handling(ptile, pbase,
-                              !BV_ISSET(packet->bases, base_number(pbase)),
-                              FALSE);
-    } base_type_iterate_end;
-    changed = TRUE;
+    } extra_type_iterate_end;
   }
 
   /* Handle changes in label */
@@ -779,7 +576,7 @@ void handle_edit_unit_remove(struct connection *pc, int owner,
     if (i >= count) {
       break;
     }
-    if (unit_type(punit) != punittype
+    if (unit_type_get(punit) != punittype
         || unit_owner(punit) != pplayer) {
       continue;
     }
@@ -815,7 +612,7 @@ void handle_edit_unit(struct connection *pc,
   struct unit *punit;
   int id;
   bool changed = FALSE;
-  int moves_left, fuel, hp;
+  int fuel, hp;
 
   id = packet->id;
   punit = game_unit_by_number(id);
@@ -825,15 +622,10 @@ void handle_edit_unit(struct connection *pc,
     return;
   }
 
-  putype = unit_type(punit);
+  putype = unit_type_get(punit);
 
-  if (has_capability("extended_move_rate", pc->capability)) {
-    moves_left = packet->moves_left_new;
-  } else {
-    moves_left = packet->moves_left_old;
-  }
-  if (moves_left != punit->moves_left) {
-    punit->moves_left = moves_left;
+  if (packet->moves_left != punit->moves_left) {
+    punit->moves_left = packet->moves_left;
     changed = TRUE;
   }
 
@@ -933,7 +725,7 @@ void handle_edit_city_create(struct connection *pc, int owner, int tile,
 
   if (size > 1) {
     /* FIXME: Slow and inefficient for large size changes. */
-    city_change_size(pcity, CLIP(1, size, MAX_CITY_SIZE), pplayer);
+    city_change_size(pcity, CLIP(1, size, MAX_CITY_SIZE), pplayer, NULL);
     send_city_info(NULL, pcity);
   }
 
@@ -991,11 +783,16 @@ void handle_edit_city(struct connection *pc,
                   packet->size, city_link(pcity));
     } else {
       /* FIXME: Slow and inefficient for large size changes. */
-      city_change_size(pcity, packet->size, NULL);
+      city_change_size(pcity, packet->size, NULL, NULL);
       changed = TRUE;
     }
   }
 
+  if (packet->history != pcity->history) {
+    pcity->history = packet->history;
+    changed = TRUE;
+  }
+
   /* Handle city improvement changes. */
   improvement_iterate(pimprove) {
     oldcity = NULL;
@@ -1111,6 +908,7 @@ void handle_edit_player_create(struct connection *pc, int tag)
 {
   struct player *pplayer;
   struct nation_type *pnation;
+  struct research *presearch;
 
   if (player_count() >= player_slot_count()) {
     notify_conn(pc->self, NULL, E_BAD_COMMAND, ftc_editor,
@@ -1136,7 +934,8 @@ void handle_edit_player_create(struct connection *pc, int tag)
     return;
   }
 
-  pplayer = server_create_player(-1, default_ai_type_name(), NULL);
+  pplayer = server_create_player(-1, default_ai_type_name(),
+                                 NULL, FALSE);
   if (!pplayer) {
     notify_conn(pc->self, NULL, E_BAD_COMMAND, ftc_editor,
                 _("Player creation failed."));
@@ -1149,19 +948,23 @@ void handle_edit_player_create(struct connection *pc, int tag)
     /* Find a color for the new player. */
     assign_player_colors();
   }
-  sz_strlcpy(pplayer->username, ANON_USER_NAME);
+  sz_strlcpy(pplayer->username, _(ANON_USER_NAME));
+  pplayer->unassigned_user = TRUE;
   pplayer->is_connected = FALSE;
-  pplayer->government = pnation->init_government;
+  pplayer->government = init_government_of_nation(pnation);
   pplayer->server.got_first_city = FALSE;
 
   pplayer->economic.gold = 0;
   pplayer->economic = player_limit_to_max_rates(pplayer);
 
-  init_tech(pplayer, TRUE);
-  give_global_initial_techs(pplayer);
-  give_nation_initial_techs(pplayer);
+  presearch = research_get(pplayer);
+  init_tech(presearch, TRUE);
+  give_initial_techs(presearch, 0);
 
   send_player_all_c(pplayer, NULL);
+  /* Send research info after player info, else the client will complain
+   * about invalid team. */
+  send_research_info(presearch, NULL);
   if (tag > 0) {
     dsend_packet_edit_object_created(pc, tag, player_number(pplayer));
   }
@@ -1200,8 +1003,9 @@ void handle_edit_player(struct connection *pc,
   struct player *pplayer;
   bool changed = FALSE, update_research = FALSE;
   struct nation_type *pnation;
-  struct player_research *research;
+  struct research *research;
   enum tech_state known;
+  struct government *gov;
 
   pplayer = player_by_number(packet->id);
   if (!pplayer) {
@@ -1211,7 +1015,7 @@ void handle_edit_player(struct connection *pc,
     return;
   }
 
-  research = player_research_get(pplayer);
+  research = research_get(pplayer);
 
 
   /* Handle player name change. */
@@ -1274,21 +1078,20 @@ void handle_edit_player(struct connection *pc,
     changed = TRUE;
     update_research = TRUE;
   }
-  
+
   /* Handle a change in known inventions. */
-  /* FIXME: Modifies struct player_research directly. */
   advance_index_iterate(A_FIRST, tech) {
-    known = player_invention_state(pplayer, tech);
+    known = research_invention_state(research, tech);
     if ((packet->inventions[tech] && known == TECH_KNOWN)
         || (!packet->inventions[tech] && known != TECH_KNOWN)) {
       continue;
     }
     if (packet->inventions[tech]) {
       /* FIXME: Side-effect modifies game.info.global_advances. */
-      player_invention_set(pplayer, tech, TECH_KNOWN);
+      research_invention_set(research, tech, TECH_KNOWN);
       research->techs_researched++;
     } else {
-      player_invention_set(pplayer, tech, TECH_UNKNOWN);
+      research_invention_set(research, tech, TECH_UNKNOWN);
       research->techs_researched--;
     }
     changed = TRUE;
@@ -1309,28 +1112,43 @@ void handle_edit_player(struct connection *pc,
     }
   }
 
+  /* Handle player government change */
+  gov = government_by_number(packet->government);
+  if (gov != pplayer->government) {
+    if (gov != game.government_during_revolution) {
+      government_change(pplayer, gov, FALSE);
+    } else {
+      int turns = revolution_length(gov, pplayer);
+
+      if (turns >= 0) {
+        pplayer->government = gov;
+        pplayer->revolution_finishes = game.info.turn + turns;
+      }
+    }
+
+    changed = TRUE;
+  }
 
   /* TODO: Handle more property edits. */
 
-
   if (update_research) {
     Tech_type_id current, goal;
 
-    player_research_update(pplayer);
+    research_update(research);
 
-    /* FIXME: Modifies struct player_research directly. */
+    /* FIXME: Modifies struct research directly. */
 
     current = research->researching;
     goal = research->tech_goal;
 
     if (current != A_UNSET) {
-      known = player_invention_state(pplayer, current);
+      known = research_invention_state(research, current);
       if (known != TECH_PREREQS_KNOWN) {
         research->researching = A_UNSET;
       }
     }
     if (goal != A_UNSET) {
-      known = player_invention_state(pplayer, goal);
+      known = research_invention_state(research, goal);
       if (known == TECH_KNOWN) {
         research->tech_goal = A_UNSET;
       }
@@ -1339,6 +1157,7 @@ void handle_edit_player(struct connection *pc,
 
     /* Inform everybody about global advances */
     send_game_info(NULL);
+    send_research_info(research, NULL);
   }
 
   if (changed) {
@@ -1503,7 +1322,7 @@ void handle_edit_startpos(struct connection *pconn,
   }
 
   /* Handle. */
-  if (packet->remove) {
+  if (packet->removal) {
     changed = map_startpos_remove(ptile);
   } else {
     if (NULL != map_startpos_get(ptile)) {
@@ -1596,9 +1415,14 @@ void handle_edit_game(struct connection *pc,
     changed = TRUE;
   }
 
-  if (0 != strncmp(packet->scenario_desc, game.scenario.description,
+  if (0 != strncmp(packet->scenario_authors, game.scenario.authors,
                    MAX_LEN_PACKET)) {
-    sz_strlcpy(game.scenario.description, packet->scenario_desc);
+    sz_strlcpy(game.scenario.authors, packet->scenario_authors);
+    changed = TRUE;
+  }
+
+  if (packet->scenario_random != game.scenario.save_random) {
+    game.scenario.save_random = packet->scenario_random;
     changed = TRUE;
   }
 
@@ -1612,12 +1436,34 @@ void handle_edit_game(struct connection *pc,
     changed = TRUE;
   }
 
+  if (packet->prevent_new_cities != game.scenario.prevent_new_cities) {
+    game.scenario.prevent_new_cities = packet->prevent_new_cities;
+    changed = TRUE;
+  }
+
+  if (packet->lake_flooding != game.scenario.lake_flooding) {
+    game.scenario.lake_flooding = packet->lake_flooding;
+    changed = TRUE;
+  }
+
   if (changed) {
     send_scenario_info(NULL);
     send_game_info(NULL);
   }
 }
 
+/****************************************************************************
+  Handle edit requests to scenario description
+****************************************************************************/
+void handle_edit_scenario_desc(struct connection *pc, const char *scenario_desc)
+{
+  if (0 != strncmp(scenario_desc, game.scenario_desc.description,
+                   MAX_LEN_PACKET)) {
+    sz_strlcpy(game.scenario_desc.description, scenario_desc);
+    send_scenario_description(NULL);
+  }
+}
+
 /****************************************************************************
   Make scenario file out of current game.
 ****************************************************************************/
@@ -1636,5 +1482,8 @@ void handle_save_scenario(struct connection *pc, const char *name)
     return;
   }
 
+  /* Client initiated scenario saving is not handmade */
+  game.scenario.handmade = FALSE;
+
   save_game(name, "Scenario", TRUE);
 }
