@@ -18,11 +18,13 @@ struct connection;
 struct conn_list;
 
 void init_new_game(void);
-void send_year_to_clients(int year);
+void send_year_to_clients(void);
 void send_game_info(struct conn_list *dest);
 
 void send_scenario_info(struct conn_list *dest);
+void send_scenario_description(struct conn_list *dest);
 
+enum unit_role_id crole_to_role_id(char crole);
 struct unit_type *crole_to_unit_type(char crole,struct player *pplayer);
 
 int update_timeout(void);
