@@ -1,4 +1,4 @@
-/********************************************************************** 
+/**********************************************************************
  Freeciv - Copyright (C) 2004 - Marcelo J. Burda
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,17 +15,24 @@
 
 typedef void (*tile_knowledge_cb)(struct tile *ptile);
 
+#define MG_UNUSED mapgen_terrain_property_invalid()
+
 void generator_free(void);
 
-void regenerate_lakes(tile_knowledge_cb knowledge_cb);
+void regenerate_lakes(void);
 void smooth_water_depth(void);
 void assign_continent_numbers(void);
 int get_lake_surrounders(Continent_id cont);
 int get_continent_size(Continent_id id);
 int get_ocean_size(Continent_id id);
 
-struct terrain *most_shallow_ocean(void);
-struct terrain *pick_ocean(int depth);
+struct terrain *most_shallow_ocean(bool frozen);
+struct terrain *pick_ocean(int depth, bool frozen);
+
+struct terrain *pick_terrain_by_flag(enum terrain_flag_id flag);
+struct terrain *pick_terrain(enum mapgen_terrain_property target,
+                             enum mapgen_terrain_property prefer,
+                             enum mapgen_terrain_property avoid);
 
 /* Provide a block to convert from native to map coordinates.  For instance
  *   do_in_map_pos(mx, my, xn, yn) {
