@@ -1,4 +1,4 @@
-/**********************************************************************
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,6 +16,7 @@
 #endif
 
 /* common */
+#include "achievements.h"
 #include "effects.h"
 #include "game.h"
 #include "government.h"
@@ -27,9 +28,26 @@
 
 /* server */
 #include "ruleset.h"
+#include "settings.h"
 
 #include "rssanity.h"
 
+/**************************************************************************
+  Is non-rule data in ruleset sane?
+**************************************************************************/
+static bool sanity_check_metadata(void)
+{
+  if (game.ruleset_summary != NULL &&
+      strlen(game.ruleset_summary) > MAX_LEN_CONTENT) {
+    log_error("Too long ruleset summary. It can be only %d bytes long. "
+              "Put longer explanations to ruleset description.",
+              MAX_LEN_CONTENT);
+    return FALSE;
+  }
+
+  return TRUE;
+}
+
 /**************************************************************************
   Does nation have tech initially?
 **************************************************************************/
@@ -79,7 +97,7 @@ static bool sanity_check_req_individual(struct requirement *preq,
                   "%s (only great wonders supported)", list_for,
                   improvement_name_translation(pimprove));
         return FALSE;
-      } else if (preq->range > REQ_RANGE_CITY && !is_wonder(pimprove)) {
+      } else if (preq->range > REQ_RANGE_TRADEROUTE && !is_wonder(pimprove)) {
         log_error("%s: %s-ranged requirement not supported for "
                   "%s (only wonders supported)", list_for,
                   req_range_name(preq->range),
@@ -117,14 +135,13 @@ static bool sanity_check_req_set(int reqs_of_type[], int local_reqs_of_type[],
     return TRUE;
   }
 
-  /* Add to counter for positive requirements 
-     For negated requirements, there are no limitations */
-  if (!preq->negated) {
+  /* Add to counter for positive requirements. */
+  if (preq->present) {
     reqs_of_type[preq->source.kind]++;
   }
   rc = reqs_of_type[preq->source.kind];
 
-  if (preq->range == REQ_RANGE_LOCAL && !preq->negated) {
+  if (preq->range == REQ_RANGE_LOCAL && preq->present) {
     local_reqs_of_type[preq->source.kind]++;
 
     switch (preq->source.kind) {
@@ -147,7 +164,7 @@ static bool sanity_check_req_set(int reqs_of_type[], int local_reqs_of_type[],
     }
   }
 
-  if (rc > 1 && !preq->negated) {
+  if (rc > 1 && preq->present) {
     /* Multiple requirements of the same type */
     switch (preq->source.kind) {
      case VUT_GOVERNMENT:
@@ -156,10 +173,14 @@ static bool sanity_check_req_set(int reqs_of_type[], int local_reqs_of_type[],
      case VUT_OTYPE:
      case VUT_SPECIALIST:
      case VUT_MINSIZE: /* Breaks nothing, but has no sense either */
+     case VUT_MINMOVES: /* Breaks nothing, but has no sense either */
+     case VUT_MINVETERAN: /* Breaks nothing, but has no sense either */
+     case VUT_MINHP: /* Breaks nothing, but has no sense either */
      case VUT_MINYEAR:
      case VUT_AI_LEVEL:
      case VUT_TERRAINALTER: /* Local range only */
      case VUT_CITYTILE:
+     case VUT_STYLE:
        /* There can be only one requirement of these types (with current
         * range limitations)
         * Requirements might be identical, but we consider multiple
@@ -190,20 +211,42 @@ static bool sanity_check_req_set(int reqs_of_type[], int local_reqs_of_type[],
        }
        break;
 
-     case VUT_SPECIAL:
-     case VUT_BASE:
-     case VUT_ROAD:
-       /* Note that there can be more than 1 special, road, or base / tile. */
+    case VUT_AGE:
+      /* There can be age of the city, unit, and player */
+      if (rc > 3) {
+        log_error("%s: Requirement list has more %s requirements than "
+                  "can ever be fulfilled.", list_for,
+                  universal_type_rule_name(&preq->source));
+        return FALSE;
+      }
+      break;
+
+     case VUT_TOPO:
+       /* Can have multiple, since it's flag based (iso & wrapx & wrapy & hex) */
+     case VUT_EXTRA:
+       /* Note that there can be more than 1 extra / tile. */
+     case VUT_MAXTILEUNITS:
+       /* Can require different numbers on e.g. local/adjacent tiles. */
      case VUT_NATION:
-       /* Can require multiple nations at World range. */
+       /* Can require multiple nations at Team/Alliance/World range. */
+     case VUT_NATIONGROUP:
+       /* Nations can be in multiple groups. */
      case VUT_NONE:
      case VUT_ADVANCE:
      case VUT_TECHFLAG:
      case VUT_IMPROVEMENT:
+     case VUT_UNITSTATE:
+       /* Can check different properties. */
      case VUT_UTFLAG:
      case VUT_UCFLAG:
      case VUT_TERRFLAG:
+     case VUT_BASEFLAG:
+     case VUT_ROADFLAG:
+     case VUT_EXTRAFLAG:
      case VUT_NATIONALITY:
+     case VUT_MINCULTURE:
+     case VUT_ACHIEVEMENT:
+     case VUT_DIPLREL:
        /* Can have multiple requirements of these types */
        break;
      case VUT_COUNT:
@@ -217,21 +260,14 @@ static bool sanity_check_req_set(int reqs_of_type[], int local_reqs_of_type[],
     }
   }
 
-  if (num_role_units(L_PARTISAN) == 0
-      && effect_cumulative_max(EFT_INSPIRE_PARTISANS, NULL) > 0) {
-    ruleset_error(LOG_ERROR,
-                  "Inspire_Partisans effect present, but no units with partisan role.");
-    return FALSE;
-  }
-
   return TRUE;
 }
 
 /**************************************************************************
-  Sanity check requirement list, including whether it's free of
+  Sanity check requirement vector, including whether it's free of
   conflicting requirements.
-  'conjunctive' should be TRUE if the list is an AND list (all requirements
-  must be active), FALSE if it's a disjunctive (OR) list.
+  'conjunctive' should be TRUE if the vector is an AND vector (all requirements
+  must be active), FALSE if it's a disjunctive (OR) vector.
   max_tiles is number of tiles that can provide requirement. Value -1
   disables checking based on number of tiles.
 
@@ -243,31 +279,6 @@ static bool sanity_check_req_set(int reqs_of_type[], int local_reqs_of_type[],
         - This function should check also problems caused by defining
           range to less than hardcoded max for requirement type
 **************************************************************************/
-static bool sanity_check_req_list(const struct requirement_list *preqs,
-                                  bool conjunctive, int max_tiles,
-                                  const char *list_for)
-{
-  int reqs_of_type[VUT_COUNT];
-  int local_reqs_of_type[VUT_COUNT];
-
-  /* Initialize requirement counters */
-  memset(reqs_of_type, 0, sizeof(reqs_of_type));
-  memset(local_reqs_of_type, 0, sizeof(local_reqs_of_type));
-
-  requirement_list_iterate(preqs, preq) {
-    if (!sanity_check_req_set(reqs_of_type, local_reqs_of_type, preq,
-                              conjunctive, max_tiles, list_for)) {
-      return FALSE;
-    }
-  } requirement_list_iterate_end;
-
-  return TRUE;
-}
-
-/**************************************************************************
-  Requirement vector version of requirement sanity checking. See
-  requirement list version for comments.
-**************************************************************************/
 static bool sanity_check_req_vec(const struct requirement_vector *preqs,
                                  bool conjunctive, int max_tiles,
                                  const char *list_for)
@@ -285,14 +296,11 @@ static bool sanity_check_req_vec(const struct requirement_vector *preqs,
       return FALSE;
     }
     requirement_vector_iterate(preqs, nreq) {
-      if (are_requirements_opposites(preq, nreq)) {
-        log_error("%s: %s ranged %s %s requirement \"%s\" "
-                  "required to be both active and negated.",
+      if (are_requirements_contradictions(preq, nreq)) {
+        log_error("%s: Requirements {%s} and {%s} contradict each other.",
                   list_for,
-                  req_range_name(preq->range),
-                  preq->survives ? "surviving" : "non surviving",
-                  universal_type_rule_name(&preq->source),
-                  universal_rule_name(&preq->source));
+                  req_to_fstring(preq),
+                  req_to_fstring(nreq));
         return FALSE;
       }
     } requirement_vector_iterate_end;
@@ -301,51 +309,16 @@ static bool sanity_check_req_vec(const struct requirement_vector *preqs,
   return TRUE;
 }
 
-/**************************************************************************
-  Check that requirement list and negated requirements list do not have
-  conflicting requirements or other problems.
-
-  Returns TRUE iff everything ok.
-**************************************************************************/
-static bool sanity_check_req_nreq_list(const struct requirement_list *preqs,
-                                       const struct requirement_list *pnreqs,
-                                       int one_tile,
-                                       const char *list_for)
-{
-  /* Check internal sanity of requirement list */
-  if (!sanity_check_req_list(preqs, TRUE, one_tile, list_for)) {
-    return FALSE;
-  }
-
-  /* There is no pnreqs in all cases */
-  if (pnreqs != NULL) {
-    sanity_check_req_list(preqs, FALSE, one_tile, list_for);
-    /* Check sanity between reqs and nreqs */
-    requirement_list_iterate(preqs, preq) {
-      requirement_list_iterate(pnreqs, pnreq) {
-        if (are_requirements_equal(preq, pnreq)) {
-          log_error("%s: Identical %s requirement in requirements and "
-                    "negated requirements.", list_for,
-                    universal_type_rule_name(&preq->source));
-          return FALSE;
-        }
-      } requirement_list_iterate_end;
-    } requirement_list_iterate_end;
-  }
-
-  return TRUE;
-}
-
 /**************************************************************************
   Sanity check callback for iterating effects cache.
 **************************************************************************/
-static bool effect_list_sanity_cb(const struct effect *peffect)
+static bool effect_list_sanity_cb(const struct effect *peffect, void *data)
 {
   int one_tile = -1; /* TODO: Determine correct value from effect.
                       *       -1 disables checking */
 
-  return sanity_check_req_nreq_list(peffect->reqs, peffect->nreqs, one_tile,
-                                    effect_type_name(peffect->type));
+  return sanity_check_req_vec(&peffect->reqs, TRUE, one_tile,
+                              effect_type_name(peffect->type));
 }
 
 /**************************************************************************
@@ -353,70 +326,69 @@ static bool effect_list_sanity_cb(const struct effect *peffect)
 **************************************************************************/
 static bool rs_barbarian_units(void)
 {
-  if (num_role_units(L_BARBARIAN) == 0
-      && BARBS_DISABLED != game.server.barbarianrate) {
-    ruleset_error(LOG_ERROR, "No role barbarian units");
-    return FALSE;
-  }
-  if (num_role_units(L_BARBARIAN_LEADER) == 0
-      && BARBS_DISABLED != game.server.barbarianrate) {
-    ruleset_error(LOG_ERROR, "No role barbarian leader units");
-    return FALSE;
-  }
-  if (num_role_units(L_BARBARIAN_BUILD) == 0
-      && BARBS_DISABLED != game.server.barbarianrate) {
-    ruleset_error(LOG_ERROR, "No role barbarian build units");
-    return FALSE;
-  }
-  if (num_role_units(L_BARBARIAN_BOAT) == 0
-      && BARBS_DISABLED != game.server.barbarianrate) {
-    ruleset_error(LOG_ERROR, "No role barbarian ship units");
-    return FALSE;
-  } else if (num_role_units(L_BARBARIAN_BOAT) > 0) {
-    struct unit_type *u;
-    enum unit_move_type mt;
-
-    u = get_role_unit(L_BARBARIAN_BOAT, 0);
-    mt = utype_move_type(u);
-    if (mt != UMT_SEA && mt != UMT_BOTH) {
-      ruleset_error(LOG_ERROR,
-                    "Barbarian boat (%s) needs to be able to move at sea.",
-                    utype_rule_name(u));
+  if (num_role_units(L_BARBARIAN) > 0) {
+    if (num_role_units(L_BARBARIAN_LEADER) == 0) {
+      ruleset_error(LOG_ERROR, "No role barbarian leader units");
       return FALSE;
     }
-  }
-  if (num_role_units(L_BARBARIAN_SEA) == 0
-      && BARBS_DISABLED != game.server.barbarianrate) {
-    ruleset_error(LOG_ERROR, "No role sea raider barbarian units");
-    return FALSE;
-  }
+    if (num_role_units(L_BARBARIAN_BUILD) == 0) {
+      ruleset_error(LOG_ERROR, "No role barbarian build units");
+      return FALSE;
+    }
+    if (num_role_units(L_BARBARIAN_BOAT) == 0) {
+      ruleset_error(LOG_ERROR, "No role barbarian ship units");
+      return FALSE;
+    } else if (num_role_units(L_BARBARIAN_BOAT) > 0) {
+      bool sea_capable = FALSE;
+      struct unit_type *u = get_role_unit(L_BARBARIAN_BOAT, 0);
 
-  unit_type_iterate(ptype) {
-    if (utype_has_role(ptype, L_BARBARIAN_BOAT)) {
-      if (ptype->transport_capacity <= 1) {
+      terrain_type_iterate(pterr) {
+        if (is_ocean(pterr)
+            && BV_ISSET(pterr->native_to, uclass_index(utype_class(u)))) {
+          sea_capable = TRUE;
+          break;
+        }
+      } terrain_type_iterate_end;
+
+      if (!sea_capable) {
         ruleset_error(LOG_ERROR,
-                      "Barbarian boat %s has no capacity for both "
-                      "leader and at least one man.",
-                      utype_rule_name(ptype));
+                      "Barbarian boat (%s) needs to be able to move at sea.",
+                      utype_rule_name(u));
         return FALSE;
       }
-
-      unit_type_iterate(pbarb) {
-        if (utype_has_role(pbarb, L_BARBARIAN_SEA)
-            || utype_has_role(pbarb, L_BARBARIAN_SEA_TECH)
-            || utype_has_role(pbarb, L_BARBARIAN_LEADER)) {
-          if (!can_unit_type_transport(ptype, utype_class(pbarb))) {
-            ruleset_error(LOG_ERROR,
-                          "Barbarian boat %s cannot transport "
-                          "barbarian cargo %s.",
-                          utype_rule_name(ptype),
-                          utype_rule_name(pbarb));
-            return FALSE;
-          }
-        }
-      } unit_type_iterate_end;
     }
-  } unit_type_iterate_end;
+    if (num_role_units(L_BARBARIAN_SEA) == 0) {
+      ruleset_error(LOG_ERROR, "No role sea raider barbarian units");
+      return FALSE;
+    }
+
+    unit_type_iterate(ptype) {
+      if (utype_has_role(ptype, L_BARBARIAN_BOAT)) {
+        if (ptype->transport_capacity <= 1) {
+          ruleset_error(LOG_ERROR,
+                        "Barbarian boat %s has no capacity for both "
+                        "leader and at least one man.",
+                        utype_rule_name(ptype));
+          return FALSE;
+        }
+
+        unit_type_iterate(pbarb) {
+          if (utype_has_role(pbarb, L_BARBARIAN_SEA)
+              || utype_has_role(pbarb, L_BARBARIAN_SEA_TECH)
+              || utype_has_role(pbarb, L_BARBARIAN_LEADER)) {
+            if (!can_unit_type_transport(ptype, utype_class(pbarb))) {
+              ruleset_error(LOG_ERROR,
+                            "Barbarian boat %s cannot transport "
+                            "barbarian cargo %s.",
+                            utype_rule_name(ptype),
+                            utype_rule_name(pbarb));
+              return FALSE;
+            }
+          }
+        } unit_type_iterate_end;
+      }
+    } unit_type_iterate_end;
+  }
 
   return TRUE;
 }
@@ -431,8 +403,8 @@ static bool rs_common_units(void)
     ruleset_error(LOG_ERROR, "No flag Settler units");
     return FALSE;
   }
-  if (num_role_units(L_EXPLORER) == 0) {
-    ruleset_error(LOG_ERROR, "No role Explorer units");
+  if (num_role_units(L_START_EXPLORER) == 0) {
+    ruleset_error(LOG_ERROR, "No role Start Explorer units");
   }
   if (num_role_units(L_FERRYBOAT) == 0) {
     ruleset_error(LOG_ERROR, "No role Ferryboat units");
@@ -441,6 +413,33 @@ static bool rs_common_units(void)
     ruleset_error(LOG_ERROR, "No role Firstbuild units");
   }
 
+  if (num_role_units(L_FERRYBOAT) > 0) {
+    bool sea_capable = FALSE;
+    struct unit_type *u = get_role_unit(L_FERRYBOAT, 0);
+
+    terrain_type_iterate(pterr) {
+      if(is_ocean(pterr)
+         && BV_ISSET(pterr->native_to, uclass_index(utype_class(u)))) {
+        sea_capable = TRUE;
+        break;
+      }
+    } terrain_type_iterate_end;
+
+    if (!sea_capable) {
+      ruleset_error(LOG_ERROR,
+                    "Ferryboat (%s) needs to be able to move at sea.",
+                    utype_rule_name(u));
+      return FALSE;
+    }
+  }
+
+  if (num_role_units(L_PARTISAN) == 0
+      && effect_cumulative_max(EFT_INSPIRE_PARTISANS, NULL) > 0) {
+    ruleset_error(LOG_ERROR,
+                  "Inspire_Partisans effect present, but no units with partisan role.");
+    return FALSE;
+  }
+
   return TRUE;
 }
 
@@ -484,7 +483,6 @@ static bool sanity_check_boolean_effects(void)
       EFT_SS_COMPONENT,
       EFT_NO_UNHAPPY,
       EFT_RAPTURE_GROW,
-      EFT_REVOLUTION_WHEN_UNHAPPY,
       EFT_HAS_SENATE,
       EFT_INSPIRE_PARTISANS,
       EFT_HAPPINESS_TO_GOLD,
@@ -497,12 +495,13 @@ static bool sanity_check_boolean_effects(void)
       EFT_IRRIG_TF_POSSIBLE,
       EFT_MINING_TF_POSSIBLE,
       EFT_NOT_TECH_SOURCE,
-      EFT_LAST
+      EFT_VICTORY,
+      EFT_COUNT
     };
   int i;
   bool ret = TRUE;
 
-  for (i = 0; boolean_effects[i] != EFT_LAST; i++) {
+  for (i = 0; boolean_effects[i] != EFT_COUNT; i++) {
     if (effect_cumulative_min(boolean_effects[i], NULL) < 0
         && effect_cumulative_max(boolean_effects[i], NULL) == 0) {
       ruleset_error(LOG_ERROR, "Boolean effect %s can get disabled, but it can't get "
@@ -528,22 +527,78 @@ bool sanity_check_ruleset_data(void)
   bool ok = TRUE; /* Store failures to variable instead of returning
                    * immediately so all errors get printed, not just first
                    * one. */
+  bool default_gov_failed = FALSE;
 
-  if (game.info.tech_cost_style == 0
+  if (!sanity_check_metadata()) {
+    ok = FALSE;
+  }
+
+  if (game.info.tech_cost_style == TECH_COST_CIV1CIV2
       && game.info.free_tech_method == FTM_CHEAPEST) {
     ruleset_error(LOG_ERROR, "Cost based free tech method, but tech cost style "
                   "1 so all techs cost the same.");
     ok = FALSE;
   }
 
+  /* Advances. */
+  advance_iterate(A_FIRST, padvance) {
+    for (i = AR_ONE; i < AR_SIZE; i++) {
+      const struct advance *preq;
+
+      if (i == AR_ROOT) {
+        /* Self rootreq is a feature. */
+        continue;
+      }
+
+      preq = advance_requires(padvance, i);
+
+      if (A_NEVER == preq) {
+        continue;
+      } else if (preq == padvance) {
+        ruleset_error(LOG_ERROR, "Tech \"%s\" requires itself.",
+                      advance_rule_name(padvance));
+        ok = FALSE;
+        continue;
+      }
+
+      advance_req_iterate(preq, preqreq) {
+        if (preqreq == padvance) {
+          ruleset_error(LOG_ERROR,
+                        "Tech \"%s\" requires itself indirectly via \"%s\".",
+                        advance_rule_name(padvance),
+                        advance_rule_name(preq));
+          ok = FALSE;
+        }
+      } advance_req_iterate_end;
+    }
+
+    if (padvance->bonus_message != NULL) {
+      if (!formats_match(padvance->bonus_message, "%s")) {
+        ruleset_error(LOG_ERROR,
+                      "Tech \"%s\" bonus message is not format with %%s for a bonus tech name.",
+                      advance_rule_name(padvance));
+        ok = FALSE;
+      }
+    }
+  } advance_iterate_end;
+
+  if (game.default_government == game.government_during_revolution) {
+    ruleset_error(LOG_ERROR,
+                  "The government form %s reserved for revolution handling has been set as "
+                  "default_government.",
+                  government_rule_name(game.government_during_revolution));
+    ok = FALSE;
+    default_gov_failed = TRUE;
+  }
+
   /* Check that all players can have their initial techs */
   nations_iterate(pnation) {
-    int i;
+    int techi;
 
     /* Check global initial techs */
-    for (i = 0; i < MAX_NUM_TECH_LIST
-         && game.rgame.global_init_techs[i] != A_LAST; i++) {
-      Tech_type_id tech = game.rgame.global_init_techs[i];
+    for (techi = 0; techi < MAX_NUM_TECH_LIST
+         && game.rgame.global_init_techs[techi] != A_LAST; techi++) {
+      Tech_type_id tech = game.rgame.global_init_techs[techi];
       struct advance *a = valid_advance_by_number(tech);
 
       if (a == NULL) {
@@ -565,10 +620,10 @@ bool sanity_check_ruleset_data(void)
     }
 
     /* Check national initial techs */
-    for (i = 0;
-         i < MAX_NUM_TECH_LIST && pnation->init_techs[i] != A_LAST;
-         i++) {
-      Tech_type_id tech = pnation->init_techs[i];
+    for (techi = 0;
+         techi < MAX_NUM_TECH_LIST && pnation->init_techs[techi] != A_LAST;
+         techi++) {
+      Tech_type_id tech = pnation->init_techs[techi];
       struct advance *a = valid_advance_by_number(tech);
 
       if (a == NULL) {
@@ -597,7 +652,7 @@ bool sanity_check_ruleset_data(void)
                     "never see them", nation_rule_name(pnation));
     }
 
-    if (pnation->init_government == game.government_during_revolution) {
+    if (!default_gov_failed && pnation->init_government == game.government_during_revolution) {
       ruleset_error(LOG_ERROR,
                     "The government form %s reserved for revolution handling has been set as "
                     "initial government for %s.",
@@ -625,7 +680,8 @@ bool sanity_check_ruleset_data(void)
     }
   } unit_type_iterate_end;
 
-  /* Some unit type flags depend on other flags to work properly. */
+  /* Some unit type properties depend on other unit type properties to work
+   * properly. */
   unit_type_iterate(putype) {
     /* "Spy" is a better "Diplomat". Until all the places that assume that
      * "Diplomat" is set if "Spy" is set is changed this limitation must be
@@ -638,6 +694,16 @@ bool sanity_check_ruleset_data(void)
                     utype_rule_name(putype));
       ok = FALSE;
     }
+
+    /* The ability to bombard requires a bombard_rate. */
+    if (utype_has_flag(putype, UTYF_BOMBARDER)
+        && putype->bombard_rate <= 0) {
+      ruleset_error(LOG_ERROR,
+                    "The unit type '%s' has the 'Bombarder' unit type flag"
+                    " but no bombard_rate is set.",
+                    utype_rule_name(putype));
+      ok = FALSE;
+    }
   } unit_type_iterate_end;
 
   /* Check that unit type fields are in range. */
@@ -650,12 +716,13 @@ bool sanity_check_ruleset_data(void)
                     "That is out of range. Max range is %d.",
                     utype_rule_name(putype),
                     putype->paratroopers_range, UNIT_MAX_PARADROP_RANGE);
+      ok = FALSE;
     }
   } unit_type_iterate_end;
 
   /* Check requirement sets against conflicting requirements.
    * Effects use requirement lists */
-  if (!iterate_effect_cache(effect_list_sanity_cb)) {
+  if (!iterate_effect_cache(effect_list_sanity_cb, NULL)) {
     ruleset_error(LOG_ERROR,
                   "Effects have conflicting or invalid requirements!");
     ok = FALSE;
@@ -685,6 +752,12 @@ bool sanity_check_ruleset_data(void)
                     "Buildings have conflicting or invalid requirements!");
       ok = FALSE;
     }
+    if (!sanity_check_req_vec(&pimprove->obsolete_by, FALSE, -1,
+                              improvement_rule_name(pimprove))) {
+      ruleset_error(LOG_ERROR,
+                    "Buildings have conflicting or invalid obsolescence req!");
+      ok = FALSE;
+    }
   } improvement_iterate_end;
 
   /* Governments */
@@ -707,35 +780,42 @@ bool sanity_check_ruleset_data(void)
                     "Specialists have conflicting or invalid requirements!");
       ok = FALSE;
     }
-
-    /* Specialist rulenames are used as an entry name in savegames. */
-    if (!is_secfile_entry_name_valid(specialist_rule_name(psp))) {
-      ruleset_error(LOG_ERROR,
-                    "Specialist rule name \"%s\" unusable in savegames!",
-                    specialist_rule_name(psp));
-      ok = FALSE;
-    }
   } specialist_type_iterate_end;
 
-  /* Bases */
-  base_type_iterate(pbase) {
-    if (!sanity_check_req_vec(&pbase->reqs, TRUE, -1,
-                              base_rule_name(pbase))) {
+  /* Extras */
+  extra_type_iterate(pextra) {
+    if (!sanity_check_req_vec(&pextra->reqs, TRUE, -1,
+                              extra_rule_name(pextra))) {
       ruleset_error(LOG_ERROR,
-                    "Bases have conflicting or invalid requirements!");
+                    "Extras have conflicting or invalid requirements!");
       ok = FALSE;
     }
-  } base_type_iterate_end;
+    if (!sanity_check_req_vec(&pextra->rmreqs, TRUE, -1,
+                              extra_rule_name(pextra))) {
+      ruleset_error(LOG_ERROR,
+                    "Extras have conflicting or invalid removal requirements!");
+      ok = FALSE;
+    }
+  } extra_type_iterate_end;
 
   /* Roads */
-  road_type_iterate(proad) {
-    if (!sanity_check_req_vec(&proad->reqs, TRUE, -1,
-                              road_rule_name(proad))) {
-      ruleset_error(LOG_ERROR,
-                    "Roads have conflicting or invalid requirements!");
-      ok = FALSE;
-    }
-  } road_type_iterate_end;
+  extra_type_by_cause_iterate(EC_ROAD, pextra) {
+    struct road_type *proad = extra_road_get(pextra);
+
+    extra_type_list_iterate(proad->integrators, iextra) {
+      struct road_type *iroad = extra_road_get(iextra);
+
+      if (road_index(proad) != road_index(iroad)
+          && !BV_ISSET(iroad->integrates, road_index(proad))) {
+        /* We don't support non-symmetric integrator relationships yet. */
+        ruleset_error(LOG_ERROR,
+                      "Road '%s' integrates with '%s' but not vice versa!",
+                      extra_rule_name(pextra),
+                      extra_rule_name(iextra));
+        ok = FALSE;
+      }
+    } extra_type_list_iterate_end;
+  } extra_type_by_cause_iterate_end;
 
   /* City styles */
   for (i = 0; i < game.control.styles_count; i++) {
@@ -747,22 +827,66 @@ bool sanity_check_ruleset_data(void)
     }
   }
 
-  terrain_type_iterate(pterr) {
-    unit_class_iterate(uc) {
-      if (BV_ISSET(pterr->native_to, uclass_index(uc))) {
-        if (is_ocean(pterr) && uc->move_type == UMT_LAND) {
+  /* Action enablers */
+  action_iterate(act) {
+    action_enabler_list_iterate(action_enablers_for_action(act), enabler) {
+      if (!sanity_check_req_vec(&(enabler->actor_reqs), TRUE, -1,
+                                "Action Enabler Actor Reqs")
+          || !sanity_check_req_vec(&(enabler->target_reqs), TRUE, -1,
+                                   "Action Enabler Target Reqs")) {
+        ruleset_error(LOG_ERROR,
+                      "Action enabler for %s has conflicting or invalid "
+                      "requirements!", action_id_rule_name(act));
+        ok = FALSE;
+      }
+
+      requirement_vector_iterate(&(enabler->target_reqs), preq) {
+        if (preq->source.kind == VUT_DIPLREL
+            && preq->range == REQ_RANGE_LOCAL) {
+          /* A Local DiplRel requirement can be expressed as a requirement
+           * in actor_reqs. Demand that it is there. This avoids breaking
+           * code that reasons about actions. */
           ruleset_error(LOG_ERROR,
-                        "Oceanic %s is native to land units.",
-                        terrain_rule_name(pterr));
-          ok = FALSE;
-        } else if (!is_ocean(pterr) && uc->move_type == UMT_SEA) {
-          ruleset_error(LOG_ERROR,
-                        "Non-oceanic %s is native to sea units.",
-                        terrain_rule_name(pterr));
+                        "Action enabler for %s has a local DiplRel "
+                        "requirement %s in target_reqs! Please read the "
+                        "section \"Requirement vector rules\" in "
+                        "doc/README.actions",
+                        action_id_rule_name(act),
+                        req_to_fstring(preq));
           ok = FALSE;
         }
+      } requirement_vector_iterate_end;
+    } action_enabler_list_iterate_end;
+  } action_iterate_end;
+
+  /* There must be basic city style for each nation style to start with */
+  styles_iterate(pstyle) {
+    if (basic_city_style_for_style(pstyle) < 0) {
+      ruleset_error(LOG_ERROR,
+                    "There's no basic city style for nation style %s",
+                    style_rule_name(pstyle));
+      ok = FALSE;
+    }
+  } styles_iterate_end;
+
+  /* Music styles */
+  music_styles_iterate(pmus) {
+    if (!sanity_check_req_vec(&pmus->reqs, TRUE, -1, "Music Style")) {
+      ruleset_error(LOG_ERROR,
+                    "Music Styles have conflicting or invalid requirements!");
+      ok = FALSE;
+    }
+  } music_styles_iterate_end;
+
+  terrain_type_iterate(pterr) {
+    if (pterr->animal != NULL) {
+      if (!is_native_to_class(utype_class(pterr->animal), pterr, NULL)) {
+        ruleset_error(LOG_ERROR,
+                      "%s has %s as animal to appear, but it's not native to the terrain.",
+                      terrain_rule_name(pterr), utype_rule_name(pterr->animal));
+        ok = FALSE;
       }
-    } unit_class_iterate_end;
+    }
   } terrain_type_iterate_end;
 
   /* Check that all unit classes can exist somewhere */
@@ -778,23 +902,13 @@ bool sanity_check_ruleset_data(void)
       } terrain_type_iterate_end;
 
       if (!can_exist) {
-        base_type_iterate(pbase) {
-          if (BV_ISSET(pbase->native_to, uclass_index(pclass))
-              && base_has_flag(pbase, BF_NATIVE_TILE)) {
+        extra_type_iterate(pextra) {
+          if (BV_ISSET(pextra->native_to, uclass_index(pclass))
+              && extra_has_flag(pextra, EF_NATIVE_TILE)) {
             can_exist = TRUE;
             break;
           }
-        } base_type_iterate_end;
-      }
-
-      if (!can_exist) {
-        road_type_iterate(proad) {
-          if (BV_ISSET(proad->native_to, uclass_index(pclass))
-             && road_has_flag(proad, RF_NATIVE_TILE)) {
-            can_exist = TRUE;
-            break;
-          }
-        } road_type_iterate_end;
+        } extra_type_iterate_end;
       }
 
       if (!can_exist) {
@@ -806,6 +920,33 @@ bool sanity_check_ruleset_data(void)
     }
   } unit_class_iterate_end;
 
+  achievements_iterate(pach) {
+    if (!pach->unique && pach->cons_msg == NULL) {
+      ruleset_error(LOG_ERROR,
+                    "Achievement %s has no message for consecutive gainers though "
+                    "it's possible to be gained by multiple players",
+                    achievement_rule_name(pach));
+      ok = FALSE;
+    }
+  } achievements_iterate_end;
+
+  if (game.server.ruledit.embedded_nations != NULL) {
+    int nati;
+
+    for (nati = 0; nati < game.server.ruledit.embedded_nations_count; nati++) {
+      struct nation_type *pnat
+        = nation_by_rule_name(game.server.ruledit.embedded_nations[nati]);
+
+      if (pnat == NULL) {
+        ruleset_error(LOG_ERROR,
+                      "There's nation %s listed in embedded nations, but there's "
+                      "no such nation.",
+                      game.server.ruledit.embedded_nations[nati]);
+        ok = FALSE;
+      }
+    }
+  }
+
   if (ok) {
     ok = rs_common_units();
   }
@@ -818,3 +959,25 @@ bool sanity_check_ruleset_data(void)
 
   return ok;
 }
+
+/**************************************************************************
+  Set and lock settings that must have certain value.
+**************************************************************************/
+bool autolock_settings(void)
+{
+  bool ok = TRUE;
+
+  if (num_role_units(L_BARBARIAN) == 0) {
+    struct setting *pset = setting_by_name("barbarians");
+
+    log_normal(_("Disabling 'barbarians' setting for lack of suitable "
+                 "unit types."));
+    setting_lock_set(pset, FALSE);
+    if (!setting_enum_set(pset, "DISABLED", NULL, NULL, 0)) {
+      ok = FALSE;
+    }
+    setting_lock_set(pset, TRUE);
+  }
+
+  return ok;
+}
