@@ -25,6 +25,7 @@
 #include "shared.h"
 
 /* common */
+#include "culture.h"
 #include "game.h"
 #include "improvement.h"
 #include "map.h"
@@ -248,7 +249,8 @@ static void get_player_landarea(struct claim_map *pcmap,
 **************************************************************************/
 void calc_civ_score(struct player *pplayer)
 {
-  struct city *pcity;
+  const struct research *presearch;
+  struct city *wonder_city;
   int landarea = 0, settledarea = 0;
   static struct claim_map cmap;
 
@@ -272,6 +274,7 @@ void calc_civ_score(struct player *pplayer)
   pplayer->score.mfg = 0;
   pplayer->score.literacy = 0;
   pplayer->score.spaceship = 0;
+  pplayer->score.culture = player_culture(pplayer);
 
   if (is_barbarian(pplayer)) {
     return;
@@ -305,12 +308,13 @@ void calc_civ_score(struct player *pplayer)
   pplayer->score.landarea = landarea;
   pplayer->score.settledarea = settledarea;
 
+  presearch = research_get(pplayer);
   advance_index_iterate(A_FIRST, i) {
-    if (player_invention_state(pplayer, i) == TECH_KNOWN) {
+    if (research_invention_state(presearch, i) == TECH_KNOWN) {
       pplayer->score.techs++;
     }
   } advance_index_iterate_end;
-  pplayer->score.techs += player_research_get(pplayer)->future_tech * 5 / 2;
+  pplayer->score.techs += research_get(pplayer)->future_tech * 5 / 2;
   
   unit_list_iterate(pplayer->units, punit) {
     if (is_military_unit(punit)) {
@@ -320,8 +324,8 @@ void calc_civ_score(struct player *pplayer)
 
   improvement_iterate(i) {
     if (is_great_wonder(i)
-        && (pcity = city_from_great_wonder(i))
-        && player_owns_city(pplayer, pcity)) {
+        && (wonder_city = city_from_great_wonder(i))
+        && player_owns_city(pplayer, wonder_city)) {
       pplayer->score.wonders++;
     }
   } improvement_iterate_end;
@@ -351,7 +355,8 @@ int get_civ_score(const struct player *pplayer)
           + pplayer->score.techs * 2
           + pplayer->score.wonders * 5
           + get_spaceship_score(pplayer)
-          + get_units_score(pplayer));
+          + get_units_score(pplayer)
+          + pplayer->score.culture / 50);
 }
 
 /**************************************************************************
@@ -362,8 +367,8 @@ static int get_spaceship_score(const struct player *pplayer)
   if (pplayer->score.spaceship == SSHIP_ARRIVED) {
     /* How much should a spaceship be worth?
      * This gives 100 points per 10,000 citizens. */
-    return (int)(100 * pplayer->spaceship.habitation
-		 * pplayer->spaceship.success_rate);
+    return (int)((pplayer->spaceship.population
+		  * pplayer->spaceship.success_rate) / 100.0);
   } else {
     return 0;
   }
