@@ -19,6 +19,7 @@
 #include "rand.h"
 
 /* common */
+#include "movement.h"
 #include "research.h"
 #include "unittype.h"
 
@@ -32,7 +33,7 @@
 #include "citytools.h"
 #include "console.h" /* enum rfc_status */
 #include "maphand.h"
-#include "movement.h"
+#include "notify.h"
 #include "plrhand.h"
 #include "srv_main.h" /* game_was_started() */
 #include "stdinhand.h"
@@ -119,7 +120,7 @@ Unit *api_edit_create_unit_full(lua_State *L, Player *pplayer, Tile *ptile,
     if (!ret) {
       luascript_log(fcl, LOG_ERROR, "create_unit_full: '%s' cannot transport "
                                     "'%s' here",
-                    utype_rule_name(unit_type(ptransport)),
+                    utype_rule_name(unit_type_get(ptransport)),
                     utype_rule_name(ptype));
       return NULL;
     }
@@ -159,7 +160,7 @@ bool api_edit_unit_teleport(lua_State *L, Unit *punit, Tile *dest)
   LUASCRIPT_CHECK_ARG_NIL(L, dest, 3, Tile, FALSE);
 
   /* Teleport first so destination is revealed even if unit dies */
-  alive = unit_move(punit, dest, 0);
+  alive = unit_move(punit, dest, 0, NULL);
   if (alive) {
     struct player *owner = unit_owner(punit);
     struct city *pcity = tile_city(dest);
@@ -195,6 +196,26 @@ void api_edit_unit_turn(lua_State *L, Unit *punit, Direction dir)
   }
 }
 
+/*****************************************************************************
+  Kill the unit.
+*****************************************************************************/
+void api_edit_unit_kill(lua_State *L, Unit *punit, const char *reason,
+                        Player *killer)
+{
+  enum unit_loss_reason loss_reason;
+
+  LUASCRIPT_CHECK_STATE(L);
+  LUASCRIPT_CHECK_ARG_NIL(L, punit, 2, Unit);
+  LUASCRIPT_CHECK_ARG_NIL(L, reason, 3, string);
+
+  loss_reason = unit_loss_reason_by_name(reason, fc_strcasecmp);
+
+  LUASCRIPT_CHECK_ARG(L, unit_loss_reason_is_valid(loss_reason), 3,
+                      "Invalid unit loss reason");
+
+  wipe_unit(punit, loss_reason, killer);
+}
+
 /*****************************************************************************
   Create a new city.
 *****************************************************************************/
@@ -266,36 +287,64 @@ void api_edit_change_gold(lua_State *L, Player *pplayer, int amount)
   sends script signal "tech_researched" with the given reason
 *****************************************************************************/
 Tech_Type *api_edit_give_technology(lua_State *L, Player *pplayer,
-                                    Tech_Type *ptech, const char *reason)
+                                    Tech_Type *ptech, int cost, bool notify,
+                                    const char *reason)
 {
-  struct player_research *presearch;
+  struct research *presearch;
   Tech_type_id id;
   Tech_Type *result;
 
   LUASCRIPT_CHECK_STATE(L, NULL);
   LUASCRIPT_CHECK_ARG_NIL(L, pplayer, 2, Player, NULL);
+  LUASCRIPT_CHECK_ARG(L, cost >= -3, 4, "Unknown give_tech() cost value", NULL);
 
-  presearch = player_research_get(pplayer);
+  presearch = research_get(pplayer);
   if (ptech) {
     id = advance_number(ptech);
   } else {
-    /* Can't just call give_immediate_free_tech() here as we want
-     * to pass correct reason to emitted signal. */
-    if (game.info.free_tech_method == FTM_CHEAPEST) {
-      id = pick_cheapest_tech(pplayer);
-    } else if (presearch->researching == A_UNSET
-               || game.info.free_tech_method == FTM_RANDOM) {
-      id = pick_random_tech(pplayer);
-    } else {
-      id = presearch->researching;
-    }
+    id = pick_free_tech(presearch);
   }
 
-  if (id == A_FUTURE || player_invention_state(pplayer, id) != TECH_KNOWN) {
-    do_free_cost(pplayer, id);
-    found_new_tech(pplayer, id, FALSE, TRUE);
+  if (is_future_tech(id)
+      || research_invention_state(presearch, id) != TECH_KNOWN) {
+    if (cost < 0) {
+      if (cost == -1) {
+        cost = game.server.freecost;
+      } else if (cost == -2) {
+        cost = game.server.conquercost;
+      } else if (cost == -3) {
+        cost = game.server.diplbulbcost;
+      } else {
+        
+        cost = 0;
+      }
+    }
+    research_apply_penalty(presearch, id, cost);
+    found_new_tech(presearch, id, FALSE, TRUE);
     result = advance_by_number(id);
-    script_tech_learned(pplayer, result, reason);
+    script_tech_learned(presearch, pplayer, result, reason);
+
+    if (notify && result != NULL) {
+      const char *adv_name = research_advance_name_translation(presearch, id);
+      char research_name[MAX_LEN_NAME * 2];
+
+      research_pretty_name(presearch, research_name, sizeof(research_name));
+
+      notify_player(pplayer, NULL, E_TECH_GAIN, ftc_server,
+                    Q_("?fromscript:You acquire %s."), adv_name);
+      notify_research(presearch, pplayer, E_TECH_GAIN, ftc_server,
+                      /* TRANS: "The Greeks ..." or "The members of
+                       * team Red ..." */
+                      Q_("?fromscript:The %s acquire %s and share this "
+                         "advance with you."),
+                      nation_plural_for_player(pplayer), adv_name);
+      notify_research_embassies(presearch, NULL, E_TECH_EMBASSY, ftc_server,
+                                /* TRANS: "The Greeks ..." or "The members of
+                                 * team Red ..." */
+                                Q_("?fromscript:The %s acquire %s."),
+                                research_name, adv_name);
+    }
+
     return result;
   } else {
     return NULL;
@@ -305,14 +354,18 @@ Tech_Type *api_edit_give_technology(lua_State *L, Player *pplayer,
 /*****************************************************************************
   Modify player's trait value.
 *****************************************************************************/
-bool api_edit_trait_mod(lua_State *L, Player *pplayer, const char *trait_name,
-                        const int mod)
+bool api_edit_trait_mod_set(lua_State *L, Player *pplayer,
+                            const char *tname, const int mod)
 {
-  enum trait tr = trait_by_name(trait_name, fc_strcasecmp);
+  enum trait tr;
 
-  if (!trait_is_valid(tr)) {
-    return FALSE;
-  }
+  LUASCRIPT_CHECK_STATE(L, -1);
+  LUASCRIPT_CHECK_ARG_NIL(L, pplayer, 2, Player, FALSE);
+  LUASCRIPT_CHECK_ARG_NIL(L, tname, 3, string, FALSE);
+
+  tr = trait_by_name(tname, fc_strcasecmp);
+
+  LUASCRIPT_CHECK_ARG(L, trait_is_valid(tr), 3, "no such trait", 0);
 
   pplayer->ai_common.traits[tr].mod += mod;
 
@@ -320,12 +373,11 @@ bool api_edit_trait_mod(lua_State *L, Player *pplayer, const char *trait_name,
 }
 
 /*****************************************************************************
-  Create a new base.
+  Create a new extra.
 *****************************************************************************/
-void api_edit_create_base(lua_State *L, Tile *ptile, const char *name,
-                          Player *pplayer)
+void api_edit_create_extra(lua_State *L, Tile *ptile, const char *name)
 {
-  struct base_type *pbase;
+  struct extra_type *pextra;
 
   LUASCRIPT_CHECK_STATE(L);
   LUASCRIPT_CHECK_ARG_NIL(L, ptile, 2, Tile);
@@ -334,10 +386,33 @@ void api_edit_create_base(lua_State *L, Tile *ptile, const char *name,
     return;
   }
 
-  pbase = base_type_by_rule_name(name);
+  pextra = extra_type_by_rule_name(name);
 
-  if (pbase) {
-    create_base(ptile, pbase, pplayer);
+  if (pextra) {
+    create_extra(ptile, pextra, NULL);
+    update_tile_knowledge(ptile);
+  }
+}
+
+/*****************************************************************************
+  Create a new base.
+*****************************************************************************/
+void api_edit_create_base(lua_State *L, Tile *ptile, const char *name,
+                          Player *pplayer)
+{
+  struct extra_type *pextra;
+
+  LUASCRIPT_CHECK_STATE(L);
+  LUASCRIPT_CHECK_ARG_NIL(L, ptile, 2, Tile);
+
+  if (!name) {
+    return;
+  }
+
+  pextra = extra_type_by_rule_name(name);
+
+  if (pextra != NULL && is_extra_caused_by(pextra, EC_BASE)) {
+    create_extra(ptile, pextra, pplayer);
     update_tile_knowledge(ptile);
   }
 }
@@ -347,7 +422,15 @@ void api_edit_create_base(lua_State *L, Tile *ptile, const char *name,
 *****************************************************************************/
 void api_edit_create_road(lua_State *L, Tile *ptile, const char *name)
 {
-  struct road_type *proad;
+  api_edit_create_extra(L, ptile, name);
+}
+
+/*****************************************************************************
+  Remove extra from tile, if present
+*****************************************************************************/
+void api_edit_remove_extra(lua_State *L, Tile *ptile, const char *name)
+{
+  struct extra_type *pextra;
 
   LUASCRIPT_CHECK_STATE(L);
   LUASCRIPT_CHECK_ARG_NIL(L, ptile, 2, Tile);
@@ -356,10 +439,10 @@ void api_edit_create_road(lua_State *L, Tile *ptile, const char *name)
     return;
   }
 
-  proad = road_type_by_rule_name(name);
+  pextra = extra_type_by_rule_name(name);
 
-  if (proad) {
-    tile_add_road(ptile, proad);
+  if (pextra != NULL && tile_has_extra(ptile, pextra)) {
+    tile_extra_rm_apply(ptile, pextra);
     update_tile_knowledge(ptile);
   }
 }
@@ -445,5 +528,27 @@ bool api_edit_unit_move(lua_State *L, Unit *punit, Tile *ptile,
   LUASCRIPT_CHECK_ARG_NIL(L, ptile, 3, Tile, FALSE);
   LUASCRIPT_CHECK_ARG(L, movecost >= 0, 4, "Negative move cost!", FALSE);
 
-  return unit_move(punit, ptile, movecost);
+  return unit_move(punit, ptile, movecost, NULL);
+}
+
+/*****************************************************************************
+  Add history to a city
+*****************************************************************************/
+void api_edit_city_add_history(lua_State *L, City *pcity, int amount)
+{
+  LUASCRIPT_CHECK_STATE(L);
+  LUASCRIPT_CHECK_SELF(L, pcity);
+
+  pcity->history += amount;
+}
+
+/*****************************************************************************
+  Add history to a player
+*****************************************************************************/
+void api_edit_player_add_history(lua_State *L, Player *pplayer, int amount)
+{
+  LUASCRIPT_CHECK_STATE(L);
+  LUASCRIPT_CHECK_SELF(L, pplayer);
+
+  pplayer->culture += amount;
 }
