@@ -32,7 +32,6 @@ void cmd_reply(enum command_id cmd, struct connection *caller,
 
 bool handle_stdin_input(struct connection *caller, char *str);
 void set_ai_level_direct(struct player *pplayer, enum ai_level level);
-void set_ai_level_directer(struct player *pplayer, enum ai_level level);
 bool read_init_script(struct connection *caller, char *script_filename,
                       bool from_cmdline, bool check);
 void show_players(struct connection *caller);
@@ -65,7 +64,7 @@ bool conn_is_kicked(struct connection *pconn, int *time_remaining);
 
 void set_running_game_access_level(void);
 
-#ifdef HAVE_LIBREADLINE
+#ifdef FREECIV_HAVE_LIBREADLINE
 char **freeciv_completion(const char *text, int start, int end);
 #endif
 
