@@ -1,4 +1,4 @@
-/**********************************************************************
+/***********************************************************************
  Freeciv - Copyright (C) 2004 - The Freeciv Project
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,12 +27,12 @@
 
 /* utility */
 #include "capability.h"
+#include "fc_cmdline.h"
 #include "fciconv.h"
 #include "fcintl.h"
 #include "log.h"
 #include "mem.h"
 #include "registry.h"
-#include "shared.h"
 #include "support.h"
 
 /* common */
@@ -53,6 +53,7 @@
 #include "climisc.h"
 #include "helpdata.h"
 #include "helpdlg_g.h"
+#include "tilespec.h"
 
 /* server */
 #include "citytools.h"
@@ -80,7 +81,8 @@ enum manuals {
 };
 
 /* This formats the manual for an HTML wiki. */
-#ifdef USE_HTML
+#ifdef MANUAL_USE_HTML
+#define FILE_EXT "html"
 #define HEADER "<html><head><link rel=\"stylesheet\" type=\"text/css\" "\
                "href=\"manual.css\"/><meta http-equiv=\"Content-Type\" "\
                "content=\"text/html; charset=UTF-8\"/></head><body>\n\n"
@@ -90,7 +92,8 @@ enum manuals {
 #define IMAGE_END ".png\">"
 #define SEPARATOR " "
 #define TAIL "</body></html>"
-#else
+#else  /* MANUAL_USE_HTML */
+#define FILE_EXT "mediawiki"
 #define HEADER " "
 #define SECTION_BEGIN "==="
 #define SECTION_END "==="
@@ -98,7 +101,9 @@ enum manuals {
 #define IMAGE_END ".png]]"
 #define SEPARATOR "----\n\n"
 #define TAIL " "
-#endif
+#endif /* MANUAL_USE_HTML */
+
+void insert_client_build_info(char *outbuf, size_t outlen);
 
 /* Needed for "About Freeciv" help */
 const char *client_string = "freeciv-manual";
@@ -119,19 +124,61 @@ static char *html_special_chars(char *str, size_t *len)
   return buf;
 }
 
-/**************************************************************************
+
+/*******************************************
   Useless stubs for compiling client code.
+*/
+
+/**************************************************************************
+  Client stub
 **************************************************************************/
 void popup_help_dialog_string(const char *item)
 {
   /* Empty stub. */
 }
 
+/**************************************************************************
+  Client stub
+**************************************************************************/
 void popdown_help_dialog(void)
 {
   /* Empty stub. */
 }
 
+struct tileset *tileset;
+
+/**************************************************************************
+  Client stub
+**************************************************************************/
+const char *tileset_name_get(struct tileset *t)
+{
+  return NULL;
+}
+
+/**************************************************************************
+  Client stub
+**************************************************************************/
+const char *tileset_version(struct tileset *t)
+{
+  return NULL;
+}
+
+/**************************************************************************
+  Client stub
+**************************************************************************/
+const char *tileset_summary(struct tileset *t)
+{
+  return NULL;
+}
+
+/**************************************************************************
+  Client stub
+**************************************************************************/
+const char *tileset_description(struct tileset *t)
+{
+  return NULL;
+}
+
 enum client_states client_state(void)
 {
   return C_S_INITIAL;
@@ -145,7 +192,7 @@ bool client_nation_is_in_current_set(const struct nation_type *pnation)
 }
 
 /**************************************************************************
-  Write a server manual in html format, then quit.
+  Write a server manual in the format chosen at build time, then quit.
 **************************************************************************/
 static bool manual_command(void)
 {
@@ -161,7 +208,7 @@ static bool manual_command(void)
   /* Reset aifill to zero */
   game.info.aifill = 0;
 
-  if (!load_rulesets(NULL, FALSE)) {
+  if (!load_rulesets(NULL, FALSE, FALSE)) {
     /* Failed to load correct ruleset */
     return FALSE;
   }
@@ -170,7 +217,8 @@ static bool manual_command(void)
     int i;
     int ri;
 
-    fc_snprintf(filename, sizeof(filename), "manual%d.html", manuals + 1);
+    fc_snprintf(filename, sizeof(filename), "%s%d.%s",
+                game.server.rulesetdir, manuals + 1, FILE_EXT);
 
     if (!is_reg_file_for_access(filename, TRUE)
         || !(doc = fc_fopen(filename, "w"))) {
@@ -257,7 +305,7 @@ static bool manual_command(void)
           fprintf(doc, "<p class=\"bounds\">%s %s</p>\n\n",
                   _("Default:"), buf);
         }
-        if (setting_changed(pset)) {
+        if (setting_non_default(pset)) {
           fprintf(doc, _("<p class=\"changed\">Value set to %s</p>\n\n"),
                   setting_value_name(pset, TRUE, buf, sizeof(buf)));
         }
@@ -316,14 +364,14 @@ static bool manual_command(void)
       if (game.control.num_road_types > 0) {
         fprintf(doc, "<th>");
       }
-      road_type_iterate(proad) {
+      extra_type_by_cause_iterate(EC_ROAD, pextra) {
         if (++ri < game.control.num_road_types) {
-          fprintf(doc, "%s<br/>", road_name_translation(proad));
+          fprintf(doc, "%s<br/>", extra_name_translation(pextra));
         } else {
           /* Last one */
-          fprintf(doc, "%s</th>", road_name_translation(proad));
+          fprintf(doc, "%s</th>", extra_name_translation(pextra));
         }
-      } road_type_iterate_end;
+      } extra_type_by_cause_iterate_end;
       fprintf(doc, "</tr>\n\n");
       terrain_type_iterate(pterrain) {
         struct resource **r;
@@ -395,15 +443,15 @@ static bool manual_command(void)
         if (game.control.num_road_types > 0) {
           fprintf(doc, "<td>");
         }
-        road_type_iterate(proad) {
+        extra_type_by_cause_iterate(EC_ROAD, pextra) {
           if (++ri < game.control.num_road_types) {
-            fprintf(doc, "%d / ", terrain_road_time(pterrain,
-                                                   road_number(proad)));
+            fprintf(doc, "%d / ", terrain_extra_build_time(pterrain, ACTIVITY_GEN_ROAD,
+                                                           pextra));
           } else {
-            fprintf(doc, "%d</td>", terrain_road_time(pterrain,
-                                                      road_number(proad)));
+            fprintf(doc, "%d</td>", terrain_extra_build_time(pterrain, ACTIVITY_GEN_ROAD,
+                                                             pextra));
           }
-        } road_type_iterate_end;
+        } extra_type_by_cause_iterate_end;
         fprintf(doc, "</tr>\n\n");
       } terrain_type_iterate_end;
 
@@ -426,6 +474,7 @@ static bool manual_command(void)
 
       improvement_iterate(pimprove) {
         char buf[64000];
+        struct advance *obs_tech = NULL;
 
         if (!valid_improvement(pimprove)
          || is_great_wonder(pimprove) == (manuals == MANUAL_BUILDINGS)) {
@@ -443,10 +492,9 @@ static bool manual_command(void)
 
         requirement_vector_iterate(&pimprove->reqs, req) {
           char text[512], text2[512];
-
           fc_snprintf(text2, sizeof(text2),
                       /* TRANS: improvement requires a feature to be absent. */
-                      req->negated ? _("no %s") : "%s",
+                      req->present ? "%s" : _("no %s"),
                       VUT_NONE != req->source.kind
                       ? universal_name_translation(&req->source,
                                                    text, sizeof(text))
@@ -454,9 +502,16 @@ static bool manual_command(void)
           fprintf(doc, "%s<br/>", text2);
         } requirement_vector_iterate_end;
 
+        requirement_vector_iterate(&pimprove->obsolete_by, pobs) {
+          if (pobs->source.kind == VUT_ADVANCE) {
+            obs_tech = pobs->source.value.advance;
+            break;
+          }
+        } requirement_vector_iterate_end;
+
         fprintf(doc, "<em>%s</em></td>\n",
-                valid_advance(pimprove->obsolete_by)
-                ? advance_name_translation(pimprove->obsolete_by)
+                obs_tech != NULL
+                ? advance_name_translation(obs_tech)
                 : Q_("?tech:None"));
         fprintf(doc, "<td>%s</td>\n</tr>\n\n", buf);
       } improvement_iterate_end;
@@ -509,11 +564,10 @@ int main(int argc, char **argv)
   /* parse command-line arguments... */
   inx = 1;
   while (inx < argc) {
-    if ((option = get_option_malloc("--ruleset", argv, &inx, argc))) {
+    if ((option = get_option_malloc("--ruleset", argv, &inx, argc, TRUE))) {
       if (ruleset != NULL) {
         fc_fprintf(stderr, _("Multiple rulesets requested. Only one "
                              "ruleset at a time is supported.\n"));
-        free(option);
       } else {
         ruleset = option;
       }
@@ -522,9 +576,9 @@ int main(int argc, char **argv)
       break;
     } else if (is_option("--version", argv[inx])) {
       showvers = TRUE;
-    } else if ((option = get_option_malloc("--log", argv, &inx, argc))) {
-      srvarg.log_filename = option; /* Never freed. */
-#ifndef NDEBUG
+    } else if ((option = get_option_malloc("--log", argv, &inx, argc, TRUE))) {
+      srvarg.log_filename = option;
+#ifndef FREECIV_NDEBUG
     } else if (is_option("--Fatal", argv[inx])) {
       if (inx + 1 >= argc || '-' == argv[inx + 1][0]) {
         srvarg.fatal_assertions = SIGABRT;
@@ -536,8 +590,8 @@ int main(int argc, char **argv)
         inx++;
         showhelp = TRUE;
       }
-#endif /* NDEBUG */
-    } else if ((option = get_option_malloc("--debug", argv, &inx, argc))) {
+#endif /* FREECIV_NDEBUG */
+    } else if ((option = get_option_malloc("--debug", argv, &inx, argc, FALSE))) {
       if (!log_parse_level_str(option, &srvarg.loglevel)) {
         showhelp = TRUE;
         break;
@@ -589,12 +643,12 @@ int main(int argc, char **argv)
                 _("Set debug log level (%d to %d)"),
                 LOG_FATAL, LOG_VERBOSE);
 #endif /* DEBUG */
-#ifndef NDEBUG
+#ifndef FREECIV_NDEBUG
     cmdhelp_add(help, "F",
                   /* TRANS: "Fatal" is exactly what user must type, do not translate. */
                 _("Fatal [SIGNAL]"),
                 _("Raise a signal on failed assertion"));
-#endif /* NDEBUG */
+#endif /* FREECIV_NDEBUG */
     cmdhelp_add(help, "h", "help",
                 _("Print a summary of the options"));
     cmdhelp_add(help, "l",
@@ -620,12 +674,19 @@ int main(int argc, char **argv)
     retval = EXIT_FAILURE;
   }
 
-  FC_FREE(ruleset);
-
   con_log_close();
   registry_module_close();
   free_libfreeciv();
   free_nls();
+  cmdline_option_values_free();
 
   return retval;
 }
+
+/**************************************************************************
+  Empty function required by helpdata
+**************************************************************************/
+void insert_client_build_info(char *outbuf, size_t outlen)
+{
+  /* Nothing here */
+}
