@@ -1,4 +1,4 @@
-/**********************************************************************
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,11 +20,11 @@
 #include <gtk/gtk.h>
 
 /* utility */
+#include "fc_cmdline.h"
 #include "fciconv.h"
 #include "fcintl.h"
 #include "log.h"
 #include "mem.h"
-#include "shared.h"
 
 /* common */
 #include "version.h"
@@ -73,6 +73,7 @@ static void modinst_quit(void)
   save_install_info_lists(&fcmp);
 
   fcmp_deinit();
+  cmdline_option_values_free();
 
   exit(EXIT_SUCCESS);
 }
@@ -628,6 +629,7 @@ int main(int argc, char *argv[])
   }
 
   fcmp_deinit();
+  cmdline_option_values_free();
 
   return EXIT_SUCCESS;
 }
