@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 #include <QVBoxLayout>
 
 // utility
+#include "fc_cmdline.h"
 #include "fciconv.h"
 #include "fcintl.h"
 #include "log.h"
@@ -148,6 +149,7 @@ int main(int argc, char **argv)
   }
 
   fcmp_deinit();
+  cmdline_option_values_free();
 
   return EXIT_SUCCESS;
 }
@@ -179,7 +181,7 @@ static void progress_callback_thr(int downloaded, int max)
 /**************************************************************************
   Setup GUI object
 **************************************************************************/
-void mpgui::setup(QWidget *central, struct fcmp_params *fcmp)
+void mpgui::setup(QWidget *central, struct fcmp_params *params)
 {
 #define URL_LABEL_TEXT N_("Modpack URL")
   QVBoxLayout *main_layout = new QVBoxLayout();
@@ -233,10 +235,10 @@ void mpgui::setup(QWidget *central, struct fcmp_params *fcmp)
   hl->addWidget(URL_label);
 
   URLedit = new QLineEdit(central);
-  if (fcmp->autoinstall == nullptr) {
+  if (params->autoinstall == nullptr) {
     URLedit->setText(DEFAULT_URL_START);
   } else {
-    URLedit->setText(QString::fromUtf8(fcmp->autoinstall));
+    URLedit->setText(QString::fromUtf8(params->autoinstall));
   }
   URLedit->setFocus();
 
