@@ -1,4 +1,4 @@
-/********************************************************************** 
+/**********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,6 +15,8 @@
 #include <fc_config.h>
 #endif /* HAVE_CONFIG_H */
 
+#include "fc_prehdrs.h"
+
 #ifdef HAVE_EXECINFO_H
 #include <execinfo.h>
 #endif
@@ -25,7 +27,8 @@
 
 #include "fcbacktrace.h"
 
-#if defined(DEBUG) && defined(HAVE_BACKTRACE)
+/* We don't want backtrace-spam to testmatic logs */
+#if defined(DEBUG) && defined(HAVE_BACKTRACE) && !defined(FREECIV_TESTMATIC)
 #define BACKTRACE_ACTIVE 1
 #endif
 
@@ -101,31 +104,41 @@ static void backtrace_log(enum log_level level, bool print_from_where,
   }
 
   if (level <= LOG_ERROR) {
-    void *buffer[MAX_NUM_FRAMES];
-    int frames;
-    char **names;
-
-    frames = backtrace(buffer, ARRAY_SIZE(buffer));
-    names = backtrace_symbols(buffer, frames);
-
-    if (names == NULL) {
-      write_backtrace_line(LOG_BACKTRACE, FALSE, NULL, "No backtrace");
-    } else {
-      int i;
-
-      write_backtrace_line(LOG_BACKTRACE, FALSE, NULL, "Backtrace:");
-
-      for (i = 0; i < MIN(frames, MAX_NUM_FRAMES); i++) {
-        char linestr[256];
-
-        fc_snprintf(linestr, sizeof(linestr), "%5d: %s", i, names[i]);
-
-        write_backtrace_line(LOG_BACKTRACE, FALSE, NULL, linestr);
-      }
-
-      free(names);
-    }
+    backtrace_print(LOG_BACKTRACE);
   }
 }
 
 #endif /* BACKTRACE_ACTIVE */
+
+/*****************************************************************************
+  Print backtrace
+*****************************************************************************/
+void backtrace_print(enum log_level level)
+{
+#ifdef BACKTRACE_ACTIVE
+  void *buffer[MAX_NUM_FRAMES];
+  int frames;
+  char **names;
+
+  frames = backtrace(buffer, ARRAY_SIZE(buffer));
+  names = backtrace_symbols(buffer, frames);
+
+  if (names == NULL) {
+    write_backtrace_line(level, FALSE, NULL, "No backtrace");
+  } else {
+    int i;
+
+    write_backtrace_line(level, FALSE, NULL, "Backtrace:");
+
+    for (i = 0; i < MIN(frames, MAX_NUM_FRAMES); i++) {
+      char linestr[256];
+
+      fc_snprintf(linestr, sizeof(linestr), "%5d: %s", i, names[i]);
+
+      write_backtrace_line(level, FALSE, NULL, linestr);
+    }
+
+    free(names);
+  }
+#endif /* BACKTRACE_ACTIVE */
+}
