@@ -1,4 +1,4 @@
-/**********************************************************************
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,7 +15,9 @@
 #include <fc_config.h>
 #endif
 
-#ifdef HAVE_SYS_TYPES_H
+#include "fc_prehdrs.h"
+
+#ifdef FREECIV_HAVE_SYS_TYPES_H
 /* Under Mac OS X sys/types.h must be included before dirent.h */
 #include <sys/types.h>
 #endif
@@ -26,7 +28,6 @@
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 
@@ -40,16 +41,6 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-
-/* Must be before <windows.h> */
-#ifdef HAVE_WINSOCK
-#ifdef HAVE_WINSOCK2
-#include <winsock2.h>
-#else  /* HAVE_WINSOCK2 */
-#include <winsock.h>
-#endif /* HAVE_WINSOCK2 */
-#endif /* HAVE_WINSOCK */
-
 #ifdef WIN32_NATIVE
 #include <windows.h>
 #include <lmcons.h>	/* UNLEN */
@@ -66,38 +57,25 @@
 #include "mem.h"
 #include "rand.h"
 #include "string_vector.h"
-#include "support.h"
 
 #include "shared.h"
 
-#ifndef PATH_SEPARATOR
-#if defined _WIN32 || defined __WIN32__ || defined __EMX__ || defined __DJGPP__
-  /* Win32, OS/2, DOS */
-# define PATH_SEPARATOR ";"
-#else
-  /* Unix */
-# define PATH_SEPARATOR ":"
-#endif
-#endif
-
-#define PARENT_DIR_OPERATOR ".."
-
 /* If no default data path is defined use the default default one */
 #ifndef DEFAULT_DATA_PATH
 #define DEFAULT_DATA_PATH "." PATH_SEPARATOR \
                           "data" PATH_SEPARATOR \
-                          "~/.freeciv/" DATASUBDIR
+                          "~" DIR_SEPARATOR ".freeciv" DIR_SEPARATOR DATASUBDIR
 #endif
 #ifndef DEFAULT_SAVE_PATH
 #define DEFAULT_SAVE_PATH "." PATH_SEPARATOR \
-                          "~/.freeciv/saves"
+                          "~" DIR_SEPARATOR ".freeciv" DIR_SEPARATOR "saves"
 #endif
 #ifndef DEFAULT_SCENARIO_PATH
 #define DEFAULT_SCENARIO_PATH                          \
   "." PATH_SEPARATOR                                   \
-  "data/scenarios" PATH_SEPARATOR                      \
-  "~/.freeciv/" DATASUBDIR "/scenarios" PATH_SEPARATOR \
-  "~/.freeciv/scenarios"
+  "data" DIR_SEPARATOR "scenarios" PATH_SEPARATOR      \
+  "~" DIR_SEPARATOR ".freeciv" DIR_SEPARATOR DATASUBDIR DIR_SEPARATOR "scenarios" PATH_SEPARATOR \
+  "~" DIR_SEPARATOR ".freeciv" DIR_SEPARATOR "scenarios"
 #endif /* DEFAULT_SCENARIO_PATH */
 
 /* environment */
@@ -131,12 +109,30 @@ static struct strvec *scenario_dir_names = NULL;
 static char *mc_group = NULL;
 static char *home_dir = NULL;
 
+static bool depr_freeciv_path_warned = FALSE;
+
 static struct astring realfile = ASTRING_INIT;
 
 static int compare_file_mtime_ptrs(const struct fileinfo *const *ppa,
                                    const struct fileinfo *const *ppb);
 
 
+/***************************************************************************
+  An AND function for fc_tristate.
+***************************************************************************/
+enum fc_tristate fc_tristate_and(enum fc_tristate one, enum fc_tristate two)
+{
+  if (TRI_NO == one || TRI_NO == two) {
+    return TRI_NO;
+  }
+
+  if (TRI_MAYBE == one || TRI_MAYBE == two) {
+    return TRI_MAYBE;
+  }
+
+  return TRI_YES;
+}
+
 /***************************************************************
   Take a string containing multiple lines and create a copy where
   each line is padded to the length of the longest line and centered.
@@ -205,162 +201,6 @@ char *create_centered_string(const char *s)
   return r;
 }
 
-/**************************************************************************
-  return a char * to the parameter of the option or NULL.
-  *i can be increased to get next string in the array argv[].
-  It is an error for the option to exist but be an empty string.
-  This doesn't use log_*() because it is used before logging is set up.
-
-  The argv strings are assumed to be in the local encoding; the returned
-  string is in the internal encoding.
-**************************************************************************/
-char *get_option_malloc(const char *option_name,
-			char **argv, int *i, int argc)
-{
-  int len = strlen(option_name);
-
-  if (strcmp(option_name, argv[*i]) == 0 ||
-      (strncmp(option_name, argv[*i], len) == 0 && argv[*i][len] == '=') ||
-      strncmp(option_name + 1, argv[*i], 2) == 0) {
-    char *opt = argv[*i] + (argv[*i][1] != '-' ? 0 : len);
-
-    if (*opt == '=') {
-      opt++;
-    } else {
-      if (*i < argc - 1) {
-	(*i)++;
-	opt = argv[*i];
-	if (strlen(opt)==0) {
-	  fc_fprintf(stderr, _("Empty argument for \"%s\".\n"), option_name);
-	  exit(EXIT_FAILURE);
-	}
-      }	else {
-	fc_fprintf(stderr, _("Missing argument for \"%s\".\n"), option_name);
-	exit(EXIT_FAILURE);
-      }
-    }
-
-    return local_to_internal_string_malloc(opt);
-  }
-
-  return NULL;
-}
-
-/***************************************************************
-  Is option some form of option_name. option_name must be
-  full length long version such as "--help"
-***************************************************************/
-bool is_option(const char *option_name,char *option)
-{
-  return (strcmp(option_name, option) == 0 ||
-	  strncmp(option_name + 1, option, 2) == 0);
-}
-
-/***************************************************************
-  Like strcspn but also handles quotes, i.e. *reject chars are
-  ignored if they are inside single or double quotes.
-***************************************************************/
-static size_t my_strcspn(const char *s, const char *reject)
-{
-  bool in_single_quotes = FALSE, in_double_quotes = FALSE;
-  size_t i, len = strlen(s);
-
-  for (i = 0; i < len; i++) {
-    if (s[i] == '"' && !in_single_quotes) {
-      in_double_quotes = !in_double_quotes;
-    } else if (s[i] == '\'' && !in_double_quotes) {
-      in_single_quotes = !in_single_quotes;
-    }
-
-    if (in_single_quotes || in_double_quotes) {
-      continue;
-    }
-
-    if (strchr(reject, s[i])) {
-      break;
-    }
-  }
-
-  return i;
-}
-
-/***************************************************************
- Splits the string into tokens. The individual tokens are
- returned. The delimiterset can freely be chosen.
-
- i.e. "34 abc 54 87" with a delimiterset of " " will yield
-      tokens={"34", "abc", "54", "87"}
-
- Part of the input string can be quoted (single or double) to embedded
- delimiter into tokens. For example,
-   command 'a name' hard "1,2,3,4,5" 99
-   create 'Mack "The Knife"'
- will yield 5 and 2 tokens respectively using the delimiterset " ,".
-
- Tokens which aren't used aren't modified (and memory is not
- allocated). If the string would yield more tokens only the first
- num_tokens are extracted.
-
- The user has the responsiblity to free the memory allocated by
- **tokens using free_tokens().
-***************************************************************/
-int get_tokens(const char *str, char **tokens, size_t num_tokens,
-	       const char *delimiterset)
-{
-  int token = 0;
-
-  fc_assert_ret_val(NULL != str, -1);
-
-  for(;;) {
-    size_t len, padlength = 0;
-
-    /* skip leading delimiters */
-    str += strspn(str, delimiterset);
-
-    if (*str == '\0') {
-      break;
-    }
-
-    len = my_strcspn(str, delimiterset);
-
-    if (token >= num_tokens) {
-      break;
-    }
-
-    /* strip start/end quotes if they exist */
-    if (len >= 2) {
-      if ((str[0] == '"' && str[len - 1] == '"')
-	  || (str[0] == '\'' && str[len - 1] == '\'')) {
-	len -= 2;
-	padlength = 1;		/* to set the string past the end quote */
-	str++;
-      }
-    }
-
-    tokens[token] = fc_malloc(len + 1);
-    (void) fc_strlcpy(tokens[token], str, len + 1);     /* adds the '\0' */
-
-    token++;
-
-    str += len + padlength;
-  }
-
-  return token;
-}
-
-/***************************************************************
-  Frees a set of tokens created by get_tokens().
-***************************************************************/
-void free_tokens(char **tokens, size_t ntokens)
-{
-  size_t i;
-  for (i = 0; i < ntokens; i++) {
-    if (tokens[i]) {
-      free(tokens[i]);
-    }
-  }
-}
-
 /***************************************************************
   Returns a statically allocated string containing a nicely-formatted
   version of the given number according to the user's locale.  (Only
@@ -714,8 +554,8 @@ size_t loud_strlcpy(char *buffer, const char *str, size_t len,
 }
 
 /****************************************************************************
-  Convert 'str' to it's string reprentation if possible. 'pint' can be NULL,
-  then it will only test 'str' only contains a number.
+  Convert 'str' to it's int reprentation if possible. 'pint' can be NULL,
+  then it will only test 'str' only contains an integer number.
 ****************************************************************************/
 bool str_to_int(const char *str, int *pint)
 {
@@ -746,6 +586,54 @@ bool str_to_int(const char *str, int *pint)
   return ('\0' == *str && (NULL == pint || 1 == sscanf(start, "%d", pint)));
 }
 
+/****************************************************************************
+  Convert 'str' to it's float reprentation if possible. 'pfloat' can be NULL,
+  then it will only test 'str' only contains a floating point number.
+****************************************************************************/
+bool str_to_float(const char *str, float *pfloat)
+{
+  bool dot;
+  const char *start;
+
+  fc_assert_ret_val(NULL != str, FALSE);
+
+  while (fc_isspace(*str)) {
+    /* Skip leading spaces. */
+    str++;
+  }
+
+  start = str;
+
+  if ('-' == *str || '+' == *str) {
+    /* Handle sign. */
+    str++;
+  }
+  while (fc_isdigit(*str)) {
+    /* Digits. */
+    str++;
+  }
+
+  if (*str == '.') {
+    dot = TRUE;
+    str++;
+
+    while (fc_isdigit(*str)) {
+      /* Digits. */
+      str++;
+    }
+  } else {
+    dot = FALSE;
+  }
+
+  while (fc_isspace(*str)) {
+    /* Ignore trailing spaces. */
+    str++;
+  }
+
+  return ('\0' == *str && dot
+          && (NULL == pfloat || 1 == sscanf(start, "%f", pfloat)));
+}
+
 /***************************************************************************
   Returns string which gives users home dir, as specified by $HOME.
   Gets value once, and then caches result.
@@ -759,59 +647,65 @@ char *user_home_dir(void)
 #else  /* AMIGA */
 
   if (home_dir == NULL) {
-    char *env = getenv("HOME");
+#ifdef FREECIV_MSWINDOWS
 
-    if (env) {
-      home_dir = fc_strdup(env);
-      log_verbose("HOME is %s", home_dir);
-    } else {
+    /* some documentation at:
+     * http://justcheckingonall.wordpress.com/2008/05/16/find-shell-folders-win32/
+     * http://archives.seul.org/or/cvs/Oct-2004/msg00082.html */
 
-#ifdef WIN32_NATIVE
+    LPITEMIDLIST pidl;
+    LPMALLOC pMalloc;
 
-      /* some documentation at:
-       * http://justcheckingonall.wordpress.com/2008/05/16/find-shell-folders-win32/
-       * http://archives.seul.org/or/cvs/Oct-2004/msg00082.html */
+    if (SUCCEEDED(SHGetSpecialFolderLocation(NULL, CSIDL_APPDATA, &pidl))) {
+      char *home_dir_in_local_encoding = fc_malloc(PATH_MAX);
 
-      LPITEMIDLIST pidl;
-      LPMALLOC pMalloc;
+      if (SUCCEEDED(SHGetPathFromIDList(pidl, home_dir_in_local_encoding))) {
+        /* convert to internal encoding */
+        home_dir = local_to_internal_string_malloc(home_dir_in_local_encoding);
+        free(home_dir_in_local_encoding);
 
-      if (SUCCEEDED(SHGetSpecialFolderLocation(NULL, CSIDL_APPDATA, &pidl))) {
+#ifdef DIR_SEPARATOR_IS_DEFAULT
+        /* replace backslashes with forward slashes */
+        {
+          char *c;
 
-        char *home_dir_in_local_encoding = fc_malloc(PATH_MAX);
-
-        if (SUCCEEDED(SHGetPathFromIDList(pidl, home_dir_in_local_encoding))) {
-        	/* convert to internal encoding */
-        	home_dir = local_to_internal_string_malloc(home_dir_in_local_encoding);
-        	free(home_dir_in_local_encoding);
-
-        	/* replace backslashes with forward slashes */
-        	char *c;
-        	for (c = home_dir; *c != 0; c++) {
-        		if (*c == '\\') {
-        			*c = '/';
-        		}
-        	}
-        } else {
-            free(home_dir_in_local_encoding);
-            home_dir = NULL;
-            log_error("Could not find home directory "
-                      "(SHGetPathFromIDList() failed).");
+          for (c = home_dir; *c != 0; c++) {
+            if (*c == '\\') {
+              *c = DIR_SEPARATOR_CHAR;
+            }
+          }
         }
-
-        SHGetMalloc(&pMalloc);
-        if (pMalloc) {
-          pMalloc->lpVtbl->Free(pMalloc, pidl);
-          pMalloc->lpVtbl->Release(pMalloc);
-        }
-
+#endif /* DIR_SEPARATOR_IS_DEFAULT */
       } else {
+        free(home_dir_in_local_encoding);
+        home_dir = NULL;
         log_error("Could not find home directory "
-                  "(SHGetSpecialFolderLocation() failed).");
+                  "(SHGetPathFromIDList() failed).");
+      }
+
+      SHGetMalloc(&pMalloc);
+      if (pMalloc) {
+        pMalloc->lpVtbl->Free(pMalloc, pidl);
+        pMalloc->lpVtbl->Release(pMalloc);
+      }
+
+    } else {
+      log_error("Could not find home directory "
+                "(SHGetSpecialFolderLocation() failed).");
+    }
+
+    if (home_dir == NULL)
+#endif /* FREECIV_MSWINDOWS */
+    {
+      char *env = getenv("HOME");
+
+      if (env) {
+        home_dir = fc_strdup(env);
+        log_verbose("HOME is %s", home_dir);
+      } else {
+        log_error("Could not find home directory (HOME is not set).");
+        home_dir = NULL;
       }
-#else  /* WIN32_NATIVE */
-      log_error("Could not find home directory (HOME is not set).");
-      home_dir = NULL;
-#endif /* WIN32_NATIVE */
     }
   }
 
@@ -917,15 +811,15 @@ static struct strvec *base_get_dirs(const char *dir_list)
 
     tok = skip_leading_spaces(tok);
     remove_trailing_spaces(tok);
-    if (strcmp(tok, "/") != 0) {
-      remove_trailing_char(tok, '/');
+    if (strcmp(tok, DIR_SEPARATOR) != 0) {
+      remove_trailing_char(tok, DIR_SEPARATOR_CHAR);
     }
 
     i = strlen(tok);
     if (tok[0] == '~') {
-      if (i > 1 && tok[1] != '/') {
+      if (i > 1 && tok[1] != DIR_SEPARATOR_CHAR) {
         log_error("For \"%s\" in path cannot expand '~'"
-                  " except as '~/'; ignoring", tok);
+                  " except as '~" DIR_SEPARATOR "'; ignoring", tok);
         i = 0;  /* skip this one */
       } else {
         char *home = user_home_dir();
@@ -1006,12 +900,18 @@ const struct strvec *get_data_dirs(void)
                 FREECIV_DATA_PATH, FREECIV_PATH);
       path = NULL;
     }
-    if (NULL == path && (path = getenv(FREECIV_PATH)) && '\0' == path[0]) {
-      /* TRANS: <FREECIV_PATH> configuration error */
-      log_error(_("\"%s\" is set but empty; using default \"%s\" "
-                 "data directories instead."),
-                FREECIV_PATH, DEFAULT_DATA_PATH);
-      path = NULL;
+    if (NULL == path && (path = getenv(FREECIV_PATH))) {
+      if (!depr_freeciv_path_warned) {
+        log_error(_("FREECIV_PATH is deprecated, and won't work in future versions."));
+        depr_freeciv_path_warned = TRUE;
+      }
+      if ('\0' == path[0]) {
+        /* TRANS: <FREECIV_PATH> configuration error */
+        log_error(_("\"%s\" is set but empty; using default \"%s\" "
+                    "data directories instead."),
+                  FREECIV_PATH, DEFAULT_DATA_PATH);
+        path = NULL;
+      }
     }
     data_dir_names = base_get_dirs(NULL != path ? path : DEFAULT_DATA_PATH);
     strvec_remove_duplicate(data_dir_names, strcmp); /* Don't set a path both. */
@@ -1051,6 +951,10 @@ const struct strvec *get_save_dirs(void)
       path = NULL;
     }
     if (NULL == path && (path = getenv(FREECIV_PATH))) {
+      if (!depr_freeciv_path_warned) {
+        log_error(_("FREECIV_PATH is deprecated, and won't work in future versions."));
+        depr_freeciv_path_warned = TRUE;
+      }
       if ('\0' == path[0]) {
         /* TRANS: <FREECIV_PATH> configuration error */
         log_error(_("\"%s\" is set but empty; using default \"%s\" "
@@ -1110,6 +1014,10 @@ const struct strvec *get_scenario_dirs(void)
       path = NULL;
     }
     if (NULL == path && (path = getenv(FREECIV_PATH))) {
+      if (!depr_freeciv_path_warned) {
+        log_error(_("FREECIV_PATH is deprecated, and won't work in future versions."));
+        depr_freeciv_path_warned = TRUE;
+      }
       if ('\0' == path[0]) {
         /* TRANS: <FREECIV_PATH> configuration error */
         log_error( _("\"%s\" is set but empty; using default \"%s\" "
@@ -1162,7 +1070,7 @@ struct strvec *fileinfolist(const struct strvec *dirs, const char *suffix)
   struct strvec *files = strvec_new();
   size_t suffix_len = strlen(suffix);
 
-  fc_assert_ret_val(!strchr(suffix, '/'), NULL);
+  fc_assert_ret_val(!strchr(suffix, DIR_SEPARATOR_CHAR), NULL);
 
   if (NULL == dirs) {
     return files;
@@ -1233,6 +1141,13 @@ struct strvec *fileinfolist(const struct strvec *dirs, const char *suffix)
 ***************************************************************************/
 const char *fileinfoname(const struct strvec *dirs, const char *filename)
 {
+#ifndef DIR_SEPARATOR_IS_DEFAULT
+  char fnbuf[filename != NULL ? strlen(filename) + 1 : 1];
+  int i;
+#else  /* DIR_SEPARATOR_IS_DEFAULT */
+  const char *fnbuf = filename;
+#endif /* DIR_SEPARATOR_IS_DEFAULT */
+
   if (NULL == dirs) {
     return NULL;
   }
@@ -1253,10 +1168,21 @@ const char *fileinfoname(const struct strvec *dirs, const char *filename)
     return astr_str(&realfile);
   }
 
+#ifndef DIR_SEPARATOR_IS_DEFAULT
+  for (i = 0; filename[i] != '\0'; i++) {
+    if (filename[i] == '/') {
+      fnbuf[i] = DIR_SEPARATOR_CHAR;
+    } else {
+      fnbuf[i] = filename[i];
+    }
+  }
+  fnbuf[i] = '\0';
+#endif /* DIR_SEPARATOR_IS_DEFAULT */
+
   strvec_iterate(dirs, dirname) {
     struct stat buf;    /* see if we can open the file or directory */
 
-    astr_set(&realfile, "%s/%s", dirname, filename);
+    astr_set(&realfile, "%s" DIR_SEPARATOR "%s", dirname, fnbuf);
     if (fc_stat(astr_str(&realfile), &buf) == 0) {
       return astr_str(&realfile);
     }
@@ -1359,7 +1285,7 @@ struct fileinfo_list *fileinfolist_infix(const struct strvec *dirs,
         size_t len = strlen(dirname) + strlen(filename) + 2;
 
         fullname = fc_malloc(len);
-        fc_snprintf(fullname, len, "%s/%s", dirname, filename);
+        fc_snprintf(fullname, len, "%s" DIR_SEPARATOR "%s", dirname, filename);
 
         if (fc_stat(fullname, &buf) == 0) {
           file = fc_malloc(sizeof(*file));
@@ -1400,12 +1326,11 @@ struct fileinfo_list *fileinfolist_infix(const struct strvec *dirs,
 /***************************************************************************
   Language environmental variable (with emulation).
 ***************************************************************************/
-char *get_langname(void)
+char *setup_langname(void)
 {
   char *langname = NULL;
 
 #ifdef ENABLE_NLS
-
   langname = getenv("LANG");
 
 #ifdef WIN32_NATIVE
@@ -1413,71 +1338,110 @@ char *get_langname(void)
   if (!langname) {
     switch (PRIMARYLANGID(GetUserDefaultLangID())) {
       case LANG_ARABIC:
-        return "ar";
+        langname = "ar";
+        break;
       case LANG_CATALAN:
-        return "ca";
+        langname = "ca";
+        break;
       case LANG_CZECH:
-        return "cs";
+        langname = "cs";
+        break;
       case LANG_DANISH:
-        return "da";
+        langname = "da";
+        break;
       case LANG_GERMAN:
-        return "de";
+        langname = "de";
+        break;
       case LANG_GREEK:
-        return "el";
+        langname = "el";
+        break;
       case LANG_ENGLISH:
         switch (SUBLANGID(GetUserDefaultLangID())) {
           case SUBLANG_ENGLISH_UK:
-            return "en_GB";
+            langname = "en_GB";
+            break;
           default:
-            return "en";
+            langname = "en";
+            break;
         }
+        break;
       case LANG_SPANISH:
-        return "es";
+        langname = "es";
+        break;
       case LANG_ESTONIAN:
-        return "et";
+        langname = "et";
+        break;
       case LANG_FARSI:
-        return "fa";
+        langname = "fa";
+        break;
       case LANG_FINNISH:
-        return "fi";
+        langname = "fi";
+        break;
       case LANG_FRENCH:
-        return "fr";
+        langname = "fr";
+        break;
       case LANG_HEBREW:
-        return "he";
+        langname = "he";
+        break;
       case LANG_HUNGARIAN:
-        return "hu";
+        langname = "hu";
+        break;
       case LANG_ITALIAN:
-        return "it";
+        langname = "it";
+        break;
       case LANG_JAPANESE:
-        return "ja";
+        langname = "ja";
+        break;
       case LANG_KOREAN:
-        return "ko";
+        langname = "ko";
+        break;
       case LANG_LITHUANIAN:
-        return "lt";
+        langname = "lt";
+        break;
       case LANG_DUTCH:
-        return "nl";
+        langname = "nl";
+        break;
       case LANG_NORWEGIAN:
-        return "nb";
+        langname = "nb";
+        break;
       case LANG_POLISH:
-        return "pl";
+        langname = "pl";
+        break;
       case LANG_PORTUGUESE:
         switch (SUBLANGID(GetUserDefaultLangID())) {
           case SUBLANG_PORTUGUESE_BRAZILIAN:
-            return "pt_BR";
+            langname = "pt_BR";
+            break;
           default:
-            return "pt";
+            langname = "pt";
+            break;
         }
+        break;
       case LANG_ROMANIAN:
-        return "ro";
+        langname = "ro";
+        break;
       case LANG_RUSSIAN:
-        return "ru";
+        langname = "ru";
+        break;
       case LANG_SWEDISH:
-        return "sv";
+        langname = "sv";
+        break;
       case LANG_TURKISH:
-        return "tr";
+        langname = "tr";
+        break;
       case LANG_UKRAINIAN:
-        return "uk";
+        langname = "uk";
+        break;
       case LANG_CHINESE:
-        return "zh_CN";
+        langname = "zh_CN";
+        break;
+    }
+
+    if (langname != NULL) {
+      static char envstr[40];
+
+      fc_snprintf(envstr, sizeof(envstr), "LANG=%s", langname);
+      putenv(envstr);
     }
   }
 #endif /* WIN32_NATIVE */
@@ -1502,17 +1466,11 @@ void init_nls(void)
 #ifdef ENABLE_NLS
 
 #ifdef WIN32_NATIVE
-  char *langname = get_langname();
-  if (langname) {
-    static char envstr[40];
-
-    fc_snprintf(envstr, sizeof(envstr), "LANG=%s", langname);
-    putenv(envstr);
-  }
+  setup_langname(); /* Makes sure LANG env variable has been set */
 #endif /* WIN32_NATIVE */
 
   (void) setlocale(LC_ALL, "");
-  (void) bindtextdomain(PACKAGE, LOCALEDIR);
+  (void) bindtextdomain(PACKAGE, get_locale_dir());
   (void) textdomain(PACKAGE);
 
   /* Don't touch the defaults when LC_NUMERIC == "C".
@@ -1710,10 +1668,10 @@ enum m_pre_result match_prefix_full(m_pre_accessor_fn_t accessor_fn,
   servers on the LAN, as specified by $FREECIV_MULTICAST_GROUP.
   Gets value once, and then caches result.
 ***************************************************************************/
-char *get_multicast_group(bool ipv6_prefered)
+char *get_multicast_group(bool ipv6_preferred)
 {
   static char *default_multicast_group_ipv4 = "225.1.1.1";
-#ifdef IPV6_SUPPORT
+#ifdef FREECIV_IPV6_SUPPORT
   /* TODO: Get useful group (this is node local) */
   static char *default_multicast_group_ipv6 = "FF31::8000:15B4";
 #endif /* IPv6 support */
@@ -1724,8 +1682,8 @@ char *get_multicast_group(bool ipv6_prefered)
     if (env) {
       mc_group = fc_strdup(env);
     } else {
-#ifdef IPV6_SUPPORT
-      if (ipv6_prefered) {
+#ifdef FREECIV_IPV6_SUPPORT
+      if (ipv6_preferred) {
         mc_group = fc_strdup(default_multicast_group_ipv6);
       } else
 #endif /* IPv6 support */
@@ -1758,8 +1716,8 @@ void free_multicast_group(void)
 ***************************************************************************/
 void interpret_tilde(char* buf, size_t buf_size, const char* filename)
 {
-  if (filename[0] == '~' && filename[1] == '/') {
-    fc_snprintf(buf, buf_size, "%s/%s", user_home_dir(), filename + 2);
+  if (filename[0] == '~' && filename[1] == DIR_SEPARATOR_CHAR) {
+    fc_snprintf(buf, buf_size, "%s" DIR_SEPARATOR "%s", user_home_dir(), filename + 2);
   } else if (filename[0] == '~' && filename[1] == '\0') {
     strncpy(buf, user_home_dir(), buf_size);
   } else  {
@@ -1775,7 +1733,7 @@ void interpret_tilde(char* buf, size_t buf_size, const char* filename)
 ***************************************************************************/
 char *interpret_tilde_alloc(const char* filename)
 {
-  if (filename[0] == '~' && filename[1] == '/') {
+  if (filename[0] == '~' && filename[1] == DIR_SEPARATOR_CHAR) {
     const char *home = user_home_dir();
     size_t sz;
     char *buf;
@@ -1802,7 +1760,7 @@ char *skip_to_basename(char *filepath)
   fc_assert_ret_val(NULL != filepath, NULL);
 
   for (j = strlen(filepath); j >= 0; j--) {
-    if (filepath[j] == '/') {
+    if (filepath[j] == DIR_SEPARATOR_CHAR) {
       return &filepath[j+1];
     }
   }
@@ -1821,22 +1779,33 @@ bool make_dir(const char *pathname)
   path = interpret_tilde_alloc(pathname);
   dir = path;
   do {
-    dir = strchr(dir, '/');
+    dir = strchr(dir, DIR_SEPARATOR_CHAR);
     /* We set the current / with 0, and restore it afterwards */
     if (dir) {
       *dir = 0;
     }
 
 #ifdef WIN32_NATIVE
-    char *path_in_local_encoding = internal_to_local_string_malloc(path);
-    _mkdir(path_in_local_encoding);
-    free(path_in_local_encoding);
-#else
+#ifdef HAVE__MKDIR
+    /* Prefer _mkdir() in Windows even if mkdir() would seem to be available -
+     * chances are that it's wrong kind of mkdir().
+     * TODO: Make a configure check for mkdir() that also makes sure that it
+     *       takes two parameters, and prefer such proper mkdir() here. */
+    {
+      char *path_in_local_encoding = internal_to_local_string_malloc(path);
+
+      _mkdir(path_in_local_encoding);
+      free(path_in_local_encoding);
+    }
+#else  /* HAVE__MKDIR */
     mkdir(path, 0755);
-#endif
+#endif /* HAVE__MKDIR */
+#else  /* WIN32_NATIVE */
+    mkdir(path, 0755);
+#endif /* WIN32_NATIVE */
 
     if (dir) {
-      *dir = '/';
+      *dir = DIR_SEPARATOR_CHAR;
       dir++;
     }
   } while (dir);
@@ -2359,7 +2328,7 @@ static size_t extract_escapes(const char *format, char *escapes,
   };
   bool reordered = FALSE;
   size_t num = 0;
-  int index = 0;
+  int idx = 0;
 
   memset(escapes, 0, max_escapes);
   format = strchr(format, '%');
@@ -2376,7 +2345,7 @@ static size_t extract_escapes(const char *format, char *escapes,
       } while (fc_isdigit(*format));
       if ('$' == *format) {
         /* Strings are reordered. */
-        sscanf(start, "%d", &index);
+        sscanf(start, "%d", &idx);
         reordered = TRUE;
       }
     }
@@ -2385,15 +2354,15 @@ static size_t extract_escapes(const char *format, char *escapes,
            && NULL == strchr(format_escapes, *format)) {
       format++;
     }
-    escapes[index] = *format;
+    escapes[idx] = *format;
 
     /* Increase the read count. */
     if (reordered) {
-      if (index > num) {
-        num = index;
+      if (idx > num) {
+        num = idx;
       }
     } else {
-      index++;
+      idx++;
       num++;
     }
 
