@@ -1,4 +1,4 @@
-/**********************************************************************
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,7 +11,7 @@
    GNU General Public License for more details.
 ***********************************************************************/
 
-/********************************************************************** 
+/***********************************************************************
   This module contains replacements for functions which are not
   available on all platforms.  Where the functions are available
   natively, these are (mostly) just wrappers.
@@ -20,7 +20,7 @@
   alternative would be to use the "standard" function name, and
   provide the implementation only if required.  However the method
   here has some advantages:
-  
+
    - We can provide definite prototypes in support.h, rather than
    worrying about whether a system prototype exists, and if so where,
    and whether it is correct.  (Note that whether or not configure
@@ -35,12 +35,14 @@
   The main disadvantage is remembering to use these "fc" functions on
   systems which have the functions natively.
 
-**********************************************************************/
+***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
 #include <fc_config.h>
 #endif
 
+#include "fc_prehdrs.h"
+
 #include <ctype.h>
 #include <errno.h>
 #include <math.h> /* ceil() */
@@ -65,7 +67,7 @@
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
-#ifdef HAVE_SYS_TYPES_H
+#ifdef FREECIV_HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
 #ifdef HAVE_UNISTD_H
@@ -74,30 +76,13 @@
 #ifdef HAVE_SYS_UTSNAME_H
 #include <sys/utsname.h>
 #endif
-#ifdef HAVE_LIBZ
+#ifdef FREECIV_HAVE_LIBZ
 #include <zlib.h>
 #endif
-
-/* Must be before <windows.h> */
-#ifdef HAVE_WINSOCK
-#ifdef HAVE_WINSOCK2
-#include <winsock2.h>
-#else  /* HAVE_WINSOCK2 */
-#include <winsock.h>
-#endif /* HAVE_WINSOCK2 */
-#endif /* HAVE_WINSOCK */
-
 #ifdef WIN32_NATIVE
 #include <process.h>
 #include <windows.h>
 #endif
-#ifdef HAVE_WINSOCK
-#ifdef HAVE_WINSOCK2
-#include <winsock2.h>
-#else  /* HAVE_WINSOCK2 */
-#include <winsock.h>
-#endif /* HAVE_WINSOCK2 */
-#endif /* HAVE_WINSOCK */
 #ifdef HAVE_STRINGS_H
 #  include <strings.h>
 #endif
@@ -106,7 +91,7 @@
 #  include <libgen.h>
 #endif
 
-#ifdef HAVE_LIBZ
+#ifdef FREECIV_HAVE_LIBZ
 #include <zlib.h>
 #endif
 
@@ -176,6 +161,71 @@ int fc_strncasecmp(const char *str0, const char *str1, size_t n)
 #endif /* HAVE_STRNCASECMP */
 }
 
+/****************************************************************************
+  Copies a string and convert the following characters:
+  - '\n' to "\\n".
+  - '\\' to "\\\\".
+  - '\"' to "\\\"".
+  See also remove_escapes().
+****************************************************************************/
+void make_escapes(const char *str, char *buf, size_t buf_len)
+{
+  char *dest = buf;
+  /* Sometimes we insert 2 characters at once ('\n' -> "\\n"), so keep
+   * place for '\0' and an extra character. */
+  const char *const max = buf + buf_len - 2;
+
+  while (*str != '\0' && dest < max) {
+    switch (*str) {
+    case '\n':
+      *dest++ = '\\';
+      *dest++ = 'n';
+      str++;
+      break;
+    case '\\':
+    case '\"':
+      *dest++ = '\\';
+      /* Fallthrough. */
+    default:
+      *dest++ = *str++;
+      break;
+    }
+  }
+  *dest = 0;
+}
+
+/****************************************************************************
+  Copies a string. Backslash followed by a genuine newline always
+  removes the newline.
+  If full_escapes is TRUE:
+    - '\n' -> newline translation.
+    - Other '\c' sequences (any character 'c') are just passed
+      through with the '\' removed (eg, includes '\\', '\"').
+  See also make_escapes().
+****************************************************************************/
+void remove_escapes(const char *str, bool full_escapes,
+                    char *buf, size_t buf_len)
+{
+  char *dest = buf;
+  const char *const max = buf + buf_len - 1;
+
+  while (*str != '\0' && dest < max) {
+    if (*str == '\\' && *(str + 1) == '\n') {
+      /* Escape followed by newline. Skip both */
+      str += 2;
+    } else if (full_escapes && *str == '\\') {
+      str++;
+      if (*str == 'n') {
+        *dest++ = '\n';
+        str++;
+      }
+    } else {
+      *dest++ = *str++;
+    }
+  }
+  *dest = '\0';
+}
+
 /***************************************************************
   Count length of string without possible surrounding quotes.
 ***************************************************************/
@@ -344,7 +394,7 @@ FILE *fc_fopen(const char *filename, const char *opentype)
   Wrapper function for gzopen() with filename conversion to local
   encoding on Windows.
 *****************************************************************/
-#ifdef HAVE_LIBZ
+#ifdef FREECIV_HAVE_LIBZ
 gzFile fc_gzopen(const char *filename, const char *opentype)
 {
 #ifdef WIN32_NATIVE
@@ -358,23 +408,23 @@ gzFile fc_gzopen(const char *filename, const char *opentype)
 	return gzopen(filename, opentype);
 #endif /* WIN32_NATIVE */
 }
-#endif /* HAVE_LIBZ */
+#endif /* FREECIV_HAVE_LIBZ */
 
 /******************************************************************
   Wrapper function for opendir() with filename conversion to local
   encoding on Windows.
 ******************************************************************/
-DIR *fc_opendir(const char *dirname)
+DIR *fc_opendir(const char *dir_to_open)
 {
 #ifdef WIN32_NATIVE
 	DIR *result;
 	char *dirname_in_local_encoding =
-	     internal_to_local_string_malloc(dirname);
+	     internal_to_local_string_malloc(dir_to_open);
 	result = opendir(dirname_in_local_encoding);
 	free(dirname_in_local_encoding);
 	return result;
 #else  /* WIN32_NATIVE */
-	return opendir(dirname);
+	return opendir(dir_to_open);
 #endif /* WIN32_NATIVE */
 }
 
@@ -505,6 +555,7 @@ char *fc_strrep_resize(char *str, size_t *len, const char *search,
                        const char *replace)
 {
   size_t len_max;
+  bool success;
 
   fc_assert_ret_val(str != NULL, NULL);
   fc_assert_ret_val(len != NULL, NULL);
@@ -520,8 +571,9 @@ char *fc_strrep_resize(char *str, size_t *len, const char *search,
     str = fc_realloc(str, len_max);
   }
 
+  success = fc_strrep(str, (*len), search, replace);
   /* should never happen */
-  fc_assert_ret_val_msg(fc_strrep(str, (*len), search, replace), NULL,
+  fc_assert_ret_val_msg(success == TRUE, NULL,
                         "Can't replace '%s' by '%s' in '%s'. To small "
                         "size after reallocation: %lu.", search, replace,
                         str, (long unsigned int)*len);
@@ -831,9 +883,9 @@ int fc_gethostname(char *buf, size_t len)
 #endif
 }
 
-#ifdef SOCKET_ZERO_ISNT_STDIN
+#ifdef FREECIV_SOCKET_ZERO_NOT_STDIN
 /**********************************************************************
-  Support for console I/O in case SOCKET_ZERO_ISNT_STDIN.
+  Support for console I/O in case FREECIV_SOCKET_ZERO_NOT_STDIN.
 ***********************************************************************/
 
 #define CONSOLE_BUF_SIZE 100
@@ -859,7 +911,7 @@ static DWORD WINAPI thread_proc(LPVOID arg)
 #endif /* WIN32_NATIVE */
 
 /**********************************************************************
-  Initialize console I/O in case SOCKET_ZERO_ISNT_STDIN.
+  Initialize console I/O in case FREECIV_SOCKET_ZERO_NOT_STDIN.
 ***********************************************************************/
 void fc_init_console(void)
 {
@@ -885,7 +937,7 @@ void fc_init_console(void)
 }
 
 /**********************************************************************
-  Read a line from console I/O in case SOCKET_ZERO_ISNT_STDIN.
+  Read a line from console I/O in case FREECIV_SOCKET_ZERO_NOT_STDIN.
 
   This returns a pointer to a statically allocated buffer.
   Subsequent calls to fc_read_console() or fc_init_console() will
@@ -927,7 +979,7 @@ char *fc_read_console(void)
 #endif /* WIN32_NATIVE */
 }
 
-#endif /* SOCKET_ZERO_ISNT_STDIN */
+#endif /* FREECIV_SOCKET_ZERO_NOT_STDIN */
 
 /**********************************************************************
   Returns TRUE iff the file is a regular file or a link to a regular
@@ -1094,112 +1146,6 @@ char fc_tolower(char c)
   return (char) tolower((int) ((unsigned char) c));
 }
 
-/*****************************************************************
-  Returns an uname like string.
-*****************************************************************/
-void fc_uname(char *buf, size_t len)
-{
-#ifdef HAVE_UNAME
-  {
-    struct utsname un;
-
-    uname(&un);
-    fc_snprintf(buf, len, "%s %s [%s]", un.sysname, un.release, un.machine);
-  }
-#else /* ! HAVE_UNAME */
-  /* Fill in here if you are making a binary without sys/utsname.h and know
-     the OS name, release number, and machine architechture */
-#ifdef WIN32_NATIVE
-  {
-    /* TODO: Add handling of newer Windows versions. */
-    char cpuname[16];
-    char *osname;
-    SYSTEM_INFO sysinfo;
-    OSVERSIONINFO osvi;
-
-    osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
-    GetVersionEx(&osvi);
-
-    switch (osvi.dwPlatformId) {
-    case VER_PLATFORM_WIN32s:
-      osname = "Win32s";
-      break;
-
-    case VER_PLATFORM_WIN32_WINDOWS:
-      osname = "Win32";
-
-      if (osvi.dwMajorVersion == 4) {
-	switch (osvi.dwMinorVersion) {
-	case  0: osname = "Win95";    break;
-	case 10: osname = "Win98";    break;
-	case 90: osname = "WinME";    break;
-	default:			    break;
-	}
-      }
-      break;
-
-    case VER_PLATFORM_WIN32_NT:
-      osname = "WinNT";
-
-      if (osvi.dwMajorVersion == 5) {
-	switch (osvi.dwMinorVersion) {
-	case 0: osname = "Win2000";   break;
-	case 1: osname = "WinXP";	    break;
-	default:			    break;
-	}
-      }
-      break;
-
-    default:
-      osname = osvi.szCSDVersion;
-      break;
-    }
-
-    GetSystemInfo(&sysinfo); 
-    switch (sysinfo.wProcessorArchitecture) {
-      case PROCESSOR_ARCHITECTURE_INTEL:
-	{
-	  unsigned int ptype;
-	  if (sysinfo.wProcessorLevel < 3) /* Shouldn't happen. */
-	    ptype = 3;
-	  else if (sysinfo.wProcessorLevel > 9) /* P4 */
-	    ptype = 6;
-	  else
-	    ptype = sysinfo.wProcessorLevel;
-
-          fc_snprintf(cpuname, sizeof(cpuname), "i%d86", ptype);
-	}
-	break;
-
-      case PROCESSOR_ARCHITECTURE_MIPS:
-	sz_strlcpy(cpuname, "mips");
-	break;
-
-      case PROCESSOR_ARCHITECTURE_ALPHA:
-	sz_strlcpy(cpuname, "alpha");
-	break;
-
-      case PROCESSOR_ARCHITECTURE_PPC:
-	sz_strlcpy(cpuname, "ppc");
-	break;
-#if 0
-      case PROCESSOR_ARCHITECTURE_IA64:
-	sz_strlcpy(cpuname, "ia64");
-	break;
-#endif
-      default:
-	sz_strlcpy(cpuname, "unknown");
-	break;
-    }
-    fc_snprintf(buf, len, "%s %ld.%ld [%s]",
-                osname, osvi.dwMajorVersion, osvi.dwMinorVersion, cpuname);
-  }
-#else  /* WIN32_NATIVE */
-  fc_snprintf(buf, len, "unknown unknown [unknown]");
-#endif /* WIN32_NATIVE */
-#endif /* HAVE_UNAME */
-}
-
 /*****************************************************************
   basename() replacement that always takes const parameter.
   POSIX basename() modifies its parameter, GNU one does not.
@@ -1216,3 +1162,15 @@ const char *fc_basename(const char *path)
 
   return basename(buf);
 }
+
+/*****************************************************************
+  Set quick_exit() callback if possible.
+*****************************************************************/
+int fc_at_quick_exit(void (*func)(void))
+{
+#ifdef HAVE_AT_QUICK_EXIT
+  return at_quick_exit(func);
+#else  /* HAVE_AT_QUICK_EXIT */
+  return -1;
+#endif /* HAVE_AT_QUICK_EXIT */
+}
