@@ -1,4 +1,4 @@
-/**********************************************************************
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,19 +18,21 @@
 extern "C" {
 #endif /* __cplusplus */
 
-/********************************************************************** 
+/***********************************************************************
   Replacements for functions which are not available on all platforms.
   Where the functions are available natively, these are just wrappers.
   See also mem.h, netintf.h, rand.h, and see support.c for more comments.
 ***********************************************************************/
 
+#include <freeciv_config.h>
+
 #include <dirent.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>		/* size_t */
 #include <sys/stat.h>
 
-#ifdef HAVE_SYS_TYPES_H
+#ifdef FREECIV_HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
 
@@ -50,31 +52,30 @@ extern "C" {
 #include <posix/be_prim.h>
 #define __bool_true_false_are_defined 1
 #else
-#ifdef HAVE_STDBOOL_H
+#ifdef FREECIV_HAVE_STDBOOL_H
 #include <stdbool.h>
 #else /* Implement <stdbool.h> ourselves */
 #undef bool
 #undef true
 #undef false
 #undef __bool_true_false_are_defined
-#define bool fc_bool
+#define bool unsigned int
 #define true  1
 #define false 0
 #define __bool_true_false_are_defined 1
-typedef unsigned int fc_bool;
-#endif /* ! HAVE_STDBOOL_H */
+#endif /* ! FREECIV_HAVE_STDBOOL_H */
 #endif /* ! __BEOS__ */
 #endif /* __cplusplus */
 
 /* intptr_t header */
 /* Prefer full inttypes.h if present. */
-#ifdef HAVE_INTTYPES_H
+#ifdef FREECIV_HAVE_INTTYPES_H
 #include <inttypes.h>
 #else
-#ifdef HAVE_STDINT_H
+#ifdef FREECIV_HAVE_STDINT_H
 #include <stdint.h>
-#endif /* HAVE_STDINT_H */
-#endif /* HAVE_INTTYPES_H */
+#endif /* FREECIV_HAVE_STDINT_H */
+#endif /* FREECIV_HAVE_INTTYPES_H */
 
 /* Want to use GCC's __attribute__ keyword to check variadic
  * parameters to printf-like functions, without upsetting other
@@ -99,7 +100,7 @@ typedef unsigned int fc_bool;
 #define fc__warn_unused_result
 #endif
 
-#ifdef WIN32_NATIVE
+#ifdef FREECIV_MSWINDOWS
 typedef long int fc_errno;
 #else
 typedef int fc_errno;
@@ -117,11 +118,11 @@ int fc_strcoll(const char *str0, const char *str1);
 int fc_stricoll(const char *str0, const char *str1);
 
 FILE *fc_fopen(const char *filename, const char *opentype);
-#ifdef HAVE_LIBZ
+#ifdef FREECIV_HAVE_LIBZ
 #include <zlib.h>
 gzFile fc_gzopen(const char *filename, const char *opentype);
 #endif
-DIR *fc_opendir(const char *dirname);
+DIR *fc_opendir(const char *dir_to_open);
 int fc_remove(const char *filename);
 int fc_stat(const char *filename, struct stat *buf);
 
@@ -153,11 +154,11 @@ int cat_snprintf(char *str, size_t n, const char *format, ...)
 
 int fc_gethostname(char *buf, size_t len);
 
-#ifdef SOCKET_ZERO_ISNT_STDIN
-/* Support for console I/O in case SOCKET_ZERO_ISNT_STDIN. */
+#ifdef FREECIV_SOCKET_ZERO_NOT_STDIN
+/* Support for console I/O in case FREECIV_SOCKET_ZERO_NOT_STDIN. */
 void fc_init_console(void);
 char *fc_read_console(void);
-#endif
+#endif /* FREECIV_SOCKET_ZERO_NOT_STDIN */
 
 bool is_reg_file_for_access(const char *name, bool write_access);
 
@@ -172,10 +173,14 @@ bool fc_isupper(char c);
 char fc_toupper(char c);
 char fc_tolower(char c);
 
-void fc_uname(char *buf, size_t len);
-
 const char *fc_basename(const char *path);
 
+void make_escapes(const char *str, char *buf, size_t buf_len);
+void remove_escapes(const char *str, bool full_escapes,
+                    char *buf, size_t buf_len);
+
+int fc_at_quick_exit(void (*func)(void));
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
