@@ -107,6 +107,14 @@ cat <<EOF
   CreateShortCut "\$SMPROGRAMS\\\$STARTMENU_FOLDER\Freeciv.lnk" "\$INSTDIR\freeciv-\${GUI_ID}.cmd" "\$DefaultLanguageCode" "\$INSTDIR\freeciv-\${GUI_ID}.exe" 0 SW_SHOWMINIMIZED
   CreateShortCut "\$SMPROGRAMS\\\$STARTMENU_FOLDER\Freeciv Server.lnk" "\$INSTDIR\freeciv-server.cmd" "\$DefaultLanguageCode" "\$INSTDIR\freeciv-server.exe" 0 SW_SHOWMINIMIZED
   CreateShortCut "\$SMPROGRAMS\\\$STARTMENU_FOLDER\Freeciv Modpack Installer.lnk" "\$INSTDIR\freeciv-mp-\${GUI_ID}.cmd" "\$DefaultLanguageCode" "\$INSTDIR\freeciv-mp-\${GUI_ID}.exe" 0 SW_SHOWMINIMIZED
+EOF
+
+if test "x$3" = "xqt" ; then
+    echo "CreateShortCut \"\$SMPROGRAMS\\\$STARTMENU_FOLDER\Freeciv Ruleset Editor.lnk\" \"\$INSTDIR\freeciv-ruledit.cmd\" \"\$DefaultLanguageCode\" \"\$INSTDIR\freeciv-ruledit.exe\" 0 SW_SHOWMINIMIZED"
+fi
+
+cat <<EOF
+
   CreateShortCut "\$SMPROGRAMS\\\$STARTMENU_FOLDER\Uninstall.lnk" "\$INSTDIR\uninstall.exe"
   CreateShortCut "\$SMPROGRAMS\\\$STARTMENU_FOLDER\Website.lnk" "\$INSTDIR\Freeciv.url"
   !insertmacro MUI_STARTMENU_WRITE_END
@@ -155,16 +163,16 @@ EOF
 ### additional languages ###
 
 cat <<EOF
-SectionGroup "Additional languages"
+SectionGroup "Additional languages (translation %)"
 
 EOF
 
-cat ../../bootstrap/langnames.txt |
-sort -k 2 |
-while read -r code name
+cat ../../bootstrap/langstat_freeciv.txt |
+sort -k 1 |
+while read -r code prct name
 do
 if test -e $1/share/locale/$code/LC_MESSAGES/freeciv.mo; then
-echo "  Section \"$name ($code)\""
+echo "  Section \"$name ($code) $prct\""
 echo "  SetOutPath \$INSTDIR\\share\\locale\\$code"
 echo "  File /r $1\\share\\locale\\$code\*.*"
 echo "  SetOutPath \$INSTDIR"
@@ -219,12 +227,12 @@ Start Menu shortcut properties."
   \${NSD_CB_AddString} \$DefaultLanguageDropList "US English (en_US)"
 EOF
 
-  cat ../../bootstrap/langnames.txt |
-  sort -k 2 |
-  while read -r code name
+  cat ../../bootstrap/langstat_freeciv.txt |
+  sort -k 1 |
+  while read -r code prct name
   do
   if test -e $1/share/locale/$code/LC_MESSAGES/freeciv.mo; then
-  echo "  \${NSD_CB_AddString} \$DefaultLanguageDropList \"$name ($code)\""
+  echo "  \${NSD_CB_AddString} \$DefaultLanguageDropList \"$name ($code) $prct\""
   fi
   done
 
@@ -243,10 +251,10 @@ EOF
   echo "    StrCpy \$DefaultLanguageCode \"en_US\""
   echo "  \${EndIf}"
 
-  cat ../../bootstrap/langnames.txt |
-  while read -r code name
+  cat ../../bootstrap/langstat_freeciv.txt |
+  while read -r code prct name
   do
-    echo "  \${If} \$LangName == \"$name ($code)\""
+    echo "  \${If} \$LangName == \"$name ($code) $prct\""
     echo "    StrCpy \$DefaultLanguageCode \"$code\""
     echo "  \${EndIf}"
   done
