@@ -113,7 +113,7 @@ cat <<EOF
   !insertmacro MUI_STARTMENU_WRITE_BEGIN Application
   CreateDirectory "\$SMPROGRAMS\\\$STARTMENU_FOLDER"
   CreateShortCut "\$SMPROGRAMS\\\$STARTMENU_FOLDER\Freeciv Server.lnk" "\$INSTDIR\freeciv-server.cmd" "\$DefaultLanguageCode" "\$INSTDIR\freeciv-server.exe" 0 SW_SHOWMINIMIZED
-  CreateShortCut "\$SMPROGRAMS\\\$STARTMENU_FOLDER\Freeciv Modpack Installer.lnk" "\$INSTDIR\freeciv-mp-gtk2.cmd" "\$DefaultLanguageCode" "\$INSTDIR\freeciv-mp-gtk2.exe" 0 SW_SHOWMINIMIZED
+  CreateShortCut "\$SMPROGRAMS\\\$STARTMENU_FOLDER\Freeciv Modpack Installer.lnk" "\$INSTDIR\freeciv-mp-gtk3.cmd" "\$DefaultLanguageCode" "\$INSTDIR\freeciv-mp-gtk3.exe" 0 SW_SHOWMINIMIZED
   CreateShortCut "\$SMPROGRAMS\\\$STARTMENU_FOLDER\Freeciv.lnk" "\$INSTDIR\freeciv-sdl.cmd" "\$DefaultLanguageCode" "\$INSTDIR\freeciv-sdl.exe" 0 SW_SHOWMINIMIZED
   CreateShortCut "\$SMPROGRAMS\\\$STARTMENU_FOLDER\Uninstall.lnk" "\$INSTDIR\uninstall.exe"
   CreateShortCut "\$SMPROGRAMS\\\$STARTMENU_FOLDER\Website.lnk" "\$INSTDIR\Freeciv.url"
@@ -163,16 +163,16 @@ EOF
 ### additional languages ###
 
 cat <<EOF
-SectionGroup "Additional languages"
+SectionGroup "Additional languages (translation %)"
 
 EOF
 
-cat ../../bootstrap/langnames.txt |
-sort -k 2 |
-while read -r code name
+cat ../../bootstrap/langstat_freeciv.txt |
+sort -k 1 |
+while read -r code prct name
 do
 if test -e $1/share/locale/$code/LC_MESSAGES/freeciv.mo; then
-echo "  Section \"$name ($code)\""
+echo "  Section \"$name ($code) $prct\""
 echo "  SetOutPath \$INSTDIR\\share\\locale\\$code"
 echo "  File /r $1\\share\\locale\\$code\*.*"
 
@@ -245,12 +245,12 @@ Start Menu shortcut properties."
   \${NSD_CB_AddString} \$DefaultLanguageDropList "US English (en_US)"
 EOF
 
-  cat ../../bootstrap/langnames.txt |
-  sort -k 2 |
-  while read -r code name
+  cat ../../bootstrap/langstat_freeciv.txt |
+  sort -k 1 |
+  while read -r code prct name
   do
   if test -e $1/share/locale/$code/LC_MESSAGES/freeciv.mo; then
-  echo "  \${NSD_CB_AddString} \$DefaultLanguageDropList \"$name ($code)\""
+  echo "  \${NSD_CB_AddString} \$DefaultLanguageDropList \"$name ($code) $prct\""
   fi
   done
 
@@ -269,10 +269,10 @@ EOF
   echo "    StrCpy \$DefaultLanguageCode \"en_US\""
   echo "  \${EndIf}"
 
-  cat ../../bootstrap/langnames.txt |
-  while read -r code name
+  cat ../../bootstrap/langstat_freeciv.txt |
+  while read -r code prct name
   do
-    echo "  \${If} \$LangName == \"$name ($code)\""
+    echo "  \${If} \$LangName == \"$name ($code) $prct\""
     echo "    StrCpy \$DefaultLanguageCode \"$code\""
     echo "  \${EndIf}"
   done
